@@ -1,5 +1,5 @@
 /*
- * $Id: casts.h 15781 2008-09-02 16:30:18Z rmanfredi $
+ * $Id: casts.h 16803 2009-05-29 14:25:43Z cbiere $
  *
  * Copyright (c) 2006 Christian Biere
  *
@@ -90,6 +90,12 @@ cast_to_gchar_ptr(void *p)
 	return p;
 }
 
+static inline G_GNUC_CONST WARN_UNUSED_RESULT ALWAYS_INLINE void *
+cast_to_void_ptr(void *p)
+{
+	return p;
+}
+
 static inline G_GNUC_CONST WARN_UNUSED_RESULT ALWAYS_INLINE char *
 cast_to_char_ptr(void *p)
 {
@@ -138,12 +144,28 @@ pointer_to_int(const void *p)
 	return pointer_to_uint(p);
 }
 
-typedef void (*func_ptr_t)(void);
+typedef void (*func_ptr_t)();
 
 static inline G_GNUC_CONST WARN_UNUSED_RESULT ALWAYS_INLINE func_ptr_t
 cast_pointer_to_func(const void *p)
 {
-	return (func_ptr_t) p;
+	union {
+		func_ptr_t func;
+		const void *ptr;
+	} tmp;
+	tmp.ptr = p;
+	return tmp.func;
+}
+
+static inline G_GNUC_CONST WARN_UNUSED_RESULT ALWAYS_INLINE void *
+cast_func_to_pointer(func_ptr_t func)
+{
+	union {
+		func_ptr_t func;
+		void *ptr;
+	} tmp;
+	tmp.func = func;
+	return tmp.ptr;
 }
 
 static inline size_t G_GNUC_CONST WARN_UNUSED_RESULT ALWAYS_INLINE
@@ -152,6 +174,30 @@ ptr_diff(const void *a, const void *b)
 	return (const char *) a - (const char *) b;
 }
 
+static inline void * G_GNUC_CONST WARN_UNUSED_RESULT ALWAYS_INLINE
+ptr_add_offset(void *p, size_t offset)
+{
+	/* Using size_t instead of 'char *' because pointers don't wrap. */
+	return (void *) ((size_t) p + offset);
+}
+
+static inline const void * G_GNUC_CONST WARN_UNUSED_RESULT ALWAYS_INLINE
+const_ptr_add_offset(const void *p, size_t offset)
+{
+	/* Using size_t instead of 'char *' because pointers don't wrap. */
+	return (const void *) ((size_t) p + offset);
+}
+
+/**
+ * Compare two generic pointers.
+ */
+static inline int G_GNUC_CONST WARN_UNUSED_RESULT ALWAYS_INLINE
+ptr_cmp(const void *a, const void *b)
+{
+	return a == b ? 0 :
+		(const char *) a < (const char *) b ? -1 : +1;
+}
+
 /**
  * Converts a filesize_t to off_t.
  *
@@ -184,6 +230,9 @@ filesize_to_off_t(filesize_t pos)
 #define UNSIGNED(value) ((value) + ((guint64)0U))
 #endif	/* UINTMAX_C */
 
+/* Normalize boolean-like values to TRUE(1) or FALSE(0) */
+#define booleanize(x) ((x) ? 1 : 0)
+
 #endif /* _casts_h_ */
 
 /* vi: set ts=4 sw=4 cindent: */
