@@ -1,5 +1,5 @@
 /*
- * $Id: common.h 16301 2009-03-29 15:31:34Z rmanfredi $
+ * $Id: common.h 17162 2009-11-03 19:55:08Z rmanfredi $
  *
  * Copyright (c) 2001-2003, Richard Eckart
  *
@@ -38,29 +38,6 @@
 
 #include "config.h"
 
-/*
- * Constants
- */
-
-#define GTA_PRODUCT_NAME "gtk-gnutella"	/**< Normally "gtk-gnutella" */
-#define GTA_VERSION 0				/**< major version */
-#define GTA_SUBVERSION 96			/**< minor version */
-#define GTA_PATCHLEVEL 6			/**< patch level or teeny version */
-#define GTA_REVISION "stable"		/**< unstable, beta, stable */
-#define GTA_REVCHAR ""				/**< (u)nstable, (b)eta, none -> stable */
-#define GTA_RELEASE "2009-03-29"	/**< ISO 8601 format YYYY-MM-DD */
-#define GTA_WEBSITE "http://gtk-gnutella.sourceforge.net/"
-
-#if defined(USE_GTK1)
-#define GTA_INTERFACE "GTK1"
-#elif defined(USE_GTK2)
-#define GTA_INTERFACE "GTK2"
-#elif defined(USE_TOPLESS)
-#define GTA_INTERFACE "Topless"
-#else
-#error "Expected Gtk+ 1.2, Gtk+ 2.x or headless as user-interface."
-#endif
-
 #ifndef HAS_LIBXML2
 #error "You need libxml2 (http://www.xmlsoft.org/) to compile gtk-gnutella"
 #endif
@@ -227,7 +204,7 @@ struct passwd
 
 #ifndef MAP_FAILED
 #define MAP_FAILED ((void *) -1)
-#endif	/* !MMAP_FAILED */
+#endif	/* !MAP_FAILED */
 
 #ifdef I_SYS_SENDFILE
 #include <sys/sendfile.h>
@@ -578,35 +555,36 @@ G_STMT_START {			\
 #define STATIC_ASSERT(x) \
 	do { switch (0) { case ((x) ? 1 : 0): case 0: break; } } while(0)
 
-#if defined(GTA_PATCHLEVEL) && (GTA_PATCHLEVEL != 0)
-#define GTA_VERSION_NUMBER \
-	STRINGIFY(GTA_VERSION) "." \
-	STRINGIFY(GTA_SUBVERSION) "." \
-	STRINGIFY(GTA_PATCHLEVEL) GTA_REVCHAR
-#else
-#define GTA_VERSION_NUMBER \
-	STRINGIFY(GTA_VERSION) "." STRINGIFY(GTA_SUBVERSION) GTA_REVCHAR
-#endif
-
-#define GTA_PORT			6346	/**< Default "standard" port */
 #define MAX_HOSTLEN			256		/**< Max length for FQDN host */
-#define GTA_BUILD					"$Revision: 16301 $"
 
 /* The next two defines came from huge.h --- Emile */
 #define SHA1_BASE16_SIZE 	40		/**< 160 bits in base16 representation */
 #define SHA1_BASE32_SIZE 	32		/**< 160 bits in base32 representation */
 #define SHA1_RAW_SIZE		20		/**< 160 bits in binary representation */
 
-#define TTH_BASE32_SIZE 	39		/**< 160 bits in base32 representation */
-#define TTH_RAW_SIZE		24		/**< 160 bits in binary representation */
+#define TTH_BASE16_SIZE 	48		/**< 192 bits in base16 representation */
+#define TTH_BASE32_SIZE 	39		/**< 192 bits in base32 representation */
+#define TTH_RAW_SIZE		24		/**< 192 bits in binary representation */
 
 #define BITPRINT_BASE32_SIZE 	72	/**< 352 bits in base32 representation
 									 **  includes dot between sha1 and tiger */
+#define BITPRINT_BASE16_SIZE	89	/**< 352 bits in base16 + separation dot */
 #define BITPRINT_RAW_SIZE		44	/**< 352 bits in binary representation */
 
 /** Maximum bytes in filename i.e., including NUL */
 #define	FILENAME_MAXBYTES 256
 
+/* Maximum path length, in bytes */
+#if defined(PATH_MAX)
+#define MAX_PATH_LEN	PATH_MAX	/* POSIX, first choice */
+#elif defined(MAXPATHLEN)
+#define MAX_PATH_LEN	MAXPATHLEN
+#elif defined(PATH_LEN)
+#define MAX_PATH_LEN	PATH_LEN
+#else
+#define MAX_PATH_LEN	2048
+#endif
+
 /*
  * Forbidden glib calls.
  */
@@ -632,10 +610,8 @@ typedef gboolean (*reclaim_fd_t)(void);
 #  ifdef gettext_noop
 #    define N_(String) gettext_noop(String)
 #  else
-#    define N_(String) (String)
+#    define N_(String) "" String ""
 #  endif
-
-
 #else
 #  define textdomain(String) (String)
 #  define gettext(String) (String)
@@ -644,7 +620,7 @@ typedef gboolean (*reclaim_fd_t)(void);
 #  define bindtextdomain(Domain,Directory) (Domain)
 #  define ngettext(Single, Plural, Number) ((Number) == 1 ? (Single) : (Plural))
 #  define _(String) (String)
-#  define N_(String) (String)
+#  define N_(String) "" String ""
 #  define Q_(String) g_strip_context ((String), (String))
 #endif /* ENABLE_NLS */
 
@@ -678,6 +654,16 @@ get_rcsid_string(void)	\
 	return rcsid;	\
 }
 
+/**
+ * Composes a 32-bit native endian integer from four characters (bytes) given
+ * in big endian byte order.
+ */
+#define FOURCC_NATIVE(a,b,c,d) ( \
+	((guint32) (unsigned char) ((a) & 0xffU) << 24) | \
+	((guint32) (unsigned char) ((b) & 0xffU) << 16) | \
+	((guint32) (unsigned char) ((c) & 0xffU) << 8)  | \
+	((guint32) (unsigned char) ((d) & 0xffU)))
+
 #include "casts.h"
 
 #endif /* _common_h_ */
