@@ -1,5 +1,5 @@
 /*
- * $Id: ban.h 12496 2006-11-26 09:22:49Z cbiere $
+ * $Id: ban.h 16106 2009-03-10 15:16:31Z rmanfredi $
  *
  * Copyright (c) 2002-2003, Raphael Manfredi
  *
@@ -55,14 +55,14 @@ typedef enum {
 void ban_init(void);
 void ban_close(void);
 ban_type_t ban_allow(const host_addr_t addr);
-void ban_record(const host_addr_t addr, const gchar *msg);
+void ban_record(const host_addr_t addr, const char *msg);
 void ban_force(struct gnutella_socket *s);
-gint ban_delay(const host_addr_t addr);
-const gchar *ban_message(const host_addr_t addr);
+int ban_delay(const host_addr_t addr);
+const char *ban_message(const host_addr_t addr);
 gboolean ban_is_banned(const host_addr_t addr);
 void ban_max_recompute(void);
 
-const gchar *ban_vendor(const gchar *vendor);
+const char *ban_vendor(const char *vendor);
 
 #endif	/* _core_ban_h_ */
 
