@@ -1,5 +1,5 @@
 /*
- * $Id: bh_download.c 16104 2009-03-10 14:58:09Z rmanfredi $
+ * $Id: bh_download.c 16880 2009-06-01 21:26:10Z cbiere $
  *
  * Copyright (c) 2005, Raphael Manfredi
  *
@@ -35,17 +35,19 @@
 
 #include "common.h"
 
-RCSID("$Id: bh_download.c 16104 2009-03-10 14:58:09Z rmanfredi $")
+RCSID("$Id: bh_download.c 16880 2009-06-01 21:26:10Z cbiere $")
 
 #include "bh_download.h"
 #include "downloads.h"
 #include "bsched.h"
+#include "dump.h"
 #include "gnet_stats.h"
 #include "rx_inflate.h"
 
 #include "lib/atoms.h"
 #include "lib/endian.h"
 #include "lib/pmsg.h"
+#include "lib/halloc.h"
 #include "lib/walloc.h"
 #include "lib/override.h"	/* Must be the last header included */
 
@@ -134,7 +136,7 @@ browse_data_read(struct browse_ctx *bc, pmsg_t *mb)
 
 		if (bc->size > bc->data_size) {
 			bc->data_size = MAX(BH_DL_DEFAULT_SIZE, bc->size);
-			bc->data = g_realloc(bc->data, bc->data_size);
+			bc->data = hrealloc(bc->data, bc->data_size);
 		}
 
 		bc->pos = 0;
@@ -182,6 +184,7 @@ browse_data_process(struct browse_ctx *bc)
 	n = node_browse_prepare(
 		&bc->host, bc->vendor, &bc->header, bc->data, bc->size);
 
+	dump_rx_packet(n);
 	gnet_stats_count_received_header(n);
 	gnet_stats_count_received_payload(n);
 
@@ -420,7 +423,7 @@ browse_host_dl_free(struct browse_ctx **ptr)
 		if (!bc->closed) {
 			search_dissociate_browse(bc->sh, bc->owner);
 		}
-		G_FREE_NULL(bc->data);
+		HFREE_NULL(bc->data);
 		wfree(bc, sizeof *bc);
 		*ptr = NULL;
 	}
