@@ -1,5 +1,5 @@
 /*
- * $Id: bh_download.h 13450 2007-04-28 20:03:18Z cbiere $
+ * $Id: bh_download.h 16104 2009-03-10 14:58:09Z rmanfredi $
  *
  * Copyright (c) 2005, Raphael Manfredi
  *
@@ -58,10 +58,10 @@ struct browse_ctx *browse_host_dl_create(
 	gpointer owner, gnet_host_t *host, gnet_search_t sh);
 void browse_host_dl_free(struct browse_ctx **ptr);
 gboolean browse_host_dl_for_search(struct browse_ctx *bc, gnet_search_t sh);
-void browse_host_dl_write(struct browse_ctx *bc, gchar *data, size_t len);
+void browse_host_dl_write(struct browse_ctx *bc, char *data, size_t len);
 gboolean browse_host_dl_receive(
 	struct browse_ctx *bc, gnet_host_t *host, struct wrap_io *wio,
-	const gchar *vendor, guint32 flags);
+	const char *vendor, guint32 flags);
 struct bio_source *browse_host_io_source(struct browse_ctx *bc);
 void browse_host_dl_close(struct browse_ctx *bc);
 void browse_host_dl_search_closed(struct browse_ctx *bc, gnet_search_t sh);
