@@ -1,5 +1,5 @@
 /*
- * $Id: bh_upload.c 16223 2009-03-20 21:48:03Z rmanfredi $
+ * $Id: bh_upload.c 17240 2010-02-13 12:26:50Z rmanfredi $
  *
  * Copyright (c) 2005, Christian Biere & Raphael Manfredi
  *
@@ -36,7 +36,9 @@
 
 #include "common.h"
 
-RCSID("$Id: bh_upload.c 16223 2009-03-20 21:48:03Z rmanfredi $")
+RCSID("$Id: bh_upload.c 17240 2010-02-13 12:26:50Z rmanfredi $")
+
+#include "gtk-gnutella.h"
 
 #include "bh_upload.h"
 #include "share.h"
@@ -56,7 +58,10 @@ RCSID("$Id: bh_upload.c 16223 2009-03-20 21:48:03Z rmanfredi $")
 #include "lib/array.h"
 #include "lib/header.h"
 #include "lib/gnet_host.h"
-#include "lib/misc.h"
+#include "lib/concat.h"
+#include "lib/halloc.h"
+#include "lib/unsigned.h"
+#include "lib/stringify.h"
 #include "lib/url.h"
 #include "lib/walloc.h"
 
@@ -83,12 +88,12 @@ enum bh_state {
 	BH_STATE_TRAILER,		/* Sending trailer data */
 	BH_STATE_EOF,			/* All data sent (End Of File) */
 
-	NUM_BH_STATES,
+	NUM_BH_STATES
 };
 
 enum bh_type {
 	BH_TYPE_HTML = 0,		/* Send back HTML */
-	BH_TYPE_QHIT,			/* Send back Gnutella query hits */
+	BH_TYPE_QHIT			/* Send back Gnutella query hits */
 };
 
 struct browse_host_upload {
@@ -153,7 +158,7 @@ static inline void
 browse_host_next_state(struct browse_host_upload *bh, enum bh_state state)
 {
 	g_assert(NULL != bh);
-	g_assert((int) state >= 0 && state < NUM_BH_STATES);
+	g_assert(UNSIGNED(state) < NUM_BH_STATES);
 	bh->w_buf = NULL;
 	bh->w_buf_size = 0;
 	bh->b_data = NULL;
@@ -196,7 +201,7 @@ browse_host_read_html(struct special_upload *ctx,
 	g_assert(NULL != dest);
 	g_assert(size <= INT_MAX);
 
-	g_assert((int) bh->state >= 0 && bh->state < NUM_BH_STATES);
+	g_assert(UNSIGNED(bh->state) < NUM_BH_STATES);
 	g_assert(bh->b_size <= INT_MAX);
 	g_assert(bh->b_offset <= bh->b_size);
 
@@ -277,7 +282,7 @@ browse_host_read_html(struct special_upload *ctx,
 						
 						dir = shared_file_relative_path(sf);
 						if (dir) {
-							name = g_strconcat(dir, "/", name_nfc, (void *) 0);
+							name = h_strconcat(dir, "/", name_nfc, (void *) 0);
 						} else {
 							name = deconstify_gchar(name_nfc);
 						}
@@ -286,7 +291,7 @@ browse_host_read_html(struct special_upload *ctx,
 						html_name = walloc(html_size);
 						html_escape(name, html_name, html_size);
 						if (name != name_nfc) {
-							G_FREE_NULL(name);
+							HFREE_NULL(name);
 						}
 					}
 
@@ -315,7 +320,7 @@ browse_host_read_html(struct special_upload *ctx,
 							"]</li>\r\n", (void *) 0);
 
 						if (escaped != name_nfc) {
-							G_FREE_NULL(escaped);
+							HFREE_NULL(escaped);
 						}
 					}
 
