@@ -1,5 +1,5 @@
 /*
- * $Id: bitzi.c 16106 2009-03-10 15:16:31Z rmanfredi $
+ * $Id: bitzi.c 17134 2009-10-29 17:08:24Z rmanfredi $
  *
  * Copyright (c) 2004, Alex Bennee <alex@bennee.com>
  *
@@ -62,6 +62,10 @@
 #include "lib/atoms.h"
 #include "lib/getdate.h"	/* date2time() */
 #include "lib/glib-missing.h"
+#include "lib/halloc.h"
+#include "lib/parse.h"
+#include "lib/path.h"
+#include "lib/stringify.h"
 #include "lib/tm.h"
 #include "lib/urn.h"
 #include "lib/walloc.h"
@@ -75,7 +79,8 @@
  *
  */
 
-static const char bitzi_url_fmt[] = "http://ticket.bitzi.com/rdf/urn:sha1:%s";
+static const char bitzi_url_fmt[] =
+	"http://bitzi.com/rdf/urn:sha1:%s?ref=gtk-gnutella";
 
 typedef struct {
 	const struct sha1 *sha1;			/**< binary SHA-1, atom */
@@ -242,15 +247,12 @@ bitzi_host_data_ind(struct http_async *unused_handle, char *data, int len)
  */
 static void
 bitzi_host_error_ind(struct http_async *handle,
-	http_errtype_t unused_type, gpointer unused_v)
+	http_errtype_t type, gpointer v)
 {
-	(void) unused_type;
-	(void) unused_v;
-
-	g_warning("bitzi_host_error_ind: failed!");
-
 	g_assert(handle == current_bitzi_request_handle);
 
+	http_async_log_error_dbg(handle, type, v, "bitzi_host_error_ind", TRUE);
+
 	/*
 	 * process what we had and clear up
 	 */
@@ -956,9 +958,6 @@ bitzi_load_cache(void)
 	oldpath = make_pathname(settings_config_dir(), "bitzi.xml.orig");
   	path = make_pathname(settings_config_dir(), "bitzi.xml");
 
-	g_assert(NULL != path);
-	g_assert(NULL != oldpath);
-
 	if (rename(path, oldpath)) {
 		g_warning("bitzi_init: failed to rename %s to %s (%s)",
 				path, oldpath, g_strerror(errno));
@@ -1062,8 +1061,8 @@ bitzi_load_cache(void)
 			ticket_count, oldpath);
 
 	/* clean-up */
-	G_FREE_NULL(path);
-	G_FREE_NULL(oldpath);
+	HFREE_NULL(path);
+	HFREE_NULL(oldpath);
 }
 
 /**
