@@ -1,5 +1,5 @@
 /*
- * $Id: bsched.c 16129 2009-03-12 06:47:48Z rmanfredi $
+ * $Id: bsched.c 17240 2010-02-13 12:26:50Z rmanfredi $
  *
  * Copyright (c) 2002-2003, Raphael Manfredi
  *
@@ -35,7 +35,7 @@
 
 #include "common.h"
 
-RCSID("$Id: bsched.c 16129 2009-03-12 06:47:48Z rmanfredi $")
+RCSID("$Id: bsched.c 17240 2010-02-13 12:26:50Z rmanfredi $")
 
 #include "bsched.h"
 #include "inet.h"
@@ -45,8 +45,11 @@ RCSID("$Id: bsched.c 16129 2009-03-12 06:47:48Z rmanfredi $")
 #include "if/gnet_property_priv.h"
 
 #include "lib/glib-missing.h"
+#include "lib/halloc.h"
 #include "lib/walloc.h"
-#include "lib/misc.h"
+#include "lib/parse.h"
+#include "lib/stringify.h"
+#include "lib/vmm.h"
 
 #include "lib/override.h"		/* Must be the last header included */
 
@@ -76,7 +79,7 @@ enum {
 };
 
 enum bsched_magic {
-	BSCHED_MAGIC = 0xee24261eU
+	BSCHED_MAGIC = 0x6e24261eU
 };
 
 /**
@@ -153,7 +156,6 @@ static int bws_in_ema = 0;
 #define BW_SLOT_MIN		64	 /**< Minimum bandwidth/slot for realloc */
 
 #define BW_OUT_UP_MIN	8192 /**< Minimum out bandwidth for becoming ultra */
-#define BW_OUT_DHT_MIN	8192 /**< Minimum out bandwidth for joining the DHT */
 #define BW_OUT_GNET_MIN	128	 /**< Minimum out bandwidth per Gnet connection */
 #define BW_OUT_LEAF_MIN	32	 /**< Minimum out bandwidth per leaf connection */
 
@@ -201,10 +203,10 @@ bsched_make(const char *name, int type, guint32 mode,
 	g_assert(type == BS_T_STREAM);		/* XXX only mode supported for now */
 	g_assert(bandwidth <= BS_BW_MAX);	/* Signed, and multiplied by 1000 */
 
-	bs = g_malloc0(sizeof(*bs));
+	bs = walloc0(sizeof *bs);
 
 	bs->magic = BSCHED_MAGIC;
-	bs->name = g_strdup(name);
+	bs->name = h_strdup(name);
 	bs->flags = mode;
 	bs->type = type;
 	bs->period = period;
@@ -253,15 +255,20 @@ bsched_free(bsched_t *bs)
 	bs->sources = NULL;
 	g_slist_free(bs->stealers);
 	bs->stealers = NULL;
-	G_FREE_NULL(bs->name);
+	HFREE_NULL(bs->name);
 	bs->magic = 0;
-	G_FREE_NULL(bs);
+	wfree(bs, sizeof *bs);
 }
 
+/**
+ * Is bandwidth scheduler saturated currently?
+ */
 gboolean
 bsched_saturated(bsched_bws_t bws)
 {
 	const bsched_t *bs = bsched_get(bws);
+	if (!(bs->flags & BS_F_ENABLED))		/* Scheduler disabled */
+		return FALSE;
 	return bs->bw_actual > bs->bw_max;
 }
 
@@ -319,7 +326,7 @@ bsched_set_urgent(bsched_bws_t bws, int amount)
 }
 
 /**
- * Add `stealer' as a bandwidth stealer for underused bandwidth in `bs'.
+ * Add `stealer' as a bandwidth stealer for underused bandwidth in `bws'.
  * Both must be either reading or writing schedulers.
  */
 static void
@@ -350,6 +357,16 @@ bsched_reset_stealers(bsched_t *bs)
 	}
 }
 
+/**
+ * Configure outband DHT traffic cross-stealing with Gnutella UDP
+ */
+static void
+bsched_dht_cross_stealing(void)
+{
+	bsched_add_stealer(BSCHED_BWS_GOUT_UDP, BSCHED_BWS_DHT_OUT);
+	bsched_add_stealer(BSCHED_BWS_DHT_OUT, BSCHED_BWS_GOUT_UDP);
+}
+
 /**
  * Allow cross-stealing of unused bandwidth between HTTP/gnet.
  */
@@ -366,6 +383,7 @@ bsched_config_steal_http_gnet(void)
 	bsched_add_stealer(BSCHED_BWS_OUT, BSCHED_BWS_GOUT);
 	bsched_add_stealer(BSCHED_BWS_OUT, BSCHED_BWS_GOUT_UDP);
 	bsched_add_stealer(BSCHED_BWS_OUT, BSCHED_BWS_GLOUT);
+	bsched_add_stealer(BSCHED_BWS_OUT, BSCHED_BWS_DHT_OUT);
 
 	bsched_add_stealer(BSCHED_BWS_GOUT, BSCHED_BWS_OUT);
 	bsched_add_stealer(BSCHED_BWS_GOUT, BSCHED_BWS_GOUT_UDP);
@@ -382,6 +400,7 @@ bsched_config_steal_http_gnet(void)
 	bsched_add_stealer(BSCHED_BWS_GLOUT, BSCHED_BWS_GOUT);
 	bsched_add_stealer(BSCHED_BWS_GLOUT, BSCHED_BWS_GOUT_UDP);
 	bsched_add_stealer(BSCHED_BWS_GLOUT, BSCHED_BWS_OUT);
+	bsched_add_stealer(BSCHED_BWS_GLOUT, BSCHED_BWS_DHT_OUT);
 
 	bsched_add_stealer(BSCHED_BWS_GLIN, BSCHED_BWS_GIN);
 	bsched_add_stealer(BSCHED_BWS_GLIN, BSCHED_BWS_GIN_UDP);
@@ -391,9 +410,11 @@ bsched_config_steal_http_gnet(void)
 	bsched_add_stealer(BSCHED_BWS_GOUT_UDP, BSCHED_BWS_GLOUT);
 	bsched_add_stealer(BSCHED_BWS_GOUT_UDP, BSCHED_BWS_OUT);
 
-	bsched_add_stealer(BSCHED_BWS_GIN_UDP, BSCHED_BWS_GIN);
-	bsched_add_stealer(BSCHED_BWS_GIN_UDP, BSCHED_BWS_GLIN);
-	bsched_add_stealer(BSCHED_BWS_GIN_UDP, BSCHED_BWS_IN);
+	bsched_add_stealer(BSCHED_BWS_DHT_OUT, BSCHED_BWS_GOUT);
+	bsched_add_stealer(BSCHED_BWS_DHT_OUT, BSCHED_BWS_GLOUT);
+	bsched_add_stealer(BSCHED_BWS_DHT_OUT, BSCHED_BWS_OUT);
+
+	bsched_dht_cross_stealing();
 }
 
 /**
@@ -410,9 +431,10 @@ bsched_config_steal_gnet(void)
 	}
 
 	bsched_add_stealer(BSCHED_BWS_GIN, BSCHED_BWS_GIN_UDP);
-	bsched_add_stealer(BSCHED_BWS_GIN_UDP, BSCHED_BWS_GIN);
 	bsched_add_stealer(BSCHED_BWS_GOUT, BSCHED_BWS_GOUT_UDP);
 	bsched_add_stealer(BSCHED_BWS_GOUT_UDP, BSCHED_BWS_GOUT);
+
+	bsched_dht_cross_stealing();
 }
 
 /*
@@ -448,9 +470,16 @@ bsched_early_init(void)
 
 	bws_set[BSCHED_BWS_LOOPBACK_OUT] = bsched_make("loopback out",
 		BS_T_STREAM, BS_F_WRITE, 0, 1000);
+
 	bws_set[BSCHED_BWS_LOOPBACK_IN] = bsched_make("loopback in",
 		BS_T_STREAM, BS_F_READ, 0, 1000);
 
+	bws_set[BSCHED_BWS_DHT_OUT] = bsched_make("DHT out",
+		BS_T_STREAM, BS_F_WRITE, GNET_PROPERTY(bw_dht_out), 1000);
+
+	bws_set[BSCHED_BWS_DHT_IN] = bsched_make("DHT in",
+		BS_T_STREAM, BS_F_READ, 0, 1000);
+
 	bws_list = g_slist_prepend(bws_list, 
 						GUINT_TO_POINTER(BSCHED_BWS_LOOPBACK_IN));
 	bws_list = g_slist_prepend(bws_list, 
@@ -461,6 +490,8 @@ bsched_early_init(void)
 						GUINT_TO_POINTER(BSCHED_BWS_GIN_UDP));
 	bws_list = g_slist_prepend(bws_list, 
 						GUINT_TO_POINTER(BSCHED_BWS_IN));
+	bws_list = g_slist_prepend(bws_list, 
+						GUINT_TO_POINTER(BSCHED_BWS_DHT_IN));
 	bws_list = g_slist_prepend(bws_list, 
 						GUINT_TO_POINTER(BSCHED_BWS_LOOPBACK_OUT));
 	bws_list = g_slist_prepend(bws_list, 
@@ -471,6 +502,8 @@ bsched_early_init(void)
 						GUINT_TO_POINTER(BSCHED_BWS_GOUT_UDP));
 	bws_list = g_slist_prepend(bws_list, 
 						GUINT_TO_POINTER(BSCHED_BWS_OUT));
+	bws_list = g_slist_prepend(bws_list, 
+						GUINT_TO_POINTER(BSCHED_BWS_DHT_OUT));
 
 	bws_out_list = g_slist_prepend(bws_in_list, 
 						GUINT_TO_POINTER(BSCHED_BWS_LOOPBACK_IN));
@@ -482,6 +515,8 @@ bsched_early_init(void)
 						GUINT_TO_POINTER(BSCHED_BWS_GIN_UDP));
 	bws_in_list = g_slist_prepend(bws_in_list, 
 						GUINT_TO_POINTER(BSCHED_BWS_IN));
+	bws_in_list = g_slist_prepend(bws_in_list, 
+						GUINT_TO_POINTER(BSCHED_BWS_DHT_IN));
 
 	bws_out_list = g_slist_prepend(bws_out_list, 
 						GUINT_TO_POINTER(BSCHED_BWS_LOOPBACK_OUT));
@@ -493,6 +528,8 @@ bsched_early_init(void)
 						GUINT_TO_POINTER(BSCHED_BWS_GOUT_UDP));
 	bws_out_list = g_slist_prepend(bws_out_list, 
 						GUINT_TO_POINTER(BSCHED_BWS_OUT));
+	bws_out_list = g_slist_prepend(bws_out_list, 
+						GUINT_TO_POINTER(BSCHED_BWS_DHT_OUT));
 }
 
 /**
@@ -509,6 +546,8 @@ bsched_correct_init(void)
 	bsched_set_bandwidth(BSCHED_BWS_GIN, GNET_PROPERTY(bw_gnet_in));
 	bsched_set_bandwidth(BSCHED_BWS_GIN_UDP, 0);
 	bsched_set_bandwidth(BSCHED_BWS_GLIN, GNET_PROPERTY(bw_gnet_lin));
+	bsched_set_bandwidth(BSCHED_BWS_DHT_OUT, GNET_PROPERTY(bw_dht_out));
+	bsched_set_bandwidth(BSCHED_BWS_DHT_IN, 0);
 }
 
 /**
@@ -634,6 +673,9 @@ bsched_enable(bsched_bws_t bws)
 	bsched_t *bs = bsched_get(bws);
 	bs->flags |= BS_F_ENABLED;
 	tm_now(&bs->last_period);
+
+	if (GNET_PROPERTY(bsched_debug))
+		g_message("BSCHED enabling \"%s\"", bs->name);
 }
 
 /**
@@ -644,6 +686,9 @@ bsched_disable(bsched_bws_t bws)
 {
 	bsched_t *bs = bsched_get(bws);
 	bs->flags &= ~BS_F_ENABLED;
+
+	if (GNET_PROPERTY(bsched_debug))
+		g_message("BSCHED disabling \"%s\"", bs->name);
 }
 
 /**
@@ -855,7 +900,7 @@ bsched_begin_timeslice(bsched_t *bs)
 		 * "instantaneous bandwidth" used.
 		 *
 		 * Slow EMA of bandwidth is computed on the last n=127 terms, which at
-		 * one computation per second, means an average of the two minutes.
+		 * one computation per second, means an average of the last two minutes.
 		 * This value is smoother and therefore more suited to use for the
 		 * remaining time estimates.
 		 *
@@ -1076,8 +1121,8 @@ bsched_set_bandwidth(bsched_bws_t bws, int bandwidth)
 	g_assert(bandwidth >= 0);
 	g_assert(bandwidth <= BS_BW_MAX);	/* Signed, and multiplied by 1000 */
 
-	if (GNET_PROPERTY(dbg))
-		g_message("bsched_set_bandwidth: using %.2f KiB/s for %s",
+	if (GNET_PROPERTY(bsched_debug))
+		g_message("BSCHED bsched_set_bandwidth: using %.2f KiB/s for %s",
 			(double) bandwidth / bs->period, bs->name);
 
 	bs->bw_per_second = bandwidth;
@@ -1149,9 +1194,9 @@ bw_available(bio_source_t *bio, int len)
 
 	available = bs->bw_max + bs->bw_stolen - bs->bw_actual;
 
-	if (GNET_PROPERTY(dbg) > 8)
-		printf("bw_available: "
-			"[fd #%d] max=%d, stolen=%d, actual=%d => avail=%d\n",
+	if (GNET_PROPERTY(bsched_debug) > 8)
+		g_message("BSCHED bw_available: "
+			"[fd #%d] max=%d, stolen=%d, actual=%d => avail=%d",
 			bio->wio->fd(bio->wio), bs->bw_max, bs->bw_stolen, bs->bw_actual,
 			available);
 
@@ -1173,8 +1218,8 @@ bw_available(bio_source_t *bio, int len)
 	used = bio->flags & BIO_F_USED;
 	active = bio->flags & BIO_F_ACTIVE;
 
-	if (GNET_PROPERTY(dbg) > 8)
-		printf("\tcapped=%s, used=%s, active=%s => avail=%d\n",
+	if (GNET_PROPERTY(bsched_debug) > 8)
+		g_message("BSCHED \tcapped=%s, used=%s, active=%s => avail=%d",
 			capped ? "y" : "n", used ? "y" : "n", active ? "y" : "n",
 			available);
 
@@ -1231,8 +1276,8 @@ bw_available(bio_source_t *bio, int len)
 			bs->bw_slot = BW_SLOT_MIN;
 		}
 
-		if (GNET_PROPERTY(dbg) > 7)
-			printf("bw_availble: new slot=%d for \"%s\" (%scapped)\n",
+		if (GNET_PROPERTY(bsched_debug) > 7)
+			g_message("BSCHED bw_availble: new slot=%d for \"%s\" (%scapped)",
 				bs->bw_slot, bs->name, capped ? "" : "un");
 	}
 
@@ -1305,9 +1350,9 @@ bw_available(bio_source_t *bio, int len)
 		if (adj > available)
 			adj = available;
 
-		if (GNET_PROPERTY(dbg) > 4)
-			printf("bw_available: \"%s\" adding %d to %d"
-				" (len=%d, capped=%d [%d-%d/%d], available=%d, used=%c)\n",
+		if (GNET_PROPERTY(bsched_debug) > 4)
+			g_message("BSCHED bw_available: \"%s\" adding %d to %d"
+				" (len=%d, capped=%d [%d-%d/%d], available=%d, used=%c)",
 				bs->name, adj, result, len, bs->bw_last_capped,
 				bs->last_used, bs->current_used, bs->count,
 				available, (bio->flags & BIO_F_USED) ? 'y' : 'n');
@@ -1403,8 +1448,8 @@ bio_write(bio_source_t *bio, gconstpointer data, size_t len)
 
 	amount = len > available ? available : len;
 
-	if (GNET_PROPERTY(dbg) > 7)
-		printf("bio_write(wio=%d, len=%d) available=%d\n",
+	if (GNET_PROPERTY(bsched_debug) > 7)
+		g_message("BSCHED bio_write(wio=%d, len=%d) available=%d",
 			bio->wio->fd(bio->wio), (int) len, (int) available);
 
 	r = bio->wio->write(bio->wio, data, amount);
@@ -1515,8 +1560,8 @@ bio_writev(bio_source_t *bio, struct iovec *iov, int iovcnt)
 	 *		--RAM, 17/03/2002
 	 */
 
-	if (GNET_PROPERTY(dbg) > 7)
-		printf("bio_writev(fd=%d, len=%d) available=%d\n",
+	if (GNET_PROPERTY(bsched_debug) > 7)
+		g_message("BSCHED bio_writev(fd=%d, len=%d) available=%d",
 			bio->wio->fd(bio->wio), (int) len, (int) available);
 
 	if (iovcnt > MAX_IOV_COUNT)
@@ -1591,8 +1636,8 @@ bio_sendto(bio_source_t *bio, const gnet_host_t *to,
 		return -1;
 	}
 
-	if (GNET_PROPERTY(dbg) > 7)
-		printf("bio_sendto(wio=%d, len=%d) available=%d\n",
+	if (GNET_PROPERTY(bsched_debug) > 7)
+		g_message("BSCHED bio_sendto(wio=%d, len=%d) available=%d",
 			bio->wio->fd(bio->wio), (int) len, (int) available);
 
 	g_assert(bio->wio != NULL);
@@ -1728,8 +1773,8 @@ bio_sendfile(sendfile_ctx_t *ctx, bio_source_t *bio, int in_fd, off_t *offset,
 
 	amount = len > available ? available : len;
 
-	if (GNET_PROPERTY(dbg) > 7)
-		printf("bsched_write(fd=%d, len=%d) available=%d\n",
+	if (GNET_PROPERTY(bsched_debug) > 7)
+		g_message("BSCHED bsched_write(fd=%d, len=%d) available=%d",
 			bio->wio->fd(bio->wio), (int) len, (int) available);
 
 #ifdef USE_MMAP
@@ -1784,7 +1829,7 @@ bio_sendfile(sendfile_ctx_t *ctx, bio_source_t *bio, int in_fd, off_t *offset,
 			old_len = ctx->map_end - ctx->map_start;
 			if (ctx->map) {
 			   	if (old_len != map_len) {
-					munmap(ctx->map, old_len);
+					vmm_munmap(ctx->map, old_len);
 					ctx->map = NULL;
 				} else {
 					flags |= MAP_FIXED;
@@ -1795,13 +1840,13 @@ bio_sendfile(sendfile_ctx_t *ctx, bio_source_t *bio, int in_fd, off_t *offset,
 			ctx->map_end = ctx->map_start + map_len;
 			g_assert(ctx->map_start < ctx->map_end);
 
-			addr = mmap(ctx->map, map_len, PROT_READ, flags, in_fd,
+			addr = vmm_mmap(ctx->map, map_len, PROT_READ, flags, in_fd,
 						ctx->map_start);
 
 			if (addr == MAP_FAILED) {
 				int saved_errno = errno;
 				if (ctx->map) {
-					munmap(ctx->map, old_len);
+					vmm_munmap(ctx->map, old_len);
 					old_len = 0;
 					flags &= ~MAP_FIXED;
 				}
@@ -1850,7 +1895,7 @@ bio_sendfile(sendfile_ctx_t *ctx, bio_source_t *bio, int in_fd, off_t *offset,
 		case (ssize_t) -1:
 			break;
 		case 0:
-			munmap(ctx->map, ctx->map_end - ctx->map_start);
+			vmm_munmap(ctx->map, ctx->map_end - ctx->map_start);
 			ctx->map = NULL;
 			break;
 		default:
@@ -1940,8 +1985,8 @@ bio_read(bio_source_t *bio, gpointer data, size_t len)
 	}
 
 	amount = len > available ? available : len;
-	if (GNET_PROPERTY(dbg) > 7)
-		printf("bsched_read(fd=%d, len=%d) available=%d\n",
+	if (GNET_PROPERTY(bsched_debug) > 7)
+		g_message("BSCHED bsched_read(fd=%d, len=%d) available=%d",
 			bio->wio->fd(bio->wio), (int) len, (int) available);
 
 	r = bio->wio->read(bio->wio, data, amount);
@@ -1955,11 +2000,9 @@ bio_read(bio_source_t *bio, gpointer data, size_t len)
 }
 
 /**
- * XXX: This is copy-paste bullshit:
- *
  * Read at most `len' bytes from `iov' to source's fd, as bandwidth permits,
  * `len' being determined by the size of the supplied I/O vector.
- * If we cannot write anything due to bandwidth constraints, return -1 with
+ * If we cannot read anything due to bandwidth constraints, return -1 with
  * errno set to EAGAIN.
  */
 ssize_t
@@ -2040,8 +2083,8 @@ bio_readv(bio_source_t *bio, struct iovec *iov, int iovcnt)
 	 *		--RAM, 17/03/2002
 	 */
 
-	if (GNET_PROPERTY(dbg) > 7)
-		printf("bio_readv(fd=%d, len=%d) available=%d\n",
+	if (GNET_PROPERTY(bsched_debug) > 7)
+		g_message("BSCHED bio_readv(fd=%d, len=%d) available=%d",
 			bio->wio->fd(bio->wio), (int) len, (int) available);
 
 	if (iovcnt > MAX_IOV_COUNT)
@@ -2140,29 +2183,16 @@ bws_read(bsched_bws_t bws, wrap_io_t *wio, gpointer data, size_t len)
  * Account for read data from UDP.
  */
 void
-bws_udp_count_read(int len)
+bws_udp_count_read(int len, gboolean dht)
 {
 	int count = BW_UDP_MSG + len;
 	bsched_t *bs;
 
-	bs = bsched_get(BSCHED_BWS_GIN_UDP);
+	bs = bsched_get(dht ? BSCHED_BWS_DHT_IN : BSCHED_BWS_GIN_UDP);
 	bsched_bw_update(bs, count, count);
 	bs->flags |= BS_F_DATA_READ;
 }
 
-/**
- * Account for written data to UDP.
- */
-void
-bws_udp_count_written(int len)
-{
-	int count = BW_UDP_MSG + len;
-	bsched_t *bs;
-
-	bs = bsched_get(BSCHED_BWS_GOUT_UDP);
-	bsched_bw_update(bs, count, count);
-}
-
 /**
  * Returns adequate b/w shaper depending on the socket type.
  *
@@ -2390,8 +2420,8 @@ bsched_heartbeat(bsched_t *bs, tm_t *tv)
 	delay = (int) ((tv->tv_sec - bs->last_period.tv_sec) * 1000 +
 		(tv->tv_usec - bs->last_period.tv_usec) / 1000);
 
-	if (GNET_PROPERTY(dbg) > 9)
-		printf("[%s] tv = %d,%d  bs = %d,%d, delay = %d\n",
+	if (GNET_PROPERTY(bsched_debug) > 9)
+		g_message("BSCHED [%s] tv = %d,%d  bs = %d,%d, delay = %d",
 			bs->name, (int) tv->tv_sec, (int) tv->tv_usec,
 			(int) bs->last_period.tv_sec, (int) bs->last_period.tv_usec,
 			delay);
@@ -2412,12 +2442,12 @@ bsched_heartbeat(bsched_t *bs, tm_t *tv)
 	 */
 
 	if (delay < bs->min_period) {
-		if (GNET_PROPERTY(dbg) && bs->last_period.tv_sec)
+		if (GNET_PROPERTY(bsched_debug) && bs->last_period.tv_sec)
 			g_warning("heartbeat (%s) noticed time jumped backwards (~%d ms)",
 				bs->name, bs->period - delay);
 		delay = bs->period;
 	} else if (delay > bs->max_period) {
-		if (GNET_PROPERTY(dbg) && bs->last_period.tv_sec)
+		if (GNET_PROPERTY(bsched_debug) && bs->last_period.tv_sec)
 			g_warning("heartbeat (%s) noticed time jumped forwards (~%d ms)",
 				bs->name, delay - bs->period);
 		delay = bs->period;
@@ -2480,7 +2510,7 @@ bsched_heartbeat(bsched_t *bs, tm_t *tv)
 	 * period, forget it: allow a full period at the nominal new settings.
 	 */
 
-	/* Forllowing is the overused "EMA" */
+	/* Following is the overused "EMA" */
 	correction = bs->bw_ema - bs->bw_stolen_ema - theoric;
 	correction = MAX(correction, overused);
 
@@ -2533,16 +2563,16 @@ bsched_heartbeat(bsched_t *bs, tm_t *tv)
 
 	bs->last_used = last_used;
 
-	if (GNET_PROPERTY(dbg) > 4) {
-		printf("bsched_timer(%s): delay=%d (EMA=%d), b/w=%d (EMA=%d), "
+	if (GNET_PROPERTY(bsched_debug) > 4) {
+		g_message("BSCHED bsched_timer(%s): delay=%d (EMA=%d), b/w=%d (EMA=%d), "
 			"overused=%d (EMA=%d) stolen=%d (EMA=%d) unwritten=%d "
-			"capped=%d (%d) used %d/%d\n",
+			"capped=%d (%d) used %d/%d",
 			bs->name, delay, bs->period_ema, bs->bw_actual, bs->bw_ema,
 			overused, bs->bw_ema - bs->bw_stolen_ema - theoric,
 			bs->bw_stolen, bs->bw_stolen_ema, bs->bw_unwritten,
 			last_capped, bs->bw_capped, bs->last_used, bs->count);
-		printf("    -> b/w delta=%d, max=%d, slot=%d, first=%d "
-			"(target %d B/s, %d slot%s, real %.02f B/s)\n",
+		g_message("BSCHED    -> b/w delta=%d, max=%d, slot=%d, first=%d "
+			"(target %d B/s, %d slot%s, real %.02f B/s)",
 			bs->bw_delta, bs->bw_max,
 			bs->count ? bs->bw_max / bs->count : 0,
 			bs->count ? (bs->bw_max + bs->bw_capped) / bs->count : 0,
@@ -2663,13 +2693,13 @@ bsched_stealbeat(bsched_t *bs)
 			xbs->bw_urgent -= amount;
 			underused -= amount;
 
-			if (GNET_PROPERTY(dbg) > 4)
-				printf("b/w sched \"%s\" urgently giving %d bytes to \"%s\""
-					" (still wants %d bytes urgently)\n",
+			if (GNET_PROPERTY(bsched_debug) > 4)
+				g_message("BSCHED b/w sched \"%s\" urgently giving %d bytes "
+					"to \"%s\" (still wants %d bytes urgently)",
 					bs->name, amount, xbs->name, xbs->bw_urgent);
 
 			if (underused <= 0)			/* Nothing left to redistribute */
-				return;
+				goto done;
 		}
 	}
 
@@ -2686,9 +2716,9 @@ bsched_stealbeat(bsched_t *bs)
 			bsched_t *xbs = l->data;
 			xbs->bw_stolen += underused / steal_count;
 
-			if (GNET_PROPERTY(dbg) > 4)
-				printf("b/w sched \"%s\" evenly giving %d bytes to \"%s\"\n",
-					bs->name, underused / steal_count, xbs->name);
+			if (GNET_PROPERTY(bsched_debug) > 4)
+				g_message("BSCHED b/w sched \"%s\" evenly giving %d bytes "
+					"to \"%s\"", bs->name, underused / steal_count, xbs->name);
 		}
 	} else {
 		for (l = all_used; l; l = g_slist_next(l)) {
@@ -2705,12 +2735,14 @@ bsched_stealbeat(bsched_t *bs)
 			else
 				xbs->bw_stolen += (int) amount;
 
-			if (GNET_PROPERTY(dbg) > 4)
-				printf("b/w sched \"%s\" giving %d bytes to \"%s\"\n",
+			if (GNET_PROPERTY(bsched_debug) > 4)
+				g_message("BSCHED b/w sched \"%s\" giving %d bytes to \"%s\"",
 					bs->name, (int) amount, xbs->name);
 		}
-		g_slist_free(all_used);
 	}
+
+done:
+	g_slist_free(all_used);
 }
 
 /**
@@ -2767,8 +2799,8 @@ bsched_timer(void)
 
 	bws_out_ema += (out_used >> 6) - (bws_out_ema >> 6);	/* Slow EMA */
 
-	if (GNET_PROPERTY(dbg) > 4)
-		printf("Outgoing b/w EMA = %d bytes/s\n", bws_out_ema);
+	if (GNET_PROPERTY(bsched_debug) > 3)
+		g_message("BSCHED outgoing b/w EMA = %d bytes/s", bws_out_ema);
 
 	for (l = bws_in_list; l; l = g_slist_next(l)) {
 		bsched_bws_t bws = GPOINTER_TO_UINT(l->data);
@@ -2784,8 +2816,8 @@ bsched_timer(void)
 
 	bws_in_ema += (in_used >> 6) - (bws_in_ema >> 6);		/* Slow EMA */
 
-	if (GNET_PROPERTY(dbg) > 4)
-		printf("Incoming b/w EMA = %d bytes/s\n", bws_in_ema);
+	if (GNET_PROPERTY(bsched_debug) > 3)
+		g_message("BSCHED incoming b/w EMA = %d bytes/s", bws_in_ema);
 
 	/*
 	 * Don't simply rely on in_used > 0 since we fake input data when
@@ -2801,8 +2833,8 @@ bsched_timer(void)
 static gboolean
 true_expr(const char *expr)
 {
-	if (GNET_PROPERTY(dbg) > 0) {
-		g_message("%s", expr);
+	if (GNET_PROPERTY(bsched_debug) > 0) {
+		g_message("BSCHED %s", expr);
 	}
 	return TRUE;
 }
