@@ -1,5 +1,5 @@
 /*
- * $Id: bsched.h 16223 2009-03-20 21:48:03Z rmanfredi $
+ * $Id: bsched.h 17077 2009-10-16 20:51:38Z rmanfredi $
  *
  * Copyright (c) 2002-2003, Raphael Manfredi
  *
@@ -69,8 +69,6 @@ bio_source_t *bsched_source_add(bsched_bws_t bs, wrap_io_t *wio, guint32 flags,
 	inputevt_handler_t callback, gpointer arg);
 void bsched_source_remove(bio_source_t *bio);
 void bsched_set_bandwidth(bsched_bws_t bs, int bandwidth);
-bio_source_t *bsched_source_add(bsched_bws_t bs, wrap_io_t *wio, guint32 flags,
-	inputevt_handler_t callback, gpointer arg);
 void bio_add_callback(bio_source_t *bio,
 	inputevt_handler_t callback, gpointer arg);
 void bio_remove_callback(bio_source_t *bio);
@@ -95,8 +93,7 @@ void bws_sock_connect_failed(enum socket_type type);
 void bws_sock_closed(enum socket_type type, gboolean remote);
 gboolean bws_can_connect(enum socket_type type);
 
-void bws_udp_count_written(int len);
-void bws_udp_count_read(int len);
+void bws_udp_count_read(int len, gboolean dht);
 
 gboolean bsched_enough_up_bandwidth(void);
 gboolean bsched_saturated(bsched_bws_t bws);
