@@ -1,5 +1,5 @@
 /*
- * $Id: downloads.h 16292 2009-03-29 12:37:30Z rmanfredi $
+ * $Id: downloads.h 17312 2010-02-27 18:53:49Z rmanfredi $
  *
  * Copyright (c) 2001-2003, Raphael Manfredi
  *
@@ -64,7 +64,6 @@ void download_stop(struct download *, download_status_t,
 void download_stop_v(struct download *d, download_status_t new_status,
     const char * reason, va_list ap);
 void download_push_ack(struct gnutella_socket *);
-void download_fallback_to_push(struct download *, gboolean, gboolean);
 void download_forget(struct download *, gboolean unavailable);
 gboolean download_start_prepare(struct download *d);
 gboolean download_start_prepare_running(struct download *d);
@@ -78,6 +77,8 @@ void download_redirect_to_server(struct download *d,
 		const host_addr_t addr, guint16 port);
 void download_actively_queued(struct download *d, gboolean queued);
 
+void download_add_push_proxy(const struct guid *guid,
+	host_addr_t addr, guint16 port);
 void download_add_push_proxies(const struct guid *,
 	gnet_host_t *proxies, int proxy_count);
 void download_proxy_dht_lookup_done(const struct guid *);
@@ -102,6 +103,11 @@ void download_proxy_newstate(struct download *d);
 void download_proxy_sent(struct download *d);
 void download_proxy_failed(struct download *d);
 
+gboolean download_known_guid(const struct guid *guid,
+	host_addr_t *addr, guint16 *port, sequence_t **proxies);
+void download_got_push_proxies(const struct guid *guid,
+	const gnet_host_vec_t *proxies);
+
 struct download * download_browse_start(
 	const char *hostname, host_addr_t addr, guint16 port,
 	const struct guid *, const gnet_host_vec_t *proxies,
@@ -123,6 +129,7 @@ void download_maybe_finished(struct download *d);
 gboolean download_handle_http(const char *url);
 gboolean download_is_stalled(struct download *);
 gboolean download_is_alive(const struct download *);
+gboolean download_is_completed_filename(const char *name);
 
 gboolean download_remove(struct download *d);
 void download_abort(struct download *d);
