@@ -1,5 +1,5 @@
 /*
- * $Id: dq.c 16169 2009-03-15 18:23:47Z rmanfredi $
+ * $Id: dq.c 17239 2010-02-13 12:25:13Z rmanfredi $
  *
  * Copyright (c) 2004, Raphael Manfredi
  *
@@ -35,7 +35,7 @@
 
 #include "common.h"
 
-RCSID("$Id: dq.c 16169 2009-03-15 18:23:47Z rmanfredi $")
+RCSID("$Id: dq.c 17239 2010-02-13 12:25:13Z rmanfredi $")
 
 #ifdef I_MATH
 #include <math.h>	/* For pow() */
@@ -62,8 +62,8 @@ RCSID("$Id: dq.c 16169 2009-03-15 18:23:47Z rmanfredi $")
 #include "lib/cq.h"
 #include "lib/endian.h"
 #include "lib/glib-missing.h"
-#include "lib/misc.h"
 #include "lib/pmsg.h"
+#include "lib/stringify.h"
 #include "lib/tm.h"
 #include "lib/walloc.h"
 
@@ -767,11 +767,6 @@ dq_fill_next_up(dquery_t *dq, struct next_up *nv, int ncount)
 /**
  * Forward message to all the leaves but the one originating this query,
  * according to their QRP tables.
- *
- * @attention
- * NB: In order to avoid qrt_build_query_target() selecting neighbouring
- * ultra nodes that support last-hop QRP, we ensure the TTL is NOT 1.
- * This is why we somehow duplicate qrt_route_query() here.
  */
 static void
 dq_sendto_leaves(dquery_t *dq, gnutella_node_t *source)
@@ -781,13 +776,19 @@ dq_sendto_leaves(dquery_t *dq, gnutella_node_t *source)
 
 	dquery_check(dq);
 	head = cast_to_gconstpointer(pmsg_start(dq->mb));
+
+	/*
+	 * NB: In order to avoid qrt_build_query_target() selecting neighbouring
+	 * ultra nodes that support last-hop QRP, we ensure the TTL is 0.
+	 * This is why we somehow duplicate qrt_route_query() here.
+	 */
+
 	nodes = qrt_build_query_target(dq->qhv,
-				gnutella_header_get_hops(head),
-				MAX(gnutella_header_get_ttl(head), 2),
-				source);
+				gnutella_header_get_hops(head), 0, source);
+
 	if (GNET_PROPERTY(dq_debug) > 4)
 		g_message("DQ QRP %s (%d word%s%s) forwarded to %d/%d leaves",
-			gmsg_infostr_full(head),
+			gmsg_infostr_full(head, pmsg_written_size(dq->mb)),
 			qhvec_count(dq->qhv), qhvec_count(dq->qhv) == 1 ? "" : "s",
 			qhvec_has_urn(dq->qhv) ? " + URN" : "",
 			g_slist_length(nodes), GNET_PROPERTY(node_leaf_count));
@@ -814,7 +815,8 @@ dq_free(dquery_t *dq)
 	int i;
 
 	dquery_check(dq);
-	g_assert(g_hash_table_lookup(dqueries, &dq->qid) == dq);
+	g_assert((dq->flags & DQ_F_EXITING) ||
+		g_hash_table_lookup(dqueries, &dq->qid) == dq);
 
 	if (GNET_PROPERTY(dq_debug) > 19)
 		g_message("DQ[%s] %s(%d secs; +%d secs) node #%s ending: "
@@ -1055,7 +1057,7 @@ dq_results_expired(cqueue_t *unused_cq, gpointer obj)
 	 */
 
 	if (!(dq->flags & (DQ_F_LEAF_GUIDED|DQ_F_ROUTING_HITS))) {
-		if (GNET_PROPERTY(dq_debug))
+		if (GNET_PROPERTY(dq_debug) > 1)
 			g_message(
 				"DQ[%s] terminating unguided & unrouted (queried %u UP%s)",
 				dquery_id_to_string(dq->qid), dq->up_sent,
@@ -1329,7 +1331,7 @@ dq_send_next(dquery_t *dq)
 	 */
 
 	if (GNET_PROPERTY(current_peermode) != NODE_P_ULTRA) {
-		if (GNET_PROPERTY(dq_debug))
+		if (GNET_PROPERTY(dq_debug) > 1)
 			g_message("DQ[%s] terminating (no longer an ultra node)",
 				dquery_id_to_string(dq->qid));
 		goto terminate;
@@ -1343,7 +1345,7 @@ dq_send_next(dquery_t *dq)
 	results = dq_kept_results(dq);
 
 	if (dq->horizon >= DQ_MAX_HORIZON || results >= dq->max_results) {
-		if (GNET_PROPERTY(dq_debug))
+		if (GNET_PROPERTY(dq_debug) > 1)
 			g_message("DQ[%s] terminating "
 				"(UPs=%u, horizon=%u >= %d, %s results=%u >= %u)",
 				dquery_id_to_string(dq->qid), dq->up_sent, dq->horizon,
@@ -1362,7 +1364,7 @@ dq_send_next(dquery_t *dq)
 	 */
 
 	if (dq->results + dq->oob_results > dq->fin_results) {
-		if (GNET_PROPERTY(dq_debug))
+		if (GNET_PROPERTY(dq_debug) > 1)
 			g_message("DQ[%s] terminating "
 				"(UPs=%u, seen=%u + OOB=%u >= %u -- %s kept=%u)",
 				dquery_id_to_string(dq->qid), dq->up_sent,
@@ -1381,7 +1383,7 @@ dq_send_next(dquery_t *dq)
 		dq->up_sent >=
 			GNET_PROPERTY(max_connections) - GNET_PROPERTY(normal_connections)
 	) {
-		if (GNET_PROPERTY(dq_debug))
+		if (GNET_PROPERTY(dq_debug) > 1)
 			g_message("DQ[%s] terminating (queried UPs=%u >= %u)",
 				dquery_id_to_string(dq->qid), dq->up_sent,
 				GNET_PROPERTY(max_connections) -
@@ -1491,8 +1493,8 @@ dq_send_next(dquery_t *dq)
 
 	if (GNET_PROPERTY(dq_debug) > 1)
 		g_message("DQ[%s] (%d secs) timeout set to %d ms (pending=%d)",
-			dquery_id_to_string(dq->qid), (int) (tm_time() - dq->start), timeout,
-			dq->pending);
+			dquery_id_to_string(dq->qid), (int) (tm_time() - dq->start),
+			timeout, dq->pending);
 
 	dq->results_ev = cq_insert(callout_queue, timeout, dq_results_expired, dq);
 	return;
@@ -1675,7 +1677,7 @@ dq_common_init(dquery_t *dq)
 			gnutella_msg_search_get_text(packet));
 	}
 
-	if (GNET_PROPERTY(dq_debug)) {
+	if (GNET_PROPERTY(dq_debug) > 1) {
 		gconstpointer packet;
 		guint16 flags;
 
@@ -1805,7 +1807,7 @@ dq_launch_net(gnutella_node_t *n, query_hashvec_t *qhv)
 
 	if (
 		NULL != oob_proxy_muid_proxied(gnutella_header_get_muid(&n->header)) ||	
-		(flags_valid && !(flags && QUERY_F_OOB_REPLY))
+		(flags_valid && !(flags & QUERY_F_OOB_REPLY))
 	)
 		dq->flags |= DQ_F_ROUTING_HITS;
 
@@ -1851,6 +1853,7 @@ void
 dq_launch_local(gnet_search_t handle, pmsg_t *mb, query_hashvec_t *qhv)
 {
 	dquery_t *dq;
+	const char *msg = NULL;
 
 	/*
 	 * Local queries are queued in the global SQ, for slow dispatching.
@@ -1858,13 +1861,13 @@ dq_launch_local(gnet_search_t handle, pmsg_t *mb, query_hashvec_t *qhv)
 	 */
 
 	if (GNET_PROPERTY(current_peermode) != NODE_P_ULTRA) {
-		if (GNET_PROPERTY(dq_debug))
-			g_warning("ignoring dynamic query \"%s\": no longer an ultra node",
-				gnutella_msg_search_get_text(pmsg_start(mb)));
+		msg = "no longer an ultra node";
+		goto ignore;
+	}
 
-		pmsg_free(mb);
-		qhvec_free(qhv);
-		return;
+	if (0 == qhvec_count(qhv)) {
+		msg = "empty hash vector";
+		goto ignore;
 	}
 
 	/*
@@ -1892,6 +1895,15 @@ dq_launch_local(gnet_search_t handle, pmsg_t *mb, query_hashvec_t *qhv)
 	dq_common_init(dq);
 	dq_sendto_leaves(dq, NULL);
 	dq_send_probe(dq);
+	return;
+
+ignore:
+	if (GNET_PROPERTY(dq_debug))
+		g_warning("ignoring local dynamic query \"%s\": %s",
+			gnutella_msg_search_get_text(pmsg_start(mb)), msg);
+
+	pmsg_free(mb);
+	qhvec_free(qhv);
 }
 
 /**
@@ -1915,7 +1927,7 @@ dq_node_removed(const node_id_t node_id)
 
 		dquery_check(dq);
 
-		if (GNET_PROPERTY(dq_debug))
+		if (GNET_PROPERTY(dq_debug) > 1)
 			g_message("DQ[%s] terminated by node #%s removal (queried %u UP%s)",
 				dquery_id_to_string(dq->qid), node_id_to_string(dq->node_id),
 				dq->up_sent, dq->up_sent == 1 ? "" : "s");
@@ -2176,7 +2188,7 @@ dq_got_query_status(const struct guid *muid,
 	 */
 
 	if (kept == 0xffff) {
-		if (GNET_PROPERTY(dq_debug))
+		if (GNET_PROPERTY(dq_debug) > 1)
 			g_message("DQ[%s] terminating at user's request (queried %u UP%s)",
 				dquery_id_to_string(dq->qid), dq->up_sent,
 				dq->up_sent == 1 ? "" : "s");
