@@ -1,7 +1,7 @@
 /*
- * $Id: extensions.c 16242 2009-03-24 19:04:20Z cbiere $
+ * $Id: extensions.c 17310 2010-02-27 17:28:36Z rmanfredi $
  *
- * Copyright (c) 2002-2003, Raphael Manfredi
+ * Copyright (c) 2002-2003, 2009, Raphael Manfredi
  *
  *----------------------------------------------------------------------
  * This file is part of gtk-gnutella.
@@ -31,11 +31,12 @@
  *
  * @author Raphael Manfredi
  * @date 2002-2003
+ * @date 2009
  */
 
 #include "common.h"
 
-RCSID("$Id: extensions.c 16242 2009-03-24 19:04:20Z cbiere $")
+RCSID("$Id: extensions.c 17310 2010-02-27 17:28:36Z rmanfredi $")
 
 #include <zlib.h>
 
@@ -44,7 +45,8 @@ RCSID("$Id: extensions.c 16242 2009-03-24 19:04:20Z cbiere $")
 
 #include "lib/atoms.h"
 #include "lib/ascii.h"
-#include "lib/misc.h"
+#include "lib/stringify.h"
+#include "lib/halloc.h"
 #include "lib/walloc.h"
 #include "lib/override.h"		/* Must be the last header included */
 
@@ -58,7 +60,7 @@ RCSID("$Id: extensions.c 16242 2009-03-24 19:04:20Z cbiere $")
 /**
  * An extension descriptor.
  *
- * The extension block is structured thustly:
+ * The extension block is structured thusly:
  *
  *    - <.................len.......................>
  *    - <..headlen.><..........paylen...............>
@@ -163,6 +165,7 @@ static const struct rwtable ggeptable[] =
 	GGEP_ID(ALT_TLS),	/**< TLS-capability bitmap for GGEP ALT */
 	GGEP_ID(BH),		/**< Browseable host indication */
 	GGEP_ID(C),			/**< Result Code in HEAD Pongs */
+	GGEP_ID(CHAT),		/**< CHAT indication in query hit trailer */
 	GGEP_ID(CT),		/**< Resource creation time */
 	GGEP_ID(DHT),		/**< DHT version and flags, in pongs */
 	GGEP_ID(DHTIPP),	/**< DHT nodes in packed IP:Port format (pongs) */
@@ -181,6 +184,7 @@ static const struct rwtable ggeptable[] =
 	GGEP_ID(IPP_TLS),	/**< TLS-capability bitmap for GGEP IPP */
 	GGEP_ID(LF),		/**< Large file size in qhits */
 	GGEP_ID(LOC),		/**< Locale preferences, for clustering  */
+	GGEP_ID(M),			/**< MIME type for queries (byte code) */
 	GGEP_ID(NP),		/**< do Not Proxy (queries; OOB) */
 	GGEP_ID(P),			/**< Push alt-locs in HEAD Pongs */
 	GGEP_ID(PATH),		/**< Shared file path, in query hits */
@@ -200,10 +204,13 @@ static const struct rwtable ggeptable[] =
 	GGEP_ID(V),			/**< Vendor code, in HEAD Pongs */
 	GGEP_ID(VC),		/**< Vendor code, in pongs */
 	GGEP_ID(VMSG),		/**< Array of supported vendor message codes */
+	GGEP_ID(XQ),		/**< eXtended Query; for longer query strings */
+	GGEP_ID(avail),		/**< Partial file, available length in ALOC v0.1 */
 	{ "client-id", EXT_T_GGEP_client_id }, /**< GUID in ALOC v0.0 */
 	GGEP_ID(features),	/**< Unknown value, PROX v0.0 */
 	GGEP_ID(firewalled),/**< Firewalled status in ALOC v0.0 */
 	{ "fwt-version", EXT_T_GGEP_fwt_version }, /**< FW2FW version, PROX v0.0 */
+	GGEP_ID(guid),		/**< Servent's GUID in NOPE v0.0 */
 	GGEP_ID(length),	/**< File length in ALOC v0.1 */
 	GGEP_ID(port),		/**< Servent's Port in ALOC v0.0 */
 	GGEP_ID(proxies),	/**< Push proxies in PROX v0.0 */
@@ -358,9 +365,9 @@ ext_names_kv_free(gpointer key, gpointer value, gpointer unused_udata)
 {
 	(void) unused_udata;
 
-	if (0 != strcmp(key, value)) {
-		G_FREE_NULL(value);
-	}
+	if (key != value)
+		HFREE_NULL(value);
+
     wfree(key, 1 + strlen(key));
 
 	return TRUE;
@@ -1134,7 +1141,7 @@ ext_ggep_inflate(const char *buf, int len, guint16 *retlen, const char *name)
 
 	rsize = len * 2;				/* Assume a 50% compression ratio */
 	rsize = MIN(rsize, GGEP_MAXLEN);
-	result = g_malloc(rsize);
+	result = halloc(rsize);
 
 	/*
 	 * Prepare call to inflate().
@@ -1148,25 +1155,28 @@ ext_ggep_inflate(const char *buf, int len, guint16 *retlen, const char *name)
 	for (;;) {
 		/*
 		 * Resize output buffer if needed.
-		 * Never grow the result buffer to more than MAX_PAYLOAD_LEN bytes.
+		 * Never grow the result buffer to more than GGEP_MAXLEN bytes.
 		 */
 
 		if (rsize == inflated) {
-			rsize += MAX(len, GGEP_GROW);
-			rsize = MIN(rsize, GGEP_MAXLEN);
-
-			if (rsize == inflated) {		/* Reached maximum size! */
-				g_warning("GGEP payload \"%s\" would be larger than %d bytes",
-					name, GGEP_MAXLEN);
+			if (rsize >= GGEP_MAXLEN) {		/* Reached maximum size! */
+				if (GNET_PROPERTY(ggep_debug)) {
+					g_warning("GGEP payload \"%s\" (%d byte%s) would "
+						"decompress to more than %d bytes",
+						name, len, 1 == len ? "" : "s", GGEP_MAXLEN);
+				}
 				failed = TRUE;
 				break;
 			}
 
-			g_assert(rsize > inflated);
+			rsize += MAX(len, GGEP_GROW);
+			rsize = MIN(rsize, GGEP_MAXLEN);
 
-			result = g_realloc(result, rsize);
+			result = hrealloc(result, rsize);
 		}
 
+		g_assert(rsize > inflated);
+
 		inz->next_out = (guchar *) result + inflated;
 		inz->avail_out = rsize - inflated;
 
@@ -1179,12 +1189,29 @@ ext_ggep_inflate(const char *buf, int len, guint16 *retlen, const char *name)
 
 		g_assert(inflated <= rsize);
 
-		if (ret == Z_STREAM_END)				/* All done! */
+		if (ret == Z_STREAM_END)			/* All done! */
 			break;
 
+		if (ret == Z_BUF_ERROR) {			/* Needs more output space */
+			if (rsize == inflated)
+				continue;
+
+			if (GNET_PROPERTY(ggep_debug)) {
+				g_warning("GGEP payload \"%s\" does not "
+					"decompress properly (consumed %d/%d byte%s)",
+					name, len - inz->avail_in, len, 1 == len ? "" : "s");
+			}
+			failed = TRUE;
+			break;
+		}
+
 		if (ret != Z_OK) {
-			g_warning("decompression of GGEP payload \"%s\" failed: %s",
-				name, zlib_strerror(ret));
+			if (GNET_PROPERTY(ggep_debug)) {
+				g_warning("decompression of GGEP payload \"%s\""
+					" (%d byte%s) failed: %s [consumed %d, inflated into %d]",
+					name, len, 1 == len ? "" : "s", zlib_strerror(ret),
+					len - inz->avail_in, inflated);
+			}
 			failed = TRUE;
 			break;
 		}
@@ -1195,18 +1222,21 @@ ext_ggep_inflate(const char *buf, int len, guint16 *retlen, const char *name)
 	 */
 
 	ret = inflateEnd(inz);
-	if (ret != Z_OK)
-		g_warning("while freeing decompressor for GGEP payload \"%s\": %s",
-			name, zlib_strerror(ret));
+	if (ret != Z_OK) {
+		if (GNET_PROPERTY(ggep_debug)) {
+			g_warning("while freeing decompressor for GGEP payload \"%s\": %s",
+				name, zlib_strerror(ret));
+		}
+	}
 
 	wfree(inz, sizeof(*inz));
 
 	/*
-	 * @return NULL on error, fill `retlen' if OK.
+	 * return NULL on error.
 	 */
 
 	if (failed) {
-		G_FREE_NULL(result);
+		HFREE_NULL(result);
 		return NULL;
 	}
 
@@ -1214,6 +1244,11 @@ ext_ggep_inflate(const char *buf, int len, guint16 *retlen, const char *name)
 
 	g_assert(*retlen == inflated);	/* Make sure it was not truncated */
 
+	if (GNET_PROPERTY(ggep_debug) > 5) {
+		g_message("decompressed GGEP payload \"%s\" (%d byte%s) into %d",
+			name, len, 1 == len ? "" : "s", inflated);
+	}
+
 	return result;					/* OK, successfully inflated */
 }
 
@@ -1257,13 +1292,14 @@ ext_ggep_decode(const extvec_t *e)
 		uncobs = walloc(plen);		/* At worse slightly oversized */
 		uncobs_len = plen;
 
-		if (!d->ext_ggep_deflate) {
-			if (!cobs_decode_into(pbase, plen, uncobs, plen, &result)) {
-				if (GNET_PROPERTY(ggep_debug))
-					g_warning("unable to decode COBS buffer");
-				goto out;
-			}
+		if (!cobs_decode_into(pbase, plen, uncobs, plen, &result)) {
+			if (GNET_PROPERTY(ggep_debug))
+				g_warning("unable to decode COBS buffer for GGEP \"%s\"",
+					d->ext_ggep_id);
+			goto out;
+		}
 
+		if (!d->ext_ggep_deflate) {
 			g_assert(result <= plen);
 
 			d->ext_payload = uncobs;
@@ -1272,12 +1308,6 @@ ext_ggep_decode(const extvec_t *e)
 
 			return;
 		} else {
-			if (!cobs_decode_into(pbase, plen, uncobs, plen, &result)) {
-				if (GNET_PROPERTY(ggep_debug))
-					g_warning("unable to decode COBS buffer");
-				goto out;
-			}
-
 			g_assert(result <= plen);
 
 			/*
@@ -1300,7 +1330,7 @@ ext_ggep_decode(const extvec_t *e)
 
 	g_assert(d->ext_ggep_deflate);
 
-	d->ext_rpaylen = 0;			/* Signals it was malloc()'ed */
+	d->ext_rpaylen = 0;			/* Signals it was halloc()'ed */
 	d->ext_payload =
 		ext_ggep_inflate(pbase, plen, &d->ext_paylen, d->ext_ggep_id);
 
@@ -1315,12 +1345,12 @@ out:
 	 */
 
 	if (d->ext_payload == NULL) {
-		if (GNET_PROPERTY(dbg) || GNET_PROPERTY(ggep_debug))
+		if (GNET_PROPERTY(ggep_debug)) {
 			g_warning("unable to get GGEP \"%s\" %d-byte payload (%s)",
 				d->ext_ggep_id, d->ext_phys_paylen,
 				(d->ext_ggep_deflate && d->ext_ggep_cobs) ? "COBS + deflated" :
 				d->ext_ggep_cobs ? "COBS" : "deflated");
-
+		}
 		d->ext_paylen = 0;
 		d->ext_payload = d->ext_phys_payload;
 	}
@@ -1585,7 +1615,7 @@ ext_reset(extvec_t *exv, int exvcnt)
 		if (d->ext_payload != NULL && d->ext_payload != d->ext_phys_payload) {
 			gpointer p = deconstify_gpointer(d->ext_payload);
 			if (d->ext_rpaylen == 0) {
-				G_FREE_NULL(p);
+				HFREE_NULL(p);
 			} else {
 				wfree(p, d->ext_rpaylen);
 				p = NULL;
