@@ -1,5 +1,5 @@
 /*
- * $Id: features.c 16279 2009-03-28 19:28:46Z rmanfredi $
+ * $Id: features.c 17305 2010-02-27 11:36:26Z rmanfredi $
  *
  * Copyright (c) 2001-2003, Raphael Manfredi
  *
@@ -35,11 +35,12 @@
 
 #include "common.h"
 
-RCSID("$Id: features.c 16279 2009-03-28 19:28:46Z rmanfredi $")
+RCSID("$Id: features.c 17305 2010-02-27 11:36:26Z rmanfredi $")
 
 #include "features.h"
 
 #include "if/gnet_property_priv.h"
+#include "if/core/main.h"
 
 #include "lib/ascii.h"
 #include "lib/header.h"
@@ -53,6 +54,7 @@ struct header_x_feature {
 	int major;
 	int minor;
 	const gboolean *guard;
+	gboolean (*guardfn)(void);
 };
 
 struct features {
@@ -126,10 +128,35 @@ header_features_add_guarded(xfeature_t xf,
 	item->major = major;
 	item->minor = minor;
 	item->guard = guard;
+	item->guardfn = NULL;
 
 	features->list = g_list_append(features->list, item);
 }
+ 
+/**
+ * Add conditional support for feature ``name'': if at run-time the value
+ * returned by the ``guardfn'' function is FALSE, the feature is not emitted.
+ */
+void
+header_features_add_guarded_function(xfeature_t xf,
+	const char *name, int major, int minor, gboolean (*guardfn)(void))
+{
+	struct header_x_feature *item;
+	struct features *features;
 
+	features = features_get(xf);
+	g_return_if_fail(features);
+
+	item = walloc(sizeof *item);
+	item->name = g_strdup(name);
+	item->major = major;
+	item->minor = minor;
+	item->guard = NULL;
+	item->guardfn = guardfn;
+
+	features->list = g_list_append(features->list, item);
+}
+ 
 /**
  * Add support for feature ``name'' with the specified version to the
  * X-Features header.
@@ -180,6 +207,9 @@ header_features_generate(xfeature_t xf, char *dst, size_t len, size_t *rw)
 		if (item->guard && !*item->guard)
 			continue;
 
+		if (item->guardfn && !(*item->guardfn)())
+			continue;
+
 		gm_snprintf(buf, sizeof buf, "%s/%d.%d",
 			item->name, item->major, item->minor);
 
@@ -188,11 +218,12 @@ header_features_generate(xfeature_t xf, char *dst, size_t len, size_t *rw)
 
 	header_fmt_end(fmt);
 
-	if ((size_t) header_fmt_length(fmt) < len - *rw) {
-		*rw += gm_snprintf(&dst[*rw], len - *rw, "%s", header_fmt_string(fmt));
+	if (header_fmt_length(fmt) < len - *rw) {
+		*rw += clamp_strncpy(&dst[*rw], len - *rw,
+				header_fmt_string(fmt), header_fmt_length(fmt));
 	}
 
-	header_fmt_free(fmt);
+	header_fmt_free(&fmt);
 }
 
 /**
@@ -204,15 +235,15 @@ gboolean
 header_get_feature(const char *name, const header_t *header,
 	guint *major, guint *minor)
 {
-	char *buf = NULL;
-	char *start;
+	static const char x_features[] = "X-Features";
+	char *buf, *start;
 
 	if (major)
 		*major = 0;
 	if (minor)
 		*minor = 0;
 
-	buf = header_get(header, "X-Features");
+	buf = header_get(header, x_features);
 
 	/*
 	 * We could also try to scan for the header: name, so this would
@@ -225,7 +256,10 @@ header_get_feature(const char *name, const header_t *header,
 		 * Actually the 'specs' say we should assume it is supported if the
 		 * X-Features header is not there. But I wouldn't count on it, and
 		 * it was only for "legacy" attributes in the HTTP file exchange.
-		 * Better safe than sorry.
+		 *
+		 * Also, for optimization purposes, the X-Features line will be sent
+		 * once per persistent HTTP connection, as the client is expected to
+		 * cache the supported features.
 		 */
 
 		return FALSE;
@@ -247,27 +281,20 @@ header_get_feature(const char *name, const header_t *header,
 		if (buf == start)
 			break;
 
-		pc = *(buf - 1);
-		if (is_ascii_space(pc) || pc == ',' || pc == ';')
-			break;			/* Found it! */
-
 		/*
 		 * Since we're looking for whole words separated by a space or the
 		 * regular header punctuation, the next match can't occur before
 		 * the end of the current string we matched...
 		 */
 
+		pc = *(buf - 1);
 		buf += strlen(name);
-	}
 
-	buf += strlen(name);		/* Should now be on the "/" sep */
+		if (*buf != '/')
+			continue;		/* Matched "barcode" when looking for "bar" */
 
-	if (*buf != '/') {
-		g_warning("[header] Malformed X-Features header, ignoring");
-		if (GNET_PROPERTY(dbg) > 2)
-			header_dump(stderr, header, NULL);
-
-		return FALSE;
+		if (is_ascii_space(pc) || pc == ',' || pc == ';')
+			break;			/* Found it! */
 	}
 
 	buf++;
@@ -275,7 +302,6 @@ header_get_feature(const char *name, const header_t *header,
 	if (*buf == '\0')
 		return FALSE;
 
-	
 	return 0 == parse_major_minor(buf, NULL, major, minor);
 }
 
