@@ -1,5 +1,5 @@
 /*
- * $Id: file_object.c 15608 2008-08-14 17:41:30Z cbiere $
+ * $Id: file_object.c 16990 2009-09-25 15:23:43Z rmanfredi $
  *
  * Copyright (c) 2006, Christian Biere
  *
@@ -86,15 +86,16 @@
 
 #include "common.h"
 
-RCSID("$Id: file_object.c 15608 2008-08-14 17:41:30Z cbiere $")
+RCSID("$Id: file_object.c 16990 2009-09-25 15:23:43Z rmanfredi $")
 
 #include "file_object.h"
 
 #include "lib/atoms.h"
 #include "lib/compat_pio.h"
+#include "lib/fd.h"
 #include "lib/file.h"
 #include "lib/iovec.h"
-#include "lib/misc.h"
+#include "lib/path.h"
 #include "lib/walloc.h"
 
 #include "lib/override.h"       /* Must be the last header included */
@@ -103,15 +104,15 @@ static GHashTable *ht_file_objects_rdonly;	/* read-only file objects */
 static GHashTable *ht_file_objects_wronly;	/* write-only file objects */
 static GHashTable *ht_file_objects_rdwr;	/* read+write-able file objects */
 
-enum file_object_magic { FILE_OBJECT_MAGIC = 0xeb084325 };	/**< Magic number */
+enum file_object_magic { FILE_OBJECT_MAGIC = 0x6b084325 };	/**< Magic number */
 
 struct file_object {
+	enum file_object_magic magic;
 	const char *pathname;	/* atom */
 	int ref_count;
 	int fd;
 	int accmode;	/* O_RDONLY, O_WRONLY, O_RDWR */
 	int removed;
-	enum file_object_magic magic;
 };
 
 /**
@@ -253,8 +254,19 @@ file_object_find(const char * const pathname, int accmode)
 	g_return_val_if_fail(is_absolute_path(pathname), NULL);
 
 	fo = g_hash_table_lookup(file_object_mode_get_table(O_RDWR), pathname);
-	if (!fo && O_RDWR != accmode) {
-		fo = g_hash_table_lookup(file_object_mode_get_table(accmode), pathname);
+
+	/*
+	 * We need to find a more specific file object if looking for O_WRONLY
+	 * or O_RDONLY ones.
+	 */
+
+	if (O_RDWR != accmode) {
+		struct file_object *xfo;
+		xfo = g_hash_table_lookup(file_object_mode_get_table(accmode), pathname);
+		if (xfo != NULL) {
+			g_assert(xfo->accmode == accmode);
+			fo = xfo;
+		}
 	}
 
 	if (fo) {
@@ -330,8 +342,7 @@ file_object_free(struct file_object * const fo)
 		file_object_remove(fo);
 	}
 
-	close(fo->fd);
-	fo->fd = -1;
+	fd_close(&fo->fd, FALSE);
 	atom_str_free_null(&fo->pathname);
 	fo->magic = 0;
 	wfree(fo, sizeof *fo);
