@@ -1,5 +1,5 @@
 /*
- * $Id: gdht.c 16025 2009-03-01 12:33:12Z rmanfredi $
+ * $Id: gdht.c 17381 2010-03-18 08:40:15Z rmanfredi $
  *
  * Copyright (c) 2008, Raphael Manfredi
  *
@@ -27,7 +27,7 @@
  * @ingroup core
  * @file
  *
- * Gnutella DHT interface.
+ * Gnutella DHT "get" interface.
  *
  * @author Raphael Manfredi
  * @date 2008
@@ -35,7 +35,7 @@
 
 #include "common.h"
 
-RCSID("$Id: gdht.c 16025 2009-03-01 12:33:12Z rmanfredi $")
+RCSID("$Id: gdht.c 17381 2010-03-18 08:40:15Z rmanfredi $")
 
 #include "gdht.h"
 #include "fileinfo.h"
@@ -45,6 +45,8 @@ RCSID("$Id: gdht.c 16025 2009-03-01 12:33:12Z rmanfredi $")
 #include "hostiles.h"
 #include "downloads.h"
 #include "gnet_stats.h"
+#include "settings.h"
+#include "ipp_cache.h"
 
 #include "if/core/guid.h"
 #include "if/dht/lookup.h"
@@ -58,7 +60,7 @@ RCSID("$Id: gdht.c 16025 2009-03-01 12:33:12Z rmanfredi $")
 #include "lib/walloc.h"
 #include "lib/override.h"		/* Must be the last header included */
 
-#define MAX_PROXIES		8		/**< Max push-proxies we collect from a PROX */
+#define MAX_PROXIES		32		/**< Max push-proxies we collect from a PROX */
 
 /**
  * Hash table holding all the pending lookups by KUID.
@@ -90,11 +92,11 @@ sha1_lookup_check(const struct sha1_lookup *slk)
 }
 
 typedef enum {
-	GUID_LOOKUP_MAGIC = 0xc65531c7U
+	GUID_LOOKUP_MAGIC = 0x465531c7U
 } glk_magic_t;
 
 /**
- * Context for PROX lookups.
+ * Context for PROX / NOPE lookups.
  */
 struct guid_lookup {
 	glk_magic_t magic;
@@ -102,6 +104,7 @@ struct guid_lookup {
 	const guid_t *guid;		/**< Servent's GUID (atom) */
 	host_addr_t addr;		/**< Servent's address */
 	guint16 port;			/**< Servent's port */
+	unsigned nope:1;		/**< Was looking for a NOPE instead of a PROX */
 };
 
 static inline void
@@ -111,13 +114,16 @@ guid_lookup_check(const struct guid_lookup *glk)
 	g_assert(GUID_LOOKUP_MAGIC == glk->magic);
 }
 
+static void gdht_guid_found(const kuid_t *kuid,
+	const lookup_val_rs_t *rs, gpointer arg);
+
 /**
  * Convert a SHA1 to the proper Kademlia key for lookups.
  *
  * @return KUID atom for SHA1 lookup: KUID = SHA1.
  */
-static kuid_t *
-kuid_from_sha1(const sha1_t *sha1)
+const kuid_t *
+gdht_kuid_from_sha1(const sha1_t *sha1)
 {
 	return kuid_get_atom((const kuid_t *) sha1);	/* Identity */
 }
@@ -127,8 +133,8 @@ kuid_from_sha1(const sha1_t *sha1)
  *
  * @return KUID atom for GUID lookup: KUID = SHA1(GUID).
  */
-static const kuid_t *
-kuid_from_guid(const struct guid *guid)
+const kuid_t *
+gdht_kuid_from_guid(const guid_t *guid)
 {
 	SHA1Context ctx;
 	struct sha1 digest;
@@ -226,6 +232,8 @@ gdht_handle_aloc(const lookup_val_rc_t *rc, const fileinfo_t *fi)
 	gboolean tls = FALSE;
 	filesize_t filesize = 0;
 	guint32 flags = 0;
+	char host[MAX_HOSTLEN];
+	const char *hostname = NULL;
 
 	g_assert(DHT_VT_ALOC == rc->type);
 
@@ -272,6 +280,15 @@ gdht_handle_aloc(const lookup_val_rc_t *rc, const fileinfo_t *fi)
 				has_tth = TRUE;
 			}
 			break;
+		case EXT_T_GGEP_HNAME:		/* GTKG-added key to ALOCs */
+			{
+				ggept_status_t ret;
+
+				ret = ggept_hname_extract(e, host, sizeof host);
+				if (GGEP_OK == ret)
+					hostname = host;
+			}
+			break;
 		default:
 			if (GNET_PROPERTY(ggep_debug) > 1 && e->ext_type == EXT_GGEP) {
 				paylen = ext_paylen(e);
@@ -324,10 +341,10 @@ gdht_handle_aloc(const lookup_val_rc_t *rc, const fileinfo_t *fi)
 	}
 
 	/*
-	 * Check that filesize matches.
+	 * Check that filesize matches, if any supplied and if known.
 	 */
 
-	if (filesize && fi->size != filesize) {
+	if (filesize != 0 && fi->size != 0 && fi->size != filesize) {
 		if (GNET_PROPERTY(download_debug))
 			g_warning("discarding %s from %s for %s: "
 				"we have size=%lu, ALOC says %lu",
@@ -370,8 +387,8 @@ gdht_handle_aloc(const lookup_val_rc_t *rc, const fileinfo_t *fi)
 		flags |= SOCK_F_TLS;
 
 	download_dht_auto_new(filepath_basename(fi->pathname),
-		fi->size,
-		rc->addr, port,
+		fi->size != 0 ? fi->size : filesize,
+		hostname, rc->addr, port,
 		&guid,
 		fi->sha1,
 		has_tth ? &tth : NULL,
@@ -395,6 +412,7 @@ gdht_sha1_found(const kuid_t *kuid, const lookup_val_rs_t *rs, gpointer arg)
 	struct sha1_lookup *slk = arg;
 	fileinfo_t *fi;
 	size_t i;
+	gboolean seen_foreign = FALSE;
 
 	g_assert(rs);
 	sha1_lookup_check(slk);
@@ -405,8 +423,6 @@ gdht_sha1_found(const kuid_t *kuid, const lookup_val_rs_t *rs, gpointer arg)
 			kuid_to_string(kuid), (gulong) rs->count,
 			1 == rs->count ? "" : "s");
 
-	gnet_stats_count_general(GNR_DHT_SUCCESSFUL_ALT_LOC_LOOKUPS, 1);
-
 	fi = file_info_by_guid(slk->fi_guid);
 
 	if (NULL == fi) {
@@ -426,9 +442,23 @@ gdht_sha1_found(const kuid_t *kuid, const lookup_val_rs_t *rs, gpointer arg)
 
 	for (i = 0; i < rs->count; i++) {
 		lookup_val_rc_t *rc = &rs->records[i];
+		if (is_my_address_and_port(rc->addr, rc->port))
+			continue;
+		if (local_addr_cache_lookup(rc->addr, rc->port))
+			continue;
+		seen_foreign = TRUE;		/* ALOC not published by ourselves */
 		gdht_handle_aloc(rc, fi);
 	}
 
+	/*
+	 * Since we can publish partial SHA-1 ourselves, we can only count
+	 * a success when we had one entry not published by ourselves.
+	 */
+
+	if (seen_foreign) {
+		gnet_stats_count_general(GNR_DHT_SUCCESSFUL_ALT_LOC_LOOKUPS, 1);
+	}
+
 cleanup:
 	gdht_free_sha1_lookup(slk, TRUE);
 }
@@ -446,7 +476,7 @@ gdht_find_sha1(const fileinfo_t *fi)
 
 	slk = walloc(sizeof *slk);
 	slk->magic = SHA1_LOOKUP_MAGIC;
-	slk->id = kuid_from_sha1(fi->sha1);
+	slk->id = gdht_kuid_from_sha1(fi->sha1);
 	slk->fi_guid = atom_guid_get(fi->guid);
 
 	/*
@@ -594,7 +624,7 @@ gdht_handle_prox(const lookup_val_rc_t *rc, struct guid_lookup *glk)
 					gnet_host_set(&proxies[proxy_count++], a, p);
 				}
 
-				bstr_destroy(bs);
+				bstr_free(&bs);
 			}
 			break;
 		default:
@@ -608,6 +638,18 @@ gdht_handle_prox(const lookup_val_rc_t *rc, struct guid_lookup *glk)
 		}
 	}
 
+	/*
+	 * If there is a SHA1 conflict, reject the PROX.
+	 */
+
+	if (!guid_eq(glk->guid, &guid)) {
+		if (GNET_PROPERTY(download_debug))
+			g_warning("discarding %s from %s for GUID %s: PROX was for GUID %s",
+				value_infostr(rc), host_addr_port_to_string(rc->addr, port),
+				guid_to_string(glk->guid), guid_hex_str(&guid));
+		goto cleanup;
+	}
+
 	/*
 	 * If we did not find any proxy, reject the PROX.
 	 */
@@ -666,34 +708,176 @@ cleanup:
 }
 
 /**
- * Callback when GUID lookup is successful.
+ * Handle DHT NOPE value received to generate a new push-proxy for the servent.
+ */
+static void
+gdht_handle_nope(const lookup_val_rc_t *rc, struct guid_lookup *glk)
+{
+	extvec_t exv[MAX_EXTVEC];
+	int exvcnt;
+	int i;
+	guid_t guid;
+	guint16 port = 0;
+
+	g_assert(DHT_VT_NOPE == rc->type);
+	guid_lookup_check(glk);
+
+	ext_prepare(exv, MAX_EXTVEC);
+	memset(guid.v, 0, GUID_RAW_SIZE);
+
+	exvcnt = ext_parse(rc->data, rc->length, exv, MAX_EXTVEC);
+
+	for (i = 0; i < exvcnt; i++) {
+		extvec_t *e = &exv[i];
+		guint16 paylen;
+
+		switch (e->ext_token) {
+		case EXT_T_GGEP_guid:
+			if (GUID_RAW_SIZE == ext_paylen(e))
+				memcpy(guid.v, ext_payload(e), GUID_RAW_SIZE);
+			break;
+		case EXT_T_GGEP_port:
+			if (2 == ext_paylen(e))
+				port = peek_be16(ext_payload(e));
+			break;
+		case EXT_T_GGEP_tls:
+			/* We don't handle TLS support indication yet -- RAM, 2010-02-27 */
+			break;
+		default:
+			if (GNET_PROPERTY(ggep_debug) > 1 && e->ext_type == EXT_GGEP) {
+				paylen = ext_paylen(e);
+				g_warning("%s: unhandled GGEP \"%s\" (%d byte%s)",
+					value_infostr(rc),
+					ext_ggep_id_str(e), paylen, paylen == 1 ? "" : "s");
+			}
+			break;
+		}
+	}
+
+	/*
+	 * If there is a DHT key conflict, reject the NOPE.
+	 */
+
+	if (!guid_eq(glk->guid, &guid)) {
+		if (GNET_PROPERTY(download_debug))
+			g_warning("discarding %s from %s for GUID %s: NOPE was for GUID %s",
+				value_infostr(rc), host_addr_port_to_string(rc->addr, port),
+				guid_to_string(glk->guid), guid_hex_str(&guid));
+		goto cleanup;
+	}
+
+	/*
+	 * Check servent's port, if specified.  It should match that of the
+	 * creator.  If not, warn, but trust what is in the NOPE.
+	 */
+
+	if (port) {
+		if (port != rc->port && GNET_PROPERTY(download_debug))
+			g_warning("%s: port mismatch: creator's was %u, "
+				"NOPE is %u for %s, known as %s here",
+				value_infostr(rc), rc->port,
+				port, host_addr_to_string(rc->addr),
+				host_addr_port_to_string(glk->addr, glk->port));
+	} else {
+		port = rc->port;
+	}
+
+	/*
+	 * Create new push-proxies.
+	 */
+
+	if (GNET_PROPERTY(download_debug) > 0)
+		g_message("adding %s (NOPE creator) as push-proxy for %s (%s)",
+			host_addr_port_to_string(rc->addr, port),
+			guid_to_string(glk->guid),
+			host_addr_port_to_string(glk->addr, glk->port));
+
+	download_add_push_proxy(glk->guid, rc->addr, port);
+
+	/* FALL THROUGH */
+
+cleanup:
+	if (exvcnt)
+		ext_reset(exv, MAX_EXTVEC);
+}
+
+/**
+ * Callback when GUID lookup is successful (PROX or NOPE values).
  */
 static void
 gdht_guid_found(const kuid_t *kuid, const lookup_val_rs_t *rs, gpointer arg)
 {
 	struct guid_lookup *glk = arg;
 	size_t i;
+	gboolean prox = FALSE;
+	gboolean nope = FALSE;
+	size_t other = 0;
 
 	g_assert(rs);
 	guid_lookup_check(glk);
 	g_assert(glk->id == kuid);		/* They are atoms */
 
-	if (GNET_PROPERTY(dht_lookup_debug) > 1)
+	if (GNET_PROPERTY(dht_lookup_debug) > 1) {
 		g_message("DHT PROX lookup for GUID %s returned %lu value%s",
 			guid_to_string(glk->guid), (gulong) rs->count,
 			1 == rs->count ? "" : "s");
-
-	gnet_stats_count_general(GNR_DHT_SUCCESSFUL_PUSH_PROXY_LOOKUPS, 1);
+	}
 
 	/*
-	 * Parse PROX results.
+	 * Parse PROX or NOPE results.
+	 *
+	 * We've been launching a generic lookup for any value.  If there is
+	 * a SHA1 collision in the DHT, we may very well get ALOC results only,
+	 * for instance.
 	 */
 
 	for (i = 0; i < rs->count; i++) {
 		lookup_val_rc_t *rc = &rs->records[i];
-		gdht_handle_prox(rc, glk);
+		switch (rc->type) {
+		case DHT_VT_PROX:
+			prox = TRUE;
+			gdht_handle_prox(rc, glk);
+			break;
+		case DHT_VT_NOPE:
+			nope = TRUE;
+			gdht_handle_nope(rc, glk);
+			break;
+		default:
+			other++;
+			break;
+		}
 	}
 
+	if (GNET_PROPERTY(dht_lookup_debug)) {
+		g_message("DHT PROX %s lookup for GUID %s returned %lu other value%s",
+			(prox || nope) ? "successful" : "failed",
+			guid_to_string(glk->guid),
+			(gulong) other, 1 == other ? "" : "s");
+	}
+
+	if (other > 0) {
+		/* Was looking for SHA1(GUID), found some other SHA1 */
+		gnet_stats_count_general(GNR_DHT_SHA1_DATA_TYPE_COLLISIONS, 1);
+	}
+
+	/*
+	 * If we got only alien values (neither PROX nor NOPE), then act as if
+	 * the lookup had failed, actually.
+	 */
+
+	if (!(prox || nope)) {
+		gdht_guid_not_found(kuid, LOOKUP_E_NOT_FOUND, glk);
+		return;
+	}
+
+	/* If we got at least one NOPE back, count a successful NOPE lookup */
+
+	gnet_stats_count_general(
+		nope ?
+			GNR_DHT_SUCCESSFUL_NODE_PUSH_ENTRY_LOOKUPS :
+			GNR_DHT_SUCCESSFUL_PUSH_PROXY_LOOKUPS,
+		1);
+
 	gdht_free_guid_lookup(glk, TRUE);
 }
 
@@ -701,7 +885,7 @@ gdht_guid_found(const kuid_t *kuid, const lookup_val_rs_t *rs, gpointer arg)
  * Launch a GUID lookup in the DHT to collect push proxies for a server.
  */
 void
-gdht_find_guid(const struct guid *guid, const host_addr_t addr, guint16 port)
+gdht_find_guid(const guid_t *guid, const host_addr_t addr, guint16 port)
 {
 	struct guid_lookup *glk;
 
@@ -711,7 +895,7 @@ gdht_find_guid(const struct guid *guid, const host_addr_t addr, guint16 port)
 
 	glk = walloc(sizeof *glk);
 	glk->magic = GUID_LOOKUP_MAGIC;
-	glk->id = kuid_from_guid(guid);
+	glk->id = gdht_kuid_from_guid(guid);
 	glk->guid = atom_guid_get(guid);
 	glk->addr = addr;
 	glk->port = port;
@@ -738,7 +922,23 @@ gdht_find_guid(const struct guid *guid, const host_addr_t addr, guint16 port)
 			guid_to_string(guid), host_addr_port_to_string(addr, port));
 
 	gm_hash_table_insert_const(guid_lookups, glk->id, glk);
-	ulq_find_value(glk->id, DHT_VT_PROX,
+
+	/*
+	 * We're looking for ANY value here, but we really expect PROX or NOPE
+	 * values back.
+	 *
+	 * Compared to issuing a PROX lookup first followed by a NOPE lookup if
+	 * we don't get result, we're more efficient but we run the risk of failing
+	 * if, bad luck, the SHA1 of the GUID we're looking conflicts with the
+	 * SHA1 of a shared file, and we'll be getting only ALOC values back.
+	 *
+	 * Still, the collision risk is low and lookups being rather costly,
+	 * it's best to only issue one.  With time, NOPE publishing will become
+	 * less frequent so we'll also better withstand evolution.
+	 *		--RAM, 2010-02-28
+	 */
+
+	ulq_find_any_value(glk->id, DHT_VT_PROX,
 		gdht_guid_found, gdht_guid_not_found, glk);
 }
 
@@ -788,9 +988,11 @@ gdht_close(void)
 {
 	g_hash_table_foreach(sha1_lookups, free_sha1_lookups_kv, NULL);
 	g_hash_table_destroy(sha1_lookups);
+	sha1_lookups = NULL;
 
 	g_hash_table_foreach(guid_lookups, free_guid_lookups_kv, NULL);
 	g_hash_table_destroy(guid_lookups);
+	guid_lookups = NULL;
 }
 
 /* vi: set ts=4 sw=4 cindent: */
