@@ -1,5 +1,5 @@
 /*
- * $Id: geo_ip.c 16106 2009-03-10 15:16:31Z rmanfredi $
+ * $Id: geo_ip.c 17194 2009-11-05 21:44:06Z rmanfredi $
  *
  * Copyright (c) 2004, Raphael Manfredi
  *
@@ -35,7 +35,7 @@
 
 #include "common.h"
 
-RCSID("$Id: geo_ip.c 16106 2009-03-10 15:16:31Z rmanfredi $")
+RCSID("$Id: geo_ip.c 17194 2009-11-05 21:44:06Z rmanfredi $")
 
 #include "geo_ip.h"
 #include "settings.h"
@@ -43,8 +43,12 @@ RCSID("$Id: geo_ip.c 16106 2009-03-10 15:16:31Z rmanfredi $")
 #include "lib/file.h"
 #include "lib/ascii.h"
 #include "lib/glib-missing.h"
+#include "lib/halloc.h"
 #include "lib/iprange.h"
 #include "lib/iso3166.h"
+#include "lib/parse.h"
+#include "lib/path.h"
+#include "lib/tm.h"
 #include "lib/walloc.h"
 #include "lib/watcher.h"
 
@@ -57,6 +61,7 @@ static const char gip_file[] = "geo-ip.txt";
 static const char gip_what[] = "Geographic IP mappings";
 
 static struct iprange_db *geo_db;	/**< The database of bogus CIDR ranges */
+static time_t geo_mtime;			/**< Modification time of loaded file */
 
 /**
  * Context used during ip_range_split() calls.
@@ -82,7 +87,7 @@ gip_add_cidr(guint32 ip, guint bits, gpointer udata)
 	gpointer ccode;
 	guint cc;
 
-	if (GNET_PROPERTY(dbg) > 4)
+	if (GNET_PROPERTY(reload_debug) > 4)
 		printf("GEO adding %s/%d for \"%s\"\n",
 			ip_to_string(ip), bits, ctx->line);
 
@@ -117,8 +122,14 @@ gip_load(FILE *f)
 	guint16 code;
 	int c;
 	struct range_context ctx;
+	struct stat buf;
 
 	geo_db = iprange_new();
+	if (-1 == fstat(fileno(f), &buf)) {
+		g_warning("cannot stat %s: %s", gip_file, g_strerror(errno));
+	} else {
+		geo_mtime = buf.st_mtime;
+	}
 
 	while (fgets(line, sizeof(line), f)) {
 		linenum++;
@@ -142,7 +153,7 @@ gip_load(FILE *f)
 		 *
 		 *    15.0.0.0 - 15.130.191.255 fr
 		 *
-		 * So we don't have to parse the two IP addresses, and compute
+		 * So we just have to parse the two IP addresses, then compute
 		 * all the ranges they cover in order to insert them into
 		 * the IP database.
 		 */
@@ -228,7 +239,7 @@ gip_load(FILE *f)
 
 	iprange_sync(geo_db);
 
-	if (GNET_PROPERTY(dbg)) {
+	if (GNET_PROPERTY(reload_debug)) {
 		g_message("loaded %u geographical IP ranges (%u hosts)",
 			iprange_get_item_count(geo_db),
 			iprange_get_host_count(geo_db));
@@ -300,7 +311,7 @@ gip_retrieve(void)
 
 	filename = make_pathname(fp[idx].dir, fp[idx].name);
 	watcher_register(filename, gip_changed, NULL);
-	G_FREE_NULL(filename);
+	HFREE_NULL(filename);
 
 	gip_load(f);
 	fclose(f);
@@ -328,7 +339,7 @@ gip_close(void)
  * Retrieves the country an address is assigned to.
  *
  * @param ha the host address to look up.
- * @return the country mapped to this IP address as an numerical encoded
+ * @return the country mapped to this IP address as a numerically-encoded
  *         country code, or ISO3166_INVALID when unknown.
  */
 guint16
@@ -350,4 +361,44 @@ gip_country(const host_addr_t ha)
 	return ISO3166_INVALID;
 }
 
+/**
+ * Same as gip_country() only returns ISO3166_INVALID if the geo_ip file
+ * is too ancient: the risk of having a wrong mapping is too high.
+ */
+guint16
+gip_country_safe(const host_addr_t ha)
+{
+	if (tm_time() - geo_mtime > 15552000)		/* ~6 months */
+		return ISO3166_INVALID;
+
+	return gip_country(ha);
+}
+
+/**
+ * Convenience routine to return the full contry name of an address.
+ */
+const char *
+gip_country_name(const host_addr_t ha)
+{
+	return iso3166_country_name(gip_country(ha));
+}
+
+/**
+ * Convenience routine to return the contry code of an address.
+ */
+const char *
+gip_country_cc(const host_addr_t ha)
+{
+	return iso3166_country_cc(gip_country(ha));
+}
+
+/**
+ * Returns the modification time of the loaded geo_ip file.
+ */
+time_t
+gip_mtime(void)
+{
+	return geo_mtime;
+}
+
 /* vi: set ts=4 sw=4 cindent: */
