@@ -1,5 +1,5 @@
 /*
- * $Id: ggep.c 16106 2009-03-10 15:16:31Z rmanfredi $
+ * $Id: ggep.c 17188 2009-11-04 22:03:22Z rmanfredi $
  *
  * Copyright (c) 2002-2004, Raphael Manfredi
  *
@@ -35,13 +35,14 @@
 
 #include "common.h"
 
-RCSID("$Id: ggep.c 16106 2009-03-10 15:16:31Z rmanfredi $")
+RCSID("$Id: ggep.c 17188 2009-11-04 22:03:22Z rmanfredi $")
 
 #include <zlib.h>	/* Z_DEFAULT_COMPRESSION */
 
 #include "ggep.h"
 #include "extensions.h"
 #include "lib/cobs.h"
+#include "lib/halloc.h"
 #include "lib/misc.h"
 #include "lib/walloc.h"
 #include "lib/zlib_util.h"
@@ -461,11 +462,11 @@ ggep_stream_end(ggep_stream_t *gs)
 			/* No overwriting */
 			g_assert((size_t) (gs->end - gs->o) <= gs->size);
 			ok = ggep_stream_write(gs, data, ilen);
-			G_FREE_NULL(data);
+			HFREE_NULL(data);
 
 			if (!ok)
 				goto cleanup;
-		} else if (GNET_PROPERTY(ggep_debug) > 2)
+		} else if (GNET_PROPERTY(ggep_debug) > 3)
 			g_warning("GGEP \"%.*s\" compressed %d bytes into %d",
 				(int) (*gs->fp & GGEP_F_IDLEN),
 				gs->fp + 1, (int) ilen, (int) plen);
@@ -535,7 +536,7 @@ ggep_stream_end(ggep_stream_t *gs)
 			gs->o = &gs->lp[1 + plen];	/* +1 to skip NUL "length" byte */
 			/* No overwriting */
 			g_assert((size_t) (gs->end - gs->o) <= gs->size);
-		} else if (GNET_PROPERTY(ggep_debug) > 2)
+		} else if (GNET_PROPERTY(ggep_debug) > 3)
 			g_message("GGEP \"%.*s\" COBS-ed into %d bytes",
 				(int) (*gs->fp & GGEP_F_IDLEN), gs->fp + 1, (int) plen);
 	}
@@ -555,7 +556,7 @@ ggep_stream_end(ggep_stream_t *gs)
 	 * stored at the beginning.  We only reserved 1 byte for the length.
 	 */
 
-	if (GNET_PROPERTY(ggep_debug) > 3)
+	if (GNET_PROPERTY(ggep_debug) > 7)
 		g_message("GGEP \"%.*s\" payload holds %d byte%s",
 			(int) (*gs->fp & GGEP_F_IDLEN), gs->fp + 1,
 			(int) plen, plen == 1 ? "" : "s");
