@@ -1,5 +1,5 @@
 /*
- * $Id: ggep.h 16106 2009-03-10 15:16:31Z rmanfredi $
+ * $Id: ggep.h 16369 2009-04-02 16:54:32Z cbiere $
  *
  * Copyright (c) 2002-2003, Raphael Manfredi
  *
@@ -42,7 +42,7 @@
 #include "lib/cobs.h"
 #include "lib/zlib_util.h"
 
-#define GGEP_MAGIC		0xC3U		/**< GGEP extension prefix */
+#define GGEP_MAGIC		((unsigned char) 0xC3U)	/**< GGEP extension prefix */
 
 /*
  * GGEP Extension Header Flags.
