@@ -1,5 +1,5 @@
 /*
- * $Id: gmsg.c 16200 2009-03-18 19:43:17Z rmanfredi $
+ * $Id: gmsg.c 17198 2009-11-05 23:36:37Z rmanfredi $
  *
  * Copyright (c) 2002-2003, Raphael Manfredi
  *
@@ -35,7 +35,7 @@
 
 #include "common.h"
 
-RCSID("$Id: gmsg.c 16200 2009-03-18 19:43:17Z rmanfredi $")
+RCSID("$Id: gmsg.c 17198 2009-11-05 23:36:37Z rmanfredi $")
 
 #include <zlib.h>	/* Z_DEFAULT_COMPRESSION */
 
@@ -55,6 +55,7 @@ RCSID("$Id: gmsg.c 16200 2009-03-18 19:43:17Z rmanfredi $")
 #include "lib/endian.h"
 #include "lib/glib-missing.h"
 #include "lib/pmsg.h"
+#include "lib/unsigned.h"
 #include "lib/walloc.h"
 #include "lib/zlib_util.h"
 #include "lib/override.h"		/* Must be the last header included */
@@ -136,7 +137,7 @@ gmsg_dump(FILE *out, gconstpointer data, guint32 size)
 {
 	g_assert(size >= GTA_HEADER_SIZE);
 
-	dump_hex(out, gmsg_infostr_full(data),
+	dump_hex(out, gmsg_infostr_full(data, size),
 		(char *) data + GTA_HEADER_SIZE, size - GTA_HEADER_SIZE);
 }
 
@@ -149,7 +150,7 @@ gmsg_split_dump(FILE *out, gconstpointer head, gconstpointer data,
 {
 	g_assert(size >= GTA_HEADER_SIZE);
 
-	dump_hex(out, gmsg_infostr_full_split(head, data),
+	dump_hex(out, gmsg_infostr_full_split(head, data, size - GTA_HEADER_SIZE),
 		data, size - GTA_HEADER_SIZE);
 }
 
@@ -161,23 +162,25 @@ gmsg_init(void)
 {
 	int i;
 
+#define VMSG_W	10		/* Special weight to flag vendor messages */
+
 	for (i = 0; i < 256; i++) {
 		const char *s = "unknown";
 		guint w = 0;
 
 		switch ((enum gta_msg) i) {
-		case GTA_MSG_DHT:            w = 0; s = "DHT"; break;
-		case GTA_MSG_INIT:           w = 1; s = "Ping"; break;
-		case GTA_MSG_SEARCH:         w = 2; s = "Query"; break;
-		case GTA_MSG_INIT_RESPONSE:  w = 3; s = "Pong"; break;
-		case GTA_MSG_SEARCH_RESULTS: w = 4; s = "Q-Hit"; break;
-		case GTA_MSG_PUSH_REQUEST:   w = 5; s = "Push"; break;
-		case GTA_MSG_RUDP:   		 		s = "RUDP"; break;
-		case GTA_MSG_VENDOR:         		s = "Vndor"; break;
-		case GTA_MSG_STANDARD:       		s = "Vstd"; break;
-		case GTA_MSG_QRP:            w = 6; s = "QRP"; break;
-		case GTA_MSG_HSEP_DATA:      		s = "HSEP"; break;
-		case GTA_MSG_BYE:      		 w = 7; s = "BYE"; break;
+		case GTA_MSG_DHT:            w = 0;      s = "DHT"; break;
+		case GTA_MSG_HSEP_DATA:      w = 0;      s = "HSEP"; break;
+		case GTA_MSG_INIT:           w = 1;      s = "Ping"; break;
+		case GTA_MSG_SEARCH:         w = 2;      s = "Query"; break;
+		case GTA_MSG_INIT_RESPONSE:  w = 3;      s = "Pong"; break;
+		case GTA_MSG_SEARCH_RESULTS: w = 4;      s = "Q-Hit"; break;
+		case GTA_MSG_PUSH_REQUEST:   w = 5;      s = "Push"; break;
+		case GTA_MSG_VENDOR:         w = VMSG_W; s = "Vndor"; break;
+		case GTA_MSG_STANDARD:       w = VMSG_W; s = "Vstd"; break;
+		case GTA_MSG_RUDP:   		 w = 6;      s = "RUDP"; break;
+		case GTA_MSG_QRP:            w = 8;      s = "QRP"; break;
+		case GTA_MSG_BYE:      		 w = 9;      s = "BYE"; break;
 		}
 		msg_name[i] = s;
 		msg_weight[i] = w;
@@ -304,7 +307,7 @@ gmsg_to_deflated_pmsg(gconstpointer msg, guint32 size)
 	if (deflated_length >= plen) {
 		if (GNET_PROPERTY(udp_debug))
 			g_message("UDP not deflating %s into %d bytes",
-				gmsg_infostr_full(msg), deflated_length);
+				gmsg_infostr_full(msg, size), deflated_length);
 
 		gnet_stats_count_general(GNR_UDP_LARGER_HENCE_NOT_COMPRESSED, 1);
 		goto send_raw;
@@ -321,7 +324,7 @@ gmsg_to_deflated_pmsg(gconstpointer msg, guint32 size)
 
 	if (GNET_PROPERTY(udp_debug))
 		g_message("UDP deflated %s into %d bytes",
-			gmsg_infostr_full(msg), deflated_length);
+			gmsg_infostr_full(msg, size), deflated_length);
 
 	{
 		gpointer header;
@@ -794,8 +797,8 @@ gmsg_query_can_send(pmsg_t *mb, const mqueue_t *q)
 
 	if (!node_query_hops_ok(n, gnutella_header_get_hops(msg))) {
 		if (GNET_PROPERTY(gmsg_debug) > 4)
-			gmsg_log_dropped(msg, "to node %s due to hops-flow",
-				node_addr(n));
+			gmsg_log_dropped_pmsg(mb,
+				"to node %s due to hops-flow", node_addr(n));
 		return FALSE;
 	}
 
@@ -804,8 +807,8 @@ gmsg_query_can_send(pmsg_t *mb, const mqueue_t *q)
 
 	if (!route_exists_for_reply(msg, gnutella_header_get_function(msg))) {
 		if (GNET_PROPERTY(gmsg_debug) > 4)
-			gmsg_log_dropped(msg, "to node %s due to no route for hits",
-				node_addr(n));
+			gmsg_log_dropped_pmsg(mb,
+				"to node %s due to no route for hits", node_addr(n));
 		return FALSE;
 	}
 
@@ -852,24 +855,36 @@ gmsg_can_drop(gconstpointer pdu, int size)
 }
 
 /**
- * Perform a priority comparison between two messages, given as the whole PDU.
+ * Perform a priority comparison between two messages, given as whole PDUs.
+ *
+ * If h2_pdu is FALSE, then h2 is only a Gnutella header, not a whole PDU.
+ * Caller must ensure that h1 points to the whole PDU, i.e. that the data
+ * immediately follows the Gnutella header.
  *
  * @return algebraic -1/0/+1 depending on relative order.
  */
 int
-gmsg_cmp(gconstpointer h1, gconstpointer h2)
+gmsg_cmp(gconstpointer h1, gconstpointer h2, gboolean h2_pdu)
 {
 	int w1, w2;
 	guint8 f1, f2;
+	guint8 hop1, hop2;
 
 	f1 = gnutella_header_get_function(h1);
 	f2 = gnutella_header_get_function(h2);
 
 	w1 = (f1 == GTA_MSG_DHT) ?
 		kmsg_weight[kademlia_header_get_function(h1)] :  msg_weight[f1];
-	w2 = (f2 == GTA_MSG_DHT) ?
+	w2 = (f2 == GTA_MSG_DHT && h2_pdu) ?
 		kmsg_weight[kademlia_header_get_function(h2)] :  msg_weight[f2];
 
+	/*
+	 * Special case for vendor messages.
+	 */
+
+	w1 = w1 == VMSG_W ?  vmsg_weight(gnutella_data(h1)) : w1;
+	w2 = (w2 == VMSG_W && h2_pdu) ? vmsg_weight(gnutella_data(h2)) : w2;
+
 	/*
 	 * The more weight a message type has, the more prioritary it is.
 	 */
@@ -885,9 +900,9 @@ gmsg_cmp(gconstpointer h1, gconstpointer h2)
 	 */
 
 	if (f1 == GTA_MSG_DHT) {
-		return f2 == GTA_MSG_DHT ?
-			CMP(gnutella_header_get_size(h2), gnutella_header_get_size(h1)) :
-			-1;
+		guint32 s1 = gnutella_header_get_size(h1);
+		guint32 s2 = gnutella_header_get_size(h2);
+		return f2 == GTA_MSG_DHT ?  CMP(s2, s1) : -1;
 	} else if (f2 == GTA_MSG_DHT) {
 		return +1;		/* Gnutella message (f1) more prioritary */
 	}
@@ -905,30 +920,50 @@ gmsg_cmp(gconstpointer h1, gconstpointer h2)
 	 * to its destination (lowest TTL).
 	 */
 
-	if (gnutella_header_get_hops(h1) == gnutella_header_get_hops(h2)) {
+	hop1 = gnutella_header_get_hops(h1);
+	hop2 = gnutella_header_get_hops(h2);
+
+	if (hop1 == hop2) {
 		switch (f1) {
 		case GTA_MSG_PUSH_REQUEST:
 		case GTA_MSG_SEARCH_RESULTS:
-			return CMP(gnutella_header_get_ttl(h2),
-						gnutella_header_get_ttl(h1));
+			{
+				guint8 t1 = gnutella_header_get_ttl(h1);
+				guint8 t2 = gnutella_header_get_ttl(h2);
+				int ttlc = CMP(t2, t1);
+				/* If same TTL, favor the shortest message */
+				if (ttlc) {
+					return ttlc;
+				} else {
+					guint32 s1 = gnutella_header_get_size(h1);
+					guint32 s2 = gnutella_header_get_size(h2);
+					return CMP(s2, s1);
+				}
+			}
 		default:
-			return 0;
+			/* Favor the shortest */
+			{
+				guint32 s1 = gnutella_header_get_size(h1);
+				guint32 s2 = gnutella_header_get_size(h2);
+				return CMP(s2, s1);
+			}
 		}
 	} else {
 		switch (f1) {
 		case GTA_MSG_INIT:
 		case GTA_MSG_SEARCH:
 		case GTA_MSG_QRP:
-			return gnutella_header_get_hops(h1) > gnutella_header_get_hops(h2)
-					? -1 : +1;
+			return hop1 > hop2 ? -1 : +1;
 		default:
-			return gnutella_header_get_hops(h1) < gnutella_header_get_hops(h2)
-					? -1 : +1;
+			return hop1 < hop2 ? -1 : +1;
 		}
 	}
 }
 
 /**
+ * @param msg		start of message (Gnutella header), followed by data
+ * @param msg_len	length of the buffer containing the header + body
+ * 
  * @returns formatted static string:
  *
  *     msg_type (payload length) [hops=x, TTL=x]
@@ -938,11 +973,44 @@ gmsg_cmp(gconstpointer h1, gconstpointer h2)
  * payload of that message.
  */
 char *
-gmsg_infostr_full(gconstpointer msg)
+gmsg_infostr_full(gconstpointer msg, size_t msg_len)
 {
-	const char *data = (const char *) msg + GTA_HEADER_SIZE;
+	const char *data = const_ptr_add_offset(msg, GTA_HEADER_SIZE);
+	size_t data_len = msg_len - GTA_HEADER_SIZE;
 
-	return gmsg_infostr_full_split(msg, data);
+	if (msg_len < GTA_HEADER_SIZE)
+		return "undecipherable (smaller than Gnutella header)";
+
+	return gmsg_infostr_full_split(msg, data, data_len);
+}
+
+/**
+ * Same a gmsg_infostr_to_buf() only we have the header and the payload.
+ * If the data follows the header, then we can also decompile DHT messages.
+ */
+static size_t
+gmsg_infostr_split_to_buf(
+	gconstpointer head, gconstpointer data, size_t data_len,
+	char *buf, size_t buf_size)
+{
+	guint8 function = gnutella_header_get_function(head);
+	guint16 size = gmsg_size(head);
+
+	if (
+		GTA_MSG_DHT == function &&
+		data_len + GTA_HEADER_SIZE >= KDA_HEADER_SIZE &&
+		data == gnutella_data(head) && data_len == size
+	) {
+		/* Data is consecutive to header and length matches with header */
+		return kmsg_infostr_to_buf(head, buf, buf_size);
+	}
+
+	return gm_snprintf(buf, buf_size, "%s (%u byte%s) %s[hops=%d, TTL=%d]",
+		gmsg_name(function),
+		size, size == 1 ? "" : "s",
+		gnutella_header_get_ttl(head) & GTA_UDP_DEFLATED ? "deflated " : "",
+		gnutella_header_get_hops(head),
+		gnutella_header_get_ttl(head) & ~GTA_UDP_DEFLATED);
 }
 
 /**
@@ -955,8 +1023,10 @@ gmsg_infostr_to_buf(gconstpointer msg, char *buf, size_t buf_size)
 	guint8 function = gnutella_header_get_function(msg);
 	guint16 size = gmsg_size(msg);
 
-	if (GTA_MSG_DHT == function)
-		return kmsg_infostr_to_buf(msg, buf, size);
+	/*
+	 * We cannot assume we have more than the Gnutella header, so
+	 * we can't go and probe DHT messages.
+	 */
 
 	return gm_snprintf(buf, buf_size, "%s (%u byte%s) %s[hops=%d, TTL=%d]",
 		gmsg_name(function),
@@ -967,26 +1037,27 @@ gmsg_infostr_to_buf(gconstpointer msg, char *buf, size_t buf_size)
 }
 
 /**
- * @returns formatted static string:
- *
- *     msg_type (payload length) [hops=x, TTL=x]
- *
- * that can also decompile vendor messages given a pointer on the header
- * and on the data of the message (which may not be consecutive in memory).
+ * Same as gmsg_infostr_full_split() but fills the supplied buffer with
+ * the formatted string and returns the amount of bytes written.
  */
-char *
-gmsg_infostr_full_split(gconstpointer head, gconstpointer data)
+static size_t
+gmsg_infostr_full_split_to_buf(gconstpointer head, gconstpointer data,
+	size_t data_len, char *buf, size_t buf_size)
 {
-	static char a[160];
+	size_t rw;
+
+	g_assert(size_is_non_negative(data_len));
+	g_assert(size_is_non_negative(buf_size));
 
 	switch (gnutella_header_get_function(head)) {
 	case GTA_MSG_VENDOR:
 	case GTA_MSG_STANDARD:
 		{
-			guint16 size = gmsg_size(head);
+			guint16 size = data_len & GTA_SIZE_MASK;
 			guint8 ttl = gnutella_header_get_ttl(head);
-			
-			gm_snprintf(a, sizeof(a), "%s %s (%u byte%s) %s[hops=%d, TTL=%d]",
+
+			rw = gm_snprintf(buf, buf_size,
+				"%s %s (%u byte%s) %s[hops=%d, TTL=%d]",
 				gmsg_name(gnutella_header_get_function(head)),
 				vmsg_infostr(data, size),
 				size, size == 1 ? "" : "s",
@@ -997,13 +1068,49 @@ gmsg_infostr_full_split(gconstpointer head, gconstpointer data)
 		}
 		break;
 	default:
-		gmsg_infostr_to_buf(head, a, sizeof a);
+		rw = gmsg_infostr_split_to_buf(head, data, data_len, buf, buf_size);
 	}
 
-	return a;
+	return rw;
 }
 
 /**
+ * Same as gmsg_infostr_full() but formats to supplied buffer ``buf''
+ * which is ``buf_len'' bytes long and returns the amount of bytes written.
+ */
+static size_t
+gmsg_infostr_full_to_buf(gconstpointer msg, size_t msg_len,
+	char *buf, size_t buf_len)
+{
+	const char *data = (const char *) msg + GTA_HEADER_SIZE;
+	size_t data_len = msg_len - GTA_HEADER_SIZE;
+
+	return gmsg_infostr_full_split_to_buf(msg, data, data_len, buf, buf_len);
+}
+
+/**
+ * Pretty-print the message information, based on the Gnutella header and
+ * possibly probing the payload if necessary (for vendor or DHT messages).
+ *
+ * @returns formatted static string:
+ *
+ *     msg_type (payload length) [hops=x, TTL=x]
+ *
+ * that can also decompile vendor messages given a pointer on the header
+ * and on the data of the message (which may not be consecutive in memory).
+ */
+char *
+gmsg_infostr_full_split(gconstpointer head, gconstpointer data, size_t data_len)
+{
+	static char buf[160];
+
+	gmsg_infostr_full_split_to_buf(head, data, data_len, buf, sizeof buf);
+	return buf;
+}
+
+/**
+ * Pretty-print the message information, based solely on the Gnutella header.
+ *
  * @returns formatted static string:
  *
  *     msg_type (payload length) [hops=x, TTL=x]
@@ -1017,34 +1124,90 @@ gmsg_infostr(gconstpointer msg)
 }
 
 /**
- * Same as gmsg_infostr(), but different static buffer.
+ * Pretty-print the message information, based solely on the Gnutella header
+ * of the message held in a gnutella node (current message received from
+ * that node and being routed or processed).
+ *
+ * The advantage over calling gmsg_infostr(&n->header) is that the node
+ * information is also printed if by chance the hop count of the message is 1.
+ *
+ * @returns formatted static string:
+ *
+ *     msg_type (payload length) [hops=x, TTL=x]
+ *
+ * if message is from a remote node, or
+ *
+ *     msg_type (payload length) [hops=x, TTL=x] //IP:port <vendor>//
+ *
+ * if message comes from a neighbour.
  */
-static char *
-gmsg_infostr2(gconstpointer msg)
+const char *
+gmsg_node_infostr(const gnutella_node_t *n)
 {
-	static char buf[80];
-	gmsg_infostr_to_buf(msg, buf, sizeof buf);
+	static char buf[120];
+	size_t w;
+
+	w = gmsg_infostr_to_buf(&n->header, buf, sizeof buf);
+
+	if (gnutella_header_get_hops(n->header) == 1) {
+		gm_snprintf(&buf[w], sizeof buf - w, " //%s <%s>//",
+			node_addr(n), node_vendor(n));
+	}
+
 	return buf;
 }
 
 /**
- * Log dropped message, and reason.
+ * Log dropped message (given with separated header and data) with reason.
  */
 void
-gmsg_log_dropped(gconstpointer msg, const char *reason, ...)
+gmsg_log_split_dropped(
+	gconstpointer head, gconstpointer data, size_t data_len,
+	const char *reason, ...)
 {
-	fputs("DROP ", stdout);
-	fputs(gmsg_infostr2(msg), stdout);	/* Allows gmsg_infostr() in arglist */
+	char rbuf[128];
+	char buf[128];
+
+	gmsg_infostr_full_split_to_buf(head, data, data_len, buf, sizeof buf);
 
 	if (reason) {
 		va_list args;
 		va_start(args, reason);
-		fputs(": ", stdout);
-		vprintf(reason, args);
+		rbuf[0] = ':';
+		rbuf[1] = ' ';
+		gm_vsnprintf(rbuf, sizeof rbuf - 2, reason, args);
 		va_end(args);
+	} else {
+		rbuf[0] = '\0';
 	}
 
-	fputc('\n', stdout);
+	g_message("DROP %s%s", buf, rbuf);
+}
+
+/**
+ * Log dropped message (held in message block) with supplied reason.
+ */
+void
+gmsg_log_dropped_pmsg(pmsg_t *mb, const char *reason, ...)
+{
+	char rbuf[128];
+	char buf[128];
+
+	gmsg_infostr_full_to_buf(pmsg_start(mb), pmsg_written_size(mb),
+		buf, sizeof buf);
+
+	if (reason) {
+		va_list args;
+		va_start(args, reason);
+		rbuf[0] = ':';
+		rbuf[1] = ' ';
+		gm_vsnprintf(&rbuf[2], sizeof rbuf - 2, reason, args);
+		va_end(args);
+	} else {
+		rbuf[0] = '\0';
+	}
+
+	g_message("DROP %s%s", buf, rbuf);
 }
 
 /**
@@ -1053,19 +1216,25 @@ gmsg_log_dropped(gconstpointer msg, const char *reason, ...)
 void
 gmsg_log_bad(const struct gnutella_node *n, const char *reason, ...)
 {
-	g_message("BAD <%s> %s ", node_vendor(n), node_addr(n));
+	char rbuf[128];
+	char buf[128];
 
-	fputs(gmsg_infostr_full_split(&n->header, n->data), stderr);
+	gmsg_infostr_full_split_to_buf(
+		&n->header, n->data, n->size, buf, sizeof buf);
 
 	if (reason) {
 		va_list args;
 		va_start(args, reason);
-		fputs(": ", stderr);
-		vfprintf(stderr, reason, args);
+		rbuf[0] = ':';
+		rbuf[1] = ' ';
+		gm_vsnprintf(rbuf, sizeof rbuf - 2, reason, args);
 		va_end(args);
+	} else {
+		rbuf[0] = '\0';
 	}
 
-	fputc('\n', stderr);
+	g_message("BAD <%s> %s %s%s",
+		node_vendor(n), node_addr(n), buf, rbuf);
 }
 
 /**
