@@ -1,5 +1,5 @@
 /*
- * $Id: gnet_stats.c 16177 2009-03-16 17:44:43Z rmanfredi $
+ * $Id: gnet_stats.c 17395 2010-03-18 21:58:48Z rmanfredi $
  *
  * Copyright (c) 2001-2003, Richard Eckart
  *
@@ -35,7 +35,7 @@
 
 #include "common.h"
 
-RCSID("$Id: gnet_stats.c 16177 2009-03-16 17:44:43Z rmanfredi $")
+RCSID("$Id: gnet_stats.c 17395 2010-03-18 21:58:48Z rmanfredi $")
 
 #include "gnet_stats.h"
 #include "gmsg.h"
@@ -69,6 +69,7 @@ gnet_stats_drop_reason_to_string(msg_drop_reason_t reason)
 		N_("Improper hops/ttl combination"), /**< MSG_DROP_IMPROPER_HOPS_TTL */
 		N_("Max TTL exceeded"),				 /**< MSG_DROP_MAX_TTL_EXCEEDED */
 		N_("Message throttle"),				 /**< MSG_DROP_THROTTLE */
+		N_("Message matched limits"),		 /**< MSG_DROP_LIMIT */
 		N_("Unusable Pong"),				 /**< MSG_DROP_PONG_UNUSABLE */
 		N_("Hard TTL limit reached"),		 /**< MSG_DROP_HARD_TTL_LIMIT */
 		N_("Max hop count reached"),		 /**< MSG_DROP_MAX_HOP_COUNT */
@@ -165,7 +166,10 @@ gnet_stats_general_to_string(gnr_stats_t type)
 	"discovered_server_guid",
 	"changed_server_guid",
 	"guid_collisions",
+	"own_guid_collisions",
 	"received_known_fw_node_info",
+	"revitalized_push_routes",
+	"collected_push_proxies",
 	"attempted_resource_switching",
 	"attempted_resource_switching_after_error",
 	"successful_resource_switching",
@@ -189,7 +193,10 @@ gnet_stats_general_to_string(gnr_stats_t type)
 	"parq_queue_sending_attempts",
 	"parq_queue_sent",
 	"parq_queue_follow_ups",
-	"seeding_of_orphan",
+	"sha1_verifications",
+	"tth_verifications",
+	"qhit_seeding_of_orphan",
+	"upload_seeding_of_orphan",
 	"dht_estimated_size",
 	"dht_kball_furthest",
 	"dht_kball_closest",
@@ -202,7 +209,9 @@ gnet_stats_general_to_string(gnr_stats_t type)
 	"dht_routing_evicted_nodes",
 	"dht_routing_evicted_firewalled_nodes",
 	"dht_routing_promoted_pending_nodes",
+	"dht_routing_pinged_promoted_nodes",
 	"dht_completed_bucket_refresh",
+	"dht_forced_bucket_refresh",
 	"dht_denied_unsplitable_bucket_refresh",
 	"dht_bucket_alive_check",
 	"dht_alive_pings_to_good_nodes",
@@ -212,6 +221,16 @@ gnet_stats_general_to_string(gnr_stats_t type)
 	"dht_keys_held",
 	"dht_cached_keys_held",
 	"dht_values_held",
+	"dht_cached_kuid_targets_held",
+	"dht_cached_roots_held",
+	"dht_cached_roots_exact_hits",
+	"dht_cached_roots_approximate_hits",
+	"dht_cached_roots_misses",
+	"dht_cached_roots_kball_lookups",
+	"dht_cached_roots_contact_refreshed",
+	"dht_cached_tokens_held",
+	"dht_cached_tokens_hits",
+	"dht_stable_nodes_held",
 	"dht_fetch_local_hits",
 	"dht_fetch_local_cached_hits",
 	"dht_returned_expanded_values",
@@ -229,10 +248,31 @@ gnet_stats_general_to_string(gnr_stats_t type)
 	"dht_dup_values",
 	"dht_kuid_collisions",
 	"dht_own_kuid_collisions",
+	"dht_rpc_kuid_reply_mismatch",
+	"dht_caching_attempts",
+	"dht_caching_successful",
+	"dht_caching_partially_successful",
+	"dht_key_offloading_checks",
+	"dht_keys_selected_for_offloading",
+	"dht_key_offloading_attempts",
+	"dht_key_offloading_successful",
+	"dht_key_offloading_partially_successful",
+	"dht_values_offloaded",
+	"dht_publishing_attempts",
+	"dht_publishing_successful",
+	"dht_publishing_partially_successful",
+	"dht_publishing_satisfactory",
+	"dht_republished_late",
+	"dht_publishing_to_self",
+	"dht_publishing_bg_attempts",
+	"dht_publishing_bg_improvements",
+	"dht_publishing_bg_successful",
+	"dht_sha1_data_type_collisions",
 	"dht_alt_loc_lookups",
 	"dht_push_proxy_lookups",
 	"dht_successful_alt_loc_lookups",
 	"dht_successful_push_proxy_lookups",
+	"dht_successful_node_push_entry_lookups",
 	"dht_seeding_of_orphan",
 	};
 
@@ -281,7 +321,7 @@ gnet_stats_count_received_header(gnutella_node_t *n)
 	guint i;
 	gnet_stats_t *stats;
 
-	stats = NODE_IS_UDP(n) ? &gnet_udp_stats : &gnet_tcp_stats;
+	stats = NODE_USES_UDP(n) ? &gnet_udp_stats : &gnet_tcp_stats;
 
     n->received++;
 
@@ -315,7 +355,7 @@ gnet_stats_count_received_payload(const gnutella_node_t *n)
 	guint i;
 	gnet_stats_t *stats;
 
-	stats = NODE_IS_UDP(n) ? &gnet_udp_stats : &gnet_tcp_stats;
+	stats = NODE_USES_UDP(n) ? &gnet_udp_stats : &gnet_tcp_stats;
 
     gnet_stats.byte.received[MSG_TOTAL] += size;
     gnet_stats.byte.received[t] += size;
@@ -343,7 +383,7 @@ gnet_stats_count_queued(const gnutella_node_t *n,
 
 	g_assert(t != MSG_UNKNOWN);
 
-	stats = NODE_IS_UDP(n) ? &gnet_udp_stats : &gnet_tcp_stats;
+	stats = NODE_USES_UDP(n) ? &gnet_udp_stats : &gnet_tcp_stats;
 
 	stats_pkg = hops ? gnet_stats.pkg.queued : gnet_stats.pkg.gen_queued;
 	stats_byte = hops ? gnet_stats.byte.queued : gnet_stats.byte.gen_queued;
@@ -373,7 +413,7 @@ gnet_stats_count_sent(const gnutella_node_t *n,
 
 	g_assert(t != MSG_UNKNOWN);
 
-	stats = NODE_IS_UDP(n) ? &gnet_udp_stats : &gnet_tcp_stats;
+	stats = NODE_USES_UDP(n) ? &gnet_udp_stats : &gnet_tcp_stats;
 
 	stats_pkg = hops ? gnet_stats.pkg.relayed : gnet_stats.pkg.generated;
 	stats_byte = hops ? gnet_stats.byte.relayed : gnet_stats.byte.generated;
@@ -399,7 +439,7 @@ gnet_stats_count_expired(const gnutella_node_t *n)
 	guint t = stats_lut[gnutella_header_get_function(&n->header)];
 	gnet_stats_t *stats;
 
-	stats = NODE_IS_UDP(n) ? &gnet_udp_stats : &gnet_tcp_stats;
+	stats = NODE_USES_UDP(n) ? &gnet_udp_stats : &gnet_tcp_stats;
 
     gnet_stats.pkg.expired[MSG_TOTAL]++;
     gnet_stats.pkg.expired[t]++;
@@ -443,7 +483,7 @@ gnet_stats_count_dropped(gnutella_node_t *n, msg_drop_reason_t reason)
 
     size = n->size + sizeof(n->header);
 	type = stats_lut[gnutella_header_get_function(&n->header)];
-	stats = NODE_IS_UDP(n) ? &gnet_udp_stats : &gnet_tcp_stats;
+	stats = NODE_USES_UDP(n) ? &gnet_udp_stats : &gnet_tcp_stats;
 
 	DROP_STATS(stats, type, size);
 	node_inc_rxdrop(n);
@@ -455,9 +495,9 @@ gnet_stats_count_dropped(gnutella_node_t *n, msg_drop_reason_t reason)
 	default: ;
 	}
 
-	if (GNET_PROPERTY(dbg) > 4)
-		gmsg_log_dropped(&n->header, "from %s <%s>: %s",
-			node_addr(n), node_vendor(n),
+	if (GNET_PROPERTY(log_dropped_gnutella))
+		gmsg_log_split_dropped(&n->header, n->data, n->size,
+			"from %s <%s>: %s", node_addr(n), node_vendor(n),
 			gnet_stats_drop_reason_to_string(reason));
 }
 
@@ -495,14 +535,14 @@ gnet_stats_count_dropped_nosize(
 	g_assert(UNSIGNED(reason) < MSG_DROP_REASON_COUNT);
 
 	type = stats_lut[gnutella_header_get_function(&n->header)];
-	stats = NODE_IS_UDP(n) ? &gnet_udp_stats : &gnet_tcp_stats;
+	stats = NODE_USES_UDP(n) ? &gnet_udp_stats : &gnet_tcp_stats;
 
 	/* Data part of message not read */
 	DROP_STATS(stats, type, sizeof(n->header));
 
-	if (GNET_PROPERTY(dbg) > 4)
-		gmsg_log_dropped(&n->header, "from %s <%s>: %s",
-			node_addr(n), node_vendor(n),
+	if (GNET_PROPERTY(log_dropped_gnutella))
+		gmsg_log_split_dropped(&n->header, n->data, 0,
+			"from %s <%s>: %s", node_addr(n), node_vendor(n),
 			gnet_stats_drop_reason_to_string(reason));
 }
 
