@@ -1,5 +1,5 @@
 /*
- * $Id: gnet_stats.h 12960 2007-03-03 13:49:54Z cbiere $
+ * $Id: gnet_stats.h 16146 2009-03-14 16:03:36Z rmanfredi $
  *
  * Copyright (c) 2001-2003, Richard Eckart
  *
@@ -44,7 +44,8 @@ void gnet_stats_count_dropped(gnutella_node_t *n,
 	msg_drop_reason_t reason);
 void gnet_stats_count_dropped_nosize(
 	const gnutella_node_t *n, msg_drop_reason_t reason);
-void gnet_stats_count_general(gnr_stats_t, guint32);
+void gnet_stats_count_general(gnr_stats_t, int);
+void gnet_stats_set_general(gnr_stats_t type, guint64 value);
 void gnet_stats_count_flowc(gconstpointer);
 
 #endif /* _core_gnet_stats_h_ */
