@@ -1,5 +1,5 @@
 /*
- * $Id: guid.c 15833 2008-09-09 18:10:33Z cbiere $
+ * $Id: guid.c 17227 2009-11-12 00:20:31Z rmanfredi $
  *
  * Copyright (c) 2002-2003, Raphael Manfredi
  *
@@ -39,7 +39,9 @@
 
 #include "common.h"
 
-RCSID("$Id: guid.c 15833 2008-09-09 18:10:33Z cbiere $")
+RCSID("$Id: guid.c 17227 2009-11-12 00:20:31Z rmanfredi $")
+
+#include "gtk-gnutella.h"
 
 #include "guid.h"
 #include "lib/misc.h"
@@ -172,8 +174,8 @@ guid_init(void)
 	gtkg_version_mark =
 		guid_gtkg_encode_version(GTA_VERSION, GTA_SUBVERSION, *rev == '\0');
 
-	if (GNET_PROPERTY(dbg))
-		printf("GTKG version mark is 0x%x\n", gtkg_version_mark);
+	if (GNET_PROPERTY(node_debug))
+		g_message("GTKG version mark is 0x%x", gtkg_version_mark);
 
 	/*
 	 * Validate that guid_random_muid() correctly marks GUIDs as being GTKG's.
@@ -206,7 +208,7 @@ guid_flag_modern(struct guid *muid)
 	 *				--RAM, 15/09/2001
 	 */
 
-	muid->v[8] = 0xff;
+	muid->v[8] = (unsigned char) 0xffU;
 	muid->v[15] = GUID_PONG_CACHING | GUID_PERSISTENT;
 }
 
@@ -391,7 +393,10 @@ guid_oob_is_gtkg(const struct guid *guid,
 	 * therefore it is masked out for comparison purposes.
 	 */
 
-	if ((peek_u8(&guid->v[15]) & ~GUID_REQUERY) != (guid_hec_oob(guid) & ~GUID_REQUERY))
+	if (
+		(peek_u8(&guid->v[15]) & ~GUID_REQUERY) !=
+			(guid_hec_oob(guid) & ~GUID_REQUERY)
+	)
 		return FALSE;
 
 	return guid_extract_gtkg_info(guid, 4, majp, minp, relp);
