@@ -1,5 +1,5 @@
 /*
- * $Id: hcache.c 16106 2009-03-10 15:16:31Z rmanfredi $
+ * $Id: hcache.c 17156 2009-11-01 20:22:05Z rmanfredi $
  *
  * Copyright (c) 2002-2003, Raphael Manfredi, Richard Eckart
  *
@@ -49,7 +49,7 @@
 
 #include "common.h"
 
-RCSID("$Id: hcache.c 16106 2009-03-10 15:16:31Z rmanfredi $")
+RCSID("$Id: hcache.c 17156 2009-11-01 20:22:05Z rmanfredi $")
 
 #include "bogons.h"
 #include "hcache.h"
@@ -58,11 +58,14 @@ RCSID("$Id: hcache.c 16106 2009-03-10 15:16:31Z rmanfredi $")
 #include "nodes.h"
 #include "pcache.h"
 #include "settings.h"
+#include "ctl.h"
 
 #include "lib/ascii.h"
 #include "lib/file.h"
 #include "lib/getdate.h"
 #include "lib/hashlist.h"
+#include "lib/path.h"
+#include "lib/timestamp.h"
 #include "lib/tm.h"
 #include "lib/walloc.h"
 
@@ -181,12 +184,15 @@ stop_mass_update(hostcache_t *hc)
         case HCACHE_TIMEOUT:
         case HCACHE_UNSTABLE:
         case HCACHE_BUSY:
+        case HCACHE_ALIEN:
             gnet_prop_set_guint32_val(hc->hosts_in_catcher,
                 hash_list_length(caches[HCACHE_TIMEOUT]->hostlist) +
                 hash_list_length(caches[HCACHE_UNSTABLE]->hostlist) +
-                hash_list_length(caches[HCACHE_BUSY]->hostlist));
+                hash_list_length(caches[HCACHE_BUSY]->hostlist) +
+                hash_list_length(caches[HCACHE_ALIEN]->hostlist));
             break;
-        default:
+		case HCACHE_NONE:
+		case HCACHE_MAX:
             g_error("stop_mass_update: unknown cache type: %d", hc->type);
         }
     }
@@ -503,6 +509,7 @@ hcache_slots_max(hcache_type_t type)
 	case HCACHE_BUSY:
 	case HCACHE_TIMEOUT:
 	case HCACHE_UNSTABLE:
+	case HCACHE_ALIEN:
 		return GNET_PROPERTY(max_bad_hosts_cached);
 	case HCACHE_NONE:
 	case HCACHE_MAX:
@@ -539,6 +546,7 @@ hcache_slots_left(hcache_type_t type)
 	case HCACHE_BUSY:
 	case HCACHE_TIMEOUT:
 	case HCACHE_UNSTABLE:
+	case HCACHE_ALIEN:
 		current = hash_list_length(caches[type]->hostlist);
 		break;
 	case HCACHE_NONE:
@@ -613,10 +621,19 @@ hcache_add_internal(hcache_type_t type, time_t added,
 		return FALSE;
     }
 
-	if (node_host_is_connected(addr, port)) {
-        stats[HCACHE_ALREADY_CONNECTED]++;
-		return FALSE;			/* Connected to that host? */
-    }
+	switch (type) {
+	case HCACHE_FRESH_ANY:
+	case HCACHE_FRESH_ULTRA:
+	case HCACHE_VALID_ANY:
+	case HCACHE_VALID_ULTRA:
+		if (node_host_is_connected(addr, port)) {
+			stats[HCACHE_ALREADY_CONNECTED]++;
+			return FALSE;			/* Connected to that host? */
+		}
+		break;
+	default:
+		break;
+	}
 
 	hc = caches[type];
     g_assert(hc->type == type);
@@ -664,6 +681,7 @@ hcache_add_internal(hcache_type_t type, time_t added,
 		case HCACHE_TIMEOUT:
 		case HCACHE_BUSY:
 		case HCACHE_UNSTABLE:
+		case HCACHE_ALIEN:
 			/*
 			 * Move host to the proper cache, if not already in one of the
 			 * "bad" caches.
@@ -673,6 +691,7 @@ hcache_add_internal(hcache_type_t type, time_t added,
 			case HCACHE_TIMEOUT:
 			case HCACHE_BUSY:
 			case HCACHE_UNSTABLE:
+			case HCACHE_ALIEN:
 				return TRUE;
 			default:
 				break;				/* Move it */
@@ -770,6 +789,9 @@ hcache_add_internal(hcache_type_t type, time_t added,
 	return TRUE;
 }
 
+/**
+ * Add host to the proper cache.
+ */
 gboolean
 hcache_add(hcache_type_t type,
 	const host_addr_t addr, guint16 port, const char *what)
@@ -784,6 +806,9 @@ gboolean
 hcache_add_caught(host_type_t type, const host_addr_t addr, guint16 port,
 	const char *what)
 {
+	if (ctl_limit(addr, CTL_D_CACHE))
+		return FALSE;
+
     switch (type) {
     case HOST_ANY:
     	return hcache_add(HCACHE_FRESH_ANY, addr, port, what);
@@ -804,6 +829,9 @@ gboolean
 hcache_add_valid(host_type_t type, const host_addr_t addr, guint16 port,
 	const char *what)
 {
+	if (ctl_limit(addr, CTL_D_CACHE))
+		return FALSE;
+
     switch (type) {
     case HOST_ANY:
     	return hcache_add(HCACHE_VALID_ANY, addr, port, what);
@@ -840,6 +868,29 @@ hcache_remove(gnet_host_t *h)
     hcache_unlink_host(hc, h);
 }
 
+/**
+ * Purge host from fresh/valid caches.
+ */
+void
+hcache_purge(const host_addr_t addr, guint16 port)
+{
+	hostcache_entry_t *hce;
+	gnet_host_t *host;
+
+	if (hcache_ht_get(addr, port, &host, &hce)) {
+		switch (hce->type) {
+		case HCACHE_FRESH_ANY:
+		case HCACHE_VALID_ANY:
+		case HCACHE_FRESH_ULTRA:
+		case HCACHE_VALID_ULTRA:
+			hcache_remove(host);
+			return;
+		default:
+			break;
+		}
+	}
+}
+
 /**
  * Do we have less that our mimumum amount of hosts in the cache?
  */
@@ -1066,7 +1117,7 @@ hcache_fill_caught_array(host_type_t type, gnet_host_t *hosts, int hcount)
 {
 	int i;
 	hostcache_t *hc = NULL;
-	GHashTable *seen_host = g_hash_table_new(host_hash, host_eq);
+	GHashTable *seen_host = g_hash_table_new(gnet_host_hash, gnet_host_eq);
 	hash_list_iter_t *iter;
 
     switch (type) {
@@ -1423,7 +1474,7 @@ hcache_timer(time_t now)
 void
 hcache_init(void)
 {
-	ht_known_hosts = g_hash_table_new(host_hash, host_eq);
+	ht_known_hosts = g_hash_table_new(gnet_host_hash, gnet_host_eq);
 
     caches[HCACHE_FRESH_ANY] = hcache_alloc(
         HCACHE_FRESH_ANY,
@@ -1462,6 +1513,11 @@ hcache_init(void)
         PROP_HOSTS_IN_BAD_CATCHER,
         "hosts.unstable");
     caches[HCACHE_UNSTABLE]->addr_only = TRUE;
+
+    caches[HCACHE_ALIEN] = hcache_alloc(
+        HCACHE_ALIEN,
+        PROP_HOSTS_IN_BAD_CATCHER,
+        "hosts.alien");
 }
 
 /**
@@ -1533,7 +1589,8 @@ hcache_close(void)
         HCACHE_VALID_ULTRA,
         HCACHE_TIMEOUT,
         HCACHE_BUSY,
-        HCACHE_UNSTABLE
+        HCACHE_UNSTABLE,
+        HCACHE_ALIEN
     };
 	guint i;
 
