@@ -1,5 +1,5 @@
 /*
- * $Id: hostiles.c 16106 2009-03-10 15:16:31Z rmanfredi $
+ * $Id: hostiles.c 17194 2009-11-05 21:44:06Z rmanfredi $
  *
  * Copyright (c) 2004, Raphael Manfredi
  * Copyright (c) 2003, Markus Goetz
@@ -40,7 +40,7 @@
 
 #include "common.h"
 
-RCSID("$Id: hostiles.c 16106 2009-03-10 15:16:31Z rmanfredi $")
+RCSID("$Id: hostiles.c 17194 2009-11-05 21:44:06Z rmanfredi $")
 
 #include "hostiles.h"
 #include "settings.h"
@@ -50,6 +50,10 @@ RCSID("$Id: hostiles.c 16106 2009-03-10 15:16:31Z rmanfredi $")
 #include "lib/ascii.h"
 #include "lib/glib-missing.h"
 #include "lib/iprange.h"
+#include "lib/halloc.h"
+#include "lib/parse.h"
+#include "lib/path.h"
+#include "lib/stringify.h"
 #include "lib/walloc.h"
 #include "lib/watcher.h"
 
@@ -102,7 +106,7 @@ hostiles_load(FILE *f, hostiles_t which)
 	int bits;
 	iprange_err_t error;
 
-	g_assert((int) which >= 0 && which < NUM_HOSTILES);
+	g_assert(UNSIGNED(which) < NUM_HOSTILES);
 	g_assert(NULL == hostile_db[which]);
 
 	hostile_db[which] = iprange_new();
@@ -142,7 +146,7 @@ hostiles_load(FILE *f, hostiles_t which)
 			break;
 			/* FALL THROUGH */
 		default:
-			if (GNET_PROPERTY(dbg) > 0 || error != IPR_ERR_RANGE_SUBNET) {
+			if (GNET_PROPERTY(reload_debug) || error != IPR_ERR_RANGE_SUBNET) {
 				g_warning("%s, line %d: rejected entry \"%s\" (%s/%d): %s",
 					hostiles_file, linenum, line, ip_to_string(ip), bits,
 					iprange_strerror(error));
@@ -153,9 +157,9 @@ hostiles_load(FILE *f, hostiles_t which)
 
 	iprange_sync(hostile_db[which]);
 
-	if (GNET_PROPERTY(dbg)) {
-		g_message("loaded %u hostile IP addresses/netmasks (%u hosts)",
-			iprange_get_item_count(hostile_db[which]),
+	if (GNET_PROPERTY(reload_debug)) {
+		g_message("loaded %u addresses/netmasks from %s (%u hosts)",
+			iprange_get_item_count(hostile_db[which]), hostiles_what[which],
 			iprange_get_host_count(hostile_db[which]));
 	}
 	return iprange_get_item_count(hostile_db[which]);
@@ -174,7 +178,7 @@ hostiles_changed(const char *filename, gpointer udata)
 	hostiles_t which;
 
 	which = GPOINTER_TO_UINT(udata);
-	g_assert((int) which >= 0 && which < NUM_HOSTILES);
+	g_assert(UNSIGNED(which) < NUM_HOSTILES);
 
 	f = file_fopen(filename, "r");
 	if (f == NULL)
@@ -199,11 +203,11 @@ hostiles_retrieve_from_file(FILE *f, hostiles_t which,
 	g_assert(f);
 	g_assert(path);
 	g_assert(filename);
-	g_assert((int) which >= 0 && which < NUM_HOSTILES);
+	g_assert(UNSIGNED(which) < NUM_HOSTILES);
 
 	pathname = make_pathname(path, filename);
 	watcher_register(pathname, hostiles_changed, GUINT_TO_POINTER(which));
-	G_FREE_NULL(pathname);
+	HFREE_NULL(pathname);
 	hostiles_load(f, which);
 }
 
@@ -223,7 +227,7 @@ hostiles_retrieve_from_file(FILE *f, hostiles_t which,
 static void
 hostiles_retrieve(hostiles_t which)
 {
-	g_assert((int) which >= 0 && which < NUM_HOSTILES);
+	g_assert(UNSIGNED(which) < NUM_HOSTILES);
 
 	switch (which) {
 	case HOSTILE_PRIVATE:
@@ -313,6 +317,9 @@ hostiles_close(void)
 	for (i = 0; i < NUM_HOSTILES; i++) {
 		hostiles_close_one(i);
 	}
+
+	gnet_prop_remove_prop_changed_listener(PROP_USE_GLOBAL_HOSTILES_TXT,
+		use_global_hostiles_txt_changed);
 }
 
 /**
