@@ -1,5 +1,5 @@
 /*
- * $Id: hosts.h 12576 2006-12-05 19:59:05Z cbiere $
+ * $Id: hosts.h 16223 2009-03-20 21:48:03Z rmanfredi $
  *
  * Copyright (c) 2001-2003, Raphael Manfredi
  *
@@ -28,7 +28,7 @@
 
 #include "common.h"
 
-#include "if/core/hosts.h"
+#include "lib/host_addr.h"
 
 /*
  * Global Data
@@ -48,14 +48,11 @@ void host_add_semi_pong(const host_addr_t addr, guint16 port);
 void host_shutdown(void);
 void host_close(void);
 
-const gchar *gnet_host_to_string(const gnet_host_t *h);
-
-void parse_netmasks(const gchar *value);
+void parse_netmasks(const char *value);
 gboolean host_is_nearby(const host_addr_t addr);
 gboolean host_is_valid(const host_addr_t addr, guint16 port);
-
-guint host_hash(gconstpointer key);
-gint host_eq(gconstpointer v1, gconstpointer v2);
-gint host_cmp(gconstpointer v1, gconstpointer v2);
+gboolean host_address_is_usable(const host_addr_t addr);
 
 #endif /* _core_hosts_h_ */
+
+/* vi: set ts=4 sw=4 cindent: */
