@@ -1,5 +1,5 @@
 /*
- * $Id: hsep.c 16289 2009-03-29 08:41:43Z rmanfredi $
+ * $Id: hsep.c 17220 2009-11-10 18:48:45Z rmanfredi $
  *
  * Copyright (c) 2004, Thomas Schuerger & Jeroen Asselman
  *
@@ -95,7 +95,7 @@
 
 #include "common.h"
 
-RCSID("$Id: hsep.c 16289 2009-03-29 08:41:43Z rmanfredi $")
+RCSID("$Id: hsep.c 17220 2009-11-10 18:48:45Z rmanfredi $")
 
 #include "gmsg.h"
 #include "routing.h"
@@ -110,6 +110,7 @@ RCSID("$Id: hsep.c 16289 2009-03-29 08:41:43Z rmanfredi $")
 
 #include "lib/endian.h"
 #include "lib/glib-missing.h"
+#include "lib/stringify.h"
 #include "lib/tm.h"
 #include "lib/walloc.h"
 #include "lib/override.h"
@@ -514,9 +515,8 @@ hsep_timer(time_t now)
  * to be closed. The connection's HSEP data is restored to
  * zero and the CAN_HSEP attribute is cleared.
  */
-
 void
-hsep_connection_close(struct gnutella_node *n)
+hsep_connection_close(struct gnutella_node *n, gboolean in_shutdown)
 {
 	unsigned int i, j;
 
@@ -527,6 +527,9 @@ hsep_connection_close(struct gnutella_node *n)
 		printf("HSEP: Deinitializing node %s\n",
 			host_addr_port_to_string(n->addr, n->port));
 
+	if (in_shutdown)
+		goto cleanup;
+
 	for (i = 1; i < G_N_ELEMENTS(hsep_global_table); i++) {
 
 		for (j = 0; j < G_N_ELEMENTS(hsep_global_table[0]); j++) {
@@ -535,19 +538,20 @@ hsep_connection_close(struct gnutella_node *n)
 		}
 	}
 
+	if (GNET_PROPERTY(hsep_debug) > 1)
+		hsep_dump_table();
+
+	hsep_fire_global_table_changed(tm_time());
+
 	/*
 	 * Clear CAN_HSEP attribute so that the HSEP code
 	 * will not use the node any longer.
 	 */
 
+cleanup:
 	n->attrs &= ~NODE_A_CAN_HSEP;
 	wfree(n->hsep, sizeof *n->hsep);
 	n->hsep = NULL;
-
-	if (GNET_PROPERTY(hsep_debug) > 1)
-		hsep_dump_table();
-
-	hsep_fire_global_table_changed(tm_time());
 }
 
 static inline void
@@ -721,7 +725,7 @@ void
 hsep_send_msg(struct gnutella_node *n, time_t now)
 {
 	hsep_triple tmp[G_N_ELEMENTS(n->hsep->sent_table)], other;
-	unsigned int i, j, msglen, triples, opttriples;
+	unsigned int i, j, msglen, msgsize, triples, opttriples;
 	gnutella_msg_hsep_t *msg;
 	hsep_ctx_t *hsep;
 
@@ -746,8 +750,8 @@ hsep_send_msg(struct gnutella_node *n, time_t now)
 	 * Allocate and initialize message to send.
 	 */
 
-	msglen = GTA_HEADER_SIZE + triples * (sizeof *msg - GTA_HEADER_SIZE);
-	msg = g_malloc(msglen);
+	msgsize = GTA_HEADER_SIZE + triples * (sizeof *msg - GTA_HEADER_SIZE);
+	msg = walloc(msgsize);
 
 	{
 		gnutella_header_t *header;
@@ -784,7 +788,7 @@ hsep_send_msg(struct gnutella_node *n, time_t now)
 	if (
 		0 == memcmp(tmp, hsep->sent_table, sizeof tmp)
 	) {
-		G_FREE_NULL(msg);
+		WFREE_NULL(msg, msgsize);
 		goto charge_timer;
 	}
 
@@ -839,7 +843,7 @@ hsep_send_msg(struct gnutella_node *n, time_t now)
 	/* send message to peer node */
 	gmsg_sendto_one(n, msg, msglen);
 
-	G_FREE_NULL(msg);
+	WFREE_NULL(msg, msgsize);
 
 	/*
 	 * Update counters.
@@ -871,7 +875,7 @@ hsep_notify_shared(guint64 own_files, guint64 own_kibibytes)
 	) {
 
 		if (GNET_PROPERTY(hsep_debug)) {
-			printf("HSEP: Shared files changed to %s (%s KiB)\n",
+			g_message("HSEP: Shared files changed to %s (%s KiB)",
 			    uint64_to_string(own_files), uint64_to_string2(own_kibibytes));
 		}
 
