@@ -1,5 +1,5 @@
 /*
- * $Id: http.c 16275 2009-03-28 17:40:31Z rmanfredi $
+ * $Id: http.c 17236 2010-02-13 09:35:31Z rmanfredi $
  *
  * Copyright (c) 2002-2003, Raphael Manfredi
  *
@@ -38,7 +38,7 @@
 
 #include "common.h"
 
-RCSID("$Id: http.c 16275 2009-03-28 17:40:31Z rmanfredi $")
+RCSID("$Id: http.c 17236 2010-02-13 09:35:31Z rmanfredi $")
 
 #include "nodes.h"
 #include "http.h"
@@ -51,10 +51,15 @@ RCSID("$Id: http.c 16275 2009-03-28 17:40:31Z rmanfredi $")
 
 #include "lib/atoms.h"
 #include "lib/ascii.h"
+#include "lib/concat.h"
 #include "lib/getline.h"
 #include "lib/glib-missing.h"
 #include "lib/header.h"
+#include "lib/parse.h"
+#include "lib/stringify.h"
+#include "lib/timestamp.h"
 #include "lib/tm.h"
+#include "lib/unsigned.h"
 #include "lib/walloc.h"
 
 #include "if/gnet_property_priv.h"
@@ -115,8 +120,8 @@ http_send_status(
 	 * Prepare flags for callbacks.
 	 */
 
-	if (saturated)					cb_flags |= HTTP_CBF_BW_SATURATED;
-	if (code == 503)				cb_flags |= HTTP_CBF_BUSY_SIGNAL;
+	if (saturated)		cb_flags |= HTTP_CBF_BW_SATURATED;
+	if (code == 503)	cb_flags |= HTTP_CBF_BUSY_SIGNAL;
 
 	/*
 	 * On 5xx errors, limit the header to 1K max, a priori.  This will be
@@ -292,17 +297,18 @@ http_send_status(
  * HTTP status callback.
  *
  * Add an X-Hostname line bearing the fully qualified hostname.
+ *
+ * The ``arg'' parameter is interpreted as a flag: if TRUE, it forces the
+ * emission of the header even if bandwidth is tight (HTTP_CBF_SMALL_REPLY).
  */
 size_t
-http_hostname_add(char *buf, size_t size, gpointer unused_arg, guint32 flags)
+http_hostname_add(char *buf, size_t size, gpointer arg, guint32 flags)
 {
 	size_t len;
 
-	(void) unused_arg;
-
-	if (flags & HTTP_CBF_SMALL_REPLY)
+	if (arg == NULL && (flags & HTTP_CBF_SMALL_REPLY))
 		return 0;
-	if (GNET_PROPERTY(server_hostname)[0])
+	if (is_null_or_empty(GNET_PROPERTY(server_hostname)))
 		return 0;
 
 	len = concat_strings(buf, size,
@@ -584,8 +590,11 @@ static const char * const parse_errstr[] = {
 const char *
 http_url_strerror(http_url_error_t errnum)
 {
-	if ((int) errnum < 0 || errnum >= G_N_ELEMENTS(parse_errstr))
-		return "Invalid error code";
+	if (UNSIGNED(errnum) >= G_N_ELEMENTS(parse_errstr)) {
+		static char buf[40];
+		gm_snprintf(buf, sizeof buf, "Invalid URL error code: %u", errnum);
+		return buf;
+	}
 
 	return parse_errstr[errnum];
 }
@@ -707,8 +716,8 @@ http_buffer_alloc(char *buf, size_t len, size_t written)
 	http_buffer_t *b;
 
 	g_assert(buf);
-	g_assert(len > 0 && len <= INT_MAX);
-	g_assert((int) written >= 0 && written < len);
+	g_assert(size_is_positive(len));
+	g_assert(written < len);
 
 	b = walloc(sizeof(*b));
 	b->hb_arena = walloc(len);		/* Should be small enough for walloc */
@@ -958,11 +967,13 @@ http_range_parse(
 	if (NULL != (str = is_strprefix(str, unit))) {
 		c = *str;
 		if (!is_ascii_space(c) && c != '=') {
-			g_warning("improper %s header from <%s>: %s", field, vendor, value);
+			if (GNET_PROPERTY(http_debug)) g_warning(
+				"improper %s header from <%s>: %s", field, vendor, value);
 			return NULL;
 		}
 	} else {
-		g_warning("improper %s header from <%s> (not bytes?): %s",
+		if (GNET_PROPERTY(http_debug)) g_warning(
+			"improper %s header from <%s> (not bytes?): %s",
 			field, vendor, value);
 		return NULL;
 	}
@@ -975,7 +986,8 @@ http_range_parse(
 	while ((c = *str)) {
 		if (c == '=') {
 			if (request) {
-				g_warning("improper %s header from <%s> (multiple '='): %s",
+				if (GNET_PROPERTY(http_debug)) g_warning(
+					"improper %s header from <%s> (multiple '='): %s",
 					field, vendor, value);
 				return NULL;
 			}
@@ -1009,24 +1021,25 @@ http_range_parse(
 
 			if (!minus_seen) {
 				if (GNET_PROPERTY(http_debug)) g_warning(
-					"weird %s header from <%s>, offset %d (no range?): "
-					"%s", field, vendor, (int) (str - value) - 1, value);
+					"weird %s header from <%s>, offset %lu (no range?): "
+					"%s", field, vendor, (unsigned long) (str - value) - 1,
+					value);
 				goto reset;
 			}
 
 			if (start == HTTP_OFFSET_MAX && !has_end) {	/* Bad negative range */
 				if (GNET_PROPERTY(http_debug)) g_warning(
-					"weird %s header from <%s>, offset %d "
+					"weird %s header from <%s>, offset %lu "
 					"(incomplete negative range): %s",
-					field, vendor, (int) (str - value) - 1, value);
+					field, vendor, (unsigned long) (str - value) - 1, value);
 				goto reset;
 			}
 
 			if (start > end) {
 				if (GNET_PROPERTY(http_debug)) g_warning(
-					"weird %s header from <%s>, offset %d "
+					"weird %s header from <%s>, offset %lu "
 					"(swapped range?): %s", field, vendor,
-					(int) (str - value) - 1, value);
+					(unsigned long) (str - value) - 1, value);
 				goto reset;
 			}
 
@@ -1036,10 +1049,10 @@ http_range_parse(
 
 			if (ignored) {
 				if (GNET_PROPERTY(http_debug)) g_warning(
-					"weird %s header from <%s>, offset %d "
+					"weird %s header from <%s>, offset %lu "
 					"(ignored range #%d): %s",
-					field, vendor, (int) (str - value) - 1, count,
-					value);
+					field, vendor, (unsigned long) (str - value) - 1,
+					count, value);
 			}
 
 			goto reset;
@@ -1051,17 +1064,18 @@ http_range_parse(
 		if (c == '-') {
 			if (minus_seen) {
 				if (GNET_PROPERTY(http_debug)) g_warning(
-					"weird %s header from <%s>, offset %d (spurious '-'): %s",
-					field, vendor, (int) (str - value) - 1, value);
+					"weird %s header from <%s>, offset %lu (spurious '-'): %s",
+					field, vendor, (unsigned long) (str - value) - 1, value);
 				goto resync;
 			}
 			minus_seen = TRUE;
 			if (!has_start) {		/* Negative range */
 				if (!request) {
 					if (GNET_PROPERTY(http_debug))
-						g_warning("weird %s header from <%s>, offset %d "
+						g_warning("weird %s header from <%s>, offset %lu "
 							"(negative range in reply): %s",
-							field, vendor, (int) (str - value) - 1, value);
+							field, vendor, (unsigned long) (str - value) - 1,
+							value);
 					goto resync;
 				}
 				start = HTTP_OFFSET_MAX;	/* Indicates negative range */
@@ -1082,9 +1096,9 @@ http_range_parse(
 
 			if (has_end) {
 				if (GNET_PROPERTY(http_debug))
-					g_warning("weird %s header from <%s>, offset %d "
+					g_warning("weird %s header from <%s>, offset %lu "
 						"(spurious boundary %s): %s",
-						field, vendor, (int) (str - value) - 1,
+						field, vendor, (unsigned long) (str - value) - 1,
 						uint64_to_string(val), value);
 				goto resync;
 			}
@@ -1100,9 +1114,9 @@ http_range_parse(
 			if (has_start) {
 				if (!minus_seen) {
 					if (GNET_PROPERTY(http_debug))
-						g_warning("weird %s header from <%s>, offset %d "
+						g_warning("weird %s header from <%s>, offset %lu "
 							"(no '-' before boundary %s): %s",
-							field, vendor, (int) (str - value) - 1,
+							field, vendor, (unsigned long) (str - value) - 1,
 							uint64_to_string(val), value);
 					goto resync;
 				}
@@ -1346,6 +1360,31 @@ http_range_merge(GSList *old_list, GSList *new_list)
 				continue;
 			}
 
+			/*
+			 * If we encounter a range ending at the same spot as the
+			 * latest entry but which starts earlier, correct the starting
+			 * point of the latest range.
+			 */
+
+			if (result_list != NULL) {
+				if (old_range->end == highest) {
+					http_range_t *latest = result_list->data;
+					g_assert(latest->end == highest);
+					if (latest->start > old_range->start)
+						latest->start = old_range->start;
+					old = g_slist_next(old);
+					continue;
+				}
+				if (new_range->end == highest) {
+					http_range_t *latest = result_list->data;
+					g_assert(latest->end == highest);
+					if (latest->start > new_range->start)
+						latest->start = new_range->start;
+					new = g_slist_next(new);
+					continue;
+				}
+			}
+
 			/*
 			 * First handle the non-overlapping case. Copy the first
 			 * non-overlapping range, and move to the next range in
@@ -1360,7 +1399,7 @@ http_range_merge(GSList *old_list, GSList *new_list)
 				continue;
 			}
 			if (old_range->end < new_range->start) {
-				highest = new_range->end;
+				highest = old_range->end;
 				result_list = g_slist_prepend(result_list,
 									http_range_clone(old_range));
 
@@ -1463,8 +1502,12 @@ guint http_async_errno;		/**< Used to return error codes during setup */
 const char *
 http_async_strerror(guint errnum)
 {
-	if (errnum >= G_N_ELEMENTS(error_str))
-		return "Invalid error code";
+	if (errnum >= G_N_ELEMENTS(error_str)) {
+		static char buf[50];
+		gm_snprintf(buf, sizeof buf,
+			"Invalid HTTP async error code: %u", errnum);
+		return buf;
+	}
 
 	return error_str[errnum];
 }
@@ -1487,7 +1530,7 @@ static const char * const http_verb[NUM_HTTP_REQTYPES] = {
 	"POST",
 };
 
-enum http_async_magic { HTTP_ASYNC_MAGIC = 0xa91cf3eeU };
+enum http_async_magic { HTTP_ASYNC_MAGIC = 0x291cf3eeU };
 
 /**
  * An asynchronous HTTP request.
@@ -1888,7 +1931,7 @@ http_async_got_reply(const struct http_async *unused_ha,
 	if (GNET_PROPERTY(http_trace) & SOCK_TRACE_IN) {
 		g_message("----Got HTTP reply from %s:",
 			host_addr_to_string(s->addr));
-		fprintf(stderr, "%s", status);
+		fprintf(stderr, "%s\n", status);
 		header_dump(stderr, header, "----");
 	}
 }
@@ -1958,14 +2001,15 @@ http_async_create(
 
 		if (string_to_host_addr(host, NULL, &ip)) {
 			host = NULL;
-			s = socket_connect(ip, uport, SOCK_TYPE_HTTP, 0);
+			s = socket_connect(ip, uport, SOCK_TYPE_HTTP, SOCK_F_FORCE);
 		} else {
-			s = socket_connect_by_name(host, uport, SOCK_TYPE_HTTP, 0);
+			s = socket_connect_by_name(host, uport,
+					SOCK_TYPE_HTTP, SOCK_F_FORCE);
 		}
 	} else {
 		host = NULL;
 		path = url;
-		s = socket_connect(addr, port, SOCK_TYPE_HTTP, 0);
+		s = socket_connect(addr, port, SOCK_TYPE_HTTP, SOCK_F_FORCE);
 	}
 
 	if (s == NULL) {
@@ -2702,10 +2746,20 @@ http_async_connected(struct http_async *ha)
  * Error indication callback which logs the error by listing the
  * initial HTTP request and the reported error cause.  The specified
  * debugging level is explicitly given.
+ *
+ * @param handle		the asynchronous HTTP request
+ * @param type			the error type, as reported to the error callback
+ * @param v				the opaque error description from error callback
+ * @param prefix		logging prefix to be inserted in logging messages
+ * @param all			if TRUE, also log explicit user cancels
+ *
+ * If no prefix is supplied, "HTTP" is used.
+ *
+ * @return TRUE if anything was logged.
  */
-void
+gboolean
 http_async_log_error_dbg(struct http_async *handle,
-	http_errtype_t type, gpointer v, guint32 dbg_level)
+	http_errtype_t type, gpointer v, const gchar *prefix, gboolean all)
 {
 	const char *url;
 	const char *req;
@@ -2713,6 +2767,7 @@ http_async_log_error_dbg(struct http_async *handle,
 	http_error_t *herror = v;
 	host_addr_t addr;
 	guint16 port;
+	const char *what = prefix != NULL ? prefix : "HTTP";
 
 	http_async_check(handle);
 
@@ -2720,58 +2775,67 @@ http_async_log_error_dbg(struct http_async *handle,
 
 	switch (type) {
 	case HTTP_ASYNC_SYSERR:
-        if (dbg_level) {
-            g_message("aborting \"%s %s\" at %s on system error: %s",
-                req, url, host_addr_port_to_string(addr, port),
-				g_strerror(error));
-        }
-		return;
+		g_message("%s: aborting \"%s %s\" at %s on system error: %s",
+			what, req, url, host_addr_port_to_string(addr, port),
+			g_strerror(error));
+		return TRUE;
 	case HTTP_ASYNC_ERROR:
 		if (error == HTTP_ASYNC_CANCELLED) {
-			if (dbg_level > 3)
-				g_message("explicitly cancelled \"%s %s\" at %s", req, url,
-					host_addr_port_to_string(addr, port));
+			if (all) {
+				g_message("%s: explicitly cancelled \"%s %s\" at %s",
+					what, req, url, host_addr_port_to_string(addr, port));
+				return TRUE;
+			}
 		} else if (error == HTTP_ASYNC_CLOSED) {
-			if (dbg_level > 3)
-				g_message("connection closed for \"%s %s\" at %s", req, url,
-					host_addr_port_to_string(addr, port));
-		} else
-            if (dbg_level) {
-                g_message("aborting \"%s %s\" at %s on error: %s", req, url,
-                    host_addr_port_to_string(addr, port),
-					http_async_strerror(error));
-            }
-		return;
+			if (all) {
+				g_message("%s: connection closed for \"%s %s\" at %s",
+					what, req, url, host_addr_port_to_string(addr, port));
+				return TRUE;
+			}
+		} else {
+			g_message("%s: aborting \"%s %s\" at %s on error: %s",
+				what, req, url,
+				host_addr_port_to_string(addr, port),
+				http_async_strerror(error));
+			return TRUE;
+		}
+		return FALSE;
 	case HTTP_ASYNC_HEADER:
-        if (dbg_level) {
-            g_message("aborting \"%s %s\" at %s on header parsing error: %s",
-                req, url, host_addr_port_to_string(addr, port),
-				header_strerror(error));
-        }
-		return;
+		g_message("%s: aborting \"%s %s\" at %s on header parsing error: %s",
+			what, req, url, host_addr_port_to_string(addr, port),
+			header_strerror(error));
+		return TRUE;
 	case HTTP_ASYNC_HTTP:
-        if (dbg_level) {
-            g_message("stopping \"%s %s\" at %s: HTTP %d %s", req, url,
-                host_addr_port_to_string(addr, port),
-				herror->code, herror->message);
-        }
-		return;
+		g_message("%s: stopping \"%s %s\" at %s: HTTP %d %s",
+			what, req, url,
+			host_addr_port_to_string(addr, port),
+			herror->code, herror->message);
+		return TRUE;
 	/* No default clause, let the compiler warn us about missing cases. */
 	}
 
 	/* In case the error was not trapped at compile time... */
 	g_error("unhandled HTTP request error type %d", type);
 	/* NOTREACHED */
+	return FALSE;	/* Avoid compiler warnings about missing returned value */
 }
 
 /**
  * Default error indication callback which logs the error by listing the
  * initial HTTP request and the reported error cause.
+ *
+ * @return whether anything was logged.
  */
-void
-http_async_log_error(struct http_async *handle, http_errtype_t type, gpointer v)
+gboolean
+http_async_log_error(struct http_async *handle,
+	http_errtype_t type, gpointer v, const char *prefix)
 {
-	return http_async_log_error_dbg(handle, type, v, GNET_PROPERTY(http_debug));
+	if (GNET_PROPERTY(http_debug)) {
+		return http_async_log_error_dbg(handle, type, v, prefix,
+			GNET_PROPERTY(http_debug) >= 4);
+	}
+
+	return FALSE;
 }
 
 /***
