@@ -1,5 +1,5 @@
 /*
- * $Id: http.h 16268 2009-03-28 09:08:21Z rmanfredi $
+ * $Id: http.h 17286 2010-02-24 18:42:18Z rmanfredi $
  *
  * Copyright (c) 2002-2003, Raphael Manfredi
  *
@@ -122,22 +122,20 @@ http_extra_body_set(http_extra_desc_t *he, const char *body)
 }
 
 static inline gboolean
-http_extra_callback_matches(http_extra_desc_t *he,
-	http_status_cb_t callback, gpointer user_arg)
+http_extra_callback_matches(http_extra_desc_t *he, http_status_cb_t callback)
 {
 	return he->he_type == HTTP_EXTRA_CALLBACK &&
-		he->he_cb == callback &&
-		he->he_arg == user_arg;
+		he->he_cb == callback;
 }
 
 /*
  * Flags used during callback invocation.
  */
 
-#define HTTP_CBF_SMALL_REPLY	0x00000001	/**< Try to emit smallest reply */
-#define HTTP_CBF_BW_SATURATED	0x00000002	/**< Bandwidth is saturated */
-#define HTTP_CBF_BUSY_SIGNAL	0x00000004	/**< Sending back a 503 "busy" */
-#define HTTP_CBF_SHOW_RANGES	0x00000008	/**< Show available ranges */
+#define HTTP_CBF_SMALL_REPLY	(1 << 0)	/**< Try to emit smallest reply */
+#define HTTP_CBF_BW_SATURATED	(1 << 1)	/**< Bandwidth is saturated */
+#define HTTP_CBF_BUSY_SIGNAL	(1 << 2)	/**< Sending back a 503 "busy" */
+#define HTTP_CBF_SHOW_RANGES	(1 << 3)	/**< Show available ranges */
 
 struct header;
 struct http_async;
@@ -327,10 +325,10 @@ http_state_t http_async_state(struct http_async *handle);
 void http_async_set_opaque(struct http_async *handle,
 		gpointer data, http_user_free_t fn);
 gpointer http_async_get_opaque(struct http_async *handle);
-void http_async_log_error(struct http_async *handle,
-		http_errtype_t type, gpointer v);
-void http_async_log_error_dbg(struct http_async *handle,
-		http_errtype_t type, gpointer v, guint32 dbg_level);
+gboolean http_async_log_error(struct http_async *handle,
+		http_errtype_t type, gpointer v, const char *prefix);
+gboolean http_async_log_error_dbg(struct http_async *handle,
+		http_errtype_t type, gpointer v, const char *prefix, gboolean all);
 
 void http_async_on_state_change(struct http_async *ha, http_state_change_t fn);
 void http_async_allow_redirects(struct http_async *ha, gboolean allow);
