@@ -1,5 +1,5 @@
 /*
- * $Id: huge.c 16106 2009-03-10 15:16:31Z rmanfredi $
+ * $Id: huge.c 17300 2010-02-26 16:33:57Z rmanfredi $
  *
  * Copyright (c) 2002-2003, Ch. Tronche & Raphael Manfredi
  *
@@ -39,7 +39,7 @@
 
 #include "common.h"
 
-RCSID("$Id: huge.c 16106 2009-03-10 15:16:31Z rmanfredi $")
+RCSID("$Id: huge.c 17300 2010-02-26 16:33:57Z rmanfredi $")
 
 #include "huge.h"
 #include "share.h"
@@ -54,8 +54,12 @@ RCSID("$Id: huge.c 16106 2009-03-10 15:16:31Z rmanfredi $")
 #include "lib/atoms.h"
 #include "lib/base32.h"
 #include "lib/file.h"
+#include "lib/halloc.h"
 #include "lib/header.h"
+#include "lib/parse.h"
+#include "lib/pattern.h"
 #include "lib/sha1.h"
+#include "lib/stringify.h"
 #include "lib/tm.h"
 #include "lib/urn.h"
 #include "lib/walloc.h"
@@ -102,15 +106,12 @@ struct sha1_cache_entry {
 static GHashTable *sha1_cache;
 
 /**
- * cache_dirty means that in-core cache is different from the one on disk when
- * TRUE.
+ * cache_dirty = TRUE means that in-core cache is different from the disk one.
  */
 static gboolean cache_dirty;
 static time_t cache_dumped;
 
-/**
- ** Elementary operations on SHA1 values
- **/
+static cpattern_t *has_http_urls;
 
 /**
  ** Handling of persistent buffer
@@ -156,7 +157,7 @@ add_volatile_cache_entry(const char *filename, filesize_t size, time_t mtime,
 	g_hash_table_insert(sha1_cache, deconstify_gchar(item->file_name), item);
 }
 
-/** Disk cache */
+/* Disk cache */
 
 static const char sha1_persistent_cache_file_header[] =
 "#\n"
@@ -219,7 +220,7 @@ add_persistent_cache_entry(const char *filename, filesize_t size,
 		g_warning("add_persistent_cache_entry: could not open \"%s\"",
 			pathname);
 	}
-	G_FREE_NULL(pathname);
+	HFREE_NULL(pathname);
 }
 
 struct dump_cache_context {
@@ -665,52 +666,6 @@ request_sha1(struct shared_file *sf)
 	}
 }
 
-/**
- ** Init
- **/
-
-/**
- * Initialize SHA1 module.
- */
-void
-huge_init(void)
-{
-	sha1_cache = g_hash_table_new(pointer_hash_func, NULL);
-	sha1_read_cache();
-}
-
-/**
- * Free SHA1 cache entry.
- */
-static gboolean
-cache_free_entry(gpointer unused_key, gpointer v, gpointer unused_udata)
-{
-	struct sha1_cache_entry *e = v;
-
-	(void) unused_key;
-	(void) unused_udata;
-
-	atom_str_free_null(&e->file_name);
-	atom_sha1_free_null(&e->sha1);
-	atom_tth_free_null(&e->tth);
-	wfree(e, sizeof *e);
-
-	return TRUE;
-}
-
-/**
- * Called when servent is shutdown.
- */
-void
-huge_close(void)
-{
-	dump_cache(FALSE);
-
-	g_hash_table_foreach_remove(sha1_cache, cache_free_entry, NULL);
-	g_hash_table_destroy(sha1_cache);
-	sha1_cache = NULL;
-}
-
 /**
  * Test whether the SHA1 in its base32/binary form is improbable.
  *
@@ -746,7 +701,7 @@ huge_improbable_sha1(const char *buf, size_t len)
  * of a SHA1 hash, and write decoded value in `sha1'.
  * Also make sure that the SHA1 is not an improbable value.
  *
- * `header' is the header of the packet where we found the SHA1, so that we
+ * `n' is the node receiving the packet where we found the SHA1, so that we
  * may trace errors if needed.
  *
  * When `check_old' is true, check the encoding against an earlier version
@@ -756,7 +711,7 @@ huge_improbable_sha1(const char *buf, size_t len)
  */
 gboolean
 huge_sha1_extract32(const char *buf, size_t len, struct sha1 *sha1,
-	gconstpointer header)
+	const struct gnutella_node *n)
 {
 	if (len != SHA1_BASE32_SIZE || huge_improbable_sha1(buf, len))
 		goto bad;
@@ -771,9 +726,11 @@ huge_sha1_extract32(const char *buf, size_t len, struct sha1 *sha1,
 	if (huge_improbable_sha1(sha1->data, sizeof sha1->data)) {
 		if (GNET_PROPERTY(dbg)) {
 			if (is_printable(buf, len)) {
-				g_warning("%s has bad SHA1 (len=%d): %.*s, hex: %s",
-					gmsg_infostr(header), (int) len, (int) len, buf,
-					data_hex_str(sha1->data, sizeof sha1->data));
+				g_warning("%s has improbable SHA1 (len=%lu): %.*s, hex: %s",
+					gmsg_node_infostr(n),
+					(unsigned long) len,
+					(int) MIN(len, (size_t) INT_MAX),
+					buf, data_hex_str(sha1->data, sizeof sha1->data));
 			} else
 				goto bad;		/* SHA1 should be printable originally */
 		}
@@ -786,13 +743,13 @@ bad:
 	if (GNET_PROPERTY(dbg)) {
 		if (is_printable(buf, len)) {
 			g_warning("%s has bad SHA1 (len=%u): %.*s",
-				gmsg_infostr(header),
+				gmsg_node_infostr(n),
 				(unsigned) len,
 				(int) MIN(len, (size_t) INT_MAX),
 				buf);
 		} else {
 			g_warning("%s has bad SHA1 (len=%u)",
-				gmsg_infostr(header), (unsigned) len);
+				gmsg_node_infostr(n), (unsigned) len);
 			if (len)
 				dump_hex(stderr, "Base32 SHA1", buf, len);
 		}
@@ -803,7 +760,7 @@ bad:
 
 gboolean
 huge_tth_extract32(const char *buf, size_t len, struct tth *tth,
-	gconstpointer header)
+	const struct gnutella_node *n)
 {
 	if (len != TTH_BASE32_SIZE)
 		goto bad;
@@ -817,13 +774,13 @@ bad:
 	if (GNET_PROPERTY(dbg)) {
 		if (is_printable(buf, len)) {
 			g_warning("%s has bad TTH (len=%u): %.*s",
-				gmsg_infostr(header),
+				gmsg_node_infostr(n),
 				(unsigned) len,
 				(int) MIN(len, (size_t) INT_MAX),
 				buf);
 		} else {
 			g_warning("%s has bad TTH (len=%u",
-				gmsg_infostr(header), (unsigned) len);
+				gmsg_node_infostr(n), (unsigned) len);
 			if (len)
 				dump_hex(stderr, "Base32 TTH", buf, len);
 		}
@@ -831,15 +788,50 @@ bad:
 	return FALSE;
 }
 
+/**
+ * Is the X-Alt header really holding a collection of IP:port (with possible
+ * push alt-locs containing a prefixing GUID) or is it mistakenly called
+ * X-Alt but is really an old X-Gnutella-Alternate-Location containing a
+ * list of HTTP URLs.
+ */
+static gboolean
+huge_is_pure_xalt(const char *value, size_t len)
+{
+	host_addr_t addr;
+
+	/*
+	 * This is pure heuristics, knowing that if we return TRUE, we'll parse
+	 * the X-Alt header in the format that is emitted by the majority of
+	 * vendors.
+	 *
+	 * We try to avoid the more costly pattern_qsearch() call if we can,
+	 * and our heuristic should catch 99% of the cases.  And pattern_qsearch()
+	 * should quickly return true if the X-Alt is not a collection of IP:port.
+	 */
+
+	if (is_strcaseprefix(value, "tls="))
+		return TRUE;
+
+	if (string_to_host_addr(value, NULL, &addr))
+		return TRUE;
+
+	if (pattern_qsearch(has_http_urls, value, len, 0, qs_any))
+		return FALSE;
+
+	return TRUE;
+}
+
 /**
  * Parse the "X-Gnutella-Alternate-Location" header if present to learn
  * about other sources for this file.
+ *
+ * Also knows about "Alternate-Location", "Alt-Location", "X-Alt" and "X-Falt".
  */
 void
 huge_collect_locations(const struct sha1 *sha1, const header_t *header)
 {
 	char *alt;
-   
+	size_t len;
 
 	g_return_if_fail(sha1);
 	g_return_if_fail(header);
@@ -857,16 +849,82 @@ huge_collect_locations(const struct sha1 *sha1, const header_t *header)
 	if (alt == NULL)
 		alt = header_get(header, "Alt-Location");
 
-	if (alt) {
+	if (alt != NULL) {
 		dmesh_collect_locations(sha1, alt);
 		return;
 	}
 
-	alt = header_get(header, "X-Alt");
+	alt = header_get_extended(header, "X-Alt", &len);
 
-	if (alt) {
-		dmesh_collect_compact_locations(sha1, alt);
+	if (alt != NULL) {
+		/*
+		 * Wonderful Shareaza now uses X-Alt but does not pass compact
+		 * locations.  In essence, they renamed Alt-Location to X-Alt
+		 * without changing the format of the value.  Great job.
+		 *		--RAM, 2010-02-22
+		 */
+
+		if (huge_is_pure_xalt(alt, len))
+			dmesh_collect_compact_locations(sha1, alt);
+		else
+			dmesh_collect_locations(sha1, alt);
     }
+
+	/*
+	 * Firewalled locations.
+	 */
+
+	alt = header_get(header, "X-Falt");
+
+	if (alt != NULL) {
+		dmesh_collect_fw_hosts(sha1, alt);
+	}
+}
+
+/**
+ * Initialize the HUGE layer.
+ */
+void
+huge_init(void)
+{
+	sha1_cache = g_hash_table_new(pointer_hash_func, NULL);
+	sha1_read_cache();
+	has_http_urls = pattern_compile("http://");
+}
+
+/**
+ * Free SHA1 cache entry.
+ */
+static gboolean
+cache_free_entry(gpointer unused_key, gpointer v, gpointer unused_udata)
+{
+	struct sha1_cache_entry *e = v;
+
+	(void) unused_key;
+	(void) unused_udata;
+
+	atom_str_free_null(&e->file_name);
+	atom_sha1_free_null(&e->sha1);
+	atom_tth_free_null(&e->tth);
+	wfree(e, sizeof *e);
+
+	return TRUE;
+}
+
+/**
+ * Called when servent is shutdown.
+ */
+void
+huge_close(void)
+{
+	dump_cache(FALSE);
+
+	g_hash_table_foreach_remove(sha1_cache, cache_free_entry, NULL);
+	g_hash_table_destroy(sha1_cache);
+	sha1_cache = NULL;
+
+	pattern_free(has_http_urls);
+	has_http_urls = NULL;
 }
 
 /*
