@@ -1,5 +1,5 @@
 /*
- * $Id: huge.h 16104 2009-03-10 14:58:09Z rmanfredi $
+ * $Id: huge.h 17205 2009-11-08 21:01:48Z rmanfredi $
  *
  * Copyright (c) 2002-2003, Ch. Tronche & Raphael Manfredi
  *
@@ -42,6 +42,7 @@
 
 #include "common.h"
 
+struct gnutella_node;
 struct shared_file;
 struct header;
 struct sha1;
@@ -64,9 +65,9 @@ gboolean huge_update_hashes(struct shared_file *sf,
 
 gboolean huge_improbable_sha1(const char *buf, size_t len);
 gboolean huge_sha1_extract32(const char *buf, size_t len, struct sha1 *sha1,
-	gconstpointer header);
+	const struct gnutella_node *n);
 gboolean huge_tth_extract32(const char *buf, size_t len, struct tth *tth,
-	gconstpointer header);
+	const struct gnutella_node *n);
 void huge_collect_locations(
 	const struct sha1 *sha1, const struct header *header);
 
