@@ -1,5 +1,5 @@
 /*
- * $Id: inet.c 16106 2009-03-10 15:16:31Z rmanfredi $
+ * $Id: inet.c 16687 2009-05-07 19:28:22Z rmanfredi $
  *
  * Copyright (c) 2001-2003, Raphael Manfredi
  *
@@ -35,7 +35,7 @@
 
 #include "common.h"
 
-RCSID("$Id: inet.c 16106 2009-03-10 15:16:31Z rmanfredi $")
+RCSID("$Id: inet.c 16687 2009-05-07 19:28:22Z rmanfredi $")
 
 #include "inet.h"
 #include "nodes.h"				/* For node_become_firewalled() */
@@ -45,8 +45,10 @@ RCSID("$Id: inet.c 16106 2009-03-10 15:16:31Z rmanfredi $")
 #include "if/gnet_property.h"
 #include "if/gnet_property_priv.h"
 
+#include "lib/aging.h"
 #include "lib/cq.h"
 #include "lib/tm.h"
+#include "lib/wd.h"
 #include "lib/walloc.h"
 #include "lib/override.h"		/* Must be the last header included */
 
@@ -73,15 +75,15 @@ static time_t fw_time = 0;				/**< When we last became firewalled */
  * To detect switching from firewalled -> non-firewalled, we use incoming
  * connections (checking done in socket_accept()).
  *
- * To detect switching from non-firewalled -> firewalled, we arm a timer
+ * To detect switching from non-firewalled -> firewalled, we arm a watchdog
  * each time we get an incoming connection.  If we don't get another
  * connection before the timer expires, we might have switched to firewalled
  * mode.
  */
 
-static cevent_t *incoming_ev;		/**< Callout queue timer */
-static cevent_t *incoming_udp_ev;	/**< Idem */
-static cevent_t *solicited_udp_ev;	/**< Idem */
+static watchdog_t *incoming_wd;
+static watchdog_t *incoming_udp_wd;
+static watchdog_t *solicited_udp_wd;
 
 /*
  * Unfortunately, to accurately detect true unsolicited UDP traffic, we have
@@ -92,13 +94,35 @@ static cevent_t *solicited_udp_ev;	/**< Idem */
 
 #define FW_UDP_WINDOW			120		/**< 2 minutes, in most firewalls */
 
-static GHashTable *outgoing_udp = NULL;		/**< Maps "IP" => "ip_record" */
+static aging_table_t *outgoing_udp;		/**< IP addresses to whom we send */
 
 struct ip_record {
 	host_addr_t addr;			/**< The IP address to which we sent data */
 	cevent_t *timeout_ev;		/**< The expiration time for the fw breach */
 };
 
+/**
+ * States of our small automaton to try to detect reception of unsolicited
+ * messages if we do not get them naturally: there are only a few such messages
+ * that can only be viewed as unsolicited.
+ *
+ * SO if we don't get these, we enter the mode UNSOLICITED_PREPARE in which
+ * we just record the addresses of whom we send things to.  We ignore any
+ * unsolicited checks for non-matching addresses at this point.  After
+ * FW_UDP_WINDOW seconds have elapsed, we enter UNSOLICITED_CHECK and the
+ * first time we get an unsolicited packet, we know...  At which point we
+ * move back to UNSOLICTED_OFF to turn off these checks.
+ */
+
+enum solicited_states {
+	UNSOLICITED_OFF = 0,
+	UNSOLICITED_PREPARE,
+	UNSOLICITED_CHECK
+};
+
+static enum solicited_states outgoing_udp_state = UNSOLICITED_OFF;
+static cevent_t *unsolicited_udp_ev;
+
 /***
  *** External connection status management structures.
  ***/
@@ -112,71 +136,10 @@ struct ip_record {
 
 #define OUTGOING_WINDOW		150			/**< Outgoing monitoring window */
 
-static gboolean activity_seen;			/**< Activity recorded in period */
-static cevent_t *outgoing_ev;			/**< Callout queue timer */
+static watchdog_t *outgoing_wd;			/**< Watchdog for outgoing activity */
 
 static void inet_set_is_connected(gboolean val);
 
-/**
- * Create a new ip_record structure.
- */
-static struct ip_record *
-ip_record_make(const host_addr_t addr)
-{
-	struct ip_record *ipr;
-
-	ipr = walloc(sizeof *ipr);
-
-	ipr->addr = addr;
-	ipr->timeout_ev = NULL;
-
-	return ipr;
-}
-
-/**
- * Free ip_record structure.
- */
-static void
-ip_record_free(struct ip_record *ipr)
-{
-	cq_cancel(callout_queue, &ipr->timeout_ev);
-	wfree(ipr, sizeof *ipr);
-}
-
-/**
- * Free ip_record structure and remove it from the `outgoing_udp' table.
- */
-static void
-ip_record_free_remove(struct ip_record *ipr)
-{
-	g_hash_table_remove(outgoing_udp, &ipr->addr);
-	ip_record_free(ipr);
-}
-
-/**
- * Touch ip_record when we send a new datagram to that IP.
- */
-static void
-ip_record_touch(struct ip_record *ipr)
-{
-	g_assert(ipr->timeout_ev != NULL);
-
-	cq_resched(callout_queue, ipr->timeout_ev, FW_UDP_WINDOW * 1000);
-}
-
-/**
- * Callout queue callback, invoked when it's time to destroy the record.
- */
-static void
-ip_record_destroy(cqueue_t *unused_cq, gpointer obj)
-{
-	struct ip_record *ipr = obj;
-
-	(void) unused_cq;
-	ipr->timeout_ev = NULL;			/* The event that fired */
-	ip_record_free_remove(ipr);
-}
-
 /**
  * Checks whether a host address is considered being "local".
  *
@@ -287,7 +250,7 @@ inet_firewalled(void)
 {
 	gnet_prop_set_boolean_val(PROP_IS_FIREWALLED, TRUE);
 	fw_time = tm_time();
-	cq_cancel(callout_queue, &incoming_ev);
+	wd_sleep(incoming_wd);
 	node_became_firewalled();
 }
 
@@ -298,27 +261,72 @@ void
 inet_udp_firewalled(void)
 {
 	gnet_prop_set_boolean_val(PROP_IS_UDP_FIREWALLED, TRUE);
-	cq_cancel(callout_queue, &incoming_udp_ev);
 	node_became_udp_firewalled();
 }
 
+/**
+ * Callout queue callback.
+ * Enter the UNSOLICITED_CHECK state.
+ */
+static void
+move_to_unsolicited_check(cqueue_t *unused_cq, gpointer unused_data)
+{
+	(void) unused_cq;
+	(void) unused_data;
+
+	unsolicited_udp_ev = NULL;		/* Event fired */
+
+	if (GNET_PROPERTY(fw_debug))
+		g_message("FW: will be now monitoring UDP for unsolicited messages");
+
+	outgoing_udp_state = UNSOLICITED_CHECK;
+}
+
+/**
+ * Enter the UNSOLICITED_PREPARE state.
+ */
+static void
+move_to_unsolicited_prepare(void)
+{
+	if (GNET_PROPERTY(fw_debug))
+		g_message("FW: set for unsolicited traffic detection in %d secs",
+			FW_UDP_WINDOW);
+
+	unsolicited_udp_ev = cq_insert(callout_queue, FW_UDP_WINDOW * 1000,
+		move_to_unsolicited_check, NULL);
+	outgoing_udp_state = UNSOLICITED_PREPARE;
+}
+
+/**
+ * Enter the UNSOLICITED_OFF state.
+ */
+static void
+move_to_unsolicited_off(void)
+{
+	if (GNET_PROPERTY(fw_debug))
+		g_message("FW: turning off unsolicited traffic detection");
+
+	outgoing_udp_state = UNSOLICITED_OFF;
+	cq_cancel(callout_queue, &unsolicited_udp_ev);	/* Paranoid */
+}
+
 /**
  * This is a callback invoked when no solicited UDP has been received
  * for some amount of time.  We conclude we're no longer able to get
  * solicited UDP traffic.
  */
-static void
-got_no_udp_solicited(cqueue_t *unused_cq, gpointer unused_obj)
+static gboolean
+got_no_udp_solicited(watchdog_t *unused_wd, gpointer unused_obj)
 {
-	(void) unused_cq;
+	(void) unused_wd;
 	(void) unused_obj;
 
-	if (GNET_PROPERTY(dbg))
+	if (GNET_PROPERTY(fw_debug))
 		g_message("FW: got no solicited UDP traffic for %d secs",
 			FW_SOLICITED_WINDOW);
 
-	solicited_udp_ev = NULL;
 	gnet_prop_set_boolean_val(PROP_RECV_SOLICITED_UDP, FALSE);
+	return FALSE;			/* Disarm watchdog */
 }
 
 /**
@@ -329,52 +337,66 @@ inet_udp_got_solicited(void)
 {
 	gnet_prop_set_boolean_val(PROP_RECV_SOLICITED_UDP, TRUE);
 
-	if (solicited_udp_ev == NULL) {
-		cq_insert(callout_queue,
-			FW_SOLICITED_WINDOW * 1000, got_no_udp_solicited, NULL);
-		if (GNET_PROPERTY(dbg))
-			g_message("FW: got solicited UDP traffic");
-	} else
-		cq_resched(callout_queue,
-			solicited_udp_ev, FW_SOLICITED_WINDOW * 1000);
+	if (wd_wakeup(solicited_udp_wd) && GNET_PROPERTY(fw_debug))
+		g_message("FW: got solicited UDP traffic");
+
+	wd_kick(solicited_udp_wd);
 }
 
 /**
  * This is a callback invoked when no incoming connection has been received
  * for some amount of time.  We conclude we became firewalled.
  */
-static void
-got_no_connection(cqueue_t *unused_cq, gpointer unused_obj)
+static gboolean
+got_no_connection(watchdog_t *unused_wd, gpointer unused_obj)
 {
-	(void) unused_cq;
+	(void) unused_wd;
 	(void) unused_obj;
 
-	if (GNET_PROPERTY(dbg))
+	if (GNET_PROPERTY(fw_debug))
 		g_message("FW: got no connection to port %u for %d secs",
 			socket_listen_port(), FW_INCOMING_WINDOW);
 
-	incoming_ev = NULL;
 	inet_firewalled();
+	return FALSE;			/* Disarm watchdog */
 }
 
 /**
  * This is a callback invoked when no unsolicited UDP datagrams have been
  * received for some amount of time.  We conclude we became firewalled.
  */
-static void
-got_no_udp_unsolicited(cqueue_t *unused_cq, gpointer unused_obj)
+static gboolean
+got_no_udp_unsolicited(watchdog_t *unused_wd, gpointer unused_obj)
 {
-	(void) unused_cq;
+	(void) unused_wd;
 	(void) unused_obj;
 
-	if (GNET_PROPERTY(dbg))
+	if (GNET_PROPERTY(fw_debug))
 		g_message("FW: got no unsolicited UDP datagram to port %u for %d secs",
 			socket_listen_port(), FW_INCOMING_WINDOW);
 
-	incoming_udp_ev = NULL;
-	inet_udp_firewalled();
-}
+	/*
+	 * If we are in the UNSOLICITED_OFF state, move to UNSOLICITED_PREPARE
+	 * for FW_UDP_WINDOW seconds.
+	 *
+	 * Otherwise, move to UNSOLICITED_OFF and state we are firewalled.
+	 */
 
+	STATIC_ASSERT(FW_UDP_WINDOW < FW_SOLICITED_WINDOW);
+
+	if (UNSOLICITED_OFF == outgoing_udp_state) {
+		move_to_unsolicited_prepare();
+	} else {
+		if (GNET_PROPERTY(fw_debug)) {
+			g_message("FW: no unsolicited UDP again for %d secs on port %u "
+				"=> firewalled", FW_SOLICITED_WINDOW, socket_listen_port());
+		}
+		move_to_unsolicited_off();
+		inet_udp_firewalled();
+	}
+
+	return TRUE;		/* Let watchdog fire again at next period */
+}
 
 /**
  * Called when we have determined we are definitely not TCP-firewalled.
@@ -385,7 +407,7 @@ inet_not_firewalled(void)
 	gnet_prop_set_boolean_val(PROP_IS_FIREWALLED, FALSE);
 	node_proxy_cancel_all();
 
-	if (GNET_PROPERTY(dbg))
+	if (GNET_PROPERTY(fw_debug))
 		g_message("FW: we're not TCP-firewalled for port %u",
 			socket_listen_port());
 }
@@ -398,7 +420,7 @@ inet_udp_not_firewalled(void)
 {
 	gnet_prop_set_boolean_val(PROP_IS_UDP_FIREWALLED, FALSE);
 
-	if (GNET_PROPERTY(dbg))
+	if (GNET_PROPERTY(fw_debug))
 		g_message("FW: we're not UDP-firewalled for port %u",
 			socket_listen_port());
 }
@@ -410,13 +432,13 @@ void
 inet_got_incoming(const host_addr_t addr)
 {
 	if (is_local_addr(addr)) {
-		if (GNET_PROPERTY(dbg))
+		if (GNET_PROPERTY(fw_debug))
 			g_message("FW: not counting local connection from %s",
 				host_addr_to_string(addr));
 		return;
 	}
 
-	if (GNET_PROPERTY(dbg) > 19)
+	if (GNET_PROPERTY(fw_debug) > 19)
 		g_message("FW: got TCP connection from %s", host_addr_to_string(addr));
 
 	/*
@@ -424,63 +446,53 @@ inet_got_incoming(const host_addr_t addr)
 	 * connected to the Internet.
 	 */
 
-	activity_seen = TRUE;				/* In case we have a timer set */
+	if (
+		wd_sleep(outgoing_wd) &&
+		!GNET_PROPERTY(is_inet_connected) &&
+		GNET_PROPERTY(fw_debug)
+	) {
+		g_message("FW: got incoming connection => connected");
+	}
 
 	if (!GNET_PROPERTY(is_inet_connected))
 		inet_set_is_connected(TRUE);
 
 	/*
-	 * If we already know we're not firewalled, we have already scheduled
-	 * a callback in the future.  We need to reschedule it, since we just
-	 * got an incoming connection.
+	 * We're not firewalled.
 	 */
 
-	if (!GNET_PROPERTY(is_firewalled)) {
-		g_assert(incoming_ev);
-		cq_resched(callout_queue, incoming_ev, FW_INCOMING_WINDOW * 1000);
-		return;
+	if (GNET_PROPERTY(is_firewalled)) {
+		wd_wakeup(incoming_wd);
+		inet_not_firewalled();
 	}
 
-	/*
-	 * Make sure we're not connecting locally.
-	 * If we're not, then we're not firewalled.
-	 */
-
-	inet_not_firewalled();
-
-	incoming_ev = cq_insert(callout_queue, FW_INCOMING_WINDOW * 1000,
-					got_no_connection, NULL);
+	wd_kick(incoming_wd);
 }
 
 /**
- * Called when we got an incoming unsolicited datagram from another
- * computer at `ip'.
+ * Called when we got an incoming unsolicited datagram from another computer.
  *
  * i.e. the datagram was sent directly to our listening socket port,
  * and not to a masqueraded port on the firewall opened because we
  * previously sent out an UDP datagram to a host and got its reply.
+ *
+ * There are some messages that we know cannot be received from UDP as
+ * a reply of some sort (i.e. sent back to us through a masquerated port).
+ * When we do get these messages, then this routine can be called explicitly.
  */
-static void
+void 
 inet_udp_got_unsolicited_incoming(void)
 {
-	/*
-	 * If we already know we're not firewalled, we have already scheduled
-	 * a callback in the future.  We need to reschedule it, since we just
-	 * got an incoming connection.
-	 */
-
-	if (!GNET_PROPERTY(is_udp_firewalled)) {
-		g_assert(incoming_udp_ev);
-		cq_resched(callout_queue, incoming_udp_ev, FW_INCOMING_WINDOW * 1000);
-		return;
+	if (outgoing_udp_state != UNSOLICITED_OFF) {
+		if (GNET_PROPERTY(fw_debug))
+			g_message("FW: got unsolicited UDP message => not firewalled");
+		move_to_unsolicited_off();
 	}
 
-	inet_udp_not_firewalled();
+	if (GNET_PROPERTY(is_udp_firewalled))
+		inet_udp_not_firewalled();
 
-	g_assert(incoming_udp_ev == NULL);
-
-	incoming_udp_ev = cq_insert(callout_queue, FW_INCOMING_WINDOW * 1000,
-							got_no_udp_unsolicited, NULL);
+	wd_kick(incoming_udp_wd);
 }
 
 /**
@@ -492,20 +504,30 @@ inet_udp_got_incoming(const host_addr_t addr)
 	if (is_local_addr(addr))
 		return;
 
-	activity_seen = TRUE;				/* In case we have a timer set */
+	if (
+		wd_sleep(outgoing_wd) &&
+		!GNET_PROPERTY(is_inet_connected) &&
+		GNET_PROPERTY(fw_debug)
+	) {
+		g_message("FW: got incoming UDP traffic => connected");
+	}
 
 	if (!GNET_PROPERTY(is_inet_connected))
 		inet_set_is_connected(TRUE);
 
+	inet_udp_got_solicited();
+
 	/*
-	 * Make sure we're not connecting locally.
-	 * If we're not, then we're not firewalled, unless we recently sent
-	 * some data to that IP address.
+	 * If checking for unsolicited traffic, then we have a match if we
+	 * get a message from an IP to whom we haven't sent anything recently.
 	 */
 
-	inet_udp_got_solicited();
-	if (NULL == g_hash_table_lookup(outgoing_udp, &addr))
+	if (
+		UNSOLICITED_CHECK == outgoing_udp_state &&
+		NULL == aging_lookup(outgoing_udp, &addr)
+	) {
 		inet_udp_got_unsolicited_incoming();
+	}
 }
 
 /**
@@ -515,16 +537,30 @@ inet_udp_got_incoming(const host_addr_t addr)
 void
 inet_udp_record_sent(const host_addr_t addr)
 {
-	struct ip_record *ipr;
+	if (UNSOLICITED_OFF == outgoing_udp_state)
+		return;		/* Not currently monitoring for unsolicited UDP */
 
-	ipr = g_hash_table_lookup(outgoing_udp, &addr);
-	if (ipr != NULL)
-		ip_record_touch(ipr);
-	else {
-		ipr = ip_record_make(addr);
-		g_hash_table_insert(outgoing_udp, &ipr->addr, ipr);
-		ipr->timeout_ev = cq_insert(callout_queue, FW_UDP_WINDOW * 1000,
-			ip_record_destroy, ipr);
+	if (!aging_lookup_revitalise(outgoing_udp, &addr)) {
+		aging_insert(outgoing_udp,
+			wcopy(&addr, sizeof addr), uint_to_pointer(1));
+	}
+}
+
+/**
+ * Called when the port changes.  We must redo UDP checks to see if we are
+ * able to receive truly unsolicited traffic on that port.
+ */
+void
+inet_udp_check_unsolicited(void)
+{
+	if (UNSOLICITED_OFF == outgoing_udp_state) {
+		if (GNET_PROPERTY(fw_debug))
+			g_message("FW: forcing checks for UDP unsolicited traffic");
+
+		move_to_unsolicited_prepare();
+	} else {
+		if (GNET_PROPERTY(fw_debug))
+			g_message("FW: already checking UDP unsolicited traffic");
 	}
 }
 
@@ -583,29 +619,23 @@ inet_set_is_connected(gboolean val)
 {
 	gnet_prop_set_boolean_val(PROP_IS_INET_CONNECTED, val);
 
-	if (GNET_PROPERTY(dbg))
+	if (GNET_PROPERTY(fw_debug))
 		g_message("FW: we're %sconnected to the Internet",
 			val ? "" : "no longer ");
 }
 
 /**
- * This callback is periodically called when there has been outgoing
- * connections attempted.
+ * This callback fires when there was no outgoing activity for the period
+ * after the watchdog was started.
  */
-static void
-check_outgoing_connection(cqueue_t *unused_cq, gpointer unused_obj)
+static gboolean
+no_outgoing_connection(watchdog_t *unused_wd, gpointer unused_obj)
 {
-	outgoing_ev = NULL;
-
-	(void) unused_cq;
+	(void) unused_wd;
 	(void) unused_obj;
 
-	if (activity_seen) {
-		activity_seen = FALSE;
-	} else {
-		/* Nothing over the period */
-		inet_set_is_connected(FALSE);
-	}
+	inet_set_is_connected(FALSE);		/* Nothing over the period */
+	return FALSE;						/* Put watchdog back to sleep */
 }
 
 /**
@@ -621,16 +651,7 @@ inet_connection_attempted(const host_addr_t addr)
 	if (is_local_addr(addr))
 		return;
 
-	/*
-	 * Start timer if not already done.
-	 */
-
-	if (!outgoing_ev) {
-		activity_seen = FALSE;
-		outgoing_ev = cq_insert(
-			callout_queue, OUTGOING_WINDOW * 1000,
-			check_outgoing_connection, NULL);
-	}
+	wd_wakeup(outgoing_wd);
 }
 
 /**
@@ -646,7 +667,13 @@ inet_connection_succeeded(const host_addr_t addr)
 	if (is_local_addr(addr))
 		return;
 
-	activity_seen = TRUE;
+	if (
+		wd_sleep(outgoing_wd) &&
+		!GNET_PROPERTY(is_inet_connected) &&
+		GNET_PROPERTY(fw_debug)
+	) {
+		g_message("FW: outgoing TCP connection succeeded => connected");
+	}
 
 	if (!GNET_PROPERTY(is_inet_connected))
 		inet_set_is_connected(TRUE);
@@ -658,12 +685,12 @@ inet_connection_succeeded(const host_addr_t addr)
 void
 inet_read_activity(void)
 {
-	activity_seen = TRUE;
-
 	/*
 	 * We're not sure activity was not with a local node, so don't
 	 * call inet_set_is_connected(TRUE) at this point!
 	 */
+
+	wd_kick(outgoing_wd);
 }
 
 /**
@@ -673,50 +700,43 @@ void
 inet_init(void)
 {
 	/*
-	 * If we persisted "is_firewalled" to FALSE, arm the no-connection timer.
+	 * Monitoring watchdogs for incoming connections.
+	 */
+
+	incoming_wd = wd_make("incoming TCP connections",
+		FW_INCOMING_WINDOW, got_no_connection, NULL, FALSE);
+
+	incoming_udp_wd = wd_make("unsolicited UDP message",
+		FW_INCOMING_WINDOW, got_no_udp_unsolicited, NULL, TRUE);
+
+	solicited_udp_wd = wd_make("solicited UDP reply",
+		FW_SOLICITED_WINDOW, got_no_udp_solicited, NULL, FALSE);
+
+	/*
+	 * Start the required watchdog depending on the persisted property values.
 	 */
 
 	if (!GNET_PROPERTY(is_firewalled))
-		incoming_ev = cq_insert(
-			callout_queue, FW_INCOMING_WINDOW * 1000,
-			got_no_connection, NULL);
-
-	/*
-	 * If we persisted "is_udp_firewalled" to FALSE, idem.
-	 */
-
-	if (!GNET_PROPERTY(is_udp_firewalled))
-		incoming_udp_ev = cq_insert(
-			callout_queue, FW_INCOMING_WINDOW * 1000,
-			got_no_udp_unsolicited, NULL);
-
-	/*
-	 * If we persisted "recv_solicited_udp" to TRUE, idem.
-	 */
+		wd_wakeup(incoming_wd);
 
 	if (GNET_PROPERTY(recv_solicited_udp))
-		solicited_udp_ev = cq_insert(
-			callout_queue, FW_SOLICITED_WINDOW * 1000,
-			got_no_udp_solicited, NULL);
+		wd_wakeup(solicited_udp_wd);
+
+	/*
+	 * Monitoring watchdog for outgoing connections.
+	 */
+
+	outgoing_wd = wd_make("outgoing TCP connections",
+		OUTGOING_WINDOW, no_outgoing_connection, NULL, FALSE);
 
 	/*
 	 * Initialize the table used to record outgoing UDP traffic.
 	 */
 
-	outgoing_udp = g_hash_table_new(host_addr_hash_func, host_addr_eq_func);
-}
+	outgoing_udp = aging_make(FW_UDP_WINDOW,
+		host_addr_hash_func, host_addr_eq_func, wfree_host_addr);
 
-/**
- * Hash table iteration callback to free the "ip_record" structure.
- */
-static void
-free_ip_record(gpointer key, gpointer value, gpointer unused_udata)
-{
-	struct ip_record *ipr = value;
-
-	(void) unused_udata;
-	g_assert(&ipr->addr == key);
-	ip_record_free(ipr);
+	outgoing_udp_state = UNSOLICITED_OFF;
 }
 
 /**
@@ -725,12 +745,12 @@ free_ip_record(gpointer key, gpointer value, gpointer unused_udata)
 void
 inet_close(void)
 {
-	g_hash_table_foreach(outgoing_udp, free_ip_record, NULL);
-	g_hash_table_destroy(outgoing_udp);
-
-	cq_cancel(callout_queue, &incoming_ev);
-	cq_cancel(callout_queue, &incoming_udp_ev);
-	cq_cancel(callout_queue, &solicited_udp_ev);
+	aging_destroy(&outgoing_udp);
+	cq_cancel(callout_queue, &unsolicited_udp_ev);
+	wd_free_null(&outgoing_wd);
+	wd_free_null(&incoming_wd);
+	wd_free_null(&incoming_udp_wd);
+	wd_free_null(&solicited_udp_wd);
 }
 
 /* vi: set ts=4 sw=4 cindent: */
