@@ -1,5 +1,5 @@
 /*
- * $Id: inet.h 9244 2005-08-17 13:44:05Z cbiere $
+ * $Id: inet.h 16687 2009-05-07 19:28:22Z rmanfredi $
  *
  * Copyright (c) 2001-2003, Raphael Manfredi
  *
@@ -55,6 +55,8 @@ void inet_got_incoming(const host_addr_t addr);
 void inet_read_activity(void);
 void inet_udp_got_incoming(const host_addr_t addr);
 void inet_udp_record_sent(const host_addr_t addr);
+void inet_udp_got_unsolicited_incoming(void);
+void inet_udp_check_unsolicited(void);
 
 void inet_connection_attempted(const host_addr_t addr);
 void inet_connection_succeeded(const host_addr_t addr);
