@@ -1,5 +1,5 @@
 /*
- * $Id: ipp_cache.c 16291 2009-03-29 09:11:14Z rmanfredi $
+ * $Id: ipp_cache.c 17156 2009-11-01 20:22:05Z rmanfredi $
  *
  * Copyright (c) 2007-2009, Raphael Manfredi
  * Copyright (c) 2006, Christian Biere
@@ -38,7 +38,7 @@
 
 #include "common.h"
 
-RCSID("$Id: ipp_cache.c 16291 2009-03-29 09:11:14Z rmanfredi $")
+RCSID("$Id: ipp_cache.c 17156 2009-11-01 20:22:05Z rmanfredi $")
 
 #include "lib/bit_array.h"
 #include "lib/file.h"
@@ -48,6 +48,7 @@ RCSID("$Id: ipp_cache.c 16291 2009-03-29 09:11:14Z rmanfredi $")
 
 #include "lib/endian.h"
 #include "lib/host_addr.h"
+#include "lib/timestamp.h"
 
 #include "hosts.h"
 #include "settings.h"
@@ -92,14 +93,14 @@ static guint
 ipp_cache_item_hash(gconstpointer key)
 {
 	const struct ipp_cache_item *item = key;
-	return host_hash(&item->host);
+	return gnet_host_hash(&item->host);
 }
 
 static int
 ipp_cache_item_eq(gconstpointer v1, gconstpointer v2)
 {
 	const struct ipp_cache_item *a = v1, *b = v2;
-	return host_eq(&a->host,& b->host);
+	return gnet_host_eq(&a->host,& b->host);
 }
 
 /**
@@ -355,7 +356,7 @@ ipp_cache_lookup_intern(const ipp_cache_t *ic,
 		gconstpointer key;
 
 		gnet_host_set(&item.host, addr, port);
-		if (hash_list_contains(ic->hosts, &item, &key)) {
+		if (hash_list_find(ic->hosts, &item, &key)) {
 			struct ipp_cache_item *item_ptr = deconstify_gpointer(key);
 			
 			if (!ipp_cache_item_expired(ic, item_ptr->seen, tm_time()))
@@ -409,7 +410,7 @@ ipp_cache_remove_intern(const ipp_cache_t *ic,
 		gconstpointer key;
 
 		gnet_host_set(&item.host, addr, port);
-		if (hash_list_contains(ic->hosts, &item, &key)) {
+		if (hash_list_find(ic->hosts, &item, &key)) {
 			struct ipp_cache_item *item_ptr = deconstify_gpointer(key);
 			
 			hash_list_remove(ic->hosts, item_ptr);
@@ -490,7 +491,7 @@ ipp_cache_parse(ipp_cache_t *ic, FILE *f)
 		tag_name = line;
 
 		tag = ipp_cache_string_to_tag(tag_name);
-		g_assert((int) tag >= 0 && tag < NUM_IPP_CACHE_TAGS);
+		g_assert(UNSIGNED(tag) < NUM_IPP_CACHE_TAGS);
 		if (IPP_CACHE_TAG_UNKNOWN != tag && !bit_array_flip(tag_used, tag)) {
 			g_warning(
 				"ipp_cache_load(\"%s\"): "
