@@ -1,5 +1,5 @@
 /*
- * $Id: matching.c 16106 2009-03-10 15:16:31Z rmanfredi $
+ * $Id: matching.c 17240 2010-02-13 12:26:50Z rmanfredi $
  *
  * Copyright (c) 2001-2003, Raphael Manfredi
  *
@@ -27,7 +27,7 @@
 
 #include "common.h"
 
-RCSID("$Id: matching.c 16106 2009-03-10 15:16:31Z rmanfredi $")
+RCSID("$Id: matching.c 17240 2010-02-13 12:26:50Z rmanfredi $")
 
 #include "matching.h"
 #include "qrp.h"				/* For qhvec_add() */
@@ -35,11 +35,11 @@ RCSID("$Id: matching.c 16106 2009-03-10 15:16:31Z rmanfredi $")
 
 #include "lib/atoms.h"
 #include "lib/ascii.h"
+#include "lib/halloc.h"
 #include "lib/pattern.h"
 #include "lib/utf8.h"
 #include "lib/wordvec.h"
 #include "lib/walloc.h"
-#include "lib/zalloc.h"
 
 #include "if/gnet_property_priv.h"
 
@@ -122,7 +122,7 @@ bin_initialize(struct st_bin *bin, int size)
 	bin->nvals = 0;
 	bin->nslots = size;
 
-	bin->vals = g_malloc(bin->nslots * sizeof bin->vals[0]);
+	bin->vals = halloc(bin->nslots * sizeof bin->vals[0]);
 	for (i = 0; i < bin->nslots; i++)
 		bin->vals[i] = NULL;
 }
@@ -147,7 +147,7 @@ bin_allocate(void)
 static void
 bin_destroy(struct st_bin *bin)
 {
-	G_FREE_NULL(bin->vals);
+	HFREE_NULL(bin->vals);
 	bin->nslots = 0;
 	bin->nvals = 0;
 }
@@ -160,7 +160,7 @@ bin_insert_item(struct st_bin *bin, struct st_entry *entry)
 {
 	if (bin->nvals == bin->nslots) {
 		bin->nslots *= 2;
-		bin->vals = g_realloc(bin->vals, bin->nslots * sizeof bin->vals[0]);
+		bin->vals = hrealloc(bin->vals, bin->nslots * sizeof bin->vals[0]);
 	}
 	bin->vals[bin->nvals++] = entry;
 }
@@ -173,7 +173,7 @@ bin_compact(struct st_bin *bin)
 {
 	g_assert(bin->vals != NULL);	/* Or it would not have been allocated */
 
-	bin->vals = g_realloc(bin->vals, bin->nvals * sizeof bin->vals[0]);
+	bin->vals = hrealloc(bin->vals, bin->nvals * sizeof bin->vals[0]);
 	bin->nslots = bin->nvals;
 }
 
@@ -273,7 +273,7 @@ st_create(search_table_t *table)
 {
 	int i;
 
-	table->bins = g_malloc(table->nbins * sizeof table->bins[0]);
+	table->bins = halloc(table->nbins * sizeof table->bins[0]);
 	for (i = 0; i < table->nbins; i++)
 		table->bins[i] = NULL;
 
@@ -297,7 +297,7 @@ st_destroy(search_table_t *table)
 				wfree(bin, sizeof *bin);
 			}
 		}
-		G_FREE_NULL(table->bins);
+		HFREE_NULL(table->bins);
 	}
 
 	if (table->all_entries.vals) {
@@ -440,6 +440,40 @@ entry_match(const char *text, size_t tlen,
 	return TRUE;
 }
 
+/**
+ * Fill non-NULL query hash vector for query routing.
+ *
+ * This needs to be called when st_search() is not called when processing
+ * a query, otherwise the qhery hash vector won't be properly initialized
+ * and the query would be improperly dropped by qrt_build_query_target(),
+ * hence never routed.
+ */
+void
+st_fill_qhv(const char *search_term, query_hashvec_t *qhv)
+{
+	char *search;
+	word_vec_t *wovec;
+	guint wocnt;
+	guint i;
+
+	if (NULL == qhv)
+		return;
+
+	search = UNICODE_CANONIZE(search_term);
+	wocnt = word_vec_make(search, &wovec);
+
+	for (i = 0; i < wocnt; i++) {
+		if (wovec[i].len >= QRP_MIN_WORD_LENGTH)
+			qhvec_add(qhv, wovec[i].word, QUERY_H_WORD);
+	}
+
+	if (search != search_term)
+		HFREE_NULL(search);
+
+	if (wocnt > 0)
+		word_vec_free(wovec, wocnt);
+}
+
 /**
  * Do an actual search.
  */
@@ -468,7 +502,7 @@ st_search(
 	guint random_offset;  /* Randomizer for search returns */
 
 	search = UNICODE_CANONIZE(search_term);
-	if (GNET_PROPERTY(search_debug) && 0 != strcmp(search, search_term)) {
+	if (GNET_PROPERTY(search_debug) > 4 && 0 != strcmp(search, search_term)) {
 		g_message(" Original search term: \"%s\"", search_term);
 		g_message("Canonical search term: \"%s\"", search);
 	}
@@ -478,28 +512,27 @@ st_search(
 	 * Find smallest bin
 	 */
 
-	if (len < 2)
-		goto finish;
+	if (len >= 2) {
+		for (i = 0; i < len - 1; i++) {
+			struct st_bin *bin;
+			if (is_ascii_space(search[i]) || is_ascii_space(search[i+1]))
+				continue;
+			key = st_key(table, search + i);
+			if ((bin = table->bins[key]) == NULL) {
+				best_bin = NULL;
+				break;
+			}
+			if (bin->nvals < best_bin_size) {
+				best_bin = bin;
+				best_bin_size = bin->nvals;
+			}
+		}
 
-	for (i = 0; i < len - 1; i++) {
-		struct st_bin *bin;
-		if (is_ascii_space(search[i]) || is_ascii_space(search[i+1]))
-			continue;
-		key = st_key(table, search + i);
-		if ((bin = table->bins[key]) == NULL) {
-			best_bin = NULL;
-			break;
-		}
-		if (bin->nvals < best_bin_size) {
-			best_bin = bin;
-			best_bin_size = bin->nvals;
-		}
+		if (GNET_PROPERTY(dbg) > 6)
+			printf("st_search(): str=\"%s\", len=%d, best_bin_size=%d\n",
+				search, len, best_bin_size);
 	}
 
-	if (GNET_PROPERTY(dbg) > 6)
-		printf("st_search(): str=\"%s\", len=%d, best_bin_size=%d\n",
-			search, len, best_bin_size);
-
 	/*
 	 * If the best_bin is NULL, we did not find a matching bin, and we're
 	 * sure we won't be able to find the search string.
@@ -633,7 +666,7 @@ st_search(
 
 finish:
 	if (search != search_term) {
-		G_FREE_NULL(search);
+		HFREE_NULL(search);
 	}
 }
 
