@@ -1,5 +1,5 @@
 /*
- * $Id: move.c 16106 2009-03-10 15:16:31Z rmanfredi $
+ * $Id: move.c 17194 2009-11-05 21:44:06Z rmanfredi $
  *
  * Copyright (c) 2002-2003, Raphael Manfredi
  *
@@ -35,7 +35,7 @@
 
 #include "common.h"
 
-RCSID("$Id: move.c 16106 2009-03-10 15:16:31Z rmanfredi $")
+RCSID("$Id: move.c 17194 2009-11-05 21:44:06Z rmanfredi $")
 
 #include "downloads.h"
 #include "fileinfo.h"
@@ -43,9 +43,13 @@ RCSID("$Id: move.c 16106 2009-03-10 15:16:31Z rmanfredi $")
 
 #include "lib/atoms.h"
 #include "lib/bg.h"
+#include "lib/compat_misc.h"
+#include "lib/fd.h"
 #include "lib/file.h"
-#include "lib/misc.h"
+#include "lib/halloc.h"
+#include "lib/stringify.h"
 #include "lib/tm.h"
+#include "lib/unsigned.h"
 #include "lib/walloc.h"
 
 #include "if/gnet_property.h"
@@ -53,18 +57,18 @@ RCSID("$Id: move.c 16106 2009-03-10 15:16:31Z rmanfredi $")
 
 #include "lib/override.h"	/* Must be the last header included */
 
-#define COPY_BLOCK_SHIFT	12			/**< Power of two of copy unit credit */
+#define COPY_BLOCK_FRAGMENT	4096		/**< Power of two of copy unit credit */
 #define COPY_BUF_SIZE		65536		/**< Size of the reading buffer */
 
 static struct bgtask *move_daemon;
 
-#define MOVED_MAGIC	0x00c0b100
+enum moved_magic_t { MOVED_MAGIC = 0x0ac0b103 };
 
 /**
  * Moving daemon context.
  */
 struct moved {
-	int magic;				/**< Magic number */
+	enum moved_magic_t magic;	/**< Magic number */
 	struct download *d;		/**< Download for which we're moving file */
 	char *buffer;			/**< Large buffer, where data is read */
 	char *target;			/**< Target file name, in case an error occurs */
@@ -151,15 +155,9 @@ d_free(gpointer ctx)
 
 	g_assert(md->magic == MOVED_MAGIC);
 
-	if (md->rd != -1) {
-		close(md->rd);
-		md->rd = -1;
-	}
-	if (md->wd != -1) {
-		close(md->wd);
-		md->wd = -1;
-	}
-	G_FREE_NULL(md->buffer);
+	fd_close(&md->rd, TRUE);
+	fd_close(&md->wd, TRUE);
+	HFREE_NULL(md->buffer);
 	md->magic = 0;
 	wfree(md, sizeof(*md));
 }
@@ -195,9 +193,9 @@ d_start(struct bgtask *h, gpointer ctx, gpointer item)
 	bg_task_signal(h, BG_SIG_TERM, d_sighandler);
 
 	md->d = we->d;
-	md->rd = file_open(download_pathname(d), O_RDONLY, 0);
 
-	if (md->rd == -1) {
+	md->rd = file_absolute_open(download_pathname(d), O_RDONLY, 0);
+	if (md->rd < 0) {
 		md->error = errno;
 		goto abort_read;
 	}
@@ -210,7 +208,8 @@ d_start(struct bgtask *h, gpointer ctx, gpointer item)
 	}
 
 	if (!S_ISREG(buf.st_mode)) {
-		g_warning("file \"%s\" is not a regular file", download_pathname(d));
+		g_warning("cannot move file \"%s\": not a regular file",
+			download_pathname(d));
 		goto abort_read;
 	}
 
@@ -225,8 +224,7 @@ d_start(struct bgtask *h, gpointer ctx, gpointer item)
 		goto abort_read;
 
 	md->wd = file_create(md->target, O_WRONLY | O_TRUNC, buf.st_mode);
-
-	if (md->wd == -1)
+	if (md->wd < 0)
 		goto abort_read;
 
 	md->start = tm_time();
@@ -234,18 +232,17 @@ d_start(struct bgtask *h, gpointer ctx, gpointer item)
 	md->copied = 0;
 	md->error = 0;
 
-	if (GNET_PROPERTY(dbg) > 1)
-		g_message("Moving \"%s\" to \"%s\"",
+	compat_fadvise_sequential(md->rd, 0, 0);
+
+	if (GNET_PROPERTY(move_debug) > 1)
+		g_message("MOVE starting moving \"%s\" to \"%s\"",
 				download_basename(d), md->target);
 
 	return;
 
 abort_read:
 	md->error = errno;
-	if (md->rd != -1) {
-		close(md->rd);
-		md->rd = -1;
-	}
+	fd_close(&md->rd, TRUE);
 	g_warning("can't copy \"%s\" to \"%s\"", download_pathname(d), we->dest);
 	return;
 }
@@ -265,20 +262,17 @@ d_end(struct bgtask *h, gpointer ctx, gpointer item)
 
 	bg_task_signal(h, BG_SIG_TERM, NULL);
 
-	if (md->rd == -1) {			/* Did not start properly */
+	if (md->rd < 0) {			/* Did not start properly */
 		g_assert(md->error);
 		goto finish;
 	}
 
-	close(md->rd);
-	md->rd = -1;
-
-	if (-1 == close(md->wd)) {
+	fd_close(&md->rd, TRUE);
+	if (fd_close(&md->wd, TRUE)) {
 		md->error = errno;
 		g_warning("error whilst closing copy target \"%s\": %s",
 			md->target, g_strerror(errno));
 	}
-	md->wd = -1;
 
 	/*
 	 * If copying went well, get rid of the source file.
@@ -330,8 +324,8 @@ error:
 	elapsed = delta_time(tm_time(), md->start);
 	elapsed = MAX(1, elapsed);		/* time warp? clock not monotic? */
 
-	if (GNET_PROPERTY(dbg) > 1)
-		printf("Moved file \"%s\" at %lu bytes/sec [error=%d]\n",
+	if (GNET_PROPERTY(move_debug) > 1)
+		g_message("MOVE moved file \"%s\" at %lu bytes/sec [error=%d]\n",
 			download_basename(md->d), (gulong) md->size / elapsed, md->error);
 
 finish:
@@ -341,7 +335,7 @@ finish:
 	} else
 		download_move_error(d);
 
-	G_FREE_NULL(md->target);
+	HFREE_NULL(md->target);
 }
 
 /**
@@ -353,12 +347,12 @@ d_step_copy(struct bgtask *h, gpointer u, int ticks)
 	struct moved *md = u;
 	ssize_t r;
 	size_t amount;
-	guint64 remain;
+	filesize_t remain;
 	int used;
 
 	g_assert(md->magic == MOVED_MAGIC);
 
-	if (md->rd == -1)			/* Could not open the file */
+	if (md->rd < 0)				/* Could not open the file */
 		return BGR_DONE;		/* Computation done */
 
 	if (md->size == 0)			/* Empty file */
@@ -366,16 +360,17 @@ d_step_copy(struct bgtask *h, gpointer u, int ticks)
 
 	g_assert(md->size > md->copied);
 	remain = md->size - md->copied;
+	remain = MIN(remain, COPY_BUF_SIZE);
 
 	/*
-	 * Each tick we have can buy us 2^COPY_BLOCK_SHIFT bytes.
+	 * Each tick we have can buy us COPY_BLOCK_FRAGMENT bytes.
 	 *
 	 * We read into a COPY_BUF_SIZE bytes buffer, and at most md->size
 	 * bytes total, to stop before the fileinfo trailer.
 	 */
 
-	amount = ticks << COPY_BLOCK_SHIFT;
-	remain = MIN(remain, COPY_BUF_SIZE);
+	amount = MAX(0, ticks);
+	amount = size_saturate_mult(amount, COPY_BLOCK_FRAGMENT);
 	amount = MIN(amount, remain);
 
 	g_assert(amount > 0);
@@ -397,8 +392,7 @@ d_step_copy(struct bgtask *h, gpointer u, int ticks)
 	 * Any partially read block counts as one block, hence the second term.
 	 */
 
-	used = (r >> COPY_BLOCK_SHIFT) +
-		((r & ((1 << COPY_BLOCK_SHIFT) - 1)) ? 1 : 0);
+	used = (r / COPY_BLOCK_FRAGMENT) + (r % COPY_BLOCK_FRAGMENT ? 1 : 0);
 
 	if (used != ticks)
 		bg_task_ticks_used(h, used);
@@ -448,7 +442,7 @@ move_init(void)
 	md->magic = MOVED_MAGIC;
 	md->rd = -1;
 	md->wd = -1;
-	md->buffer = g_malloc(COPY_BUF_SIZE);
+	md->buffer = halloc(COPY_BUF_SIZE);
 	md->target = NULL;
 
 	move_daemon = bg_daemon_create("file moving",
