@@ -1,5 +1,5 @@
 /*
- * $Id: move.h 12496 2006-11-26 09:22:49Z cbiere $
+ * $Id: move.h 16104 2009-03-10 14:58:09Z rmanfredi $
  *
  * Copyright (c) 2002-2003, Raphael Manfredi
  *
@@ -44,7 +44,7 @@
 
 void move_init(void);
 void move_close(void);
-void move_queue(struct download *d, const gchar *dest, const gchar *ext);
+void move_queue(struct download *d, const char *dest, const char *ext);
 
 #endif	/* _core_move_h_ */
 
