@@ -1,5 +1,5 @@
 /*
- * $Id: mq.c 16264 2009-03-27 21:24:11Z rmanfredi $
+ * $Id: mq.c 17088 2009-10-18 15:01:49Z rmanfredi $
  *
  * Copyright (c) 2002-2003, Raphael Manfredi
  *
@@ -35,7 +35,7 @@
 
 #include "common.h"
 
-RCSID("$Id: mq.c 16264 2009-03-27 21:24:11Z rmanfredi $")
+RCSID("$Id: mq.c 17088 2009-10-18 15:01:49Z rmanfredi $")
 
 #define MQ_INTERNAL
 #include "mq.h"
@@ -46,8 +46,9 @@ RCSID("$Id: mq.c 16264 2009-03-27 21:24:11Z rmanfredi $")
 
 #include "lib/cq.h"
 #include "lib/glib-missing.h"	/* For gm_snprintf() */
-#include "lib/misc.h"			/* For size_saturate_add() */
+#include "lib/halloc.h"
 #include "lib/pmsg.h"
+#include "lib/unsigned.h"		/* For size_saturate_add() */
 #include "lib/walloc.h"
 
 #include "if/gnet_property_priv.h"
@@ -136,6 +137,15 @@ mq_would_flow_control(const mqueue_t *q, size_t additional)
 	return size_saturate_add(q->size, additional) >= UNSIGNED(q->hiwat);
 }
 
+/**
+ * Are we already at or above the low watermark?
+ */
+gboolean
+mq_above_low_watermark(const mqueue_t *q)
+{
+	return q->size >= q->lowat;
+}
+
 /**
  * Compute readable queue information.
  */
@@ -186,7 +196,7 @@ mq_add_linkable(mqueue_t *q, GList *l)
 	g_assert(l->data != NULL);
 	
 	if (qown == NULL)
-		qown = g_hash_table_new(NULL, NULL);
+		qown = NOT_LEAKING(g_hash_table_new(NULL, NULL));
 
 	owner = g_hash_table_lookup(qown, l);
 	if (owner) {
@@ -238,7 +248,7 @@ mq_check_track(mqueue_t *q, int offset, const char *where, int line)
 	g_assert(q);
 
 	if (qown == NULL)
-		qown = g_hash_table_new(NULL, NULL);
+		qown = NOT_LEAKING(g_hash_table_new(NULL, NULL));
 
 	if (q->magic != MQ_MAGIC)
 		g_error("BUG: %s at %s:%d", mq_info(q), where, line);
@@ -571,7 +581,7 @@ mq_enter_flowc(mqueue_t *q)
 	node_tx_enter_flowc(q->node);	/* Signal flow control */
 
 	if (MQ_DEBUG_LVL(q) > 4)
-		printf("entering FLOWC for node %s (%d bytes queued)\n",
+		g_message("entering FLOWC for node %s (%d bytes queued)",
 			node_addr(q->node), q->size);
 }
 
@@ -584,7 +594,7 @@ mq_leave_flowc(mqueue_t *q)
 	g_assert(q->flags & MQ_FLOWC);
 
 	if (MQ_DEBUG_LVL(q) > 4)
-		printf("leaving %s for node %s (%d bytes queued)\n",
+		g_message("leaving %s for node %s (%d bytes queued)",
 			(q->flags & MQ_SWIFT) ? "SWIFT" : "FLOWC",
 			node_addr(q->node), q->size);
 
@@ -724,7 +734,7 @@ qlink_cmp(const void *a, const void *b)
 	const pmsg_t *m1 = (*l1)->data, *m2 = (*l2)->data;
 
 	if (pmsg_prio(m1) == pmsg_prio(m2))
-		return gmsg_cmp(pmsg_start(m1), pmsg_start(m2));
+		return gmsg_cmp(pmsg_start(m1), pmsg_start(m2), TRUE);
 	else
 		return pmsg_prio(m1) < pmsg_prio(m2) ? -1 : +1;
 }
@@ -740,7 +750,7 @@ qlink_create(mqueue_t *q)
 
 	g_assert(q->qlink == NULL);
 
-	q->qlink = g_malloc(q->count * sizeof q->qlink[0]);
+	q->qlink = halloc(q->count * sizeof q->qlink[0]);
 
 	/*
 	 * Prepare sorting of queued messages.
@@ -777,7 +787,7 @@ qlink_free(mqueue_t *q)
 {
 	g_assert(q->qlink);
 
-	G_FREE_NULL(q->qlink);
+	HFREE_NULL(q->qlink);
 	q->qlink_count = 0;
 }
 
@@ -808,7 +818,7 @@ qlink_insert_before(mqueue_t *q, int hint, GList *l)
 	 */
 
 	q->qlink_count++;
-	q->qlink = g_realloc(q->qlink, q->qlink_count * sizeof q->qlink[0]);
+	q->qlink = hrealloc(q->qlink, q->qlink_count * sizeof q->qlink[0]);
 
 	{
 		int i;
@@ -842,7 +852,7 @@ qlink_insert(mqueue_t *q, GList *l)
 	if (high < 0) {
 		g_assert(q->count == 1);		/* `l' is already part of the queue */
 		q->qlink_count++;
-		q->qlink = g_realloc(q->qlink, q->qlink_count * sizeof q->qlink[0]);
+		q->qlink = hrealloc(q->qlink, q->qlink_count * sizeof q->qlink[0]);
 		q->qlink[0] = l;
 		return;
 	}
@@ -862,7 +872,7 @@ qlink_insert(mqueue_t *q, GList *l)
 
 	if (qlink[high] != NULL && qlink_cmp(&l, &qlink[high]) >= 0) {
 		q->qlink_count++;
-		q->qlink = g_realloc(q->qlink, q->qlink_count * sizeof q->qlink[0]);
+		q->qlink = hrealloc(q->qlink, q->qlink_count * sizeof q->qlink[0]);
 		q->qlink[q->qlink_count - 1] = l;
 		return;
 	}
@@ -1048,32 +1058,26 @@ qlink_remove(mqueue_t *q, GList *l)
 }
 
 /**
- * Remove from the queue enough messages that are less prioritary than
- * the current one, so as to make sure we can enqueue it.
+ * Attempt to make room in the queue to be able to enqueue the new message
+ * whose header is specified.
  *
  * If `offset' is not null, it may be set with the offset within qlink where
- * the message immediately more prioritary than `mb' can be found.  It is
- * up to the caller to initialize it with -1 and check whether it has been
- * set.
+ * the message immediately more prioritary than the new one can be found.
+ * It is up to the caller to initialize it with -1 and check whether it has
+ * been set.
+ *
+ * @param q			the queue
+ * @param header	pointer to the header of the new message
+ * @param msglen	if non-zero, header points to a full PDU of msglen bytes
+ * @param prio		the priority of the new message we want to enqueue
+ * @param needed	the amount of room we want to make in the queue
+ * @param offset	see above description
  *
  * @returns TRUE if we were able to make enough room.
  */
 static gboolean
-make_room(mqueue_t *q, pmsg_t *mb, int needed, int *offset)
-{
-	char *header = pmsg_start(mb);
-	guint prio = pmsg_prio(mb);
-
-	return make_room_header(q, header, prio, needed, offset);
-}
-
-/**
- * Same as make_room(), but we are not given a "pmsg_t" as a comparison
- * point but a Gnutella header and a message priority explicitly.
- */
-static gboolean
-make_room_header(
-	mqueue_t *q, char *header, guint prio, int needed, int *offset)
+make_room_internal(mqueue_t *q,
+	char *header, size_t msglen, guint prio, int needed, int *offset)
 {
 	int n;
 	int dropped = 0;				/* Amount of messages dropped */
@@ -1083,7 +1087,7 @@ make_room_header(
 	mq_check(q, 0);
 
 	if (MQ_DEBUG_LVL(q) > 5)
-		printf("%s try to make room for %d bytes in queue 0x%lx (node %s)\n",
+		g_message("%s try to make room for %d bytes in queue 0x%lx (node %s)",
 			(q->flags & MQ_SWIFT) ? "SWIFT" : "FLOWC",
 			needed, (gulong) q, node_addr(q->node));
 
@@ -1175,7 +1179,7 @@ restart:
 		 * (it's necessarily >= 0 if we're in the loop)
 		 */
 
-		if (gmsg_cmp(cmb_start, header) >= 0) {
+		if (gmsg_cmp(cmb_start, header, msglen != 0) >= 0) {
 			if (offset != NULL)
 				*offset = n;
 			break;
@@ -1197,11 +1201,13 @@ restart:
 		 * Drop message.
 		 */
 
-		if (MQ_DEBUG_LVL(q) > 4)
-			gmsg_log_dropped(pmsg_start(cmb),
-				"to %s node %s, in favor of %s",
+		if (MQ_DEBUG_LVL(q) > 4) {
+			gmsg_log_dropped_pmsg(cmb, "to %s %s node %s, in favor of %s",
 				(q->flags & MQ_SWIFT) ? "SWIFT" : "FLOWC",
-				node_addr(q->node), gmsg_infostr(header));
+				NODE_IS_UDP(q->node) ? "UDP" : "TCP",
+				node_addr(q->node), msglen ?
+					gmsg_infostr_full(header, msglen) : gmsg_infostr(header));
+		}
 
 		gnet_stats_count_flowc(pmsg_start(cmb));
 		cmb_size = pmsg_size(cmb);
@@ -1221,7 +1227,7 @@ restart:
 		node_add_txdrop(q->node, dropped);	/* Dropped during TX */
 
 	if (MQ_DEBUG_LVL(q) > 5)
-		printf("%s end purge: %d bytes (count=%d) for node %s, need=%d\n",
+		g_message("%s end purge: %d bytes (count=%d) for node %s, need=%d",
 			(q->flags & MQ_SWIFT) ? "SWIFT" : "FLOWC",
 			q->size, q->count, node_addr(q->node), needed);
 
@@ -1252,6 +1258,38 @@ restart:
 	return needed <= 0;		/* Can be 0 if we broke out loop above */
 }
 
+/**
+ * Remove from the queue enough messages that are less prioritary than
+ * the current one, so as to make sure we can enqueue it.
+ *
+ * If `offset' is not null, it may be set with the offset within qlink where
+ * the message immediately more prioritary than `mb' can be found.  It is
+ * up to the caller to initialize it with -1 and check whether it has been
+ * set.
+ *
+ * @returns TRUE if we were able to make enough room.
+ */
+static gboolean
+make_room(mqueue_t *q, pmsg_t *mb, int needed, int *offset)
+{
+	char *header = pmsg_start(mb);
+	guint prio = pmsg_prio(mb);
+	size_t msglen = pmsg_written_size(mb);
+
+	return make_room_internal(q, header, msglen, prio, needed, offset);
+}
+
+/**
+ * Same as make_room(), but we are not given a "pmsg_t" as a comparison
+ * point but a Gnutella header and a message priority explicitly.
+ */
+static gboolean
+make_room_header(
+	mqueue_t *q, char *header, guint prio, int needed, int *offset)
+{
+	return make_room_internal(q, header, 0, prio, needed, offset);
+}
+
 /**
  * Put message in this queue.
  */
@@ -1281,8 +1319,9 @@ mq_puthere(mqueue_t *q, pmsg_t *mb, int msize)
 	) {
 		g_assert(pmsg_is_unread(mb));			/* Not partially written */
 		if (MQ_DEBUG_LVL(q) > 4)
-			gmsg_log_dropped(pmsg_start(mb),
-				"to FLOWC node %s, %d bytes queued",
+			gmsg_log_dropped_pmsg(mb, "to %s %s node %s, %d bytes queued",
+				(q->flags & MQ_SWIFT) ? "SWIFT" : "FLOWC",
+				NODE_IS_UDP(q->node) ? "UDP" : "TCP",
 				node_addr(q->node), q->size);
 
 		gnet_stats_count_flowc(pmsg_start(mb));
@@ -1319,15 +1358,19 @@ mq_puthere(mqueue_t *q, pmsg_t *mb, int msize)
 
 		if (has_normal_prio) {
 			if (MQ_DEBUG_LVL(q) > 4)
-				gmsg_log_dropped(pmsg_start(mb),
-					"to FLOWC node %s, %d bytes queued [FULL]",
+				gmsg_log_dropped_pmsg(mb,
+					"to %s %s node %s, %d bytes queued [FULL]",
+					(q->flags & MQ_SWIFT) ? "SWIFT" : "FLOWC",
+					NODE_IS_UDP(q->node) ? "UDP" : "TCP",
 					node_addr(q->node), q->size);
 
 			node_inc_txdrop(q->node);		/* Dropped during TX */
 		} else {
 			if (MQ_DEBUG_LVL(q) > 4)
-				gmsg_log_dropped(pmsg_start(mb),
-					"to FLOWC node %s, %d bytes queued [KILLING]",
+				gmsg_log_dropped_pmsg(mb,
+					"to %s %s node %s, %d bytes queued [KILLING]",
+					(q->flags & MQ_SWIFT) ? "SWIFT" : "FLOWC",
+					NODE_IS_UDP(q->node) ? "UDP" : "TCP",
 					node_addr(q->node), q->size);
 
 			/*
