@@ -1,5 +1,5 @@
 /*
- * $Id: mq_udp.c 16204 2009-03-18 20:03:57Z rmanfredi $
+ * $Id: mq_udp.c 16738 2009-05-19 20:19:26Z rmanfredi $
  *
  * Copyright (c) 2002-2003, Raphael Manfredi
  *
@@ -35,7 +35,7 @@
 
 #include "common.h"
 
-RCSID("$Id: mq_udp.c 16204 2009-03-18 20:03:57Z rmanfredi $")
+RCSID("$Id: mq_udp.c 16738 2009-05-19 20:19:26Z rmanfredi $")
 
 #define MQ_INTERNAL
 #include "mq.h"
@@ -311,7 +311,8 @@ again:
 
 	if (q->flags & (MQ_CLEAR | MQ_DISCARD)) {
 		pmsg_mark_sent(mb);	/* Let them think it was sent */
-		goto cleanup;		/* Drop message */
+		pmsg_free(mb);		/* Drop message */
+		return;
 	}
 
 	mq_check(q, 0);
@@ -391,7 +392,8 @@ again:
 			}
 
 			if (GNET_PROPERTY(mq_udp_debug) > 5)
-				g_message("MQ UDP sent %s", gmsg_infostr_full(pmsg_start(mb)));
+				g_message("MQ UDP sent %s",
+					gmsg_infostr_full(pmsg_start(mb), pmsg_written_size(mb)));
 
 			goto cleanup;
 		}
@@ -403,8 +405,10 @@ again:
 		 */
 
 		if (written > 0) {
-			g_warning("partial UDP write (%d bytes) to %s for %d-byte datagram",
-				(int) written, gnet_host_to_string(to), (int) size);
+			g_warning(
+				"partial UDP write (%lu bytes) to %s for %lu-byte datagram",
+				(unsigned long) written, gnet_host_to_string(to),
+				(unsigned long) size);
 			goto cleanup;
 		}
 
@@ -412,7 +416,8 @@ again:
 	}
 
 	if (GNET_PROPERTY(mq_udp_debug) > 5)
-		g_message("MQ UDP queued %s", gmsg_infostr_full(pmsg_start(mb)));
+		g_message("MQ UDP queued %s",
+			gmsg_infostr_full(pmsg_start(mb), pmsg_written_size(mb)));
 
 	/*
 	 * Attach the destination information as metadata to the message, unless
