@@ -1,5 +1,5 @@
 /*
- * $Id: namesize.h 12496 2006-11-26 09:22:49Z cbiere $
+ * $Id: namesize.h 16106 2009-03-10 15:16:31Z rmanfredi $
  *
  * Copyright (c) 2002-2003, Raphael Manfredi
  *
@@ -43,7 +43,7 @@
  */
 
 typedef struct namesize {
-	const gchar *name;		/**< Filename (atom) */
+	const char *name;		/**< Filename (atom) */
 	filesize_t size;	/**< File size */
 } namesize_t;
 
@@ -52,9 +52,9 @@ typedef struct namesize {
  */
 
 guint namesize_hash(gconstpointer key);
-gint namesize_eq(gconstpointer a, gconstpointer b);
+int namesize_eq(gconstpointer a, gconstpointer b);
 
-namesize_t *namesize_make(const gchar *name, filesize_t size);
+namesize_t *namesize_make(const char *name, filesize_t size);
 void namesize_free(namesize_t *ns);
 
 #endif	/* _core_namesize_h_ */
