@@ -1,5 +1,5 @@
 /*
- * $Id: nodes.h 16179 2009-03-16 18:16:53Z rmanfredi $
+ * $Id: nodes.h 17392 2010-03-18 17:02:16Z rmanfredi $
  *
  * Copyright (c) 2001-2003, Raphael Manfredi
  *
@@ -41,8 +41,11 @@
 #include "if/core/guid.h"
 #include "if/core/hcache.h"
 #include "if/core/nodes.h"
+#include "if/dht/dht.h"
 
+#include "lib/cq.h"
 #include "lib/header.h"
+#include "lib/sequence.h"
 
 struct guid;
 
@@ -117,7 +120,7 @@ typedef struct gnutella_node {
 	guint8 qrp_minor;			/**< Query routing protocol minor number */
 	guint8 uqrp_major;			/**< UP Query routing protocol major number */
 	guint8 uqrp_minor;			/**< UP Query routing protocol minor number */
-	const char *vendor;		/**< Vendor information (always UTF-8) */
+	const char *vendor;			/**< Vendor information (always UTF-8) */
 	vendor_code_t vcode;		/**< Vendor code (vcode.u32 == 0 if unknown) */
 	gpointer io_opaque;			/**< Opaque I/O callback information */
 
@@ -128,7 +131,7 @@ typedef struct gnutella_node {
 	guint16 size; /**< How many bytes we need to read for the current message */
 	guint16 header_flags;		/**< Header flags (new message architecture) */
 
-	char *data;				/**< data of the current message */
+	char *data;					/**< data of the current message */
 	guint32 pos;				/**< write position in data */
 
 	gnet_node_state_t status;	/**< See possible values below */
@@ -194,6 +197,7 @@ typedef struct gnutella_node {
 	time_delta_t alive_period;	/**< Period for sending alive pings (secs) */
 
 	wrap_buf_t hello;			/**< Spill buffer for GNUTELLA HELLO */
+	cevent_t *dht_nope_ev;		/**< Periodic event for NOPE DHT publishing */
 
 	/*
 	 * Round-trip time (RTT) measurements operated via Time Sync
@@ -274,33 +278,33 @@ typedef struct gnutella_node {
  */
 
 enum {
-	NODE_F_HDSK_PING	= 1 << 0,	/**< Expecting handshake ping */
-	NODE_F_STALE_QRP	= 1 << 1,	/**< Is sending a stale QRP patch */
-	NODE_F_INCOMING		= 1 << 2,	/**< Incoming (permanent) connection */
-	NODE_F_ESTABLISHED	= 1 << 3,	/**< Gnutella connection established */
-	NODE_F_VALID		= 1 << 4,	/**< Handshaked with a Gnutella node */
-	NODE_F_ALIEN_IP		= 1 << 5,	/**< Pong-IP did not match TCP/IP addr */
-	NODE_F_WRITABLE		= 1 << 6,	/**< Node is writable */
-	NODE_F_READABLE		= 1 << 7,	/**< Node is readable, process queries */
-	NODE_F_BYE_SENT		= 1 << 8,	/**< Bye message was queued */
-	NODE_F_NOREAD		= 1 << 9,	/**< Prevent further reading from node */
-	NODE_F_EOF_WAIT		= 1 << 10,	/**< At final shutdown, waiting EOF */
-	NODE_F_CLOSING		= 1 << 11,	/**< Initiated bye or shutdown */
-	NODE_F_ULTRA		= 1 << 12,	/**< Is one of our ultra nodes */
-	NODE_F_LEAF			= 1 << 13,	/**< Is one of our leaves */
-	NODE_F_CRAWLER		= 1 << 14,	/**< Is a Gnutella Crawler */
-	NODE_F_FAKE_NAME	= 1 << 15,	/**< Was unable to validate GTKG name */
-	NODE_F_PROXY		= 1 << 16,	/**< Sent a push-proxy request to it */
-	NODE_F_PROXIED		= 1 << 17,	/**< We are push-proxy for that node */
-	NODE_F_QRP_SENT		= 1 << 18,	/**< Undergone 1 complete QRP sending */
-	NODE_F_TSYNC_WAIT	= 1 << 19,	/**< Time sync pending via TCP */
-	NODE_F_TSYNC_TCP	= 1 << 20,	/**< No replies via UDP, use TCP */
-	NODE_F_GTKG			= 1 << 21,	/**< Node is another gtk-gnutella */
-	NODE_F_FORCE		= 1 << 22,	/**< Connection is forced */
-	NODE_F_NO_OOB_PROXY	= 1 << 23,	/**< Do not OOB proxy the leaf */
-	NODE_F_TLS			= 1 << 24,	/**< TLS-tunneled */
+	NODE_F_VMSG_SUPPORT	= 1 << 26,	/**< Indicated which VMSGs are supported */
 	NODE_F_CAN_TLS		= 1 << 25,	/**< Indicated support for TLS */
-	NODE_F_VMSG_SUPPORT	= 1 << 26	/**< Indicated which VMSGs are supported */
+	NODE_F_TLS			= 1 << 24,	/**< TLS-tunneled */
+	NODE_F_NO_OOB_PROXY	= 1 << 23,	/**< Do not OOB proxy the leaf */
+	NODE_F_FORCE		= 1 << 22,	/**< Connection is forced */
+	NODE_F_GTKG			= 1 << 21,	/**< Node is another gtk-gnutella */
+	NODE_F_TSYNC_TCP	= 1 << 20,	/**< No replies via UDP, use TCP */
+	NODE_F_TSYNC_WAIT	= 1 << 19,	/**< Time sync pending via TCP */
+	NODE_F_QRP_SENT		= 1 << 18,	/**< Undergone 1 complete QRP sending */
+	NODE_F_PROXIED		= 1 << 17,	/**< We are push-proxy for that node */
+	NODE_F_PROXY		= 1 << 16,	/**< Sent a push-proxy request to it */
+	NODE_F_FAKE_NAME	= 1 << 15,	/**< Was unable to validate GTKG name */
+	NODE_F_CRAWLER		= 1 << 14,	/**< Is a Gnutella Crawler */
+	NODE_F_LEAF			= 1 << 13,	/**< Is one of our leaves */
+	NODE_F_ULTRA		= 1 << 12,	/**< Is one of our ultra nodes */
+	NODE_F_CLOSING		= 1 << 11,	/**< Initiated bye or shutdown */
+	NODE_F_EOF_WAIT		= 1 << 10,	/**< At final shutdown, waiting EOF */
+	NODE_F_NOREAD		= 1 << 9,	/**< Prevent further reading from node */
+	NODE_F_BYE_SENT		= 1 << 8,	/**< Bye message was queued */
+	NODE_F_READABLE		= 1 << 7,	/**< Node is readable, process queries */
+	NODE_F_WRITABLE		= 1 << 6,	/**< Node is writable */
+	NODE_F_ALIEN_IP		= 1 << 5,	/**< Pong-IP did not match TCP/IP addr */
+	NODE_F_VALID		= 1 << 4,	/**< Handshaked with a Gnutella node */
+	NODE_F_ESTABLISHED	= 1 << 3,	/**< Gnutella connection established */
+	NODE_F_INCOMING		= 1 << 2,	/**< Incoming (permanent) connection */
+	NODE_F_STALE_QRP	= 1 << 1,	/**< Is sending a stale QRP patch */
+	NODE_F_HDSK_PING	= 1 << 0	/**< Expecting handshake ping */
 };
 
 /*
@@ -308,30 +312,31 @@ enum {
  */
 
 enum {
-	NODE_A_BYE_PACKET	= 1 << 0,	/**< Supports Bye-Packet */
-	NODE_A_PONG_CACHING	= 1 << 1,	/**< Supports Pong-Caching */
-	NODE_A_PONG_ALIEN	= 1 << 2,	/**< Alien Pong-Caching scheme */
-	NODE_A_QHD_NO_VTAG	= 1 << 3,	/**< Servent has no vendor tag in QHD */
-	NODE_A_RX_INFLATE	= 1 << 4,	/**< Reading compressed data */
-	NODE_A_TX_DEFLATE	= 1 << 5,	/**< Sending compressed data */
-	NODE_A_ULTRA		= 1 << 6,	/**< Node wants to be an Ultrapeer */
-	NODE_A_NO_ULTRA		= 1 << 7,	/**< Node is NOT ultra capable */
-	NODE_A_UP_QRP		= 1 << 8,	/**< Supports intra-UP QRP */
-	NODE_A_GUIDANCE		= 1 << 9,	/**< Can leaf-guide dyn queries */
-	NODE_A_TIME_SYNC	= 1 << 10,	/**< Supports time sync */
-	NODE_A_CRAWLABLE	= 1 << 11,	/**< Node can be UDP-crawled */
-	NODE_A_DYN_QUERY	= 1 << 12,	/**< Node can perform dynamic queries */
-	NODE_A_CAN_SFLAG	= 1 << 13,	/**< Node supports flags in headers */
- 
-	NODE_A_UNUSED_1		= 1 << 14,	/**< UNUSED */
-	NODE_A_UNUSED_2		= 1 << 15,	/**< UNUSED */
-	NODE_A_CAN_HSEP		= 1 << 16,	/**< Node supports HSEP */
-	NODE_A_CAN_QRP		= 1 << 17,	/**< Node supports query routing */
-	NODE_A_CAN_VENDOR	= 1 << 18,	/**< Node supports vendor messages */
-	NODE_A_CAN_ULTRA	= 1 << 19,	/**< Node is ultra capable */
-	NODE_A_CAN_INFLATE	= 1 << 20,	/**< Node capable of inflating */
+	NODE_A_CAN_DHT		= 1 << 24,	/**< Indicated support for DHT */
+	NODE_A_FIREWALLED	= 1 << 23,	/**< Node determined to be firewalled */
+	NODE_A_CAN_SVN_NOTIFY =	1 << 22,	/**< Supports SVN release notifies */
 	NODE_A_CAN_HEAD		= 1 << 21,	/**< Supports HEAD ping (vendor message) */
-	NODE_A_CAN_SVN_NOTIFY =	1 << 22	/**< Supports SVN release notifications */
+	NODE_A_CAN_INFLATE	= 1 << 20,	/**< Node capable of inflating */
+	NODE_A_CAN_ULTRA	= 1 << 19,	/**< Node is ultra capable */
+	NODE_A_CAN_VENDOR	= 1 << 18,	/**< Node supports vendor messages */
+	NODE_A_CAN_QRP		= 1 << 17,	/**< Node supports query routing */
+	NODE_A_CAN_HSEP		= 1 << 16,	/**< Node supports HSEP */
+	NODE_A_UDP			= 1 << 15,	/**< Node uses UDP for traffic */
+	NODE_A_UNUSED_1		= 1 << 14,	/**< UNUSED */
+	NODE_A_CAN_SFLAG	= 1 << 13,	/**< Node supports flags in headers */
+	NODE_A_DYN_QUERY	= 1 << 12,	/**< Node can perform dynamic queries */
+	NODE_A_CRAWLABLE	= 1 << 11,	/**< Node can be UDP-crawled */
+	NODE_A_TIME_SYNC	= 1 << 10,	/**< Supports time sync */
+	NODE_A_GUIDANCE		= 1 << 9,	/**< Can leaf-guide dyn queries */
+	NODE_A_UP_QRP		= 1 << 8,	/**< Supports intra-UP QRP */
+	NODE_A_NO_ULTRA		= 1 << 7,	/**< Node is NOT ultra capable */
+	NODE_A_ULTRA		= 1 << 6,	/**< Node wants to be an Ultrapeer */
+	NODE_A_TX_DEFLATE	= 1 << 5,	/**< Sending compressed data */
+	NODE_A_RX_INFLATE	= 1 << 4,	/**< Reading compressed data */
+	NODE_A_QHD_NO_VTAG	= 1 << 3,	/**< Servent has no vendor tag in QHD */
+	NODE_A_PONG_ALIEN	= 1 << 2,	/**< Alien Pong-Caching scheme */
+	NODE_A_PONG_CACHING	= 1 << 1,	/**< Supports Pong-Caching */
+	NODE_A_BYE_PACKET	= 1 << 0	/**< Supports Bye-Packet */
 };
  
 /*
@@ -423,6 +428,7 @@ enum {
 #define NODE_UP_QRP(n)			((n)->attrs & NODE_A_UP_QRP)
 #define NODE_LEAF_GUIDE(n)		((n)->attrs & NODE_A_GUIDANCE)
 #define NODE_CAN_INFLATE(n)		((n)->attrs & NODE_A_CAN_INFLATE)
+#define NODE_USES_UDP(n)		((n)->attrs & NODE_A_UDP)
 
 /*
  * Peer inspection macros
@@ -432,6 +438,7 @@ enum {
 #define NODE_IS_NORMAL(n)		((n)->peermode == NODE_P_NORMAL)
 #define NODE_IS_ULTRA(n)		((n)->peermode == NODE_P_ULTRA)
 #define NODE_IS_UDP(n)			((n)->peermode == NODE_P_UDP)
+#define NODE_IS_DHT(n)			((n)->peermode == NODE_P_DHT)
 
 /*
  * Macros.
@@ -513,7 +520,7 @@ void node_slow_timer(time_t now);
 void node_timer(time_t now);
 guint connected_nodes(void);
 guint node_count(void);
-guint node_keep_missing(void);
+int node_keep_missing(void);
 guint node_missing(void);
 guint node_leaves_missing(void);
 guint node_outdegree(void);
@@ -581,9 +588,11 @@ void node_proxying_remove(gnutella_node_t *n);
 gboolean node_proxying_add(gnutella_node_t *n, const struct guid *guid);
 void node_proxy_add(gnutella_node_t *n, const host_addr_t addr, guint16 port);
 void node_proxy_cancel_all(void);
+size_t node_http_fw_node_info_add(char *buf, size_t size, gboolean);
 size_t node_http_proxies_add(char *buf, size_t size,
 			gpointer arg, guint32 flags);
-const GSList *node_push_proxies(void);
+sequence_t *node_push_proxies(void);
+const gnet_host_t *node_oldest_push_proxy(void);
 const GSList *node_all_nodes(void);
 
 gnutella_node_t *node_by_id(const node_id_t node_id);
@@ -595,11 +604,13 @@ void node_became_udp_firewalled(void);
 void node_set_socket_rx_size(int rx_size);
 
 mqueue_t *node_udp_get_outq(enum net_type net);
-gboolean node_udp_is_flow_controlled(void);
-gboolean node_udp_would_flow_control(size_t additional);
+gboolean node_dht_is_flow_controlled(void);
+gboolean node_dht_would_flow_control(size_t additional);
+gboolean node_dht_above_low_watermark(void);
 void node_udp_disable(void);
-void node_udp_process(struct gnutella_socket *s);
+void node_udp_process(gnutella_node_t *n, struct gnutella_socket *s);
 gnutella_node_t *node_udp_get_addr_port(const host_addr_t addr, guint16 port);
+gnutella_node_t *node_dht_get_addr_port(const host_addr_t addr, guint16 port);
 
 void node_can_tsync(gnutella_node_t *n);
 void node_crawl(gnutella_node_t *n, int ucnt, int lcnt, guint8 features);
@@ -608,7 +619,8 @@ void node_update_udp_socket(void);
 void node_check_remote_ip_header(const host_addr_t peer, header_t *head);
 
 guint feed_host_cache_from_headers(header_t *headers,
-	host_type_t sender, gboolean gnet, const host_addr_t peer);
+	host_type_t sender, gboolean gnet, const host_addr_t peer,
+	const char *vendor);
 
 gnutella_node_t *node_browse_prepare(
 	gnet_host_t *host, const char *vendor, gnutella_header_t *header,
@@ -616,6 +628,8 @@ gnutella_node_t *node_browse_prepare(
 void node_browse_cleanup(gnutella_node_t *n);
 void node_kill_hostiles(void);
 void node_supports_tls(struct gnutella_node *);
+void node_supports_dht(struct gnutella_node *, dht_mode_t);
+void node_is_firewalled(gnutella_node_t *n);
 
 node_id_t node_id_get_self(void);
 gboolean node_id_self(const node_id_t node_id);
@@ -623,7 +637,7 @@ gboolean node_id_self(const node_id_t node_id);
 static inline void
 node_check(const struct gnutella_node * const n)
 {
-	g_assert(n);
+	g_assert(n != NULL);
 	g_assert(NODE_MAGIC == n->magic);
 }
 
@@ -633,6 +647,12 @@ node_guid(const struct gnutella_node * const n)
 	return n->guid;
 }
 
+static inline node_id_t
+node_get_id(const struct gnutella_node * const n)
+{
+	return n->id;
+}
+
 gboolean node_set_guid(struct gnutella_node *n, const struct guid *guid);
 struct gnutella_node *node_by_guid(const struct guid *guid);
 
