@@ -1,5 +1,5 @@
 /*
- * $Id: ntp.c 13777 2007-05-29 00:23:11Z cbiere $
+ * $Id: ntp.c 16223 2009-03-20 21:48:03Z rmanfredi $
  *
  * Copyright (c) 2004, Raphael Manfredi
  *
@@ -35,19 +35,18 @@
 
 #include "common.h"
 
-RCSID("$Id: ntp.c 13777 2007-05-29 00:23:11Z cbiere $")
+RCSID("$Id: ntp.c 16223 2009-03-20 21:48:03Z rmanfredi $")
 
 #include "ntp.h"
 #include "settings.h"
 #include "sockets.h"
 
-#include "if/core/hosts.h"
-
 #include "if/gnet_property.h"
 #include "if/gnet_property_priv.h"
 
 #include "lib/cq.h"
 #include "lib/endian.h"
+#include "lib/gnet_host.h"
 #include "lib/misc.h"
 #include "lib/tm.h"
 #include "lib/override.h"				/* Must be the last header included */
@@ -174,7 +173,7 @@ static gboolean
 ntp_send_probes(void)
 {
 	static const struct {
-		const gchar *addr;
+		const char *addr;
 	} hosts[] = {
 #if 0
 		/* Skip this for now. We check replies only against 127.0.0.1 and ::1
@@ -248,12 +247,12 @@ ntp_got_reply(struct gnutella_socket *s)
 	tm_t replied;
 	tm_t got;
 	tm_t offset;
-	gdouble clock_offset;
+	double clock_offset;
 
 	tm_now_exact(&got);
 
 	if (s->pos != NTP_MINSIZE && s->pos != NTP_MAXSIZE) {
-		g_warning("got weird reply from NTP server (%d bytes)", (gint) s->pos);
+		g_warning("got weird reply from NTP server (%d bytes)", (int) s->pos);
 		return;
 	}
 
