@@ -1,5 +1,5 @@
 /*
- * $Id: oob.c 16106 2009-03-10 15:16:31Z rmanfredi $
+ * $Id: oob.c 17156 2009-11-01 20:22:05Z rmanfredi $
  *
  * Copyright (c) 2004, Raphael Manfredi
  *
@@ -35,7 +35,7 @@
 
 #include "common.h"
 
-RCSID("$Id: oob.c 16106 2009-03-10 15:16:31Z rmanfredi $")
+RCSID("$Id: oob.c 17156 2009-11-01 20:22:05Z rmanfredi $")
 
 #include "oob.h"
 #include "hosts.h"
@@ -321,7 +321,8 @@ servent_service(cqueue_t *cq, gpointer obj)
 
 	if (GNET_PROPERTY(udp_debug) > 19)
 		g_message("UDP queuing OOB %s to %s for %s",
-			gmsg_infostr_full(pmsg_start(mb)), gnet_host_to_string(s->host),
+			gmsg_infostr_full(pmsg_start(mb), pmsg_written_size(mb)),
+			gnet_host_to_string(s->host),
 			guid_hex_str(cast_to_guid_ptr_const(pmsg_start(mb))));
 
 	/*
@@ -669,7 +670,7 @@ void
 oob_init(void)
 {
 	results_by_muid = g_hash_table_new(guid_hash, guid_eq);
-	servent_by_host = g_hash_table_new(host_hash, host_eq);
+	servent_by_host = g_hash_table_new(gnet_host_hash, gnet_host_eq);
 }
 
 /**
