@@ -1,5 +1,5 @@
 /*
- * $Id: oob_proxy.h 12824 2007-02-13 04:49:50Z cbiere $
+ * $Id: oob_proxy.h 16106 2009-03-10 15:16:31Z rmanfredi $
  *
  * Copyright (c) 2004, Raphael Manfredi
  *
@@ -44,16 +44,17 @@
 
 struct array;
 struct gnutella_node;
+struct guid;
 
 void oob_proxy_init(void);
 void oob_proxy_close(void);
 
 void oob_proxy_create(struct gnutella_node *n);
 gboolean oob_proxy_pending_results(
-	struct gnutella_node *n, const gchar *muid,
-	gint hits, gboolean udp_firewalled, const struct array *token);
+	struct gnutella_node *n, const struct guid *muid,
+	int hits, gboolean udp_firewalled, const struct array *token);
 gboolean oob_proxy_got_results(struct gnutella_node *n, guint results);
-const gchar *oob_proxy_muid_proxied(const gchar *muid);
+const struct guid *oob_proxy_muid_proxied(const struct guid *muid);
 
 #endif /* _core_oob_proxy_h_ */
 
