@@ -1,5 +1,5 @@
 /*
- * $Id: pcache.c 16223 2009-03-20 21:48:03Z rmanfredi $
+ * $Id: pcache.c 17246 2010-02-14 14:46:07Z rmanfredi $
  *
  * Copyright (c) 2001-2003, Raphael Manfredi
  *
@@ -35,7 +35,7 @@
 
 #include "common.h"
 
-RCSID("$Id: pcache.c 16223 2009-03-20 21:48:03Z rmanfredi $")
+RCSID("$Id: pcache.c 17246 2010-02-14 14:46:07Z rmanfredi $")
 
 #include "alive.h"
 #include "extensions.h"
@@ -66,6 +66,7 @@ RCSID("$Id: pcache.c 16223 2009-03-20 21:48:03Z rmanfredi $")
 #include "lib/atoms.h"
 #include "lib/endian.h"
 #include "lib/gnet_host.h"
+#include "lib/pow2.h"
 #include "lib/tm.h"
 #include "lib/walloc.h"
 #include "lib/override.h"	/* Must be the last header included */
@@ -300,17 +301,18 @@ build_pong_msg(host_addr_t sender_addr, guint16 sender_port,
 			ggep_stream_end(&gs);
 		}
 
-		if (meta->flags & PONG_META_HAS_GUE)	/* GUESS support */
+		if (meta->flags & PONG_META_HAS_GUE) {	/* GUESS support */
 			ggep_stream_pack(&gs, GGEP_NAME(GUE),
 				cast_to_gpointer(&meta->guess), 1, 0);
+		}
 
 		if (meta->flags & PONG_META_HAS_UP) {	/* Ultrapeer info */
 			gboolean ok;
 
 			ok = ggep_stream_begin(&gs, GGEP_NAME(UP), 0) &&
 			ggep_stream_write(&gs, &meta->version_up, 1) &&
-			ggep_stream_write(&gs, &meta->up_slots, 1) &&
 			ggep_stream_write(&gs, &meta->leaf_slots, 1) &&
+			ggep_stream_write(&gs, &meta->up_slots, 1) &&
 			ggep_stream_end(&gs);
 		}
 
@@ -468,8 +470,8 @@ build_pong_msg(host_addr_t sender_addr, guint16 sender_port,
 		char ip_port[6];
 
 		/* Ip Port (not UHC IPP!)*/
-		if (GNET_PROPERTY(pcache_debug) || GNET_PROPERTY(ggep_debug))
-			g_message("adding GGEP IP for %s",
+		if (GNET_PROPERTY(pcache_debug) > 1 || GNET_PROPERTY(ggep_debug) > 1)
+			g_message("adding GGEP IP to pong for %s",
 				host_addr_port_to_string(sender_addr, sender_port));
 
 		poke_be32(&ip_port[0], host_addr_ipv4(sender_addr));
@@ -589,9 +591,9 @@ ping_type(const gnutella_node_t *n)
 	}
 
 	if ((flags & PING_F_UHC) && GNET_PROPERTY(ggep_debug) > 1)
-		printf("%s: UHC ping requesting %s slots from %s\n",
-			gmsg_infostr(&n->header),
-			(flags & PING_F_UHC_ANY) ?	"unspecified" :
+		g_message("%s: UHC ping requesting %s slots from %s",
+			gmsg_node_infostr(n),
+			(flags & PING_F_UHC_ANY) ?	"any" :
 			(flags & PING_F_UHC_ULTRA) ?	"ultra" : "leaf",
 			host_addr_port_to_string(n->addr, n->port));
 
@@ -668,8 +670,8 @@ send_personal_info(struct gnutella_node *n, gboolean control,
 
 	if (GNET_PROPERTY(current_peermode) == NODE_P_ULTRA) {
 		local_meta.flags |= PONG_META_HAS_UP;
-		local_meta.up_slots = MIN(node_missing(), 255);
 		local_meta.leaf_slots = MIN(node_leaves_missing(), 255);
+		local_meta.up_slots = MIN(node_missing(), 255);
 	}
 
 	if ((flags & PING_F_IP)) {
@@ -1053,7 +1055,7 @@ found:
 	*port = last_port = cp->info.port;
 
 	if (GNET_PROPERTY(pcache_debug) > 8)
-		printf("returning recent %s PONG %s\n",
+		g_message("returning recent %s PONG %s",
 			host_type_to_string(type),
 			host_addr_port_to_string(cp->info.addr, cp->info.port));
 
@@ -1069,7 +1071,7 @@ add_recent_pong(host_type_t type, struct cached_pong *cp)
 {
 	struct recent *rec;
 
-	g_assert((int) type >= 0 && type < HOST_MAX);
+	g_assert(UNSIGNED(type) < HOST_MAX);
 
 	rec = &recent_pongs[type];
 
@@ -1129,7 +1131,7 @@ pcache_clear_recent(host_type_t type)
 	GList *l;
 	struct recent *rec;
 
-	g_assert((int) type >= 0 && type < HOST_MAX);
+	g_assert(UNSIGNED(type) < HOST_MAX);
 
 	rec = &recent_pongs[type];
 
@@ -1193,7 +1195,7 @@ pcache_expire(void)
 	}
 
 	if (GNET_PROPERTY(pcache_debug) > 4)
-		printf("Pong CACHE expired (%d entr%s, %d in reserve)\n",
+		g_message("Pong CACHE expired (%d entr%s, %d in reserve)",
 			entries, entries == 1 ? "y" : "ies", hcache_size(HOST_ANY));
 }
 
@@ -1215,7 +1217,7 @@ pcache_close(void)
 		g_hash_table_destroy(recent_pongs[type].ht_recent_pongs);
 	}
 
-	aging_destroy(udp_pings);
+	aging_destroy(&udp_pings);
 }
 
 /**
@@ -1288,7 +1290,7 @@ ping_all_neighbours(void)
 		if (!(n->attrs & NODE_A_PONG_CACHING))
 			n->next_ping = time_advance(now, OLD_PING_PERIOD);
 
-		if (GNET_PROPERTY(pcache_debug))
+		if (GNET_PROPERTY(pcache_debug) > 1)
 			g_message("PCACHE pinging \"%s\" %s",
 				(n->attrs & NODE_A_PONG_CACHING) ? "new" : "old",
 				host_addr_port_to_string(n->addr, n->port));
@@ -1377,7 +1379,7 @@ setup_pong_demultiplexing(struct gnutella_node *n, guint8 ttl)
 		n->pong_needed[h] = amount;
 		remains -= amount;
 		if (GNET_PROPERTY(pcache_debug) > 7)
-			printf("pong_needed[%d] = %d, remains = %d\n", h, amount, remains);
+			g_message("pong_needed[%d] = %d, remains = %d", h, amount, remains);
 	}
 
 	g_assert(remains == 0);
@@ -1444,8 +1446,8 @@ iterate_on_cached_line(
 		n->pong_missing--;
 
 		if (GNET_PROPERTY(pcache_debug) > 7)
-			printf("iterate: sent cached pong %s (hops=%d, TTL=%d) to %s, "
-				"missing=%d %s\n",
+			g_message("iterate: sent cached pong %s (hops=%d, TTL=%d) to %s, "
+				"missing=%d %s",
 				host_addr_port_to_string(cp->info.addr, cp->info.port),
 				hops, ttl,
 				node_addr(n), n->pong_missing, strict ? "STRICT" : "loose");
@@ -1630,8 +1632,8 @@ pong_all_neighbours_but_one(
 			hops + 1, ttl, &cn->ping_guid, &cp->info, cp->meta);
 
 		if (GNET_PROPERTY(pcache_debug) > 7)
-			printf("pong_all: sent cached pong %s (hops=%d, TTL=%d) to %s "
-				"missing=%d\n",
+			g_message("pong_all: sent cached pong %s (hops=%d, TTL=%d) to %s "
+				"missing=%d",
 				host_addr_port_to_string(cp->info.addr, cp->info.port),
 				hops, ttl, node_addr(cn), cn->pong_missing);
 	}
@@ -1645,14 +1647,14 @@ static void
 pong_random_leaf(struct cached_pong *cp, guint8 hops, guint8 ttl)
 {
 	const GSList *sl;
-	int leaves;
+	unsigned leaves;
 	struct gnutella_node *leaf = NULL;
 
 	g_assert(GNET_PROPERTY(current_peermode) == NODE_P_ULTRA);
 
 	for (sl = node_all_nodes(), leaves = 0; sl; sl = g_slist_next(sl)) {
 		struct gnutella_node *cn = sl->data;
-		int threshold;
+		unsigned threshold;
 
 		if (cn->pong_missing)	/* A job for pong_all_neighbours_but_one() */
 			continue;
@@ -1672,9 +1674,9 @@ pong_random_leaf(struct cached_pong *cp, guint8 hops, guint8 ttl)
 		 */
 
 		leaves++;
-		threshold = (int) (1000.0 / leaves);
+		threshold = 1000.0 / leaves;
 
-		if ((int) random_value(999) < threshold)
+		if (random_value(999) < threshold)
 			leaf = cn;
 	}
 
@@ -1690,7 +1692,7 @@ pong_random_leaf(struct cached_pong *cp, guint8 hops, guint8 ttl)
 			&cp->info, cp->meta);
 
 		if (GNET_PROPERTY(pcache_debug) > 7)
-			printf("pong_random_leaf: sent pong %s (hops=%d, TTL=%d) to %s\n",
+			g_message("pong_random_leaf: sent pong %s (hops=%d, TTL=%d) to %s",
 				host_addr_port_to_string(cp->info.addr, cp->info.port),
 				hops, ttl, node_addr(leaf));
 	}
@@ -1789,8 +1791,8 @@ pong_extract_metadata(struct gnutella_node *n)
 				payload = ext_payload(e);
 				ALLOCATE(UP);
 				meta->version_up = payload[0];
-				meta->up_slots = payload[1];
-				meta->leaf_slots = payload[2];
+				meta->leaf_slots = payload[1];
+				meta->up_slots = payload[2];
 			}
 			break;
 		case EXT_T_GGEP_VC:
@@ -1839,10 +1841,10 @@ pong_extract_metadata(struct gnutella_node *n)
 			}
 			break;
 		default:
-			if (GNET_PROPERTY(ggep_debug) > 1 && e->ext_type == EXT_GGEP) {
+			if (GNET_PROPERTY(ggep_debug) > 3 && e->ext_type == EXT_GGEP) {
 				paylen = ext_paylen(e);
 				g_warning("%s: unhandled GGEP \"%s\" (%d byte%s)",
-					gmsg_infostr(&n->header), ext_ggep_id_str(e),
+					gmsg_node_infostr(n), ext_ggep_id_str(e),
 					paylen, paylen == 1 ? "" : "s");
 			}
 			break;
@@ -1870,7 +1872,7 @@ record_fresh_pong(
 	struct cached_pong *cp;
 	guint8 hop;
 
-	g_assert((int) type >= 0 && type < HOST_MAX);
+	g_assert(UNSIGNED(type) < HOST_MAX);
 
 	cp = walloc(sizeof *cp);
 
@@ -1901,8 +1903,7 @@ pcache_udp_ping_received(struct gnutella_node *n)
 
 	/*
 	 * If we got a PING whose MUID is our node's GUID, then it's a reply
-	 * to our "UDP Connect Back" message.  Ignore it, we've already
-	 * noticed that we got an unsolicited UDP message.
+	 * to our "UDP Connect Back" message.
 	 */
 
 	if (
@@ -1910,7 +1911,17 @@ pcache_udp_ping_received(struct gnutella_node *n)
 			gnutella_header_get_muid(&n->header))
 	) {
 		if (GNET_PROPERTY(udp_debug) > 19)
-			printf("UDP got unsolicited PING matching our GUID!\n");
+			g_message("UDP got unsolicited PING matching our GUID!");
+		inet_udp_got_unsolicited_incoming();
+		return;
+	}
+
+	/*
+	 * Don't answer to pings from bad nodes (includes "alien" hosts).
+	 */
+
+	if (hcache_node_is_bad(n->addr)) {
+		gnet_stats_count_dropped(n, MSG_DROP_BAD_RETURN_ADDRESS);
 		return;
 	}
 
@@ -1925,6 +1936,7 @@ pcache_udp_ping_received(struct gnutella_node *n)
 
 	aging_insert(udp_pings,
 		wcopy(&n->addr, sizeof n->addr), GUINT_TO_POINTER(1));
+
 	send_personal_info(n, FALSE, ping_type(n));
 }
 
@@ -2000,7 +2012,7 @@ pcache_ping_received(struct gnutella_node *n)
 		(n->attrs & (NODE_A_PONG_CACHING|NODE_A_PONG_ALIEN)) ==
 			NODE_A_PONG_CACHING
 	) {
-		if (GNET_PROPERTY(pcache_debug) || GNET_PROPERTY(dbg))
+		if (GNET_PROPERTY(pcache_debug))
 			g_warning("node %s (%s) [%d.%d] claimed ping reduction, "
 				"got ping with hops=%d", node_addr(n),
 				node_vendor(n),
@@ -2108,9 +2120,12 @@ pcache_udp_pong_received(struct gnutella_node *n)
 			payload = ext_payload(e);
 
 			if (paylen % 6) {
-				g_warning("%s (UDP): bad length for GGEP \"%s\" (%d byte%s)",
-					gmsg_infostr(&n->header), ext_ggep_id_str(e),
-					paylen, paylen == 1 ? "" : "s");
+				if (GNET_PROPERTY(pcache_debug) || GNET_PROPERTY(ggep_debug)) {
+					g_warning("%s (UDP): "
+						"bad length for GGEP \"%s\" (%d byte%s)",
+						gmsg_node_infostr(n), ext_ggep_id_str(e),
+						paylen, paylen == 1 ? "" : "s");
+				}
 			} else {
 				switch (e->ext_token) {
 				case EXT_T_GGEP_IPP:
@@ -2147,7 +2162,7 @@ pcache_udp_pong_received(struct gnutella_node *n)
 			if (GNET_PROPERTY(ggep_debug) > 1 && e->ext_type == EXT_GGEP) {
 				paylen = ext_paylen(e);
 				g_warning("%s (UDP): unhandled GGEP \"%s\" (%d byte%s)",
-					gmsg_infostr(&n->header), ext_ggep_id_str(e),
+					gmsg_node_infostr(n), ext_ggep_id_str(e),
 					paylen, paylen == 1 ? "" : "s");
 			}
 			break;
@@ -2377,10 +2392,10 @@ pcache_pong_received(struct gnutella_node *n)
 	 */
 
 	if (!(n->attrs & NODE_A_PONG_CACHING)) {
-		int ratio = (int) random_value(100);
+		unsigned ratio = random_value(100);
 		if (ratio >= OLD_CACHE_RATIO) {
 			if (GNET_PROPERTY(pcache_debug) > 7)
-				printf("NOT CACHED pong %s (hops=%d, TTL=%d) from OLD %s\n",
+				g_message("NOT CACHED pong %s (hops=%d, TTL=%d) from OLD %s",
 					host_addr_port_to_string(addr, port),
 					gnutella_header_get_hops(&n->header),
 					gnutella_header_get_ttl(&n->header),
@@ -2404,7 +2419,7 @@ pcache_pong_received(struct gnutella_node *n)
 		add_recent_pong(HOST_ULTRA, cp);
 
 	if (GNET_PROPERTY(pcache_debug) > 6)
-		printf("CACHED %s pong %s (hops=%d, TTL=%d) from %s %s\n",
+		g_message("CACHED %s pong %s (hops=%d, TTL=%d) from %s %s",
 			ptype == HOST_ULTRA ? "ultra" : "normal",
 			host_addr_port_to_string(addr, port),
 			gnutella_header_get_hops(&n->header),
