@@ -1,5 +1,5 @@
 /*
- * $Id: pcache.h 13834 2007-06-16 01:30:05Z cbiere $
+ * $Id: pcache.h 15814 2008-09-07 03:09:56Z cbiere $
  *
  * Copyright (c) 2001-2003, Raphael Manfredi
  *
@@ -57,12 +57,17 @@ typedef struct pong_meta {
 	host_addr_t sender_addr;	/**< For GGEP "IP" */
 	guint16 sender_port;  		/**< For GGEP "IP" */
 
-	guint32 daily_uptime; /**< Node's daily uptime, from GGEP "DU" */
-	guint8 up_slots;      /**< Free UP slots, from GGEP "UP" */
-	guint8 leaf_slots;    /**< Free leaf slots, from GGEP "UP" */
-	guint8 version_up;    /**< Ultrapeer version protocol, from GGEP "UP" */
-	guint8 version_ua;    /**< Servent version, from GGEP "VC" */
-	guint8 flags;	      /**< Validation flags */
+	guint32 daily_uptime;	/**< Node's daily uptime, from GGEP "DU" */
+	guint8 up_slots;		/**< Free UP slots, from GGEP "UP" */
+	guint8 leaf_slots;		/**< Free leaf slots, from GGEP "UP" */
+	guint8 version_up;		/**< Ultrapeer version protocol, from GGEP "UP" */
+	guint8 version_ua;		/**< Servent version, from GGEP "VC" */
+
+	guint8 dht_major;    	/**< DHT major version, from GGEP "DHT" */
+	guint8 dht_minor;    	/**< DHT minor version, from GGEP "DHT" */
+	guint8 dht_mode;    	/**< DHT mode, from GGEP "DHT" */
+
+	guint8 flags;			/**< Validation flags */
 } pong_meta_t;
 
 enum {
@@ -72,15 +77,20 @@ enum {
 	PONG_META_HAS_LOC	= (1 << 3), /**< The "LOC" fields are valid */
 	PONG_META_HAS_DU	= (1 << 4), /**< The "DU" fields are valid */
 	PONG_META_HAS_IPV6	= (1 << 5), /**< The "IPV6" fields are valid */
-	PONG_META_HAS_TLS	= (1 << 6)  /**< The "TLS" fields are valid */
+	PONG_META_HAS_TLS	= (1 << 6), /**< The "TLS" fields are valid */
+	PONG_META_HAS_DHT	= (1 << 7), /**< The "DHT" fields are valid */
+
+	PONG_META_HAS_END
 };
 
 /**
  * Global Functions.
  */
 
-gnutella_msg_init_t *build_ping_msg(
-	const gchar *muid, guint8 ttl, gboolean uhc, guint32 *size);
+struct guid;
+
+gnutella_msg_init_t *build_ping_msg(const struct guid *,
+						guint8 ttl, gboolean uhc, guint32 *size);
 
 /*
  * Public interface.
@@ -97,6 +107,7 @@ void pcache_pong_fake(struct gnutella_node *n,
 	const host_addr_t addr, guint16 port);
 gboolean pcache_get_recent(host_type_t type, host_addr_t *addr, guint16 *port);
 void pcache_clear_recent(host_type_t type);
+void ping_all_neighbours(void);
 
 #endif /* _core_pcache_h_ */
 /* vi: set ts=4 sw=4 cindent: */
