@@ -1,5 +1,5 @@
 /*
- * $Id: qhit.c 16171 2009-03-15 19:22:28Z rmanfredi $
+ * $Id: qhit.c 17194 2009-11-05 21:44:06Z rmanfredi $
  *
  * Copyright (c) 2004, Raphael Manfredi
  *
@@ -35,7 +35,9 @@
 
 #include "common.h"
 
-RCSID("$Id: qhit.c 16171 2009-03-15 19:22:28Z rmanfredi $")
+RCSID("$Id: qhit.c 17194 2009-11-05 21:44:06Z rmanfredi $")
+
+#include "gtk-gnutella.h"
 
 #include "bsched.h"
 #include "dmesh.h"		/* For dmesh_fill_alternate() */
@@ -57,6 +59,7 @@ RCSID("$Id: qhit.c 16171 2009-03-15 19:22:28Z rmanfredi $")
 #include "lib/getdate.h"
 #include "lib/endian.h"
 #include "lib/misc.h"
+#include "lib/sequence.h"
 #include "lib/tm.h"
 #include "lib/override.h"			/* Must be the last header included */
 
@@ -66,7 +69,7 @@ RCSID("$Id: qhit.c 16171 2009-03-15 19:22:28Z rmanfredi $")
  */
 #define QHIT_MAX_RESULTS	10		/**< Maximum amount of hits in a query hit */
 #define QHIT_MAX_ALT		10		/**< Send out 10 alt-locs per entry, max */
-#define QHIT_MAX_PROXIES	5		/**< Send out 5 push-proxies at most */
+#define QHIT_MAX_PROXIES	8		/**< Send out 8 push-proxies at most */
 #define QHIT_MAX_GGEP		512		/**< Allocated room for trailing GGEP */
 #define QHIT_SIZE_THRESHOLD	2016	/**< Flush query hits larger than this */
 
@@ -337,6 +340,13 @@ qhit_send_node(gpointer data, size_t len, gpointer udata)
 	gmsg_sendto_one(n, data, len);
 }
 
+static void
+qhit_log_ggep_write_failure(const char *id)
+{
+	if (GNET_PROPERTY(qhit_debug))
+		g_warning("QHIT could not write GGEP \"%s\" extension", id);
+}
+
 /**
  * Flush pending search request to the network.
  */
@@ -421,7 +431,7 @@ flush_match(void)
 			ggep_stream_end(&gs);
 
 		if (!ok)
-			g_warning("could not write GGEP \"GTKGV1\" extension in query hit");
+			qhit_log_ggep_write_failure("GTKGV1");
 	}
 
 	{
@@ -430,8 +440,9 @@ flush_match(void)
 		if (
 			token->data &&
 			!ggep_stream_pack(&gs, GGEP_NAME(SO), token->data, token->size, 0)
-		)
-			g_warning("could not add GGEP \"SO\" extension to query hit");
+		) {
+			qhit_log_ggep_write_failure("SO");
+		}
 	}
 
 	/*
@@ -440,12 +451,12 @@ flush_match(void)
 	 */
 
 	if (GNET_PROPERTY(is_firewalled)) {
-		const GSList *nodes = node_push_proxies();
+		sequence_t *seq = node_push_proxies();
 
-		if (nodes != NULL) {
+		if (!sequence_is_empty(seq)) {
 			guchar tls_bytes[(QHIT_MAX_PROXIES + 7) / 8];
 			guint tls_index, tls_length;
-			const GSList *iter;
+			sequence_iter_t *iter;
 			gboolean ok;
 
 			ok = ggep_stream_begin(&gs, GGEP_NAME(PUSH), 0);
@@ -453,17 +464,26 @@ flush_match(void)
 			tls_index = 0;
 			tls_length = 0;
 
-			for (iter = nodes; ok && NULL != iter; iter = g_slist_next(iter)) {
-				const struct gnutella_node *n = iter->data;
+			/*
+			 * We iterate backwards to get the most stable of our push proxies,
+			 * namely those to which we've been connected for the longest time.
+			 */
+
+			iter = sequence_backward_iterator(seq, TRUE);
+
+			while (ok && sequence_iter_has_previous(iter)) {
+				const gnet_host_t *host = sequence_iter_previous(iter);
+				host_addr_t addr = gnet_host_get_addr(host);
+				guint16 port = gnet_host_get_port(host);
 				char proxy[6];
 
-				if (NET_TYPE_IPV4 != host_addr_net(n->proxy_addr))
+				if (NET_TYPE_IPV4 != host_addr_net(addr))
 					continue;
 
-				poke_be32(&proxy[0], host_addr_ipv4(n->proxy_addr));
-				poke_le16(&proxy[4], n->proxy_port);
+				poke_be32(&proxy[0], host_addr_ipv4(addr));
+				poke_le16(&proxy[4], port);
 				ok = ggep_stream_write(&gs, proxy, sizeof proxy);
-				if (NODE_F_CAN_TLS & n->flags) {
+				if (tls_cache_lookup(addr, port)) {
 					tls_bytes[tls_index >> 3] |= 0x80U >> (tls_index & 7);
 					tls_length = (tls_index >> 3) + 1;
 				}
@@ -473,18 +493,19 @@ flush_match(void)
 			}
 
 			ok = ok && ggep_stream_end(&gs);
+			sequence_iterator_release(&iter);
 
 			if (!ok)
-				g_warning("could not write GGEP \"PUSH\" extension "
-					"in query hit");
+				qhit_log_ggep_write_failure("PUSH");
 
 			if (ok && tls_length > 0) {
 				ok = ggep_stream_pack(&gs, GGEP_NAME(PUSH_TLS),
 						tls_bytes, tls_length, 0);
 				if (!ok)
-					g_warning("could not write GGEP \"PUSH_TLS\" extension");
+					qhit_log_ggep_write_failure("PUSH_TLS");
 			}
 		}
+		sequence_release(&seq);
 	}
 
 	/*
@@ -495,7 +516,7 @@ flush_match(void)
 	if (
 		!GNET_PROPERTY(is_firewalled) &&
 		GNET_PROPERTY(give_server_hostname) &&
-		'\0' != GNET_PROPERTY(server_hostname)[0]
+		!is_null_or_empty(GNET_PROPERTY(server_hostname))
 	) {
 		gboolean ok;
 
@@ -505,8 +526,7 @@ flush_match(void)
 				0);
 
 		if (!ok)
-			g_warning("could not write GGEP \"HNAME\" extension "
-				"in query hit");
+			qhit_log_ggep_write_failure("HNAME");
 	}
 
 	{
@@ -516,18 +536,24 @@ flush_match(void)
 			const guint8 *ipv6 = host_addr_ipv6(&addr);
 
 			if (!ggep_stream_pack(&gs, GGEP_GTKG_NAME(IPV6), ipv6, 16, 0))
-				g_warning("could not write GGEP \"GTKG.IPV6\" extension "
-						"into query hit");
+				qhit_log_ggep_write_failure("GTKG.IPV6");
 		}
 	}
 
 	if (tls_enabled()) {
 		if (!ggep_stream_pack(&gs, GGEP_NAME(TLS), NULL, 0, 0))
-			g_warning("could not write GGEP \"TLS\" extension into QHIT");
-		if (!ggep_stream_pack(&gs, GGEP_GTKG_NAME(TLS), NULL, 0, 0))
-			g_warning("could not write GGEP \"GTKG.TLS\" extension into QHIT");
-	}
+			qhit_log_ggep_write_failure("TLS");
 
+		/*
+		 * FIXME TODO: Drop sending GTKG.TLS after 0.96.7 is out, only
+		 * sending the now standardized "TLS" extension which GTKG has been
+		 * understanding since 0.96.6.
+		 *		--RAM, 2009-10-24
+		 */
+
+		if (!ggep_stream_pack(&gs, GGEP_GTKG_NAME(TLS), NULL, 0, 0))
+			qhit_log_ggep_write_failure("GTKG.TLS");
+	}
 
 	/*
 	 * Advertise the Browse Host extension in the results if the feature is
@@ -536,7 +562,7 @@ flush_match(void)
 
 	if (GNET_PROPERTY(browse_host_enabled)) {
 		if (!ggep_stream_pack(&gs, GGEP_NAME(BH), NULL, 0, 0))
-			g_warning("could not write GGEP \"BH\" extension into query hit");
+			qhit_log_ggep_write_failure("BH");
 	}
 
 	ggep_len = ggep_stream_close(&gs);
@@ -558,7 +584,9 @@ flush_match(void)
 
 failure:
 
-	g_warning("Created query hit was too big, discarding");
+	if (GNET_PROPERTY(qhit_debug))
+		g_warning("QHIT message was too big, discarding");
+
 	found_clear();
 }
 
@@ -688,7 +716,7 @@ add_file(const struct shared_file *sf)
 			ggep_stream_end(&gs);
 
 		if (!ok)
-			g_warning("could not write GGEP \"H\" extension in query hit");
+			qhit_log_ggep_write_failure("H");
 	}
 
 	/*
@@ -696,6 +724,7 @@ add_file(const struct shared_file *sf)
 	 * Now they are still unaware of GGEP "H" but emit GGEP "TT" with the
 	 * hash in binary form.
 	 */
+
 	if (sha1_available && !found_ggep_h()) {
 		const struct tth * const tth = shared_file_tth(sf);
 
@@ -703,7 +732,7 @@ add_file(const struct shared_file *sf)
 			ok = ggep_stream_pack(&gs,
 						GGEP_NAME(TT), tth->data, TTH_RAW_SIZE, GGEP_W_COBS);
 			if (!ok)
-				g_warning("could not write GGEP \"TT\" extension in query hit");
+				qhit_log_ggep_write_failure("TT");
 		}
 	}
 
@@ -718,11 +747,11 @@ add_file(const struct shared_file *sf)
 
 		len = ggept_filesize_encode(shared_file_size(sf), buf);
 
-		g_assert(len > 0 && len <= (int) sizeof buf);
+		g_assert(len > 0 && UNSIGNED(len) <= sizeof buf);
 
 		ok = ggep_stream_pack(&gs, GGEP_NAME(LF), buf, len, GGEP_W_COBS);
 		if (!ok)
-			g_warning("could not write GGEP \"LF\" extension in query hit");
+			qhit_log_ggep_write_failure("LF");
 	}
 
 	/*
@@ -767,13 +796,13 @@ add_file(const struct shared_file *sf)
 		ok = ok && ggep_stream_end(&gs);
 
 		if (!ok)
-			g_warning("could not write GGEP \"ALT\" extension in query hit");
+			qhit_log_ggep_write_failure("ALT");
 
 		if (ok && tls_length > 0) {
 			ok = ggep_stream_pack(&gs, GGEP_NAME(ALT_TLS),
 					tls_bytes, tls_length, GGEP_W_COBS);
 			if (!ok)
-				g_warning("could not write GGEP \"ALT_TLS\" extension");
+				qhit_log_ggep_write_failure("ALT_TLS");
 		}
 	}
 
@@ -783,7 +812,7 @@ add_file(const struct shared_file *sf)
 		if (rp) {
 			ok = ggep_stream_pack(&gs, GGEP_NAME(PATH), rp, strlen(rp), 0);
 			if (!ok)
-				g_warning("could not add GGEP \"PATH\" extension to query hit");
+				qhit_log_ggep_write_failure("PATH");
 		}
 	}
 
@@ -803,11 +832,11 @@ add_file(const struct shared_file *sf)
 			mtime = MAX(0, mtime);
 
 			len = ggept_ct_encode(mtime, buf);
-			g_assert(len >= 0 && len <= (int) sizeof buf);
+			g_assert(UNSIGNED(len) <= sizeof buf);
 
 			ok = ggep_stream_pack(&gs, GGEP_NAME(CT), buf, len, GGEP_W_COBS);
 			if (!ok)
-				g_warning("could not write GGEP \"CT\" extension in query hit");
+				qhit_log_ggep_write_failure("CT");
 		}
 	}
 
