@@ -1,5 +1,5 @@
 /*
- * $Id: routing.c 16179 2009-03-16 18:16:53Z rmanfredi $
+ * $Id: routing.c 17314 2010-02-27 19:42:50Z rmanfredi $
  *
  * Copyright (c) 2001-2003, Raphael Manfredi
  *
@@ -35,7 +35,7 @@
 
 #include "common.h"
 
-RCSID("$Id: routing.c 16179 2009-03-16 18:16:53Z rmanfredi $")
+RCSID("$Id: routing.c 17314 2010-02-27 19:42:50Z rmanfredi $")
 
 #include "search.h"			/* For search_passive. */
 #include "routing.h"
@@ -156,7 +156,19 @@ static const char * const banned_push[] = {
 	"27630b632f070ca9ffc48eb06a72c700",		/**< Morpheus?, 2005-08-30 */
 	"58585858585858585858585858585858",		/**< Probably an init bug! */
 };
-static GHashTable *ht_banned_push = NULL;
+static GHashTable *ht_banned_push;
+
+/**
+ * Starving GUIDs for push routing.
+ *
+ * Downloads which require a push route to a given GUID but have no identified
+ * push route are said to be "starving" for that GUID.  They can of course
+ * rely on push-proxies to contact the node, or look through the DHT.
+ *
+ * But still, in case we happen to see a query hit that comes from one of
+ * the starving GUID, it's good to notify the download layer.
+ */
+static GHashTable *ht_starving_guid;
 
 /**
  * Push-proxy table.
@@ -164,7 +176,7 @@ static GHashTable *ht_banned_push = NULL;
  * It maps a GUID to a node, so that we can easily send a push message
  * on behalf of a requesting node to the proper connection.
  */
-static GHashTable *ht_proxyfied = NULL;
+static GHashTable *ht_proxyfied;
 
 /**
  * Routing logging.
@@ -179,7 +191,7 @@ struct route_log {
 	gboolean handle;			/**< Whether message will be handled */
 	gboolean local;				/**< Whether message originated locally */
 	gboolean new;				/**< Whether message is a new message */
-	char extra[80];			/**< Extra text for logging */
+	char extra[80];				/**< Extra text for logging */
 	struct route_dest dest;		/**< Message destination */
 };
 
@@ -191,6 +203,53 @@ is_banned_push(const struct guid *guid)
 	return NULL != g_hash_table_lookup(ht_banned_push, guid);
 }
 
+/**
+ * Remove starving condition for a GUID.
+ */
+void
+route_starving_remove(const guid_t *guid)
+{
+ 	/*
+	 * The GUID atom is still referred to by the server,
+	 * so don't clear anything.
+	 */
+
+	g_hash_table_remove(ht_starving_guid, guid);
+}
+
+/**
+ * Add starving condition for ``guid''.
+ *
+ * When we learn about a new route for that GUID, the callback will be
+ * triggered, with the GUID as argument.
+ *
+ * @attention
+ * NB: assumes ``guid'' is already an atom linked somehow to ``server''.
+ */
+void
+route_starving_add(const guid_t *guid, route_starving_cb_t cb)
+{
+	gm_hash_table_replace_const(ht_starving_guid, guid,
+		cast_func_to_pointer(cb));
+}
+
+/**
+ * Invoked when we discover a new route for a given GUID.
+ *
+ * Check whether a GUID was recorded as starving and invoke the callback
+ * if it was.
+ */
+static void
+route_starving_check(const guid_t *guid)
+{
+	route_starving_cb_t cb;
+
+	cb = cast_pointer_to_func(g_hash_table_lookup(ht_starving_guid, guid));
+
+	if (cb != NULL)
+		(*cb)(guid);
+}
+
 /**
  * Record message parameters.
  */
@@ -463,13 +522,13 @@ get_next_slot(void)
 	idx = routing.next_idx;
 	chunk_idx = CHUNK_INDEX(idx);
 
-	g_assert((int) chunk_idx >= 0 && chunk_idx < MAX_CHUNKS);
+	g_assert(UNSIGNED(chunk_idx) < MAX_CHUNKS);
 
 	chunk = routing.chunks[chunk_idx];
 
 	if (chunk == NULL) {
 
-		g_assert((int) idx >= routing.capacity);
+		g_assert(idx >= UNSIGNED(routing.capacity));
 
 		/*
 		 * Chunk does not exist yet, determine whether we should create
@@ -478,8 +537,8 @@ get_next_slot(void)
 
 		if (idx > 0 && elapsed > TABLE_MIN_CYCLE) {
 			if (GNET_PROPERTY(routing_debug))
-				printf("RT cycling over table, elapsed=%d, holds %d / %d\n",
-					(int) elapsed, routing.count, routing.capacity);
+				printf("RT cycling over table, elapsed=%u, holds %d / %d\n",
+					(unsigned) elapsed, routing.count, routing.capacity);
 
 			chunk_idx = 0;
 			idx = routing.next_idx = 0;
@@ -497,7 +556,7 @@ get_next_slot(void)
 				g_malloc0(CHUNK_MESSAGES * sizeof(struct message *));
 
 			if (GNET_PROPERTY(routing_debug))
-				printf("RT created new chunk #%d, now holds %d / %d\n",
+				g_message("RT created new chunk #%d, now holds %d / %d",
 					chunk_idx, routing.count, routing.capacity);
 
 			slot = routing.chunks[chunk_idx];	/* First slot in new chunk */
@@ -513,16 +572,16 @@ get_next_slot(void)
 
 		if (idx == 0) {
 			if (GNET_PROPERTY(routing_debug))
-				printf("RT cycling over FORCED, elapsed=%d, holds %d / %d\n",
-					(int) elapsed, routing.count, routing.capacity);
+				g_message("RT cycling over FORCED, elapsed=%u, holds %d / %d",
+					(unsigned) elapsed, routing.count, routing.capacity);
 
 			routing.last_rotation = now;
 		}
 	}
 
 	g_assert(slot != NULL);
-	g_assert(idx == (guint) routing.next_idx);
-	g_assert((int) idx >= 0 && idx < (guint) routing.capacity);
+	g_assert(idx == UNSIGNED(routing.next_idx));
+	g_assert(idx < UNSIGNED(routing.capacity));
 
 	/*
 	 * It's OK to go beyond the last allocated chunk (a new chunk will
@@ -800,10 +859,11 @@ routing_init(void)
 	routing.last_rotation = tm_time();
 
 	/*
-	 * Push proxification.
+	 * Push proxification and starving GUIDs.
 	 */
 
 	ht_proxyfied = g_hash_table_new(guid_hash, guid_eq);
+	ht_starving_guid = g_hash_table_new(guid_hash, guid_eq);
 }
 
 /**
@@ -1509,7 +1569,7 @@ route_push(struct route_log *route_log,
 
 	if (is_banned_push(guid)) {
 		if (GNET_PROPERTY(routing_debug) > 3)
-			gmsg_log_dropped(&sender->header,
+			gmsg_log_split_dropped(&sender->header, sender->data, sender->size,
 				"from %s, banned GUID %s",
 				node_addr(sender), guid_hex_str(guid));
 
@@ -1702,6 +1762,7 @@ route_query_hit(struct route_log *route_log,
 
 			if (!is_banned_push(origin_guid)) {
 				message_add(origin_guid, QUERY_HIT_ROUTE_SAVE, sender);
+				route_starving_check(origin_guid);
 			}
 		} else if (m->routes == NULL || !node_sent_message(sender, m)) {
 			struct route_data *route;
@@ -2193,6 +2254,14 @@ routing_close(void)
 
 	g_hash_table_destroy(ht_proxyfied);
 	ht_proxyfied = NULL;
+
+	cnt = g_hash_table_size(ht_starving_guid);
+	if (cnt != 0)
+		g_warning("starving GUID table still holds %u entr%s",
+			cnt, cnt == 1 ? "y" : "ies");
+
+	g_hash_table_destroy(ht_starving_guid);
+	ht_starving_guid = NULL;
 }
 
 /* vi: set ts=4 sw=4 cindent: */
