@@ -1,5 +1,5 @@
 /*
- * $Id: routing.h 16179 2009-03-16 18:16:53Z rmanfredi $
+ * $Id: routing.h 17289 2010-02-25 17:06:34Z rmanfredi $
  *
  * Copyright (c) 2001-2003, Raphael Manfredi
  *
@@ -40,7 +40,7 @@ typedef enum {
 	ROUTE_NONE = 0,			/**< No route, message stops here */
 	ROUTE_ONE,				/**< Route to single node */
 	ROUTE_ALL_BUT_ONE,		/**< Route to all nodes but one */
-	ROUTE_MULTI,			/**< Route to list of nodes */
+	ROUTE_MULTI				/**< Route to list of nodes */
 } route_type_t;
 
 /**
@@ -55,6 +55,11 @@ struct route_dest {
 	} ur;
 };
 
+/**
+ * Starving GUID callback.
+ */
+typedef void (*route_starving_cb_t)(const struct guid *);
+
 /*
  * Global Functions
  */
@@ -74,5 +79,8 @@ gboolean route_proxy_add(const struct guid *guid, struct gnutella_node *n);
 void route_proxy_remove(const struct guid *guid);
 struct gnutella_node *route_proxy_find(const struct guid *guid);
 
+void route_starving_add(const struct guid *guid, route_starving_cb_t cb);
+void route_starving_remove(const struct guid *guid);
+
 #endif /* _core_routing_h_ */
 /* vi: set ts=4 sw=4 cindent: */
