@@ -1,5 +1,5 @@
 /*
- * $Id: rudp.c 16106 2009-03-10 15:16:31Z rmanfredi $
+ * $Id: rudp.c 16357 2009-04-01 18:28:39Z rmanfredi $
  *
  * Copyright (c) 2006, Christian Biere
  *
@@ -35,7 +35,7 @@
 
 #include "common.h"
 
-RCSID("$Id: rudp.c 16106 2009-03-10 15:16:31Z rmanfredi $")
+RCSID("$Id: rudp.c 16357 2009-04-01 18:28:39Z rmanfredi $")
 
 #include "gnutella.h"
 #include "nodes.h"
@@ -59,20 +59,20 @@ static const guint16 RUDP_PROTO_VERSION = 0;
 
 /* Standardized RUDP packet opcodes */
 enum rudp_op {
-  RUDP_OP_SYN       = 0x00,
-  RUDP_OP_ACK       = 0x01,
-  RUDP_OP_KEEPALIVE = 0x02,
-  RUDP_OP_DATA      = 0x03,
-  RUDP_OP_FIN       = 0x04
+  RUDP_OP_SYN        = 0x00,
+  RUDP_OP_ACK        = 0x01,
+  RUDP_OP_KEEP_ALIVE = 0x02,
+  RUDP_OP_DATA       = 0x03,
+  RUDP_OP_FIN        = 0x04
 };
 
 /* Standardized RUDP FIN codes */
 enum rudp_fin_reason {
-  RUDP_FIN_CLOSE    = 0x00,
-  RUDP_FIN_ACK      = 0x01,
-  RUDP_FIN_TIMEOUT  = 0x02,
-  RUDP_FIN_TOO_BIG  = 0x03,
-  RUDP_FIN_TOO_BAD  = 0x04
+  RUDP_FIN_CLOSE     = 0x00,
+  RUDP_FIN_ACK       = 0x01,
+  RUDP_FIN_TIMEOUT   = 0x02,
+  RUDP_FIN_TOO_BIG   = 0x03,
+  RUDP_FIN_TOO_BAD   = 0x04
 };
 
 /* Raw layout of a RUDP packet header. This overlays/replaces the GUID area of
@@ -178,7 +178,7 @@ rudp_op_to_string(guint8 op)
 #define CASE(x) case ( RUDP_OP_ ##x ) : return #x;
     CASE(SYN)
     CASE(ACK)
-    CASE(KEEPALIVE)
+    CASE(KEEP_ALIVE)
     CASE(DATA)
     CASE(FIN)
 #undef CASE
@@ -910,7 +910,7 @@ rudp_handle_packet(const host_addr_t addr, guint16 port,
 	case RUDP_OP_SYN:
 		rudp_handle_syn(con, addr, port, rudp_header);
 		break;
-	case RUDP_OP_KEEPALIVE:
+	case RUDP_OP_KEEP_ALIVE:
 		rudp_handle_keep_alive(con, rudp_header);
 		break;
 	case RUDP_OP_ACK:
