@@ -1,5 +1,5 @@
 /*
- * $Id: rx.h 13504 2007-05-05 21:04:17Z rmanfredi $
+ * $Id: rx.h 16223 2009-03-20 21:48:03Z rmanfredi $
  *
  * Copyright (c) 2002-2003, Raphael Manfredi
  *
@@ -38,8 +38,8 @@
 
 #include "common.h" 
 
-#include "pmsg.h"
-#include "if/core/hosts.h"
+#include "lib/gnet_host.h"
+#include "lib/pmsg.h"
 
 struct rxdriver;
 struct gnutella_node;
@@ -56,7 +56,7 @@ typedef struct rxdriver {
 	const struct rxdrv_ops *ops;	/**< Dynamically dispatched operations */
 	struct rxdriver *upper;			/**< Layer above, NULL if none */
 	struct rxdriver *lower;			/**< Layer underneath, NULL if none */
-	gint flags;						/**< Driver flags */
+	int flags;						/**< Driver flags */
 	rx_data_t data_ind;				/**< Data indication routine */
 	gpointer opaque;				/**< Used by heirs to store specific info */
 } rxdrv_t;
