@@ -1,5 +1,5 @@
 /*
- * $Id: rxbuf.h 12496 2006-11-26 09:22:49Z cbiere $
+ * $Id: rxbuf.h 15557 2008-08-10 08:00:42Z rmanfredi $
  *
  * Copyright (c) 2002-2003, Raphael Manfredi
  *
@@ -37,7 +37,7 @@
 #define _core_rxbuf_h_
 
 #include "common.h"
-#include "pmsg.h"
+#include "lib/pmsg.h"
 
 /*
  * Public interface
