@@ -1,5 +1,5 @@
 /*
- * $Id: settings.c 16133 2009-03-13 13:00:20Z rmanfredi $
+ * $Id: settings.c 17157 2009-11-01 22:41:55Z rmanfredi $
  *
  * Copyright (c) 2001-2003, Raphael Manfredi, Richard Eckart
  *
@@ -54,6 +54,8 @@
 #include "upload_stats.h"
 #include "routing.h"			/* For gnet_reset_guid() */
 #include "ipp_cache.h"
+#include "ctl.h"
+#include "pdht.h"
 
 #include "if/gnet_property.h"
 #include "if/gnet_property_priv.h"
@@ -65,16 +67,23 @@
 
 #include "lib/bit_array.h"
 #include "lib/cq.h"
+#include "lib/compat_misc.h"
 #include "lib/debug.h"
+#include "lib/fd.h"
 #include "lib/file.h"
 #include "lib/getphysmemsize.h"
 #include "lib/glib-missing.h"
+#include "lib/halloc.h"
+#include "lib/palloc.h"
+#include "lib/parse.h"
 #include "lib/sha1.h"
 #include "lib/tm.h"
+#include "lib/vmm.h"
+#include "lib/zalloc.h"
 
 #include "lib/override.h"		/* Must be the last header included */
 
-RCSID("$Id: settings.c 16133 2009-03-13 13:00:20Z rmanfredi $")
+RCSID("$Id: settings.c 17157 2009-11-01 22:41:55Z rmanfredi $")
 
 static const char config_file[] = "config_gnet";
 static const char ul_stats_file[] = "upload_stats";
@@ -104,6 +113,7 @@ static prop_set_t *properties = NULL;
  */
 
 static const char pidfile[] = "gtk-gnutella.pid";
+static const char dirlockfile[] = ".gtk-gnutella.lock";
 
 static gboolean settings_init_running;
 
@@ -194,15 +204,14 @@ is_my_address_and_port(const host_addr_t addr, guint16 port)
  * Look for any existing PID file. If found, look at the pid recorded
  * there and make sure it has died. Abort operations if it hasn't...
  *
- * @returns On success a non-negative value is returned.
- *          If check_only was FALSE, the file descriptor of the pidfile
- *			is returned, if check_only was TRUE, zero is returned on success.
+ * @returns Returns zero on success and -1 on failure.
+ *          If fd_ptr is NULL the lock is only tested but not created.
  *			On failure errno is set to EEXIST, if the PID file was already
  *			locked. Other errno values imply that the PID file could not
  *			be created.
  */
 static int
-ensure_unicity(const char *file, gboolean check_only)
+ensure_unicity(const char *file, int *fd_ptr)
 {
 	gboolean locked = FALSE;
 	int fd;
@@ -211,12 +220,19 @@ ensure_unicity(const char *file, gboolean check_only)
 
 	fd = file_create(file, O_RDWR, PID_FILE_MODE);
 	if (fd < 0) {
-		if (!check_only) {
+		int saved_errno = errno;
+
+		if (fd_ptr || GNET_PROPERTY(lockfile_debug)) {
 			g_warning("could not create \"%s\": %s", file, g_strerror(errno));
 		}
+
+		errno = saved_errno;
 		return -1;
 	}
 
+	if (GNET_PROPERTY(lockfile_debug)) {
+		g_message("file \"%s\" opened", file);
+	}
 /* FIXME: These might be enums, a compile-time check would be better */
 #if defined(F_SETLK) && defined(F_WRLCK)
 	{
@@ -230,10 +246,16 @@ ensure_unicity(const char *file, gboolean check_only)
 		/* l_start and l_len are zero, which means the whole file is locked */
 
 		locking_failed = -1 == fcntl(fd, F_SETLK, &fl);
+
+		if (GNET_PROPERTY(lockfile_debug)) {
+			g_message("file \"%s\" fcntl-locking %s", file,
+				locking_failed ? "failed" : "succeeded");
+		}
+
 		if (locking_failed) {
 			int saved_errno = errno;
 
-			if (!check_only) {
+			if (fd_ptr || GNET_PROPERTY(lockfile_debug)) {
 				g_warning("fcntl(%d, F_SETLK, ...) failed for \"%s\": %s",
 					fd, file, g_strerror(saved_errno));
 				/*
@@ -247,7 +269,11 @@ ensure_unicity(const char *file, gboolean check_only)
 
 				if (-1 != fcntl(fd, F_GETLK, &fl)) {
 					g_warning("another gtk-gnutella process seems to "
-							"be still running (pid=%lu)", (gulong) fl.l_pid);
+							"be using \"%s\" (pid=%lu)",
+							file, (gulong) fl.l_pid);
+				} else {
+					g_warning("fcntl(%d, F_GETLK, ...) failed for \"%s\": %s",
+						fd, file, g_strerror(errno));
 				}
 			}
 
@@ -260,17 +286,23 @@ ensure_unicity(const char *file, gboolean check_only)
 	}
 #endif /* F_SETLK && F_WRLCK */
 
-	/* Maybe F_SETLK is not supported by the OS or filesystem,
-	 * fall back to weaker PID locking */
+	/*
+	 * Maybe F_SETLK is not supported by the OS or filesystem?
+	 * Fall back to weaker PID locking
+	 */
+
 	if (!locked) {
 		ssize_t r;
 		char buf[33];
 
+		if (GNET_PROPERTY(lockfile_debug)) {
+			g_message("file \"%s\" being read for PID", file);
+		}
 		r = read(fd, buf, sizeof buf - 1);
 		if ((ssize_t) -1 == r) {
 			/* This would be odd */
-			if (!check_only) {
-				g_warning("could not read pidfile \"%s\": %s",
+			if (fd_ptr || GNET_PROPERTY(lockfile_debug)) {
+				g_warning("could not read file \"%s\": %s",
 					file, g_strerror(errno));
 			}
 			goto failed;
@@ -290,10 +322,15 @@ ensure_unicity(const char *file, gboolean check_only)
 			if (!error && u > 1) {
 				pid_t pid = u;
 
+				if (GNET_PROPERTY(lockfile_debug)) {
+					g_message("file \"%s\" trying to send SIGZERO to PID %lu",
+						file, (unsigned long) pid);
+				}
+
 				if (0 == kill(pid, 0)) {
-					if (!check_only) {
+					if (fd_ptr) {
 						g_warning("another gtk-gnutella process seems to "
-							"be still running (pid=%lu)", (gulong) pid);
+							"be using \"%s\" (pid=%lu)", file, (gulong) pid);
 					}
 					goto failed;
 				}
@@ -301,51 +338,66 @@ ensure_unicity(const char *file, gboolean check_only)
 		}
 	}
 
-	if (check_only) {
+	if (GNET_PROPERTY(lockfile_debug)) {
+		g_message("file \"%s\" LOCKED (mode %s)",
+			file, fd_ptr ? "check" : "permanent");
+	}
+
+	if (NULL == fd_ptr) {
 		/*
 		 * We keep the empty PID file around. Otherwise,
 		 * there's a race-condition without fcntl() locking.
 		 */
-		close(fd);
-		return 0;
+		fd_close(&fd, TRUE);
+	} else {
+		/* Keep the fd open, otherwise the lock is lost */
+		*fd_ptr = fd;
 	}
 
-	/* Keep the fd open, otherwise the lock is lost */
-	return fd;
+	return 0;
 
 failed:
 
-	close(fd);
+	if (GNET_PROPERTY(lockfile_debug)) {
+		g_message("file \"%s\" NOT LOCKED", file);
+	}
+	fd_close(&fd, TRUE);
 	errno = EEXIST;
+	if (fd_ptr) {
+		*fd_ptr = -1;
+	}
 	return -1;
 }
 
 /**
- * Write our pid to the pidfile.
+ * Write our pid to the lockfile, opened as "fd".
  */
 static void
-save_pid(int fd)
+save_pid(int fd, const char *path)
 {
 	size_t len;
 	char buf[32];
 
-	g_assert(-1 != fd);
+	g_assert(fd >= 0);
 
-	gm_snprintf(buf, sizeof buf, "%lu\n", (gulong) getpid());
-	len = strlen(buf);
+	len = gm_snprintf(buf, sizeof buf, "%lu\n", (gulong) getpid());
 
+	if (GNET_PROPERTY(lockfile_debug)) {
+		g_message("file \"%s\" about to be written with PID %lu on fd #%d",
+			path, (gulong) getpid(), fd);
+	}
 	if (-1 == ftruncate(fd, 0))	{
-		g_warning("ftruncate() failed for pidfile: %s", g_strerror(errno));
+		g_warning("ftruncate() failed for \"%s\": %s",
+			path, g_strerror(errno));
 		return;
 	}
-
 	if (0 != lseek(fd, 0, SEEK_SET))	{
-		g_warning("lseek() failed for pidfile: %s", g_strerror(errno));
+		g_warning("lseek() failed for \"%s\": %s", path, g_strerror(errno));
 		return;
 	}
-
-	if (len != (size_t) write(fd, buf, len))
-		g_warning("could not flush pidfile: %s", g_strerror(errno));
+	if (len != (size_t) write(fd, buf, len)) {
+		g_warning("could not flush \"%s\": %s", path, g_strerror(errno));
+	}
 }
 
 /* ----------------------------------------- */
@@ -356,8 +408,8 @@ save_pid(int fd)
 void
 settings_early_init(void)
 {
-	config_dir = g_strdup(getenv("GTK_GNUTELLA_DIR"));
-	home_dir = g_strdup(eval_subst("~"));
+	config_dir = h_strdup(getenv("GTK_GNUTELLA_DIR"));
+	home_dir = h_strdup(eval_subst("~"));
 	if (home_dir) {
 		if (!is_absolute_path(home_dir)) {
 			g_error("$HOME must point to an absolute path!");
@@ -374,42 +426,85 @@ settings_early_init(void)
 	}
 }
 
+/**
+ * Make sure there is only one process leaving file "path/lockfile" around.
+ *
+ * @param path			the path where the lockfile is to be held
+ * @param lockfile		the basename of the locking file
+ * @param fd_ptr		if non-NULL, return the opened file descriptor here
+ *
+ * @return 0 if OK, -1 on error with errno set.
+ */
+static int
+settings_unique_usage(const char *path, const char *lockfile, int *fd_ptr)
+{
+	char *file;
+	int saved_errno, ret;
+
+	g_assert(path != NULL);
+	g_assert(lockfile != NULL);
+
+	file = make_pathname(path, lockfile);
+	ret = ensure_unicity(file, fd_ptr);
+	saved_errno = errno;
+
+	if (0 == ret && fd_ptr) {
+		save_pid(*fd_ptr, file);
+	}
+	HFREE_NULL(file);
+
+	errno = saved_errno;
+	/* The file descriptor must be kept open */
+	return ret;
+}
+
 /**
  * Tries to ensure that the current process is the only running instance
  * gtk-gnutella for the current value of GTK_GNUTELLA_DIR.
  *
- * @param check_only If TRUE, no warnings are emitted and a possibly created
- *                   PID file is automagically removed.
- * @returns On success zero is returned, otherwise a non-zero is returned
+ * @returns On success zero is returned, otherwise -1 is returned
  *			and errno is set.
  */
-int
-settings_ensure_unicity(gboolean check_only)
+static int
+settings_ensure_unicity(void)
 {
 	int fd;
 
 	g_assert(config_dir);
 
-	{
-		char *path;
-		int saved_errno;
-		
-		path = make_pathname(config_dir, pidfile);
-		fd = ensure_unicity(path, check_only);
-		saved_errno = errno;
-		G_FREE_NULL(path);
-		errno = saved_errno;
+	return settings_unique_usage(config_dir, pidfile, &fd);
+}
+
+int
+settings_is_unique_instance(void)
+{
+	g_assert(config_dir);
+
+	return settings_unique_usage(config_dir, pidfile, NULL) && EEXIST == errno;
+}
+
+/**
+ * Tries to ensure that the current process is the only writer in
+ * the directory where files are saved.
+ *
+ * @return 0 on success, -1 otherwise with errno set.
+ */
+static int
+settings_ensure_unique_save_file_path(void)
+{
+	static int save_file_path_lock = -1;
+	int fd;
+	int ret;
+
+	ret = settings_unique_usage(GNET_PROPERTY(save_file_path),
+				dirlockfile, &fd);
+
+	if (0 == ret) {
+		fd_close(&save_file_path_lock, TRUE);
+		save_file_path_lock = fd;
 	}
 
-	if (fd < 0) {
-		return -1;
-	}
-
-	if (!check_only) {
-		save_pid(fd);
-	}
-	/* The file descriptor must be kept open */
-	return 0;
+	return ret;
 }
 
 static void
@@ -470,12 +565,6 @@ settings_init(void)
 		}
 	}
 
-	/* Ensure this is the only instance running */
-	if (0 != settings_ensure_unicity(FALSE)) {
-		g_warning(_("You seem to have left another gtk-gnutella running\n"));
-		exit(EXIT_FAILURE);
-	}
-
 	/*
 	 * Parse the configuration.
 	 *
@@ -491,6 +580,18 @@ settings_init(void)
 		gnet_reset_guid();
 	}
 
+	/*
+	 * Ensure this is the only instance running.
+	 *
+	 * This is done after loading the configuration file to benefit from
+	 * the "lockfile_debug" property.
+	 */
+
+	if (0 != settings_ensure_unicity()) {
+		g_warning(_("You seem to have left another gtk-gnutella running"));
+		exit(EXIT_FAILURE);
+	}
+
 	if (debugging(0)) {
 		g_message("detected amount of physical RAM: %s",
 			short_size(memory, GNET_PROPERTY(display_metric_units)));
@@ -498,6 +599,8 @@ settings_init(void)
 			short_kb_size(amount, GNET_PROPERTY(display_metric_units)));
 		g_message("process can use %u file descriptors", max_fd);
 		g_message("max I/O vector size is %d items", MAX_IOV_COUNT);
+		g_message("virtual memory page size is %lu bytes",
+			(gulong) compat_pagesize());
 	}
 
 	{
@@ -505,7 +608,7 @@ settings_init(void)
 
 		path = make_pathname(config_dir, ul_stats_file);
 		upload_stats_load_history(path);	/* Loads the upload statistics */
-		G_FREE_NULL(path);
+		HFREE_NULL(path);
 	}
 
 
@@ -565,7 +668,7 @@ settings_ipc_dir(void)
 	if (!path) {
 		path = make_pathname(settings_config_dir(), "ipc");
 	}
-	return path;
+	return NOT_LEAKING(path);
 
 }
 /**
@@ -579,7 +682,7 @@ settings_local_socket_path(void)
 	if (!path) {
 		path = make_pathname(settings_ipc_dir(), "socket");
 	}
-	return path;
+	return NOT_LEAKING(path);
 }
 
 /**
@@ -604,21 +707,22 @@ settings_dns_net(void)
 }
 
 /**
- * Remove pidfile.
+ * Remove "path/lockfile".
  */
 static void
-settings_remove_pidfile(void)
+settings_remove_lockfile(const char *path, const char *lockfile)
 {
-	char *path;
+	char *file;
 
-	g_assert(config_dir);
+	g_return_if_fail(!is_null_or_empty(path));
+	g_return_if_fail(lockfile);
 
-	path = make_pathname(config_dir, pidfile);
-	g_return_if_fail(NULL != path);
-	if (-1 == unlink(path))
-		g_warning("could not remove pidfile \"%s\": %s",
-			path, g_strerror(errno));
-	G_FREE_NULL(path);
+	file = make_pathname(path, lockfile);
+	if (-1 == unlink(file)) {
+		g_warning("could not remove lockfile \"%s\": %s",
+			file, g_strerror(errno));
+	}
+	HFREE_NULL(file);
 }
 
 static void
@@ -837,11 +941,12 @@ settings_save_if_dirty(void)
 void
 settings_close(void)
 {
-	settings_remove_pidfile();
+	settings_remove_lockfile(config_dir, pidfile);
+	settings_remove_lockfile(GNET_PROPERTY(save_file_path), dirlockfile);
     gnet_prop_shutdown();
 
-	G_FREE_NULL(home_dir);
-	G_FREE_NULL(config_dir);
+	HFREE_NULL(home_dir);
+	HFREE_NULL(config_dir);
 }
 
 static void
@@ -883,6 +988,13 @@ gnet_get_bw_stats(gnet_bw_source type, gnet_bw_stats_t *s)
     case BW_LEAF_OUT:
 		bw_stats(s, GNET_PROPERTY(bws_glout_enabled), BSCHED_BWS_GLOUT);
 		return;
+    case BW_DHT_IN:
+		/* Cannot limit incoming DHT traffic (UDP) */
+		bw_stats(s, FALSE, BSCHED_BWS_DHT_IN);
+		return;
+    case BW_DHT_OUT:
+		bw_stats(s, GNET_PROPERTY(bws_dht_out_enabled), BSCHED_BWS_DHT_OUT);
+		return;
     }
 	g_assert_not_reached();
 }
@@ -992,6 +1104,13 @@ update_address_lifetime(void)
 	 */
 
 	remember_local_addr_port();
+
+	/*
+	 * If our address or port changed, we may have to republish our push
+	 * proxies to the DHT.
+	 */
+
+	pdht_prox_publish_if_changed();
 }
 
 /**
@@ -1089,6 +1208,7 @@ max_bad_hosts_cached_changed(property_t prop)
     hcache_prune(HCACHE_BUSY);
     hcache_prune(HCACHE_TIMEOUT);
     hcache_prune(HCACHE_UNSTABLE);
+    hcache_prune(HCACHE_ALIEN);
 
     return FALSE;
 }
@@ -1174,9 +1294,10 @@ enable_dht_changed(property_t prop)
 	
     gnet_prop_get_boolean_val(prop, &enabled);
 	if (enabled) {
+		/* Will start the DHT if UDP enabled otherwise */
 		dht_initialize(TRUE);
 	} else {
-		dht_close();
+		dht_close(FALSE);
 	}
 
 	return FALSE;
@@ -1349,6 +1470,7 @@ listen_port_changed(property_t prop)
 	if (!settings_init_running) {
 		inet_firewalled();
 		inet_udp_firewalled();
+		inet_udp_check_unsolicited();
 	}
 
 	/*
@@ -1426,6 +1548,12 @@ bw_gnet_lout_enabled_changed(property_t prop)
 	return bw_switch(prop, BSCHED_BWS_GLOUT);
 }
 
+static gboolean
+bw_dht_out_enabled_changed(property_t prop)
+{
+	return bw_switch(prop, BSCHED_BWS_DHT_OUT);
+}
+
 static gboolean
 node_sendqueue_size_changed(property_t unused_prop)
 {
@@ -1451,23 +1579,76 @@ scan_extensions_changed(property_t prop)
     return FALSE;
 }
 
+static int
+request_directory(const char *pathname)
+{
+	if (!is_absolute_path(pathname)) {
+		errno = EINVAL;
+		return -1;
+	}
+
+	if (is_directory(pathname))
+		return 0;
+
+	g_message("Attempt to create directory \"%s\"", pathname);
+
+	if (0 == create_directory(pathname, DEFAULT_DIRECTORY_MODE))
+		return 0;
+
+	g_message("Attempt failed: \"%s\"", g_strerror(errno));
+	return -1;
+}
+
 static gboolean
 file_path_changed(property_t prop)
 {
-    char *s;
+    char *pathname;
 
-	s = gnet_prop_get_string(prop, NULL, 0);
-	g_assert(s != NULL);
+	pathname = gnet_prop_get_string(prop, NULL, 0);
+	request_directory(pathname);
+    G_FREE_NULL(pathname);
+    return FALSE;
+}
 
-	if (!is_directory(s)) {
-		g_message("Attempt to create directory \"%s\"", s);
+static gboolean
+save_file_path_changed(property_t prop)
+{
+	static char *old_path;
+	char *path;
 
-		if (0 != create_directory(s, DEFAULT_DIRECTORY_MODE))
-			g_message("Attempt failed: \"%s\"", g_strerror(errno));
+	path = gnet_prop_get_string(prop, NULL, 0);
+
+	if (GNET_PROPERTY(lockfile_debug)) {
+		g_message("save_file_path_change(): path=\"%s\"\n\told_path=\"%s\"",
+			NULL_STRING(path), NULL_STRING(old_path));
 	}
 
-    G_FREE_NULL(s);
-    return FALSE;
+	if (
+		!is_null_or_empty(path) &&
+		(NULL == old_path || 0 != strcmp(path, old_path))
+	) {
+		gboolean failure = FALSE;
+
+		if (request_directory(path)) {
+			failure = TRUE;
+		} else if (settings_ensure_unique_save_file_path()) {
+			failure = TRUE;
+			gcu_statusbar_warning(
+				"Save path already used by another gtk-gnutella!");
+		}
+		if (failure) {
+			g_warning(
+				"not changing save file path to \"%s\", keeping old \"%s\"",
+					path, NULL_STRING(old_path));
+			gnet_prop_set_string(prop, old_path);
+			G_FREE_NULL(path);
+			return TRUE; /* Force changed value */
+		}
+	}
+	
+	G_FREE_NULL(old_path);
+	old_path = NOT_LEAKING(path);
+	return FALSE;
 }
 
 static gboolean
@@ -1487,6 +1668,17 @@ shared_dirs_paths_changed(property_t prop)
 	return FALSE;
 }
 
+static gboolean
+country_limits_changed(property_t prop)
+{
+    char *limits;
+
+	limits = gnet_prop_get_string(prop, NULL, 0);
+	ctl_parse(GNET_PROPERTY(ancient_version) ? NULL : limits);
+    G_FREE_NULL(limits);
+    return FALSE;
+}
+
 static gboolean
 local_netmasks_string_changed(property_t prop)
 {
@@ -1595,6 +1787,17 @@ bw_gnet_lout_changed(property_t prop)
     return FALSE;
 }
 
+static gboolean
+bw_dht_out_changed(property_t prop)
+{
+    guint32 val;
+
+    gnet_prop_get_guint32_val(prop, &val);
+    bsched_set_bandwidth(BSCHED_BWS_DHT_OUT, val);
+
+    return FALSE;
+}
+
 static gboolean
 bw_allow_stealing_changed(property_t prop)
 {
@@ -1621,6 +1824,50 @@ node_online_mode_changed(property_t prop)
     return FALSE;
 }
 
+static gboolean
+zalloc_always_gc_changed(property_t prop)
+{
+	gboolean val;
+
+	gnet_prop_get_boolean_val(prop, &val);
+	set_zalloc_always_gc(val);
+
+    return FALSE;
+}
+
+static gboolean
+zalloc_debug_changed(property_t prop)
+{
+	guint32 val;
+
+	gnet_prop_get_guint32_val(prop, &val);
+	set_zalloc_debug(val);
+
+    return FALSE;
+}
+
+static gboolean
+palloc_debug_changed(property_t prop)
+{
+	guint32 val;
+
+	gnet_prop_get_guint32_val(prop, &val);
+	set_palloc_debug(val);
+
+    return FALSE;
+}
+
+static gboolean
+vmm_debug_changed(property_t prop)
+{
+	guint32 val;
+
+	gnet_prop_get_guint32_val(prop, &val);
+	set_vmm_debug(val);
+
+    return FALSE;
+}
+
 static gboolean
 lib_debug_changed(property_t prop)
 {
@@ -1632,6 +1879,17 @@ lib_debug_changed(property_t prop)
     return FALSE;
 }
 
+static gboolean
+lib_stats_changed(property_t prop)
+{
+	guint32 val;
+
+	gnet_prop_get_guint32_val(prop, &val);
+	set_library_stats(val);
+
+    return FALSE;
+}
+
 static gboolean
 forced_local_ip_changed(property_t prop)
 {
@@ -1948,6 +2206,11 @@ static prop_map_t property_map[] = {
         bw_gnet_lout_enabled_changed,
         FALSE
     },
+    {
+        PROP_BW_DHT_OUT_ENABLED,
+        bw_dht_out_enabled_changed,
+        TRUE		/* Want it inited initially */
+    },
     {
         PROP_SCAN_EXTENSIONS,
         scan_extensions_changed,
@@ -1955,7 +2218,7 @@ static prop_map_t property_map[] = {
     },
     {
         PROP_SAVE_FILE_PATH,
-        file_path_changed,
+        save_file_path_changed,
         TRUE
     },
     {
@@ -1978,6 +2241,11 @@ static prop_map_t property_map[] = {
         local_netmasks_string_changed,
         TRUE
     },
+    {
+        PROP_COUNTRY_LIMITS,
+        country_limits_changed,
+        TRUE
+    },
     {
         PROP_HARD_TTL_LIMIT,
         hard_ttl_limit_changed,
@@ -2018,6 +2286,11 @@ static prop_map_t property_map[] = {
         bw_gnet_lout_changed,
         FALSE
     },
+    {
+        PROP_BW_DHT_OUT,
+        bw_dht_out_changed,
+        FALSE
+    },
     {
         PROP_BW_ALLOW_STEALING,
         bw_allow_stealing_changed,
@@ -2028,10 +2301,35 @@ static prop_map_t property_map[] = {
 		node_online_mode_changed,
 		TRUE						/* Need to call callback at init time */
 	},
+    {
+        PROP_VMM_DEBUG,
+        vmm_debug_changed,
+        TRUE
+    },
+    {
+        PROP_ZALLOC_DEBUG,
+        zalloc_debug_changed,
+        TRUE
+    },
+    {
+        PROP_PALLOC_DEBUG,
+        palloc_debug_changed,
+        TRUE
+    },
     {
         PROP_LIB_DEBUG,
         lib_debug_changed,
         TRUE
+    },
+    {
+        PROP_LIB_STATS,
+        lib_stats_changed,
+        TRUE
+    },
+    {
+        PROP_ZALLOC_ALWAYS_GC,
+        zalloc_always_gc_changed,
+        TRUE
     },
 	{
 		PROP_FORCE_LOCAL_IP,
@@ -2131,7 +2429,7 @@ static gboolean init_list[GNET_PROPERTY_NUM];
 static void
 settings_callbacks_init(void)
 {
-    guint n;
+    unsigned n;
 
     for (n = 0; n < GNET_PROPERTY_NUM; n ++)
         init_list[n] = FALSE;
@@ -2145,12 +2443,11 @@ settings_callbacks_init(void)
         property_t prop = property_map[n].prop;
         guint32 idx = prop - GNET_PROPERTY_MIN;
 
-        if (!init_list[idx])
-           init_list[idx] = TRUE;
-        else
-            g_warning("settings_callbacks_init:"
-                " property %d already mapped", n);
+        if (init_list[idx]) {
+            g_error("settings_callbacks_init: property %u already mapped", n);
+		}
 
+		init_list[idx] = TRUE;
         if (property_map[n].cb) {
             gnet_prop_add_prop_changed_listener(
                 property_map[n].prop,
