@@ -1,5 +1,5 @@
 /*
- * $Id: settings.h 16106 2009-03-10 15:16:31Z rmanfredi $
+ * $Id: settings.h 16720 2009-05-11 22:13:11Z cbiere $
  *
  * Copyright (c) 2001-2003, Raphael Manfredi, Richard Eckart
  *
@@ -57,7 +57,7 @@ gboolean is_my_address(const host_addr_t addr);
 gboolean is_my_address_and_port(const host_addr_t addr, guint16 port);
 
 void settings_early_init(void);
-int settings_ensure_unicity(gboolean silent);
+gboolean settings_is_unique_instance(void);
 void settings_init(void);
 void settings_save_if_dirty(void);
 void settings_shutdown(void);
