@@ -1,5 +1,5 @@
 /*
- * $Id: share.c 16106 2009-03-10 15:16:31Z rmanfredi $
+ * $Id: share.c 17240 2010-02-13 12:26:50Z rmanfredi $
  *
  * Copyright (c) 2001-2005, Raphael Manfredi
  * Copyright (c) 2000 Daniel Walker (dwalker@cats.ucsc.edu)
@@ -38,7 +38,7 @@
 
 #include "common.h"
 
-RCSID("$Id: share.c 16106 2009-03-10 15:16:31Z rmanfredi $")
+RCSID("$Id: share.c 17240 2010-02-13 12:26:50Z rmanfredi $")
 
 #include "share.h"
 #include "gmsg.h"
@@ -50,7 +50,6 @@ RCSID("$Id: share.c 16106 2009-03-10 15:16:31Z rmanfredi $")
 #include "ggep_type.h"
 #include "gnet_stats.h"
 #include "guid.h"
-#include "hostiles.h"
 #include "qhit.h"
 #include "oob.h"
 #include "oob_proxy.h"
@@ -58,6 +57,7 @@ RCSID("$Id: share.c 16106 2009-03-10 15:16:31Z rmanfredi $")
 #include "settings.h"
 #include "hosts.h"
 #include "upload_stats.h"
+#include "publisher.h"
 
 #include "if/gnet_property.h"
 #include "if/gnet_property_priv.h"
@@ -68,6 +68,7 @@ RCSID("$Id: share.c 16106 2009-03-10 15:16:31Z rmanfredi $")
 #include "lib/bg.h"
 #include "lib/endian.h"
 #include "lib/file.h"
+#include "lib/halloc.h"
 #include "lib/hashlist.h"
 #include "lib/listener.h"
 #include "lib/mime_type.h"
@@ -104,7 +105,7 @@ struct shared_file {
 
 	enum mime_type mime_type;	/* MIME type of the file */
 
-	int refcnt;				/**< Reference count */
+	int refcnt;					/**< Reference count */
 	guint32 flags;				/**< See below for definition */
 };
 
@@ -114,7 +115,7 @@ struct shared_file {
 struct special_file {
 	const char *path;			/* URL path */
 	const char *file;			/* File name to load from disk */
-	enum mime_type type;	/* MIME type of the file */
+	enum mime_type type;		/* MIME type of the file */
 	const char *what;			/* Description of the file for traces */
 };
 
@@ -147,7 +148,6 @@ static struct recursive_scan *recursive_scan_context;
  * This tree maps a SHA1 hash (base-32 encoded) onto the corresponding
  * shared_file if we have one.
  */
-
 static GTree *sha1_to_share;
 
 /**
@@ -268,6 +268,11 @@ shared_file_free(shared_file_t **sf_ptr)
 	}
 }
 
+/**
+ * Set canonic, NFC and NFKC normalized names.
+ *
+ * @return whether an error occurred
+ */
 static gboolean
 shared_file_set_names(shared_file_t *sf, const char *filename)
 {
@@ -277,13 +282,9 @@ shared_file_set_names(shared_file_t *sf, const char *filename)
 
 	/* Set the NFC normalized name. */
 	{	
-		char *name;
-
-		name = filename_to_utf8_normalized(filename, UNI_NORM_NETWORK);
+		char *name = filename_to_utf8_normalized(filename, UNI_NORM_NETWORK);
 		sf->name_nfc = atom_str_get(name);
-		if (name != filename) {
-			G_FREE_NULL(name);
-		}
+		G_FREE_NULL(name);
 	}
 
 	/*
@@ -305,7 +306,7 @@ shared_file_set_names(shared_file_t *sf, const char *filename)
 		name_canonic = UNICODE_CANONIZE(name);
 		sf->name_canonic = atom_str_get(name_canonic);
 		if (name_canonic != name) {
-			G_FREE_NULL(name_canonic);
+			HFREE_NULL(name_canonic);
 		}
 		if (name != sf->name_nfc) {
 			G_FREE_NULL(name);
@@ -371,16 +372,15 @@ share_special_load(const struct special_file *sp)
 	{
 		char *filename = make_pathname(fp[idx].dir, fp[idx].name);
 		sf->file_path = atom_str_get(filename);
-		G_FREE_NULL(filename);
+		HFREE_NULL(filename);
 	}
 	if (shared_file_set_names(sf, sp->file)) {
 		shared_file_free(&sf);
-		return NULL;
+	} else {
+		sf->mime_type = sp->type;
 	}
-	sf->mime_type = sp->type;
 
 	fclose(f);
-
 	return sf;
 }
 
@@ -762,15 +762,12 @@ get_relative_path(const char *base_dir, const char *pathname)
 	if (s) {
 		s = skip_dir_separators(s);
 		if ('\0' != s[0]) {
-			char *normalized, *nfc_str;
+			char *nfc_str;
 
-			normalized = normalize_dir_separators(s);
-			nfc_str = filename_to_utf8_normalized(normalized, UNI_NORM_NETWORK);
+			nfc_str = filename_to_utf8_normalized(s, UNI_NORM_NETWORK);
+			normalize_dir_separators(nfc_str);
 			relative_path = atom_str_get(nfc_str);
-			if (nfc_str != normalized) {
-				G_FREE_NULL(nfc_str);
-			}
-			G_FREE_NULL(normalized);
+			G_FREE_NULL(nfc_str);
 		}
 	}
 	return relative_path;
@@ -917,26 +914,6 @@ share_scan_add_file(const char *relative_path,
 	return sf;
 }
 
-/**
- * Tries to extrace the file mode from a struct dirent. Not all systems
- * support this, in which case zero is returned. Types other than regular
- * files, directories and symlinks are ignored and gain a value of zero
- * as well.
- */
-static mode_t
-dir_entry_mode(const struct dirent *dir_entry)
-{
-	g_assert(dir_entry);
-#ifdef HAS_DIRENT_D_TYPE
-	switch (dir_entry->d_type) {
-	case DT_DIR: return S_IFDIR;
-	case DT_LNK: return S_IFLNK;
-	case DT_REG: return S_IFREG;
-	}
-#endif	/* HAS_DIRENT_WITH_D_TYPE */
-	return 0;
-}
-
 /**
  * Check whether the given directory is one which should never be shared.
  * This is not meant to be exhaustive but test for common configuration
@@ -966,15 +943,16 @@ directory_is_unshareable(const char *dir)
 		return TRUE;
 	}
 
-	if (TRUE == is_same_file(dir, GNET_PROPERTY(save_file_path))) {
-		g_warning("Refusing to share directory for incomplete files: %s",
-			dir);
+	if (
+		!is_null_or_empty(GNET_PROPERTY(save_file_path)) &&
+		TRUE == is_same_file(dir, GNET_PROPERTY(save_file_path))
+	) {
+		g_warning("Refusing to share directory for incomplete files: %s", dir);
 		return TRUE;
 	}
 
 	if (TRUE == is_same_file(dir, GNET_PROPERTY(bad_file_path))) {
-		g_warning("Refusing to share directory for corrupted files: %s",
-			dir);
+		g_warning("Refusing to share directory for corrupted files: %s", dir);
 		return TRUE;
 	}
 
@@ -993,6 +971,8 @@ struct recursive_scan {
 	slist_t *base_dirs;			/* list of string atoms */
 	slist_t *sub_dirs;			/* list of g_malloc()ed strings */
 	slist_t *shared_files;		/* list of struct shared_file */
+	int idx;					/* iterating index */
+	int ticks;					/* ticks used */
 };
 
 static inline void
@@ -1046,6 +1026,16 @@ static void recursive_sf_unref(gpointer o)
 	shared_file_unref(&sf);
 }
 
+/**
+ * Encapsulation of hfree() in case TRACK_MALLOC is defined and hfree() is
+ * really a macro, not a function.
+ */
+static void
+do_hfree(void *p)
+{
+	hfree(p);
+}
+
 static void
 recursive_scan_free(struct recursive_scan **ctx_ptr)
 {
@@ -1064,7 +1054,7 @@ recursive_scan_free(struct recursive_scan **ctx_ptr)
 		recursive_scan_closedir(ctx);
 
 		slist_free_all(&ctx->base_dirs, (slist_destroy_cb) atom_str_free);
-		slist_free_all(&ctx->sub_dirs, g_free);
+		slist_free_all(&ctx->sub_dirs, do_hfree);
 		slist_free_all(&ctx->shared_files, recursive_sf_unref);
 
 		atom_str_free_null(&ctx->base_dir);
@@ -1118,8 +1108,8 @@ share_free(void)
 	g_slist_free(shared_files);
 	shared_files = NULL;
 
-	G_FREE_NULL(file_table);
-	G_FREE_NULL(sorted_file_table);
+	HFREE_NULL(file_table);
+	HFREE_NULL(sorted_file_table);
 }
 
 /**
@@ -1173,176 +1163,6 @@ shared_file_sort_by_name(const void *f1, const void *f2)
 	return 0 != ret ? ret : strcmp((*sf1)->name_nfc, (*sf2)->name_nfc);
 }
 
-/**
- * NOTE: This step is a rather big atomic block. Ideally, it should be broken
- * down in smaller steps to avoid temporary unresponsiveness. However, compared
- * to the time it takes to scan files, especially on slower disks, this
- * finishing step is neglible even for thousands of shared files.
- */
-static void
-recursive_scan_finish(struct recursive_scan *ctx)
-{
-	GSList *sl;
-	time_delta_t elapsed;
-	size_t i;
-
-	recursive_scan_check(ctx);
-
-	/*
-	 * Done scanning for the files. Now process them.
-	 */
-
-	elapsed = delta_time(tm_time_exact(),
-				GNET_PROPERTY(library_rescan_started));
-	elapsed = MAX(0, elapsed);
-	gnet_prop_set_timestamp_val(PROP_LIBRARY_RESCAN_FINISHED, tm_time());
-	gnet_prop_set_guint32_val(PROP_LIBRARY_RESCAN_DURATION, elapsed);
-
-	reinit_sha1_table();
-	share_free();
-
-	g_assert(file_basenames == NULL);
-	file_basenames = g_hash_table_new(g_str_hash, g_str_equal);
-
-	g_assert(search_table);
-	st_create(search_table);
-
-	files_scanned = slist_length(ctx->shared_files);
-	bytes_scanned = 0;
-
-	g_assert(shared_files == NULL);
-
-	while (slist_length(ctx->shared_files) > 0) {
-		struct shared_file *sf = slist_shift(ctx->shared_files);
-
-		shared_file_check(sf);
-		bytes_scanned += sf->file_size;
-		st_insert_item(search_table, sf->name_canonic, sf);
-		shared_files = g_slist_prepend(shared_files, sf);
-		upload_stats_enforce_local_filename(sf);
-	}
-
-	/* Compact the search table */
-	st_compact(search_table);
-
-	file_table = g_malloc0((files_scanned + 1) * sizeof file_table[0]);
-
-	/*
-	 * We over-allocate the file_table by one entry so that even when they
-	 * don't share anything, the `file_table' pointer is not NULL.
-	 * This will prevent us giving back "rebuilding library" when we should
-	 * actually return "not found" for user download requests.
-	 *		--RAM, 23/10/2002
-	 */
-	
-	i = 0;
-	for (sl = shared_files; sl; sl = g_slist_next(sl)) {
-		struct shared_file *sf = sl->data;
-
-		shared_file_check(sf);
-		g_assert(!(SHARE_F_INDEXED & sf->flags));
-		file_table[i++] = sf;
-	}
-
-	/* Sort file list by modification time to get a relatively stable index */
-	qsort(file_table, files_scanned, sizeof file_table[0],
-		shared_file_sort_by_mtime);
-
-	/*
-	 * In order to quickly locate files based on indicies, build a table
-	 * of all shared files.  This table is only accessible via shared_file().
-	 * NB: file indicies start at 1, but indexing in table start at 0.
-	 *		--RAM, 08/10/2001
-	 */
-
-	for (i = 0; i < files_scanned; i++) {
-		struct shared_file *sf;
-		guint val;
-
-		/* FIXME: Must be handled in the background */
-
-	   	sf = file_table[i];
-		if (!sf)
-			continue;
-
-		/* Set file_index based on new sort order */
-		sf->file_index = i + 1;
-		sf->flags |= SHARE_F_INDEXED;
-		shared_file_check(sf);
-
-		/* We must not change the file index after request_sha1() */
-		if (!request_sha1(sf)) {
-			file_table[i] = NULL;
-			continue;
-		}
-
-		/*
-		 * In order to transparently handle files requested with the wrong
-		 * indices, for older servents that would not know how to handle a
-		 * return code of "301 Moved" with a Location header, we keep track
-		 * of individual basenames of files, recording the index of each file.
-		 * As soon as there is a clash, we revoke the entry by storing
-		 * FILENAME_CLASH instead, which cannot be a valid index.
-		 *		--RAM, 06/06/2002
-		 */
-
-		val = GPOINTER_TO_UINT(
-			g_hash_table_lookup(file_basenames, sf->name_nfc));
-
-		/*
-		 * The following works because 0 cannot be a valid file index.
-		 */
-
-		val = (val != 0) ? FILENAME_CLASH : sf->file_index;
-		g_hash_table_insert(file_basenames, deconstify_gchar(sf->name_nfc),
-			GUINT_TO_POINTER(val));
-		sf->flags |= SHARE_F_BASENAME;
-	}
-
-	sorted_file_table = g_memdup(file_table,
-							(files_scanned + 1) * sizeof file_table[0]);
-	qsort(sorted_file_table, files_scanned, sizeof sorted_file_table[0],
-		shared_file_sort_by_name);
-
-	/* Set the sort index used for sorted file listings */
-	for (i = 0; i < files_scanned; i++) {
-		struct shared_file *sf;
-
-	   	sf = sorted_file_table[i];
-		if (!sf)
-			continue;
-		shared_file_check(sf);
-		sf->sort_index = i + 1;
-	}
-
-	gcu_gui_update_files_scanned();		/* Final view */
-
-	/*
-	 * Query routing table update.
-	 */
-
-	gnet_prop_set_timestamp_val(PROP_QRP_INDEXING_STARTED, tm_time_exact());
-
-	qrp_prepare_computation();
-
-	i = 0;
-	for (sl = shared_files; sl; sl = g_slist_next(sl)) {
-		struct shared_file *sf = sl->data;
-
-		/* FIXME: Must be handled in the background */
-		shared_file_check(sf);
-		qrp_add_file(sf);
-	}
-
-	qrp_finalize_computation();
-
-	elapsed = delta_time(tm_time(), GNET_PROPERTY(qrp_indexing_started));
-	elapsed = MAX(0, elapsed);
-	gnet_prop_set_guint32_val(PROP_QRP_INDEXING_DURATION, elapsed);
-
-	gnet_prop_set_boolean_val(PROP_LIBRARY_REBUILDING, FALSE);
-}
-
 static void
 recursive_scan_opendir(struct recursive_scan *ctx, const char * const dir)
 {
@@ -1391,6 +1211,20 @@ recursive_scan_readdir(struct recursive_scan *ctx)
 		}
 
 		sb.st_mode = dir_entry_mode(dir_entry);
+		switch (sb.st_mode) {
+		case 0:
+		case S_IFREG:
+		case S_IFDIR:
+		case S_IFLNK:
+			break;
+		default:
+			if (GNET_PROPERTY(share_debug)) {
+				g_warning("Skipping file of unknown type \"%s\" in \"%s\"",
+					ctx->current_dir, dir_entry->d_name);
+			}
+			goto finish;
+		}
+
 		if (
 			S_ISLNK(sb.st_mode) &&
 			GNET_PROPERTY(scan_ignore_symlink_dirs) &&
@@ -1406,6 +1240,8 @@ recursive_scan_readdir(struct recursive_scan *ctx)
 			goto finish;
 		}
 
+		ctx->ticks += 10;	/* Heavier work */
+
 		fullpath = make_pathname(ctx->current_dir, dir_entry->d_name);
 		if (S_ISREG(sb.st_mode) || S_ISDIR(sb.st_mode)) {
 			if (stat(fullpath, &sb)) {
@@ -1472,7 +1308,7 @@ recursive_scan_readdir(struct recursive_scan *ctx)
 	}
 
 finish:
-	G_FREE_NULL(fullpath);
+	HFREE_NULL(fullpath);
 }
 
 /**
@@ -1491,7 +1327,7 @@ recursive_scan_next_dir(struct recursive_scan *ctx)
 	   
 		dir = slist_shift(ctx->sub_dirs);
 		recursive_scan_opendir(ctx, dir);
-		G_FREE_NULL(dir);
+		HFREE_NULL(dir);
 		return FALSE;
 	} else if (slist_length(ctx->base_dirs) > 0) {
 		atom_str_free_null(&ctx->base_dir);
@@ -1508,20 +1344,292 @@ static bgret_t
 recursive_scan_step_compute(struct bgtask *bt, void *data, int ticks)
 {
 	struct recursive_scan *ctx = data;
-	int i = 0;
+
+	recursive_scan_check(ctx);
+
+	ctx->ticks = 0;
+	do {
+		if (recursive_scan_next_dir(ctx)) {
+			bg_task_ticks_used(bt, ctx->ticks);
+			return BGR_NEXT;
+		}
+		ctx->ticks++;
+	} while (ctx->ticks < ticks);
+
+	return BGR_MORE;
+}
+
+static bgret_t
+recursive_scan_step_compute_done(struct bgtask *bt, void *data, int ticks)
+{
+	struct recursive_scan *ctx = data;
 
 	recursive_scan_check(ctx);
 	(void) ticks;
-	(void) bt;
 
-	do {
-		if (recursive_scan_next_dir(ctx)) {
-			recursive_scan_finish(ctx);
-			return BGR_DONE;
+	reinit_sha1_table();
+	share_free();
+
+	g_assert(file_basenames == NULL);
+	file_basenames = g_hash_table_new(g_str_hash, g_str_equal);
+
+	g_assert(search_table);
+	st_create(search_table);
+
+	files_scanned = slist_length(ctx->shared_files);
+	bytes_scanned = 0;
+
+	g_assert(shared_files == NULL);
+
+	bg_task_ticks_used(bt, 0);
+	return BGR_NEXT;
+}
+
+static bgret_t
+recursive_scan_step_build_search_table(struct bgtask *bt, void *data, int ticks)
+{
+	struct recursive_scan *ctx = data;
+
+	recursive_scan_check(ctx);
+
+	ctx->ticks = 0;
+
+	while (slist_length(ctx->shared_files) > 0) {
+		struct shared_file *sf;
+
+		if (ctx->ticks++ >= ticks)
+			return BGR_MORE;
+
+		sf = slist_shift(ctx->shared_files);
+		shared_file_check(sf);
+		bytes_scanned += sf->file_size;
+		st_insert_item(search_table, sf->name_canonic, sf);
+		shared_files = g_slist_prepend(shared_files, sf);
+		upload_stats_enforce_local_filename(sf);
+	}
+
+	/* Compact the search table */
+	st_compact(search_table);
+	ctx->ticks += 5;
+
+	bg_task_ticks_used(bt, ctx->ticks);
+	return BGR_NEXT;
+}
+
+static bgret_t
+recursive_scan_step_build_file_table(struct bgtask *bt, void *data, int ticks)
+{
+	struct recursive_scan *ctx = data;
+	GSList *sl;
+	int i;
+
+	recursive_scan_check(ctx);
+	(void) ticks;
+
+	file_table = halloc0((files_scanned + 1) * sizeof file_table[0]);
+
+	/*
+	 * We over-allocate the file_table by one entry so that even when they
+	 * don't share anything, the `file_table' pointer is not NULL.
+	 * This will prevent us giving back "rebuilding library" when we should
+	 * actually return "not found" for user download requests.
+	 *		--RAM, 23/10/2002
+	 */
+	
+	for (i = 0, sl = shared_files; sl; sl = g_slist_next(sl)) {
+		struct shared_file *sf = sl->data;
+
+		shared_file_check(sf);
+		g_assert(!(SHARE_F_INDEXED & sf->flags));
+		file_table[i++] = sf;
+	}
+
+	/* Sort file list by modification time to get a relatively stable index */
+	qsort(file_table, files_scanned, sizeof file_table[0],
+		shared_file_sort_by_mtime);
+
+	bg_task_ticks_used(bt, i / 10);
+	ctx->idx = 0;				/* Prepares next step */
+
+	return BGR_NEXT;
+}
+
+static bgret_t
+recursive_scan_step_request_sha1(struct bgtask *bt, void *data, int ticks)
+{
+	struct recursive_scan *ctx = data;
+
+	recursive_scan_check(ctx);
+
+	/*
+	 * In order to quickly locate files based on indicies, build a table
+	 * of all shared files.  This table is only accessible via shared_file().
+	 * NB: file indicies start at 1, but indexing in table start at 0.
+	 *		--RAM, 08/10/2001
+	 */
+
+	ctx->ticks = 0;
+
+	for (/* empty */; UNSIGNED(ctx->idx) < files_scanned; ctx->idx++) {
+		struct shared_file *sf;
+		guint val;
+		int i = ctx->idx;
+
+	   	sf = file_table[i];
+		if (!sf)
+			continue;
+
+		if (ctx->ticks++ >= ticks)
+			return BGR_MORE;
+
+		/* Set file_index based on new sort order */
+		sf->file_index = i + 1;
+		sf->flags |= SHARE_F_INDEXED;
+		shared_file_check(sf);
+
+		/* We must not change the file index after request_sha1() */
+		if (!request_sha1(sf)) {
+			file_table[i] = NULL;
+			continue;
 		}
-	} while (++i < ticks);
 
-	return BGR_MORE;
+		/*
+		 * In order to transparently handle files requested with the wrong
+		 * indices, for older servents that would not know how to handle a
+		 * return code of "301 Moved" with a Location header, we keep track
+		 * of individual basenames of files, recording the index of each file.
+		 * As soon as there is a clash, we revoke the entry by storing
+		 * FILENAME_CLASH instead, which cannot be a valid index.
+		 *		--RAM, 06/06/2002
+		 */
+
+		val = GPOINTER_TO_UINT(
+			g_hash_table_lookup(file_basenames, sf->name_nfc));
+
+		/*
+		 * The following works because 0 cannot be a valid file index.
+		 */
+
+		val = (val != 0) ? FILENAME_CLASH : sf->file_index;
+		g_hash_table_insert(file_basenames, deconstify_gchar(sf->name_nfc),
+			GUINT_TO_POINTER(val));
+		sf->flags |= SHARE_F_BASENAME;
+	}
+
+	bg_task_ticks_used(bt, ctx->ticks);
+	return BGR_NEXT;
+}
+
+static bgret_t
+recursive_scan_step_update_scan_timing(struct bgtask *bt, void *data, int ticks)
+{
+	struct recursive_scan *ctx = data;
+	time_delta_t elapsed;
+
+	recursive_scan_check(ctx);
+	(void) ticks;
+
+	elapsed = delta_time(tm_time_exact(),
+				GNET_PROPERTY(library_rescan_started));
+	elapsed = MAX(0, elapsed);
+	gnet_prop_set_timestamp_val(PROP_LIBRARY_RESCAN_FINISHED, tm_time());
+	gnet_prop_set_guint32_val(PROP_LIBRARY_RESCAN_DURATION, elapsed);
+
+	bg_task_ticks_used(bt, 0);
+	return BGR_NEXT;
+}
+
+static bgret_t
+recursive_scan_step_build_sorted_table(struct bgtask *bt, void *data, int ticks)
+{
+	struct recursive_scan *ctx = data;
+	int i;
+
+	recursive_scan_check(ctx);
+	(void) ticks;
+
+	sorted_file_table = hcopy(file_table,
+							(files_scanned + 1) * sizeof file_table[0]);
+	qsort(sorted_file_table, files_scanned, sizeof sorted_file_table[0],
+		shared_file_sort_by_name);
+
+	/* Set the sort index used for sorted file listings */
+	for (i = 0; UNSIGNED(i) < files_scanned; i++) {
+		struct shared_file *sf;
+
+	   	sf = sorted_file_table[i];
+		if (!sf)
+			continue;
+		shared_file_check(sf);
+		sf->sort_index = i + 1;
+	}
+
+	gcu_gui_update_files_scanned();		/* Final view */
+
+	bg_task_ticks_used(bt, files_scanned / 10);
+	return BGR_NEXT;
+}
+
+static bgret_t
+recursive_scan_step_prepare_qrp(struct bgtask *bt, void *data, int ticks)
+{
+	struct recursive_scan *ctx = data;
+
+	recursive_scan_check(ctx);
+	(void) ticks;
+
+	gnet_prop_set_timestamp_val(PROP_QRP_INDEXING_STARTED, tm_time_exact());
+	qrp_prepare_computation();
+	ctx->idx = 0;
+
+	bg_task_ticks_used(bt, 0);
+	return BGR_NEXT;
+}
+
+static bgret_t
+recursive_scan_step_update_qrp(struct bgtask *bt, void *data, int ticks)
+{
+	struct recursive_scan *ctx = data;
+
+	recursive_scan_check(ctx);
+
+	ctx->ticks = 0;
+
+	for (/* empty */; UNSIGNED(ctx->idx) < files_scanned; ctx->idx++) {
+		struct shared_file *sf = sorted_file_table[ctx->idx];
+
+		if (!sf)
+			continue;
+
+		if (ctx->ticks++ >= ticks)
+			return BGR_MORE;
+
+		shared_file_check(sf);
+		qrp_add_file(sf);
+	}
+
+	bg_task_ticks_used(bt, ctx->ticks);
+	return BGR_NEXT;
+}
+
+static bgret_t
+recursive_scan_step_finalize(struct bgtask *bt, void *data, int ticks)
+{
+	struct recursive_scan *ctx = data;
+	time_delta_t elapsed;
+
+	recursive_scan_check(ctx);
+	(void) bt;
+	(void) ticks;
+
+	qrp_finalize_computation();
+
+	elapsed = delta_time(tm_time(), GNET_PROPERTY(qrp_indexing_started));
+	elapsed = MAX(0, elapsed);
+	gnet_prop_set_guint32_val(PROP_QRP_INDEXING_DURATION, elapsed);
+
+	gnet_prop_set_boolean_val(PROP_LIBRARY_REBUILDING, FALSE);
+	return BGR_DONE;
 }
 
 static void
@@ -1530,10 +1638,21 @@ recursive_scan_create_task(struct recursive_scan *ctx)
 	recursive_scan_check(ctx);
 
 	if (NULL == ctx->task) {
-		static const bgstep_cb_t step[] = { recursive_scan_step_compute };
+		static const bgstep_cb_t steps[] = {
+			recursive_scan_step_compute,
+			recursive_scan_step_compute_done,
+			recursive_scan_step_build_search_table,
+			recursive_scan_step_build_file_table,
+			recursive_scan_step_request_sha1,
+			recursive_scan_step_update_scan_timing,
+			recursive_scan_step_build_sorted_table,
+			recursive_scan_step_prepare_qrp,
+			recursive_scan_step_update_qrp,
+			recursive_scan_step_finalize,
+		};
 
 		ctx->task = bg_task_create("recursive scan",
-							step, G_N_ELEMENTS(step),
+							steps, G_N_ELEMENTS(steps),
 			  				ctx, recursive_scan_context_free,
 							NULL, NULL);
 	}
@@ -1639,6 +1758,7 @@ shared_file_set_sha1(struct shared_file *sf, const struct sha1 *sha1)
 	 * put into the tree again. This might happen if a SHA-1 calculation
 	 * from a previous rescan finishes after newly initiated rescan.
 	 */
+
 	if ((SHARE_F_INDEXED & sf->flags) && sf->sha1) {
 		struct shared_file *current;
 		gpointer key;
@@ -1659,7 +1779,13 @@ shared_file_set_sha1(struct shared_file *sf, const struct sha1 *sha1)
 					shared_file_path(current));
 			}
 		} else {
+			/*
+			 * New SHA-1 known for this file entry.
+			 * Record in the set of shared SHA-1s and publish to the DHT.
+			 */
+		
 			g_tree_insert(sha1_to_share, deconstify_gpointer(sf->sha1), sf);
+			publisher_add(sf->sha1);
 		}
 	}
 }
