@@ -1,5 +1,5 @@
 /*
- * $Id: sockets.c 16285 2009-03-28 21:18:01Z rmanfredi $
+ * $Id: sockets.c 17130 2009-10-28 11:29:04Z rmanfredi $
  *
  * Copyright (c) 2001-2003, Raphael Manfredi
  * Copyright (c) 2000 Daniel Walker (dwalker@cats.ucsc.edu)
@@ -38,7 +38,7 @@
 
 #include "common.h"
 
-RCSID("$Id: sockets.c 16285 2009-03-28 21:18:01Z rmanfredi $")
+RCSID("$Id: sockets.c 17130 2009-10-28 11:29:04Z rmanfredi $")
 
 #ifdef I_NETDB
 #include <netdb.h>
@@ -63,6 +63,8 @@ RCSID("$Id: sockets.c 16285 2009-03-28 21:18:01Z rmanfredi $")
 #include "settings.h"
 #include "ipp_cache.h"
 #include "gnet_stats.h"
+#include "geo_ip.h"
+#include "ctl.h"
 
 #include "shell/shell.h"
 
@@ -72,10 +74,14 @@ RCSID("$Id: sockets.c 16285 2009-03-28 21:18:01Z rmanfredi $")
 #include "lib/socket.h"
 #include "lib/adns.h"
 #include "lib/ascii.h"
+#include "lib/fd.h"
 #include "lib/getline.h"
 #include "lib/glib-missing.h"
 #include "lib/endian.h"
+#include "lib/halloc.h"
 #include "lib/header.h"
+#include "lib/stringify.h"
+#include "lib/timestamp.h"
 #include "lib/tm.h"
 #include "lib/walloc.h"
 
@@ -131,7 +137,7 @@ socket_alloc_buffer(struct gnutella_socket *s)
 
 	if (!s->buf) {
 		s->buf_size = SOCK_BUFSZ;
-		s->buf = g_malloc(s->buf_size);
+		s->buf = halloc(s->buf_size);
 	}
 }
 
@@ -143,7 +149,7 @@ socket_free_buffer(struct gnutella_socket *s)
 
 	if (s->buf) {
 		s->buf_size = 0;
-		G_FREE_NULL(s->buf);
+		HFREE_NULL(s->buf);
 	}
 }
 
@@ -179,6 +185,28 @@ socket_ipv6_trt_map(const host_addr_t addr)
 	return addr;
 }
 
+/**
+ * Stringify a socket connection type.
+ */
+static const char *
+socket_type_to_string(enum socket_type type)
+{
+	switch (type) {
+	case SOCK_TYPE_UNKNOWN:		return "unknown";
+	case SOCK_TYPE_CONTROL:		return "Gnet";
+	case SOCK_TYPE_DOWNLOAD:	return "HTTP-download";
+	case SOCK_TYPE_UPLOAD:		return "HTTP-upload";
+	case SOCK_TYPE_HTTP:		return "HTTP";
+	case SOCK_TYPE_SHELL:		return "shell";
+	case SOCK_TYPE_CONNBACK:	return "Gnet-connect-back";
+	case SOCK_TYPE_PPROXY:		return "HTTP-push-proxy";
+	case SOCK_TYPE_DESTROYING:	return "(destroying)";
+	case SOCK_TYPE_UDP:			return "UDP";
+	}
+
+	return "?";
+}
+
 /**
  * Return the file descriptor to use for I/O monitoring callbacks on
  * the socket.
@@ -240,8 +268,8 @@ socket_evt_set(struct gnutella_socket *s,
 	s->tls.cb_data = data;
 
 	if (GNET_PROPERTY(tls_debug) > 4) {
-		g_message("socket_evt_set: fd=%d, cond=%s, handler=%p",
-			fd, inputevt_cond_to_string(cond), handler);
+		g_message("socket_evt_set: fd=%d, cond=%s",
+			fd, inputevt_cond_to_string(cond));
 	}
 	s->gdk_tag = inputevt_add(fd, cond, handler, data);
 	g_assert(0 != s->gdk_tag);
@@ -264,8 +292,8 @@ socket_evt_clear(struct gnutella_socket *s)
 	if (s->gdk_tag) {
 		if (GNET_PROPERTY(tls_debug) > 4) {
 			int fd = socket_evt_fd(s);
-			g_message("socket_evt_clear: fd=%d, cond=%s, handler=%p",
-				fd, inputevt_cond_to_string(s->tls.cb_cond), s->tls.cb_handler);
+			g_message("socket_evt_clear: fd=%d, cond=%s",
+				fd, inputevt_cond_to_string(s->tls.cb_cond));
 		}
 
 		s->tls.cb_cond = 0;
@@ -1053,8 +1081,10 @@ connect_socksv5(struct gnutella_socket *s)
 				g_strerror(errno));
 			return ECONNREFUSED;
 		}
-		if (GNET_PROPERTY(socket_debug))
-			g_message("connect_socksv5: Step 5, bytes recv'd %d\n", (int) ret);
+		if (GNET_PROPERTY(socket_debug)) {
+			g_message("connect_socksv5: Step 5, bytes recv'd %lu\n",
+				(unsigned long) ret);
+		}
 		if ((size_t) ret != size) {
 			g_warning("Short reply from SOCKS server");
 			return ECONNREFUSED;
@@ -2152,14 +2182,27 @@ socket_accept(gpointer data, int unused_source, inputevt_cond_t cond)
 		t->addr = socket_addr_get_addr(&addr);
 		t->port = socket_addr_get_port(&addr);
 		if (!is_host_addr(t->addr)) {
-			g_warning("Incoming TCP connection from unidentifiable source");
+			g_warning("incoming TCP connection from unidentifiable source");
 			socket_free_null(&t);
 			return;
 		}
-		g_warning("Had to use getpeername() after accept(): peer=%s",
+		g_warning("had to use getpeername() after accept(): peer=%s",
 			host_addr_port_to_string(t->addr, t->port));
 	}
 
+	if (
+		(t->flags & SOCK_F_TCP) &&
+		ctl_limit(t->addr, CTL_S_ANY_TCP | CTL_D_STEALTH)
+	) {
+		if (GNET_PROPERTY(ctl_debug) > 2) {
+			g_message("CTL closing incoming TCP connection from %s [%s]",
+				host_addr_port_to_string(t->addr, t->port),
+				gip_country_cc(t->addr));
+		}
+		socket_free_null(&t);
+		return;
+	}
+
 	t->tls.enabled = s->tls.enabled; /* Inherit from listening socket */
 	t->tls.stage = SOCK_TLS_NONE;
 	t->tls.ctx = NULL;
@@ -2368,7 +2411,13 @@ socket_udp_accept(struct gnutella_socket *s)
 
 	g_assert((size_t) r <= s->buf_size);
 
-	bws_udp_count_read(r);
+	/*
+	 * We're too low level to account for the proper bandwidth here as we
+	 * want to distinguish between UDP Gnutella traffic and DHT traffic.
+	 *
+	 * This will be done in udp_receieved() which we're about to call.
+	 */
+
 	s->pos = r;
 
 	/*
@@ -2380,6 +2429,7 @@ socket_udp_accept(struct gnutella_socket *s)
 
 	if (!is_host_addr(s->addr)) {
 		gnet_stats_count_general(GNR_UDP_BOGUS_SOURCE_IP, 1);
+		bws_udp_count_read(r, FALSE);	/* Assume not from DHT */
 		errno = EINVAL;
 		return (ssize_t) -1;
 	}
@@ -2389,9 +2439,11 @@ socket_udp_accept(struct gnutella_socket *s)
 
 		settings_addr_changed(dst_addr, s->addr);
 
-		/* Show the destination address only when it differs from
+		/*
+		 * Show the destination address only when it differs from
 		 * the last seen or if the debug level is higher than 1.
 		 */
+
 		if (
 			GNET_PROPERTY(socket_debug) > 1 ||
 			!host_addr_equal(last_addr, dst_addr)
@@ -2420,7 +2472,7 @@ socket_udp_event(gpointer data, int unused_source, inputevt_cond_t cond)
 	struct gnutella_socket *s = data;
 	size_t avail, rd;
 	unsigned i;
-	tm_t start;
+	tm_t start, end;
 
 	(void) unused_source;
 
@@ -2439,8 +2491,7 @@ socket_udp_event(gpointer data, int unused_source, inputevt_cond_t cond)
 	 * as there are often several packets queued.
 	 */
 
-	if (GNET_PROPERTY(socket_debug))
-		tm_now_exact(&start);
+	tm_now_exact(&start);
 
 	avail = inputevt_data_available();
 	avail = (avail != 0) ? avail : 64 * 1024;
@@ -2468,14 +2519,18 @@ socket_udp_event(gpointer data, int unused_source, inputevt_cond_t cond)
 		 * it refers to header or control msg data. */
 		if (avail <= 32)
 			break;
+
+		/* Do not monopolize CPU for too long -- break out after 150 ms */
+		tm_now_exact(&end);
+		if (tm_elapsed_ms(&end, &start) > 150)
+			break;
+
 	} while (i < MAX_UDP_RECV_LOOP);
 
 	if (i > 16 && GNET_PROPERTY(socket_debug)) {
-		tm_t end;
-		tm_now_exact(&end);
 		g_message(
-			"socket_udp_event() iterated %u times, read %lu bytes in %lf secs",
-			i, (unsigned long) rd, tm_elapsed_f(&end, &start));
+			"socket_udp_event() iterated %u times, read %lu bytes in %u usecs",
+			i, (unsigned long) rd, (unsigned) tm_elapsed_us(&end, &start));
 	}
 }
 
@@ -2604,6 +2659,52 @@ socket_set_quickack(struct gnutella_socket *s, int on)
  * Sockets creation
  */
 
+/**
+ * Verify that connection can be made to an addr.
+ * @return 0 if OK.
+ */
+static int
+socket_connection_allowed(const host_addr_t addr, enum socket_type type)
+{
+	unsigned flag = 0;
+
+	if (hostiles_check(addr)) {
+		if (GNET_PROPERTY(socket_debug)) {
+			g_warning("not connecting [%s] to hostile host %s",
+				socket_type_to_string(type), host_addr_to_string(addr));
+		}
+		errno = EPERM;
+		return -1;
+	}
+
+	switch (type) {
+	case SOCK_TYPE_DOWNLOAD:	flag = CTL_D_OUTGOING; break;
+	case SOCK_TYPE_HTTP:		flag = CTL_D_OUTGOING; break;
+	case SOCK_TYPE_CONTROL:		flag = CTL_D_GNUTELLA; break;
+	case SOCK_TYPE_UPLOAD:		flag = CTL_D_INCOMING; break;
+	case SOCK_TYPE_CONNBACK:
+		flag = ctl_limit(addr, CTL_D_STEALTH) ? CTL_D_GNUTELLA : 0;
+		break;
+	default:
+		g_warning("socket_connect_prepare(): unexpected type \"%s\"",
+			socket_type_to_string(type));
+		flag = CTL_D_OUTGOING;
+		break;
+	}
+
+	if (ctl_limit(addr, flag)) {
+		if (GNET_PROPERTY(socket_debug) || GNET_PROPERTY(ctl_debug)) {
+			g_warning("CTL not connecting [%s] to host %s [%s]",
+				socket_type_to_string(type), host_addr_to_string(addr),
+				gip_country_cc(addr));
+		}
+		errno = EPERM;
+		return -1;
+	}
+
+	return 0;
+}
+
 /**
  * Called to prepare the creation of the socket connection.
  *
@@ -2621,11 +2722,10 @@ socket_connect_prepare(struct gnutella_socket *s,
 	/* Filter out flags which we cannot accept */
 	flags &= (SOCK_F_TLS | SOCK_F_FORCE);
 
-	if (!(s->flags & SOCK_F_FORCE) && hostiles_check(addr)) {
-		g_warning("Not connecting to hostile host %s",
-			host_addr_to_string(addr));
-		errno = EPERM;
-		return -1;
+	if (!(s->flags & SOCK_F_FORCE) && is_host_addr(addr)) {
+		if (0 != socket_connection_allowed(addr, type))
+			return -1;
+		flags |= SOCK_F_PREPARED;
 	}
 
 	if (0 == (SOCK_F_TLS & flags) && tls_cache_lookup(addr, port)) {
@@ -2726,11 +2826,15 @@ socket_connect_finalize(struct gnutella_socket *s, const host_addr_t ha)
 
 	/*
 	 * Allow forced connections to an hostile host.
+	 *
+	 * If SOCK_F_PREPARED is set, then we've already checked for hostiles
+	 * in socket_connect_prepare(), where we already knew the address, and
+	 * there's no need to redo it now.
 	 */
 
-	if (!(s->flags & SOCK_F_FORCE) && hostiles_check(ha)) {
-		g_warning("Not connecting to hostile host %s", host_addr_to_string(ha));
-		goto failure;	/* Not connecting to hostile host */
+	if (!(s->flags & (SOCK_F_FORCE | SOCK_F_PREPARED))) {
+		if (0 != socket_connection_allowed(ha, s->type))
+			goto failure;	/* Not connecting to hostile host */
 	}
 
 	s->addr = ha;
@@ -3293,7 +3397,8 @@ socket_udp_listen(host_addr_t bind_addr, guint16 port)
 
 	s = socket_alloc();
 
-	socket_alloc_buffer(s);
+	s->buf_size = SOCK_LBUFSZ;		/* Larger buffer to allow big payloads */
+	s->buf = halloc(s->buf_size);
 	s->type = SOCK_TYPE_UDP;
 	s->direction = SOCK_CONN_LISTENING;
 	s->file_desc = fd;
@@ -3530,11 +3635,16 @@ socket_tx_shutdown(struct gnutella_socket *s)
 	if (s->flags & SOCK_F_SHUTDOWN)
 		return;
 
-	/* EINVAL and ENOTCONN may occur if connect() didn't succeed */
+	/*
+	 * EINVAL and ENOTCONN may occur if connect() didn't succeed.
+	 * ECONNRESET may occur when TX shutdown happens late and the other
+	 * side of the connection already closed its socket.
+	 */
 	if (
 		-1 == shutdown(s->file_desc, SHUT_WR) &&
 		EINVAL != errno &&
-		ENOTCONN != errno
+		ENOTCONN != errno &&
+		ECONNRESET != errno
 	) {
 		g_warning("unable to shutdown TX on fd#%d: %s",
 			s->file_desc, g_strerror(errno));
