@@ -1,5 +1,5 @@
 /*
- * $Id: spam.c 16106 2009-03-10 15:16:31Z rmanfredi $
+ * $Id: spam.c 16990 2009-09-25 15:23:43Z rmanfredi $
  *
  * Copyright (c) 2006, Christian Biere
  * Copyright (c) 2004, Raphael Manfredi
@@ -41,7 +41,7 @@
 
 #include "common.h"
 
-RCSID("$Id: spam.c 16106 2009-03-10 15:16:31Z rmanfredi $")
+RCSID("$Id: spam.c 16990 2009-09-25 15:23:43Z rmanfredi $")
 
 #include "spam.h"
 #include "settings.h"
@@ -52,8 +52,10 @@ RCSID("$Id: spam.c 16106 2009-03-10 15:16:31Z rmanfredi $")
 #include "lib/bit_array.h"
 #include "lib/file.h"
 #include "lib/getdate.h"
-#include "lib/misc.h"
+#include "lib/halloc.h"
 #include "lib/glib-missing.h"
+#include "lib/parse.h"
+#include "lib/path.h"
 #include "lib/walloc.h"
 #include "lib/watcher.h"
 #include "lib/utf8.h"
@@ -331,7 +333,7 @@ spam_load(FILE *f)
 					item.damaged = TRUE;
 					g_warning("spam_load(): Filename pattern is not UTF-8.");
 				} else {
-					item.name = g_strdup(value);
+					item.name = h_strdup(value);
 				}
 			}
 			break;
@@ -422,7 +424,7 @@ spam_load(FILE *f)
 
 		if (item.done) {
 			/* Reset state */
-			G_FREE_NULL(item.name);
+			HFREE_NULL(item.name);
 			item = zero_item;
 			bit_array_clear_range(tag_used, 0, NUM_SPAM_TAGS - 1U);
 		}
@@ -469,7 +471,7 @@ spam_retrieve_from_file(FILE *f, const char *path, const char *filename)
 
 	pathname = make_pathname(path, filename);
 	watcher_register(pathname, spam_changed, NULL);
-	G_FREE_NULL(pathname);
+	HFREE_NULL(pathname);
 	spam_load(f);
 }
 
