@@ -1,5 +1,5 @@
 /*
- * $Id: sq.c 16160 2009-03-15 15:22:28Z rmanfredi $
+ * $Id: sq.c 16554 2009-04-23 18:06:56Z rmanfredi $
  *
  * Copyright (c) 2002-2003, Alex Bennee <alex@bennee.com> & Raphael Manfredi
  *
@@ -46,7 +46,7 @@
 
 #include "common.h"
 
-RCSID("$Id: sq.c 16160 2009-03-15 15:22:28Z rmanfredi $")
+RCSID("$Id: sq.c 16554 2009-04-23 18:06:56Z rmanfredi $")
 
 #include "sq.h"					/* search_queue structures */
 #include "mq_tcp.h"
@@ -310,8 +310,12 @@ sq_puthere(squeue_t *sq, gnet_search_t sh, pmsg_t *mb, query_hashvec_t *qhv)
 	g_assert(sq);
 	g_assert(mb);
 
-	if (sqh_exists(sq, sh))
+	if (sqh_exists(sq, sh)) {
+		pmsg_free(mb);
+		if (qhv)
+			qhvec_free(qhv);
 		return;						/* Search already in queue */
+	}
 
 	sb = smsg_alloc(sh, mb, qhv);
 
@@ -411,8 +415,8 @@ retry:
 		if (GNET_PROPERTY(current_peermode) != NODE_P_ULTRA)
 			return;
 
-		if (node_keep_missing() * 3 > 2 * GNET_PROPERTY(up_connections))
-			return;							/* Not enough nodes for querying */
+		if (3*UNSIGNED(node_keep_missing()) > 2*GNET_PROPERTY(up_connections))
+			return;		/* Not enough nodes for querying */
 	}
 
 	/*
