@@ -1,5 +1,5 @@
 /*
- * $Id: thex_download.c 15948 2009-02-07 17:18:13Z rmanfredi $
+ * $Id: thex_download.c 17386 2010-03-18 08:50:43Z rmanfredi $
  *
  * Copyright (c) 2005, Raphael Manfredi
  * Copyright (c) 2005, Martijn van Oosterhout <kleptog@svana.org>
@@ -37,7 +37,7 @@
 
 #include "common.h"
 
-RCSID("$Id: thex_download.c 15948 2009-02-07 17:18:13Z rmanfredi $")
+RCSID("$Id: thex_download.c 17386 2010-03-18 08:50:43Z rmanfredi $")
 
 #include <libxml/parser.h>                                                      
 #include <libxml/tree.h>                                                        
@@ -54,14 +54,17 @@ RCSID("$Id: thex_download.c 15948 2009-02-07 17:18:13Z rmanfredi $")
 #include "lib/atoms.h"
 #include "lib/ascii.h"
 #include "lib/endian.h"
+#include "lib/parse.h"
 #include "lib/pmsg.h"
+#include "lib/stringify.h"
 #include "lib/tigertree.h"
+#include "lib/halloc.h"
 #include "lib/walloc.h"
 
 #include "lib/override.h"	/* Must be the last header included */
 
 #define THEX_DOWNLOAD_DEFAULT_SIZE	4096	/* Default data buffer size */
-#define THEX_DOWNLOAD_MAX_SIZE		(256 * 1024)	/* 256 KiB */
+#define THEX_DOWNLOAD_MAX_SIZE		(260 * 1024)	/* 256 KiB + overhead */
 
 struct thex_download {
 	gpointer owner;					/**< Download owning us */
@@ -70,6 +73,7 @@ struct thex_download {
 	char *data;						/**< Where payload data is stored */
 	size_t data_size;				/**< Size of data buffer */
 	size_t pos;						/**< Reading position */
+	size_t max_size;				/**< Max size we'll read */
 	const struct sha1 *sha1;		/**< SHA1 atom; refers to described file */
 	const struct tth *tth;			/**< TTH atom; refers to described file */
 	struct tth *leaves;				/**< g_memdup()ed leave TTHs */
@@ -145,12 +149,12 @@ thex_download_data_read(struct thex_download *ctx, pmsg_t *mb)
 	g_assert(ctx->pos <= ctx->data_size);
 
 	while ((size = pmsg_size(mb)) > 0) {
-		if (ctx->pos + size > THEX_DOWNLOAD_MAX_SIZE)
+		if (ctx->pos + size > ctx->max_size)
 			return TRUE;
 
 		if (size > ctx->data_size - ctx->pos) {
 			ctx->data_size += MAX(size, ctx->data_size);
-			ctx->data = g_realloc(ctx->data, ctx->data_size);
+			ctx->data = hrealloc(ctx->data, ctx->data_size);
 		}
 		ctx->pos += pmsg_read(mb, &ctx->data[ctx->pos], size);
 	}
@@ -219,14 +223,14 @@ verify_element(xmlNode *node, const char *prop, const char *expect)
 	value = STRTRACK(xml_get_string(node, prop));
   	if (NULL == value) {
 		if (GNET_PROPERTY(tigertree_debug)) {
-			g_message("Couldn't find property \"%s\" of node \"%s\"",
+			g_message("TTH couldn't find property \"%s\" of node \"%s\"",
 				prop, node->name);
 		}
 		goto finish;
 	}
 	if (0 != strcmp(value, expect)) {
 		if (GNET_PROPERTY(tigertree_debug)) {
-			g_message("Property %s/%s doesn't match expected value \"%s\", "
+			g_message("TTH property %s/%s doesn't match expected value \"%s\", "
 				"got \"%s\"",
 				node->name, prop, expect, value);
 		}
@@ -250,7 +254,7 @@ thex_download_handle_xml(struct thex_download *ctx,
 
 	if (size <= 0) {
 		if (GNET_PROPERTY(tigertree_debug)) {
-			g_message("XML record has no data");
+			g_message("TTH XML record has no data");
 		}
 		goto finish;
 	}
@@ -258,7 +262,7 @@ thex_download_handle_xml(struct thex_download *ctx,
 	doc = xmlReadMemory(data, size, "noname.xml", NULL, 0);
 	if (NULL == doc) {
 		if (GNET_PROPERTY(tigertree_debug)) {
-			g_message("Cannot parse XML record");
+			g_message("TTH cannot parse XML record");
 		}
 		goto finish;
 	}
@@ -267,7 +271,7 @@ thex_download_handle_xml(struct thex_download *ctx,
   	hashtree = find_element_by_name(root, "hashtree");
 	if (NULL == hashtree) {
 		if (GNET_PROPERTY(tigertree_debug)) {
-			g_message("Couldn't find hashtree element");
+			g_message("TTH couldn't find hashtree element");
 		}
 		goto finish;
 	}
@@ -280,7 +284,7 @@ thex_download_handle_xml(struct thex_download *ctx,
 			goto finish;
 	} else {
 		if (GNET_PROPERTY(tigertree_debug)) {
-			g_message("Couldn't find hashtree/file element");
+			g_message("TTH couldn't find hashtree/file element");
 		}
 		goto finish;
 	}
@@ -293,7 +297,7 @@ thex_download_handle_xml(struct thex_download *ctx,
 			goto finish;
 	} else {
 		if (GNET_PROPERTY(tigertree_debug)) {
-			g_message("Couldn't find hashtree/digest element");
+			g_message("TTH couldn't find hashtree/digest element");
 		}
     	goto finish;
 	}
@@ -309,18 +313,18 @@ thex_download_handle_xml(struct thex_download *ctx,
 		value = STRTRACK(xml_get_string(node, "uri"));
 		if (NULL == value) {
 			if (GNET_PROPERTY(tigertree_debug)) {
-				g_message("Couldn't find property \"uri\" of node \"%s\"",
+				g_message("TTH couldn't find property \"uri\" of node \"%s\"",
 					node->name);
 			}
 			goto finish;
 		}
-		hashtree_id = g_strdup(value);
+		hashtree_id = h_strdup(value);
 		xml_string_free(&value);
 
 		value = STRTRACK(xml_get_string(node, "depth"));
 		if (NULL == value) {
 			if (GNET_PROPERTY(tigertree_debug)) {
-				g_message("Couldn't find property \"depth\" of node \"%s\"",
+				g_message("TTH couldn't find property \"depth\" of node \"%s\"",
 					node->name);
 			}
 			goto finish;
@@ -330,7 +334,7 @@ thex_download_handle_xml(struct thex_download *ctx,
 		error |= ctx->depth > tt_full_depth(ctx->filesize);
 		if (error) {
 			ctx->depth = 0;
-			g_message("Bad value for \"depth\" of node \"%s\": \"%s\"",
+			g_message("TTH bad value for \"depth\" of node \"%s\": \"%s\"",
 				node->name, value);
 		}
 		xml_string_free(&value);
@@ -338,7 +342,7 @@ thex_download_handle_xml(struct thex_download *ctx,
 			goto finish;
 	} else {
 		if (GNET_PROPERTY(tigertree_debug))
-			g_message("Couldn't find hashtree/serializedtree element");
+			g_message("TTH couldn't find hashtree/serializedtree element");
 		goto finish;
 	}
 
@@ -350,7 +354,7 @@ finish:
 		doc = NULL;
 	}
 	if (!success) {
-		G_FREE_NULL(hashtree_id);
+		HFREE_NULL(hashtree_id);
 	}
 	return hashtree_id;
 }
@@ -367,26 +371,27 @@ thex_download_handle_hashtree(struct thex_download *ctx,
 
 	if (size <= 0) {
 		if (GNET_PROPERTY(tigertree_debug)) {
-			g_message("Hashtree record has no data");
+			g_message("TTH hashtree record has no data");
 		}
 		goto finish;
 	}
 	if (size < TTH_RAW_SIZE) {
 		if (GNET_PROPERTY(tigertree_debug)) {
-			g_message("Hashtree record is too small");
+			g_message("TTH hashtree record is too small");
 		}
 		goto finish;
 	}
 	if (size % TTH_RAW_SIZE) {
 		if (GNET_PROPERTY(tigertree_debug)) {
-			g_message("Hashtree has bad size");
+			g_message("TTH hashtree has bad size");
 		}
 		goto finish;
 	}
 	memcpy(tth.data, data, TTH_RAW_SIZE);
 	if (!tth_eq(&tth, ctx->tth)) {
 		if (GNET_PROPERTY(tigertree_debug)) {
-			g_message("Hashtree has different root hash %s", tth_base32(&tth));
+			g_message("TTH hashtree has different root hash %s",
+				tth_base32(&tth));
 		}
 		goto finish;
 	}
@@ -407,14 +412,14 @@ thex_download_handle_hashtree(struct thex_download *ctx,
 			ctx->depth--;
 		}
 		if (GNET_PROPERTY(tigertree_debug)) {
-			g_message("Calculated depth of hashtree: %u", ctx->depth);
+			g_message("TTH calculated depth of hashtree: %u", ctx->depth);
 		}
 		n_leaves = tt_node_count_at_depth(ctx->filesize, ctx->depth);
 	}
 
 	if (ctx->depth < good_depth) {
 		if (GNET_PROPERTY(tigertree_debug)) {
-			g_message("Tree depth (%u) is below the good depth (%u)",
+			g_message("TTH tree depth (%u) is below the good depth (%u)",
 				ctx->depth, good_depth);
 		}
 	}
@@ -428,7 +433,7 @@ thex_download_handle_hashtree(struct thex_download *ctx,
 
 	if (n_nodes < start + n_leaves) {
 		if (GNET_PROPERTY(tigertree_debug)) {
-			g_message("Hashtree has too few nodes "
+			g_message("TTH hashtree has too few nodes "
 				"(filesize=%s depth=%u nodes=%lu expected=%lu)",
 				filesize_to_string(ctx->filesize),
 				ctx->depth,
@@ -444,7 +449,8 @@ thex_download_handle_hashtree(struct thex_download *ctx,
 	tth = tt_root_hash(leaves, n_leaves);
 	if (!tth_eq(&tth, ctx->tth)) {
 		if (GNET_PROPERTY(tigertree_debug)) {
-			g_message("Hashtree does not match root hash %s", tth_base32(&tth));
+			g_message("TTH hashtree does not match root hash %s",
+				tth_base32(&tth));
 		}
 		goto finish;
 	}
@@ -507,7 +513,7 @@ dime_find_record(const GSList *records, const char *type, const char *id)
 	}
 
 	if (GNET_PROPERTY(tigertree_debug)) {
-		g_message("Could not find record (type=\"%s\", id=%s%s%s)",
+		g_message("TTH could not find record (type=\"%s\", id=%s%s%s)",
 			type,
 			id ? "\"" : "",
 			id ? id : "<none>",
@@ -552,7 +558,7 @@ thex_download_finished(struct thex_download *ctx)
 		hashtree_id = thex_download_handle_xml(ctx, data, size);
 		if (NULL == hashtree_id) {
 			if (GNET_PROPERTY(tigertree_debug)) {
-				g_message("Could not determine hashtree ID");
+				g_message("TTH could not determine hashtree ID");
 				dump_hex(stderr, "THEX data", ctx->data, ctx->data_size);
 			}
 			/* Bug workaround:
@@ -569,7 +575,7 @@ thex_download_finished(struct thex_download *ctx)
 			 */
 			record = dime_find_record(records, THEX_TREE_TYPE, NULL);
 		}
-		G_FREE_NULL(hashtree_id);
+		HFREE_NULL(hashtree_id);
 
 		if (NULL == record) {
 			if (GNET_PROPERTY(tigertree_debug)) {
@@ -586,7 +592,7 @@ thex_download_finished(struct thex_download *ctx)
 
 	} else {
 		if (GNET_PROPERTY(tigertree_debug)) {
-			g_message("Could not parse DIME records");
+			g_message("TTH could not parse DIME records");
 			dump_hex(stderr, "THEX data", ctx->data, ctx->data_size);
 		}
 		goto finish;
@@ -595,7 +601,7 @@ thex_download_finished(struct thex_download *ctx)
 
 finish:
 	dime_list_free(&records);
-	G_FREE_NULL(ctx->data);
+	HFREE_NULL(ctx->data);
 	return success;
 }
 
@@ -655,17 +661,18 @@ static const struct rx_inflate_cb thex_rx_inflate_cb = {
 };
 
 /**
- * Prepare reception of query hit data by building an appropriate RX stack.
+ * Prepare reception of THEX data by building an appropriate RX stack.
  *
  * @return TRUE if we may continue with the download.
  */
 gboolean
 thex_download_receive(struct thex_download *ctx,
+	filesize_t content_length,
 	gnet_host_t *host, struct wrap_io *wio, guint32 flags)
 {
 	g_assert(ctx != NULL);
 
-	ctx->host = *host;			/* Struct copy */
+	ctx->host = *host;				/* Struct copy */
 
 	/*
 	 * Freeing of the RX stack must be asynchronous: each time we establish
@@ -679,6 +686,20 @@ thex_download_receive(struct thex_download *ctx,
 		ctx->rx = NULL;
 	}
 
+	/*
+	 * If there is a Content-Length indication in the HTTP reply, it is
+	 * supplied here and will be used as a limit of the data we'll read.
+	 *
+	 * If there was none (for instance if the output is chunked), then 0
+	 * is given and we'll use a hardwired maximum.
+	 */
+
+	if (content_length > MAX_INT_VAL(size_t))
+		return FALSE;
+
+	ctx->max_size = content_length ?
+		(size_t) content_length : THEX_DOWNLOAD_MAX_SIZE;
+
 	{
 		struct rx_link_args args;
 
@@ -774,7 +795,7 @@ thex_download_free(struct thex_download **ptr)
 			rx_free(ctx->rx);
 			ctx->rx = NULL;
 		}
-		G_FREE_NULL(ctx->data);
+		HFREE_NULL(ctx->data);
 		G_FREE_NULL(ctx->leaves);
 		atom_sha1_free_null(&ctx->sha1);
 		atom_tth_free_null(&ctx->tth);
