@@ -1,5 +1,5 @@
 /*
- * $Id: tls_common.c 15053 2007-12-08 01:15:45Z cbiere $
+ * $Id: tls_common.c 17110 2009-10-23 17:11:14Z rmanfredi $
  *
  * Copyright (c) 2006, Christian Biere
  *
@@ -32,7 +32,7 @@
 
 #include "common.h"
 
-RCSID("$Id: tls_common.c 15053 2007-12-08 01:15:45Z cbiere $")
+RCSID("$Id: tls_common.c 17110 2009-10-23 17:11:14Z rmanfredi $")
 
 #ifdef HAS_GNUTLS
 #include <gnutls/gnutls.h>
@@ -47,8 +47,12 @@ RCSID("$Id: tls_common.c 15053 2007-12-08 01:15:45Z cbiere $")
 #include "if/core/settings.h"
 
 #include "lib/array.h"
+#include "lib/concat.h"
+#include "lib/file.h"
+#include "lib/halloc.h"
 #include "lib/header.h"
-#include "lib/misc.h"
+#include "lib/path.h"
+#include "lib/stringify.h"
 #include "lib/walloc.h"
 
 #include "lib/override.h"		/* Must be the last header included */
@@ -145,9 +149,9 @@ tls_socket_evt_change(struct gnutella_socket *s, inputevt_cond_t cond)
 
 		if (GNET_PROPERTY(tls_debug) > 1) {
 			int fd = socket_evt_fd(s);
-			g_message("tls_socket_evt_change: fd=%d, cond=%s -> %s, handler=%p",
+			g_message("tls_socket_evt_change: fd=%d, cond=%s -> %s",
 				fd, inputevt_cond_to_string(s->tls.cb_cond),
-				inputevt_cond_to_string(cond), s->tls.cb_handler);
+				inputevt_cond_to_string(cond));
 		}
 		if (s->gdk_tag) {
 			inputevt_remove(s->gdk_tag);
@@ -468,6 +472,10 @@ tls_global_init(void)
 	};
 	char *cert_file, *key_file;
 
+#if !defined(REMAP_ZALLOC) && !defined(TRACK_MALLOC) && !defined(TRACK_ZALLOC)
+	gnutls_global_set_mem_functions(halloc, halloc, NULL, hrealloc, hfree);
+#endif
+
 	if (gnutls_global_init()) {
 		g_error("gnutls_global_init() failed");
 	}
@@ -491,14 +499,20 @@ tls_global_init(void)
 			gnutls_certificate_set_dh_params(server_cert_cred, get_dh_params());
 		}
 	}
-	G_FREE_NULL(key_file);
-	G_FREE_NULL(cert_file);
+	HFREE_NULL(key_file);
+	HFREE_NULL(cert_file);
 
 	header_features_add(FEATURES_CONNECTIONS, f.name, f.major, f.minor);
 	header_features_add(FEATURES_DOWNLOADS, f.name, f.major, f.minor);
 	header_features_add(FEATURES_UPLOADS, f.name, f.major, f.minor);
 }
 
+void
+tls_global_close(void)
+{
+	gnutls_global_deinit();
+}
+
 static ssize_t
 tls_write_intern(struct wrap_io *wio, gconstpointer buf, size_t size)
 {
@@ -953,6 +967,12 @@ tls_global_init(void)
 	/* Nothing to do */
 }
 
+void
+tls_global_close(void)
+{
+	/* Nothing to do */
+}
+
 const char *
 tls_version_string(void)
 {
