@@ -1,5 +1,5 @@
 /*
- * $Id: token.c 16301 2009-03-29 15:31:34Z rmanfredi $
+ * $Id: token.c 17403 2010-03-21 10:04:32Z rmanfredi $
  *
  * Copyright (c) 2003, Raphael Manfredi
  *
@@ -35,7 +35,7 @@
 
 #include "common.h"
 
-RCSID("$Id: token.c 16301 2009-03-29 15:31:34Z rmanfredi $")
+RCSID("$Id: token.c 17403 2010-03-21 10:04:32Z rmanfredi $")
 
 #include "token.h"
 #include "clock.h"
@@ -241,6 +241,37 @@ static const char *keys_096_6[] = {
 	"587d 2f6f a442 d49f 8de3 af20 a7c8 ac20",
 };
 
+static const char *keys_096_7[] = {
+	"c75f 6c38 449c fb4d 7636 58b0 5b48 a977",
+	"04f8 bb8b 3a87 015a e019 506f 02cf e458",
+	"fa6a e76c d3bb a724 4132 4b8e 917e 102c",
+	"de57 24d2 691d 5e14 bce3 bb1e 55ee b852",
+	"f5d2 625b 19c2 6c82 9ee0 859f c55b faed",
+	"6f56 d29c 9f50 365c 87d8 ce38 ba58 96a0",
+	"27a8 c873 187c ab36 d8a7 f26e 848d dabb",
+	"8240 ceba 9a61 9133 872e 7cbb 1ff7 3c44",
+	"c174 550d ebdf bbc4 fd6e a90e c37a 3ade",
+	"cedc 9c8c e3cc 320b 4d27 cb50 91c9 e711",
+	"de55 4003 91fb 4c85 ac25 70fa d129 f6c6",
+	"5329 12d3 d43e 7747 5ff3 8c16 41b3 2234",
+	"dea9 6231 7068 7079 dfd8 241d b0fc 69f7",
+	"bdc2 3f85 9ed8 9208 4508 9fdc 68ae 0772",
+	"5bd4 fcf2 e7b8 abee f0a2 26ef 59d1 f502",
+	"0664 ffe6 8605 8eb2 6978 bb32 4e35 eabf",
+	"3787 d073 e6bc 7ee4 bc83 721e 9987 302e",
+	"22d3 cc29 ba7e 7457 af72 9036 2de6 bc62",
+	"2b78 c4ca 4a82 3c1e 8fd8 7913 4e70 a56d",
+	"50af 1568 02fc 694f 7486 6c4d 8610 b536",
+	"a4f8 a838 686f 1bdc 2fd7 d4cc 59f7 9c0e",
+	"0424 595c 7e43 4d4a c293 2dea 836a a370",
+	"d659 2830 a507 9555 c2c3 9226 aae2 d8dd",
+	"eac6 4e0d 25b5 4547 b92d 042c 5ae6 b34e",
+	"8d3c ab1c a0aa 0f1e cd4a 301d 97c3 f4c4",
+	"57b6 f87c ec89 67f3 633e 82bf e7e7 3e54",
+	"e8ba 7daf cb5b 3782 b816 98fa 179c ee6b",
+	"7ec0 052b 954e cfd7 1605 e95c fa86 61de",
+};
+
 /**
  * Describes the keys to use depending on the version.
  */
@@ -278,6 +309,15 @@ struct tokkey {
 		{ 0, 96, 6, '\0', 0, 0, 1238338209 },		/* 2009-03-29 */
 		keys_096_6, G_N_ELEMENTS(keys_096_6),
 	},
+	{
+		{ 0, 96, 7, '\0', 0, 0, 1269813600 },		/* 2010-03-29 */
+		keys_096_7, G_N_ELEMENTS(keys_096_7),
+	},
+	/* No new keys for 0.96.8: bugfix release of 0.96.7 */
+	{
+		{ 0, 96, 8, '\0', 0, 0, 1269813601 },		/* 2010-03-29 */
+		keys_096_7, G_N_ELEMENTS(keys_096_7),
+	},
 };
 
 /**
@@ -311,31 +351,83 @@ tok_strerror(tok_error_t errnum)
 {
 	STATIC_ASSERT(G_N_ELEMENTS(tok_errstr) == TOK_MAX_ERROR);
 
-	if ((int) errnum < 0 || errnum >= G_N_ELEMENTS(tok_errstr))
+	if (UNSIGNED(errnum) >= G_N_ELEMENTS(tok_errstr))
 		return "Invalid error code";
 
 	return tok_errstr[errnum];
 }
 
+/**
+ * Based on the timestamp, determine the proper token keys to use limiting
+ * to the first ``count'' items.
+ *
+ * @return NULL if we cannot locate any suitable keys.
+ */
+static const struct tokkey *
+find_tokkey_upto(time_t now, size_t count)
+{
+	time_t adjusted = now - VERSION_ANCIENT_BAN;
+	guint i;
+
+	g_assert(count <= G_N_ELEMENTS(token_keys));
+
+	for (i = 0; i < count; i++) {
+		const struct tokkey *tk = &token_keys[i];
+		if (tk->ver.timestamp > adjusted)
+			return tk;
+	}
+
+	return NULL;
+}
+
 /**
  * Based on the timestamp, determine the proper token keys to use.
  *
  * @return NULL if we cannot locate any suitable keys.
  */
-static const struct tokkey *
+static inline const struct tokkey *
 find_tokkey(time_t now)
 {
-	time_t adjusted = now - VERSION_ANCIENT_BAN;
-	const struct tokkey *tk;
+	return find_tokkey_upto(now, G_N_ELEMENTS(token_keys));
+}
+
+/**
+ * Based on the timestamp and their advertised version, find out the
+ * token key they used.
+ *
+ * @return NULL if we cannot locate any suitable keys.
+ */
+static const struct tokkey *
+find_tokkey_version(const version_t *ver, time_t now)
+{
 	guint i;
 
+	/*
+	 * All versions before r16370 used the first key set when they expired.
+	 * If we're more recent, we probably have a stripped list of past key
+	 * sets, and therefore cannot validate their token.
+	 */
+
+	if (ver->build < 16370)
+		return find_tokkey(now);
+
+	/*
+	 * Expired servents will always use their last key set.  Even if we're
+	 * more recent, we can try to validate by mimicing the behaviour of
+	 * these servents.
+	 *
+	 * We determine the index of the last key set that they must know about
+	 * and we look for the token up to that index only.
+	 */
+
 	for (i = 0; i < G_N_ELEMENTS(token_keys); i++) {
-		tk = &token_keys[i];
-		if (tk->ver.timestamp > adjusted)
-			return tk;
+		const struct tokkey *tk = &token_keys[i];
+		time_delta_t dtime = delta_time(ver->timestamp, tk->ver.timestamp);
+		if (dtime < 0 && -dtime >= 86400)
+			break;				/* Not within a day and older than token */
 	}
 
-	return NULL;
+	return find_tokkey_upto(now, i);
 }
 
 /**
@@ -379,16 +471,36 @@ random_key(time_t now, guint *idx, const struct tokkey **tkused)
 			warned = TRUE;
 		}
 
-		tk = &token_keys[0];	/* They'll have problems with their token */
+		STATIC_ASSERT(G_N_ELEMENTS(token_keys) >= 1);
+
+		/* Pick the latest (most recent) key set from the array */
+		tk = &token_keys[G_N_ELEMENTS(token_keys) - 1];
 	}
 
-	random_idx = random_value(tk->count - 1);
+	random_idx = random_u32() % tk->count;
 	*idx = random_idx;
 	*tkused = tk;
 
 	return tk->keys[random_idx];
 }
 
+static guint16
+tok_crc(guint32 crc, const struct tokkey *tk)
+{
+	const char **keys = tk->keys;
+	size_t i;
+
+	i = tk->count;
+	while (i-- > 0) {
+		const char *k = *keys++;
+		crc = crc32_update_crc(crc, k, strlen(k));
+	}
+	crc ^= (crc >> 8);
+	crc &= 0x00ff00ffU;
+	crc |= crc >> 8;
+	return crc & 0xffffU;
+}
+
 /**
  * Generate new token for given version string.
  */
@@ -400,14 +512,12 @@ tok_generate(time_t now, const char *version)
 	char lvldigest[LEVEL_SIZE];
 	char lvlbase64[LEVEL_BASE64_SIZE + 1];
 	const struct tokkey *tk;
+	guint32 crc32;
 	guint idx;
 	const char *key;
 	SHA1Context ctx;
     struct sha1 sha1;
-	guint8 seed[3];
-	guint32 now32;
 	int lvlsize;
-	int klen;
 	int i;
 
 	/*
@@ -415,16 +525,12 @@ tok_generate(time_t now, const char *version)
 	 */
 
 	key = random_key(now, &idx, &tk);
-	seed[0] = random_value(0xff);
-	seed[1] = random_value(0xff);
-	seed[2] = random_value(0xff) & 0xe0;	/* Upper 3 bits only */
-	seed[2] |= idx;							/* Has 5 bits for the index */
-
 	now = clock_loc2gmt(now);				/* As close to GMT as possible */
 
-	now32 = (guint32) htonl((guint32) now);
-	memcpy(&digest[0], &now32, 4);
-	memcpy(&digest[4], &seed, 3);
+	poke_be32(&digest[0], now);
+	random_bytes(&digest[4], 3);
+	digest[6] &= 0xe0U;			/* Upper 3 bits only */
+	digest[6] |= idx & 0xffU;	/* Has 5 bits for the index */
 
 	SHA1Reset(&ctx);
 	SHA1Input(&ctx, key, strlen(key));
@@ -438,20 +544,11 @@ tok_generate(time_t now, const char *version)
 	 */
 
 	lvlsize = G_N_ELEMENTS(token_keys) - (tk - token_keys);
-	now32 = crc32_update_crc(0, digest, TOKEN_VERSION_SIZE);
-	klen = strlen(tk->keys[0]);
+	crc32 = crc32_update_crc(0, digest, TOKEN_VERSION_SIZE);
 
-	for (i = 0; i < lvlsize; i++, tk++) {
-		guint j;
-		guint32 crc = now32;
-		const guchar *c = (const guchar *) &crc;
-
-		for (j = 0; j < tk->count; j++)
-			crc = crc32_update_crc(crc, tk->keys[j], klen);
-
-		crc = htonl(crc);
-		lvldigest[i*2] = c[0] ^ c[1];
-		lvldigest[i*2+1] = c[2] ^ c[3];
+	for (i = 0; i < lvlsize; i++) {
+		poke_be16(&lvldigest[i*2], tok_crc(crc32, tk));
+		tk++;
 	}
 
 	/*
@@ -501,7 +598,7 @@ tok_version(void)
 	G_FREE_NULL(toklevel);
 	toklevel = tok_generate(now, version_string);
 
-	return toklevel;
+	return NOT_LEAKING(toklevel);
 }
 
 /**
@@ -532,7 +629,7 @@ tok_short_version(void)
 	G_FREE_NULL(toklevel);
 	toklevel = tok_generate(now, version_short_string);
 
-	return toklevel;
+	return NOT_LEAKING(toklevel);
 }
 
 /**
@@ -547,7 +644,7 @@ tok_version_valid(
 {
 	time_t now = tm_time();
 	time_t stamp;
-	guint32 stamp32;
+	guint32 crc;
 	const struct tokkey *tk;
 	const struct tokkey *rtk;
 	const struct tokkey *latest;
@@ -562,9 +659,7 @@ tok_version_valid(
 	int toklen;
 	int lvllen;
 	int lvlsize;
-	int klen;
 	guint i;
-	char *c = (char *) &stamp32;
 
 	end = strchr(tokenb64, ';');		/* After 25/02/2003 */
 	toklen = end ? (end - tokenb64) : len;
@@ -579,8 +674,7 @@ tok_version_valid(
 	if (!base64_decode_into(tokenb64, toklen, token, TOKEN_VERSION_SIZE))
 		return TOK_BAD_ENCODING;
 
-	memcpy(&stamp32, token, 4);
-	stamp = (time_t) ntohl(stamp32);
+	stamp = (time_t) peek_be32(&token);
 
 	/*
 	 * Use that stamp, whose precision is TOKEN_LIFE, to update our
@@ -592,7 +686,10 @@ tok_version_valid(
 	if (ABS(stamp - clock_loc2gmt(now)) > TOKEN_CLOCK_SKEW)
 		return TOK_BAD_STAMP;
 
-	tk = find_tokkey(stamp);				/* The keys they used */
+	if (!version_fill(version, &rver))		/* Remote version */
+		return TOK_BAD_VERSION;
+
+	tk = find_tokkey_version(&rver, stamp);	/* The keys they used */
 	if (tk == NULL)
 		return TOK_BAD_KEYS;
 
@@ -611,9 +708,6 @@ tok_version_valid(
 	if (0 != memcmp(&token[7], digest.data, SHA1_RAW_SIZE))
 		return TOK_INVALID;
 
-	if (!version_fill(version, &rver))		/* Remote version */
-		return TOK_BAD_VERSION;
-
 	if (version_build_cmp(&rver, &tk->ver) < 0)
 		return TOK_OLD_VERSION;
 
@@ -642,7 +736,7 @@ tok_version_valid(
 	lvllen = len - toklen - 2;				/* Forget about "; " */
 	end += 2;								/* Skip "; " */
 
-	if (lvllen >= (int) sizeof(lvldigest) || lvllen <= 0)
+	if (UNSIGNED(lvllen) >= sizeof(lvldigest) || lvllen <= 0)
 		return TOK_BAD_LEVEL_LENGTH;
 
 	if (lvllen & 0x3)
@@ -668,20 +762,12 @@ tok_version_valid(
 
 	rtk = tk + (lvlsize - 1);				/* Keys at that level */
 
-	stamp32 = crc32_update_crc(0, token, TOKEN_VERSION_SIZE);
-	klen = strlen(rtk->keys[0]);
-
-	for (i = 0; i < rtk->count; i++)
-		stamp32 = crc32_update_crc(stamp32, rtk->keys[i], klen);
-
-	stamp32 = htonl(stamp32);
+	crc = crc32_update_crc(0, token, TOKEN_VERSION_SIZE);
+	crc = tok_crc(crc, rtk);
 
 	lvlsize--;								/* Move to 0-based offset */
 
-	if (lvldigest[2*lvlsize] != (c[0] ^ c[1]))
-		return TOK_INVALID_LEVEL;
-
-	if (lvldigest[2*lvlsize+1] != (c[2] ^ c[3]))
+	if (peek_be16(&lvldigest[2*lvlsize]) != crc)
 		return TOK_INVALID_LEVEL;
 
 	for (i = 0; i < G_N_ELEMENTS(token_keys); i++) {
