@@ -1,5 +1,5 @@
 /*
- * $Id: tth_cache.c 15102 2008-02-15 21:38:12Z cbiere $
+ * $Id: tth_cache.c 16990 2009-09-25 15:23:43Z rmanfredi $
  *
  * Copyright (c) 2006, Christian Biere
  *
@@ -48,14 +48,17 @@
 
 #include "common.h"
 
-RCSID("$Id: tth_cache.c 15102 2008-02-15 21:38:12Z cbiere $")
+RCSID("$Id: tth_cache.c 16990 2009-09-25 15:23:43Z rmanfredi $")
 
 #include "settings.h"
 #include "tth_cache.h"
 
 #include "lib/atoms.h"
+#include "lib/fd.h"
 #include "lib/file.h"
-#include "lib/misc.h"
+#include "lib/halloc.h"
+#include "lib/path.h"
+#include "lib/stringify.h"
 #include "lib/tigertree.h"
 #include "lib/walloc.h"
 
@@ -75,7 +78,7 @@ tth_cache_directory(void)
 	if (!directory) {
 		directory = make_pathname(settings_config_dir(), "tth_cache");
 	}
-	return directory;
+	return NOT_LEAKING(directory);
 }
 
 static char *
@@ -86,7 +89,7 @@ tth_cache_pathname(const struct tth *tth)
 	g_assert(tth);
 	
 	hash = tth_base32(tth);
-	return g_strdup_printf("%s%c%2.2s%c%s",
+	return h_strdup_printf("%s%c%2.2s%c%s",
 			tth_cache_directory(), G_DIR_SEPARATOR,
 			&hash[0], G_DIR_SEPARATOR, &hash[2]);
 }
@@ -108,9 +111,9 @@ tth_cache_file_create(const struct tth *tth)
 		if (0 == create_directory(dir, DEFAULT_DIRECTORY_MODE)) {
 			fd = file_create(pathname, accmode, TTH_FILE_MODE);
 		}
-		G_FREE_NULL(dir);
+		HFREE_NULL(dir);
 	}
-	G_FREE_NULL(pathname);
+	HFREE_NULL(pathname);
 	return fd;
 }
 
@@ -124,7 +127,7 @@ tth_cache_file_open(const struct tth *tth)
 
 	pathname = tth_cache_pathname(tth);
 	fd = file_open_missing(pathname, O_RDONLY);
-	G_FREE_NULL(pathname);
+	HFREE_NULL(pathname);
 	return fd;
 }
 
@@ -138,7 +141,7 @@ tth_cache_file_exists(const struct tth *tth)
 
 	pathname = tth_cache_pathname(tth);
 	ret = file_exists(pathname);
-	G_FREE_NULL(pathname);
+	HFREE_NULL(pathname);
 	return ret;
 }
 
@@ -176,7 +179,7 @@ tth_cache_insert(const struct tth *tth, const struct tth *leaves, int n_leaves)
 		} else if ((size_t) ret != size) {
 			g_warning("tth_cache_insert(): incomplete write()");
 		}
-		close(fd);
+		fd_close(&fd, TRUE);
 	}
 }
 
@@ -229,7 +232,7 @@ tth_cache_lookup(const struct tth *tth, filesize_t filesize)
 		} else {
 			leave_count = tth_cache_leave_count(tth, &sb);
 		}
-		G_FREE_NULL(pathname);
+		HFREE_NULL(pathname);
 	} else {
 		leave_count = 1;
 	}
@@ -245,7 +248,7 @@ tth_cache_remove(const struct tth *tth)
 
 	pathname = tth_cache_pathname(tth);
 	remove(pathname);
-	G_FREE_NULL(pathname);
+	HFREE_NULL(pathname);
 }
 
 static size_t
@@ -282,7 +285,7 @@ tth_cache_get_leaves(const struct tth *tth,
 				}
 			}
 		}
-		close(fd);
+		fd_close(&fd, TRUE);
 	}
 	return num_leaves;
 }
