@@ -1,5 +1,5 @@
 /*
- * $Id: tx_chunk.c 13777 2007-05-29 00:23:11Z cbiere $
+ * $Id: tx_chunk.c 16106 2009-03-10 15:16:31Z rmanfredi $
  *
  * Copyright (c) 2005, Raphael Manfredi
  *
@@ -55,7 +55,7 @@
 
 #include "common.h"
 
-RCSID("$Id: tx_chunk.c 13777 2007-05-29 00:23:11Z cbiere $")
+RCSID("$Id: tx_chunk.c 16106 2009-03-10 15:16:31Z rmanfredi $")
 
 #include "tx.h"
 #include "tx_chunk.h"
@@ -73,13 +73,13 @@ RCSID("$Id: tx_chunk.c 13777 2007-05-29 00:23:11Z cbiere $")
  * upper layers until we have some room to send them.
  */
 
-#define CHUNK_DIGITS	4		/* At most that many digits in hexa */
+#define CHUNK_DIGITS 16 /* At most that many digits in hexa; 64-bit */
 
 /*
  * Private attributes for the link.
  */
 struct attr {
-	gchar head[CHUNK_DIGITS+5];	/**< Chunk header: hexa size + 2CRLF + NUL */
+	char head[CHUNK_DIGITS + 5];/**< Chunk header: hexa size + 2CRLF + NUL */
 	ssize_t head_len;			/**< Length of chunk header */
 	ssize_t head_remain;		/**< Amount of unwritten header data */
 	ssize_t data_remain;		/**< Data required to complete chunk */
@@ -141,7 +141,7 @@ static ssize_t
 chunk_begin(txdrv_t *tx, size_t len, gboolean final)
 {
 	struct attr *attr = tx->opaque;
-	gint hlen = 0;
+	size_t hlen = 0;
 
 	g_assert(0 == attr->data_remain);
 	g_assert(0 == attr->head_remain);
@@ -310,7 +310,7 @@ tx_chunk_write(txdrv_t *tx, gconstpointer data, size_t len)
 {
 	struct attr *attr = tx->opaque;
 	size_t remain = len;
-	const gchar *ptr = data;
+	const char *ptr = data;
 	size_t written = 0;
 
 	g_assert((size_t) -1 != len && len > 0);
@@ -358,7 +358,7 @@ tx_chunk_write(txdrv_t *tx, gconstpointer data, size_t len)
  * @return amount of bytes written, or -1 on error.
  */
 static ssize_t
-tx_chunk_writev(txdrv_t *tx, struct iovec *iov, gint iovcnt)
+tx_chunk_writev(txdrv_t *tx, struct iovec *iov, int iovcnt)
 {
 	ssize_t written = 0;
 
