@@ -1,5 +1,5 @@
 /*
- * $Id: tx_deflate.c 16106 2009-03-10 15:16:31Z rmanfredi $
+ * $Id: tx_deflate.c 17209 2009-11-08 21:37:28Z rmanfredi $
  *
  * Copyright (c) 2002-2003, Raphael Manfredi
  *
@@ -37,7 +37,7 @@
 
 #include "common.h"
 
-RCSID("$Id: tx_deflate.c 16106 2009-03-10 15:16:31Z rmanfredi $")
+RCSID("$Id: tx_deflate.c 17209 2009-11-08 21:37:28Z rmanfredi $")
 
 #include <zlib.h>
 
@@ -143,8 +143,8 @@ deflate_send(txdrv_t *tx)
 	r = tx_write(tx->lower, b->rptr, len);
 
 	if (GNET_PROPERTY(dbg) > 9)
-		printf("deflate_send: (%s) wrote %d bytes (buffer #%d) [%c%c]\n",
-			gnet_host_to_string(&tx->host), (int) r, attr->send_idx,
+		printf("deflate_send: (%s) wrote %lu bytes (buffer #%d) [%c%c]\n",
+			gnet_host_to_string(&tx->host), (unsigned long) r, attr->send_idx,
 			(attr->flags & DF_FLOWC) ? 'C' : '-',
 			(attr->flags & DF_FLUSH) ? 'f' : '-');
 
@@ -262,6 +262,16 @@ deflate_flush(txdrv_t *tx)
 	int ret;
 	int old_avail;
 
+	/*
+	 * Normally we come here with something already written, so outz->next_in
+	 * should be initialized.  If it is not the case, then it means we
+	 * never wrote anything, so there is nothing for us to do: we're simply
+	 * called through tx_deflate_close().
+	 */
+
+	if (NULL == outz->next_in)
+		goto done;
+
 retry:
 	b = &attr->buf[attr->fill_idx];	/* Buffer we fill */
 
@@ -284,7 +294,6 @@ retry:
 	outz->avail_in = 0;
 
 	g_assert(outz->avail_out > 0);
-	g_assert(outz->next_in);		/* We previously wrote something */
 
 	ret = deflate(outz, (tx->flags & TX_CLOSING) ? Z_FINISH : Z_SYNC_FLUSH);
 
@@ -340,6 +349,8 @@ retry:
 		goto retry;
 	}
 
+done:
+
 	attr->unflushed = 0;
 	attr->flags &= ~DF_FLUSH;
 
@@ -572,12 +583,12 @@ deflate_service(gpointer data)
 	g_assert(attr->send_idx < BUFFER_COUNT);
 
 	if (GNET_PROPERTY(dbg) > 9)
-		printf("deflate_service: (%s) %s(buffer #%d, %d bytes held) [%c%c]\n",
+		printf("deflate_service: (%s) %s(buffer #%d, %lu bytes held) [%c%c]\n",
 			gnet_host_to_string(&tx->host),
 			(tx->flags & TX_ERROR) ? "ERROR " : "",
 			attr->send_idx,
 			attr->send_idx >= 0 ?
-				(int) (attr->buf[attr->send_idx].wptr -
+				(unsigned long) (attr->buf[attr->send_idx].wptr -
 						attr->buf[attr->send_idx].rptr) : 0,
 			(attr->flags & DF_FLOWC) ? 'C' : '-',
 			(attr->flags & DF_FLUSH) ? 'f' : '-');
@@ -605,9 +616,9 @@ deflate_service(gpointer data)
 
 	if (b->wptr >= b->end) {
 		if (GNET_PROPERTY(dbg) > 9)
-			printf("deflate_service: (%s) sending fill buffer #%d, %d bytes\n",
+			printf("deflate_service: (%s) sending fill buffer #%d, %lu bytes\n",
 				gnet_host_to_string(&tx->host), attr->fill_idx,
-				(int) (b->wptr - b->rptr));
+				(unsigned long) (b->wptr - b->rptr));
 
 		deflate_rotate_and_send(tx);	/* Can set TX_ERROR */
 
@@ -699,9 +710,46 @@ tx_deflate_init(txdrv_t *tx, gpointer args)
 	outz->zfree = zlib_free_func;
 	outz->opaque = NULL;
 
-	ret = deflateInit2(outz, Z_DEFAULT_COMPRESSION, Z_DEFLATED,
-			targs->gzip ? (-MAX_WBITS) : MAX_WBITS,
-			MAX_MEM_LEVEL, Z_DEFAULT_STRATEGY);
+	/*
+	 * Reduce memory requirements for deflation when running as an ultrapeer.
+	 *
+	 * Memory used for deflation is:
+	 *
+	 *	(1 << (window_bits +2)) +  (1 << (mem_level + 9))
+	 *
+	 * For leaves, we use window_bits = 15 and mem_level = 9, which makes
+	 * for 128 KiB + 256 KiB = 384 KiB per connection (TX side).
+	 *
+	 * For ultra peers, we use window_bits = 14 and mem_level = 6, so this
+	 * uses 64 KiB + 32 KiB = 96 KiB only.
+	 *
+	 * Since ultra peers have many more connections than leaves, the memory
+	 * savings are drastic, yet compression levels remain around 50% (varies
+	 * depending on the nature of the traffic, of course).
+	 *
+	 *		--RAM, 2009-04-09
+	 */
+
+	{
+		int window_bits = MAX_WBITS;		/* Must be 8 .. MAX_WBITS */
+		int mem_level = MAX_MEM_LEVEL;		/* Must be 1 .. MAX_MEM_LEVEL */
+		int level = Z_BEST_COMPRESSION;
+
+		if (GNET_PROPERTY(current_peermode) == NODE_P_ULTRA) {
+			window_bits = 14;
+			mem_level = 6;
+			level = Z_DEFAULT_COMPRESSION;
+		}
+
+		g_assert(window_bits >= 8 && window_bits <= MAX_WBITS);
+		g_assert(mem_level >= 1 && mem_level <= MAX_MEM_LEVEL);
+		g_assert(level == Z_DEFAULT_COMPRESSION ||
+			(level >= Z_BEST_SPEED && level <= Z_BEST_COMPRESSION));
+
+		ret = deflateInit2(outz, level, Z_DEFLATED,
+				targs->gzip ? (-window_bits) : window_bits, mem_level,
+				Z_DEFAULT_STRATEGY);
+	}
 
 	if (Z_OK != ret) {
 		g_warning("unable to initialize compressor for peer %s: %s",
@@ -736,7 +784,7 @@ tx_deflate_init(txdrv_t *tx, gpointer args)
 
 	if (attr->gzip.enabled) {
 		/* See RFC 1952 - GZIP file format specification version 4.3 */
-		static const char header[] = {
+		static const unsigned char header[] = {
 			0x1f, 0x8b, /* gzip magic */
 			0x08,		/* compression method: deflate */
 			0,			/* flags: none */
