@@ -1,5 +1,5 @@
 /*
- * $Id: uhc.c 16106 2009-03-10 15:16:31Z rmanfredi $
+ * $Id: uhc.c 16990 2009-09-25 15:23:43Z rmanfredi $
  *
  * Copyright (c) 2004, Raphael Manfredi
  *
@@ -35,7 +35,7 @@
 
 #include "common.h"
 
-RCSID("$Id: uhc.c 16106 2009-03-10 15:16:31Z rmanfredi $")
+RCSID("$Id: uhc.c 16990 2009-09-25 15:23:43Z rmanfredi $")
 
 #include "gnutella.h"
 #include "guid.h"
@@ -53,7 +53,7 @@ RCSID("$Id: uhc.c 16106 2009-03-10 15:16:31Z rmanfredi $")
 #include "lib/endian.h"
 #include "lib/glib-missing.h"
 #include "lib/hashlist.h"
-#include "lib/misc.h"
+#include "lib/parse.h"
 
 #include "if/gnet_property_priv.h"
 #include "if/bridge/c2ui.h"
@@ -99,6 +99,8 @@ static const struct {
 #else	/* !USE_LOCAL_UHC */
 	{ "drei.gtkg.net:62666" },
 	{ "g6.dns6.org:1337" },
+	{ "gnutelladev1.udp-host-cache.com:1234" },
+	{ "gnutelladev2.udp-host-cache.com:5678" },
 	{ "gwc.ak-electron.eu:12060" },
 	{ "gwc.chickenkiller.com:8080" },
 	{ "leet.gtkg.org:1337" },
@@ -213,8 +215,8 @@ uhc_list_add(const char *host)
 	g_return_if_fail(host);
 
 	uhc = uhc_new(host);
-	if (hash_list_contains(uhc_list, uhc, NULL)) {
-		g_warning("Duplicate bootstrap UHC: \"%s\"", uhc->host);
+	if (hash_list_contains(uhc_list, uhc)) {
+		g_warning("duplicate bootstrap UHC: \"%s\"", uhc->host);
 		uhc_free(&uhc);
 		return;
 	}
