@@ -1,5 +1,5 @@
 /*
- * $Id: uploads.h 16106 2009-03-10 15:16:31Z rmanfredi $
+ * $Id: uploads.h 17307 2010-02-27 11:40:09Z rmanfredi $
  *
  * Copyright (c) 2001-2003, Raphael Manfredi
  * Copyright (c) 2000 Daniel Walker (dwalker@cats.ucsc.edu)
@@ -58,14 +58,14 @@ struct upload_http_cb {
 	time_t mtime;				/**< File modification time */
 };
 
-enum upload_magic { UPLOAD_MAGIC = 0xac20f063U };	/**< Magic number */
+enum upload_magic { UPLOAD_MAGIC = 0x2c20f063U };	/**< Magic number */
 
 struct upload {
 	enum upload_magic magic;
     gnet_upload_t upload_handle;
 	guint32 flags;					/**< Operating flags */
 	upload_stage_t status;
-	int error_sent;				/**< HTTP error code sent back */
+	int error_sent;					/**< HTTP error code sent back */
 	gpointer io_opaque;				/**< Opaque I/O callback information */
 	struct parq_ul_queued *parq_ul;	/**< PARQ information */
 
@@ -107,7 +107,8 @@ struct upload {
 	guint16 gnet_port;			/**< Advertised Gnet port, for browsing */
 	guint16 country;			/**< Country of origin, ISO3166 code */
 
-	const char *user_agent;	/**< Remote user agent */
+	const char *user_agent;		/**< Remote user agent */
+	const struct guid *guid;	/**< Remote servent GUID (atom), if known */
 
 	filesize_t file_size;
 	filesize_t skip;			/**< First byte to send, inclusive */
@@ -117,22 +118,23 @@ struct upload {
 	filesize_t total_requested;	/**< Total amount of bytes requested */
 	filesize_t downloaded;		/**< What they claim as downloaded so far */
 
-	int http_major;			/**< HTTP major version */
-	int http_minor;			/**< HTTP minor version */
+	int http_major;				/**< HTTP major version */
+	int http_minor;				/**< HTTP minor version */
 
-	gboolean keep_alive;		/**< Keep HTTP connection? */
-	gboolean push;
-	gboolean queue;				/**< Similar to PUSH, but for PARQ's QUEUE */
-	gboolean accounted;			/**< True when upload was accounted for */
-	gboolean n2r;				/**< True when they sent an N2R request */
-	gboolean browse_host;		/**< True when they sent a Browse Host req. */
-	gboolean from_browser;		/**< True when request likely from browser */
-	gboolean head_only;
-	gboolean is_followup;
-	gboolean was_actively_queued;
-	gboolean was_running;
-	gboolean last_was_error;	/**< Whether last request was an error */
-	gboolean parq_status;
+	unsigned keep_alive:1;		/**< Keep HTTP connection? */
+	unsigned push:1;
+	unsigned queue:1;			/**< Similar to PUSH, but for PARQ's QUEUE */
+	unsigned accounted:1;		/**< True when upload was accounted for */
+	unsigned n2r:1;				/**< True when they sent an N2R request */
+	unsigned browse_host:1;		/**< True when they sent a Browse Host req. */
+	unsigned from_browser:1;	/**< True when request likely from browser */
+	unsigned head_only:1;
+	unsigned is_followup:1;
+	unsigned was_actively_queued:1;
+	unsigned was_running:1;
+	unsigned last_was_error:1;	/**< Whether last request was an error */
+	unsigned parq_status:1;
+	unsigned fwalt:1;			/**< Downloader accepts firewalled locations */
 };
 
 static inline void
@@ -158,6 +160,9 @@ upload_is_special(const struct upload *u)
  */
 
 enum {
+	UPLOAD_F_NORMAL_LIMIT	= 1 << 5,	/**< Normal limits */
+	UPLOAD_F_STEALTH_LIMIT	= 1 << 4,	/**< Stealth limits */
+	UPLOAD_F_LIMITED		= 1 << 3,	/**< Subject to limitation */
 	UPLOAD_F_WAS_PLAIN		= 1 << 2,	/**< Prev request was for plain file  */
 	UPLOAD_F_EARLY_STALL	= 1 << 1,	/**< Pre-stalling condition */
 	UPLOAD_F_STALLED		= 1 << 0	/**< Stall condition present */
