@@ -1,5 +1,5 @@
 /*
- * $Id: version.c 16106 2009-03-10 15:16:31Z rmanfredi $
+ * $Id: version.c 17247 2010-02-14 16:14:14Z rmanfredi $
  *
  * Copyright (c) 2002-2003, Raphael Manfredi
  *
@@ -35,7 +35,9 @@
 
 #include "common.h"
 
-RCSID("$Id: version.c 16106 2009-03-10 15:16:31Z rmanfredi $")
+RCSID("$Id: version.c 17247 2010-02-14 16:14:14Z rmanfredi $")
+
+#include "gtk-gnutella.h"
 
 #include "version.h"
 #include "token.h"
@@ -50,7 +52,9 @@ RCSID("$Id: version.c 16106 2009-03-10 15:16:31Z rmanfredi $")
 #include "lib/ascii.h"
 #include "lib/getdate.h"
 #include "lib/glib-missing.h"
+#include "lib/parse.h"
 #include "lib/tm.h"
+#include "lib/timestamp.h"
 #include "lib/utf8.h"
 #include "lib/override.h"		/* Must be the last header included */
 
@@ -88,8 +92,8 @@ version_get_code(void)
 static void
 version_dump(const char *str, const version_t *ver, const char *cmptag)
 {
-	printf("VERSION%s \"%s\":\n"
-		"\tmajor=%u minor=%u patch=%u tag=%c taglevel=%u build=%u\n",
+	g_message("VERSION%s \"%s\": "
+		"major=%u minor=%u patch=%u tag=%c taglevel=%u build=%u",
 		cmptag, str, ver->major, ver->minor, ver->patchlevel,
 		ver->tag ? ver->tag : ' ', ver->taglevel, ver->build);
 }
@@ -280,7 +284,7 @@ version_parse(const char *str, version_t *ver)
 	} else
 		ver->build = 0;
 
-	if (GNET_PROPERTY(dbg) > 6)
+	if (GNET_PROPERTY(version_debug) > 1)
 		version_dump(str, ver, "#");
 
 	return TRUE;
@@ -428,7 +432,7 @@ version_check(const char *str, const char *token, const host_addr_t addr)
 
 	cmp = version_cmp(target_version, &their_version);
 
-	if (GNET_PROPERTY(dbg) > 6)
+	if (GNET_PROPERTY(version_debug) > 1)
 		version_dump(str, &their_version,
 			cmp == 0 ? "=" :
 			cmp > 0 ? "-" : "+");
@@ -439,8 +443,8 @@ version_check(const char *str, const char *token, const host_addr_t addr)
 
 	version_stamp(str, &their_version);
 
-	if (GNET_PROPERTY(dbg) > 6)
-		printf("VERSION time=%d\n", (int) their_version.timestamp);
+	if (GNET_PROPERTY(version_debug) > 3)
+		g_message("VERSION time=%d", (int) their_version.timestamp);
 
 	/*
 	 * If version claims something older than TOKEN_START_DATE, then
@@ -451,7 +455,7 @@ version_check(const char *str, const char *token, const host_addr_t addr)
 		tok_error_t error;
 
 		if (token == NULL) {
-            if (GNET_PROPERTY(dbg)) {
+            if (GNET_PROPERTY(version_debug)) {
                 g_message("got GTKG vendor string \"%s\" without token!", str);
             }
 			return FALSE;	/* Can't be correct */
@@ -474,7 +478,7 @@ version_check(const char *str, const char *token, const host_addr_t addr)
 			error = TOK_OK;		/* Our keys have expired, cannot validate */
 
 		if (error != TOK_OK) {
-            if (GNET_PROPERTY(dbg)) {
+            if (GNET_PROPERTY(version_debug)) {
                 g_message("vendor string \"%s\" [%s] has wrong token "
                     "\"%s\": %s ", str, host_addr_to_string(addr), token,
                     tok_strerror(error));
@@ -504,12 +508,12 @@ version_check(const char *str, const char *token, const host_addr_t addr)
 			|| their_version.build > target_version->build) &&
 		target_version == &last_rel_version
 	) {
-		if (GNET_PROPERTY(dbg) > 6)
-			printf("VERSION is a SVN update of a release\n");
+		if (GNET_PROPERTY(version_debug) > 3)
+			g_message("VERSION is a SVN update of a release");
 
 		if (version_build_cmp(&last_dev_version, &their_version) > 0) {
-			if (GNET_PROPERTY(dbg) > 6)
-				printf("VERSION is less recent than latest dev we know\n");
+			if (GNET_PROPERTY(version_debug) > 3)
+				g_message("VERSION is less recent than latest dev we know");
 			return TRUE;
 		}
 		target_version = &last_dev_version;
@@ -539,8 +543,8 @@ version_check(const char *str, const char *token, const host_addr_t addr)
 	 * We found a more recent version than the last version seen.
 	 */
 
-	if (GNET_PROPERTY(dbg) > 4)
-		printf("more recent %s VERSION \"%s\"\n",
+	if (GNET_PROPERTY(version_debug) > 1)
+		g_message("more recent %s VERSION \"%s\"",
 			target_version == &last_dev_version ? "dev" : "rel", str);
 
 	*target_version = their_version;		/* struct copy */
@@ -677,8 +681,7 @@ version_init(void)
  *
  * If the version being ran is not a stable one, warn after 60 days, otherwise
  * warn after a year.  If we're not "expired" yet but are approaching the
- * deadline, start to remind them.  After the deadline plus some grace period,
- * refuse to run unless a special property is set.
+ * deadline, start to remind them. 
  */
 void
 version_ancient_warn(void)
@@ -701,7 +704,16 @@ version_ancient_warn(void)
 	elapsed = delta_time(now, our_version.timestamp);
 
 	if (elapsed > VERSION_ANCIENT_WARN || tok_is_ancient(now)) {
-		g_warning("version of gtk-gnutella is too old, you should upgrade!");
+		static gboolean warned = FALSE;
+		if (GNET_PROPERTY(version_debug)) {
+			g_message("VERSION our_version = %s (elapsed = %ld, token %s)",
+				timestamp_to_string(our_version.timestamp),
+				(long) elapsed, tok_is_ancient(now) ? "ancient" : "ok");
+		}
+		if (!warned) {
+			g_warning("version of gtk-gnutella is too old, please upgrade!");
+			warned = TRUE;
+		}
         gnet_prop_set_boolean_val(PROP_ANCIENT_VERSION, TRUE);
 		return;
 	}
