@@ -1,5 +1,5 @@
 /*
- * $Id: vmsg.c 16211 2009-03-19 16:06:24Z rmanfredi $
+ * $Id: vmsg.c 17365 2010-03-06 19:15:17Z rmanfredi $
  *
  * Copyright (c) 2003, Raphael Manfredi
  *
@@ -35,7 +35,7 @@
 
 #include "common.h"
 
-RCSID("$Id: vmsg.c 16211 2009-03-19 16:06:24Z rmanfredi $")
+RCSID("$Id: vmsg.c 17365 2010-03-06 19:15:17Z rmanfredi $")
 
 #include "clock.h"
 #include "dmesh.h"
@@ -65,6 +65,8 @@ RCSID("$Id: vmsg.c 16211 2009-03-19 16:06:24Z rmanfredi $")
 
 #include "if/gnet_property.h"
 #include "if/gnet_property_priv.h"
+#include "if/dht/dht.h"			/* For dht_enabled() */
+#include "if/dht/kademlia.h"	/* For KDA_VERSION_* */
 
 #include "lib/array.h"
 #include "lib/atoms.h"
@@ -72,7 +74,9 @@ RCSID("$Id: vmsg.c 16211 2009-03-19 16:06:24Z rmanfredi $")
 #include "lib/endian.h"
 #include "lib/glib-missing.h"
 #include "lib/hashlist.h"
+#include "lib/patricia.h"
 #include "lib/pmsg.h"
+#include "lib/timestamp.h"
 #include "lib/tm.h"
 #include "lib/urn.h"
 #include "lib/vendors.h"
@@ -229,7 +233,7 @@ vmsg_handle(struct gnutella_node *n)
 
 	if (n->size < expected_size) {
 		gnet_stats_count_dropped(n, MSG_DROP_TOO_SMALL);
-		if (GNET_PROPERTY(dbg) || GNET_PROPERTY(vmsg_debug))
+		if (GNET_PROPERTY(vmsg_debug))
 			gmsg_log_bad(n, "message has only %u bytes, needs at least %u",
 				(unsigned) n->size, expected_size);
 		return;
@@ -243,7 +247,7 @@ vmsg_handle(struct gnutella_node *n)
 
 	if (GNET_PROPERTY(vmsg_debug) > 4)
 		g_message("VMSG %s \"%s\": %s/%uv%u from %s",
-			gmsg_infostr(&n->header), found ? vmsg.name : "UNKNOWN",
+			gmsg_node_infostr(n), found ? vmsg.name : "UNKNOWN",
 			vendor_code_to_string(vc.u32), id, version,
 			host_addr_port_to_string(n->addr, n->port));
 
@@ -259,7 +263,7 @@ vmsg_handle(struct gnutella_node *n)
 		(*vmsg.handler)(n, &vmsg, n->data + sizeof(*v), n->size - sizeof(*v));
 	} else {
 		gnet_stats_count_dropped(n, MSG_DROP_UNKNOWN_TYPE);
-		if (GNET_PROPERTY(dbg) || GNET_PROPERTY(vmsg_debug))
+		if (GNET_PROPERTY(vmsg_debug))
 			gmsg_log_bad(n, "unknown vendor message");
 	}
 }
@@ -338,8 +342,8 @@ vmsg_bad_payload(struct gnutella_node *n,
 	n->n_bad++;
 	gnet_stats_count_dropped(n, MSG_DROP_BAD_SIZE);
 
-	if (GNET_PROPERTY(dbg) || GNET_PROPERTY(vmsg_debug))
-		gmsg_log_bad(n, "Bad payload size %lu for %s/%dv%d (%s), expected %lu",
+	if (GNET_PROPERTY(vmsg_debug))
+		gmsg_log_bad(n, "Bad payload size %lu for %s/%uv%u (%s), expected %lu",
 			(gulong) size, vendor_code_to_string(vmsg->vendor), vmsg->id,
 			vmsg->version, vmsg->name, (gulong) expected);
 
@@ -403,6 +407,22 @@ handle_features_supported(struct gnutella_node *n,
 		if (0 != version && 0 == strcmp(feature, "TLS!")) {
 			node_supports_tls(n);
 		}
+
+		/* Any of ADHT, PDHT or LDHT means DHT is supported */
+		if (feature[0] && 0 == strcmp(&feature[1], "DHT")) {
+			dht_mode_t mode;
+			gboolean known = TRUE;
+			switch (feature[0]) {
+			case 'A': mode = DHT_MODE_ACTIVE; break;
+			case 'P': mode = DHT_MODE_PASSIVE; break;
+			case 'L': mode = DHT_MODE_PASSIVE_LEAF; break;
+			case 'I': mode = DHT_MODE_INACTIVE; break;
+			default:  known = FALSE; break;
+			}
+			if (known) {
+				node_supports_dht(n, mode);
+			}
+		}
 	}
 }
 
@@ -630,13 +650,12 @@ handle_proxy_req(struct gnutella_node *n,
 	 * Add proxying info for this node.  On successful completion,
 	 * we'll send an acknowledgement.
 	 *
-	 * We'll reply with a message at the same version as the one we got.
+	 * We always use version 2 to reply, see comment in vmsg_send_proxy_req().
 	 */
 
 	if (node_proxying_add(n, gnutella_header_get_muid(&n->header))) {
 		/* MUID is the node's GUID */
-		vmsg_send_proxy_ack(n, gnutella_header_get_muid(&n->header),
-			vmsg->version);
+		vmsg_send_proxy_ack(n, gnutella_header_get_muid(&n->header), 2);
 	}
 }
 
@@ -653,12 +672,28 @@ vmsg_send_proxy_req(struct gnutella_node *n, const struct guid *muid)
 
 	msgsize = vmsg_fill_header(v_tmp_header, 0, sizeof v_tmp);
 	gnutella_header_set_muid(v_tmp_header, muid);
-	(void) vmsg_fill_type(v_tmp_data, T_LIME, 21, 2);
+
+	/*
+	 * LimeWire only supports version 1 of the Push Proxy Request but will
+	 * send a Push Proxy Ack at version 2.  They did not think that people
+	 * would be using the version of the request message to indicate the
+	 * level of support for the acknowledgement.
+	 *
+	 * So from now on, stick to sending version 1, and always reply with
+	 * version 2, regardless of what they send us, thereby mimicking their
+	 * (broken) behaviour.  Nowadays, everybody should support the version 2
+	 * of the acknowledgement anyway.
+	 *		--RAM, 2009-10-30
+	 */
+
+	(void) vmsg_fill_type(v_tmp_data, T_LIME, 21, 1);
 
 	gmsg_sendto_one(n, v_tmp, msgsize);
 
-	if (GNET_PROPERTY(vmsg_debug) > 2)
-		g_message("sent proxy REQ to %s <%s>", node_addr(n), node_vendor(n));
+	if (GNET_PROPERTY(vmsg_debug) > 2) {
+		g_message("VMSG sent proxy REQ to %s <%s>",
+			node_addr(n), node_vendor(n));
+	}
 }
 
 /**
@@ -686,10 +721,11 @@ handle_proxy_ack(struct gnutella_node *n,
 
 	port = peek_le16(payload);
 
-	if (GNET_PROPERTY(vmsg_debug) > 2)
-		g_message("got proxy ACK from %s <%s>: proxy at %s",
+	if (GNET_PROPERTY(vmsg_debug) > 2) {
+		g_message("VMSG got proxy ACK v%u from %s <%s>: proxy at %s",
+			vmsg->version,
 			node_addr(n), node_vendor(n), host_addr_port_to_string(ha, port));
-
+	}
 
 	if (!host_is_valid(ha, port)) {
 		g_warning("got improper address %s in %s from %s <%s>",
@@ -698,7 +734,7 @@ handle_proxy_ack(struct gnutella_node *n,
 		return;
 	}
 	if (hostiles_check(ha)) {
-		g_message("got proxy ACK from hostile host %s <%s>: proxy at %s",
+		g_message("VMSG got proxy ACK from hostile host %s <%s>: proxy at %s",
 			node_addr(n), node_vendor(n), host_addr_port_to_string(ha, port));
 		return;
 	}
@@ -798,7 +834,8 @@ vmsg_send_qstat_answer(struct gnutella_node *n,
 
 	if (GNET_PROPERTY(vmsg_debug) > 2)
 		g_message("VMSG sending %s with hits=%u to %s <%s>",
-			gmsg_infostr_full(v_tmp), hits, node_addr(n), node_vendor(n));
+			gmsg_infostr_full(v_tmp, msgsize),
+			hits, node_addr(n), node_vendor(n));
 
 	gmsg_ctrl_sendto_one(n, v_tmp, msgsize);	/* Send it ASAP */
 }
@@ -834,7 +871,8 @@ vmsg_send_proxy_cancel(struct gnutella_node *n)
 	gmsg_sendto_one(n, v_tmp, msgsize);
 
 	if (GNET_PROPERTY(vmsg_debug) > 2)
-		g_message("sent proxy CANCEL to %s <%s>", node_addr(n), node_vendor(n));
+		g_message("VMSG sent proxy CANCEL to %s <%s>",
+			node_addr(n), node_vendor(n));
 }
 
 /**
@@ -1005,15 +1043,23 @@ handle_oob_reply_ack(struct gnutella_node *n,
 	oob_deliver_hits(n, gnutella_header_get_muid(&n->header), wanted, &token);
 }
 
+/**
+ * Node does not want us to OOB-proxy its queries.
+ */
 static void
 handle_oob_proxy_veto(struct gnutella_node *n,
 	const struct vmsg *vmsg, const char *payload, size_t size)
 {
+	const char *msg = NULL;
+
 	if (NODE_IS_UDP(n)) {
-		g_warning("got %s/%uv%u from TCP via %s, ignoring",
-			vendor_code_to_string(vmsg->vendor),
-			vmsg->id, vmsg->version, node_addr(n));
-		return;
+		msg = "UDP";
+		goto ignore;
+	}
+
+	if (!NODE_IS_LEAF(n)) {
+		msg = "non-leaf node";
+		goto ignore;
 	}
 
 	if (size > 0 && peek_u8(payload) < 3) {
@@ -1022,6 +1068,15 @@ handle_oob_proxy_veto(struct gnutella_node *n,
 	} else {
 		n->flags |= NODE_F_NO_OOB_PROXY;
 	}
+
+	return;
+
+ignore:
+	if (GNET_PROPERTY(vmsg_debug)) {
+		g_warning("got %s/%uv%u from %s via %s, ignoring",
+			vendor_code_to_string(vmsg->vendor),
+			vmsg->id, vmsg->version, msg, node_addr(n));
+	}
 }
 
 /**
@@ -1061,7 +1116,7 @@ vmsg_send_oob_reply_ack(struct gnutella_node *n,
 	udp_send_msg(n, v_tmp, msgsize);
 
 	if (GNET_PROPERTY(vmsg_debug) > 2)
-		g_message("sent OOB reply ACK %s to %s for %u hit%s",
+		g_message("VMSG sent OOB reply ACK %s to %s for %u hit%s",
 			guid_hex_str(muid), node_addr(n), want, want == 1 ? "" : "s");
 }
 
@@ -1376,7 +1431,7 @@ vmsg_send_udp_crawler_pong(struct gnutella_node *n, pmsg_t *mb)
 		guint8 nleaves = peek_u8(&payload[1]);
 
 		g_message("VMSG sending %s with up=%u and leaves=%u to %s",
-			gmsg_infostr_full(v_tmp), nup, nleaves, node_addr(n));
+			gmsg_infostr_full(v_tmp, msgsize), nup, nleaves, node_addr(n));
 	}
 
 	udp_send_msg(n, v_tmp, msgsize);
@@ -1600,7 +1655,7 @@ latest_svn_release_changed(property_t prop)
 		signature = array_init(data, data_length);
 
 		if (svn_release_notification_verify(revision, date, &signature)) {
-			g_message("SVN release notify signature is valid: r%u (%s)",
+			g_message("VMSG SVN release notify signature is valid: r%u (%s)",
 				revision, timestamp_to_string(date));
 
 			memcpy(svn_release_signature.data, data, data_length);
@@ -1700,7 +1755,7 @@ handle_svn_release_notify(struct gnutella_node *n,
 		gnet_prop_set_string(PROP_LATEST_SVN_RELEASE_SIGNATURE, hex);
 		G_FREE_NULL(hex);
 	} else {
-		g_message("BAD %s v%u from %s over %s (TTL=%u, hops=%u, size=%lu)",
+		g_message("VMSG BAD %s v%u from %s over %s (TTL=%u, hops=%u, size=%lu)",
 			vmsg->name,
 			vmsg->version,
 			node_addr(n),
@@ -1731,7 +1786,7 @@ enum {
 	VMSG_HEAD_STATUS_FIREWALLED	 = 1 << 2,
 	VMSG_HEAD_STATUS_DOWNLOADING = 1 << 3,
 	
-	VMSG_HEAD_CODE_MASK			= 0x03,
+	VMSG_HEAD_CODE_MASK			= 0x03
 };
 
 /**
@@ -1819,7 +1874,7 @@ vmsg_send_head_pong_v1(struct gnutella_node *n, const struct sha1 *sha1,
 	paysize = p - payload;
 
 	if (GNET_PROPERTY(vmsg_debug) > 1) {
-		g_message("sending HEAD Pong v1 to %s (%u bytes)",
+		g_message("VMSG sending HEAD Pong v1 to %s (%u bytes)",
 			node_addr(n), paysize);
 	}
 
@@ -1934,7 +1989,7 @@ vmsg_send_head_pong_v2(struct gnutella_node *n, const struct sha1 *sha1,
 	paysize += ggep_len;
 
 	if (GNET_PROPERTY(vmsg_debug) > 1) {
-		g_message("sending HEAD Pong v2 to %s (%u bytes)",
+		g_message("VMSG sending HEAD Pong v2 to %s (%u bytes)",
 			node_addr(n), paysize);
 	}
 
@@ -2034,7 +2089,7 @@ head_ping_register_intern(const struct guid *muid,
 			return NULL;
 		}
 	}
-	if (hash_list_contains(head_pings, muid, NULL)) {
+	if (hash_list_contains(head_pings, muid)) {
 		/* Probably a duplicate */
 		return NULL;
 	}
@@ -2126,7 +2181,7 @@ head_ping_is_registered(const struct guid *muid)
 }
 
 /**
- * Send a "HEAD Ping" -- LIME/23v1
+ * Send a "HEAD Ping" -- LIME/23v2
  *
  * This message is used to gather information about an urn:sha1, such as
  * getting more alternate location, or the list of available ranges.
@@ -2197,7 +2252,7 @@ vmsg_send_head_ping(const struct sha1 *sha1, host_addr_t addr, guint16 port,
 	if (head_ping_register_own(muid, sha1, n)) {
 		if (GNET_PROPERTY(vmsg_debug) > 1) {
 			g_message(
-				"sending HEAD Ping to %s (%u bytes) for urn:sha1:%s",
+				"VMSG sending HEAD Ping to %s (%u bytes) for urn:sha1:%s",
 					node_addr(n), paysize, sha1_base32(sha1));
 		}
 		vmsg_send_data(n, v_tmp, msgsize);
@@ -2255,7 +2310,8 @@ head_ping_target_by_guid(const struct guid *guid)
 	if (n) {
 	   	if (!(NODE_A_CAN_HEAD & n->attrs)) {
 			if (GNET_PROPERTY(vmsg_debug)) {
-				g_message("HEAD Ping target %s does not support HEAD pings",
+				g_message(
+					"VMSG HEAD Ping target %s does not support HEAD pings",
 					node_addr(n));
 			}
 			n = NULL;
@@ -2292,7 +2348,7 @@ handle_head_ping(struct gnutella_node *n,
 		return;
 
 	if (GNET_PROPERTY(vmsg_debug) > 1) {
-		g_message("got %s v%u from %s over %s (TTL=%u, hops=%u, size=%lu)",
+		g_message("VMSG got %s v%u from %s over %s (TTL=%u, hops=%u, size=%lu)",
 			vmsg->name,
 			vmsg->version,
 			node_addr(n),
@@ -2302,13 +2358,17 @@ handle_head_ping(struct gnutella_node *n,
 			(unsigned long) size);
 	}
 
+	if (NODE_IS_UDP(n))
+		inet_udp_got_unsolicited_incoming();
+
 	flags = peek_u8(&payload[0]);
 	if (
 		is_strcaseprefix(&payload[1], urn_prefix) &&
 		urn_get_sha1(&payload[1], &sha1)
 	) {
 		if (GNET_PROPERTY(vmsg_debug)) {
-			g_message("HEAD Ping for %s%s", urn_prefix, sha1_to_string(sha1));
+			g_message("VMSG HEAD Ping for %s%s",
+				urn_prefix, sha1_to_string(&sha1));
 		}
 	} else {
 		if (GNET_PROPERTY(vmsg_debug)) {
@@ -2333,7 +2393,8 @@ handle_head_ping(struct gnutella_node *n,
 		}
 		if (has_guid) {
 		   	if (GNET_PROPERTY(vmsg_debug) > 1) {
-				g_message("HEAD Ping carries GUID %s", guid_hex_str(&guid));
+				g_message("VMSG HEAD Ping carries GUID %s",
+					guid_hex_str(&guid));
 			}
 		} else {
 		   	if (GNET_PROPERTY(vmsg_debug) > 1) {
@@ -2347,13 +2408,13 @@ handle_head_ping(struct gnutella_node *n,
 
 		if (NODE_P_LEAF == GNET_PROPERTY(current_peermode)) {
 		   	if (GNET_PROPERTY(vmsg_debug)) {
-				g_message("not forwarding HEAD Ping as leaf");
+				g_message("VMSG not forwarding HEAD Ping as leaf");
 			}
 			return;
 		}
 		if (gnutella_header_get_hops(&n->header) > 0) {
 		   	if (GNET_PROPERTY(vmsg_debug)) {
-				g_message("not forwarding forwarded HEAD Ping");
+				g_message("VMSG not forwarding forwarded HEAD Ping");
 			}
 			return;
 		}
@@ -2370,14 +2431,15 @@ handle_head_ping(struct gnutella_node *n,
 
 			if (head_ping_register_forwarded(muid, &sha1, n)) {
 				if (GNET_PROPERTY(vmsg_debug) > 1) {
-					g_message("forwarding HEAD Ping to %s", node_addr(target));
+					g_message("VMSG forwarding HEAD Ping to %s",
+						node_addr(target));
 				}
 				gmsg_split_sendto_one(target, header, n->data,
 					GTA_HEADER_SIZE + n->size);
 			}
 		} else {
 			if (GNET_PROPERTY(vmsg_debug)) {
-				g_message("no route found for HEAD Ping");
+				g_message("VMSG no route found for HEAD Ping");
 			}
 		}
 	} else {
@@ -2393,7 +2455,7 @@ handle_head_ping(struct gnutella_node *n,
 			 * (404).
 			 */
 			if (GNET_PROPERTY(vmsg_debug)) {
-				g_message("HEAD Ping whilst rebuilding library");
+				g_message("VMSG HEAD Ping whilst rebuilding library");
 			}
 		} else {
 			if (sf) {
@@ -2403,7 +2465,7 @@ handle_head_ping(struct gnutella_node *n,
 				fi = shared_file_fileinfo(sf);
 				if (fi) {
 					if (GNET_PROPERTY(vmsg_debug)) {
-						g_message("HEAD Ping for partial file");
+						g_message("VMSG HEAD Ping for partial file");
 					}
 					if (GNET_PROPERTY(pfsp_server)) {
 						code = VMSG_HEAD_CODE_PARTIAL;
@@ -2415,13 +2477,13 @@ handle_head_ping(struct gnutella_node *n,
 					}
 				}  else {
 					if (GNET_PROPERTY(vmsg_debug)) {
-						g_message("HEAD Ping for shared file");
+						g_message("VMSG HEAD Ping for shared file");
 					}
 					code = VMSG_HEAD_CODE_COMPLETE;
 				}
 			} else {
 				if (GNET_PROPERTY(vmsg_debug)) {
-					g_message("HEAD Ping for unknown file");
+					g_message("VMSG HEAD Ping for unknown file");
 				}
 				code = VMSG_HEAD_CODE_NOT_FOUND;
 			}
@@ -2511,7 +2573,7 @@ forward_head_pong(struct gnutella_node *n,
 			pmsg_t *mb;
 
 			if (GNET_PROPERTY(vmsg_debug)) {
-				g_message("Forwarding HEAD Pong to %s", node_addr(target));
+				g_message("VMSG forwarding HEAD Pong to %s", node_addr(target));
 			}
 
 			memcpy(header, n->header, GTA_HEADER_SIZE);
@@ -2569,7 +2631,7 @@ handle_head_pong_v1(const struct head_ping_source *source,
 
 	if (GNET_PROPERTY(vmsg_debug) > 1) {
 		g_message(
-			"HEAD Pong v1 vendor=%s, %s%s, result=\"%s%s%s\", queue=%d",
+			"VMSG HEAD Pong v1 vendor=%s, %s%s, result=\"%s%s%s\", queue=%d",
 			vendor,
 			source->ping.sha1 ? "urn:sha1:" : "<unknown hash>",
 			source->ping.sha1 ? sha1_base32(source->ping.sha1) : "",
@@ -2616,7 +2678,7 @@ handle_head_pong_v1(const struct head_ping_source *source,
 			return;
 		} else {
 			if (GNET_PROPERTY(vmsg_debug)) {
-				g_message("HEAD Pong carries ranges (%u bytes)", len);
+				g_message("VMSG HEAD Pong carries ranges (%u bytes)", len);
 			}
 			p += 2;
 			p += len;
@@ -2635,7 +2697,8 @@ handle_head_pong_v1(const struct head_ping_source *source,
 			return;
 		} else {
 			if (GNET_PROPERTY(vmsg_debug)) {
-				g_message("HEAD Pong carries firewalled alt-locs (%u bytes)",
+				g_message(
+					"VMSG HEAD Pong carries firewalled alt-locs (%u bytes)",
 					len);
 			}
 			p += 2;
@@ -2658,7 +2721,7 @@ handle_head_pong_v1(const struct head_ping_source *source,
 			return;
 		} else {
 			if (GNET_PROPERTY(vmsg_debug))
-				g_message("HEAD Pong carries %u alt-locs", len / 6);
+				g_message("VMSG HEAD Pong carries %u alt-locs", len / 6);
 
 			p += 2;				/* Skip length indication */
 			if (node_id_self(source->ping.node_id) && source->ping.sha1) {
@@ -2743,11 +2806,11 @@ handle_head_pong_v2(const struct head_ping_source *source,
 				const char *name = ext_ggep_id_str(e);
 
 				if (name[0]) {
-					g_message(
-						"HEAD Pong carries unhandled GGEP \"%s\" (%lu byte)",
+					g_message("VMSG HEAD Pong carries unhandled "
+						"GGEP \"%s\" (%lu byte)",
 						name, (unsigned long) ext_paylen(e));
 				} else {
-					g_message("HEAD Pong carries unknown extra payload");
+					g_message("VMSG HEAD Pong carries unknown extra payload");
 				}
 			}
 			break;
@@ -2759,7 +2822,7 @@ handle_head_pong_v2(const struct head_ping_source *source,
 
 	if (GNET_PROPERTY(vmsg_debug) > 1) {
 		g_message(
-			"HEAD Pong v2 vendor=%s, %s%s, result=\"%s%s%s\", queue=%d",
+			"VMSG HEAD Pong v2 vendor=%s, %s%s, result=\"%s%s%s\", queue=%d",
 			vendor,
 			source->ping.sha1 ? "urn:sha1:" : "<unknown hash>",
 			source->ping.sha1 ? sha1_base32(source->ping.sha1) : "",
@@ -2816,7 +2879,7 @@ handle_head_pong(struct gnutella_node *n,
 		return;
 
 	if (GNET_PROPERTY(vmsg_debug) > 1) {
-		g_message("got %s v%u from %s over %s (TTL=%u, hops=%u, size=%lu)",
+		g_message("VMSG got %s v%u from %s over %s (TTL=%u, hops=%u, size=%lu)",
 			vmsg->name,
 			vmsg->version,
 			node_addr(n),
@@ -3101,6 +3164,9 @@ vmsg_features_add(struct vmsg_features *vmf, const char *name, guint16 version)
 	vmf->pos += 6;
 }
 
+/**
+ * Tell node about which features we're currently supporting.
+ */
 void
 vmsg_send_features_supported(struct gnutella_node *n)
 {
@@ -3113,17 +3179,121 @@ vmsg_send_features_supported(struct gnutella_node *n)
 	vmsg_features_reset(&vmf, payload, VMSG_PAYLOAD_MAX);
 
 	vmsg_features_add(&vmf, "HSEP", 1);
-	vmsg_features_add(&vmf, "F2FT", 0); /* No support for NAT-to-NAT */
+	/* No support for NAT-to-NAT -- signal version as -1, not 0 */
+	vmsg_features_add(&vmf, "F2FT", (guint16) -1);
+	/* TCP-incoming connections: are possible if not firewalled */
 	vmsg_features_add(&vmf, "TCPI", GNET_PROPERTY(is_firewalled) ? 0 : 1);
 	if (tls_enabled()) {
 		vmsg_features_add(&vmf, "TLS!", 1);
 	}
+	if (dht_enabled()) {
+		/* DHT mode: ADHT = active, PDHT = passive, LDHT = leaf */
+		vmsg_features_add(&vmf,
+			GNET_PROPERTY(is_firewalled) ? "PDHT" : "ADHT",
+			(KDA_VERSION_MAJOR << 8) + KDA_VERSION_MINOR);
+	}
 
 	paysize = vmsg_features_get_length(&vmf);
 	msgsize = vmsg_fill_header(v_tmp_header, paysize, sizeof v_tmp);
 	gmsg_sendto_one(n, v_tmp, msgsize);
 }
 
+/**
+ * Definition of vendor message sorting weight, for gmsg_cmp().
+ * Note that we don't care about the message version here.
+ */
+struct vmsg_weight {
+	guint32 vendor;
+	guint16 id;
+	int weight;
+};
+
+static const struct vmsg_weight vmsg_weight_map[] = {
+	{ T_0000,  0,  7 },		/* Messages Supported */
+	{ T_0000, 10,  7 },		/* Features Supported */
+	{ T_BEAR,  4,  8 },		/* Hops Flow */
+	{ T_BEAR,  7,  3 },		/* TCP Connect Back */
+	{ T_BEAR, 11,  6 },		/* Query Status Request */
+	{ T_BEAR, 12,  8 },		/* Query Status Response */
+	{ T_GTKG,  7,  3 },		/* UDP Connect Back */
+	{ T_GTKG,  9,  1 },		/* Time Sync Request */
+	{ T_GTKG, 10,  2 },		/* Time Sync Reply */
+	{ T_GTKG, 21,  0 },		/* Push-Proxy Cancel */
+	{ T_GTKG, 22,  0 },		/* Node Info Request */
+	{ T_GTKG, 23,  0 },		/* Node Info Reply */
+	{ T_GTKG, 24,  9 },		/* SVN Release Notify */
+	{ T_LIME,  5,  1 },		/* UDP Crawler Ping */
+	{ T_LIME, 11,  6 },		/* OOB Reply ACK */
+	{ T_LIME, 12,  7 },		/* OOB Reply Indication */
+	{ T_LIME, 13,  2 },		/* OOB Proxy Veto */
+	{ T_LIME, 21,  8 },		/* Push-Proxy Request */
+	{ T_LIME, 22,  8 },		/* Push-Proxy ACK */
+	{ T_LIME, 23,  4 },		/* HEAD Ping */
+	{ T_LIME, 24,  2 },		/* HEAD Pong */
+};
+
+static patricia_t *pt_weight;
+
+#define VMSG_TYPE_LEN		6		/* 6 bytes to identify a message */
+#define VMSG_TYPE_BITLEN	(VMSG_TYPE_LEN * 8)
+
+/**
+ * @return vendor message weight given beginning of vendor message payload.
+ */
+guint8
+vmsg_weight(gconstpointer data)
+{
+	gpointer value;
+
+	value = patricia_lookup(pt_weight, data);
+
+	return pointer_to_uint(value) & 0xff;
+}
+
+/**
+ * Construct the PATRICIA that will be used to quickly determine the
+ * weight of a vendor message by looking at the first 6 bytes of the
+ * message payload.
+ */
+static void
+vmsg_init_weight(void)
+{
+	size_t i;
+
+	pt_weight = patricia_create(VMSG_TYPE_BITLEN);
+
+	for (i = 0; i < G_N_ELEMENTS(vmsg_weight_map); i++) {
+		const struct vmsg_weight *vw = &vmsg_weight_map[i];
+		gnutella_vendor_t *key = walloc(VMSG_TYPE_LEN);
+
+		gnutella_vendor_set_code(key, vw->vendor);
+		gnutella_vendor_set_selector_id(key, vw->id);
+
+		patricia_insert(pt_weight, key, uint_to_pointer(vw->weight));
+	}
+}
+
+static gboolean
+pt_weight_free(gpointer key, size_t keybits, gpointer value, gpointer u)
+{
+	(void) keybits;
+	(void) value;
+	(void) u;
+	wfree(key, VMSG_TYPE_LEN);
+	return TRUE;
+}
+
+/**
+ * Cleanup the weights.
+ */
+static void
+vmsg_close_weight(void)
+{
+	patricia_foreach_remove(pt_weight, pt_weight_free, NULL);
+	patricia_destroy(pt_weight);
+	pt_weight = NULL;
+}
+
 /**
  * Initialize vendor messages.
  */
@@ -3131,25 +3301,41 @@ void
 vmsg_init(void)
 {
 	size_t i;
+	char data[VMSG_TYPE_LEN];
+	gnutella_vendor_t *weight_key = (void *) data;
+
+	vmsg_init_weight();
 
 	ht_vmsg = g_hash_table_new(vmsg_hash_func, vmsg_eq_func);
+
 	for (i = 0; i < G_N_ELEMENTS(vmsg_map); i++) {
 		gconstpointer key = &vmsg_map[i];
 		gm_hash_table_insert_const(ht_vmsg, key, key);
+
+		gnutella_vendor_set_code(weight_key, vmsg_map[i].vendor);
+		gnutella_vendor_set_selector_id(weight_key, vmsg_map[i].id);
+
+		if (!patricia_contains(pt_weight, weight_key)) {
+			g_error("vendor message %s/%u missing from vmsg_weight_map[]",
+				vendor_code_to_string(vmsg_map[i].vendor),
+				vmsg_map[i].id);
+		}
 	}
 
 	head_pings = hash_list_new(guid_hash, guid_eq);
 	head_ping_ev = cq_insert(callout_queue, HEAD_PING_PERIODIC_MS,
 					head_ping_timer, NULL);
-
 }
 
 void
 vmsg_close(void)
 {
+	vmsg_close_weight();
 	head_ping_expire(TRUE);
 	hash_list_free(&head_pings);
 	cq_cancel(callout_queue, &head_ping_ev);
+	g_hash_table_destroy(ht_vmsg);
+	ht_vmsg = NULL;
 }
 
 /* vi: set ts=4 sw=4 cindent: */
