@@ -1,7 +1,7 @@
 /*
- * $Id: kmsg.c 16254 2009-03-27 15:23:09Z rmanfredi $
+ * $Id: kmsg.c 17354 2010-03-03 22:54:50Z rmanfredi $
  *
- * Copyright (c) 2008, Raphael Manfredi
+ * Copyright (c) 2008-2009, Raphael Manfredi
  *
  *----------------------------------------------------------------------
  * This file is part of gtk-gnutella.
@@ -30,12 +30,12 @@
  * Kademlia Messages.
  *
  * @author Raphael Manfredi
- * @date 2008
+ * @date 2008-2009
  */
 
 #include "common.h"
 
-RCSID("$Id: kmsg.c 16254 2009-03-27 15:23:09Z rmanfredi $")
+RCSID("$Id: kmsg.c 17354 2010-03-03 22:54:50Z rmanfredi $")
 
 #include "kmsg.h"
 #include "knode.h"
@@ -46,9 +46,12 @@ RCSID("$Id: kmsg.c 16254 2009-03-27 15:23:09Z rmanfredi $")
 #include "values.h"
 #include "storage.h"
 
+#include "core/bsched.h"
 #include "core/gnet_stats.h"
 #include "core/hosts.h"
 #include "core/hostiles.h"
+#include "core/inet.h"
+#include "core/gmsg.h"
 #include "core/udp.h"
 #include "core/nodes.h"
 #include "core/guid.h"
@@ -61,10 +64,11 @@ RCSID("$Id: kmsg.c 16254 2009-03-27 15:23:09Z rmanfredi $")
 #include "if/gnet_property_priv.h"
 
 #include "lib/bstr.h"
-#include "lib/misc.h"
 #include "lib/host_addr.h"
 #include "lib/glib-missing.h"
 #include "lib/pmsg.h"
+#include "lib/stringify.h"
+#include "lib/unsigned.h"
 #include "lib/vendors.h"
 #include "lib/walloc.h"
 
@@ -79,6 +83,24 @@ RCSID("$Id: kmsg.c 16254 2009-03-27 15:23:09Z rmanfredi $")
  */
 #define KMSG_FOUND_NODE_SIZE	727
 
+/**
+ * The aimed length for STORE messages.
+ *
+ * The overhead of a STORE message is variable, because the security token is
+ * of variable length, and there can be several DHT values in one single
+ * STORE message.
+ *
+ * The maximum possible length would be 652 bytes with a 512-byte long value
+ * payload, and using an IPv6 address for the creator.  However, the current
+ * specifications (v0.0) require the DHT value header to use an IPv4 address,
+ * so the maximum message length is 640 bytes.
+ *
+ * In practice, we aim for 512-byte long messages (including overhead) and
+ * only include 1 single DHT value if the message would end-up being longer.
+ */
+#define KMSG_STORE_AIMED_SIZE	512
+#define KMSG_STORE_MAX_SIZE		640
+
 typedef void (*kmsg_handler_t)(knode_t *kn, struct gnutella_node *n,
 	const kademlia_header_t *header,
 	guint8 extlen, const void *payload, size_t len);
@@ -92,7 +114,6 @@ struct kmsg {
 	const char *name;
 };
 
-static const struct kmsg kmsg_map[];
 static const struct kmsg *kmsg_find(guint8 function);
 
 /**
@@ -160,7 +181,7 @@ kmsg_handle(knode_t *kn,
 	if (GNET_PROPERTY(dht_debug > 1)) {
 		g_message("DHT got %s from %s",
 			kmsg_infostr(header), knode_to_string(kn));
-		if (len && GNET_PROPERTY(dht_debug > 19))
+		if (len && (GNET_PROPERTY(dht_trace) & SOCK_TRACE_IN))
 			dump_hex(stderr, "UDP payload", payload, len);
 		
 	}
@@ -302,8 +323,8 @@ serialize_size_estimate(pmsg_t *mb)
 /**
  * Serialize a contact to message block.
  */
-static void
-serialize_contact(pmsg_t *mb, const knode_t *kn)
+void
+kmsg_serialize_contact(pmsg_t *mb, const knode_t *kn)
 {
 	pmsg_write_be32(mb, kn->vcode.u32);
 	pmsg_write_u8(mb, kn->major);
@@ -321,12 +342,12 @@ serialize_contact_vector(pmsg_t *mb, knode_t **kvec, size_t klen)
 {
 	size_t i;
 
-	g_assert(klen < 256);
+	g_assert(klen <= MAX_INT_VAL(guint8));
 
 	pmsg_write_u8(mb, klen);
 
 	for (i = 0; i < klen; i++)
-		serialize_contact(mb, kvec[i]);
+		kmsg_serialize_contact(mb, kvec[i]);
 }
 
 /**
@@ -356,56 +377,6 @@ kmsg_deserialize_contact(bstr_t *bs)
 	return knode_new(&kuid, 0, addr, port, vcode, major, minor);
 }
 
-/**
- * Deserialize a DHT value.
- *
- * @return the deserialized DHT value, or NULL if an error occurred.
- */
-dht_value_t *
-kmsg_deserialize_dht_value(bstr_t *bs)
-{
-	dht_value_t *dv;
-	kuid_t id;
-	knode_t *creator;
-	guint8 major, minor;
-	guint16 length;
-	gpointer data = NULL;
-	guint32 type;
-
-	creator = kmsg_deserialize_contact(bs);
-	if (!creator)
-		return NULL;
-
-	bstr_read(bs, id.v, KUID_RAW_SIZE);
-	bstr_read_be32(bs, &type);
-	bstr_read_u8(bs, &major);
-	bstr_read_u8(bs, &minor);
-	bstr_read_be16(bs, &length);
-
-	if (bstr_has_error(bs))
-		goto error;
-
-	if (length && length <= DHT_VALUE_MAX_LEN) {
-		data = walloc(length);
-		bstr_read(bs, data, length);
-	} else {
-		bstr_skip(bs, length);
-	}
-
-	if (bstr_has_error(bs))
-		goto error;
-
-	dv = dht_value_make(creator, &id, type, major, minor, data, length);
-	knode_free(creator);
-	return dv;
-
-error:
-	knode_free(creator);
-	if (data)
-		wfree(data, length);
-	return NULL;
-}
-
 /**
  * Send a pong message back to the host who sent the ping.
  */
@@ -454,7 +425,7 @@ k_send_pong(struct gnutella_node *n, const guid_t *muid)
 			kmsg_infostr(header), (unsigned long) pmsg_size(mb),
 			host_addr_port_to_string(n->addr, n->port));
 
-	udp_send_mb(n, mb);
+	udp_dht_send_mb(n, mb);
 }
 
 /**
@@ -530,22 +501,7 @@ k_send_find_node_response(
 			(unsigned long) klen, klen == 1 ? "" : "s",
 			host_addr_port_to_string(n->addr, n->port));
 
-	udp_send_mb(n, mb);
-}
-
-/**
- * qsort() callback to compare two DHT values on a length basis.
- */
-static int
-dht_value_cmp(const void *a, const void *b)
-{
-	const dht_value_t * const *pa = a;
-	const dht_value_t * const *pb = b;
-	const dht_value_t *va = *pa;
-	const dht_value_t *vb = *pb;
-
-	return va->length == vb->length ? 0 :
-		va->length < vb->length ? -1 : +1;
+	udp_dht_send_mb(n, mb);
 }
 
 /**
@@ -625,7 +581,7 @@ k_send_find_value_response(
 	for (i = 0; i < vlen; i++) {
 		size_t secondary_size = (vlen - i) * KUID_RAW_SIZE + 1;
 		dht_value_t *v = vvec[i];
-		size_t value_size = 61 + v->length;	/* See assert below */
+		size_t value_size = DHT_VALUE_HEADER_SIZE + dht_value_length(v);
 
 		g_assert((size_t) pmsg_available(mb) >= secondary_size);
 
@@ -639,20 +595,9 @@ k_send_find_value_response(
 		}
 
 		/* That's the specs and the 61 above depends on the following... */
-		g_assert(NET_TYPE_IPV4 == host_addr_net(v->creator->addr));
-
-		/* DHT value header */
-		serialize_contact(mb, v->creator);
-		pmsg_write(mb, v->id, KUID_RAW_SIZE);
-		pmsg_write_be32(mb, v->type);
-		pmsg_write_u8(mb, v->major);
-		pmsg_write_u8(mb, v->minor);
-		pmsg_write_be16(mb, v->length);
-
-		/* DHT value data */
-		if (v->length)
-			pmsg_write(mb, v->data, v->length);
+		g_assert(NET_TYPE_IPV4 == host_addr_net(dht_value_creator(v)->addr));
 
+		dht_value_serialize(mb, v);
 		values++;
 
 		if (GNET_PROPERTY(dht_debug) > 4)
@@ -679,7 +624,7 @@ k_send_find_value_response(
 		pmsg_write_u8(mb, remain);
 		for (/* empty */; i < vlen; i++) {
 			dht_value_t *v = vvec[i];
-			pmsg_write(mb, v->creator->id, KUID_RAW_SIZE);
+			pmsg_write(mb, dht_value_creator(v)->id, KUID_RAW_SIZE);
 			secondaries++;
 
 			if (GNET_PROPERTY(dht_debug) > 4)
@@ -719,7 +664,7 @@ k_send_find_value_response(
 			secondaries, secondaries == 1 ? "" : "s",
 			host_addr_port_to_string(n->addr, n->port));
 
-	udp_send_mb(n, mb);
+	udp_dht_send_mb(n, mb);
 }
 
 /**
@@ -787,8 +732,8 @@ k_send_store_response(
 			break;
 
 		/* Serialize status */
-		pmsg_write(mb, vec[i]->id->v, KUID_RAW_SIZE);
-		pmsg_write(mb, vec[i]->creator->id->v, KUID_RAW_SIZE);
+		pmsg_write(mb, dht_value_key(vec[i])->v, KUID_RAW_SIZE);
+		pmsg_write(mb, dht_value_creator(vec[i])->id->v, KUID_RAW_SIZE);
 		pmsg_write_be16(mb, status[i]);
 		pmsg_write_be16(mb, 0);		/* Aren't we verbose enough already? */
 	}
@@ -816,7 +761,7 @@ k_send_store_response(
 			i, i == 1 ? "" : "es",
 			host_addr_port_to_string(n->addr, n->port));
 
-	udp_send_mb(n, mb);
+	udp_dht_send_mb(n, mb);
 
 	/*
 	 * Cleanup.
@@ -933,7 +878,7 @@ k_handle_pong(knode_t *kn, struct gnutella_node *n,
 
 	warn_unparsed_trailer(kn, header, bs);
 
-	bstr_destroy(bs);
+	bstr_free(&bs);
 	return;
 
 error:
@@ -943,7 +888,7 @@ error:
 			(unsigned long) len, len == 1 ? "" : "s", knode_to_string(kn),
 			reason, bstr_error(bs));
 
-	bstr_destroy(bs);
+	bstr_free(&bs);
 }
 
 /**
@@ -959,8 +904,11 @@ answer_find_node(struct gnutella_node *n,
 	const knode_t *kn, const kuid_t *id, const guid_t *muid)
 {
 	knode_t *kvec[KDA_K];
+	int requested = KDA_K;
 	int cnt;
 	const char *msg = NULL;
+	gboolean delayed;
+	gboolean within_kball;
 
 	/*
 	 * If the UDP queue is flow controlled already, there's no need to
@@ -968,9 +916,9 @@ answer_find_node(struct gnutella_node *n,
 	 * is likely to be dropped by the queue later on.
 	 */
 
-	if (node_udp_is_flow_controlled()) {
+	if (node_dht_is_flow_controlled()) {
 		msg = "is flow-controlled";
-		goto dropped;
+		goto flow_controlled;
 	}
 
 	/*
@@ -979,26 +927,128 @@ answer_find_node(struct gnutella_node *n,
 	 * the case in practice as long as we've been looking up our own KUID once.
 	 */
 
-	if (node_udp_would_flow_control(KMSG_FOUND_NODE_SIZE)) {
+	if (node_dht_would_flow_control(KMSG_FOUND_NODE_SIZE)) {
 		msg = "would flow-control";
-		goto dropped;
+		goto flow_controlled;
+	}
+
+	/*
+	 * We're going to reply differently depending on whether the key falls
+	 * within our k-ball or whether we are starting to have some noticeable
+	 * delays in the outgoing UDP queue.
+	 */
+
+	delayed = node_dht_above_low_watermark();
+	within_kball = keys_within_kball(id);
+
+	if (within_kball) {
+		if (
+			bsched_saturated(BSCHED_BWS_DHT_OUT) &&
+			kuid_eq(id, get_our_kuid())
+		) {
+			/*
+			 * If they are looking for precisely our KUID, then most probably
+			 * all they want is our security token.  Since bandwidth is
+			 * saturated, just give them that.
+			 */
+
+			goto only_token;
+		}
+
+		/*
+		 * Regardless of bandwidth considerations, we need to fully reply to
+		 * requests for which we could be one of the k-closest nodes otherwise
+		 * the looking node would not be able to find accurately the set of
+		 * k-closest nodes surrounding the looked-up key.
+		 */
+
+		goto answer;
+	}
+
+	/*
+	 * If the request comes from a firewalled node, then let's be stricter.
+	 *
+	 * Indeed, a firewalled node will not participate to the DHT structure
+	 * fully since it cannot answer FIND_NODE and STORE requests, by definition.
+	 *
+	 * We are an active node (since we're replying to the RPC) and therefore
+	 * are paying a greater bandwidth price for the benefit of the whole
+	 * network, but this should not penalize us if there are more passive than
+	 * active nodes out there.
+	 *
+	 * So, if the request comes from a firewalled node and we are already
+	 * suffering from outgoing traffic congestion, limit the number of replies
+	 * to KDA_K / 2 provided the key falls in our space, KDA_K / 4 otherwise.
+	 *
+	 * Furthermore, if we already have enough outgoing pending traffic in
+	 * the queue, drop the request.
+	 */
+
+	if (kn->flags & KNODE_F_FIREWALLED) {
+		if (delayed) {
+			msg = "above low watermark with passive requestor";
+			goto flow_controlled;
+		}
+		if (bsched_saturated(BSCHED_BWS_DHT_OUT)) {
+			requested = keys_is_foreign(id) ? KDA_K / 4 : KDA_K / 2;
+		}
+	} else if (delayed) {
+		requested = keys_is_foreign(id) ? KDA_K / 2 : 3 * KDA_K / 4;
 	}
 
 	/*
 	 * OK, perform the lookup and send them the answer.
 	 */
 
-	cnt = dht_fill_closest(id, kvec, KDA_K, kn->id, TRUE);
+answer:
+
+	if (GNET_PROPERTY(dht_debug > 2)) {
+		g_message("DHT processing FIND_NODE %s "
+			"%s%s%s: giving %d node%s",
+			kuid_to_hex_string(id),
+			delayed ? "[UDP delayed] " : "",
+			within_kball ? "[in k-ball] " :
+				keys_is_foreign(id) ? "[foreign] " : "",
+			(kn->flags & KNODE_F_FIREWALLED) ? "[passive]" : "[active]",
+			requested, 1 == requested ? "" : "s");
+	}
+
+	cnt = dht_fill_closest(id, kvec, requested, kn->id, TRUE);
 	k_send_find_node_response(n, kn, kvec, cnt, muid);
 	return;
 
-dropped:
+flow_controlled:
+	/*
+	 * If they are looking for our KUID, they probably want our security token.
+	 * Answer with no nodes to limit message size.
+	 *
+	 * Otherwise, don't bother replying.
+	 */
+
+	if (kuid_eq(id, get_our_kuid())) {
+		goto only_token;
+	} else {
+		if (GNET_PROPERTY(dht_debug)) {
+			g_message("DHT ignoring FIND_NODE %s: UDP queue %s",
+				kuid_to_hex_string(id), msg);
+		}
+
+		gnet_stats_count_dropped(n, MSG_DROP_FLOW_CONTROL);
+	}
+
+	return;
+
+only_token:
+	/*
+	 * Only send back our security token, with no nodes.
+	 */
+
 	if (GNET_PROPERTY(dht_debug)) {
-		g_message("DHT ignoring FIND_NODE %s: UDP queue %s",
+		g_message("DHT limiting FIND_NODE %s (ourselves): UDP queue %s",
 			kuid_to_hex_string(id), msg);
 	}
 
-	gnet_stats_count_dropped(n, MSG_DROP_FLOW_CONTROL);
+	k_send_find_node_response(n, kn, kvec, 0, muid);
 }
 
 /**
@@ -1035,6 +1085,12 @@ k_handle_find_node(knode_t *kn, struct gnutella_node *n,
 		g_message("DHT node %s looking for %s",
 			knode_to_string(kn), kuid_to_hex_string(id));
 
+	/*
+	 * Signal that we got an unsolicited UDP message.
+	 */
+
+	inet_udp_got_unsolicited_incoming();
+
 	g_assert(len == KUID_RAW_SIZE);
 
 	/*
@@ -1051,11 +1107,11 @@ k_handle_find_node(knode_t *kn, struct gnutella_node *n,
 	 */
 
 	if (!peer_replication(kn, id) && keys_is_store_loaded(id)) {
-		if (GNET_PROPERTY(dht_debug > 2))
+		if (GNET_PROPERTY(dht_debug > 2)) {
 			g_message("DHT key %s getting too many STORE, "
 				"ignoring FIND_NODE from %s",
 				kuid_to_hex_string(id), knode_to_string(kn));
-
+		}
 		gnet_stats_count_dropped(n, MSG_DROP_DHT_TOO_MANY_STORE);
 		return;
 	}
@@ -1160,7 +1216,7 @@ k_handle_store(knode_t *kn, struct gnutella_node *n,
 	vec = walloc(values * sizeof *vec);
 
 	for (i = 0; i < values; i++) {
-		dht_value_t *v = kmsg_deserialize_dht_value(bs);
+		dht_value_t *v = dht_value_deserialize(bs);
 
 		if (NULL == v) {
 			gm_snprintf(msg, sizeof msg,
@@ -1180,8 +1236,11 @@ k_handle_store(knode_t *kn, struct gnutella_node *n,
 		 * need to bother.
 		 */
 
-		if ((kn->flags & KNODE_F_PCONTACT) && kuid_eq(kn->id, v->creator->id)) {
-			knode_t *cn = deconstify_gpointer(v->creator);
+		if (
+			(kn->flags & KNODE_F_PCONTACT) &&
+			kuid_eq(kn->id, dht_value_creator(v)->id)
+		) {
+			knode_t *cn = deconstify_gpointer(dht_value_creator(v));
 
 			if (GNET_PROPERTY(dht_storage_debug))
 				g_warning(
@@ -1240,7 +1299,7 @@ cleanup:
 		wfree(vec, values * sizeof *vec);
 	}
 
-	bstr_destroy(bs);
+	bstr_free(&bs);
 }
 
 /**
@@ -1369,6 +1428,42 @@ k_handle_find_value(knode_t *kn, struct gnutella_node *n,
 	vcnt = keys_get(id, type, secondary, count,
 		vvec, G_N_ELEMENTS(vvec), &load, &cached);
 
+	/*
+	 * If we have no items of the requested value type, we need to act
+	 * as if we were not holding the key.
+	 *
+	 * The reason is that due to value caching, we may have stored some
+	 * cached results for a key, but only for a particular type.  There may
+	 * be actual results for the type they are looking up now further down
+	 * in the path, at some node closest to the key than we are.
+	 *
+	 * Furthermore, we need to carefully distinguish between an initial
+	 * lookup, asking for a value, and secondary key fetches.  The initial
+	 * lookup cannot know the secondary keys, so it cannot specify any.
+	 * If a secondary key lookup was issued, and we have no value to
+	 * return, we'll reply with an empty message.
+	 */
+
+	if (0 == vcnt && NULL == secondary) {
+		if (GNET_PROPERTY(dht_debug) || GNET_PROPERTY(dht_storage_debug))
+			g_message("DHT FETCH \"%s\" %s not found in existing key (%s)",
+				dht_value_type_to_string(type),
+				kuid_to_hex_string(id), kuid_to_string(id));
+
+		answer_find_node(n, kn, id, kademlia_header_get_muid(header));
+		goto cleanup;
+	}
+
+	/*
+	 * Send back the values we found (could be none for secondary keys).
+	 */
+
+	if (GNET_PROPERTY(dht_debug) || GNET_PROPERTY(dht_storage_debug))
+		g_message("DHT FETCH \"%s\" %s (%s) FOUND %d %svalue%s",
+			dht_value_type_to_string(type),
+			kuid_to_hex_string(id), kuid_to_string(id),
+			vcnt, cached ? "cached " : "", 1 == vcnt ? "" : "s");
+
 	k_send_find_value_response(n,
 		kn, vvec, vcnt, load, cached, kademlia_header_get_muid(header));
 
@@ -1402,14 +1497,18 @@ cleanup:
 			dht_value_free(vvec[i], TRUE);
 	}
 
-	bstr_destroy(bs);
+	bstr_free(&bs);
 }
 
 /**
- * Handle node lookup answers (FIND_NODE_RESPONSE and FIND_VALUE_RESPONSE).
+ * Handle node RPC answers (FIND_NODE_RESPONSE, FIND_VALUE_RESPONSE and
+ * STORE_RESPONSE).
+ *
+ * If the message is for a recognized RPC (based on the message's MUID), then
+ * it will be handled by the RPC callbacks.  Otherwise, the message is ignored.
  */
 static void
-k_handle_lookup(knode_t *kn, struct gnutella_node *n,
+k_handle_rpc_reply(knode_t *kn, struct gnutella_node *n,
 	const kademlia_header_t *header, guint8 extlen,
 	const void *payload, size_t len)
 {
@@ -1433,23 +1532,34 @@ k_handle_lookup(knode_t *kn, struct gnutella_node *n,
 /**
  * Send message to the Kademlia node.
  */
-static void
+void
 kmsg_send_mb(knode_t *kn, pmsg_t *mb)
 {
-	struct gnutella_node *n = node_udp_get_addr_port(kn->addr, kn->port);
+	struct gnutella_node *n = node_dht_get_addr_port(kn->addr, kn->port);
 
 	knode_check(kn);
 
+	if (NULL == n) {
+		/* E.g. we're given an IPv6 node address but IPv6 support is off */
+		if (GNET_PROPERTY(dht_debug)) {
+			int len = pmsg_size(mb);
+			g_message("DHT discarding %s (%d bytes) to %s",
+				kmsg_infostr(pmsg_start(mb)), len, knode_to_string(kn));
+		}
+		pmsg_free(mb);
+		return;
+	}
+
 	if (GNET_PROPERTY(dht_debug) > 3) {
 		int len = pmsg_size(mb);
 		g_message("DHT sending %s (%d bytes) to %s, RTT=%u",
 			kmsg_infostr(pmsg_start(mb)), len, knode_to_string(kn), kn->rtt);
-		if (GNET_PROPERTY(dht_debug) > 19)
+		if (GNET_PROPERTY(dht_trace) & SOCK_TRACE_OUT)
 			dump_hex(stderr, "UDP datagram", pmsg_start(mb), len);
 	}
 
 	kn->last_sent = tm_time();
-	udp_send_mb(n, mb);
+	udp_dht_send_mb(n, mb);
 }
 
 /**
@@ -1516,7 +1626,7 @@ kmsg_send_find_value(knode_t *kn, const kuid_t *id, dht_value_type_t type,
 	int i;
 
 	g_assert(skeys == NULL || scnt > 0);
-	g_assert(scnt >= 0 && scnt < 256);
+	g_assert(scnt >= 0 && scnt <= MAX_INT_VAL(guint8));
 
 	/* Header + target KUID + count + array-of-sec-keys + type */
 	msize = KDA_HEADER_SIZE + KUID_RAW_SIZE + 1 +
@@ -1539,6 +1649,145 @@ kmsg_send_find_value(knode_t *kn, const kuid_t *id, dht_value_type_t type,
 	kmsg_send_mb(kn, mb);
 }
 
+/**
+ * Patch the STORE message value count byte if we have finally put more than
+ * one values in the message and adjust the Kademlia header size to match
+ * that of the built buffer, since the message block can be larger than
+ * the message actually serialized to it.
+ */
+static void
+store_finalize_pmsg(pmsg_t *mb, int values_held, pmsg_offset_t value_count)
+{
+	guint8 function = kademlia_header_get_function(pmsg_start(mb));
+
+	g_assert(KDA_MSG_STORE_REQUEST == function);
+
+	/* Patch value count if more than 1 value is held in message */
+	if (values_held > 1) {
+		pmsg_offset_t cur = pmsg_write_offset(mb);
+		pmsg_seek(mb, value_count);
+		pmsg_write_u8(mb, values_held);
+		pmsg_seek(mb, cur);		/* Critical: must go back to end */
+	}
+
+	/* Adjust Kademlia header size -- message block can be larger */
+	kademlia_header_set_size(pmsg_start(mb),
+		pmsg_written_size(mb) - KDA_HEADER_SIZE);
+}
+
+/**
+ * Build store(values) messages.
+ *
+ * There are ``vcnt'' values to store, and we need at most 1 message per
+ * value.  We try to stuff as many values per message as we can, aiming
+ * to produce messages that are around KMSG_STORE_AIMED_SIZE bytes, including
+ * Kademlia header overhead + DHT value header overhead.
+ *
+ * @param token		security token
+ * @param toklen	length of security token, in bytes (can be 0)
+ * @param vvec		vector of values to store
+ * @param vcnt		amount of values in ``vvec''
+ *
+ * @return a GSList of message blocks (pmsg_t *), with blank MUIDs, meant
+ * to be overwritten by the RPC layer.
+ * It is up to the caller to free up the list and the blocks.
+ */
+GSList *
+kmsg_build_store(const void *token, size_t toklen, dht_value_t **vvec, int vcnt)
+{
+	int i;
+	GSList *result = NULL;
+	pmsg_t *mb = NULL;
+	int vheld = 0;
+	pmsg_offset_t value_count = 0;
+
+	g_assert(vvec);
+	g_assert(size_is_non_negative(toklen));
+	g_assert(token == NULL || size_is_positive(toklen));
+	g_assert(vcnt > 0 && vcnt <= MAX_INT_VAL(guint8));
+
+	/*
+	 * Sort values by increasing size, so that we can stuff as many smaller
+	 * values as possible in the first messages.
+	 */
+
+	qsort(vvec, vcnt, sizeof vvec[0], dht_value_cmp);
+
+	for (i = 0; i < vcnt; i++) {
+		dht_value_t *v = vvec[i];
+		size_t value_size = DHT_VALUE_HEADER_SIZE + dht_value_length(v);
+
+		/*
+		 * If value does not fit in message, flush the current one.
+		 */
+
+		if (mb && UNSIGNED(pmsg_available(mb)) < value_size) {
+			g_assert(vheld > 0 && vheld <= MAX_INT_VAL(guint8));
+
+			store_finalize_pmsg(mb, vheld, value_count);
+
+			result = g_slist_prepend(result, mb);
+			mb = NULL;
+			vheld = 0;
+		}
+
+		/*
+		 * If no current message, allocate one suitable for holding the
+		 * current value.
+		 */
+
+		if (NULL == mb) {
+			int msize;
+			kademlia_header_t *header;
+
+			/* Header + token length + security token + count + value(s) */
+			msize = KDA_HEADER_SIZE + 1 + toklen + 1 + value_size;
+			if (vcnt - i > 1)		/* Can hope to pack more than 1 value */
+				msize = MAX(msize, KMSG_STORE_AIMED_SIZE);
+
+			mb = pmsg_new(PMSG_P_DATA, NULL, msize);
+
+			/*
+			 * Build message header.
+			 *
+			 * When we stuff more than one value in the message, we cannot
+			 * know what the final size is going to be a priori, so it will
+			 * get patched at the end in store_finalize_pmsg().
+			 */
+
+			header = (kademlia_header_t *) pmsg_start(mb);
+			kmsg_build_header(header, KDA_MSG_STORE_REQUEST, 0, 0, &blank_guid);
+			pmsg_seek(mb, KDA_HEADER_SIZE);		/* Start of payload */
+			pmsg_write_u8(mb, toklen);
+			pmsg_write(mb, token, toklen);
+
+			/* Assume 1 value will be held -- field patched if needed */
+			value_count = pmsg_write_offset(mb);
+			pmsg_write_u8(mb, 1);
+		}
+
+		/*
+		 * Serialize current value at the tail of the current message.
+		 */
+
+		g_assert(UNSIGNED(pmsg_available(mb)) >= value_size);
+
+		dht_value_serialize(mb, v);
+		vheld++;
+	}
+
+	/*
+	 * Flush current message.
+	 */
+
+	g_assert(mb != NULL);
+	g_assert(vheld > 0 && vheld <= MAX_INT_VAL(guint8));
+
+	store_finalize_pmsg(mb, vheld, value_count);
+
+	return g_slist_prepend(result, mb);
+}
+
 /**
  * Main entry point for DHT messages received from UDP.
  *
@@ -1554,7 +1803,7 @@ kmsg_send_find_value(knode_t *kn, const kuid_t *id, dht_value_type_t type,
  * @param len		total length of the message (header + data)
  * @param addr		address from which we received the datagram
  * @param port		port from which we received the datagram
- * @param n			the UDP Gnutella node, for some core function calls
+ * @param n			the DHT Gnutella node, for some core function calls
  */
 void kmsg_received(
 	gconstpointer data, size_t len,
@@ -1575,6 +1824,9 @@ void kmsg_received(
 	guint8 flags;
 	guint16 extended_length;
 
+	g_assert(len >= GTA_HEADER_SIZE);	/* Valid Gnutella packet at least */
+	g_assert(NODE_IS_DHT(n));
+
 	/*
 	 * If DHT is not enabled, drop the message now.
 	 */
@@ -1651,13 +1903,30 @@ void kmsg_received(
 	 */
 
 	if (!(flags & KDA_MSG_F_FIREWALLED) && !host_is_valid(kaddr, kport)) {
-		if (GNET_PROPERTY(dht_debug))
-			g_warning("DHT bad contact address %s (%s v%u.%u), "
-				"forcing \"firewalled\" flag",
-				host_addr_port_to_string(kaddr, kport),
-				vendor_code_to_string(vcode.u32), kmajor, kminor);
+		host_addr_t raddr;
+		guint16 rport;
 
-		flags |= KDA_MSG_F_FIREWALLED;
+		/*
+		 * LimeWire nodes suffer from a bug whereby the contact is not
+		 * firewalled but replies to RPC requests come back with an
+		 * advertised address of 127.0.0.1.  This is annoying but hopefully
+		 * fixable, at the cost of extra processing.
+		 */
+
+		if (dht_rpc_info(kademlia_header_get_muid(header), &raddr, &rport)) {
+			if (GNET_PROPERTY(dht_debug)) {
+				g_warning("DHT fixing contact address %s (%s v%u.%u) kuid=%s "
+					"to %s:%u on RPC reply (%s UDP info)",
+					host_addr_port_to_string(kaddr, kport),
+					vendor_code_to_string(vcode.u32), kmajor, kminor,
+					kuid_to_hex_string(id),
+					host_addr_to_string(raddr), rport,
+					host_addr_equal(addr, raddr) && port == rport ?
+						"matches" : "still different from");
+			}
+			kaddr = raddr;
+			kport = rport;
+		}
 	}
 
 	/*
@@ -1675,6 +1944,50 @@ void kmsg_received(
 		goto drop;
 	}
 
+	/*
+	 * If we got the UDP message from another address than the one we
+	 * have in the contact information, it is not necessarily an error.
+	 *
+	 * We always reply to the address we had in the UDP message, but when
+	 * contacting the node, we use the address in the contact information.
+	 * The gnutella_node structure keeps track of the origin of the UDP message.
+	 *
+	 * If the UDP port matches the port advertised in the contact information,
+	 * then it probably means the servent is unable to figure its outgoing
+	 * IP address correctly.
+	 */
+
+	if (
+		!(flags & KDA_MSG_F_FIREWALLED) &&
+		(!host_addr_equal(addr, kaddr) || port != kport)
+	) {
+		if (GNET_PROPERTY(dht_debug)) {
+			g_warning("DHT contact address is %s "
+				"but message came from %s (%s v%u.%u) kuid=%s",
+				host_addr_port_to_string(kaddr, kport),
+				host_addr_port_to_string2(addr, port),
+				vendor_code_to_string(vcode.u32), kmajor, kminor,
+				kuid_to_hex_string(id));
+		}
+	}
+
+	/*
+	 * If they set kport to 0, act as if they were firewalled.
+	 */
+
+	if (!(flags & KDA_MSG_F_FIREWALLED) && 0 == kport) {
+		if (GNET_PROPERTY(dht_debug)) {
+			g_warning("DHT contact port is zero, forcing firewalled status "
+				"for %s (%s v%u.%u@%s) kuid=%s",
+				host_addr_port_to_string(kaddr, kport),
+				vendor_code_to_string(vcode.u32), kmajor, kminor,
+				host_addr_port_to_string2(addr, port),
+				kuid_to_hex_string(id));
+		}
+		flags |= KDA_MSG_F_FIREWALLED;
+	}
+
+
 	/*
 	 * See whether we already have this node in the routing table.
 	 */
@@ -1684,6 +1997,57 @@ void kmsg_received(
 	g_assert(kn == NULL || !(kn->flags & KNODE_F_FIREWALLED));
 
 	if (NULL == kn) {
+		gboolean patched = FALSE;
+
+		/*
+		 * If the node is not already in our routing table, but its advertised
+		 * contact information is wrong and it is not presenting itself as
+		 * being firewalled, attempt to use the UDP address and port.
+		 *
+		 * Moreover, we are flagging the node with KNODE_F_PCONTACT, so
+		 * any value it attempts to store will have its creator address
+		 * corrected.
+		 */
+
+		if (
+			!(flags & KDA_MSG_F_FIREWALLED) &&
+			(!host_is_valid(kaddr, kport) || !host_addr_equal(addr, kaddr))
+		) {
+			if (port == kport) {
+				if (GNET_PROPERTY(dht_debug)) {
+					g_warning("DHT fixing contact address %s (%s v%u.%u) "
+						"kuid=%s, "
+						"not firewalled, replacing with UDP source %s:%u",
+						host_addr_port_to_string(kaddr, kport),
+						vendor_code_to_string(vcode.u32), kmajor, kminor,
+						kuid_to_hex_string(id),
+						host_addr_to_string(addr), port);
+				}
+			} else {
+				if (GNET_PROPERTY(dht_debug)) {
+					g_warning("DHT fixing contact address %s (%s v%u.%u) "
+						"kuid=%s, "
+						"not firewalled, replacing with UDP source IP %s and "
+						"ignoring UDP port %u",
+						host_addr_port_to_string(kaddr, kport),
+						vendor_code_to_string(vcode.u32), kmajor, kminor,
+						kuid_to_hex_string(id),
+						host_addr_to_string(addr), port);
+				}
+				/*
+				 * kport is probably their advertised listening port, and
+				 * the UDP port is different because of NAT: replying on
+				 * that port would work for a while, until the NAT times out.
+				 *
+				 * We don't know whether kport is forwarded on the router
+				 * though, but since the host did not set the "firewalled" bit
+				 * we have to assume it is.
+				 */
+			}
+			kaddr = addr;
+			patched = TRUE;
+		}
+
 		if (GNET_PROPERTY(dht_debug) > 2)
 			g_message("DHT traffic from new %s%snode %s at %s (%s v%u.%u)",
 				(flags & KDA_MSG_F_FIREWALLED) ? "firewalled " : "",
@@ -1693,10 +2057,67 @@ void kmsg_received(
 				vendor_code_to_string(vcode.u32), kmajor, kminor);
 
 		kn = knode_new(id, flags, kaddr, kport, vcode, kmajor, kminor);
+
+		if (patched)
+			kn->flags |= KNODE_F_PCONTACT;
+
 		if (!(flags & (KDA_MSG_F_FIREWALLED | KDA_MSG_F_SHUTDOWNING)))
 			dht_traffic_from(kn);
 	} else {
-		if (GNET_PROPERTY(dht_debug) > 2)
+		/*
+		 * This here is again a workaround for LimeWire's bug: a good
+		 * non-firewalled host that is known in our routing table with a
+		 * proper address has to be identical to one bearing the same KUID
+		 * with a non-routable address as long as the ports are identical.
+		 *
+		 * This needs to be done here before we attempt to initiate an
+		 * address validation because the node's address would appear to have
+		 * changed...
+		 */
+
+		if (!(flags & KDA_MSG_F_FIREWALLED)) {
+			if (
+				kport == kn->port &&
+				host_is_valid(kn->addr, kn->port) &&
+				(
+					!host_is_valid(kaddr, kport) ||
+					!host_addr_equal(addr, kaddr)
+				)
+			) {
+				if (GNET_PROPERTY(dht_debug)) {
+					g_warning("DHT fixing contact address %s "
+						"(%s v%u.%u) kuid=%s "
+						"to %s:%u based on routing table (%s UDP info)",
+						host_addr_port_to_string(kaddr, kport),
+						vendor_code_to_string(vcode.u32), kmajor, kminor,
+						kuid_to_hex_string(id),
+						host_addr_to_string(kn->addr), kn->port,
+						host_addr_equal(addr, kn->addr) && port == kport ?
+							"matches" : "still different from");
+				}
+				kaddr = kn->addr;
+				/* Port identical, as checked in test */
+				kn->flags |= KNODE_F_PCONTACT;	/* To adapt creator later */
+				kn->flags &= ~KNODE_F_FOREIGN_IP;
+			} else {
+				kn->flags &= ~(KNODE_F_PCONTACT | KNODE_F_FOREIGN_IP);
+				if (!host_addr_equal(addr, kaddr)) {
+					if (GNET_PROPERTY(dht_debug)) {
+						g_warning("DHT not fixing contact address %s "
+							"(%s v%u.%u) kuid=%s but keeping "
+							"routing table info %s:%u (UDP came from %s)",
+							host_addr_port_to_string(kaddr, kport),
+							vendor_code_to_string(vcode.u32), kmajor, kminor,
+							kuid_to_hex_string(id),
+							host_addr_to_string(kn->addr), kn->port,
+							host_addr_port_to_string2(addr, port));
+					}
+					kn->flags |= KNODE_F_FOREIGN_IP;
+				}
+			}
+		}
+
+		if (GNET_PROPERTY(dht_debug) > 2) {
 			g_message("DHT traffic from known %s %s%snode %s at %s (%s v%u.%u)",
 				knode_status_to_string(kn->status),
 				(flags & KDA_MSG_F_FIREWALLED) ? "firewalled " : "",
@@ -1704,6 +2125,7 @@ void kmsg_received(
 				kuid_to_hex_string(id),
 				host_addr_port_to_string(kaddr, kport),
 				vendor_code_to_string(vcode.u32), kmajor, kminor);
+		}
 
 		/*
 		 * Make sure the IP has not changed for the node.
@@ -1772,25 +2194,6 @@ void kmsg_received(
 		}
 	}
 
-	/*
-	 * If we got the UDP message from another address than the one we
-	 * have in the contact information, it is not necessarily an error.
-	 * However, we keep track of that by flagging the node.
-	 *
-	 * We always reply to the address we had in the UDP message, but when
-	 * contacting the node, we use the address in the contact information.
-	 * The gnutella_node structure keeps track of the origin of the UDP message.
-	 */
-
-	if (!(kn->flags & KNODE_F_FIREWALLED) && !host_addr_equal(addr, kaddr)) {
-		if (GNET_PROPERTY(dht_debug))
-			g_warning("DHT contact address is %s but message came from %s",
-				host_addr_port_to_string(kaddr, kport),
-				host_addr_to_string(addr));
-
-		kn->flags |= KNODE_F_FOREIGN_IP;
-	}
-
 	/*
 	 * If host is firewalled, ignore private host address in the Kademlia
 	 * header and use the addr/port from the UDP datagram.
@@ -1809,7 +2212,7 @@ void kmsg_received(
 	}
 
 	kmsg_handle(kn, n, header, extended_length,
-		(char *) header + extended_length + KDA_HEADER_SIZE,
+		ptr_add_offset(header, extended_length + KDA_HEADER_SIZE),
 		len - KDA_HEADER_SIZE - extended_length);
 
 	knode_free(kn);		/* Will free only if not still referenced */
@@ -1818,7 +2221,9 @@ void kmsg_received(
 
 drop:
 	if (GNET_PROPERTY(dht_debug)) {
-		g_warning("DHT got invalid Kademlia packet from UDP (%s): %s",
+		g_warning("DHT got invalid Kademlia packet (%u bytes) "
+			"\"%s\" from UDP (%s): %s",
+			(unsigned) len, gmsg_infostr_full(data, len),
 			host_addr_port_to_string(addr, port), reason);
 		if (len)
 			dump_hex(stderr, "UDP datagram", data, len);
@@ -1826,17 +2231,17 @@ drop:
 }
 
 static const struct kmsg kmsg_map[] = {
-	{ 0x00,							NULL,					"invalid"		},
+	{ 0x00,							NULL, /* Invalid */		"invalid"		},
 	{ KDA_MSG_PING_REQUEST,			k_handle_ping,			"PING"			},
 	{ KDA_MSG_PING_RESPONSE,		k_handle_pong,			"PONG"			},
 	{ KDA_MSG_STORE_REQUEST,		k_handle_store,			"STORE"			},
-	{ KDA_MSG_STORE_RESPONSE,		NULL,					"STORE_ACK"		},
+	{ KDA_MSG_STORE_RESPONSE,		k_handle_rpc_reply,		"STORE_ACK"		},
 	{ KDA_MSG_FIND_NODE_REQUEST,	k_handle_find_node,		"FIND_NODE"		},
-	{ KDA_MSG_FIND_NODE_RESPONSE,	k_handle_lookup,		"FOUND_NODE"	},
+	{ KDA_MSG_FIND_NODE_RESPONSE,	k_handle_rpc_reply,		"FOUND_NODE"	},
 	{ KDA_MSG_FIND_VALUE_REQUEST,	k_handle_find_value,	"FIND_VALUE"	},
-	{ KDA_MSG_FIND_VALUE_RESPONSE,	k_handle_lookup,		"VALUE"			},
-	{ KDA_MSG_STATS_REQUEST,		NULL,					"STATS"			},
-	{ KDA_MSG_STATS_RESPONSE,		NULL,					"STATS_ACK" 	},
+	{ KDA_MSG_FIND_VALUE_RESPONSE,	k_handle_rpc_reply,		"VALUE"			},
+	{ KDA_MSG_STATS_REQUEST,		NULL, /* Deprecated */	"STATS"			},
+	{ KDA_MSG_STATS_RESPONSE,		NULL, /* Deprecated */	"STATS_ACK" 	},
 };
 
 /**
