@@ -1,7 +1,7 @@
 /*
- * $Id: publish.h 16209 2009-03-19 12:55:55Z rmanfredi $
+ * $Id: publish.h 17157 2009-11-01 22:41:55Z rmanfredi $
  *
- * Copyright (c) 2008, Raphael Manfredi
+ * Copyright (c) 2009, Raphael Manfredi
  *
  *----------------------------------------------------------------------
  * This file is part of gtk-gnutella.
@@ -36,12 +36,21 @@
 #ifndef _dht_publish_h_
 #define _dht_publish_h_
 
+#include "if/dht/publish.h"
+
+#include "values.h"
+#include "lookup.h"
+
 /*
  * Public interface.
  */
 
 void publish_init(void);
-void publish_close(void);
+void publish_close(gboolean exiting);
+
+publish_t *publish_cache(const kuid_t *key,
+	lookup_rc_t *target, dht_value_t **vvec, int vcnt);
+publish_t *publish_offload(const knode_t *kn, GSList *keys);
 
 #endif	/* _dht_publish_h_ */
 
