@@ -1,7 +1,7 @@
 /*
- * $Id: routing.c 16246 2009-03-26 21:54:43Z rmanfredi $
+ * $Id: routing.c 17364 2010-03-06 11:15:15Z rmanfredi $
  *
- * Copyright (c) 2006-2008, Raphael Manfredi
+ * Copyright (c) 2006-2009, Raphael Manfredi
  *
  *----------------------------------------------------------------------
  * This file is part of gtk-gnutella.
@@ -50,12 +50,12 @@
  * KUID, as defined in the is_splitable() routine.
  *
  * @author Raphael Manfredi
- * @date 2006-2008
+ * @date 2006-2009
  */
 
 #include "common.h"
 
-RCSID("$Id: routing.c 16246 2009-03-26 21:54:43Z rmanfredi $")
+RCSID("$Id: routing.c 17364 2010-03-06 11:15:15Z rmanfredi $")
 
 #include <math.h>
 
@@ -70,11 +70,15 @@ RCSID("$Id: routing.c 16246 2009-03-26 21:54:43Z rmanfredi $")
 #include "ulq.h"
 #include "kmsg.h"
 #include "publish.h"
+#include "roots.h"
+#include "tcache.h"
+#include "stable.h"
 
 #include "core/settings.h"
 #include "core/gnet_stats.h"
 #include "core/guid.h"
 #include "core/nodes.h"
+#include "core/sockets.h"
 
 #include "if/gnet_property.h"
 #include "if/gnet_property_priv.h"
@@ -88,7 +92,11 @@ RCSID("$Id: routing.c 16246 2009-03-26 21:54:43Z rmanfredi $")
 #include "lib/hashlist.h"
 #include "lib/host_addr.h"
 #include "lib/map.h"
+#include "lib/patricia.h"
 #include "lib/stats.h"
+#include "lib/stringify.h"
+#include "lib/timestamp.h"
+#include "lib/vendors.h"
 #include "lib/walloc.h"
 #include "lib/override.h"		/* Must be the last header included */
 
@@ -229,12 +237,15 @@ enum bootsteps {
 	BOOT_OWN,					/**< Looking for own KUID */
 	BOOT_COMPLETING,			/**< Completing further bucket bootstraps */
 	BOOT_COMPLETED,				/**< Fully bootstrapped */
+	BOOT_SHUTDOWN,				/**< Shutdowning */
 
 	BOOT_MAX_VALUE
 };
 
+static gboolean initialized;		/**< Whether dht_init() was called */
 static gboolean bootstrapping;		/**< Whether we are bootstrapping */
 static enum bootsteps boot_status;	/**< Booting status */
+static enum bootsteps old_boot_status = BOOT_NONE;
 
 static struct kbucket *root = NULL;	/**< The root of the routing table tree. */
 static kuid_t *our_kuid;			/**< Our own KUID (atom) */
@@ -250,10 +261,10 @@ static void dht_route_retrieve(void);
 static struct kbucket *dht_find_bucket(const kuid_t *id);
 
 /*
- * Define DEBUGGING only to enable more costly run-time assertions which
+ * Define DHT_ROUTING_DEBUG to enable more costly run-time assertions which
  * make all hash list insertions O(n), basically.
  */
-#undef DEBUGGING
+#undef DHT_ROUTING_DEBUG
 
 static const char * const boot_status_str[] = {
 	"not bootstrapped yet",			/**< BOOT_NONE */
@@ -261,6 +272,7 @@ static const char * const boot_status_str[] = {
 	"looking for our KUID",			/**< BOOT_OWN */
 	"completing bucket bootstrap",	/**< BOOT_COMPLETING */
 	"completely bootstrapped",		/**< BOOT_COMPLETING */
+	"shutdowning",					/**< BOOT_SHUTDOWN */
 };
 
 /**
@@ -279,6 +291,22 @@ boot_status_to_string(enum bootsteps status)
 	return boot_status_str[i];
 }
 
+/**
+ * Give a textual representation of the DHT mode.
+ */
+const char *
+dht_mode_to_string(dht_mode_t mode)
+{
+	switch (mode) {
+	case DHT_MODE_INACTIVE:		return "inactive";
+	case DHT_MODE_ACTIVE:		return "active";
+	case DHT_MODE_PASSIVE:		return "passive";
+	case DHT_MODE_PASSIVE_LEAF:	return "leaf";
+	}
+
+	return "unknown";
+}
+
 /**
  * Is bucket a leaf?
  */
@@ -415,15 +443,6 @@ dht_seeded(void)
 	return root && !is_leaf(root);		/* We know more than "k" hosts */
 }
 
-/**
- * Is the DHT enabled?
- */
-gboolean
-dht_enabled(void)
-{
-	return GNET_PROPERTY(enable_udp) && GNET_PROPERTY(enable_dht);
-}
-
 /**
  * Compute the hash list storing nodes with a given status.
  */
@@ -542,7 +561,7 @@ list_update_stats(knode_status_t status, int delta)
 	/* NOTREACHED */
 }
 
-#ifdef DEBUGGING
+#ifdef DHT_ROUTING_DEBUG
 /**
  * Check bucket list consistency.
  */
@@ -573,7 +592,7 @@ check_leaf_list_consistency(
 }
 #else
 #define check_leaf_list_consistency(a, b, c)
-#endif	/* DEBUGGING */
+#endif	/* DHT_ROUTING_DEBUG */
 
 /**
  * Get our KUID.
@@ -584,6 +603,22 @@ get_our_kuid(void)
 	return our_kuid;
 }
 
+/**
+ * Get our Kademlia node, with an IPv4 listening address.
+ */
+knode_t *
+get_our_knode(void)
+{
+	vendor_code_t gtkg;
+
+	gtkg.u32 = T_GTKG;
+
+	return knode_new(our_kuid,
+		GNET_PROPERTY(is_udp_firewalled) ? KDA_MSG_F_FIREWALLED : 0,
+		listen_addr(), socket_listen_port(), gtkg,
+		KDA_VERSION_MAJOR, KDA_VERSION_MINOR);
+}
+
 /*
  * Hash and equals functions for other_size items.
  *
@@ -700,8 +735,10 @@ forget_hashlist_node(gpointer knode, gpointer unused_data)
 	 * to KNODE_UNKNOWN for knode_dispose().
 	 */
 
-	if (1 == kn->refcnt)
-		kn->status = KNODE_UNKNOWN;
+	if (BOOT_SHUTDOWN == boot_status)
+		kn->status = KNODE_UNKNOWN;		/* No longer in route table */
+	else if (1 == kn->refcnt)
+		kn->status = KNODE_UNKNOWN;		/* For knode_dispose() */
 
 	knode_free(kn);
 }
@@ -839,8 +876,8 @@ recursively_apply(
 	if (r == NULL)
 		return;
 
-	recursively_apply(r->one, f, u);
 	recursively_apply(r->zero, f, u);
+	recursively_apply(r->one, f, u);
 	(*f)(r, u);
 }
 
@@ -881,6 +918,16 @@ bucket_refresh_status(const kuid_t *kuid, lookup_error_t error, gpointer arg)
 {
 	struct kbucket *kb = arg;
 
+	/*
+	 * Handle disabling of DHT whilst we were busy looking.
+	 */
+
+	if (NULL == root || LOOKUP_E_CANCELLED == error) {
+		if (GNET_PROPERTY(dht_debug))
+			g_message("DHT disabled during bucket refresh");
+		return;
+	}
+
 	if (GNET_PROPERTY(dht_debug) || GNET_PROPERTY(dht_lookup_debug)) {
 		g_message("DHT bucket refresh with %s "
 			"for %s %s (good: %u, stale: %u, pending: %u) completed: %s",
@@ -898,7 +945,7 @@ bucket_refresh_status(const kuid_t *kuid, lookup_error_t error, gpointer arg)
  * Issue a bucket refresh, if needed.
  */
 static void
-dht_bucket_refresh(struct kbucket *kb)
+dht_bucket_refresh(struct kbucket *kb, gboolean forced)
 {
 	kuid_t id;
 
@@ -910,8 +957,9 @@ dht_bucket_refresh(struct kbucket *kb)
 
 	if (boot_status != BOOT_COMPLETED) {
 		if (GNET_PROPERTY(dht_debug))
-			g_warning("DHT not fully bootstrapped, denying refresh of %s "
+			g_warning("DHT not fully bootstrapped, denying %srefresh of %s "
 				"(good: %u, stale: %u, pending: %u)",
+				forced ? "forced " : "",
 				kbucket_to_string(kb), list_count(kb, KNODE_GOOD),
 				list_count(kb, KNODE_STALE), list_count(kb, KNODE_PENDING));
 		return;
@@ -927,19 +975,26 @@ dht_bucket_refresh(struct kbucket *kb)
 	if (list_count(kb, KNODE_GOOD) == K_BUCKET_GOOD && !is_splitable(kb)) {
 		gnet_stats_count_general(GNR_DHT_DENIED_UNSPLITABLE_BUCKET_REFRESH, 1);
 		if (GNET_PROPERTY(dht_debug))
-			g_message("DHT denying refresh of non-splitable full %s "
+			g_message("DHT denying %srefresh of non-splitable full %s "
 				"(good: %u, stale: %u, pending: %u)",
+				forced ? "forced " : "",
 				kbucket_to_string(kb), list_count(kb, KNODE_GOOD),
 				list_count(kb, KNODE_STALE), list_count(kb, KNODE_PENDING));
 		return;
 	}
 
-	if (GNET_PROPERTY(dht_debug))
-		g_message("DHT initiating refresh of %ssplitable %s "
+	if (GNET_PROPERTY(dht_debug)) {
+		g_message("DHT initiating %srefresh of %ssplitable %s "
 			"(good: %u, stale: %u, pending: %u)",
+			forced ? "forced " : "",
 			is_splitable(kb) ? "" : "non-", kbucket_to_string(kb),
 			list_count(kb, KNODE_GOOD), list_count(kb, KNODE_STALE),
 			list_count(kb, KNODE_PENDING));
+	}
+
+	if (forced) {
+		gnet_stats_count_general(GNR_DHT_FORCED_BUCKET_REFRESH, 1);
+	}
 
 	/*
 	 * Generate a random KUID falling within this bucket's range.
@@ -958,9 +1013,12 @@ dht_bucket_refresh(struct kbucket *kb)
 	 * We're more aggressive for our k-bucket because we do not want to
 	 * end the lookup when we have k items in our path: we really want
 	 * to find the closest node we can.
+	 *
+	 * Likewise for forced refreshes, we want to converge to the random KUID
+	 * in order to hopefully fill the bucket, somehow.
 	 */
 
-	if (kb->ours)
+	if (kb->ours || forced)
 		(void) lookup_find_node(&id, NULL, bucket_refresh_status, kb);
 	else
 		(void) lookup_bucket_refresh(&id, bucket_refresh_status, kb);
@@ -996,7 +1054,7 @@ completion_iterate(struct bootstrap *b)
 	}
 
 	if (GNET_PROPERTY(dht_debug))
-		g_warning("DHT completing bootstrap with KUID %s (%d leading bit%s)",
+		g_warning("DHT completing bootstrap with KUID %s (%d bit%s)",
 			kuid_to_hex_string(&b->id), b->bits, 1 == b->bits ? "" : "s");
 }
 
@@ -1010,10 +1068,16 @@ bootstrap_completion_status(
 	struct bootstrap *b = arg;
 
 	/*
-	 * XXX Handle disabling of DHT whilst we are busy looking.
-	 * XXX This applies to other lookup callbacks as well.
+	 * Handle disabling of DHT whilst we were busy looking.
 	 */
 
+	if (NULL == root || LOOKUP_E_CANCELLED == error) {
+		wfree(b, sizeof *b);
+		if (GNET_PROPERTY(dht_debug))
+			g_warning("DHT disabled during bootstrap");
+		return;
+	}
+
 	if (GNET_PROPERTY(dht_debug) || GNET_PROPERTY(dht_lookup_debug))
 		g_message("DHT bootstrap with ID %s (%d bit%s) done: %s",
 			kuid_to_hex_string(kuid), b->bits, 1 == b->bits ? "" : "s",
@@ -1035,7 +1099,9 @@ bootstrap_completion_status(
 		return;
 	}
 
-	b->bits--;
+	if (LOOKUP_E_OK == error || LOOKUP_E_PARTIAL == error)
+		b->bits--;
+
 	completion_iterate(b);
 }
 
@@ -1080,6 +1146,16 @@ bootstrap_status(const kuid_t *kuid, lookup_error_t error, gpointer unused_arg)
 
 	bootstrapping = FALSE;
 
+	/*
+	 * Handle disabling of DHT whilst we were busy looking.
+	 */
+
+	if (NULL == root || LOOKUP_E_CANCELLED == error) {
+		if (GNET_PROPERTY(dht_debug))
+			g_warning("DHT disabled during bootstrap");
+		return;
+	}
+
 	if (GNET_PROPERTY(dht_debug))
 		g_message("DHT bootstrapping was %s seeded",
 			dht_seeded() ? "successfully" : "not fully");
@@ -1111,6 +1187,13 @@ bootstrap_status(const kuid_t *kuid, lookup_error_t error, gpointer unused_arg)
 void
 dht_attempt_bootstrap(void)
 {
+	/*
+	 * If the DHT is not initialized, ignore silently.
+	 */
+
+	if (NULL == root)
+		return;
+
 	/*
 	 * If we are already completely bootstrapped, ignore.
 	 */
@@ -1141,12 +1224,33 @@ dht_attempt_bootstrap(void)
 
 /**
  * Runtime (re)-initialization of the DHT.
+ * If UDP or the DHT is not enabled, do nothing.
  */
 void
 dht_initialize(gboolean post_init)
 {
 	size_t i;
 
+	if (!initialized)
+		return;				/* dht_init() not called yet */
+
+	if (!dht_enabled()) {
+		/* UDP or DHT not both enabled */
+		if (GNET_PROPERTY(dht_debug)) {
+			g_message("DHT will not initialize: UDP %s, DHT %s, port %u",
+				GNET_PROPERTY(enable_udp) ? "on" : "off",
+				GNET_PROPERTY(enable_dht) ? "on" : "off",
+				GNET_PROPERTY(listen_port));
+		}
+		return;
+	}
+
+	if (root != NULL) {
+		if (GNET_PROPERTY(dht_debug))
+			g_message("DHT already initialized");
+		return;				/* already initialized */
+	}
+
 	if (GNET_PROPERTY(dht_debug))
 		g_message("DHT initializing (%s init)",
 			post_init ? "post" : "first");
@@ -1186,6 +1290,9 @@ dht_initialize(gboolean post_init)
 	keys_init();
 	values_init();
 	publish_init();
+	roots_init();
+	tcache_init();
+	stable_init();
 
 	if (post_init)
 		dht_attempt_bootstrap();
@@ -1208,6 +1315,7 @@ dht_reset_kuid(void)
 void
 dht_init(void)
 {
+	initialized = TRUE;
 	boot_status = BOOT_NONE;
 
 	/*
@@ -1590,10 +1698,14 @@ add_node(struct kbucket *kb, knode_t *kn, knode_status_t new)
  * If the bucket that should manage the node is already full and it cannot
  * be split further, we need to see whether we don't have stale nodes in
  * there.  In which case the addition is pending, until we know for sure.
+ *
+ * @return TRUE if we added the node to the table.
  */
-static void
+static gboolean
 dht_add_node_to_bucket(knode_t *kn, struct kbucket *kb, gboolean traffic)
 {
+	gboolean added = FALSE;
+
 	knode_check(kn);
 	g_assert(is_leaf(kb));
 	g_assert(kb->nodes->all != NULL);
@@ -1608,6 +1720,7 @@ dht_add_node_to_bucket(knode_t *kn, struct kbucket *kb, gboolean traffic)
 		add_node(kb, kn, KNODE_GOOD);
 		stats.good++;
 		gnet_stats_count_general(GNR_DHT_ROUTING_GOOD_NODES, +1);
+		added = TRUE;
 		goto done;
 	}
 
@@ -1628,6 +1741,7 @@ dht_add_node_to_bucket(knode_t *kn, struct kbucket *kb, gboolean traffic)
 			add_node(kb, kn, KNODE_GOOD);
 			stats.good++;
 			gnet_stats_count_general(GNR_DHT_ROUTING_GOOD_NODES, +1);
+			added = TRUE;
 			goto done;
 		}
 	}
@@ -1646,10 +1760,34 @@ dht_add_node_to_bucket(knode_t *kn, struct kbucket *kb, gboolean traffic)
 		add_node(kb, kn, KNODE_PENDING);
 		gnet_stats_count_general(GNR_DHT_ROUTING_PENDING_NODES, +1);
 		stats.pending++;
+		added = TRUE;
 	}
 
 done:
 	check_leaf_bucket_consistency(kb);
+
+	return added;
+}
+
+/*
+ * If there's only one reference to this node, attempt to move
+ * it around if it can serve memory compaction.
+ *
+ * @return pointer to moved node
+ */
+static knode_t *
+move_node(struct kbucket *kb, knode_t *kn)
+{
+	if (1 == knode_refcnt(kn)) {
+		knode_t *moved = wmove(kn, sizeof *kn);
+		if (moved != kn) {
+			g_hash_table_remove(kb->nodes->all, moved->id);
+			g_hash_table_insert(kb->nodes->all, moved->id, moved);
+			return moved;
+		}
+	}
+
+	return kn;
 }
 
 /**
@@ -1696,6 +1834,8 @@ promote_pending_node(struct kbucket *kb)
 		hash_list_iter_release(&iter);
 
 		if (selected) {
+			time_delta_t elapsed;
+
 			if (GNET_PROPERTY(dht_debug))
 				g_message("DHT promoting %s node %s at %s to good in %s",
 					knode_status_to_string(selected->status),
@@ -1706,6 +1846,13 @@ promote_pending_node(struct kbucket *kb)
 			hash_list_remove(kb->nodes->pending, selected);
 			list_update_stats(KNODE_PENDING, -1);
 
+			/*
+			 * If there's only one reference to this node, attempt to move
+			 * it around if it can serve memory compaction.
+			 */
+
+			selected = move_node(kb, selected);
+
 			/*
 			 * Picked up node is the most recently seen pending node (at the
 			 * tail of the list), but it is not necessarily the latest seen
@@ -1722,8 +1869,17 @@ promote_pending_node(struct kbucket *kb)
 			 * ping it to make sure it's still alive.
 			 */
 
-			if (delta_time(tm_time(), selected->last_seen) >= ALIVENESS_PERIOD)
+			elapsed = delta_time(tm_time(), selected->last_seen);
+
+			if (elapsed >= ALIVENESS_PERIOD) {
+				if (GNET_PROPERTY(dht_debug))
+					g_message("DHT pinging promoted node (last seen %s)",
+						short_time(elapsed));
+
 				dht_rpc_ping(selected, NULL, NULL);
+				gnet_stats_count_general(
+					GNR_DHT_ROUTING_PINGED_PROMOTED_NODES, 1);
+			}
 
 			gnet_stats_count_general(GNR_DHT_ROUTING_PROMOTED_PENDING_NODES, 1);
 		}
@@ -1857,7 +2013,7 @@ dht_set_node_status(knode_t *kn, knode_status_t new)
 	if (tkn == kn && kn->status == new)
 		return;
 
-	if (GNET_PROPERTY(dht_debug))
+	if (GNET_PROPERTY(dht_debug) > 1)
 		g_message("DHT node %s at %s (%s in table) moving from %s to %s",
 			kuid_to_hex_string(kn->id),
 			host_addr_port_to_string(kn->addr, kn->port),
@@ -1955,6 +2111,11 @@ dht_set_node_status(knode_t *kn, knode_status_t new)
 		}
 	}
 
+	/*
+	 * Take this opportunity to move the node around if interesting.
+	 */
+
+	tkn = move_node(kb, tkn);
 	hash_list_append(hl, tkn);
 	list_update_stats(new, +1);
 
@@ -2031,8 +2192,11 @@ dht_record_activity(knode_t *kn)
  *
  * @param kn		the node we're trying to add
  * @param traffic	whether node was passively collected or we got data from it
+ *
+ * @return TRUE if we added the node to the table, FALSE if we rejected it or
+ * if it was already present.
  */
-static void
+static gboolean
 record_node(knode_t *kn, gboolean traffic)
 {
 	struct kbucket *kb;
@@ -2058,7 +2222,7 @@ record_node(knode_t *kn, gboolean traffic)
 				knode_to_string(kn));
 		if (!is_my_address_and_port(kn->addr, kn->port))
 			gnet_stats_count_general(GNR_DHT_OWN_KUID_COLLISIONS, 1);
-		return;
+		return FALSE;
 	}
 
 	g_assert(!g_hash_table_lookup(kb->nodes->all, kn->id));
@@ -2076,7 +2240,7 @@ record_node(knode_t *kn, gboolean traffic)
 				kuid_to_hex_string(kn->id),
 				host_addr_port_to_string(kn->addr, kn->port),
 				kbucket_to_string(kb));
-		return;
+		return FALSE;
 	}
 
 	/*
@@ -2088,16 +2252,17 @@ record_node(knode_t *kn, gboolean traffic)
 	if (traffic)
 		dht_record_activity(kn);
 
-	dht_add_node_to_bucket(kn, kb, traffic);
+	return dht_add_node_to_bucket(kn, kb, traffic);
 }
 
 /**
- * Record traffic from node.
+ * Record traffic from a new node.
  */
 void
 dht_traffic_from(knode_t *kn)
 {
-	record_node(kn, TRUE);
+	if (record_node(kn, TRUE))
+		keys_offload(kn);
 
 	/*
 	 * If not bootstrapped yet, we just got our seed.
@@ -2113,12 +2278,13 @@ dht_traffic_from(knode_t *kn)
 }
 
 /**
- * Add node to the table.
+ * Add node to the table after KUID verification.
  */
 static void
-dht_add_node(knode_t *node)
+dht_add_node(knode_t *kn)
 {
-	record_node(node, FALSE);
+	if (record_node(kn, FALSE))
+		keys_offload(kn);
 }
 
 /**
@@ -2222,12 +2388,27 @@ bucket_alive_check(cqueue_t *unused_cq, gpointer obj)
 
 	install_alive_check(kb);
 
-	if (GNET_PROPERTY(dht_debug))
+	if (GNET_PROPERTY(dht_debug)) {
 		g_message("DHT starting alive check on %s "
 			"(good: %u, stale: %u, pending: %u)",
 			kbucket_to_string(kb),
 			list_count(kb, KNODE_GOOD), list_count(kb, KNODE_STALE),
 			list_count(kb, KNODE_PENDING));
+	}
+
+	/*
+	 * If there are less than half the maximum amount of good nodes in the
+	 * bucket, force a bucket refresh.
+	 */
+
+	if (list_count(kb, KNODE_GOOD) < K_BUCKET_GOOD / 2) {
+		if (GNET_PROPERTY(dht_debug)) {
+			g_message("DHT forcing refresh of %s %s",
+				0 == list_count(kb, KNODE_GOOD) ? "empty" : "depleted",
+				kbucket_to_string(kb));
+		}
+		dht_bucket_refresh(kb, TRUE);
+	}
 
 	/*
 	 * Ping only the good contacts from which we haven't heard since the
@@ -2288,7 +2469,7 @@ bucket_refresh(cqueue_t *unused_cq, gpointer obj)
 	kb->nodes->last_lookup = tm_time();
 	install_bucket_refresh(kb);
 
-	dht_bucket_refresh(kb);
+	dht_bucket_refresh(kb, FALSE);
 }
 
 /**
@@ -2321,12 +2502,12 @@ dht_compute_size_estimate(patricia_t *pt, const kuid_t *kuid, int amount)
 	/*
 	 * Here is the algorithm used to compute the size estimate.
 	 *
-	 * We perform a routing table lookup of the NCNT nodes closest to our
-	 * own KUID.  Once we have that, we can estimate the sparseness of the
+	 * We perform a routing table lookup of the NCNT nodes closest to a
+	 * given KUID.  Once we have that, we can estimate the sparseness of the
 	 * results by computing:
 	 *
-	 *  Nodes = { node_1 .. node_n } sorted by increasing distance to our KUID
-	 *  D = sum of Di*i for i = 1..NCNT and Di = distance(node_i, our_kuid)
+	 *  Nodes = { node_1 .. node_n } sorted by increasing distance to the KUID
+	 *  D = sum of Di*i for i = 1..NCNT and Di = distance(node_i, KUID)
 	 *  S = sum of i*i for i = 1..NCNT
 	 *
 	 *  D/S represents the sparseness of the results.  If all results were
@@ -2345,7 +2526,7 @@ dht_compute_size_estimate(patricia_t *pt, const kuid_t *kuid, int amount)
 	STATIC_ASSERT(MAX_INT_VAL(guint32) >= NCNT * NCNT * NCNT);
 	STATIC_ASSERT(MAX_INT_VAL(guint8) >= NCNT);
 
-	while (patricia_iter_next(iter, (gpointer) &id, NULL, NULL)) {
+	while (patricia_iter_next(iter, (void *) &id, NULL, NULL)) {
 		kuid_t di;
 		gboolean saturated = FALSE;
 
@@ -2391,19 +2572,19 @@ dht_compute_size_estimate(patricia_t *pt, const kuid_t *kuid, int amount)
 		g_message("DHT target KUID is %s (%d node%s wanted, %u used)",
 			kuid_to_hex_string(kuid), amount, 1 == amount ? "" : "s",
 			(unsigned) (i - 1));
-		g_message("DHT dsum is %s = %lf", kuid_to_hex_string(&dsum), ds);
-		g_message("DHT squares is %s = %lf (%d)",
+		g_message("DHT dsum is %s = %f", kuid_to_hex_string(&dsum), ds);
+		g_message("DHT squares is %s = %f (%d)",
 			kuid_to_hex_string(&sq), s, squares);
 
-		g_message("DHT sparseness over %d nodes is %s = %lf (%lf)",
-			i - 1, kuid_to_hex_string(&sparseness),
+		g_message("DHT sparseness over %u nodes is %s = %f (%f)",
+			(unsigned) i - 1, kuid_to_hex_string(&sparseness),
 			kuid_to_double(&sparseness), ds / s);
 	}
 
 	/*
 	 * We can't divide 2^160 by the sparseness because we can't represent
 	 * that number in a KUID.  We're going to divide 2^160 - 1 instead, which
-	 * won't make much of a difference, and we add one.
+	 * won't make much of a difference, and we add one (for ourselves).
 	 */
 
 	kuid_divide(&max, &sparseness, &estimate, &r);
@@ -2445,13 +2626,18 @@ update_cached_size_estimate(void)
 	guint64 min = 0;
 	guint64 max = MAX_INT_VAL(guint64);
 
+	/*
+	 * Only retain the points that fall within one standard deviation of
+	 * the mean to remove obvious aberration.
+	 */
+
 	n = statx_n(stats.lookdata);
 	if (n > 1) {
 		guint64 sdev = (guint64) statx_sdev(stats.lookdata);
 		guint64 avg = (guint64) statx_avg(stats.lookdata);
-		if (2 * sdev < avg)
-			min = avg - 2 * sdev;
-		max = avg + 2 * sdev;
+		if (sdev < avg)
+			min = avg - sdev;
+		max = avg + sdev;
 	}
 
 	for (i = 0; i < K_REGIONS; i++) {
@@ -2490,7 +2676,7 @@ update_cached_size_estimate(void)
 			count, 1 == count ? "" : "s", n + 1 - count);
 		if (n > 1) {
 			g_message(
-				"DHT collected average is %.0lf (%d points), sdev = %.2lf",
+				"DHT collected average is %.0f (%d points), sdev = %.2f",
 				statx_avg(stats.lookdata), n, statx_sdev(stats.lookdata));
 		}
 	}
@@ -2518,14 +2704,9 @@ dht_update_subspace_size_estimate(
 
 	/*
 	 * See whether we have to trim some nodes (among the furthest).
-	 *
-	 * Trim the last KDA_ALPHA nodes in the path, since they could be
-	 * the furthest ones (the first we queried).
 	 */
 
 	kept = patricia_count(pt);
-	if (kept > KDA_ALPHA)
-		kept -= KDA_ALPHA;
 	if (kept > UNSIGNED(amount))
 		kept = amount;
 
@@ -2567,7 +2748,7 @@ dht_update_subspace_size_estimate(
 	statx_add(stats.lookdata, (double) estimate);
 
 	if (GNET_PROPERTY(dht_debug)) {
-		g_message("DHT subspace %02x estimate is %s (over %u/%d nodes)",
+		g_message("DHT subspace \"%02x\" estimate is %s (over %u/%d nodes)",
 			subspace, uint64_to_string(estimate), (unsigned) kept, amount);
 	}
 
@@ -2591,8 +2772,10 @@ dht_expire_size_estimates(void)
 			statx_remove(stats.lookdata, (double) stats.lookups[i].estimate);
 			stats.lookups[i].computed = 0;
 
-			if (GNET_PROPERTY(dht_debug))
-				g_message("DHT expired subspace %02x local size estimate", i);
+			if (GNET_PROPERTY(dht_debug)) {
+				g_message(
+					"DHT expired subspace \"%02x\" local size estimate", i);
+			}
 		}
 
 		stamp = stats.network[i].updated;
@@ -2606,8 +2789,10 @@ dht_expire_size_estimates(void)
 			}
 			stats.network[i].updated = 0;
 
-			if (GNET_PROPERTY(dht_debug))
-				g_message("DHT expired subspace %02x remote size estimates", i);
+			if (GNET_PROPERTY(dht_debug)) {
+				g_message(
+					"DHT expired subspace \"%02x\" remote size estimates", i);
+			}
 		}
 	}
 }
@@ -2631,7 +2816,7 @@ dht_update_size_estimate(void)
 	guint64 estimate;
 	gboolean alive = TRUE;
 
-	if (!GNET_PROPERTY(enable_dht))
+	if (!dht_enabled())
 		return;
 
 	kvec = walloc(K_LOCAL_ESTIMATE * sizeof(knode_t *));
@@ -2730,7 +2915,7 @@ dht_record_size_estimate(knode_t *kn, kuid_t *size)
 	os = walloc(sizeof *os);
 	os->id = kuid_get_atom(kn->id);
 
-	if (hash_list_contains(hl, os, &key)) {
+	if (hash_list_find(hl, os, &key)) {
 		/* This should happen only infrequently */
 		other_size_free(os);
 		data = deconstify_gpointer(key);
@@ -2811,10 +2996,9 @@ fill_closest_in_bucket(
 
 	/*
 	 * If we can determine that we do not have enough good nodes in the bucket
-	 * to fill the vector, use also the stale nodes for which the "grace"
-	 * period since the timeout has passed. Finally, also consider "pending"
-	 * nodes if we really miss nodes (excluding shutdowning ones), provided
-	 * we got traffic from them recently (defined by the aliveness period).
+	 * to fill the vector, consider "pending" nodes (excluding shutdowning
+	 * ones), provided we got traffic from them recently (defined by the
+	 * aliveness period).
 	 */
 
 	good = hash_list_list(kb->nodes->good);
@@ -2836,34 +3020,6 @@ fill_closest_in_bucket(
 		good = g_list_remove(good, kn);
 	}
 
-	if (available < kcnt) {
-		GList *stale = hash_list_list(kb->nodes->stale);
-
-		while (stale) {
-			knode_t *kn = stale->data;
-
-			knode_check(kn);
-			g_assert(KNODE_STALE == kn->status);
-
-			/*
-			 * Limit to stale nodes that can be recontacted and which
-			 * have only 1 timeout recorded.  Others are likely to be
-			 * really dead or changed their IP address.
-			 */
-
-			if (
-				knode_can_recontact(kn) && 1 == kn->rpc_timeouts &&
-				(!exclude || !kuid_eq(kn->id, exclude)) &&
-				(!alive || (kn->flags & KNODE_F_ALIVE))
-			) {
-				nodes = g_list_prepend(nodes, kn);
-				available++;
-			}
-
-			stale = g_list_remove(stale, kn);
-		}
-	}
-
 	if (available < kcnt) {
 		GList *pending = hash_list_list(kb->nodes->pending);
 		time_t now = tm_time();
@@ -2877,7 +3033,12 @@ fill_closest_in_bucket(
 			if (
 				!(kn->flags & KNODE_F_SHUTDOWNING) &&
 				(!exclude || !kuid_eq(kn->id, exclude)) &&
-				(!alive || delta_time(now, kn->last_seen) < ALIVENESS_PERIOD)
+				(!alive ||
+					(
+						(kn->flags & KNODE_F_ALIVE) &&
+						delta_time(now, kn->last_seen) < ALIVENESS_PERIOD
+					)
+				)
 			) {
 				nodes = g_list_prepend(nodes, kn);
 				available++;
@@ -2964,7 +3125,8 @@ recursively_fill_closest_from(
  * @param kvec		base of the "knode_t *" vector
  * @param kcnt		size of the "knode_t *" vector
  * @param exclude	the KUID to exclude (NULL if no exclusion)
- * @param alive		whether we want only know-to-be-alive nodes
+ * @param alive		whether we want only known-to-be-alive nodes
+ *
  * @return the amount of entries filled in the vector.
  */
 int
@@ -3120,45 +3282,44 @@ write_node(const knode_t *kn, FILE *f)
 }
 
 /**
- * Recursively store all good nodes from leaf buckets.
+ * Store all good nodes from a leaf bucket.
  */
 static void
-recursively_store_bucket(struct kbucket *kb, FILE *f)
+dht_store_leaf_bucket(struct kbucket *kb, gpointer u)
 {
-	if (is_leaf(kb)) {
-		hash_list_iter_t *iter;
+	FILE *f = u;
+	hash_list_iter_t *iter;
 
-		/*
-		 * All good nodes are persisted.
-		 */
+	if (!is_leaf(kb))
+		return;
 
-		iter = hash_list_iterator(kb->nodes->good);
-		while (hash_list_iter_has_next(iter)) {
-			const knode_t *kn;
+	/*
+	 * All good nodes are persisted.
+	 */
 
-			kn = hash_list_iter_next(iter);
-			write_node(kn, f);
-		}
-		hash_list_iter_release(&iter);
+	iter = hash_list_iterator(kb->nodes->good);
+	while (hash_list_iter_has_next(iter)) {
+		const knode_t *kn;
 
-		/*
-		 * Stale nodes for which the RPC timeout condition was cleared
-		 * are also elected.
-		 */
-
-		iter = hash_list_iterator(kb->nodes->stale);
-		while (hash_list_iter_has_next(iter)) {
-			const knode_t *kn;
-
-			kn = hash_list_iter_next(iter);
-			if (!kn->rpc_timeouts)
-				write_node(kn, f);
-		}
-		hash_list_iter_release(&iter);
-	} else {
-		recursively_store_bucket(kb->zero, f);
-		recursively_store_bucket(kb->one, f);
+		kn = hash_list_iter_next(iter);
+		write_node(kn, f);
 	}
+	hash_list_iter_release(&iter);
+
+	/*
+	 * Stale nodes for which the RPC timeout condition was cleared
+	 * are also elected.
+	 */
+
+	iter = hash_list_iterator(kb->nodes->stale);
+	while (hash_list_iter_has_next(iter)) {
+		const knode_t *kn;
+
+		kn = hash_list_iter_next(iter);
+		if (!kn->rpc_timeouts)
+			write_node(kn, f);
+	}
+	hash_list_iter_release(&iter);
 }
 
 /**
@@ -3192,7 +3353,7 @@ dht_route_store(void)
 	);
 
 	if (root)
-		recursively_store_bucket(root, f);
+		recursively_apply(root, dht_store_leaf_bucket, f);
 
 	file_config_close(f, &fp);
 	stats.dirty = FALSE;
@@ -3235,9 +3396,11 @@ other_size_free_cb(gpointer other_size, gpointer unused_data)
 
 /**
  * Shutdown the DHT.
+ *
+ * @param exiting	whether gtk-gnutella is exiting altogether
  */
 void
-dht_close(void)
+dht_close(gboolean exiting)
 {
 	size_t i;
 
@@ -3255,14 +3418,20 @@ dht_close(void)
 	 * the RPC and lookups, which rely on the routing table.
 	 */
 
-	publish_close();
+	lookup_close(exiting);
+	publish_close(exiting);
+	ulq_close(exiting);
+	stable_close();
+	tcache_close();
+	roots_close();
 	values_close();
 	keys_close();
-	ulq_close();
-	lookup_close();
 	dht_rpc_close();
 	token_close();
 
+	old_boot_status = boot_status;
+	boot_status = BOOT_SHUTDOWN;
+
 	recursively_apply(root, dht_free_bucket, NULL);
 	root = NULL;
 	kuid_atom_free_null(&our_kuid);
@@ -3489,6 +3658,7 @@ dht_ping(host_addr_t addr, guint16 port)
 	 */
 
 	dht_rpc_ping_extended(kn, RPC_CALL_NO_VERIFY, dht_ping_cb, NULL);
+	knode_free(kn);
 }
 
 /**
@@ -3664,6 +3834,8 @@ dht_route_parse(FILE *f)
 	gboolean done = FALSE;
 	time_delta_t most_recent = REFRESH_PERIOD;
 	time_t now = tm_time();
+	patricia_t *nodes;
+	patricia_iter_t *iter;
 	/* Variables filled for each entry */
 	host_addr_t addr;
 	guint16 port;
@@ -3675,6 +3847,7 @@ dht_route_parse(FILE *f)
 	g_return_if_fail(f);
 
 	bit_array_clear_range(tag_used, 0, NUM_DHT_ROUTE_TAGS);
+	nodes = patricia_create(KUID_RAW_BITSIZE);
 
 	while (fgets(line, sizeof line, f)) {
 		const char *tag_name, *value;
@@ -3711,7 +3884,7 @@ dht_route_parse(FILE *f)
 		tag_name = line;
 
 		tag = dht_route_string_to_tag(tag_name);
-		g_assert((int) tag >= 0 && tag <= NUM_DHT_ROUTE_TAGS);
+		g_assert(UNSIGNED(tag) <= NUM_DHT_ROUTE_TAGS);
 
 		if (tag != DHT_ROUTE_TAG_UNKNOWN && !bit_array_flip(tag_used, tag)) {
 			g_warning("dht_route_parse(): "
@@ -3788,7 +3961,6 @@ dht_route_parse(FILE *f)
 
 		if (done) {
 			knode_t *kn;
-			knode_t *tkn;
 			time_delta_t delta;
 
 			/*
@@ -3817,15 +3989,10 @@ dht_route_parse(FILE *f)
 			if (!knode_is_usable(kn)) {
 				g_warning("DHT ignoring persisted unusable %s",
 					knode_to_string(kn));
-			} else if ((tkn = dht_find_node(kn->id))) {
-				g_warning("DHT ignoring persisted dup %s (has %s already)",
-					knode_to_string(kn), knode_to_string2(tkn));
 			} else {
-				dht_add_node(kn);
+				patricia_insert(nodes, kn->id, kn);
 			}
 
-			knode_free(kn);
-
 			/* Reset state */
 			done = FALSE;
 			bit_array_clear_range(tag_used, 0, NUM_DHT_ROUTE_TAGS);
@@ -3837,6 +4004,33 @@ dht_route_parse(FILE *f)
 		break;
 	}
 
+	/*
+	 * Now insert the recorded nodes in topological order, so that
+	 * we fill the closest subtree first and minimize the level of
+	 * splitting in the furthest parts of the tree.
+	 */
+
+	iter = patricia_metric_iterator_lazy(nodes, our_kuid, TRUE);
+
+	while (patricia_iter_has_next(iter)) {
+		knode_t *tkn;
+		knode_t *kn = patricia_iter_next_value(iter);
+		if ((tkn = dht_find_node(kn->id))) {
+			g_warning("DHT ignoring persisted dup %s (has %s already)",
+				knode_to_string(kn), knode_to_string2(tkn));
+		} else {
+			if (!record_node(kn, FALSE)) {
+				/* This can happen when the furthest subtrees are full */
+				if (GNET_PROPERTY(dht_debug)) {
+					g_message("DHT ignored persisted %s", knode_to_string(kn));
+				}
+			}
+		}
+	}
+	patricia_iterator_release(&iter);
+	patricia_foreach(nodes, knode_patricia_free, NULL);
+	patricia_destroy(nodes);
+
 	/*
 	 * If the delta is smaller than half the bucket refresh period, we
 	 * can consider the table as being bootstrapped: they are restarting
@@ -3846,6 +4040,8 @@ dht_route_parse(FILE *f)
 	if (dht_seeded()) {
 		boot_status =
 			most_recent < REFRESH_PERIOD / 2 ? BOOT_COMPLETED : BOOT_SEEDED;
+		if (old_boot_status != BOOT_NONE && old_boot_status != BOOT_COMPLETED)
+			boot_status = old_boot_status;
 	}
 
 	if (GNET_PROPERTY(dht_debug))
