@@ -1,5 +1,5 @@
 /*
- * $Id: ulq.c 15858 2008-11-07 19:15:10Z rmanfredi $
+ * $Id: ulq.c 17317 2010-02-27 23:41:12Z rmanfredi $
  *
  * Copyright (c) 2008, Raphael Manfredi
  *
@@ -55,7 +55,7 @@
 
 #include "common.h"
 
-RCSID("$Id: ulq.c 15858 2008-11-07 19:15:10Z rmanfredi $")
+RCSID("$Id: ulq.c 17317 2010-02-27 23:41:12Z rmanfredi $")
 
 #include "ulq.h"
 #include "kuid.h"
@@ -81,7 +81,7 @@ RCSID("$Id: ulq.c 15858 2008-11-07 19:15:10Z rmanfredi $")
 #define vema(x)	((x) >> ULQ_EMA_SHIFT)
 
 enum ulq_magic {
-	ULQ_MAGIC = 0xfc777379U
+	ULQ_MAGIC = 0x7c777379U
 };
 
 /**
@@ -107,7 +107,7 @@ enum ulqitem_magic {
  */
 struct ulq_item {
 	enum ulqitem_magic magic;
-	lookup_type_t type;				/**< Type of lookup (NODE or VALUE) */
+	lookup_type_t type;				/**< Type of lookup (STORE or VALUE) */
 	const kuid_t *kuid;				/**< KUID to look for (atom) */
 	struct ulq *uq;					/**< The queue where item lies */
 	union {
@@ -131,7 +131,9 @@ struct ulq_item {
 enum ulq_qtype {
 	ULQ_PROX = 0,					/**< Push proxy lookups */
 	ULQ_ALOC,						/**< Alt-loc lookups */
-	ULQ_NODE,						/**< Node lookups (for publishing) */
+	ULQ_STORE,						/**< Node lookups (for publishing) */
+	ULQ_PRIO,						/**< Prioritary lookups */
+	ULQ_OTHER,						/**< Other types of lookups */
 
 	ULQ_QUEUE_COUNT					/**< Amount of queues */
 };
@@ -144,7 +146,6 @@ static cevent_t *service_ev;				/**< Servicing event */
  */
 static struct ulq_sched {
 	slist_t *runq;					/**< Runnable queues */
-	guint weight;					/**< Total weight among queues */
 	int running;					/**< Total running lookups */
 	int pending;					/**< Total pending lookups */
 	int bw_in_ema;					/**< Slow EMA of incoming b/w per lookup */
@@ -161,14 +162,14 @@ static void ulq_delay_servicing(void);
 static inline void
 ulq_check(const struct ulq *uq)
 {
-	g_assert(uq);
+	g_assert(uq != NULL);
 	g_assert(ULQ_MAGIC == uq->magic);
 }
 
 static inline void
 ulq_item_check(const struct ulq_item *ui)
 {
-	g_assert(ui);
+	g_assert(ui != NULL);
 	g_assert(ULQ_ITEM_MAGIC == ui->magic);
 }
 
@@ -200,6 +201,8 @@ free_ulq_item(struct ulq_item *ui)
 	ulq_item_check(ui);
 
 	kuid_atom_free(ui->kuid);
+	ui->kuid = NULL;
+	ui->magic = 0;
 	wfree(ui, sizeof *ui);
 }
 
@@ -267,7 +270,7 @@ ulq_error_cb(const kuid_t *kuid, lookup_error_t error, gpointer arg)
 	struct ulq_item *ui = arg;
 
 	ulq_item_check(ui);
-	g_assert(LOOKUP_VALUE == ui->type || LOOKUP_NODE == ui->type);
+	g_assert(LOOKUP_VALUE == ui->type || LOOKUP_STORE == ui->type);
 	g_assert(ui->kuid == kuid);		/* Atoms */
 
 	(*ui->err)(ui->kuid, error, ui->arg);
@@ -299,7 +302,7 @@ ulq_node_found_cb(const kuid_t *kuid, const lookup_rs_t *rs, gpointer arg)
 	struct ulq_item *ui = arg;
 
 	ulq_item_check(ui);
-	g_assert(LOOKUP_NODE == ui->type);
+	g_assert(LOOKUP_STORE == ui->type);
 	g_assert(ui->kuid == kuid);		/* Atoms */
 
 	(*ui->u.fn.ok)(ui->kuid, rs, ui->arg);
@@ -328,7 +331,7 @@ ulq_lookup_stats(const kuid_t *kuid,
 	 * correct the read values later on by the same amount.
 	 */
 
-	if (ls->elapsed) {
+	if (ls->elapsed > 0.0) {
 		avg = (ls->bw_incoming << ULQ_EMA_SHIFT) / ls->elapsed;
 		sched.bw_in_ema += (avg >> 4) - (sched.bw_in_ema >> 4);
 
@@ -375,7 +378,7 @@ ulq_lookup_stats(const kuid_t *kuid,
 static const char *
 ulq_queue_status(void)
 {
-	static char buf[80];
+	static char buf[120];
 	size_t i, offset = 0;
 
 	for (i = 0; i < G_N_ELEMENTS(ulq); i++) {
@@ -419,10 +422,13 @@ ulq_launch(struct ulq *uq)
 		nl = lookup_find_value(ui->kuid, ui->u.fv.vtype,
 			ulq_value_found_cb, ulq_error_cb, ui);
 		goto initialized;
-	case LOOKUP_NODE:
-		nl = lookup_find_node(ui->kuid, ulq_node_found_cb, ulq_error_cb, ui);
+	case LOOKUP_STORE:
+		nl = lookup_store_nodes(ui->kuid, ulq_node_found_cb, ulq_error_cb, ui);
 		goto initialized;
+		break;
 	case LOOKUP_REFRESH:
+	case LOOKUP_NODE:
+	case LOOKUP_TOKEN:
 		break;
 	}
 	nl = NULL;
@@ -431,6 +437,7 @@ ulq_launch(struct ulq *uq)
 initialized:
 
 	if (nl) {
+		ulq_item_check(ui);
 		slist_append(uq->launched, ui);
 		uq->running++;
 		uq->scheduled++;
@@ -439,6 +446,14 @@ initialized:
 
 		if (LOOKUP_VALUE == ui->type) {
 			switch (ui->u.fv.vtype) {
+			case DHT_VT_ANY:
+				/*
+				 * We use generic lookups to locate PROX or NOPE values.
+				 * These are scheduled in the ULQ_PROX queue.
+				 */
+				if (uq == ulq[ULQ_PROX])
+					gnet_stats_count_general(GNR_DHT_PUSH_PROXY_LOOKUPS, 1);
+				break;
 			case DHT_VT_PROX:
 				gnet_stats_count_general(GNR_DHT_PUSH_PROXY_LOOKUPS, 1);
 				break;
@@ -543,7 +558,7 @@ ulq_service(void)
 		 * queries, stop launching.
 		 */
 
-		if (node_udp_would_flow_control(KDA_ALPHA * vema(sched.sz_out_ema)))
+		if (node_dht_would_flow_control(KDA_ALPHA * vema(sched.sz_out_ema)))
 			break;
 
 		if (0 == slist_length(sched.runq))
@@ -610,7 +625,7 @@ ulq_do_service(cqueue_t *unused_cq, gpointer unused_obj)
 	 * a sudden release of many lookups when we resume.
 	 */
 
-	if (node_udp_is_flow_controlled()) {
+	if (node_dht_is_flow_controlled()) {
 		if (GNET_PROPERTY(dht_ulq_debug))
 			g_warning("DHT ULQ deferring servicing: UDP queue flow-controlled");
 
@@ -646,18 +661,24 @@ ulq_needs_servicing(void)
 }
 
 /**
- * @return proper lookup queue depending on the type of data.
+ * @return proper lookup queue depending on the type of data and whether
+ * the lookup is flagged as urgent.
  */
 static struct ulq *
-ulq_get(lookup_type_t ltype, dht_value_type_t vtype)
+ulq_get(lookup_type_t ltype, dht_value_type_t vtype, gboolean prioritary)
 {
+	if (prioritary)
+		return ulq[ULQ_PRIO];
+
 	switch (ltype) {
-	case LOOKUP_NODE:
-		return ulq[ULQ_NODE];
+	case LOOKUP_STORE:
+		return ulq[ULQ_STORE];
 		break;
 	case LOOKUP_VALUE:
 		break;
 	case LOOKUP_REFRESH:
+	case LOOKUP_NODE:
+	case LOOKUP_TOKEN:
 		g_assert_not_reached();
 	}
 
@@ -666,10 +687,11 @@ ulq_get(lookup_type_t ltype, dht_value_type_t vtype)
 	switch (vtype) {
 	case DHT_VT_ALOC:
 		return ulq[ULQ_ALOC];
+	case DHT_VT_NOPE:
 	case DHT_VT_PROX:
 		return ulq[ULQ_PROX];
 	default:
-		return ulq[ULQ_ALOC];		/* XXX or dedicated queue for others? */
+		return ulq[ULQ_OTHER];
 	}
 }
 
@@ -693,13 +715,13 @@ ulq_putq(struct ulq *uq, struct ulq_item *ui)
 }
 
 /**
- * Enqueue node lookup.
+ * Enqueue store roots lookup.
  *
  * This is meant to be used only via user store operations, and is not to be
  * directly invoked by user code.
  */
 void
-ulq_find_node(const kuid_t *kuid,
+ulq_find_store_roots(const kuid_t *kuid, gboolean prioritary,
 	lookup_cb_ok_t ok, lookup_cb_err_t error, gpointer arg)
 {
 	struct ulq_item *ui;
@@ -708,16 +730,16 @@ ulq_find_node(const kuid_t *kuid,
 	g_assert(ok);
 	g_assert(error);
 
-	uq = ulq_get(LOOKUP_NODE, DHT_VT_BINARY);
+	uq = ulq_get(LOOKUP_STORE, DHT_VT_BINARY, prioritary);
 
-	ui = allocate_ulq_item(LOOKUP_NODE,  kuid, error, arg);
+	ui = allocate_ulq_item(LOOKUP_STORE,  kuid, error, arg);
 	ui->u.fn.ok = ok;
 
 	ulq_putq(uq, ui);
 }
 
 /**
- * Enqueue value lookup.
+ * Enqueue value lookup of specific type.
  */
 void
 ulq_find_value(const kuid_t *kuid, dht_value_type_t type,
@@ -729,7 +751,7 @@ ulq_find_value(const kuid_t *kuid, dht_value_type_t type,
 	g_assert(ok);
 	g_assert(error);
 
-	uq = ulq_get(LOOKUP_VALUE, type);
+	uq = ulq_get(LOOKUP_VALUE, type, FALSE);
 
 	ui = allocate_ulq_item(LOOKUP_VALUE,  kuid, error, arg);
 	ui->u.fv.ok = ok;
@@ -738,6 +760,31 @@ ulq_find_value(const kuid_t *kuid, dht_value_type_t type,
 	ulq_putq(uq, ui);
 }
 
+/**
+ * Enqueue value lookup for any value type.
+ *
+ * The specified ``queue_type'' is simply used to select the proper
+ * scheduling queue.
+ */
+void
+ulq_find_any_value(const kuid_t *kuid, dht_value_type_t queue_type,
+	lookup_cbv_ok_t ok, lookup_cb_err_t error, gpointer arg)
+{
+	struct ulq_item *ui;
+	struct ulq *uq;
+
+	g_assert(ok);
+	g_assert(error);
+
+	uq = ulq_get(LOOKUP_VALUE, queue_type, FALSE);
+
+	ui = allocate_ulq_item(LOOKUP_VALUE,  kuid, error, arg);
+	ui->u.fv.ok = ok;
+	ui->u.fv.vtype = DHT_VT_ANY;	/* Generic type */
+
+	ulq_putq(uq, ui);
+}
+
 /**
  * Initialize a user lookup queue.
  *
@@ -770,38 +817,69 @@ ulq_init_queue(const char *name, int weight)
 void
 ulq_init(void)
 {
-	size_t i;
-
 	STATIC_ASSERT(ULQ_QUEUE_COUNT == G_N_ELEMENTS(ulq));
 
-	ulq[ULQ_PROX] = ulq_init_queue("PROX", 60);
-	ulq[ULQ_ALOC] = ulq_init_queue("ALOC", 20);
-	ulq[ULQ_NODE] = ulq_init_queue("NODE", 20);
+	/*
+	 * The weights are arbitrary values, not necessarily summing to 100.
+	 *
+	 * To avoid too frequent schduler resets, they are somehow scaled, i.e.
+	 * we would not put 12,3,4,1 as weights, but 60,15,20,5 to ensure we will
+	 * serve a fair amount of entries in each queue between each scheduler
+	 * resets -- see ulq_sched_reset().
+	 *
+	 * Values must not be too large though, or a given queue would monopolize
+	 * the scheduling as long as it has entries and others have expired all
+	 * their slots.
+	 *
+	 * The ULQ_PRIO queue is meant for things that should be scheduled more
+	 * urgently, and its weight must therefore be a number greater than the
+	 * others but not too large or a bug (like too many enqueuing of prioritary
+	 * lookups) would litterally starve the other queues.
+	 */
 
+	ulq[ULQ_PROX]	= ulq_init_queue("PROX", 60);
+	ulq[ULQ_ALOC]	= ulq_init_queue("ALOC", 10);
+	ulq[ULQ_STORE]	= ulq_init_queue("STORE", 25);
+	ulq[ULQ_OTHER]	= ulq_init_queue("OTHER", 5);
+	ulq[ULQ_PRIO]	= ulq_init_queue("PRIO", 100);
+
+	memset(&sched, 0, sizeof sched);
 	sched.runq = slist_new();
-
-	for (i = 0; i < G_N_ELEMENTS(ulq); i++)
-		sched.weight += ulq[i]->weight;
 }
 
 /**
  * FIFO free item freeing callback.
  */
 static void
-free_fifo_item(gpointer item, gpointer unused_data)
+free_fifo_item(gpointer item, gpointer data)
 {
-	(void) unused_data;
+	struct ulq_item *ui = item;
+	gboolean *exiting = data;
 
-	free_ulq_item(item);
+	ulq_item_check(ui);
+
+	/*
+	 * If the process is not exiting, we're simply shutdowning the DHT
+	 * layer and therefore callers need to be informed that their enqueued
+	 * lookups were cancelled, so that they may cleanup after themselves.
+	 */
+
+	if (!*exiting)
+		(*ui->err)(ui->kuid, LOOKUP_E_CANCELLED, ui->arg);
+
+	free_ulq_item(ui);
 }
 
 /**
  * Shutdown the user lookup queue.
+ *
+ * @param exiting	whether the whole process is exiting
  */
 void
-ulq_close(void)
+ulq_close(gboolean exiting)
 {
 	size_t i;
+	gboolean one = TRUE;
 
 	cq_cancel(callout_queue, &service_ev);
 	slist_free(&sched.runq);
@@ -810,9 +888,19 @@ ulq_close(void)
 		struct ulq *uq = ulq[i];
 
 		if (uq) {
-			fifo_free_all(uq->q, free_fifo_item, NULL);
-			slist_foreach(uq->launched, free_fifo_item, NULL);
+			/*
+			 * Do not invoke callback for launched lookups, they will be
+			 * duly cancelled by lookup_close(): tell free_fifo_item() that
+			 * we are exiting.
+			 *
+			 * Enqueued lookups on the other hand (still in the FIFO) need
+			 * to be properly cleaned-up by forcing the registered error
+			 * callback, since there is no lookup object yet.
+			 */
+
+			slist_foreach(uq->launched, free_fifo_item, &one);
 			slist_free(&uq->launched);
+			fifo_free_all(uq->q, free_fifo_item, &exiting);
 			wfree(uq, sizeof *uq);
 
 			ulq[i] = NULL;
