@@ -1,5 +1,5 @@
 /*
- * $Id: ulq.h 15666 2008-08-18 21:18:24Z rmanfredi $
+ * $Id: ulq.h 17215 2009-11-09 21:14:29Z rmanfredi $
  *
  * Copyright (c) 2008, Raphael Manfredi
  *
@@ -44,10 +44,7 @@
  */
 
 void ulq_init(void);
-void ulq_close(void);
-
-void ulq_find_node(const kuid_t *kuid,
-	lookup_cb_ok_t ok, lookup_cb_err_t error, gpointer arg);
+void ulq_close(gboolean exiting);
 
 #endif /* _dht_ulq_h_ */
 
