@@ -1,5 +1,5 @@
 /*
- * $Id: values.c 16213 2009-03-19 23:06:09Z rmanfredi $
+ * $Id: values.c 17383 2010-03-18 08:44:53Z rmanfredi $
  *
  * Copyright (c) 2008, Raphael Manfredi
  *
@@ -64,7 +64,7 @@
 
 #include "common.h"
 
-RCSID("$Id: values.c 16213 2009-03-19 23:06:09Z rmanfredi $")
+RCSID("$Id: values.c 17383 2010-03-18 08:44:53Z rmanfredi $")
 
 #include "values.h"
 #include "kuid.h"
@@ -72,6 +72,8 @@ RCSID("$Id: values.c 16213 2009-03-19 23:06:09Z rmanfredi $")
 #include "storage.h"
 #include "acct.h"
 #include "keys.h"
+#include "kmsg.h"
+#include "stable.h"
 
 #include "if/gnet_property.h"
 #include "if/gnet_property_priv.h"
@@ -86,18 +88,33 @@ RCSID("$Id: values.c 16213 2009-03-19 23:06:09Z rmanfredi $")
 #include "lib/dbmw.h"
 #include "lib/glib-missing.h"
 #include "lib/host_addr.h"
-#include "lib/misc.h"
+#include "lib/parse.h"
 #include "lib/pmsg.h"
+#include "lib/stringify.h"
 #include "lib/tm.h"
+#include "lib/unsigned.h"
 #include "lib/vendors.h"
 #include "lib/walloc.h"
 #include "lib/override.h"		/* Must be the last header included */
 
+/*
+ * We set a maximum amount of values that can be published locally to our
+ * node from a given IP address or from a class C network, in order to
+ * avoid massive attacks / DHT pollution attempts.
+ *
+ * However, we must understand that due to caching, we may very well be
+ * handed out values that fall well outside our k-ball.  So we must keep
+ * the thresholds high enough to not limit useful STORE requests.
+ */
+#define MAX_VALUES_IP	128		/**< Max # of values allowed per IP address */
+#define MAX_VALUES_NET	1024	/**< Max # of values allowed per class C net */
+
 #define MAX_VALUES		131072	/**< Max # of values we accept to manage */
-#define MAX_VALUES_IP	16		/**< Max # of values allowed per IP address */
-#define MAX_VALUES_NET	256		/**< Max # of values allowed per class C net */
 #define EXPIRE_PERIOD	30		/**< Asynchronous expire period: 30 secs */
 
+#define VALUES_DB_CACHE_SIZE 1024	/**< Amount of values to keep cached */
+#define RAW_DB_CACHE_SIZE	 512	/**< Amount of raw data to keep cached */
+
 #define equiv(p,q)  (!(p) == !(q))
 
 /**
@@ -118,7 +135,7 @@ RCSID("$Id: values.c 16213 2009-03-19 23:06:09Z rmanfredi $")
  */
 struct valuedata {
 	kuid_t id;					/**< The primary key of the value */
-	time_t publish;				/**< Initial publish time at our node */
+	time_t publish;				/**< Last publish time at our node */
 	time_t replicated;			/**< Last replication time */
 	time_t expire;				/**< Expiration time */
 	/* Creator information */
@@ -134,6 +151,13 @@ struct valuedata {
 	guint8 value_minor;			/**< Minor version of value */
 	guint16 length;				/**< Value length */
 	gboolean original;			/**< Whether we got data from creator */
+	/* Statistics */
+	time_t created;				/**< When we first created the value */
+	guint32 n_republish;		/**< Amount of republishing we had */
+	guint32 n_replication;		/**< Amount of replication we had */
+	guint32 s_elapsed_publish;	/**< Sum of elapsed time between publications */
+	guint32 s_elapsed_replicat;	/**< Sum of elapsed time between replications */
+	guint32 n_requests;			/**< Amount of time value was requested */
 };
 
 /**
@@ -182,9 +206,7 @@ static dbmw_t *db_expired;
 static char db_expbase[] = "dht_expired";
 static char db_expwhat[] = "DHT expired values";
 
-static cevent_t *expire_ev;			/**< Event for periodic value expiration */
-
-static void install_periodic_expire(void);
+static cperiodic_t *values_expire_ev;	/**< Value expire periodic event */
 
 /**
  * @return amount of values managed.
@@ -222,12 +244,19 @@ serialize_valuedata(pmsg_t *mb, gconstpointer data)
 	pmsg_write_u8(mb, vd->value_minor);
 	pmsg_write_be16(mb, vd->length);
 	pmsg_write_boolean(mb, vd->original);
+	/* Statistics */
+	pmsg_write_time(mb, vd->created);
+	pmsg_write_be32(mb, vd->n_republish);
+	pmsg_write_be32(mb, vd->n_replication);
+	pmsg_write_be32(mb, vd->s_elapsed_publish);
+	pmsg_write_be32(mb, vd->s_elapsed_replicat);
+	pmsg_write_be32(mb, vd->n_requests);
 }
 
 /**
  * Deserialization routine for valuedata.
  */
-static gboolean
+static void
 deserialize_valuedata(bstr_t *bs, gpointer valptr, size_t len)
 {
 	struct valuedata *vd = valptr;
@@ -251,17 +280,84 @@ deserialize_valuedata(bstr_t *bs, gpointer valptr, size_t len)
 	bstr_read_u8(bs, &vd->value_minor);
 	bstr_read_be16(bs, &vd->length);
 	bstr_read_boolean(bs, &vd->original);
+	/* Statistics */
+	bstr_read_time(bs, &vd->created);
+	bstr_read_be32(bs, &vd->n_republish);
+	bstr_read_be32(bs, &vd->n_replication);
+	bstr_read_be32(bs, &vd->s_elapsed_publish);
+	bstr_read_be32(bs, &vd->s_elapsed_replicat);
+	bstr_read_be32(bs, &vd->n_requests);
+}
 
-	if (bstr_has_error(bs))
-		return FALSE;
-	else if (bstr_unread_size(bs)) {
-		/* Something is wrong, we're not deserializing the right data */
-		g_warning("DHT deserialization of valuedata: has %lu unread bytes",
-			(gulong) bstr_unread_size(bs));
-		return FALSE;
-	}
+/**
+ * Converts store status code to string.
+ */
+static const char * const store_errstr[] = {
+	"INVALID",								/* O */
+	"OK",									/**< STORE_SC_OK */
+	"Error",								/**< STORE_SC_ERROR */
+	"Node is full for this key",			/**< STORE_SC_FULL */
+	"Node is loaded for this key",			/**< STORE_SC_LOADED */
+	"Node is both loaded and full for key",	/**< STORE_SC_FULL_LOADED */
+	"Value is too large",					/**< STORE_SC_TOO_LARGE */
+	"Storage space exhausted",				/**< STORE_SC_EXHAUSTED */
+	"Creator is not acceptable",			/**< STORE_SC_BAD_CREATOR */
+	"Analyzed value did not validate",		/**< STORE_SC_BAD_VALUE */
+	"Improper value type",					/**< STORE_SC_BAD_TYPE */
+	"Storage quota for creator reached",	/**< STORE_SC_QUOTA */
+	"Replicated data is different",			/**< STORE_SC_DATA_MISMATCH */
+	"Invalid security token",				/**< STORE_SC_BAD_TOKEN */
+	"Value has already expired",			/**< STORE_SC_EXPIRED */
+	"Database I/O error",					/**< STORE_SC_DB_IO */
+};
 
-	return TRUE;
+/**
+ * @return human-readable error string corresponding to error code `errnum'.
+ */
+const char *
+dht_store_error_to_string(guint16 errnum)
+{
+	if (errnum == 0 || errnum >= G_N_ELEMENTS(store_errstr))
+		return "Invalid error code";
+
+	return store_errstr[errnum];
+}
+
+/**
+ * A DHT value.
+ */
+struct dht_value {
+	const knode_t *creator;	/**< The creator of the value */
+	kuid_t *id;				/**< The key of the value (atom) */
+	dht_value_type_t type;	/**< Type of values */
+	guint8 major;			/**< Value's major version */
+	guint8 minor;			/**< Value's minor version */
+	guint16 length;			/**< Length of value */
+	gconstpointer data;		/**< The actual data value */
+};
+
+const kuid_t *
+dht_value_key(const dht_value_t *v)
+{
+	return v->id;
+}
+
+const knode_t *
+dht_value_creator(const dht_value_t *v)
+{
+	return v->creator;
+}
+
+dht_value_type_t
+dht_value_type(const dht_value_t *v)
+{
+	return v->type;
+}
+
+guint16
+dht_value_length(const dht_value_t *v)
+{
+	return v->length;
 }
 
 /**
@@ -277,7 +373,7 @@ deserialize_valuedata(bstr_t *bs, gpointer valptr, size_t len)
  */
 dht_value_t *
 dht_value_make(const knode_t *creator,
-	kuid_t *primary_key, dht_value_type_t type,
+	const kuid_t *primary_key, dht_value_type_t type,
 	guint8 major, guint8 minor, gpointer data, guint16 length)
 {
 	dht_value_t *v;
@@ -360,6 +456,9 @@ dht_value_lifetime(dht_value_type_t type)
 	case DHT_VT_PROX:
 		lifetime = DHT_VALUE_PROX_EXPIRE;
 		break;
+	case DHT_VT_NOPE:
+		lifetime = DHT_VALUE_NOPE_EXPIRE;
+		break;
 	case DHT_VT_ALOC:
 		lifetime = DHT_VALUE_ALOC_EXPIRE;
 		break;
@@ -462,6 +561,108 @@ dht_value_to_string(const dht_value_t *v)
 	return buf;
 }
 
+/**
+ * Serialize a DHT value.
+ */
+void
+dht_value_serialize(pmsg_t *mb, const dht_value_t *v)
+{
+	/* DHT value header */
+	kmsg_serialize_contact(mb, v->creator);
+	pmsg_write(mb, v->id, KUID_RAW_SIZE);
+	pmsg_write_be32(mb, v->type);
+	pmsg_write_u8(mb, v->major);
+	pmsg_write_u8(mb, v->minor);
+	pmsg_write_be16(mb, v->length);
+
+	/* DHT value data */
+	if (v->length)
+		pmsg_write(mb, v->data, v->length);
+}
+
+/**
+ * Deserialize a DHT value.
+ *
+ * @return the deserialized DHT value, or NULL if an error occurred.
+ */
+dht_value_t *
+dht_value_deserialize(bstr_t *bs)
+{
+	dht_value_t *dv;
+	kuid_t id;
+	knode_t *creator;
+	guint8 major, minor;
+	guint16 length;
+	gpointer data = NULL;
+	guint32 type;
+
+	creator = kmsg_deserialize_contact(bs);
+	if (!creator)
+		return NULL;
+
+	bstr_read(bs, id.v, KUID_RAW_SIZE);
+	bstr_read_be32(bs, &type);
+	bstr_read_u8(bs, &major);
+	bstr_read_u8(bs, &minor);
+	bstr_read_be16(bs, &length);
+
+	if (bstr_has_error(bs))
+		goto error;
+
+	if (length && length <= DHT_VALUE_MAX_LEN) {
+		data = walloc(length);
+		bstr_read(bs, data, length);
+	} else {
+		bstr_skip(bs, length);
+	}
+
+	if (bstr_has_error(bs))
+		goto error;
+
+	dv = dht_value_make(creator, &id, type, major, minor, data, length);
+	knode_free(creator);
+	return dv;
+
+error:
+	knode_free(creator);
+	if (data)
+		wfree(data, length);
+	return NULL;
+}
+
+/**
+ * qsort() callback to compare two DHT values on a length basis.
+ */
+int
+dht_value_cmp(const void *a, const void *b)
+{
+	const dht_value_t * const *pa = a;
+	const dht_value_t * const *pb = b;
+	const dht_value_t *va = *pa;
+	const dht_value_t *vb = *pb;
+
+	return va->length == vb->length ? 0 :
+		va->length < vb->length ? -1 : +1;
+}
+
+/**
+ * Fill lookup result record with value.
+ *
+ * @attention
+ * The filled record becomes the owner of the value data.
+ */
+void
+dht_value_fill_record(const dht_value_t *v, lookup_val_rc_t *rc)
+{
+	rc->data = v->data;
+	rc->length = (size_t) v->length;
+	rc->addr = v->creator->addr;
+	rc->type = v->type;
+	rc->port = v->creator->port;
+	rc->major = v->major;
+	rc->minor = v->minor;
+}
+
 /**
  * Hash a pair of KUIDs.
  */
@@ -536,6 +737,14 @@ kuid_pair_has_expired(const kuid_t *key, const kuid_t *skey)
 	if (DBMAP_MAP == dbmw_map_type(db_expired))
 		return;
 
+	/*
+	 * Only cache expired keys within our k-ball.  Anything outside
+	 * is just cached data, not replication from our k-closest nodes.
+	 */
+
+	if (!keys_within_kball(key))
+		return;
+
 	kuid_pair_fill(buf, sizeof buf, key, skey);
 	dbmw_write(db_expired, buf, NULL, 0);
 }
@@ -573,13 +782,14 @@ get_valuedata(guint64 dbkey)
 	vd = dbmw_read(db_valuedata, &dbkey, NULL);
 
 	if (vd == NULL) {
-		/* XXX Must handle I/O errors correctly */
 		if (dbmw_has_ioerr(db_valuedata)) {
-			g_warning("DB I/O error, bad things will happen...");
-			return NULL;
+			g_warning("DBMW \"%s\" I/O error, bad things could happen...",
+				dbmw_name(db_valuedata));
+		} else {
+			g_warning("value for DB-key %s exists but not found in DBMW \"%s\"",
+				uint64_to_string(dbkey), dbmw_name(db_valuedata));
 		}
-		g_error("Value for DB-key %s supposed to exist but was not found in DB",
-			uint64_to_string(dbkey));
+		return NULL;
 	}
 
 	return vd;
@@ -596,11 +806,12 @@ delete_valuedata(guint64 dbkey, gboolean has_expired)
 {
 	const struct valuedata *vd;
 
-	vd = get_valuedata(dbkey);
-
-	g_assert(vd);					/* XXX handle I/O errors correctly */
 	g_assert(values_managed > 0);
 
+	vd = get_valuedata(dbkey);
+	if (NULL == vd)
+		return;			/* I/O error or corrupted data */
+
 	values_managed--;
 	acct_net_update(values_per_class_c, vd->addr, NET_CLASS_C_MASK, -1);
 	acct_net_update(values_per_ip, vd->addr, NET_IPv4_MASK, -1);
@@ -629,7 +840,7 @@ reclaim_dbkey(gpointer key, gpointer u_value, gpointer u_data)
 	delete_valuedata(*dbatom, TRUE);
 
 	if (GNET_PROPERTY(dht_storage_debug) > 2)
-		g_message("DHT DB-key %s reclaimed", uint64_to_string(*dbatom));
+		g_message("DHT value DB-key %s reclaimed", uint64_to_string(*dbatom));
 
 	atom_uint64_free(dbatom);
 	return TRUE;
@@ -645,18 +856,51 @@ values_reclaim_expired(void)
 }
 
 /**
- *  Callout queue callback for periodic value expiration.
+ *  Callout queue periodic event for value expiration.
  */
-static void
-values_periodic_expire(cqueue_t *unused_cq, gpointer unused_obj)
+static gboolean
+values_periodic_expire(gpointer unused_obj)
 {
-	(void) unused_cq;
 	(void) unused_obj;
 
-	install_periodic_expire();
 	values_reclaim_expired();
+	return TRUE;		/* Keep calling */
 }
 
+/**
+ * Log statistics about an expired value.
+ */
+static void
+log_expired_value_stats(guint64 dbkey, const struct valuedata *vd)
+{
+	if (NULL == vd)
+		return;
+
+	g_message("DHT STORE expiring \"%s\" %s "
+		"life=%s, republish#=%u, replication#=%u, request#=%u, dbkey=%s",
+		dht_value_type_to_string(vd->type),
+		kuid_to_hex_string(&vd->id),
+		compact_time(delta_time(tm_time(), vd->created)),
+		(unsigned) vd->n_republish, (unsigned) vd->n_replication,
+		(unsigned) vd->n_requests, uint64_to_string(dbkey));
+
+	if (GNET_PROPERTY(dht_storage_debug) > 1) {
+		guint32 avg_publish = 0;
+		guint32 avg_replicate = 0;
+
+		if (vd->n_republish)
+			avg_publish = vd->s_elapsed_publish / vd->n_republish;
+		if (vd->n_replication > 1) {
+			avg_replicate =
+				vd->s_elapsed_replicat / (vd->n_replication - 1);
+		}
+		g_message("DHT STORE averages for \"%s\" %s "
+			"between republish=%s, replication=%s",
+			dht_value_type_to_string(vd->type),
+			kuid_to_hex_string(&vd->id),
+			compact_time(avg_publish), short_time_ascii(avg_replicate));
+	}
+}
 /**
  * Remember that a value has expired, if we did not already know about it.
  *
@@ -664,15 +908,18 @@ values_periodic_expire(cqueue_t *unused_cq, gpointer unused_obj)
  * to avoid perturbation in the caller's data structures.
  */
 static void
-values_expire(guint64 dbkey)
+values_expire(guint64 dbkey, const struct valuedata *vd)
 {
 	const guint64 *dbatom;
 
 	if (g_hash_table_lookup(expired, &dbkey))
 		return;
 
+	if (GNET_PROPERTY(dht_storage_debug))
+		log_expired_value_stats(dbkey, vd);
+
 	if (GNET_PROPERTY(dht_storage_debug) > 2)
-		g_message("DHT DB-key %s expired", uint64_to_string(dbkey));
+		g_message("DHT value DB-key %s expired", uint64_to_string(dbkey));
 
 	dbatom = atom_uint64_get(&dbkey);
 	gm_hash_table_insert_const(expired, dbatom, GINT_TO_POINTER(1));
@@ -697,7 +944,8 @@ values_unexpire(guint64 dbkey)
 		atom_uint64_free(dbatom);
 
 		if (GNET_PROPERTY(dht_storage_debug) > 2)
-			g_message("DHT DB-key %s un-expired", uint64_to_string(dbkey));
+			g_message("DHT value DB-key %s un-expired",
+				uint64_to_string(dbkey));
 	}
 }
 
@@ -715,10 +963,11 @@ values_has_expired(guint64 dbkey, time_t now, time_t *expire)
 	struct valuedata *vd;
 
 	vd = get_valuedata(dbkey);
-	g_assert(vd);		/* XXX better handling for I/O errors */
+	if (NULL == vd)
+		return FALSE;		/* I/O error or corrupted database */
 
 	if (now >= vd->expire)  {
-		values_expire(dbkey);
+		values_expire(dbkey, vd);
 		return TRUE;
 	}
 
@@ -821,6 +1070,10 @@ validate_quotas(const dht_value_t *v)
 	int count;
 	const knode_t *c = v->creator;
 
+	/* Specifications say: creator address must be IPv4 */
+	if (NET_TYPE_IPV4 != host_addr_net(c->addr))
+		return STORE_SC_BAD_CREATOR;
+
 	count = acct_net_get(values_per_class_c, c->addr, NET_CLASS_C_MASK);
 
 	if (GNET_PROPERTY(dht_storage_debug) > 2) {
@@ -899,9 +1152,15 @@ validate_new_acceptable(const dht_value_t *v)
 /**
  * Compute value's expiration time based on the proximity we have with the key,
  * the status of our k-ball at the time of publication and the type of data.
+ *
+ * @param key		the primary key
+ * @param type		the value type
+ * @param cn		the creator node, NULL if not an original
+ * @param created	the creation date (first publish from creator)
  */
 static time_t
-values_expire_time(const kuid_t *key, dht_value_type_t type)
+values_expire_time(const kuid_t *key, dht_value_type_t type,
+	const knode_t *cn, time_t created)
 {
 	time_delta_t lifetime = dht_value_lifetime(type);
 	double decimation = keys_decimation_factor(key);
@@ -909,6 +1168,35 @@ values_expire_time(const kuid_t *key, dht_value_type_t type)
 	g_assert(decimation > 0.0);
 	g_assert(lifetime > 0);
 
+	/*
+	 * If value is an original and it's being republished, augment its
+	 * default lifetime proportionally to the expected probability of
+	 * presence in one republishing period, given the known node's lifetime,
+	 * as long as this probability is greater than 0.5.
+	 *
+	 * This strategy avoid too early expiration of values republished by
+	 * stable nodes, provided we are one of the k-closest nodes (decimation
+	 * is 1.0).  It favors stable data by "reserving" one of the limited
+	 * value slots for the key.
+	 */
+
+	if (cn != NULL && decimation <= 1.0) {	/* Signals an original, for us */
+		time_delta_t alive = delta_time(tm_time(), created);
+		double p = stable_alive_probability(alive, DHT_VALUE_REPUBLISH);
+
+		if (p > 0.5) {
+			lifetime += lifetime * 3.0 * (p - 0.5);
+
+			if (GNET_PROPERTY(dht_storage_debug)) {
+				g_message("DHT STORE boosted expire of \"%s\" %s to %s, "
+					"life=%s for creator %s", dht_value_type_to_string(type),
+					kuid_to_hex_string(key),
+					compact_time(lifetime), compact_time2(alive),
+					knode_to_string(cn));
+			}
+		}
+	}
+
 	return time_advance(tm_time(), (gulong) (lifetime / decimation));
 }
 
@@ -940,9 +1228,6 @@ fill_valuedata(struct valuedata *vd, const knode_t *cn, const dht_value_t *v)
 		acct_net_update(values_per_ip, cn->addr, NET_IPv4_MASK, +1);
 	}
 
-	vd->publish = tm_time();
-	vd->replicated = 0;
-	vd->expire = values_expire_time(v->id, v->type);
 	vd->vcode = cn->vcode;			/* struct copy */
 	vd->addr = cn->addr;			/* struct copy */
 	vd->port = cn->port;
@@ -952,6 +1237,9 @@ fill_valuedata(struct valuedata *vd, const knode_t *cn, const dht_value_t *v)
 	vd->value_major = v->major;
 	vd->value_minor = v->minor;
 	vd->length = v->length;
+
+	vd->expire = values_expire_time(v->id, v->type,
+		vd->original ? cn : NULL, vd->created);
 }
 
 /**
@@ -996,18 +1284,18 @@ values_remove(const knode_t *kn, const dht_value_t *v)
 	 */
 
 	if (GNET_PROPERTY(dht_storage_debug) > 1) {
-		struct valuedata *vd;
+		struct valuedata *vd = get_valuedata(dbkey);
+		if (vd) {
+			g_assert(kuid_eq(&vd->id, v->id));		/* Primary key */
+			g_assert(kuid_eq(&vd->cid, cn->id));	/* Secondary key */
 
-		vd = get_valuedata(dbkey);
-
-		g_assert(vd);							/* XXX handle DB failures */
-		g_assert(kuid_eq(&vd->id, v->id));		/* Primary key */
-		g_assert(kuid_eq(&vd->cid, cn->id));	/* Secondary key */
-
-		g_message("DHT STORE creator %s deleting %u-byte %s value %s",
-			kuid_to_hex_string(cn->id), vd->length,
-			dht_value_type_to_string(vd->type),
-			kuid_to_hex_string2(v->id));
+			g_message("DHT STORE creator %s deleting %u-byte %s value %s"
+				" (life %s)",
+				kuid_to_hex_string(cn->id), vd->length,
+				dht_value_type_to_string(vd->type),
+				kuid_to_hex_string2(v->id),
+				short_time_ascii(delta_time(tm_time(), vd->created)));
+		}
 	}
 
 	delete_valuedata(dbkey, FALSE);		/* Voluntarily deleted */
@@ -1028,6 +1316,77 @@ done:
 	return STORE_SC_OK;		/* Always succeeds */
 }
 
+/**
+ * Update statistics by counting a value republishing.
+ *
+ * @attention
+ * Must be called before updating vd->publish again.
+ */
+static void
+value_count_republish(struct valuedata *vd)
+{
+	time_t now = tm_time();
+
+	STATIC_ASSERT(DHT_VALUE_EXPIRE < MAX_INT_VAL(gint32));
+
+	/*
+	 * We only count a republishing if the value was an original, i.e. not
+	 * obtained through replication or caching.
+	 */
+
+	if (vd->original) {
+		guint32 elapsed = (guint32) delta_time(now, vd->publish);
+
+		vd->n_republish++;
+		vd->s_elapsed_publish =
+			guint32_saturate_add(vd->s_elapsed_publish, elapsed);
+
+		if (GNET_PROPERTY(dht_storage_debug)) {
+			g_message("DHT STORE republishing of \"%s\" %s #%u after %s, "
+				"life=%s", dht_value_type_to_string(vd->type),
+				kuid_to_hex_string(&vd->id), (unsigned) vd->n_republish,
+				compact_time(elapsed),
+				compact_time2(delta_time(tm_time(), vd->created)));
+		}
+	}
+
+	vd->publish = now;
+	gnet_stats_count_general(GNR_DHT_REPUBLISH, 1);
+}
+
+/**
+ * Update statistics by counting a value replication.
+ */
+static void
+value_count_replication(struct valuedata *vd)
+{
+	time_t now = tm_time();
+
+	STATIC_ASSERT(DHT_VALUE_EXPIRE < MAX_INT_VAL(gint32));
+
+	/*
+	 * Only update the sum of the elapsed time between replications if
+	 * we already replicated the value.
+	 */
+
+	vd->n_replication++;
+
+	if (0 != vd->replicated) {
+		guint32 elapsed = (guint32) delta_time(now, vd->replicated);
+		vd->s_elapsed_replicat =
+			guint32_saturate_add(vd->s_elapsed_replicat, elapsed);
+
+		if (GNET_PROPERTY(dht_storage_debug))
+			g_message("DHT STORE replication of \"%s\" %s #%u after %s",
+				dht_value_type_to_string(vd->type),
+				kuid_to_hex_string(&vd->id), (unsigned) vd->n_replication,
+				compact_time(elapsed));
+	}
+
+	vd->replicated = now;
+	gnet_stats_count_general(GNR_DHT_REPLICATION, 1);
+}
+
 /**
  * Publish or replicate value in our local data store.
  *
@@ -1043,8 +1402,8 @@ values_publish(const knode_t *kn, const dht_value_t *v)
 	gboolean check_data = FALSE;
 
 	/*
-	 * Look whether we already hold this value (in which case it would
-	 * be a replication or a republishing from original creator).
+	 * Look whether we already hold this value (in which case it could
+	 * be either a replication or a republishing from the original creator).
 	 */
 
 	dbkey = keys_has(v->id, v->creator->id, TRUE);
@@ -1073,8 +1432,6 @@ values_publish(const knode_t *kn, const dht_value_t *v)
 			}
 			vd->original = TRUE;
 		} else {
-			if (NET_TYPE_IPV4 != host_addr_net(cn->addr))
-				return STORE_SC_BAD_CREATOR;
 			if (kuid_pair_was_expired(kn->id, cn->id))
 				goto expired;
 			vd->original = FALSE;
@@ -1083,6 +1440,8 @@ values_publish(const knode_t *kn, const dht_value_t *v)
 		dbkey = valueid++;
 		vd->id = *v->id;				/* struct copy */
 		vd->cid = *cn->id;				/* struct copy */
+		vd->created = tm_time();		/* First time we see this value */
+		vd->publish = vd->created;
 		fill_valuedata(vd, cn, v);
 
 		keys_add_value(v->id, cn->id, dbkey, vd->expire);
@@ -1095,12 +1454,14 @@ values_publish(const knode_t *kn, const dht_value_t *v)
 
 		vd = get_valuedata(dbkey);
 
+		if (NULL == vd)
+			return STORE_SC_DB_IO;		/* I/O error or corrupted DB */
+
 		/*
 		 * If one the following assertions fails, then it means our data
 		 * management is wrong and we messed up severely somewhere.
 		 */
 
-		g_assert(vd);						/* XXX handle DB failures */
 		g_assert(kuid_eq(&vd->id, v->id));				/* Primary key */
 		g_assert(kuid_eq(&vd->cid, v->creator->id));	/* Secondary key */
 
@@ -1153,13 +1514,18 @@ values_publish(const knode_t *kn, const dht_value_t *v)
 					"with %s", vd->length, dht_value_type_to_string(vd->type),
 					dht_value_to_string(v));
 
+			/*
+			 * Update statistics before changing the old valuedata structure.
+			 */
+
+			value_count_republish(vd);
+
 			vd->original = TRUE;
 			fill_valuedata(vd, cn, v);
 
 			values_unexpire(dbkey);
 			kuid_pair_was_republished(&vd->id, &vd->cid);
 			keys_update_value(&vd->id, vd->expire);
-			gnet_stats_count_general(GNR_DHT_REPUBLISH, 1);
 		}
 	}
 
@@ -1194,8 +1560,7 @@ values_publish(const knode_t *kn, const dht_value_t *v)
 		 * and we're done.
 		 */
 
-		vd->replicated = tm_time();
-		gnet_stats_count_general(GNR_DHT_REPLICATION, 1);
+		value_count_replication(vd);
 	} else {
 		/*
 		 * We got either new data or something republished by the creator.
@@ -1320,8 +1685,10 @@ values_store(const knode_t *kn, const dht_value_t *v, gboolean token)
 
 done:
 	if (GNET_PROPERTY(dht_storage_debug) > 1)
-		g_message("DHT STORE status for %s is %u (%s)",
-			kuid_to_hex_string(v->id), status, store_error_to_string(status));
+		g_message("DHT STORE status for \"%s\" %s is %u (%s)",
+			dht_value_type_to_string(v->type),
+			kuid_to_hex_string(v->id), status,
+			dht_store_error_to_string(status));
 
 	return status;
 }
@@ -1361,7 +1728,7 @@ values_get(guint64 dbkey, dht_value_type_t type)
 	 */
 
 	if (tm_time() >= vd->expire) {
-		values_expire(dbkey);
+		values_expire(dbkey, vd);
 		return NULL;
 	}
 
@@ -1369,6 +1736,8 @@ values_get(guint64 dbkey, dht_value_type_t type)
 	 * OK, we have a value and its type matches.  Build the DHT value.
 	 */
 
+	vd->n_requests++;
+
 	if (vd->length) {
 		size_t length;
 		gpointer data;
@@ -1391,13 +1760,6 @@ values_get(guint64 dbkey, dht_value_type_t type)
 	return v;
 }
 
-static void
-install_periodic_expire(void)
-{
-	expire_ev = cq_insert(callout_queue, EXPIRE_PERIOD * 1000,
-		values_periodic_expire, NULL);
-}
-
 /**
  * Initialize values management.
  */
@@ -1405,25 +1767,26 @@ void
 values_init(void)
 {
 	db_valuedata = storage_create(db_valwhat, db_valbase,
-		sizeof(guint64), sizeof(struct valuedata),
-		serialize_valuedata, deserialize_valuedata,
-		1, uint64_hash, uint64_eq);
+		sizeof(guint64), sizeof(struct valuedata), 0,
+		serialize_valuedata, deserialize_valuedata, NULL,
+		VALUES_DB_CACHE_SIZE, uint64_mem_hash, uint64_mem_eq);
 
 	db_rawdata = storage_create(db_rawwhat, db_rawbase,
-		sizeof(guint64), DHT_VALUE_MAX_LEN,
-		NULL, NULL,
-		1, uint64_hash, uint64_eq);
+		sizeof(guint64), DHT_VALUE_MAX_LEN, 0,
+		NULL, NULL, NULL,
+		RAW_DB_CACHE_SIZE, uint64_mem_hash, uint64_mem_eq);
 
 	db_expired = storage_create(db_expwhat, db_expbase,
-		2 * KUID_RAW_SIZE, 0,
-		NULL, NULL,
-		1, kuid_pair_hash, kuid_pair_eq);
+		2 * KUID_RAW_SIZE, 0, 0,
+		NULL, NULL, NULL,
+		0, kuid_pair_hash, kuid_pair_eq);
 
 	values_per_ip = acct_net_create();
 	values_per_class_c = acct_net_create();
 	expired = g_hash_table_new(uint64_hash, uint64_eq);
 
-	install_periodic_expire();
+	values_expire_ev = cq_periodic_add(callout_queue,
+		EXPIRE_PERIOD * 1000, values_periodic_expire, NULL);
 }
 
 static void
@@ -1443,16 +1806,18 @@ expired_free_kv(gpointer key, gpointer u_val, gpointer u_data)
 void
 values_close(void)
 {
-	storage_delete(db_valuedata, db_valbase);
-	storage_delete(db_rawdata, db_rawbase);
-	storage_delete(db_expired, db_expbase);
+	storage_delete(db_valuedata);
+	storage_delete(db_rawdata);
+	storage_delete(db_expired);
 	db_valuedata = db_rawdata = db_expired = NULL;
 	acct_net_free(&values_per_ip);
 	acct_net_free(&values_per_class_c);
+	cq_periodic_remove(callout_queue, &values_expire_ev);
+	values_managed = 0;
 
 	g_hash_table_foreach(expired, expired_free_kv, NULL);
 	g_hash_table_destroy(expired);
-	cq_cancel(callout_queue, &expire_ev);
+	expired = NULL;
 }
 
 /* vi: set ts=4 sw=4 cindent: */
