@@ -1,5 +1,5 @@
 /*
- * $Id: c2ui.c 15323 2008-05-02 20:04:24Z rmanfredi $
+ * $Id: c2ui.c 16909 2009-06-05 23:49:38Z cbiere $
  *
  * Copyright (c) 2004, Emile Roberts
  *
@@ -129,7 +129,7 @@ gcu_gtk_main_flush(void)
 
 /**	search interface functions (CORE -> UI) */
 gboolean
-gcu_search_gui_new_search(const gchar *query, flag_t flags)
+gcu_search_gui_new_search(const gchar *query, guint32 flags)
 {
 	if (!running_topless) {
 		return search_gui_new_search(query, flags, NULL);
