@@ -1,5 +1,5 @@
 /*
- * $Id: ui2c.c 16238 2009-03-23 13:31:54Z rmanfredi $
+ * $Id: ui2c.c 17077 2009-10-16 20:51:38Z rmanfredi $
  *
  * Copyright (c) 2004, Emile Roberts
  *
@@ -62,6 +62,7 @@
 #include "if/core/version.h"
 #include "if/core/main.h"
 #include "if/bridge/ui2c.h"
+#include "if/dht/dht.h"
 
 /*
  *	Functions the UI uses to access the CORE
@@ -747,7 +748,7 @@ guc_search_is_frozen(gnet_search_t sh)
 
 enum search_new_result
 guc_search_new(gnet_search_t *ptr, const char *query,
-	time_t create_time, guint lifetime, guint32 reissue_timeout, flag_t flags)
+	time_t create_time, guint lifetime, guint32 reissue_timeout, guint32 flags)
 {
 	return search_new(ptr, query,
 				create_time, lifetime, reissue_timeout, flags);
@@ -963,6 +964,12 @@ guc_upload_stats_clear_all(void)
 	upload_stats_clear_all();
 }
 
+gboolean
+guc_dht_enabled(void)
+{
+	return dht_enabled();
+}
+
 /**	version interface functions (UI -> Core)*/
 const char *
 guc_version_get_version_string(void)
