@@ -1,5 +1,5 @@
 /*
- * $Id: bsched.h 16110 2009-03-10 15:27:43Z rmanfredi $
+ * $Id: bsched.h 17077 2009-10-16 20:51:38Z rmanfredi $
  *
  * Copyright (c) 2001-2003, Raphael Manfredi
  *
@@ -44,6 +44,8 @@ typedef enum {
 	BSCHED_BWS_GOUT_UDP,
 	BSCHED_BWS_LOOPBACK_IN,
 	BSCHED_BWS_LOOPBACK_OUT,
+	BSCHED_BWS_DHT_IN,
+	BSCHED_BWS_DHT_OUT,
 
 	NUM_BSCHED_BWS,
 	BSCHED_BWS_INVALID = NUM_BSCHED_BWS
@@ -53,7 +55,7 @@ typedef enum {
  * Source under bandwidth control.
  */
 
-enum bio_source_magic { BIO_SOURCE_MAGIC = 0x80b3bf07U };
+enum bio_source_magic { BIO_SOURCE_MAGIC = 0x7fb3bf07U };
 
 typedef struct bio_source {
 	enum bio_source_magic magic;	/**< magic for consistency checks */
