@@ -1,5 +1,5 @@
 /*
- * $Id: downloads.h 16287 2009-03-28 21:56:51Z rmanfredi $
+ * $Id: downloads.h 17319 2010-02-28 09:32:47Z rmanfredi $
  *
  * Copyright (c) 2001-2003, Raphael Manfredi, Richard Eckart
  *
@@ -28,10 +28,12 @@
 
 #include "lib/tm.h"				/* For tm_t */
 #include "lib/event.h"			/* For frequency_t */
+#include "lib/iso3166.h"		/* For iso3166_code_is_valid() */
 #include "lib/hashlist.h"
 #include "lib/list.h"
 #include "lib/slist.h"
 
+#include "core/pproxy.h"
 #include "core/rx.h"
 #include "core/rx_link.h"
 #include "core/rx_chunk.h"
@@ -91,11 +93,10 @@ struct dl_server {
 	list_t *list[DL_LIST_SZ];	/**< Download lists */
 	const char *vendor;			/**< Remote server vendor string (atom) */
 	const char *hostname;		/**< Remote hostname, if known (atom) */
-	hash_list_t *proxies;		/**< Known push proxies (gnet_host_t) */
+	pproxy_set_t *proxies;		/**< Known push proxies */
 	GHashTable *sha1_counts;
 	time_t retry_after;		/**< Time at which we may retry from this host */
 	time_t dns_lookup;		/**< Last DNS lookup for hostname */
-	time_t proxies_stamp;	/**< Time when list was last updated */
 	time_t last_connect;	/**< When we last connected to that server */
 	struct vernum parq_version; /**< Supported queueing version */
 	guint speed_avg;			/**< Average (EMA) upload speed, in bytes/sec */
@@ -106,7 +107,8 @@ struct dl_server {
 static inline gboolean
 dl_server_valid(const struct dl_server *s)
 {
-	return s != NULL && s->magic == DL_SERVER_MAGIC;
+	return s != NULL && s->magic == DL_SERVER_MAGIC &&
+			iso3166_code_is_valid(s->country);
 }
 
 /**
@@ -169,7 +171,6 @@ enum download_magic { DOWNLOAD_MAGIC = 0x2dd6efe9 };	/**< Magic number */
 struct download {
 	enum download_magic magic;	/**< Magic number */
     gnet_src_t src_handle;      /**< Handle */
-	gboolean src_handle_valid;	/**< TRUE if src_handle is initialized */
 
 	char error_str[256];		/**< Used to snprintf() error strings */
 	download_status_t status;   /**< Current status of the download */
@@ -226,11 +227,11 @@ struct download {
 	guint32 flags;
 	guint32 cflags;
 
-	gboolean keep_alive;		/**< Keep HTTP connection? */
-	gboolean push;				/**< Currently in push mode */
-	gboolean always_push;		/**< Always use the push method for this */
-	gboolean got_giv;			/**< Whether initiated from GIV reception */
-	gboolean unavailable;		/**< Set on Timout, Push route lost */
+	unsigned src_handle_valid:1;/**< TRUE if src_handle is initialized */
+	unsigned keep_alive:1;		/**< Keep HTTP connection? */
+	unsigned always_push:1;		/**< Always send PUSH to connect */
+	unsigned got_giv:1;			/**< Whether initiated from GIV reception */
+	unsigned unavailable:1;		/**< Set on Timout, Push route lost */
 
 	struct cproxy *cproxy;		/**< Push proxy being used currently */
 	struct parq_dl_queued *parq_dl;	/**< Queuing status */
@@ -243,6 +244,8 @@ struct download {
  */
 
 enum {
+	DL_F_CLONED			= 1 << 29,	/**< Parent of a cloned download */
+	DL_F_PARTIAL		= 1 << 28,	/**< Partial source */
 	DL_F_FROM_ERROR		= 1 << 27,	/**< Switched after error occurred */
 	DL_F_FROM_PLAIN		= 1 << 26,	/**< Switched from plain file download */
 	DL_F_SWITCHED		= 1 << 25,	/**< Scheduled after resource switching */
@@ -277,6 +280,9 @@ enum {
  * Server attributes.
  */
 enum {
+	DLS_A_FW_SOURCE		= 1 << 13,	/**< Files from server can go to the mesh */
+	DLS_A_FWALT			= 1 << 12,	/**< Server accepts firewalled locations */
+	DLS_A_TLS			= 1 << 11,	/**< Server known to support TLS */
 	DLS_A_DHT_PROX		= 1 << 10,	/**< Looking for push-proxies in DHT */
 	DLS_A_G2_ONLY		= 1 << 9,	/**< Server is "G2" only */
 	DLS_A_FOOBAR		= 1 << 8,	/**< Server is foobar */
@@ -314,6 +320,7 @@ enum {
  * on the remote size, in case the file is partial.
  */
 #define download_ranges(d)		((d)->ranges)
+#define download_ranges_size(d)	((d)->ranges_size)
 
 /*
  * State inspection macros.
@@ -377,7 +384,8 @@ enum {
 	|| (d)->status == GTA_DL_FALLBACK	\
 	|| (d)->status == GTA_DL_PUSH_SENT	)
 
-#define DOWNLOAD_IS_IN_PUSH_MODE(d) (d->push)
+#define DOWNLOAD_IS_IN_PUSH_MODE(d) \
+	(d->always_push && !(d->flags & DL_F_PUSH_IGN))
 
 gboolean download_has_blank_guid(const struct download *);
 
@@ -435,6 +443,7 @@ void download_auto_new(const char *filename,
 
 void download_dht_auto_new(const char *filename,
  	filesize_t size,
+	const char *hostname,
 	const host_addr_t addr,
 	guint16 port,
 	const struct guid *guid,
