@@ -1,5 +1,5 @@
 /*
- * $Id: fileinfo.h 16238 2009-03-23 13:31:54Z rmanfredi $
+ * $Id: fileinfo.h 17282 2010-02-23 21:44:08Z rmanfredi $
  *
  * Copyright (c) 2001-2003, Raphael Manfredi, Richard Eckart
  *
@@ -27,7 +27,7 @@
 #define _if_core_fileinfo_h_
 
 #include "common.h"
-
+#include "lib/path.h"
 #include "if/core/downloads.h"	/* For gnet_srt_t */
 
 struct shared_file;
@@ -38,26 +38,27 @@ struct download;
  */
 
 enum {
-	FI_F_SUSPEND		= 1 << 0,	/**< Marked "suspended" new downloads */
-	FI_F_DISCARD		= 1 << 1,	/**< Discard fileinfo if refcount = 0 */
-	FI_F_TRANSIENT		= 1 << 2,	/**< Don't persist to disk */
-	FI_F_MARK			= 1 << 3,	/**< Marked during traversal */
-	FI_F_PAUSED			= 1 << 4,	/**< Paused by user */
-	FI_F_SEEDING		= 1 << 5,	/**< Seeding after successful download */
-	FI_F_STRIPPED		= 1 << 6,	/**< Fileinfo trailler has been stripped */
-	FI_F_FETCH_TTH		= 1 << 7,	/**< Tigertree data is being downloaded */
-	FI_F_UNLINKED		= 1 << 8,	/**< Removed from disk */
-	FI_F_BAD_BITPRINT	= 1 << 9,	/**< SHA1 + TTH combination is bad */
+	FI_F_MOVING			= 1 << 11,	/**< Moving file (or about to) */
 	FI_F_VERIFYING		= 1 << 10,	/**< Verifying SHA1 or TTH */
+	FI_F_BAD_BITPRINT	= 1 << 9,	/**< SHA1 + TTH combination is bad */
+	FI_F_UNLINKED		= 1 << 8,	/**< Removed from disk */
+	FI_F_FETCH_TTH		= 1 << 7,	/**< Tigertree data is being downloaded */
+	FI_F_STRIPPED		= 1 << 6,	/**< Fileinfo trailler has been stripped */
+	FI_F_SEEDING		= 1 << 5,	/**< Seeding after successful download */
+	FI_F_PAUSED			= 1 << 4,	/**< Paused by user */
+	FI_F_MARK			= 1 << 3,	/**< Marked during traversal */
+	FI_F_TRANSIENT		= 1 << 2,	/**< Don't persist to disk */
+	FI_F_DISCARD		= 1 << 1,	/**< Discard fileinfo if refcount = 0 */
+	FI_F_SUSPEND		= 1 << 0	/**< Marked "suspended" new downloads */
 };
 
 /**
  * These used to be in fileinfo.h, but we need them now at several places.
  */
 enum dl_chunk_status {
-    DL_CHUNK_EMPTY = 0,			/**< No data available yet */
-    DL_CHUNK_BUSY  = 1,			/**< Being downloaded */
     DL_CHUNK_DONE  = 2,			/**< Downloaded */
+    DL_CHUNK_BUSY  = 1,			/**< Being downloaded */
+    DL_CHUNK_EMPTY = 0			/**< No data available yet */
 };
 
 typedef guint32 gnet_fi_t;
@@ -80,7 +81,7 @@ typedef struct gnet_fi_status {
 	filesize_t  done;
 	
 	filesize_t  uploaded;
-	filesize_t  sha1_hashed;
+	filesize_t  vrfy_hashed;
 	filesize_t	copied;
 
 	time_t		modified;
@@ -101,9 +102,11 @@ typedef struct gnet_fi_status {
 
 	unsigned	complete:1;
 	unsigned	verifying:1;
+	unsigned	moving:1;
 	unsigned 	sha1_matched:1;
 	unsigned	finished:1;
 	unsigned	seeding:1;
+	unsigned	tth_check:1;
 } gnet_fi_status_t;
 
 typedef struct gnet_fi_chunks {
@@ -114,7 +117,7 @@ typedef struct gnet_fi_chunks {
 } gnet_fi_chunks_t;
 
 enum fi_magic {
-	FI_MAGIC = 0xd99892e7
+	FI_MAGIC = 0x599892e7
 };
 
 struct guid;
@@ -152,11 +155,6 @@ typedef struct dl_file_info {
 	GSList *seen_on_network;  /**< List of ranges available on network */
 	guint32 generation;		/**< Generation number, incremented on disk update */
 	struct shared_file *sf;	/**< When PFSP-server is enabled, share this file */
-	gboolean file_size_known; /**< File size known? */
-	gboolean use_swarming;	/**< Use swarming? */
-	gboolean dirty;			/**< Does it need saving? */
-	gboolean dirty_status;  /**< Notify about status change on next interval */
-	gboolean hashed;		/**< In hash tables? */
 	guint32  active_queued; /**< Actively queued sources */
 	guint32  passive_queued;/**< Passively queued sources */
 
@@ -173,10 +171,21 @@ typedef struct dl_file_info {
 	 * This group of fields is used by the background SHA1 and moving daemons.
 	 */
 
-	filesize_t cha1_hashed;	/**< Amount of bytes hashed so far */
+	filesize_t vrfy_hashed;	/**< Amount of bytes hashed so far during verify */
 	filesize_t copied;		/**< Amount of bytes copied so far */
-	unsigned cha1_elapsed;	/**< Time spent to compute the SHA1 */
+	unsigned vrfy_elapsed;	/**< Time spent to compute the hash */
 	unsigned copy_elapsed;	/**< Time spent to copy the file */
+
+	/*
+	 * Booleans (bit fields used since gboolean uses too much space).
+	 */
+
+	unsigned file_size_known:1;	/**< File size known? */
+	unsigned use_swarming:1;	/**< Use swarming? */
+	unsigned dirty:1;			/**< Does it need saving? */
+	unsigned dirty_status:1;  	/**< Notify status change on next interval */
+	unsigned hashed:1;			/**< In hash tables? */
+	unsigned tth_check:1;		/**< TTH checking performed? */
 } fileinfo_t;
 
 static inline void
@@ -218,13 +227,6 @@ FILE_INFO_FINISHED(const fileinfo_t *fi)
 		&& FILE_INFO_COMPLETE(fi);
 }
 
-static inline gboolean
-FILE_INFO_COMPLETE_AFTER(const fileinfo_t *fi, filesize_t off)
-{
-	file_info_check(fi);
-	return fi->file_size_known && off >= fi->size - fi->done;
-}
-
 typedef void (*fi_listener_t) (gnet_fi_t);
 typedef void (*src_listener_t) (gnet_src_t);
 
