@@ -1,5 +1,5 @@
 /*
- * $Id: gnutella.h 15814 2008-09-07 03:09:56Z cbiere $
+ * $Id: gnutella.h 16424 2009-04-05 18:45:53Z rmanfredi $
  *
  * Copyright (c) 2001-2003, Raphael Manfredi
  *
@@ -71,6 +71,12 @@ struct gnutella_header_ {
 
 typedef guint8 gnutella_header_t[GTA_HEADER_SIZE];
 
+static inline gconstpointer
+gnutella_data(gconstpointer header)
+{
+	return (char *) header + GTA_HEADER_SIZE;
+}
+
 static inline struct guid *
 gnutella_header_muid(gnutella_header_t *header)
 {
