@@ -1,5 +1,5 @@
 /*
- * $Id: guid.h 12761 2007-01-16 16:07:30Z cbiere $
+ * $Id: guid.h 15814 2008-09-07 03:09:56Z cbiere $
  *
  * Copyright (c) 2004, Raphael Manfredi
  *
@@ -31,9 +31,26 @@
 #include "lib/misc.h"		/* For GUID_RAW_SIZE */
 
 typedef struct guid {
-	guchar v[GUID_RAW_SIZE];
+	char v[GUID_RAW_SIZE];
 } guid_t;
 
-extern const gchar blank_guid[GUID_RAW_SIZE];
+/**
+ * No alignment requirements but ptr must point to GUID_RAW_SIZE or more bytes.
+ */
+static inline guid_t *
+cast_to_guid_ptr(char *ptr)
+{
+	return (guid_t *) ptr;
+}
+
+static inline const guid_t *
+cast_to_guid_ptr_const(const char *ptr)
+{
+	return (const guid_t *) ptr;
+}
+
+extern const struct guid blank_guid;
+
+gboolean guid_is_blank(const struct guid *);
 
 #endif /* _if_core_guid_h_ */
