@@ -1,5 +1,5 @@
 /*
- * $Id: hcache.h 12761 2007-01-16 16:07:30Z cbiere $
+ * $Id: hcache.h 16969 2009-09-14 19:33:25Z rmanfredi $
  *
  * Copyright (c) 2001-2003, Raphael Manfredi, Richard Eckart
  *
@@ -45,7 +45,8 @@ typedef enum {
                                gave us a timeout during connection. */
     HCACHE_BUSY,          /**< We put in this list all the Gnet nodes which
                                gave us a 503 (busy) during connection. */
-    HCACHE_UNSTABLE,
+    HCACHE_UNSTABLE,      /**< Unstable IPs */
+    HCACHE_ALIEN,         /**< Alien networks (protected by auth challenges) */
 	HCACHE_NONE,
     HCACHE_MAX
 } hcache_type_t;
