@@ -1,5 +1,5 @@
 /*
- * $Id: net_stats.h 16177 2009-03-16 17:44:43Z rmanfredi $
+ * $Id: net_stats.h 17395 2010-03-18 21:58:48Z rmanfredi $
  *
  * Copyright (c) 2001-2003, Raphael Manfredi, Richard Eckart
  *
@@ -62,6 +62,7 @@ typedef enum msg_drop_reason {
 	MSG_DROP_IMPROPER_HOPS_TTL,
 	MSG_DROP_MAX_TTL_EXCEEDED,
 	MSG_DROP_THROTTLE,
+	MSG_DROP_LIMIT,
 	MSG_DROP_PONG_UNUSABLE,
 	MSG_DROP_HARD_TTL_LIMIT,
 	MSG_DROP_MAX_HOP_COUNT,
@@ -149,7 +150,10 @@ typedef enum {
 	GNR_DISCOVERED_SERVER_GUID,
 	GNR_CHANGED_SERVER_GUID,
 	GNR_GUID_COLLISIONS,
+	GNR_OWN_GUID_COLLISIONS,
 	GNR_RECEIVED_KNOWN_FW_NODE_INFO,
+	GNR_REVITALIZED_PUSH_ROUTES,
+	GNR_COLLECTED_PUSH_PROXIES,
 	GNR_ATTEMPTED_RESOURCE_SWITCHING,
 	GNR_ATTEMPTED_RESOURCE_SWITCHING_AFTER_ERROR,
 	GNR_SUCCESSFUL_RESOURCE_SWITCHING,
@@ -173,7 +177,10 @@ typedef enum {
 	GNR_PARQ_QUEUE_SENDING_ATTEMPTS,
 	GNR_PARQ_QUEUE_SENT,
 	GNR_PARQ_QUEUE_FOLLOW_UPS,
-	GNR_SEEDING_OF_ORPHAN,
+	GRN_SHA1_VERIFICATIONS,
+	GRN_TTH_VERIFICATIONS,
+	GNR_QHIT_SEEDING_OF_ORPHAN,
+	GNR_UPLOAD_SEEDING_OF_ORPHAN,
 	GNR_DHT_ESTIMATED_SIZE,
 	GNR_DHT_KBALL_FURTHEST,
 	GNR_DHT_KBALL_CLOSEST,
@@ -186,7 +193,9 @@ typedef enum {
 	GNR_DHT_ROUTING_EVICTED_NODES,
 	GNR_DHT_ROUTING_EVICTED_FIREWALLED_NODES,
 	GNR_DHT_ROUTING_PROMOTED_PENDING_NODES,
+	GNR_DHT_ROUTING_PINGED_PROMOTED_NODES,
 	GNR_DHT_COMPLETED_BUCKET_REFRESH,
+	GNR_DHT_FORCED_BUCKET_REFRESH,
 	GNR_DHT_DENIED_UNSPLITABLE_BUCKET_REFRESH,
 	GNR_DHT_BUCKET_ALIVE_CHECK,
 	GNR_DHT_ALIVE_PINGS_TO_GOOD_NODES,
@@ -196,6 +205,16 @@ typedef enum {
 	GNR_DHT_KEYS_HELD,
 	GNR_DHT_CACHED_KEYS_HELD,
 	GNR_DHT_VALUES_HELD,
+	GNR_DHT_CACHED_KUID_TARGETS_HELD,
+	GNR_DHT_CACHED_ROOTS_HELD,
+	GNR_DHT_CACHED_ROOTS_EXACT_HITS,
+	GNR_DHT_CACHED_ROOTS_APPROXIMATE_HITS,
+	GNR_DHT_CACHED_ROOTS_MISSES,
+	GNR_DHT_CACHED_ROOTS_KBALL_LOOKUPS,
+	GNR_DHT_CACHED_ROOTS_CONTACT_REFRESHED,
+	GNR_DHT_CACHED_TOKENS_HELD,
+	GNR_DHT_CACHED_TOKENS_HITS,
+	GNR_DHT_STABLE_NODES_HELD,
 	GNR_DHT_FETCH_LOCAL_HITS,
 	GNR_DHT_FETCH_LOCAL_CACHED_HITS,
 	GNR_DHT_RETURNED_EXPANDED_VALUES,
@@ -213,10 +232,31 @@ typedef enum {
 	GNR_DHT_DUP_VALUES,
 	GNR_DHT_KUID_COLLISIONS,
 	GNR_DHT_OWN_KUID_COLLISIONS,
+	GNR_DHT_RPC_KUID_REPLY_MISMATCH,
+	GNR_DHT_CACHING_ATTEMPTS,
+	GNR_DHT_CACHING_SUCCESSFUL,
+	GNR_DHT_CACHING_PARTIALLY_SUCCESSFUL,
+	GNR_DHT_KEY_OFFLOADING_CHECKS,
+	GNR_DHT_KEYS_SELECTED_FOR_OFFLOADING,
+	GNR_DHT_KEY_OFFLOADING_ATTEMPTS,
+	GNR_DHT_KEY_OFFLOADING_SUCCESSFUL,
+	GNR_DHT_KEY_OFFLOADING_PARTIALLY_SUCCESSFUL,
+	GNR_DHT_VALUES_OFFLOADED,
+	GNR_DHT_PUBLISHING_ATTEMPTS,
+	GNR_DHT_PUBLISHING_SUCCESSFUL,
+	GNR_DHT_PUBLISHING_PARTIALLY_SUCCESSFUL,
+	GNR_DHT_PUBLISHING_SATISFACTORY,
+	GNR_DHT_REPUBLISHED_LATE,
+	GNR_DHT_PUBLISHING_TO_SELF,
+	GNR_DHT_PUBLISHING_BG_ATTEMPTS,
+	GNR_DHT_PUBLISHING_BG_IMPROVEMENTS,
+	GNR_DHT_PUBLISHING_BG_SUCCESSFUL,
+	GNR_DHT_SHA1_DATA_TYPE_COLLISIONS,
 	GNR_DHT_ALT_LOC_LOOKUPS,
 	GNR_DHT_PUSH_PROXY_LOOKUPS,
 	GNR_DHT_SUCCESSFUL_ALT_LOC_LOOKUPS,
 	GNR_DHT_SUCCESSFUL_PUSH_PROXY_LOOKUPS,
+	GNR_DHT_SUCCESSFUL_NODE_PUSH_ENTRY_LOOKUPS,
 	GNR_DHT_SEEDING_OF_ORPHAN,
 	
 	GNR_TYPE_COUNT /* number of general stats */
@@ -253,7 +293,9 @@ typedef enum {
 	BW_LEAF_IN,
 	BW_LEAF_OUT,
 	BW_GNET_UDP_IN,
-	BW_GNET_UDP_OUT
+	BW_GNET_UDP_OUT,
+	BW_DHT_IN,
+	BW_DHT_OUT
 } gnet_bw_source;
 
 typedef struct gnet_bw_stats {
@@ -279,4 +321,5 @@ const char *gnet_stats_general_to_string(gnr_stats_t type);
 #endif /* CORE_SOURCES */
 
 #endif /* _if_core_net_stats_h_ */
+
 /* vi: set ts=4 sw=4 cindent: */
