@@ -1,5 +1,5 @@
 /*
- * $Id: nodes.h 16223 2009-03-20 21:48:03Z rmanfredi $
+ * $Id: nodes.h 17077 2009-10-16 20:51:38Z rmanfredi $
  *
  * Copyright (c) 2001-2003, Raphael Manfredi, Richard Eckart
  *
@@ -52,7 +52,7 @@ guint node_id_hash(gconstpointer node_id);
 enum rnode_mode {
 	RNODE_M_AUTO = 0,
 	RNODE_M_LEAF = 1,
-	RNODE_M_ULTRA = 2,
+	RNODE_M_ULTRA = 2
 };
 
 /**
@@ -311,6 +311,7 @@ typedef enum {
 	NODE_P_NORMAL,				/**< Normal legacy node */
 	NODE_P_CRAWLER,				/**< Crawler node */
 	NODE_P_UDP,					/**< UDP "fake" node */
+	NODE_P_DHT,					/**< DHT "fake" node (UDP-only traffic) */
 	NODE_P_UNKNOWN				/**< Unknown mode yet */
 } node_peer_t;
 
