@@ -1,5 +1,5 @@
 /*
- * $Id: pproxy.h,v 1.4 2005/08/17 13:44:05 cbiere Exp $
+ * $Id: pproxy.h 16111 2009-03-10 15:32:51Z rmanfredi $
  *
  * Copyright (c) 2003, Raphael Manfredi
  *
@@ -43,6 +43,7 @@
 #include "lib/host_addr.h"	/* For host_addr_t */
 
 struct download;
+struct guid;
 
 /**
  * A client push proxy request.
@@ -53,8 +54,8 @@ struct cproxy {
 
 	host_addr_t addr;		/**< IP of the proxy servent */
 	guint16 port;			/**< Port of the proxy servent */
-	gchar *server;			/**< Server string */
-	gchar *guid;			/**< GUID (atom) to which push should be sent */
+	const char *server;	/**< Server string */
+	const struct guid *guid;/**< GUID (atom) to which push should be sent */
 	guint32 file_idx;		/**< File index to request */
 	gpointer http_handle;	/**< Asynchronous HTTP request handle */
 	guint32 flags;			/**< Operating flags */
