@@ -1,5 +1,5 @@
 /*
- * $Id: settings.h 13005 2007-03-07 23:30:55Z cbiere $
+ * $Id: settings.h 16111 2009-03-10 15:32:51Z rmanfredi $
  *
  * Copyright (c) 2004, Raphael Manfredi
  *
@@ -37,9 +37,9 @@
 host_addr_t listen_addr(void);
 host_addr_t listen_addr6(void);
 host_addr_t listen_addr_by_net(enum net_type net);
-const gchar *settings_config_dir(void);
-const gchar *settings_home_dir(void);
-const gchar *settings_local_socket_path(void);
+const char *settings_config_dir(void);
+const char *settings_home_dir(void);
+const char *settings_local_socket_path(void);
 enum net_type settings_dns_net(void);
 
 #endif /* CORE_SOURCES */
