@@ -1,5 +1,5 @@
 /*
- * $Id: sockets.h 15991 2009-02-27 09:14:08Z rmanfredi $
+ * $Id: sockets.h 16995 2009-09-26 22:50:43Z rmanfredi $
  *
  * Copyright (c) 2003, Raphael Manfredi
  *
@@ -26,7 +26,8 @@
 #ifndef _if_core_sockets_h_
 #define _if_core_sockets_h_
 
-#define SOCK_BUFSZ	4096
+#define SOCK_BUFSZ	4096		/**< Buffer size for connected sockets */
+#define SOCK_LBUFSZ	32768		/**< Buffer size for datagram sockets */
 
 /***
  *** Proxy protocols
@@ -72,9 +73,10 @@ enum {
 	SOCK_F_CORKED		= (1UL << 6),  /**< Set if TCP_CORK is enabled */
 	SOCK_F_SHUTDOWN		= (1UL << 7),  /**< Set if shutdown() was called */
 	SOCK_F_OMIT_TOKEN	= (1UL << 8),  /**< If set X-Token header is omitted */
-	SOCK_F_LOCAL		= (1UL << 29), /**< Is a local socket */
-	SOCK_F_UDP			= (1UL << 30), /**< Is a UDP socket */
-	SOCK_F_TCP			= (1UL << 31)  /**< Is a TCP socket */
+	SOCK_F_PREPARED		= (1UL << 9),  /**< Prepared (known address) */
+	SOCK_F_LOCAL		= (1UL << 28), /**< Is a local socket */
+	SOCK_F_UDP			= (1UL << 29), /**< Is a UDP socket */
+	SOCK_F_TCP			= (1UL << 30)  /**< Is a TCP socket */
 };
 
 #endif /* _if_core_sockets_h_ */
