@@ -1,5 +1,5 @@
 /*
- * $Id: uploads.h 16101 2009-03-10 14:44:05Z rmanfredi $
+ * $Id: uploads.h 17202 2009-11-07 13:18:20Z rmanfredi $
  *
  * Copyright (c) 2001-2003, Raphael Manfredi, Richard Eckart
  *
@@ -47,7 +47,7 @@ typedef enum {
 	GTA_UL_QUEUE            = 9,    /**< Send a queue (Similar to push) */
 	GTA_UL_QUEUE_WAITING    = 10,   /**< Connect back with GTA_UL_QUEUE was
 									     success now waiting for a response */
-	GTA_UL_EXPECTING        = 11,   /**< Expecting follow-up HTTP request */
+	GTA_UL_EXPECTING        = 11    /**< Expecting follow-up HTTP request */
 } upload_stage_t;
 
 typedef struct gnet_upload_status {
@@ -125,12 +125,12 @@ struct ul_stats {
 	filesize_t size;
 	guint32 attempts;
 	guint32 complete;
-	time_t rtime;		/**< time of last request */
-	time_t dtime;		/**< time of last downloaded bytes */
+	time_t rtime;			/**< time of last request */
+	time_t dtime;			/**< time of last downloaded bytes */
 	guint64 bytes_sent;
-	double norm;		/**< bytes sent / file size */
+	double norm;			/**< bytes sent / file size */
 	const struct sha1 *sha1;	/**< SHA1 of file (atom), if known, or NULL */
-	void *user_data;	/**< Used by the GUI side to store context */
+	void *user_data;		/**< Used by the GUI side to store context */
 };
 
 /*
@@ -163,7 +163,6 @@ void upload_remove_upload_removed_listener(upload_removed_listener_t);
 void upload_add_upload_info_changed_listener(upload_info_changed_listener_t);
 void upload_remove_upload_info_changed_listener(upload_info_changed_listener_t);
 void upload_kill(gnet_upload_t);
-void upload_stats_prune_nonexistent(void);
 
 #endif /* CORE_SOURCES */
 #endif /* _if_core_uploads_h_ */
