@@ -1,5 +1,5 @@
 /*
- * $Id: gnet_stats.h,v 1.1 2004/09/03 20:34:15 rmanfredi Exp $
+ * $Id: gnet_stats.h 8035 2005-04-24 18:51:00Z cbiere $
  *
  * Copyright (c) 2004, Raphael Manfredi
  *
@@ -29,7 +29,7 @@
 typedef enum gnet_stats_source {
 	GNET_STATS_FULL = 0,
 	GNET_STATS_TCP_ONLY = 1,
-	GNET_STATS_UDP_ONLY = 2,
+	GNET_STATS_UDP_ONLY = 2
 } gnet_stats_source_t;
 
 #endif /* _if_ui_gtk_gnet_stats_h_ */
