@@ -1,5 +1,5 @@
 /*
- * $Id: search.h 14457 2007-08-14 04:23:12Z cbiere $
+ * $Id: search.h 16909 2009-06-05 23:49:38Z cbiere $
  *
  * Copyright (c) 2004, Raphael Manfredi
  *
@@ -36,7 +36,7 @@
 
 struct search;
 
-gboolean search_gui_new_search(const gchar *, flag_t flags, 
+gboolean search_gui_new_search(const gchar *, guint32 flags, 
 			struct search **search);
 
 #endif /* GUI_SOURCES */
