@@ -1,5 +1,5 @@
 /*
- * $Id: adns.c 16108 2009-03-10 15:19:52Z rmanfredi $
+ * $Id: adns.c 17007 2009-09-29 18:09:22Z rmanfredi $
  *
  * Copyright (c) 2004, Christian Biere
  *
@@ -35,11 +35,12 @@
 
 #include "common.h"
 
-RCSID("$Id: adns.c 16108 2009-03-10 15:19:52Z rmanfredi $")
+RCSID("$Id: adns.c 17007 2009-09-29 18:09:22Z rmanfredi $")
 
 #include "adns.h"
 #include "atoms.h"
 #include "debug.h"
+#include "fd.h"
 #include "inputevt.h"
 #include "ascii.h"
 #include "glib-missing.h"
@@ -431,7 +432,6 @@ adns_gethostbyname(const struct adns_request *req, struct adns_response *ans)
 		struct adns_reverse_reply *reply = &ans->reply.reverse;
 		const char *host;
 
-
 		if (common_dbg > 1) {
 			g_message("adns_gethostbyname: Resolving \"%s\" ...",
 					host_addr_to_string(query->addr));
@@ -439,7 +439,7 @@ adns_gethostbyname(const struct adns_request *req, struct adns_response *ans)
 
 		reply->addr = query->addr;
 		host = host_addr_to_name(query->addr);
-		g_strlcpy(reply->hostname, host ? host : "", sizeof reply->hostname);
+		clamp_strcpy(reply->hostname, sizeof reply->hostname, host ? host : "");
 	} else {
 		const struct adns_query *query = &req->query.by_addr;
 		struct adns_reply *reply = &ans->reply.by_addr;
@@ -450,7 +450,7 @@ adns_gethostbyname(const struct adns_request *req, struct adns_response *ans)
 			g_message("adns_gethostbyname: Resolving \"%s\" ...",
 				query->hostname);
 		}
-		g_strlcpy(reply->hostname, query->hostname, sizeof reply->hostname);
+		clamp_strcpy(reply->hostname, sizeof reply->hostname, query->hostname);
 
 		sl_addr = name_to_host_addr(query->hostname, query->net);
 		for (sl = sl_addr; NULL != sl; sl = g_slist_next(sl)) {
@@ -848,8 +848,8 @@ adns_init(void)
 
 	set_close_on_exec(adns_query_fd);
 	set_close_on_exec(fd_reply[0]);
-	socket_set_nonblocking(adns_query_fd);
-	socket_set_nonblocking(fd_reply[0]);
+	fd_set_nonblocking(adns_query_fd);
+	fd_set_nonblocking(fd_reply[0]);
 	
 	adns_reply_event_id = inputevt_add(fd_reply[0], INPUT_EVENT_RX,
 							adns_reply_callback, NULL);
@@ -976,8 +976,8 @@ adns_resolve(const char *hostname, enum net_type net,
 	reply->hostname[0] = '\0';
 	reply->addrs[0] = zero_host_addr;
 
-	hostname_len = g_strlcpy(query->hostname, hostname, sizeof query->hostname);
-	if (hostname_len >= sizeof query->hostname) {
+	hostname_len = clamp_strcpy(query->hostname, sizeof query->hostname, hostname);
+	if ('\0' != hostname[hostname_len]) {
 		/* truncation detected */
 		adns_invoke_user_callback(&ans);
 		return FALSE; /* synchronous */
@@ -991,7 +991,7 @@ adns_resolve(const char *hostname, enum net_type net,
 	}
 
 	ascii_strlower(query->hostname, hostname);
-	g_strlcpy(reply->hostname, query->hostname, sizeof reply->hostname);
+	clamp_strcpy(reply->hostname, sizeof reply->hostname, query->hostname);
 	
 	if (
 		adns_cache_lookup(adns_cache, tm_time(), query->hostname,
