@@ -1,5 +1,5 @@
 /*
- * $Id: atoms.c 16242 2009-03-24 19:04:20Z cbiere $
+ * $Id: atoms.c 17383 2010-03-18 08:44:53Z rmanfredi $
  *
  * Copyright (c) 2002-2003, Raphael Manfredi
  *
@@ -41,8 +41,9 @@
 #include "common.h"
 
 #include "atoms.h"
+#include "endian.h"
 
-RCSID("$Id: atoms.c 16242 2009-03-24 19:04:20Z cbiere $")
+RCSID("$Id: atoms.c 17383 2010-03-18 08:44:53Z rmanfredi $")
 
 #if 0
 #define PROTECT_ATOMS
@@ -78,6 +79,7 @@ typedef enum {
 #endif	/* ATOMS_HAVE_MAGIC */
 
 
+#include "stringify.h"
 #include "misc.h"
 #include "glib-missing.h"
 #include "walloc.h"
@@ -85,13 +87,8 @@ typedef enum {
 #include "override.h"		/* Must be the last header included */
 
 union mem_chunk {
-  void      *next;
-  guint8   u8;
-  guint16  u16;
-  guint32  u32;
-  guint64  u64;
-  float     f;
-  double    d;
+  void *next;
+  char align[MEM_ALIGNBYTES];
 };
 
 #define ATOM_TYPE_MASK	((size_t) 0x07)
@@ -100,28 +97,34 @@ union mem_chunk {
  * Atoms are ref-counted.
  *
  * The reference count is held at the beginning of the data arena.
- * What we return to the outside is a pointer to arena[], not to
- * the atom structure.
+ * What we return to the outside is the value of atom_arena(), not a
+ * pointer to the atom structure.
  */
 typedef struct atom {
+#ifdef ATOMS_HAVE_MAGIC
+	atom_prot_magic_t magic;	/**< Magic should be at the beginning */
+#endif /* ATOM_HAVE_MAGIC */
+	int refcnt;				/**< Amount of references */
 #ifdef TRACK_ATOMS
 	GHashTable *get;		/**< Allocation spots */
 	GHashTable *free;		/**< Free spots */
 #endif
-#ifdef ATOMS_HAVE_MAGIC
-	atom_prot_magic_t magic;
-#endif /* PROTECT_ATOMS */
-	int refcnt;			/**< Amount of references */
-	/* Ensure `arena' is properly aligned */
-	union mem_chunk arena[1];			/* Start of user arena */
 } atom_t;
 
-#define ARENA_OFFSET	G_STRUCT_OFFSET(struct atom, arena)
+#define ARENA_OFFSET \
+	(MEM_ALIGNBYTES * ((sizeof(atom_t) / MEM_ALIGNBYTES) + \
+		((sizeof(atom_t) % MEM_ALIGNBYTES) ? 1 : 0)))
 
 static inline atom_t *
 atom_from_arena(gconstpointer key)
 {
-	return (gpointer) ((char *) key - ARENA_OFFSET);
+	return (atom_t *) (((char *) key) - ARENA_OFFSET);
+}
+
+static inline void *
+atom_arena(const atom_t *a)
+{
+	return ((char *) a) + ARENA_OFFSET;
 }
 
 static inline void
@@ -144,7 +147,7 @@ struct mem_cache {
 	size_t hold;
 };
 
-static struct mem_cache *mc[9];
+static struct mem_cache *mem_cache[9];
 
 static struct mem_pool *
 mem_pool_new(size_t size, size_t hold)
@@ -159,7 +162,7 @@ mem_pool_new(size_t size, size_t hold)
   ps = compat_pagesize();
   len = hold * size;
   
-  mp = alloc_pages(len);
+  mp = vmm_alloc(len);
   if (mp) {
     size_t i, step = size / sizeof mp->chunks[0];
     
@@ -259,9 +262,9 @@ atom_get_mc(size_t size)
 {
 	guint i;
 
-	for (i = 0; i < G_N_ELEMENTS(mc); i++) {
-		if (size <= mc[i]->size)
-			return mc[i];
+	for (i = 0; i < G_N_ELEMENTS(mem_cache); i++) {
+		if (size <= mem_cache[i]->size)
+			return mem_cache[i];
 	}
 	return NULL;
 }
@@ -343,7 +346,7 @@ atom_alloc(size_t size)
 		a = mem_alloc(mc_ptr);
 		mem_unprotect(a, size);
 	} else {
-		a = alloc_pages(size);
+		a = vmm_alloc(size);
 	}
 	ATOM_SET_MAGIC(a);
 
@@ -377,7 +380,7 @@ atom_dealloc(atom_t *a, size_t size)
 		 * relies on executable heap-memory because the page is now
 		 * mapped (PROT_READ | PROT_WRITE).
 		 */
-		FREE_PAGES_NULL(a, size);
+		VMM_FREE_NULL(a, size);
 	}
 }
 #else
@@ -465,15 +468,14 @@ str_str(gconstpointer v)
 }
 
 /**
- * Hash `len' bytes starting from `key'.
+ * Hash `len' bytes starting from `data'.
  */
 guint
-binary_hash(const guchar *key, guint len)
+binary_hash(gconstpointer data, size_t len)
 {
-	guint hash = len;
-	guint i;
-	guint remain;
-	guint t4;
+	const unsigned char *key = data;
+	size_t i, remain, t4;
+	guint32 hash;
 
 	remain = len & 0x3;
 	t4 = len & ~0x3U;
@@ -481,15 +483,13 @@ binary_hash(const guchar *key, guint len)
 	g_assert(remain + t4 == len);
 	g_assert(remain <= 3);
 
+	hash = len;
 	for (i = 0; i < t4; i += 4) {
 		static const guint32 x[] = {
 			0xb0994420, 0x01fa96e3, 0x05066d0e, 0x50c3c22a,
 			0xec99f01f, 0xc0eaa79d, 0x157d4257, 0xde2b8419
 		};
-		hash ^= key[i] |
-			((guint) key[i + 1] << 8) |
-			((guint) key[i + 2] << 16) |
-			((guint) key[i + 3] << 24);
+		hash ^= peek_le32(&key[i]);
 		hash += x[(i >> 2) & 0x7];
 		hash = (hash << 24) ^ (hash >> 8);
 	}
@@ -705,6 +705,30 @@ uint64_hash(gconstpointer p)
 	return v ^ (v >> 32);
 }
 
+/**
+ * Test two 64-bit integers for equality, with pointers not necessarily
+ * aligned on 64-bit quantities.
+ *
+ * @return whether both referenced 64-bit integers are equal.
+ */
+int
+uint64_mem_eq(gconstpointer a, gconstpointer b)
+{
+	return a == b || 0 == memcmp(a, b, sizeof(guint64));
+}
+
+/**
+ * Calculate the 32-bit hash of a 64-bit integer whose address is not
+ * necessarily aligned on 64-bit quantities.
+ *
+ * @return the 32-bit hash value for the referenced 64-bit integer.
+ */
+guint
+uint64_mem_hash(gconstpointer p)
+{
+	return binary_hash(p, sizeof(guint64));
+}
+
 /**
  * @return printable form of a filesize_t, as pointer to static data.
  */
@@ -781,14 +805,22 @@ uint32_str(gconstpointer v)
 void
 atoms_init(void)
 {
+	gboolean has_setting = FALSE;
+	struct atom_settings {
+		guint8 track_atoms;
+		guint8 protect_atoms;
+		guint8 atoms_have_magic;
+	} settings;
 	guint i;
 
+	memset(&settings, 0, sizeof settings);
+
 	STATIC_ASSERT(NUM_ATOM_TYPES <= (ATOM_TYPE_MASK + 1));
 	STATIC_ASSERT(NUM_ATOM_TYPES == G_N_ELEMENTS(atoms));
 
 #ifdef PROTECT_ATOMS
-	for (i = 0; i < G_N_ELEMENTS(mc); i++) {
-		mc[i] = mem_new((2 * sizeof (union mem_chunk)) << i);
+	for (i = 0; i < G_N_ELEMENTS(mem_cache); i++) {
+		mem_cache[i] = mem_new((2 * sizeof (union mem_chunk)) << i);
 	}
 #endif /* PROTECT_ATOMS */
 
@@ -798,6 +830,30 @@ atoms_init(void)
 		td->table = g_hash_table_new(td->hash_func, td->eq_func);
 	}
 	ht_all_atoms = g_hash_table_new(pointer_hash_func, NULL);
+
+	/*
+	 * Log atoms configuration.
+	 */
+
+#ifdef TRACK_ATOMS
+	settings.track_atoms = TRUE;
+	has_setting = TRUE;
+#endif
+#ifdef PROTECT_ATOMS
+	settings.protect_atoms = TRUE;
+	has_setting = TRUE;
+#endif
+#ifdef ATOMS_HAVE_MAGIC
+	settings.atoms_have_magic = TRUE;
+	has_setting = TRUE;
+#endif
+
+	if (has_setting) {
+		g_message("atom settings: %s%s%s",
+			settings.track_atoms ? "TRACK_ATOMS " : "",
+			settings.protect_atoms ? "PROTECT_ATOMS " : "",
+			settings.atoms_have_magic ? "ATOMS_HAVE_MAGIC " : "");
+	}
 }
 
 static inline size_t
@@ -835,15 +891,11 @@ atom_get(enum atom_type type, gconstpointer key)
 	gpointer orig_key;
 	size_t size;
 
-#ifdef __GNUC__
 	STATIC_ASSERT(0 == ARENA_OFFSET % MEM_ALIGNBYTES);
-#else
-	/* ARENA_OFFSET is no constant expression with some Sun compiler */
-	g_assert(0 == ARENA_OFFSET % MEM_ALIGNBYTES);
-#endif /* __GNUC__ */
+	STATIC_ASSERT(ARENA_OFFSET >= sizeof(atom_t));
 	
     g_assert(key != NULL);
-	g_assert((int) type >= 0 && (guint) type < G_N_ELEMENTS(atoms));
+	g_assert(UNSIGNED(type) < G_N_ELEMENTS(atoms));
 
 	td = &atoms[type];		/* Where atoms of this type are held */
 
@@ -892,11 +944,11 @@ atom_get(enum atom_type type, gconstpointer key)
 
 		len = (*td->len_func)(key);
 		g_assert(len < ((size_t) -1) - ARENA_OFFSET);
-		size = round_size((ATOM_TYPE_MASK + 1), ARENA_OFFSET + len);
+		size = round_size_fast((ATOM_TYPE_MASK + 1), ARENA_OFFSET + len);
 
 		a = atom_alloc(size);
 		a->refcnt = 1;
-		memcpy(a->arena, key, len);
+		memcpy(atom_arena(a), key, len);
 		atom_protect(a, size);
 
 		/*
@@ -904,10 +956,10 @@ atom_get(enum atom_type type, gconstpointer key)
 		 */
 
 		value = (gpointer) (size | (guint) type);
-		g_hash_table_insert(ht_all_atoms, a->arena, value);
-		g_hash_table_insert(td->table, a->arena, (gpointer) size);
+		g_hash_table_insert(ht_all_atoms, atom_arena(a), value);
+		g_hash_table_insert(td->table, atom_arena(a), (gpointer) size);
 
-		return a->arena;
+		return atom_arena(a);
 	}
 }
 
@@ -922,7 +974,7 @@ atom_free(enum atom_type type, gconstpointer key)
 	atom_t *a;
 
     g_assert(key != NULL);
-	g_assert((int) type >= 0 && (guint) type < G_N_ELEMENTS(atoms));
+	g_assert(UNSIGNED(type) < G_N_ELEMENTS(atoms));
 
 	size = atom_is_registered(type, key);
 	g_assert(size > 0);
