@@ -1,5 +1,5 @@
 /*
- * $Id: bg.c 16129 2009-03-12 06:47:48Z rmanfredi $
+ * $Id: bg.c 17033 2009-10-05 20:51:00Z rmanfredi $
  *
  * Copyright (c) 2002-2003, Raphael Manfredi
  *
@@ -39,7 +39,7 @@
 
 #include "common.h"
 
-RCSID("$Id: bg.c 16129 2009-03-12 06:47:48Z rmanfredi $")
+RCSID("$Id: bg.c 17033 2009-10-05 20:51:00Z rmanfredi $")
 
 #include "bg.h"
 #include "debug.h"
@@ -50,7 +50,7 @@ RCSID("$Id: bg.c 16129 2009-03-12 06:47:48Z rmanfredi $")
 #include "override.h"		/* Must be the last header included */
 
 enum bgtask_magic {
-	BGTASK_MAGIC	  = 0xbacc931dU,
+	BGTASK_MAGIC	  = 0x3acc931dU,
 	BGTASK_DEAD_MAGIC = 0x6f5c8a03U
 };
 
@@ -810,9 +810,8 @@ bg_task_ticks_used(struct bgtask *bt, int used)
 	bg_task_check(bt);
 	g_assert(bt->flags & TASK_F_RUNNING);
 	g_assert(used >= 0);
-	g_assert(used <= bt->ticks);
 
-	bt->ticks_used = used;
+	bt->ticks_used = MIN(used, bt->ticks);
 
 	if (used == 0)
 		bt->flags |= TASK_F_NOTICK;			/* Won't update tick info */
@@ -873,7 +872,7 @@ bg_task_ended(struct bgtask *bt)
 	 * The following makes sure we pickup a new item at the next iteration.
 	 */
 
-	bt->tick_cost = 0;					/* Will restart at 1 tick next time */
+	bt->tick_cost = 0.0;			/* Will restart at 1 tick next time */
 	bt->seqno = 0;
 	bt->step = 0;
 
@@ -939,8 +938,7 @@ bg_sched_timer(gboolean overloaded)
 		 * than DELTA_FACTOR.
 		 */
 
-		if (bt->tick_cost) {
-			g_assert(bt->tick_cost > 0);
+		if (bt->tick_cost > 0.0) {
 			g_assert(bt->prev_ticks >= 0);
 			g_assert(bt->prev_ticks <= INT_MAX / DELTA_FACTOR);
 
@@ -1051,7 +1049,7 @@ bg_sched_timer(gboolean overloaded)
 			else {
 				bt->seqno = 0;
 				bt->step++;
-				bt->tick_cost = 0;	/* Don't know cost of this new step */
+				bt->tick_cost = 0.0;	/* Don't know cost of this new step */
 			}
 			break;
 		case BGR_MORE:
