@@ -2,6 +2,8 @@
  * Copyright (c) 2006 Christian Biere <christianbiere@gmx.de>
  * All rights reserved.
  *
+ * Copyright (c) 2009 Raphael Manfredi <Raphael_Manfredi@pobox.com>
+ *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
  * are met:
@@ -36,18 +38,25 @@
  *
  * @author Christian Biere
  * @date 2006
+ * @author Raphael Manfredi
+ * @date 2009
  */
 
 #include "common.h"
 
-RCSID("$Id: crash.c 14954 2007-09-28 19:23:13Z bpringlemeir $")
+RCSID("$Id: crash.c 17347 2010-03-03 14:45:48Z rmanfredi $")
 
-#include "lib/misc.h"
-#include "lib/vmm.h"
-#include "lib/crash.h"
+#include "fd.h"
+#include "misc.h"
+#include "offtime.h"
+#include "timestamp.h"
+#include "tm.h"
+#include "crash.h"
 
 #include "lib/override.h"		/* Must be the last header included */
 
+static time_delta_t crash_gmtoff;	/**< Offset to GMT, supposed to be fixed */
+
 static struct {
 	const char *pathname;	/* The file to execute. */
 	const char *argv0;		/* The original argv[0]. */
@@ -72,34 +81,111 @@ static const struct {
 #undef D
 };
 
-#define print_str(x) \
-G_STMT_START { \
-	if (iov_cnt < G_N_ELEMENTS(iov)) { \
-		const char *ptr = (x); \
-		if (ptr) { \
-			iov[iov_cnt].iov_base = (char *) ptr; \
-			iov[iov_cnt].iov_len = strlen(ptr); \
-			iov_cnt++; \
-		} \
-	} \
-} G_STMT_END
+/**
+ * Append positive value to buffer, formatted as "%02u".
+ *
+ * @return amount of characters written.
+ */
+static size_t
+crash_append_fmt_02u(char *buf, size_t buflen, long v)
+{
+	if (buflen < 2 || v < 0)
+		return 0;
+
+	if (v >= 100)
+		v %= 100;
+
+	if (v < 10) {
+		buf[0] = '0';
+		buf[1] = dec_digit(v);
+	} else {
+		int d = v % 10;
+		int c = v /= 10;
+		buf[0] = dec_digit(c);
+		buf[1] = dec_digit(d);
+	}
+
+	return 2;
+}
+
+/**
+ * Append a character to supplied buffer.
+ *
+ * @return amount of characters written.
+ */
+static size_t
+crash_append_fmt_c(char *buf, size_t buflen, unsigned char c)
+{
+	if (buflen < 1)
+		return 0;
+
+	buf[0] = c;
+	return 1;
+}
+
+/**
+ * Fill supplied buffer with the current time formatted as yy-mm-dd HH:MM:SS
+ * and should be at least 18 chars long or the string will be truncated.
+ *
+ * This routine can safely be used in a signal handler as it does not rely
+ * on unsafe calls.
+ */
+void
+crash_time(char *buf, size_t buflen)
+{
+	struct tm tm;
+	size_t rw = 0;
+
+	if (0 == buflen)
+		return;
+
+	if (!offtime(tm_time() + crash_gmtoff, 0, &tm)) {
+		buf[0] = '\0';
+		return;
+	}
+
+	rw += crash_append_fmt_02u(&buf[rw], buflen - rw,
+		(TM_YEAR_ORIGIN + tm.tm_year) % 100);
+	rw += crash_append_fmt_c(&buf[rw], buflen - rw, '-');
+	rw += crash_append_fmt_02u(&buf[rw], buflen - rw, tm.tm_mon + 1);
+	rw += crash_append_fmt_c(&buf[rw], buflen - rw, '-');
+	rw += crash_append_fmt_02u(&buf[rw], buflen - rw, tm.tm_mday);
+	rw += crash_append_fmt_c(&buf[rw], buflen - rw, ' ');
+	rw += crash_append_fmt_02u(&buf[rw], buflen - rw, tm.tm_hour);
+	rw += crash_append_fmt_c(&buf[rw], buflen - rw, ':');
+	rw += crash_append_fmt_02u(&buf[rw], buflen - rw, tm.tm_min);
+	rw += crash_append_fmt_c(&buf[rw], buflen - rw, ':');
+	rw += crash_append_fmt_02u(&buf[rw], buflen - rw, tm.tm_sec);
+
+	rw++;	/* Trailing NUL */
+	buf[MIN(rw, buflen) - 1] = '\0';
+}
 
 static void
 crash_message(const char *reason)
 {
-	struct iovec iov[5];
+	struct iovec iov[8];
 	unsigned iov_cnt = 0;
 	char pid_buf[22];
-	
-	print_str("CRASH (pid=");
-	print_str(print_number(pid_buf, sizeof pid_buf, getpid()));
-	print_str(") by ");
-	print_str(reason);
-	print_str("\n");
+	char time_buf[18];
+
+	crash_time(time_buf, sizeof time_buf);
+
+	print_str(time_buf);			/* 0 */
+	print_str(" CRASH (pid=");		/* 1 */
+	print_str(print_number(pid_buf, sizeof pid_buf, getpid()));	/* 2 */
+	print_str(") by ");				/* 3 */
+	print_str(reason);				/* 4 */
+	if (vars.pathname) {
+		print_str(" -- calling ");	/* 5 */
+		print_str(vars.pathname);	/* 6 */
+	} else if (vars.pause_process) {
+		print_str(" -- pausing");	/* 5 */
+	}
+	print_str("\n");				/* 7, at most */
 	writev(STDERR_FILENO, iov, iov_cnt);
 }
 
-
 static void
 crash_exec(const char *pathname, const char *argv0)
 {
@@ -194,6 +280,8 @@ crash_init(const char *pathname, const char *argv0, int pause_process)
 	for (i = 0; i < G_N_ELEMENTS(signals); i++) {
 		set_signal(signals[i].signo, crash_handler);
 	}
+
+	crash_gmtoff = timestamp_gmt_offset(tm_time_exact(), NULL);
 }
 
 /* vi: set ts=4 sw=4 cindent: */
