@@ -1,5 +1,5 @@
 /*
- * $Id: crc.c,v 1.8 2005/09/10 10:10:36 daichik Exp $
+ * $Id: crc.c 16108 2009-03-10 15:19:52Z rmanfredi $
  *
  * Copyright (c) 2003, Raphael Manfredi
  *
@@ -35,7 +35,7 @@
 
 #include "common.h"
 
-RCSID("$Id: crc.c,v 1.8 2005/09/10 10:10:36 daichik Exp $");
+RCSID("$Id: crc.c 16108 2009-03-10 15:19:52Z rmanfredi $")
 
 #include "crc.h"
 #include "override.h"		/* Must be the last header included */
@@ -74,7 +74,7 @@ crc32_gen_crc_table(void)
 	guint32 i, crc_accum;
 
 	for (i = 0; i < 256; i++) {
-		gint j;
+		int j;
 
 		crc_accum = i << 24;
 		for (j = 0; j < 8; j++) {
