@@ -1,7 +1,7 @@
 /*
- * $Id: dbmw.c 16013 2009-02-28 22:01:16Z rmanfredi $
+ * $Id: dbmw.c 17067 2009-10-12 19:38:55Z rmanfredi $
  *
- * Copyright (c) 2008, Raphael Manfredi
+ * Copyright (c) 2008-2009, Raphael Manfredi
  *
  *----------------------------------------------------------------------
  * This file is part of gtk-gnutella.
@@ -31,12 +31,12 @@
  * of data structures and cache management.
  *
  * @author Raphael Manfredi
- * @date 2008
+ * @date 2008-2009
  */
 
 #include "common.h"
 
-RCSID("$Id: dbmw.c 16013 2009-02-28 22:01:16Z rmanfredi $")
+RCSID("$Id: dbmw.c 17067 2009-10-12 19:38:55Z rmanfredi $")
 
 #include "debug.h"
 #include "dbmw.h"
@@ -45,16 +45,20 @@ RCSID("$Id: dbmw.c 16013 2009-02-28 22:01:16Z rmanfredi $")
 #include "pmsg.h"
 #include "bstr.h"
 #include "hashlist.h"
+#include "stringify.h"
 #include "zalloc.h"
 #include "walloc.h"
 #include "override.h"			/* Must be the last header included */
 
 #define DBMW_CACHE	128			/**< Default amount of items to cache */
 
+enum dbmw_magic { DBMW_MAGIC = 0x28e7e7d2U };
+
 /**
  * Our DBM wrapper.
  */
 struct dbmw {
+	enum dbmw_magic magic;
 	dbmap_t *dm;				/**< Underlying database manager */
 	const char *name;			/**< DB name, for logging */
 	pmsg_t *mb;					/**< Message block used for serialization */
@@ -65,16 +69,26 @@ struct dbmw {
 	guint64 w_access;			/**< Number of write accesses */
 	guint64 r_hits;				/**< Number of read cache hits */
 	guint64 w_hits;				/**< Number of write cache hits */
-	size_t key_size;			/**< Size of keys */
-	size_t value_size;			/**< Maximum size of values */
+	size_t key_size;			/**< Size of keys (constant) */
+	size_t value_size;			/**< Maximum size of values (structure) */
+	size_t value_data_size;		/**< Maximum size of values (serialized form) */
 	size_t max_cached;			/**< Max amount of items to cache */
 	dbmw_serialize_t pack;		/**< Serialization routine for values */
 	dbmw_deserialize_t unpack;	/**< Deserialization routine for values */
+	dbmw_free_t valfree;		/**< Free routine for deserialized values */
 	int error;					/**< Last errno value */
-	gboolean ioerr;				/**< Had I/O error */
-	gboolean count_needs_sync;	/**< Whether we need to sync to get count */
+	guint8 ioerr;				/**< Had I/O error */
+	guint8 count_needs_sync;	/**< Whether we need to sync to get count */
+	guint8 is_volatile;			/**< Whether database dies when map dies */
 };
 
+static inline void
+dbmw_check(const dbmw_t *dw)
+{
+	g_assert(dw != NULL);
+	g_assert(DBMW_MAGIC == dw->magic);
+}
+
 /**
  * A cached entry (deserialized value). 
  *
@@ -117,14 +131,28 @@ dbmw_map_type(const dbmw_t *dw)
 	return dbmap_type(dw->dm);
 }
 
+/**
+ * @return name of DBMW.
+ */
+const char *
+dbmw_name(const dbmw_t *dw)
+{
+	return dw->name;
+}
+
 /**
  * @return amount of items held.
  */
 size_t
 dbmw_count(dbmw_t *dw)
 {
+	/*
+	 * Must write pending new items first and delete cached items to allow
+	 * proper count in the underlying map.
+	 */
+
 	if (dw->count_needs_sync)
-		dbmw_sync(dw);				/* Must write pending new items first */
+		dbmw_sync(dw, DBMW_SYNC_CACHE);
 
 	return dbmap_count(dw->dm);
 }
@@ -132,41 +160,57 @@ dbmw_count(dbmw_t *dw)
 /**
  * Create a new DBM wrapper over already created DB map.
  *
- * @param dm			The database (already opened)
- * @param name			Database name, for logs
- * @param key_size		Constant key size, in bytes
- * @param value_size	Maximum value size, in bytes
- * @param pack			Serialization routine for values
- * @param unpack		Deserialization routine for values
- * @param cache_size	Amount of items to cache (0 = no cache, 1 = default)
- * @param hash_func		Key hash function
- * @param eq_func		Key equality test function
+ * If value_dsize is 0, the length for value_size is used.
+ *
+ * @param dm				The database (already opened)
+ * @param name				Database name, for logs
+ * @param key_size			Constant key size, in bytes
+ * @param value_size		Maximum value size, in bytes (structure)
+ * @param value_data_size	Maximum value size, in bytes (serialized form)
+ * @param pack				Serialization routine for values
+ * @param unpack			Deserialization routine for values
+ * @param valfree			Free routine for value (or NULL if none needed)
+ * @param cache_size		Amount of items to cache (0 = no cache, 1 = default)
+ * @param hash_func			Key hash function
+ * @param eq_func			Key equality test function
  *
  * If serialization and deserialization routines are NULL pointers, data
  * will be stored and retrieved as-is.  In that case, they must be both
  * NULL.
  */
 dbmw_t *
-dbmw_create(dbmap_t *dm, const char *name, size_t key_size, size_t value_size,
-	dbmw_serialize_t pack, dbmw_deserialize_t unpack,
+dbmw_create(dbmap_t *dm, const char *name, size_t key_size,
+	size_t value_size, size_t value_data_size,
+	dbmw_serialize_t pack, dbmw_deserialize_t unpack, dbmw_free_t valfree,
 	size_t cache_size, GHashFunc hash_func, GEqualFunc eq_func)
 {
 	dbmw_t *dw;
 
 	g_assert(key_size);
 	g_assert(pack == NULL || value_size);
-	g_assert(sdbm_is_storable(key_size, value_size));	/* SDBM constraint */
 	g_assert((pack != NULL) == (unpack != NULL));
+	g_assert(valfree == NULL || unpack != NULL);
 	g_assert(dm);
 	g_assert(dbmap_key_size(dm) == key_size);
 
 	dw = walloc0(sizeof *dw);
 
+	dw->magic = DBMW_MAGIC;
 	dw->dm = dm;
 	dw->name = name;
 
 	dw->key_size = key_size;
 	dw->value_size = value_size;
+	dw->value_data_size = 0 == value_data_size ? value_size : value_data_size;
+
+	/* Make sure we do not violate the SDBM constraint */
+	g_assert(sdbm_is_storable(key_size, dw->value_data_size));
+
+	/*
+	 * There must be a serialization routine if the serialized length is not
+	 * the same as the structure length.
+	 */
+	g_assert(dw->value_size == dw->value_data_size || pack != NULL);
 
 	/*
 	 * For a small amount of items, a PATRICIA tree is more efficient
@@ -181,6 +225,7 @@ dbmw_create(dbmap_t *dm, const char *name, size_t key_size, size_t value_size,
 	dw->keys = hash_list_new(hash_func, eq_func);
 	dw->pack = pack;
 	dw->unpack = unpack;
+	dw->valfree = valfree;
 
 	/*
 	 * If a serialization routine is provided, we'll also have a need for
@@ -189,7 +234,7 @@ dbmw_create(dbmap_t *dm, const char *name, size_t key_size, size_t value_size,
 
 	if (dw->pack) {
 		dw->bs = bstr_create();
-		dw->mb = pmsg_new(PMSG_P_DATA, NULL, value_size);
+		dw->mb = pmsg_new(PMSG_P_DATA, NULL, dw->value_data_size);
 	}
 
 	/*
@@ -211,18 +256,21 @@ dbmw_create(dbmap_t *dm, const char *name, size_t key_size, size_t value_size,
 
 	if (common_dbg)
 		g_message("DBMW created \"%s\" with %s back-end "
-			"(max cached = %lu, key=%lu bytes, value=%lu bytes max)",
+			"(max cached = %lu, key=%lu bytes, value=%lu bytes, "
+			"%lu max serialized)",
 			dw->name, dbmw_map_type(dw) == DBMAP_SDBM ? "sdbm" : "map",
 			(gulong) dw->max_cached,
-			(gulong) dw->key_size, (gulong) dw->value_size);
+			(gulong) dw->key_size, (gulong) dw->value_size,
+			(gulong) dw->value_data_size);
 
 	return dw;
 }
 
 /**
  * Write back cached value to disk.
+ * @return TRUE on success
  */
-static void
+static gboolean
 write_back(dbmw_t *dw, gconstpointer key, struct cached *value)
 {
 	dbmap_datum_t dval;
@@ -272,14 +320,41 @@ write_back(dbmw_t *dw, gconstpointer key, struct cached *value)
 	} else if (dbmap_has_ioerr(dw->dm)) {
 		dw->ioerr = TRUE;
 		dw->error = errno;
-		g_warning("DBMW I/O error whilst %s dirty entry for \"%s\": %s",
-			value->absent ? "deleting" : "flushing", dw->name,
+		g_warning("DBMW \"%s\" I/O error whilst %s dirty entry: %s",
+			dw->name, value->absent ? "deleting" : "flushing",
 			dbmap_strerror(dw->dm));
 	} else {
-		g_warning("DBMW error whilst %s dirty entry for \"%s\": %s",
-			value->absent ? "deleting" : "flushing", dw->name,
+		g_warning("DBMW \"%s\" error whilst %s dirty entry: %s",
+			dw->name, value->absent ? "deleting" : "flushing",
 			dbmap_strerror(dw->dm));
 	}
+
+	return ok;
+}
+
+/**
+ * Free memory used to hold the value in the cache.
+ *
+ * If a free routine was registered, it is invoked on the value to cleanup
+ * any dynamically allocated structure created during the value deserialization
+ * and which is still referenced by the value.
+ *
+ * When ``reclaim'' is FALSE, the value free routine is invoked on the
+ * old value to reclaim value-specific allocations, but the value arena
+ * is not freed and can be reused.
+ */
+static void
+free_value(const dbmw_t *dw, struct cached *cv, gboolean reclaim)
+{
+	if (cv->len) {
+		if (dw->valfree)
+			(*dw->valfree)(cv->data, cv->len);
+		if (reclaim) {
+			wfree(cv->data, cv->len);
+			cv->len = 0;
+			cv->data = NULL;
+		}
+	}
 }
 
 /**
@@ -317,9 +392,7 @@ remove_entry(dbmw_t *dw, gconstpointer key, gboolean dispose, gboolean flush)
 	 * Dispose of the cache structure.
 	 */
 
-	if (old->len)
-		wfree(old->data, old->len);
-
+	free_value(dw, old, TRUE);
 	wfree(old, sizeof *old);
 
 	return NULL;
@@ -346,12 +419,11 @@ allocate_entry(dbmw_t *dw, gconstpointer key, struct cached *filled)
 	struct cached *entry;
 	gpointer saved_key;
 
-	g_assert(!hash_list_contains(dw->keys, key, NULL));
+	g_assert(!hash_list_contains(dw->keys, key));
 	g_assert(!map_contains(dw->values, key));
 	g_assert(!filled || (!filled->len == !filled->data));
 
-	saved_key = walloc(dw->key_size);
-	memcpy(saved_key, key, dw->key_size);
+	saved_key = wcopy(key, dw->key_size);
 
 	/*
 	 * If we have less keys cached than our maximum, add it.
@@ -393,7 +465,8 @@ allocate_entry(dbmw_t *dw, gconstpointer key, struct cached *filled)
  * Fill cache entry structure with value data, marking it dirty and present.
  */
 static void
-fill_entry(struct cached *entry, gpointer value, size_t length)
+fill_entry(const dbmw_t *dw,
+	struct cached *entry, gpointer value, size_t length)
 {
 	/*
 	 * Try to reuse old entry arena if same size.
@@ -407,16 +480,13 @@ fill_entry(struct cached *entry, gpointer value, size_t length)
 	if (length != (size_t) entry->len) {
 		gpointer arena = NULL;
 
-		if (length) {
-			arena = walloc(length);
-			memcpy(arena, value, length);
-		}
-		if (entry->len)
-			wfree(entry->data, entry->len);
-
+		if (length)
+			arena = wcopy(value, length);
+		free_value(dw, entry, TRUE);
 		entry->data = arena;
 		entry->len = length;
 	} else if (value != entry->data && length) {
+		free_value(dw, entry, FALSE);
 		memcpy(entry->data, value, length);
 	}
 
@@ -426,27 +496,114 @@ fill_entry(struct cached *entry, gpointer value, size_t length)
 	g_assert(!entry->len == !entry->data);
 }
 
+/**
+ * Context for flushes.
+ */
+struct flush_context {
+	dbmw_t *dw;
+	ssize_t amount;
+	gboolean error;
+};
+
 /**
  * Map iterator to flush dirty cached entries.
  */
 static void
 flush_dirty(gpointer key, gpointer value, gpointer data)
 {
-	dbmw_t *dw = data;
+	struct flush_context *ctx = data;
 	struct cached *entry = value;
 
-	if (entry->dirty)
-		write_back(dw, key, entry);
+	if (entry->dirty) {
+		if (write_back(ctx->dw, key, entry))
+			ctx->amount++;
+		else
+			ctx->error = TRUE;
+	}
 }
 
 /**
  * Synchronize dirty values.
+ *
+ * The ``which'' argument is a bitfield indicating the set of things to
+ * synchronize:
+ *
+ * DBMW_SYNC_CACHE requests that dirty values from the local DBMW cache
+ * be flushed to the DB map layer immediately.
+ *
+ * DBMW_SYNC_MAP requests that the DB map layer be flushed, if it is backed
+ * by disk data.
+ *
+ * @return amount of value flushes plus amount of sdbm page flushes, -1 if
+ * an error occurred.
  */
-void
-dbmw_sync(dbmw_t *dw)
+ssize_t
+dbmw_sync(dbmw_t *dw, int which)
 {
-	map_foreach(dw->values, flush_dirty, dw);
-	dw->count_needs_sync = FALSE;
+	ssize_t amount = 0;
+	gboolean error = FALSE;
+
+	if (which & DBMW_SYNC_CACHE) {
+		struct flush_context ctx;
+
+		ctx.dw = dw;
+		ctx.error = FALSE;
+		ctx.amount = 0;
+
+		map_foreach(dw->values, flush_dirty, &ctx);
+		if (!ctx.error)
+			dw->count_needs_sync = FALSE;
+
+		amount += ctx.amount;
+		error = ctx.error;
+	}
+	if (which & DBMW_SYNC_MAP) {
+		ssize_t ret = dbmap_sync(dw->dm);
+		if (-1 == ret)
+			error = TRUE;
+		else
+			amount += ret;
+	}
+
+	return error ? -1 : amount;
+}
+
+/**
+ * Attempt to shrink DB size.
+ *
+ * @return TRUE if successful (not implying that anything was actually shrunk).
+ */
+gboolean
+dbmw_shrink(dbmw_t *dw)
+{
+	return dbmap_shrink(dw->dm);
+}
+
+/**
+ * Wrapper to the user-supplied deserialization routine for values.
+ *
+ * @param dw		the DBM wrapper object
+ * @param bs		the initialized binary stream from which we're reading
+ * @param valptr	where deserialization should be done
+ * @param len		length of arena at valptr, for assertions
+ *
+ * @return TRUE if deserialization was OK.
+ */
+static gboolean
+dbmw_deserialize(const dbmw_t *dw, bstr_t *bs, gpointer valptr, size_t len)
+{
+	(*dw->unpack)(bs, valptr, len);
+
+	if (bstr_has_error(bs))
+		return FALSE;
+	else if (bstr_unread_size(bs)) {
+		/* Something is wrong, we're not deserializing the right data? */
+		g_warning("DBMW \"%s\" deserialization has %lu trailing unread bytes",
+			dbmw_name(dw), (gulong) bstr_unread_size(bs));
+		return FALSE;
+	}
+
+	return TRUE;
 }
 
 /**
@@ -463,11 +620,20 @@ write_immediately(dbmw_t *dw, gconstpointer key, gpointer value, size_t length)
 	tmp.absent = FALSE;
 
 	write_back(dw, key, &tmp);
+
+	/*
+	 * Free any dynamically allocated memory in the value, through
+	 * registered value cleanup callback.
+	 */
+
+	if (length && dw->valfree)
+		(*dw->valfree)(value, length);
 }
 
 /**
  * Write value to the database file immediately, without caching for write-back
- * nor for future reading.
+ * nor for future reading.  If defined, the registered value cleanup callback
+ * is invoked before returning.
  *
  * @param dw		the DBM wrapper
  * @param key		the key (constant-width, determined at open time)
@@ -477,7 +643,7 @@ write_immediately(dbmw_t *dw, gconstpointer key, gpointer value, size_t length)
 void
 dbmw_write_nocache(dbmw_t *dw, gconstpointer key, gpointer value, size_t length)
 {
-	g_assert(dw);
+	dbmw_check(dw);
 	g_assert(key);
 	g_assert(length <= dw->value_size);
 	g_assert(length || value == NULL);
@@ -490,6 +656,10 @@ dbmw_write_nocache(dbmw_t *dw, gconstpointer key, gpointer value, size_t length)
 /**
  * Write value to the database file, possibly caching it and deferring write.
  *
+ * Any registered value cleanup callback will be invoked right after the value
+ * is written to disk (for immediated writes) or removed from the cache (for
+ * deferred writes).
+ *
  * @param dw		the DBM wrapper
  * @param key		the key (constant-width, determined at open time)
  * @param value		the start of the value in memory
@@ -500,7 +670,7 @@ dbmw_write(dbmw_t *dw, gconstpointer key, gpointer value, size_t length)
 {
 	struct cached *entry;
 
-	g_assert(dw);
+	dbmw_check(dw);
 	g_assert(key);
 	g_assert(length <= dw->value_size);
 	g_assert(length || value == NULL);
@@ -514,11 +684,11 @@ dbmw_write(dbmw_t *dw, gconstpointer key, gpointer value, size_t length)
 			dw->w_hits++;
 		else if (entry->absent)
 			dw->count_needs_sync = TRUE;	/* Key exists now */
-		fill_entry(entry, value, length);
+		fill_entry(dw, entry, value, length);
 		hash_list_moveto_tail(dw->keys, key);
 	} else if (dw->max_cached > 1) {
 		entry = allocate_entry(dw, key, NULL);
-		fill_entry(entry, value, length);
+		fill_entry(dw, entry, value, length);
 		dw->count_needs_sync = TRUE;	/* Does not know whether key exists */
 	} else { 
 		write_immediately(dw, key, value, length);
@@ -547,7 +717,7 @@ dbmw_read(dbmw_t *dw, gconstpointer key, size_t *lenptr)
 	struct cached *entry;
 	dbmap_datum_t dval;
 
-	g_assert(dw);
+	dbmw_check(dw);
 	g_assert(key);
 
 	dw->r_access++;
@@ -569,7 +739,7 @@ dbmw_read(dbmw_t *dw, gconstpointer key, size_t *lenptr)
 	if (dbmap_has_ioerr(dw->dm)) {
 		dw->ioerr = TRUE;
 		dw->error = errno;
-		g_warning("DBMW I/O error whilst reading entry for \"%s\": %s",
+		g_warning("DBMW \"%s\" I/O error whilst reading entry: %s",
 			dw->name, dbmap_strerror(dw->dm));
 		return NULL;
 	} else if (NULL == dval.data)
@@ -595,9 +765,10 @@ dbmw_read(dbmw_t *dw, gconstpointer key, size_t *lenptr)
 
 		bstr_reset(dw->bs, dval.data, dval.len, BSTR_F_ERROR);
 
-		if (!(*dw->unpack)(dw->bs, entry->data, dw->value_size)) {
-			g_error("DBMW deserialization error for %s: %s",
+		if (!dbmw_deserialize(dw, dw->bs, entry->data, dw->value_size)) {
+			g_warning("DBMW \"%s\" deserialization error: %s",
 				dw->name, bstr_error(dw->bs));
+			/* Not calling value free routine on deserialization failures */
 			wfree(entry->data, dw->value_size);
 			wfree(entry, sizeof *entry);
 			return NULL;
@@ -609,9 +780,8 @@ dbmw_read(dbmw_t *dw, gconstpointer key, size_t *lenptr)
 		g_assert(dw->value_size >= dval.len);
 
 		if (dval.len) {
-			entry->data = walloc(dval.len);
 			entry->len = dval.len;
-			memcpy(entry->data, dval.data, dval.len);
+			entry->data = wcopy(dval.data, dval.len);
 		} else {
 			entry->data = NULL;
 			entry->len = 0;
@@ -641,7 +811,7 @@ dbmw_exists(dbmw_t *dw, gconstpointer key)
 	struct cached *entry;
 	gboolean ret;
 
-	g_assert(dw);
+	dbmw_check(dw);
 	g_assert(key);
 
 	dw->r_access++;
@@ -657,7 +827,7 @@ dbmw_exists(dbmw_t *dw, gconstpointer key)
 	if (dbmap_has_ioerr(dw->dm)) {
 		dw->ioerr = TRUE;
 		dw->error = errno;
-		g_warning("DBMW I/O error whilst checking key existence for \"%s\": %s",
+		g_warning("DBMW \"%s\" I/O error whilst checking key existence: %s",
 			dw->name, dbmap_strerror(dw->dm));
 		return FALSE;
 	}
@@ -688,7 +858,7 @@ dbmw_delete(dbmw_t *dw, gconstpointer key)
 {
 	struct cached *entry;
 
-	g_assert(dw);
+	dbmw_check(dw);
 	g_assert(key);
 
 	dw->w_access++;
@@ -699,7 +869,7 @@ dbmw_delete(dbmw_t *dw, gconstpointer key)
 			dw->w_hits++;
 		if (!entry->absent) {
 			dw->count_needs_sync = TRUE;	/* Deferred delete */
-			fill_entry(entry, NULL, 0);
+			fill_entry(dw, entry, NULL, 0);
 			entry->absent = TRUE;
 		}
 		hash_list_moveto_tail(dw->keys, key);
@@ -709,7 +879,7 @@ dbmw_delete(dbmw_t *dw, gconstpointer key)
 		if (dbmap_has_ioerr(dw->dm)) {
 			dw->ioerr = TRUE;
 			dw->error = errno;
-			g_warning("DBMW I/O error whilst deleting key for \"%s\": %s",
+			g_warning("DBMW \"%s\" I/O error whilst deleting key: %s",
 				dw->name, dbmap_strerror(dw->dm));
 		}
 
@@ -734,19 +904,38 @@ dbmw_delete(dbmw_t *dw, gconstpointer key)
 /**
  * Map iterator to free cached entries.
  */
-static void
+static gboolean
 free_cached(gpointer key, gpointer value, gpointer data)
 {
 	dbmw_t *dw = data;
 	struct cached *entry = value;
 
-	g_assert(!entry->dirty);
+	dbmw_check(dw);
+	g_assert(dw->is_volatile || !entry->dirty);
 	g_assert(!entry->len == !entry->data);
 
-	if (entry->len)
-		wfree(entry->data, entry->len);
+	free_value(dw, entry, TRUE);
 	wfree(key, dw->key_size);
 	wfree(entry, sizeof *entry);
+	return TRUE;
+}
+
+/**
+ * Clear the cache, discard everything.
+ */
+static void
+dbmw_clear_cache(dbmw_t *dw)
+{
+	dbmw_check(dw);
+
+	/*
+	 * In the cache, the hash list and the value cache share the same
+	 * key pointers.  Therefore, we need to iterate on the map only
+	 * to free both at the same time.
+	 */
+
+	hash_list_clear(dw->keys);
+	map_foreach_remove(dw->values, free_cached, dw);
 }
 
 /**
@@ -755,9 +944,9 @@ free_cached(gpointer key, gpointer value, gpointer data)
 void
 dbmw_destroy(dbmw_t *dw, gboolean close_map)
 {
-	dbmw_sync(dw);
+	dbmw_check(dw);
 
-	if (common_dbg)
+	if (common_stats)
 		g_message("DBMW destroying \"%s\" with %s back-end "
 			"(read cache hits = %.2f%% on %s request%s, "
 			"write cache hits = %.2f%% on %s request%s)",
@@ -768,26 +957,158 @@ dbmw_destroy(dbmw_t *dw, gboolean close_map)
 			uint64_to_string2(dw->w_access), 1 == dw->w_access ? "" : "s");
 
 	/*
-	 * In the cache, the hash list and the value cache share the same
-	 * key pointers.  Therefore, we need to iterate on the map only
-	 * to free both at the same time.
+	 * If we close the map and we're volatile, there's no need to flush
+	 * the cache as the data is going to be gone soon anyway.
 	 */
 
-	map_foreach(dw->values, free_cached, dw);
+	if (!close_map || !dw->is_volatile) {
+		dbmw_sync(dw, DBMW_SYNC_CACHE);
+	}
+
+	dbmw_clear_cache(dw);
 	hash_list_free(&dw->keys);
 	map_destroy(dw->values);
 
 	if (dw->mb)
 		pmsg_free(dw->mb);
-	if (dw->bs)
-		bstr_destroy(dw->bs);
+	bstr_free(&dw->bs);
 
 	if (close_map)
 		dbmap_destroy(dw->dm);
 
+	dw->magic = 0;
 	wfree(dw, sizeof *dw);
 }
 
+/**
+ * Structure used as context by dbmw_foreach_*trampoline().
+ */
+struct foreach_ctx {
+	union {
+		dbmw_cb_t cb;
+		dbmw_cbr_t cbr;
+	} u;
+	gpointer arg;
+	const dbmw_t *dw;
+};
+
+/**
+ * Common code for dbmw_foreach_trampoline() and
+ * dbmw_foreach_remove_trampoline().
+ */
+static gboolean
+dbmw_foreach_common(gboolean removing,
+	gpointer key, dbmap_datum_t *d, gpointer arg)
+{
+	struct foreach_ctx *ctx = arg;
+	const dbmw_t *dw = ctx->dw;
+	struct cached *entry;
+
+	dbmw_check(dw);
+
+	entry = map_lookup(dw->values, key);
+	if (entry != NULL) {
+		if (entry->absent)
+			return TRUE;		/* Key was deleted in cache */
+		if (removing) {
+			return (*ctx->u.cbr)(key, entry->data, entry->len, ctx->arg);
+		} else {
+			(*ctx->u.cb)(key, entry->data, entry->len, ctx->arg);
+			return FALSE;
+		}
+	} else {
+		gboolean status = FALSE;
+		gpointer data = d->data;
+		size_t len = d->len;
+
+		/*
+		 * Deserialize data if needed, but do not cache this value.
+		 * Iterating over the map must not disrupt the cache.
+		 */
+
+		if (dw->unpack) {
+			len = dw->value_size;
+			data = walloc(len);
+
+			bstr_reset(dw->bs, d->data, d->len, BSTR_F_ERROR);
+
+			if (!dbmw_deserialize(dw, dw->bs, data, len)) {
+				g_warning("DBMW \"%s\" deserialization error: %s",
+					dw->name, bstr_error(dw->bs));
+				/* Not calling value free routine on deserialization failures */
+				wfree(data, len);
+				return FALSE;
+			}
+		}
+
+		if (removing) {
+			status = (*ctx->u.cbr)(key, data, len, ctx->arg);
+		} else {
+			(*ctx->u.cb)(key, data, len, ctx->arg);
+		}
+
+		if (dw->unpack) {
+			if (dw->valfree)
+				(*dw->valfree)(data, len);
+			wfree(data, len);
+		}
+
+		return status;
+	}
+}
+
+/**
+ * Trampoline to invoke the DB map iterator and do the proper casts.
+ */
+static void
+dbmw_foreach_trampoline(gpointer key, dbmap_datum_t *d, gpointer arg)
+{
+	dbmw_foreach_common(FALSE, key, d, arg);
+}
+
+/**
+ * Trampoline to invoke the map iterator and do the proper casts.
+ */
+static gboolean
+dbmw_foreach_remove_trampoline(gpointer key, dbmap_datum_t *d, gpointer arg)
+{
+	return dbmw_foreach_common(TRUE, key, d, arg);
+}
+
+/**
+ * Iterate over the DB, invoking the callback on each item along with the
+ * supplied argument.
+ */
+void dbmw_foreach(const dbmw_t *dw, dbmw_cb_t cb, gpointer arg)
+{
+	struct foreach_ctx ctx;
+
+	dbmw_check(dw);
+
+	ctx.u.cb = cb;
+	ctx.arg = arg;
+	ctx.dw = dw;
+
+	dbmap_foreach(dw->dm, dbmw_foreach_trampoline, &ctx);
+}
+
+/**
+ * Iterate over the DB, invoking the callback on each item along with the
+ * supplied argument and removing the item when the callback returns TRUE.
+ */
+void dbmw_foreach_remove(const dbmw_t *dw, dbmw_cbr_t cbr, gpointer arg)
+{
+	struct foreach_ctx ctx;
+
+	dbmw_check(dw);
+
+	ctx.u.cbr = cbr;
+	ctx.arg = arg;
+	ctx.dw = dw;
+
+	dbmap_foreach_remove(dw->dm, dbmw_foreach_remove_trampoline, &ctx);
+}
+
 /**
  * Snapshot all the keys, returning them into a singly linked list.
  * To free the returned keys, use the dbmw_free_all_keys() helper.
@@ -795,6 +1116,8 @@ dbmw_destroy(dbmw_t *dw, gboolean close_map)
 GSList *
 dbmw_all_keys(const dbmw_t *dw)
 {
+	dbmw_check(dw);
+
 	return dbmap_all_keys(dw->dm);
 }
 
@@ -804,7 +1127,78 @@ dbmw_all_keys(const dbmw_t *dw)
 void
 dbmw_free_all_keys(const dbmw_t *dw, GSList *keys)
 {
+	dbmw_check(dw);
+
 	dbmap_free_all_keys(dw->dm, keys);
 }
 
+/**
+ * Store DBMW map to disk in an SDBM database, at the specified base.
+ * Two files are created (using suffixes .pag and .dir).
+ *
+ * @param dw		the DBMW map to store
+ * @param base		base path for the persistent database
+ * @param inplace	if TRUE and map was an SDBM already, persist as itself
+ *
+ * @return TRUE on success.
+ */
+gboolean
+dbmw_store(dbmw_t *dw, const char *base, gboolean inplace)
+{
+	dbmw_check(dw);
+
+	dbmw_sync(dw, DBMW_SYNC_CACHE);
+	return dbmap_store(dw->dm, base, inplace);
+}
+
+/**
+ * Copy all the data from one DBMW map to another, replacing values if the
+ * destination is not empty and already holds some data.
+ *
+ * @return TRUE on success.
+ */
+gboolean
+dbmw_copy(dbmw_t *from, dbmw_t *to)
+{
+	dbmw_check(from);
+	dbmw_check(to);
+
+	dbmw_sync(from, DBMW_SYNC_CACHE);
+	dbmw_sync(to, DBMW_SYNC_CACHE);
+	dbmw_clear_cache(to);
+
+	/*
+	 * Since ``from'' was sync'ed and the cache from ``to'' was cleared,
+	 * we can ignore caches and handle the copy at the dbmap level.
+	 */
+
+	return dbmap_copy(from->dm, to->dm);
+}
+
+/**
+ * Set the map cache size, as an amount of 1 KiB pages.
+ * @return TRUE on success.
+ */
+gboolean
+dbmw_set_map_cache(dbmw_t *dw, long pages)
+{
+	dbmw_check(dw);
+
+	return 0 == dbmap_set_cachesize(dw->dm, pages);
+}
+
+/**
+ * Flag whether database is volatile (never outlives a close).
+ *
+ * @return TRUE on success.
+ */
+gboolean
+dbmw_set_volatile(dbmw_t *dw, gboolean is_volatile)
+{
+	dbmw_check(dw);
+
+	dw->is_volatile = TRUE;
+	return 0 == dbmap_set_volatile(dw->dm, is_volatile);
+}
+
 /* vi: set ts=4 sw=4 cindent: */
