@@ -1,5 +1,5 @@
 /*
- * $Id: debug.h 15432 2008-07-17 17:19:38Z rmanfredi $
+ * $Id: debug.h 16619 2009-05-01 20:04:18Z rmanfredi $
  *
  * Copyright (c) 2008, Raphael Manfredi
  *
@@ -39,8 +39,10 @@
 #include "common.h"
 
 extern guint32 common_dbg;		/**< Debug level for library files */
+extern guint32 common_stats;	/**< Log level for library statistics */
 
 void set_library_debug(guint32 level);
+void set_library_stats(guint32 level);
 
 #endif /* _debug_h_ */
 
