@@ -1,5 +1,5 @@
 /*
- * $Id: event.c 16103 2009-03-10 14:53:39Z rmanfredi $
+ * $Id: event.c 16403 2009-04-04 21:32:20Z rmanfredi $
  *
  * Copyright (c) 2002-2003, Richard Eckart
  *
@@ -33,7 +33,7 @@
 
 #include "common.h"
 
-RCSID("$Id: event.c 16103 2009-03-10 14:53:39Z rmanfredi $")
+RCSID("$Id: event.c 16403 2009-04-04 21:32:20Z rmanfredi $")
 
 #include "event.h"
 #include "misc.h"
@@ -71,7 +71,7 @@ event_new(const char *name)
     evt = g_new0(struct event, 1);
     evt->name = name;
 
-    return evt;
+    return NOT_LEAKING(evt);	/* Allocated once, never freed */
 }
 
 /**
