@@ -1,5 +1,5 @@
 /*
- * $Id: event.h 9380 2005-08-31 21:37:34Z rmanfredi $
+ * $Id: event.h 16103 2009-03-10 14:53:39Z rmanfredi $
  *
  * Copyright (c) 2002-2003, Richard Eckart
  *
@@ -43,12 +43,12 @@ struct subscriber {
 };
 
 typedef struct event {
-    const gchar *name;
+    const char *name;
     guint32      triggered_count;
     GSList      *subscribers;
 } event_t;
 
-struct event *event_new(const gchar *name);
+struct event *event_new(const char *name);
 
 #define event_destroy(evt) G_STMT_START {                          \
     real_event_destroy(evt);                                       \
