@@ -1,5 +1,5 @@
 /*
- * $Id: fast_assert.c 13669 2007-05-16 15:49:55Z cbiere $
+ * $Id: fast_assert.c 17213 2009-11-09 17:13:18Z rmanfredi $
  *
  * Copyright (c) 2006, Christian Biere
  *
@@ -35,24 +35,12 @@
 
 #include "common.h"
 
-RCSID("$Id: fast_assert.c 13669 2007-05-16 15:49:55Z cbiere $")
+RCSID("$Id: fast_assert.c 17213 2009-11-09 17:13:18Z rmanfredi $")
 
+#include "lib/crash.h"				/* For print_str() and crash_time() */
 #include "lib/fast_assert.h"
-#include "lib/misc.h"
 #include "lib/override.h"			/* Must be the last header included */
 
-#define print_str(x) \
-G_STMT_START { \
-	if (iov_cnt < G_N_ELEMENTS(iov)) { \
-		const char *ptr = (x); \
-		if (ptr) { \
-			iov[iov_cnt].iov_base = (char *) ptr; \
-			iov[iov_cnt].iov_len = strlen(ptr); \
-			iov_cnt++; \
-		} \
-	} \
-} G_STMT_END
-
 /**
  * @note For maximum safety this is kept signal-safe, so that we can
  *       even use assertions in signal handlers. See also:
@@ -62,10 +50,14 @@ static void
 assertion_message(const assertion_data * const data, int fatal)
 {
 	char line_buf[22];
+	char time_buf[18];
 	struct iovec iov[16];
 	unsigned iov_cnt = 0;
 
-	print_str(fatal ? "FATAL: " : "WARNING: ");
+	crash_time(time_buf, sizeof time_buf);
+
+	print_str(time_buf);
+	print_str(fatal ? " FATAL: " : " WARNING: ");
 	if (data->expr) {
 		print_str("Assertion failure in ");
 	} else {
@@ -83,13 +75,21 @@ assertion_message(const assertion_data * const data, int fatal)
 	writev(STDERR_FILENO, iov, iov_cnt);
 }
 
-void NON_NULL_PARAM((1)) REGPARM(1)
+/*
+ * Due to an optimizer bug in gcc 4.2.1 (and maybe later verions), avoid
+ * specifying the REGPARM(1) attribute in the assertion_xxx() routines
+ * or the pointer being passed will be garbage, causing a segmentation fault
+ * in assertion_message().
+ *		--RAM, 2009-10-31
+ */
+
+void NON_NULL_PARAM((1)) /* REGPARM(1) */
 assertion_warning(const assertion_data * const data)
 {
 	assertion_message(data, FALSE);
 }
 
-void G_GNUC_NORETURN NON_NULL_PARAM((1)) REGPARM(1)
+void G_GNUC_NORETURN NON_NULL_PARAM((1)) /* REGPARM(1) */
 assertion_failure(const assertion_data * const data)
 {
 	assertion_message(data, TRUE);
