@@ -1,5 +1,5 @@
 /*
- * $Id: file.c 16108 2009-03-10 15:19:52Z rmanfredi $
+ * $Id: file.c 17071 2009-10-13 14:40:20Z rmanfredi $
  *
  * Copyright (c) 2002-2003, Raphael Manfredi
  *
@@ -35,11 +35,14 @@
 
 #include "common.h"
 
-RCSID("$Id: file.c 16108 2009-03-10 15:19:52Z rmanfredi $")
+RCSID("$Id: file.c 17071 2009-10-13 14:40:20Z rmanfredi $")
 
 #include "file.h"
 #include "debug.h"
-#include "misc.h"
+#include "fd.h"
+#include "halloc.h"
+#include "path.h"
+#include "timestamp.h"
 #include "tm.h"
 #include "override.h"		/* Must be the last header included */
 
@@ -56,6 +59,18 @@ static const char empty_str[] = "";
  */
 static reclaim_fd_t reclaim_fd = NULL;
 
+/**
+ * Check for file existence.
+ */
+gboolean
+file_exists(const char *pathname)
+{
+  	struct stat st;
+
+    g_assert(pathname);
+    return 0 == stat(pathname, &st) && S_ISREG(st.st_mode);
+}
+
 /**
  * Register fd reclaiming callback.
  * Use NULL to unregister it.
@@ -101,11 +116,11 @@ open_read(
 
 	path = make_pathname(fv->dir, fv->name);
 	if (!is_absolute_path(path)) {
-		G_FREE_NULL(path);
+		HFREE_NULL(path);
 		return NULL;
 	}
 
-	path_orig = g_strdup_printf("%s.%s", path, orig_ext);
+	path_orig = h_strdup_printf("%s.%s", path, orig_ext);
 	in = fopen(path, "r");
 	if (in) {
 		if (renaming && -1 == rename(path, path_orig))
@@ -136,7 +151,7 @@ open_read(
 	if (in != NULL) {
 		instead = instead_str;
 
-		G_FREE_NULL(path);
+		HFREE_NULL(path);
 		path = path_orig;
 		path_orig = NULL;
 	}
@@ -152,7 +167,7 @@ open_read(
 		instead = instead_str;
 
 		for (xfv = fv + 1, xfvcnt = fvcnt - 1; xfvcnt; xfv++, xfvcnt--) {
-			G_FREE_NULL(path);
+			HFREE_NULL(path);
 			path = make_pathname(xfv->dir, xfv->name);
 			idx++;
 			if (NULL != path && NULL != (in = fopen(path, "r")))
@@ -174,8 +189,8 @@ open_read(
 
 out:
 
-	G_FREE_NULL(path);
-	G_FREE_NULL(path_orig);
+	HFREE_NULL(path);
+	HFREE_NULL(path_orig);
 	if (in != NULL && chosen != NULL)
 		*chosen = idx;
 
@@ -239,7 +254,7 @@ file_config_open(const char *what, const file_path_t *fv)
 	FILE *out = NULL;
 	char *path;
 
-	path = g_strconcat(fv->dir, G_DIR_SEPARATOR_S, fv->name, ".",
+	path = h_strconcat(fv->dir, G_DIR_SEPARATOR_S, fv->name, ".",
 				new_ext, (void *) 0);
 	g_return_val_if_fail(NULL != path, NULL);
 
@@ -247,7 +262,7 @@ file_config_open(const char *what, const file_path_t *fv)
 		out = file_fopen(path, "w");
 		if (out == NULL)
 			g_warning("unable to persist %s", what);
-		G_FREE_NULL(path);
+		HFREE_NULL(path);
 	}
 	return out;
 }
@@ -280,7 +295,7 @@ file_config_close(FILE *out, const file_path_t *fv)
 
 	path = make_pathname(fv->dir, fv->name);
 	g_return_val_if_fail(NULL != path, FALSE);
-	path_new = g_strdup_printf("%s.%s", path, new_ext);
+	path_new = h_strdup_printf("%s.%s", path, new_ext);
 	if (NULL == path_new)
 		goto failed;
 
@@ -294,8 +309,8 @@ file_config_close(FILE *out, const file_path_t *fv)
 
 failed:
 
-	G_FREE_NULL(path_new);
-	G_FREE_NULL(path);
+	HFREE_NULL(path_new);
+	HFREE_NULL(path);
 	return success;
 }
 
@@ -334,12 +349,13 @@ file_path_set(file_path_t *fp, const char *dir, const char *name)
  * case no error is logged for ENOENT.
  */
 static int
-do_open(const char *path, int flags, int mode, gboolean missing)
+do_open(const char *path, int flags, int mode,
+	gboolean missing, gboolean absolute)
 {
 	const char *what;
 	int fd;
 
-	if (!is_absolute_path(path)) {
+	if (absolute && !is_absolute_path(path)) {
 		errno = EPERM;
 		return -1;
 	}
@@ -406,43 +422,57 @@ do_open(const char *path, int flags, int mode, gboolean missing)
 /**
  * Open file, returning file descriptor or -1 on error with errno set.
  * Errors are logged as a warning.
+ *
+ * This is a perfect replacement for the open() system call, with logging
+ * on errors.
  */
 int
 file_open(const char *path, int flags, int mode)
 {
-	return do_open(path, flags, mode, FALSE);
+	return do_open(path, flags, mode, FALSE, FALSE);
 }
 
 /**
- * Open file, returning file descriptor or -1 on error with errno set.
+ * Open file given with absolute pathname.
+ * Returns file descriptor or -1 on error with errno set.
+ * Errors are logged as a warning.
+ */
+int
+file_absolute_open(const char *path, int flags, int mode)
+{
+	return do_open(path, flags, mode, FALSE, TRUE);
+}
+
+/**
+ * Open absolute file, returning file descriptor or -1 on error with errno set.
  * Errors are logged as a warning, unless the file is missing, in which
  * case nothing is logged.
  */
 int
 file_open_missing(const char *path, int flags)
 {
-	return do_open(path, flags, 0, TRUE);
+	return do_open(path, flags, 0, TRUE, TRUE);
 }
 
 /**
- * Create file, returning file descriptor or -1 on error with errno set.
+ * Create absolute file, returning file descriptor or -1 on error with errno set.
  * Errors are logged as a warning.
  */
 int
 file_create(const char *path, int flags, int mode)
 {
-	return do_open(path, flags | O_CREAT, mode, FALSE);
+	return do_open(path, flags | O_CREAT, mode, FALSE, TRUE);
 }
 
 /**
- * Create file, returning file descriptor or -1 on error with errno set.
+ * Create absolute file, returning file descriptor or -1 on error with errno set.
  * Errors are logged as a warning, unless the error is ENOENT which means
  * the directory does not exist.
  */
 int
 file_create_missing(const char *path, int flags, int mode)
 {
-	return do_open(path, flags | O_CREAT, mode, TRUE);
+	return do_open(path, flags | O_CREAT, mode, TRUE, TRUE);
 }
 
 /**
@@ -519,32 +549,4 @@ file_fopen_missing(const char *path, const char *mode)
 	return do_fopen(path, mode, TRUE);
 }
 
-void
-file_set_nonblocking(int fd)
-{
-	int ret, flags;
-
-	ret = fcntl(fd, F_GETFL, 0);
-	flags = ret | VAL_O_NONBLOCK;
-	if (flags != ret)
-		fcntl(fd, F_SETFL, flags);
-}
-
-/**
- * Closes the file and sets the descriptor to -1. Does nothing if
- * the descriptor is already -1.
- * @param fd_ptr Must point to a non-negative file descriptor or -1.
- */
-void
-file_close(int *fd_ptr)
-{
-	g_assert(fd_ptr);
-	g_assert(*fd_ptr >= -1);
-
-	if (-1 != *fd_ptr) {
-		close(*fd_ptr);
-		*fd_ptr = -1;
-	}
-}
-
 /* vi: set ts=4: */
