@@ -1,5 +1,5 @@
 /*
- * $Id: file.h 16108 2009-03-10 15:19:52Z rmanfredi $
+ * $Id: file.h 17071 2009-10-13 14:40:20Z rmanfredi $
  *
  * Copyright (c) 2002-2003, Raphael Manfredi
  *
@@ -51,6 +51,7 @@ typedef struct {
  */
 
 void file_register_fd_reclaimer(reclaim_fd_t callback);
+gboolean file_exists(const char *pathname);
 
 FILE *file_config_open_read(
 	const char *what, const file_path_t *fv, int fvcnt);
@@ -65,14 +66,12 @@ void file_config_preamble(FILE *out, const char *what);
 void file_path_set(file_path_t *fp, const char *dir, const char *name);
 
 int file_open(const char *path, int flags, int mode);
+int file_absolute_open(const char *path, int flags, int mode);
 int file_open_missing(const char *path, int flags);
 int file_create(const char *path, int flags, int mode);
 int file_create_missing(const char *path, int flags, int mode);
 FILE *file_fopen(const char *path, const char *mode);
 FILE *file_fopen_missing(const char *path, const char *mode);
 
-void file_set_nonblocking(int fd);
-void file_close(int *);
-
 #endif /* _file_ */
 /* vi: set ts=4 sw=4 cindent: */
