@@ -1,25 +1,39 @@
-#include <stdlib.h>
 #ifndef lint
-#if 0
-static char yysccsid[] = "@(#)yaccpar	1.9 (Berkeley) 02/21/93";
-#else
-#if defined(__NetBSD__) && defined(__IDSTRING)
-__IDSTRING(yyrcsid, "$NetBSD: skeleton.c,v 1.26 2006/05/24 18:01:43 christos Exp $");
-#endif /* __NetBSD__ && __IDSTRING */
-#endif /* 0 */
-#endif /* lint */
+static const char yysccsid[] = "@(#)yaccpar	1.9 (Berkeley) 02/21/93";
+#endif
+
+#include <stdlib.h>
+#include <string.h>
+
 #define YYBYACC 1
 #define YYMAJOR 1
 #define YYMINOR 9
-#define YYLEX yylex()
-#define YYEMPTY -1
-#define yyclearin (yychar=(YYEMPTY))
-#define yyerrok (yyerrflag=0)
-#define YYRECOVERING (yyerrflag!=0)
+#define YYPATCH 20090221
+
+#define YYEMPTY        (-1)
+#define yyclearin      (yychar = YYEMPTY)
+#define yyerrok        (yyerrflag = 0)
+#define YYRECOVERING() (yyerrflag != 0)
+
+/* compatibility with bison */
+#ifdef YYPARSE_PARAM
+/* compatibility with FreeBSD */
+#ifdef YYPARSE_PARAM_TYPE
+#define YYPARSE_DECL() yyparse(YYPARSE_PARAM_TYPE YYPARSE_PARAM)
+#else
+#define YYPARSE_DECL() yyparse(void *YYPARSE_PARAM)
+#endif
+#else
+#define YYPARSE_DECL() yyparse(void)
+#endif /* YYPARSE_PARAM */
+
+extern int YYPARSE_DECL();
+
+static int yygrowstack(void);
 #define YYPREFIX "yy"
 #line 2 "getdate.y"
 /*
- * $Id: getdate.c 12242 2006-11-06 20:41:55Z cbiere $
+ * $Id: getdate.c 17213 2009-11-09 17:13:18Z rmanfredi $
  *
  * Date parsing.
  *
@@ -97,6 +111,8 @@ __IDSTRING(yyrcsid, "$NetBSD: skeleton.c,v 1.26 2006/05/24 18:01:43 christos Exp
 #define ISDIGIT(c) ((unsigned) (c) - '0' <= 9)
 
 #include "getdate.h"
+#include "offtime.h"		/* For TM_YEAR_ORIGIN */
+#include "timestamp.h"		/* For diff_tm */
 
 /* Some old versions of bison generate parsers that use bcopy.
    That loses on systems that don't provide the function, so we have
@@ -210,12 +226,12 @@ static int	yyRelMonth;
 static int	yyRelSeconds;
 static int	yyRelYear;
 
-#line 196 "getdate.y"
+#line 198 "getdate.y"
 typedef union {
     int			Number;
     enum _MERIDIAN	Meridian;
 } YYSTYPE;
-#line 219 "y.tab.c"
+#line 234 "y.tab.c"
 #define tAGO 257
 #define tDAY 258
 #define tDAY_UNIT 259
@@ -235,7 +251,7 @@ typedef union {
 #define tNUMBER_T 273
 #define tNUMBER_DOT 274
 #define YYERRCODE 256
-const short yylhs[] = {                                        -1,
+static const short yylhs[] = {                           -1,
     0,    0,    2,    2,    2,    2,    2,    2,    3,    3,
     3,    3,    3,    3,    3,    3,    4,    4,    4,    9,
     6,    6,    6,    5,    5,    5,    5,    5,    5,    5,
@@ -243,7 +259,7 @@ const short yylhs[] = {                                        -1,
    10,   10,   10,   10,   10,   10,   10,   10,   10,   10,
    10,   10,    8,    1,    1,
 };
-const short yylen[] = {                                         2,
+static const short yylen[] = {                            2,
     0,    2,    1,    1,    1,    1,    1,    1,    2,    4,
     4,    6,    6,    6,    7,    6,    1,    1,    2,    3,
     1,    2,    2,    3,    5,    3,    3,    3,    2,    4,
@@ -251,7 +267,7 @@ const short yylen[] = {                                         2,
     2,    2,    1,    2,    2,    1,    2,    2,    1,    2,
     2,    1,    1,    0,    1,
 };
-const short yydefred[] = {                                      1,
+static const short yydefred[] = {                         1,
     0,    0,   43,   18,   46,   49,    0,   40,   52,    0,
     0,   37,    0,    2,    3,    4,    5,    6,    7,    8,
     0,   22,    0,   42,   45,   48,   39,   51,   36,   23,
@@ -260,10 +276,10 @@ const short yydefred[] = {                                      1,
    30,   55,   11,    0,   10,    0,    0,    0,   25,    0,
    12,   13,    0,    0,    0,   15,   20,
 };
-const short yydgoto[] = {                                       1,
+static const short yydgoto[] = {                          1,
    55,   14,   15,   16,   17,   18,   19,   20,   62,   21,
 };
-const short yysindex[] = {                                      0,
+static const short yysindex[] = {                         0,
  -245,  -40,    0,    0,    0,    0, -260,    0,    0, -230,
   -47,    0, -249,    0,    0,    0,    0,    0,    0,    0,
  -239,    0,  -25,    0,    0,    0,    0,    0,    0,    0,
@@ -272,7 +288,7 @@ const short yysindex[] = {                                      0,
     0,    0,    0, -240,    0, -228, -261, -227,    0,  -18,
     0,    0, -225, -223,  -18,    0,    0,
 };
-const short yyrindex[] = {                                      0,
+static const short yyrindex[] = {                         0,
     0,    1,    0,    0,    0,    0,    0,    0,    0,    0,
   124,    0,   16,    0,    0,    0,    0,    0,    0,    0,
    31,    0,   46,    0,    0,    0,    0,    0,    0,    0,
@@ -281,11 +297,11 @@ const short yyrindex[] = {                                      0,
     0,    0,    0,    0,    0,    0,   91,    0,    0,   76,
     0,    0,  106,    0,    0,    0,    0,
 };
-const short yygindex[] = {                                      0,
+static const short yygindex[] = {                         0,
   -12,    0,    0,    0,    0,    0,    0,    0,  -15,    0,
 };
 #define YYTABLESIZE 396
-const short yytable[] = {                                      41,
+static const short yytable[] = {                         41,
    21,   54,   52,   22,   47,   45,   46,   60,   48,   23,
    40,   42,    2,    3,    4,   17,    5,   43,   44,    6,
     7,    8,    9,   10,   11,   12,   13,   49,   24,   57,
@@ -327,7 +343,7 @@ const short yytable[] = {                                      41,
    31,    0,   31,   53,    0,   31,   31,   31,   31,    0,
     0,   31,   31,   53,    0,   53,
 };
-const short yycheck[] = {                                      47,
+static const short yycheck[] = {                         47,
     0,   58,  264,   44,  269,  269,  270,  269,  273,  270,
    58,  261,  258,  259,  260,    0,  262,  257,   44,  265,
   266,  267,  268,  269,  270,  271,  272,  270,  259,  270,
@@ -375,7 +391,8 @@ const short yycheck[] = {                                      47,
 #endif
 #define YYMAXTOKEN 274
 #if YYDEBUG
-const char * const yyname[] = {
+static const char *yyname[] = {
+
 "end-of-file",0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
 0,0,0,0,0,0,0,0,0,0,"','",0,0,"'/'",0,0,0,0,0,0,0,0,0,0,"':'",0,0,0,0,0,0,0,0,0,
 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
@@ -387,7 +404,7 @@ const char * const yyname[] = {
 "tMONTH","tMONTH_UNIT","tSEC_UNIT","tSNUMBER","tUNUMBER","tYEAR_UNIT","tZONE",
 "tNUMBER_T","tNUMBER_DOT",
 };
-const char * const yyrule[] = {
+static const char *yyrule[] = {
 "$accept : spec",
 "spec :",
 "spec : spec item",
@@ -444,34 +461,43 @@ const char * const yyrule[] = {
 "number : tUNUMBER",
 "o_merid :",
 "o_merid : tMERIDIAN",
+
 };
 #endif
+#if YYDEBUG
+#include <stdio.h>
+#endif
+
+/* define the initial stack-sizes */
 #ifdef YYSTACKSIZE
 #undef YYMAXDEPTH
-#define YYMAXDEPTH YYSTACKSIZE
+#define YYMAXDEPTH  YYSTACKSIZE
 #else
 #ifdef YYMAXDEPTH
 #define YYSTACKSIZE YYMAXDEPTH
 #else
-#define YYSTACKSIZE 10000
-#define YYMAXDEPTH 10000
+#define YYSTACKSIZE 500
+#define YYMAXDEPTH  500
 #endif
 #endif
-#define YYINITSTACKSIZE 200
-int yydebug;
-int yynerrs;
-int yyerrflag;
-int yychar;
-short *yyssp;
+
+#define YYINITSTACKSIZE 500
+
+int      yydebug;
+int      yynerrs;
+int      yyerrflag;
+int      yychar;
+short   *yyssp;
 YYSTYPE *yyvsp;
-YYSTYPE yyval;
-YYSTYPE yylval;
-short *yyss;
-short *yysslim;
-YYSTYPE *yyvs;
-int yystacksize;
-int yyparse(void);
-#line 500 "getdate.y"
+YYSTYPE  yyval;
+YYSTYPE  yylval;
+
+/* variables for the parser stack */
+static short   *yyss;
+static short   *yysslim;
+static YYSTYPE *yyvs;
+static unsigned yystacksize;
+#line 502 "getdate.y"
 
 /* Month and day table. */
 static TABLE const MonthDayTable[] = {
@@ -701,7 +727,7 @@ static int ToYear(int Year)
     if (Year < 69)
 	Year += 2000;
     else if (Year < 100)
-	Year += 1900;
+	Year += TM_YEAR_ORIGIN;
 
     return Year;
 }
@@ -879,27 +905,6 @@ static int yylex(void)
     }
 }
 
-#define TM_YEAR_ORIGIN 1900
-
-/* Yield A - B, measured in seconds.  */
-static long difftm(struct tm *a, struct tm *b)
-{
-    int ay = a->tm_year + (TM_YEAR_ORIGIN - 1);
-    int by = b->tm_year + (TM_YEAR_ORIGIN - 1);
-    long days = (
-		    /* difference in day of year */
-		    a->tm_yday - b->tm_yday
-		    /* + intervening leap days */
-		    + ((ay >> 2) - (by >> 2))
-		    - (ay / 100 - by / 100)
-		    + ((ay / 100 >> 2) - (by / 100 >> 2))
-		    /* + difference in years * 365 */
-		    + (long) (ay - by) * 365);
-    return (60 * (60 * (24 * days + (a->tm_hour - b->tm_hour))
-		  + (a->tm_min - b->tm_min))
-	    + (a->tm_sec - b->tm_sec));
-}
-
 /*
  * date2time
  *
@@ -997,7 +1002,7 @@ time_t date2time(const char *p, time_t now)
     }
 
     if (yyHaveZone) {
-	long delta = yyTimezone * 60L + difftm(&tm, gmtime(&Start));
+	long delta = yyTimezone * 60L + diff_tm(&tm, gmtime(&Start));
 	if ((Start + delta < Start) != (delta < 0))
 	    return -1;		/* time_t overflow */
 	Start += delta;
@@ -1033,12 +1038,12 @@ int main(int ac, char *av[])
     /* NOTREACHED */
 }
 #endif				/* defined (TEST) */
-#line 1037 "y.tab.c"
+#line 1041 "y.tab.c"
 /* allocate initial stack or double stack size, up to YYMAXDEPTH */
-static int yygrowstack(void);
 static int yygrowstack(void)
 {
-    int newsize, i;
+    int i;
+    unsigned newsize;
     short *newss;
     YYSTYPE *newvs;
 
@@ -1048,13 +1053,22 @@ static int yygrowstack(void)
         return -1;
     else if ((newsize *= 2) > YYMAXDEPTH)
         newsize = YYMAXDEPTH;
+
     i = yyssp - yyss;
-    if ((newss = (short *)realloc(yyss, newsize * sizeof *newss)) == NULL)
+    newss = (yyss != 0)
+          ? (short *)realloc(yyss, newsize * sizeof(*newss))
+          : (short *)malloc(newsize * sizeof(*newss));
+    if (newss == 0)
         return -1;
-    yyss = newss;
+
+    yyss  = newss;
     yyssp = newss + i;
-    if ((newvs = (YYSTYPE *)realloc(yyvs, newsize * sizeof *newvs)) == NULL)
+    newvs = (yyvs != 0)
+          ? (YYSTYPE *)realloc(yyvs, newsize * sizeof(*newvs))
+          : (YYSTYPE *)malloc(newsize * sizeof(*newvs));
+    if (newvs == 0)
         return -1;
+
     yyvs = newvs;
     yyvsp = newvs + i;
     yystacksize = newsize;
@@ -1062,18 +1076,19 @@ static int yygrowstack(void)
     return 0;
 }
 
-#define YYABORT goto yyabort
+#define YYABORT  goto yyabort
 #define YYREJECT goto yyabort
 #define YYACCEPT goto yyaccept
-#define YYERROR goto yyerrlab
+#define YYERROR  goto yyerrlab
+
 int
-yyparse(void)
+YYPARSE_DECL()
 {
     int yym, yyn, yystate;
 #if YYDEBUG
     const char *yys;
 
-    if ((yys = getenv("YYDEBUG")) != NULL)
+    if ((yys = getenv("YYDEBUG")) != 0)
     {
         yyn = *yys;
         if (yyn >= '0' && yyn <= '9')
@@ -1083,12 +1098,14 @@ yyparse(void)
 
     yynerrs = 0;
     yyerrflag = 0;
-    yychar = (-1);
+    yychar = YYEMPTY;
+    yystate = 0;
 
     if (yyss == NULL && yygrowstack()) goto yyoverflow;
     yyssp = yyss;
     yyvsp = yyvs;
-    *yyssp = yystate = 0;
+    yystate = 0;
+    *yyssp = 0;
 
 yyloop:
     if ((yyn = yydefred[yystate]) != 0) goto yyreduce;
@@ -1118,9 +1135,10 @@ yyloop:
         {
             goto yyoverflow;
         }
-        *++yyssp = yystate = yytable[yyn];
+        yystate = yytable[yyn];
+        *++yyssp = yytable[yyn];
         *++yyvsp = yylval;
-        yychar = (-1);
+        yychar = YYEMPTY;
         if (yyerrflag > 0)  --yyerrflag;
         goto yyloop;
     }
@@ -1131,12 +1149,14 @@ yyloop:
         goto yyreduce;
     }
     if (yyerrflag) goto yyinrecovery;
-    goto yynewerror;
-yynewerror:
+
     yyerror("syntax error");
+
     goto yyerrlab;
+
 yyerrlab:
     ++yynerrs;
+
 yyinrecovery:
     if (yyerrflag < 3)
     {
@@ -1155,7 +1175,8 @@ yyinrecovery:
                 {
                     goto yyoverflow;
                 }
-                *++yyssp = yystate = yytable[yyn];
+                yystate = yytable[yyn];
+                *++yyssp = yytable[yyn];
                 *++yyvsp = yylval;
                 goto yyloop;
             }
@@ -1185,9 +1206,10 @@ yyinrecovery:
                     YYPREFIX, yystate, yychar, yys);
         }
 #endif
-        yychar = (-1);
+        yychar = YYEMPTY;
         goto yyloop;
     }
+
 yyreduce:
 #if YYDEBUG
     if (yydebug)
@@ -1195,42 +1217,45 @@ yyreduce:
                 YYPREFIX, yystate, yyn, yyrule[yyn]);
 #endif
     yym = yylen[yyn];
-    yyval = yyvsp[1-yym];
+    if (yym)
+        yyval = yyvsp[1-yym];
+    else
+        memset(&yyval, 0, sizeof yyval);
     switch (yyn)
     {
 case 3:
-#line 218 "getdate.y"
-{
+#line 220 "getdate.y"
+	{
 	    yyHaveTime++;
 	}
 break;
 case 4:
-#line 221 "getdate.y"
-{
+#line 223 "getdate.y"
+	{
 	    yyHaveZone++;
 	}
 break;
 case 5:
-#line 224 "getdate.y"
-{
+#line 226 "getdate.y"
+	{
 	    yyHaveDate++;
 	}
 break;
 case 6:
-#line 227 "getdate.y"
-{
+#line 229 "getdate.y"
+	{
 	    yyHaveDay++;
 	}
 break;
 case 7:
-#line 230 "getdate.y"
-{
+#line 232 "getdate.y"
+	{
 	    yyHaveRel++;
 	}
 break;
 case 9:
-#line 236 "getdate.y"
-{
+#line 238 "getdate.y"
+	{
 	    yyHour = yyvsp[-1].Number;
 	    yyMinutes = 0;
 	    yySeconds = 0;
@@ -1238,8 +1263,8 @@ case 9:
 	}
 break;
 case 10:
-#line 242 "getdate.y"
-{
+#line 244 "getdate.y"
+	{
 	    yyHour = yyvsp[-3].Number;
 	    yyMinutes = yyvsp[-1].Number;
 	    yySeconds = 0;
@@ -1247,8 +1272,8 @@ case 10:
 	}
 break;
 case 11:
-#line 248 "getdate.y"
-{
+#line 250 "getdate.y"
+	{
 	    yyHour = yyvsp[-3].Number;
 	    yyMinutes = yyvsp[-1].Number;
 	    yyMeridian = MER24;
@@ -1259,8 +1284,8 @@ case 11:
 	}
 break;
 case 12:
-#line 257 "getdate.y"
-{
+#line 259 "getdate.y"
+	{
 	    yyHour = yyvsp[-5].Number;
 	    yyMinutes = yyvsp[-3].Number;
 	    yySeconds = yyvsp[-1].Number;
@@ -1268,8 +1293,8 @@ case 12:
 	}
 break;
 case 13:
-#line 263 "getdate.y"
-{
+#line 265 "getdate.y"
+	{
 	    yyHour = yyvsp[-5].Number;
 	    yyMinutes = yyvsp[-3].Number;
 	    yySeconds = yyvsp[-1].Number;
@@ -1277,8 +1302,8 @@ case 13:
 	}
 break;
 case 14:
-#line 269 "getdate.y"
-{
+#line 271 "getdate.y"
+	{
 	    yyHour = yyvsp[-5].Number;
 	    yyMinutes = yyvsp[-3].Number;
 	    yySeconds = yyvsp[-1].Number;
@@ -1286,8 +1311,8 @@ case 14:
 	}
 break;
 case 15:
-#line 275 "getdate.y"
-{
+#line 277 "getdate.y"
+	{
 	    yyHour = yyvsp[-6].Number;
 	    yyMinutes = yyvsp[-4].Number;
 	    yySeconds = yyvsp[-2].Number;
@@ -1296,8 +1321,8 @@ case 15:
 	}
 break;
 case 16:
-#line 282 "getdate.y"
-{
+#line 284 "getdate.y"
+	{
 	    yyHour = yyvsp[-5].Number;
 	    yyMinutes = yyvsp[-3].Number;
 	    yySeconds = yyvsp[-1].Number;
@@ -1309,26 +1334,26 @@ case 16:
 	}
 break;
 case 17:
-#line 294 "getdate.y"
-{
+#line 296 "getdate.y"
+	{
 	    yyTimezone = yyvsp[0].Number;
 	}
 break;
 case 18:
-#line 297 "getdate.y"
-{
+#line 299 "getdate.y"
+	{
 	    yyTimezone = yyvsp[0].Number - 60;
 	}
 break;
 case 19:
-#line 301 "getdate.y"
-{
+#line 303 "getdate.y"
+	{
 	    yyTimezone = yyvsp[-1].Number - 60;
 	}
 break;
 case 20:
-#line 306 "getdate.y"
-{
+#line 308 "getdate.y"
+	{
 	    /* ISO 8601 format.  +02:00 -- RAM */
 
 		yyTimezone = yyvsp[-2].Number < 0
@@ -1337,36 +1362,36 @@ case 20:
 	}
 break;
 case 21:
-#line 315 "getdate.y"
-{
+#line 317 "getdate.y"
+	{
 	    yyDayOrdinal = 1;
 	    yyDayNumber = yyvsp[0].Number;
 	}
 break;
 case 22:
-#line 319 "getdate.y"
-{
+#line 321 "getdate.y"
+	{
 	    yyDayOrdinal = 1;
 	    yyDayNumber = yyvsp[-1].Number;
 	}
 break;
 case 23:
-#line 323 "getdate.y"
-{
+#line 325 "getdate.y"
+	{
 	    yyDayOrdinal = yyvsp[-1].Number;
 	    yyDayNumber = yyvsp[0].Number;
 	}
 break;
 case 24:
-#line 329 "getdate.y"
-{
+#line 331 "getdate.y"
+	{
 	    yyMonth = yyvsp[-2].Number;
 	    yyDay = yyvsp[0].Number;
 	}
 break;
 case 25:
-#line 333 "getdate.y"
-{
+#line 335 "getdate.y"
+	{
 	  /* Interpret as YYYY/MM/DD if $1 >= 1000, otherwise as DD/MM/YYYY.
 	     The goal in recognizing YYYY/MM/DD is solely to support legacy
 	     machine-generated dates like those in an RCS log listing.  If
@@ -1386,8 +1411,8 @@ case 25:
 	}
 break;
 case 26:
-#line 351 "getdate.y"
-{
+#line 353 "getdate.y"
+	{
 	    /* ISO 8601 format.  yyyy-mm-dd.  */
 	    yyYear = yyvsp[-2].Number;
 	    yyMonth = -yyvsp[-1].Number;
@@ -1395,8 +1420,8 @@ case 26:
 	}
 break;
 case 27:
-#line 357 "getdate.y"
-{
+#line 359 "getdate.y"
+	{
 	    /* ISO 8601 format.  yyyy-mm-ddT -- RAM */
 	    yyYear = yyvsp[-2].Number;
 	    yyMonth = -yyvsp[-1].Number;
@@ -1404,8 +1429,8 @@ case 27:
 	}
 break;
 case 28:
-#line 363 "getdate.y"
-{
+#line 365 "getdate.y"
+	{
 	    /* e.g. 17-JUN-1992.  */
 	    yyDay = yyvsp[-2].Number;
 	    yyMonth = yyvsp[-1].Number;
@@ -1413,38 +1438,38 @@ case 28:
 	}
 break;
 case 29:
-#line 369 "getdate.y"
-{
+#line 371 "getdate.y"
+	{
 	    yyMonth = yyvsp[-1].Number;
 	    yyDay = yyvsp[0].Number;
 	}
 break;
 case 30:
-#line 373 "getdate.y"
-{
+#line 375 "getdate.y"
+	{
 	    yyMonth = yyvsp[-3].Number;
 	    yyDay = yyvsp[-2].Number;
 	    yyYear = yyvsp[0].Number;
 	}
 break;
 case 31:
-#line 378 "getdate.y"
-{
+#line 380 "getdate.y"
+	{
 	    yyMonth = yyvsp[0].Number;
 	    yyDay = yyvsp[-1].Number;
 	}
 break;
 case 32:
-#line 382 "getdate.y"
-{
+#line 384 "getdate.y"
+	{
 	    yyMonth = yyvsp[-1].Number;
 	    yyDay = yyvsp[-2].Number;
 	    yyYear = yyvsp[0].Number;
 	}
 break;
 case 33:
-#line 389 "getdate.y"
-{
+#line 391 "getdate.y"
+	{
 	    yyRelSeconds = -yyRelSeconds;
 	    yyRelMinutes = -yyRelMinutes;
 	    yyRelHour = -yyRelHour;
@@ -1454,116 +1479,116 @@ case 33:
 	}
 break;
 case 35:
-#line 400 "getdate.y"
-{
+#line 402 "getdate.y"
+	{
 	    yyRelYear += yyvsp[-1].Number * yyvsp[0].Number;
 	}
 break;
 case 36:
-#line 403 "getdate.y"
-{
+#line 405 "getdate.y"
+	{
 	    yyRelYear += yyvsp[-1].Number * yyvsp[0].Number;
 	}
 break;
 case 37:
-#line 406 "getdate.y"
-{
+#line 408 "getdate.y"
+	{
 	    yyRelYear++;
 	}
 break;
 case 38:
-#line 409 "getdate.y"
-{
+#line 411 "getdate.y"
+	{
 	    yyRelMonth += yyvsp[-1].Number * yyvsp[0].Number;
 	}
 break;
 case 39:
-#line 412 "getdate.y"
-{
+#line 414 "getdate.y"
+	{
 	    yyRelMonth += yyvsp[-1].Number * yyvsp[0].Number;
 	}
 break;
 case 40:
-#line 415 "getdate.y"
-{
+#line 417 "getdate.y"
+	{
 	    yyRelMonth++;
 	}
 break;
 case 41:
-#line 418 "getdate.y"
-{
+#line 420 "getdate.y"
+	{
 	    yyRelDay += yyvsp[-1].Number * yyvsp[0].Number;
 	}
 break;
 case 42:
-#line 421 "getdate.y"
-{
+#line 423 "getdate.y"
+	{
 	    yyRelDay += yyvsp[-1].Number * yyvsp[0].Number;
 	}
 break;
 case 43:
-#line 424 "getdate.y"
-{
+#line 426 "getdate.y"
+	{
 	    yyRelDay++;
 	}
 break;
 case 44:
-#line 427 "getdate.y"
-{
+#line 429 "getdate.y"
+	{
 	    yyRelHour += yyvsp[-1].Number * yyvsp[0].Number;
 	}
 break;
 case 45:
-#line 430 "getdate.y"
-{
+#line 432 "getdate.y"
+	{
 	    yyRelHour += yyvsp[-1].Number * yyvsp[0].Number;
 	}
 break;
 case 46:
-#line 433 "getdate.y"
-{
+#line 435 "getdate.y"
+	{
 	    yyRelHour++;
 	}
 break;
 case 47:
-#line 436 "getdate.y"
-{
+#line 438 "getdate.y"
+	{
 	    yyRelMinutes += yyvsp[-1].Number * yyvsp[0].Number;
 	}
 break;
 case 48:
-#line 439 "getdate.y"
-{
+#line 441 "getdate.y"
+	{
 	    yyRelMinutes += yyvsp[-1].Number * yyvsp[0].Number;
 	}
 break;
 case 49:
-#line 442 "getdate.y"
-{
+#line 444 "getdate.y"
+	{
 	    yyRelMinutes++;
 	}
 break;
 case 50:
-#line 445 "getdate.y"
-{
+#line 447 "getdate.y"
+	{
 	    yyRelSeconds += yyvsp[-1].Number * yyvsp[0].Number;
 	}
 break;
 case 51:
-#line 448 "getdate.y"
-{
+#line 450 "getdate.y"
+	{
 	    yyRelSeconds += yyvsp[-1].Number * yyvsp[0].Number;
 	}
 break;
 case 52:
-#line 451 "getdate.y"
-{
+#line 453 "getdate.y"
+	{
 	    yyRelSeconds++;
 	}
 break;
 case 53:
-#line 457 "getdate.y"
-{
+#line 459 "getdate.y"
+	{
 	    if (yyHaveTime && yyHaveDate && !yyHaveRel)
 	      yyYear = yyvsp[0].Number;
 	    else
@@ -1595,18 +1620,18 @@ case 53:
 	  }
 break;
 case 54:
-#line 490 "getdate.y"
-{
+#line 492 "getdate.y"
+	{
 	    yyval.Meridian = MER24;
 	  }
 break;
 case 55:
-#line 494 "getdate.y"
-{
+#line 496 "getdate.y"
+	{
 	    yyval.Meridian = yyvsp[0].Meridian;
 	  }
 break;
-#line 1610 "y.tab.c"
+#line 1634 "y.tab.c"
     }
     yyssp -= yym;
     yystate = *yyssp;
@@ -1653,13 +1678,16 @@ to state %d\n", YYPREFIX, *yyssp, yystate);
     {
         goto yyoverflow;
     }
-    *++yyssp = yystate;
+    *++yyssp = (short) yystate;
     *++yyvsp = yyval;
     goto yyloop;
+
 yyoverflow:
     yyerror("yacc stack overflow");
+
 yyabort:
     return (1);
+
 yyaccept:
     return (0);
 }
