@@ -1,5 +1,5 @@
 /*
- * $Id: getline.h 16273 2009-03-28 17:21:01Z rmanfredi $
+ * $Id: getline.h 16369 2009-04-02 16:54:32Z cbiere $
  *
  * Copyright (c) 2001-2003, Raphael Manfredi
  *
@@ -47,7 +47,7 @@
 typedef enum getline_result {
 	READ_MORE,		/**< OK, expecting more */
 	READ_DONE,		/**< OK, got whole line */
-	READ_OVERFLOW,	/**< Reached max line size */
+	READ_OVERFLOW	/**< Reached max line size */
 } getline_result_t;
 
 /**
