@@ -1,5 +1,5 @@
 /*
- * $Id: glib-missing.c 16247 2009-03-26 23:03:44Z rmanfredi $
+ * $Id: glib-missing.c 17164 2009-11-03 19:57:25Z rmanfredi $
  *
  * Copyright (c) 2003, Raphael Manfredi
  *
@@ -41,204 +41,61 @@
 
 #include "common.h"
 
-RCSID("$Id: glib-missing.c 16247 2009-03-26 23:03:44Z rmanfredi $")
+RCSID("$Id: glib-missing.c 17164 2009-11-03 19:57:25Z rmanfredi $")
 
 #include "glib-missing.h"
 #include "ascii.h"
 #include "iovec.h"
-#include "misc.h"
+#include "unsigned.h"
 #include "utf8.h"
 
 #include "override.h"		/* Must be the last header included */
 
-#if defined(USE_GLIB1) && !defined(TRACK_MALLOC) && defined(USE_HALLOC)
-static GMemVTable gm_vtable;
-
-#define GM_VTABLE_METHOD(method, params) \
-	(gm_vtable.gmvt_ ## method \
-	 ? (gm_vtable.gmvt_ ## method params) \
-	 : (method params))
-
-static inline ALWAYS_INLINE gpointer
-gm_malloc(gulong size)
+#if !defined(HAS_STRLCPY) && !defined(USE_GLIB2)
+size_t
+strlcpy(char *dst, const char *src, size_t dst_size)
 {
-	return GM_VTABLE_METHOD(malloc, (size));
-}
+	char *d = dst;
+	const char *s = src;
 
-static inline ALWAYS_INLINE gpointer
-gm_malloc0(gulong size)
-{
-	return GM_VTABLE_METHOD(calloc, (1, size));
-}
+	g_assert(NULL != dst);
+	g_assert(NULL != src);
 
-static inline ALWAYS_INLINE gpointer
-gm_realloc(gpointer p, gulong size)
-{
-	return GM_VTABLE_METHOD(realloc, (p, size));
-}
+	if (dst_size--) {
+		size_t i = 0;
 
-static inline ALWAYS_INLINE void
-gm_free(gpointer p)
-{
-	return GM_VTABLE_METHOD(free, (p));
-}
-
-#define try_malloc malloc
-static inline ALWAYS_INLINE gpointer
-gm_try_malloc(gulong size)
-{
-	return GM_VTABLE_METHOD(try_malloc, (size));
-}
-#undef try_malloc
-
-#define try_realloc realloc
-static inline ALWAYS_INLINE gpointer
-gm_try_realloc(gpointer p, gulong size)
-{
-	return GM_VTABLE_METHOD(try_realloc, (p, size));
-}
-#undef try_realloc
-
-/***
- *** Remap g_malloc() and friends to be able to emulate g_mem_set_vtable()
- *** with GTK1.  Fortunately, glib1.x placed the allocation routines in
- *** a dedicated mem.o file, so we may safely redefine them here.
- ***
- *** NOTE: This a hack and does not work on some platforms.
- ***/
-
-#undef g_malloc
-gpointer
-g_malloc(gulong size)
-{
-	gpointer p;
-
-	if (size == 0)
-		return NULL;
-
-	p = gm_malloc(size);
-
-	if (p)
-		return p;
-
-	g_error("allocation of %lu bytes failed", size);
-	return NULL;
-}
-
-#undef g_malloc0
-gpointer
-g_malloc0(gulong size)
-{
-	gpointer p;
-
-	if (size == 0)
-		return NULL;
-
-	p = gm_malloc(size);
-
-	if (p) {
-		memset(p, 0, size);
-		return p;
+		while (i < dst_size) {
+			if (!(*d++ = *s++))
+				return i;
+			i++;
+		}
+		dst[dst_size] = '\0';
 	}
-
-	g_error("allocation of %lu bytes failed", size);
-	return NULL;
+ 	while (*s)
+		s++;
+	return s - src;
 }
+#endif /* HAS_STRLCPY */
 
-#undef g_realloc
-gpointer
-g_realloc(gpointer p, gulong size)
+#if !defined(HAS_STRLCAT) && !defined(USE_GLIB2)
+size_t
+strlcat(char *dst, const char *src, size_t dst_size)
 {
-	gpointer n;
+	size_t n;
+	
+	g_assert(NULL != dst);
+	g_assert(NULL != src);
 
-	if (size == 0) {
-		gm_free(p);
-		return NULL;
+	n = strlen(dst);	
+	if (n < dst_size) {
+		dst_size -= n;
+	} else if (dst_size > 0) {
+		dst[dst_size - 1] = '\0';
+		dst_size = 0;
 	}
-
-	n = gm_realloc(p, size);
-
-	if (n)
-		return n;
-
-	g_error("re-allocation of %lu bytes failed", size);
-	return NULL;
+	return n += g_strlcpy(&dst[n], src, dst_size);
 }
-
-#undef g_free
-void
-g_free(gpointer p)
-{
-	gm_free(p);
-}
-
-#undef g_try_malloc
-gpointer
-g_try_malloc(gulong size)
-{
-	return size > 0 ? gm_try_malloc(size) : NULL;
-}
-
-#undef g_try_realloc
-gpointer
-g_try_realloc(gpointer p, gulong size)
-{
-	return size > 0 ? gm_try_realloc(p, size) : NULL;
-}
-
-/**
- * Emulates a calloc().
- */
-static gpointer
-emulate_calloc(gsize n, gsize m)
-{
-	gpointer p;
-
-	if (n > 0 && m > 0 && m < ((size_t) -1) / n) {
-		size_t size = n * m;
-		p = gm_malloc(size);
-		memset(p, 0, size);
-	} else {
-		p = NULL;
-	}
-	return p;
-}
-
-/**
- * Sets the GMemVTable to use for memory allocation.
- * This function must be called before using any other GLib functions.
- *
- * The vtable only needs to provide malloc(), realloc(), and free() functions;
- * GLib can provide default implementations of the others.
- * The malloc() and realloc() implementations should return NULL on failure, 
- */
-void
-g_mem_set_vtable(GMemVTable *vtable)
-{
-	gm_vtable.gmvt_malloc = vtable->gmvt_malloc;
-	gm_vtable.gmvt_realloc = vtable->gmvt_realloc;
-	gm_vtable.gmvt_free = vtable->gmvt_free;
-
-	gm_vtable.gmvt_calloc = vtable->gmvt_calloc
-		? vtable->gmvt_calloc
-		: emulate_calloc;
-	gm_vtable.gmvt_try_malloc = vtable->gmvt_try_malloc
-		? vtable->gmvt_try_malloc
-		: vtable->gmvt_malloc;
-	gm_vtable.gmvt_try_realloc = vtable->gmvt_try_realloc
-		? vtable->gmvt_try_realloc
-		: vtable->gmvt_realloc;
-}
-
-/**
- * Are we using system's malloc?
- */
-gboolean
-g_mem_is_system_malloc(void)
-{
-	return !gm_vtable.gmvt_malloc;
-}
-#endif	/* USE_GLIB1 */
+#endif /* HAS_STRLCAT */
 
 #ifndef TRACK_MALLOC
 /**
@@ -351,6 +208,19 @@ g_hash_table_replace(GHashTable *ht, gpointer key, gpointer value)
 	g_hash_table_remove(ht, key);
 	g_hash_table_insert(ht, key, value);
 }
+
+gboolean
+gm_hash_table_remove(GHashTable *ht, gconstpointer key)
+{
+	/* In glib 1.x, g_hash_table_remove() does not return anything */
+
+	if (g_hash_table_lookup(ht, key)) {
+		g_hash_table_remove(ht, key);
+		return TRUE;
+	}
+
+	return FALSE;
+}
 #endif	/* USE_GLIB1 */
 
 /**
@@ -393,9 +263,9 @@ buf_vprintf(char *dst, size_t size, const char *fmt, va_list args)
   
 	g_assert(size > 0);	
 	buf	= g_strdup_vprintf(fmt, args);
-	len = g_strlcpy(dst, buf, size);
+	len = clamp_strcpy(dst, size, buf);
 	G_FREE_NULL(buf);
-	return MIN((size - 1), len);
+	return len;
 }
 #endif	/* HAS_VSNPRINTF */
 
@@ -602,147 +472,6 @@ g_string_append_len(GString *gs, const char *val, gssize len)
 }
 #endif	/* USE_GLIB1 */
 
-/**
- * Creates a valid and sanitized filename from the supplied string. For most
- * Unix-like platforms anything goes but for security reasons, shell meta
- * characters are replaced by harmless characters.
- *
- * @param filename the suggested filename.
- * @param no_spaces if TRUE, spaces are replaced with underscores.
- * @param no_evil if TRUE, "evil" characters are replaced with underscores.
- *
- * @returns a newly allocated string or ``filename'' if it was a valid filename
- *		    already.
- */
-char *
-gm_sanitize_filename(const char *filename,
-		gboolean no_spaces, gboolean no_evil)
-{
-	const char *s;
-	char *q;
-
-	g_assert(filename);
-
-	/* Make sure the filename isn't too long */
-	if (strlen(filename) >= FILENAME_MAXBYTES) {
-		q = g_malloc(FILENAME_MAXBYTES);
-		filename_shrink(filename, q, FILENAME_MAXBYTES);
-		s = q;
-	} else {
-		s = filename;
-		q = NULL;
-	}
-
-	/* Replace shell meta characters and likely problematic characters */
-	{
-		static const char evil[] = "$&*\\`:;()'\"<>?|~\177";
-		size_t i;
-		guchar c;
-		
-		for (i = 0; '\0' != (c = s[i]); i++) {
-			if (
-				c < 32
-				|| is_ascii_cntrl(c)
-				|| G_DIR_SEPARATOR == c
-				|| '/' == c 
-				|| (0 == i && ('.' == c || '-' == c))
-				|| (no_spaces && is_ascii_space(c))
-				|| (no_evil && NULL != strchr(evil, c))
-		   ) {
-				if (!q)
-					q = g_strdup(s);
-				q[i] = '_';
-			}
-		}
-	}
-
-	return q ? q : deconstify_gchar(s);
-}
-
-/**
- * Make filename prettier, by removing leading "_", making sure the filename
- * does not start with "-" or ".", and stripping consecutive "_" or "_" that
- * surround a punctuation character.
- *
- * Finally, ensure the filename is not completely empty, as this is
- * awkward to manipulate from a shell.
- *
- * @param filename	the filename to beautify
- *
- * @returns a newly allocated string holding the beautified filename, even if
- * it is a mere copy of the original.
- */
-char *
-gm_beautify_filename(const char *filename)
-{
-	const char *s;
-	char *q;
-	guchar c;
-	size_t len;
-	size_t j = 0;
-	static const char punct[] = "_-+=.,<>{}[]";	/* 1st MUST be '_' */
-	static const char strip[] = "_-.";
-	static const char empty[] = "{empty}";
-
-	g_assert(filename);
-
-	s = filename;
-	len = strlen(filename);
-	q = g_malloc(len + 1);		/* Trailing NUL */
-
-	while ((c = *s++)) {
-		guchar d;
-
-		/* Beautified filename cannot start with stripped characters */
-		if (j == 0) {
-			if (NULL == strchr(strip, c))
-				q[j++] = c;
-			continue;
-		}
-
-		g_assert(j > 0);
-
-		d = q[j - 1];		/* Last char we've kept in beautified name */
-
-		/* A "_" followed by a punctuation character, strip the "_" */
-		if (d == '_' && NULL != strchr(punct, c)) {
-			q[j - 1] = c;
-			continue;
-		}
-
-		/* A punctuation character followed by "_", ignore that "_" */
-		if (NULL != strchr(&punct[1], d) && c == '_')
-			continue;
-
-		q[j++] = c;
-	}
-
-	g_assert(j <= len);
-	q[j] = '\0';
-
-	/* Ensure we have no empty name */
-	if (j == 0) {
-		G_FREE_NULL(q);
-		return g_strdup(empty);
-	}
-
-	/*
-	 * If there was an extension following stripped chars (e.g. "_.ext"),
-	 * then the filename kept will become "ext" (we assume a valid extension
-	 * cannot contain "escaped" chars).  In which case we will prepend the
-	 * string "{empty}." to it.
-	 */
-
-	if (NULL == strchr(q, '.') && j < len && '.' == filename[len - j]) {
-		char *r = g_strconcat(empty, ".", q, (void *) 0);
-		G_FREE_NULL(q);
-
-		return r;
-	}
-
-	return q;
-}
-
 /**
  * Frees the GString context but keeps the string data itself and returns
  * it. With Gtk+ 2.x g_string_free(gs, FALSE) would do the job but the
@@ -912,4 +641,269 @@ g_list_sort_with_data(
 }
 #endif	/* USE_GLIB1 */
 
+/***
+ *** This set of routines must be kept at the tail of the file because they
+ *** undefine malloc, g_malloc, etc... which are possibly set up by override.h.
+ ***/
+
+#if defined(USE_GLIB1) && \
+	(defined(USE_HALLOC) || defined(TRACK_MALLOC) || defined(TRACK_ZALLOC) || \
+		defined(REMAP_ZALLOC))
+
+static GMemVTable gm_vtable;
+
+#define GM_VTABLE_METHOD(method, params) \
+	(gm_vtable.gmvt_ ## method \
+	 ? (gm_vtable.gmvt_ ## method params) \
+	 : (method params))
+
+#undef malloc
+static inline ALWAYS_INLINE gpointer
+gm_malloc(gulong size)
+{
+	return GM_VTABLE_METHOD(malloc, (size));
+}
+
+#undef calloc
+static inline ALWAYS_INLINE gpointer
+gm_malloc0(gulong size)
+{
+	return GM_VTABLE_METHOD(calloc, (1, size));
+}
+
+#undef realloc
+static inline ALWAYS_INLINE gpointer
+gm_realloc(gpointer p, gulong size)
+{
+	return GM_VTABLE_METHOD(realloc, (p, size));
+}
+
+#undef free
+static inline ALWAYS_INLINE void
+gm_free(gpointer p)
+{
+	return GM_VTABLE_METHOD(free, (p));
+}
+
+#define try_malloc malloc
+static inline ALWAYS_INLINE gpointer
+gm_try_malloc(gulong size)
+{
+	return GM_VTABLE_METHOD(try_malloc, (size));
+}
+#undef try_malloc
+
+#define try_realloc realloc
+static inline ALWAYS_INLINE gpointer
+gm_try_realloc(gpointer p, gulong size)
+{
+	return GM_VTABLE_METHOD(try_realloc, (p, size));
+}
+#undef try_realloc
+
+/***
+ *** Remap g_malloc() and friends to be able to emulate g_mem_set_vtable()
+ *** with GTK1.  Fortunately, glib1.x placed the allocation routines in
+ *** a dedicated mem.o file, so we may safely redefine them here.
+ ***
+ *** NOTE: This a hack and does not work on some platforms.
+ ***/
+
+#undef g_malloc
+gpointer
+g_malloc(gulong size)
+{
+	if (G_LIKELY(size != 0)) {
+		gpointer p = gm_malloc(size);
+
+		if (p)
+			return p;
+
+		g_error("allocation of %lu bytes failed", size);
+	}
+	return NULL;
+}
+
+#undef g_malloc0
+gpointer
+g_malloc0(gulong size)
+{
+
+	if (G_LIKELY(size != 0)) {
+		gpointer p = gm_malloc(size);
+
+		if (p) {
+			memset(p, 0, size);
+			return p;
+		}
+
+		g_error("allocation of %lu bytes failed", size);
+	}
+	return NULL;
+}
+
+#undef g_realloc
+gpointer
+g_realloc(gpointer p, gulong size)
+{
+	gpointer n;
+
+	if (G_UNLIKELY(0 == size)) {
+		gm_free(p);
+		return NULL;
+	}
+
+	n = gm_realloc(p, size);
+
+	if (n)
+		return n;
+
+	g_error("re-allocation of %lu bytes failed", size);
+	return NULL;
+}
+
+#undef g_free
+void
+g_free(gpointer p)
+{
+	gm_free(p);
+}
+
+#undef g_try_malloc
+gpointer
+g_try_malloc(gulong size)
+{
+	return size > 0 ? gm_try_malloc(size) : NULL;
+}
+
+#undef g_try_realloc
+gpointer
+g_try_realloc(gpointer p, gulong size)
+{
+	return size > 0 ? gm_try_realloc(p, size) : NULL;
+}
+
+/**
+ * Emulates a calloc().
+ */
+static gpointer
+emulate_calloc(gsize n, gsize m)
+{
+	gpointer p;
+
+	if (n > 0 && m > 0 && m < ((size_t) -1) / n) {
+		size_t size = n * m;
+		p = gm_malloc(size);
+		memset(p, 0, size);
+	} else {
+		p = NULL;
+	}
+	return p;
+}
+
+/**
+ * Sets the GMemVTable to use for memory allocation.
+ * This function must be called before using any other GLib functions.
+ *
+ * The vtable only needs to provide malloc(), realloc(), and free() functions;
+ * GLib can provide default implementations of the others.
+ * The malloc() and realloc() implementations should return NULL on failure, 
+ */
+void
+g_mem_set_vtable(GMemVTable *vtable)
+{
+	gm_vtable.gmvt_malloc = vtable->gmvt_malloc;
+	gm_vtable.gmvt_realloc = vtable->gmvt_realloc;
+	gm_vtable.gmvt_free = vtable->gmvt_free;
+
+	gm_vtable.gmvt_calloc = vtable->gmvt_calloc
+		? vtable->gmvt_calloc
+		: emulate_calloc;
+	gm_vtable.gmvt_try_malloc = vtable->gmvt_try_malloc
+		? vtable->gmvt_try_malloc
+		: vtable->gmvt_malloc;
+	gm_vtable.gmvt_try_realloc = vtable->gmvt_try_realloc
+		? vtable->gmvt_try_realloc
+		: vtable->gmvt_realloc;
+}
+
+/**
+ * Are we using system's malloc?
+ */
+gboolean
+g_mem_is_system_malloc(void)
+{
+	return NULL == gm_vtable.gmvt_malloc ||
+		cast_pointer_to_func(gm_vtable.gmvt_malloc) ==
+			cast_pointer_to_func(real_malloc) ||
+		cast_pointer_to_func(gm_vtable.gmvt_malloc) ==
+			cast_pointer_to_func(malloc);
+}
+#endif	/* USE_GLIB1 */
+
+/**
+ * Safe reallocation routine during final memory cleanup.
+ */
+static inline void *
+safe_realloc(void *p, size_t len)
+{
+	if (NULL == p) {
+		return malloc(len);
+	} else if (0 == len) {
+		/* NOTHING */
+	} else {
+		g_error("no realloc() allowed during final memory cleanup");
+	}
+
+	return NULL;
+}
+
+/**
+ * Safe free routine during final memory cleanup.
+ */
+static inline void
+safe_free(void *unused_p)
+{
+	(void) unused_p;
+	/* NOTHING */
+}
+
+/**
+ * Install safe memory vtable for final memory cleanup.
+ *
+ * When the memory vtable has been customized, redirecting g_malloc() to
+ * some other routine like halloc(), we can't easily perform final shutdown
+ * of the zalloc() and walloc() memory allocators because any call to
+ * log something still present could allocate memory and reenter code that
+ * is using the data structures being cleaned up.
+ *
+ * At this time though, we don't really care about freeing allocated memory
+ * since we're about to exit, but we want to be able to allocate new one
+ * safely.
+ */
+void
+gm_mem_set_safe_vtable(void)
+{
+#if defined(USE_HALLOC) || defined(TRACK_MALLOC) || defined(TRACK_ZALLOC) || \
+		defined(REMAP_ZALLOC)
+	static GMemVTable vtable;
+
+	if (g_mem_is_system_malloc())
+		return;
+
+#if GLIB_CHECK_VERSION(2,0,0)
+	vtable.malloc = real_malloc;
+	vtable.realloc = safe_realloc;
+	vtable.free = safe_free;
+#else	/* GLib < 2.0.0 */
+	vtable.gmvt_malloc = real_malloc;
+	vtable.gmvt_realloc = safe_realloc;
+	vtable.gmvt_free = safe_free;
+#endif	/* GLib >= 2.0.0 */
+
+	g_mem_set_vtable(&vtable);
+
+#endif	/* USE_HALLOC || TRACK_MALLOC || TRACK_ZALLOC || REMAP_ZALLOC */
+}
+
 /* vi: set ts=4 sw=4 cindent: */
