@@ -1,5 +1,5 @@
 /*
- * $Id: halloc.h 12568 2006-12-04 14:54:49Z cbiere $
+ * $Id: halloc.h 17187 2009-11-04 19:48:41Z bpringlemeir $
  *
  * Copyright (c) 2006, Christian Biere
  *
@@ -39,36 +39,56 @@
 #include "common.h"
 
 /*
- * Under REMAP_ZALLOC control, those routines are remapped to malloc/free.
+ * Under TRACK_ZALLOC we remap halloc() and halloc0() in case these routines
+ * perform their allocation through zalloc().
  */
 
-#if defined(USE_DMALLOC) && !defined(REMAP_ZALLOC)
-#define REMAP_ZALLOC
-#endif
-
-#ifdef REMAP_ZALLOC
-
-static inline void *halloc(size_t n)	{ return g_malloc(n); }
-static inline void *halloc0(size_t n)	{ return g_malloc0(n); }
-static inline void hfree(void *p)		{ return g_free(p); }
-
-static inline void *
-hrealloc(void *p, size_t n)
-{
-	return g_realloc(p, n);
-}
-
-#else	/* !REMAP_ZALLOC */
+#ifdef TRACK_ZALLOC
+#define halloc(_s)	halloc_track(_s, _WHERE_, __LINE__)
+#define halloc0(_s)	halloc0_track(_s, _WHERE_, __LINE__)
 
+void *halloc_track(size_t size,
+	const char *file, int line) WARN_UNUSED_RESULT G_GNUC_MALLOC;
+void *halloc0_track(size_t size,
+	const char *file, int line) WARN_UNUSED_RESULT G_GNUC_MALLOC;
+#else
+#ifndef TRACK_MALLOC
 void *halloc(size_t size) WARN_UNUSED_RESULT G_GNUC_MALLOC;
 void *halloc0(size_t size) WARN_UNUSED_RESULT G_GNUC_MALLOC;
+#endif
+#endif	/* TRACK_ZALLOC */
+
+/*
+ * Under TRACK_MALLOC control, these routines are remapped to malloc()/free().
+ */
+
+#ifndef TRACK_MALLOC
 void hfree(void *ptr);
-void *hrealloc(void *old, size_t size) WARN_UNUSED_RESULT G_GNUC_MALLOC;
+void *hrealloc(void *old, size_t size) WARN_UNUSED_RESULT;
 
-#endif	/* REMAP_ZALLOC */
+static inline void *
+hcopy(const void *p, size_t size)
+{
+	void *cp = halloc(size);
+	memcpy(cp, p, size);
+	return cp;
+}
 
-void halloc_init(void);
+char *h_strdup(const char *str) WARN_UNUSED_RESULT
+	G_GNUC_MALLOC;
+char *h_strndup(const char *str, size_t n) WARN_UNUSED_RESULT
+	G_GNUC_MALLOC;
+char *h_strjoinv(const char *separator, char **str_array);
+void h_strfreev(char **str_array);
+char *h_strconcat(const char *str1, ...) WARN_UNUSED_RESULT G_GNUC_MALLOC 
+	G_GNUC_NULL_TERMINATED;
+char *h_strdup_printf(const char *format, ...) G_GNUC_PRINTF(1, 2);
+#endif	/* !TRACK_MALLOC */
+
+void halloc_init(gboolean replace_malloc);
 void hdestroy(void);
+gboolean halloc_replaces_malloc(void);
+gboolean halloc_is_available(void);
 
 size_t halloc_bytes_allocated(void);
 size_t halloc_chunks_allocated(void);
