@@ -1,5 +1,5 @@
 /*
- * $Id: host_addr.c 16108 2009-03-10 15:19:52Z rmanfredi $
+ * $Id: host_addr.c 16990 2009-09-25 15:23:43Z rmanfredi $
  *
  * Copyright (c) 2001-2003, Raphael Manfredi
  * Copyright (c) 2005, Christian Biere
@@ -36,7 +36,7 @@
 
 #include "common.h"
 
-RCSID("$Id: host_addr.c 16108 2009-03-10 15:19:52Z rmanfredi $")
+RCSID("$Id: host_addr.c 16990 2009-09-25 15:23:43Z rmanfredi $")
 
 #ifdef I_NET_IF
 #include <net/if.h>		/* For IFF_* flags */
@@ -53,8 +53,11 @@ RCSID("$Id: host_addr.c 16108 2009-03-10 15:19:52Z rmanfredi $")
 #include "ascii.h"
 #include "atoms.h"				/* For binary_hash */
 #include "host_addr.h"
-#include "misc.h"
+#include "concat.h"
 #include "endian.h"
+#include "glib-missing.h"		/* For g_strlcpy() */
+#include "parse.h"
+#include "stringify.h"
 #include "walloc.h"
 
 #include "override.h"		/* Must be the last header included */
@@ -1124,6 +1127,9 @@ host_addr_eq_func(gconstpointer p, gconstpointer q)
 	return host_addr_equal(*a, *b);
 }
 
+/**
+ * Aging table callback.
+ */
 void
 wfree_host_addr(gpointer key, gpointer unused_data)
 {
