@@ -1,5 +1,5 @@
 /*
- * $Id: host_addr.h 16108 2009-03-10 15:19:52Z rmanfredi $
+ * $Id: host_addr.h 17028 2009-10-04 15:39:42Z rmanfredi $
  *
  * Copyright (c) 2001-2003, Raphael Manfredi
  * Copyright (c) 2005, Christian Biere
@@ -404,7 +404,7 @@ host_addr_hash(host_addr_t ha)
 		return ha.net;
 	}
 	g_assert_not_reached();
-	return -1;
+	return (guint32) -1;
 }
 
 /**
