@@ -1,5 +1,5 @@
 /*
- * $Id: html.c 15802 2008-09-04 22:29:48Z cbiere $
+ * $Id: html.c 16990 2009-09-25 15:23:43Z rmanfredi $
  *
  * Copyright (c) 2007 Christian Biere
  *
@@ -35,14 +35,16 @@
 
 #include "common.h"
 
-RCSID("$Id: html.c 15802 2008-09-04 22:29:48Z cbiere $")
+RCSID("$Id: html.c 16990 2009-09-25 15:23:43Z rmanfredi $")
 
 #include "ascii.h"
+#include "fd.h"
 #include "html.h"
 #include "html_entities.h"
 #include "misc.h"
 #include "utf8.h"
 #include "walloc.h"
+#include "vmm.h"
 
 #include "override.h"		/* Must be the last header included */
 
@@ -190,7 +192,7 @@ parse_attribute(const struct array attr)
 		len++;
 	}
 
-	if (len > 0 && len < (int)G_N_ELEMENTS(name)) {
+	if (len > 0 && len < G_N_ELEMENTS(name)) {
 		name[len] = '\0';
 		for (i = 0; i < G_N_ELEMENTS(tab); i++) {
 			if (0 == strcmp(name, tab[i].name))
@@ -278,7 +280,7 @@ parse_tag(const struct array tag)
 		len++;
 	}
 
-	if (len > 0 && len < (int)G_N_ELEMENTS(name)) {
+	if (len > 0 && len < G_N_ELEMENTS(name)) {
 		name[len] = '\0';
 		for (i = 0; i < G_N_ELEMENTS(tab); i++) {
 			if (0 == strcmp(name, tab[i].name))
@@ -726,22 +728,19 @@ html_load_file(struct html_output *output, int fd)
 		goto error;
 	}
 	size = sb.st_size;
-	p = mmap(0, size, PROT_READ, MAP_PRIVATE, fd, 0);
+	p = vmm_mmap(0, size, PROT_READ, MAP_PRIVATE, fd, 0);
 	if (MAP_FAILED == p) {
 		perror("open");
 		goto error;
 	}
-	close(fd);
-	fd = -1;
+	fd_close(&fd, FALSE);
 
 	ret = html_load_memory(output, array_init(p, size));
 
 error:
-	if (fd >= 0) {
-		close(fd);
-	}
+	fd_close(&fd, TRUE);
 	if (MAP_FAILED != p) {
-		munmap(p, size);
+		vmm_munmap(p, size);
 	}
 	return ret;
 }
