@@ -1,5 +1,5 @@
 /*
- * $Id: idtable.c 13049 2007-03-11 18:29:58Z cbiere $
+ * $Id: idtable.c 15402 2008-07-09 10:47:26Z rmanfredi $
  *
  * Copyright (c) 2001-2003, Richard Eckart
  *
@@ -23,9 +23,17 @@
  *----------------------------------------------------------------------
  */
 
+/**
+ * @ingroup lib
+ * @file
+ *
+ * @author Richard Eckart
+ * @date 2001
+ */
+
 #include "common.h"
 
-RCSID("$Id: idtable.c 13049 2007-03-11 18:29:58Z cbiere $")
+RCSID("$Id: idtable.c 15402 2008-07-09 10:47:26Z rmanfredi $")
 
 #include "lib/glib-missing.h"
 #include "lib/idtable.h"
@@ -57,7 +65,7 @@ idtable_new(void)
 
 	tbl = walloc(sizeof *tbl);
 	*tbl = zero_idtable;
-	tbl->last_id = (random_raw() & IDTABLE_MASK) + IDTABLE_BASE;
+	tbl->last_id = (random_u32() & IDTABLE_MASK) + IDTABLE_BASE;
 	tbl->ht = g_hash_table_new(NULL, NULL);
 	return tbl;
 }
