@@ -1,5 +1,5 @@
 /*
- * $Id: inputevt.c 16108 2009-03-10 15:19:52Z rmanfredi $
+ * $Id: inputevt.c 16990 2009-09-25 15:23:43Z rmanfredi $
  *
  * Copyright (c) 2002, ko (ko-@wanadoo.fr)
  * Copyright (c) 2005, Christian Biere
@@ -44,7 +44,7 @@
 
 #include "common.h"
 
-RCSID("$Id: inputevt.c 16108 2009-03-10 15:19:52Z rmanfredi $")
+RCSID("$Id: inputevt.c 16990 2009-09-25 15:23:43Z rmanfredi $")
 
 #ifdef HAS_KQUEUE
 #include <sys/event.h>
@@ -96,6 +96,7 @@ struct inputevt_array {
 #endif	/* */
 
 #include "inputevt.h"
+#include "fd.h"
 #include "misc.h"
 #include "walloc.h"
 #include "tm.h"
@@ -1097,7 +1098,15 @@ inputevt_add(int fd, inputevt_cond_t cond,
 void
 inputevt_close(void)
 {
-	/* no cleanup required */
+#if !defined(USE_POLL)
+	struct poll_ctx *poll_ctx;
+	
+	poll_ctx = get_global_poll_ctx();
+	g_hash_table_destroy(poll_ctx->ht);
+	G_FREE_NULL(poll_ctx->used);
+	G_FREE_NULL(poll_ctx->relay);
+	G_FREE_NULL(poll_ctx->ev_arr.ev);
+#endif
 }
 
 /* vi: set ts=4 sw=4 cindent: */
