@@ -1,5 +1,5 @@
 /*
- * $Id: iso3166.c 16108 2009-03-10 15:19:52Z rmanfredi $
+ * $Id: iso3166.c 17288 2010-02-25 17:02:34Z rmanfredi $
  *
  * Copyright (c) 2004, Christian Biere
  *
@@ -35,12 +35,12 @@
 
 #include "common.h"
 
-RCSID("$Id: iso3166.c 16108 2009-03-10 15:19:52Z rmanfredi $")
+RCSID("$Id: iso3166.c 17288 2010-02-25 17:02:34Z rmanfredi $")
 
 #include "ascii.h"
 #include "atoms.h"
 #include "iso3166.h"
-#include "misc.h"
+#include "parse.h"
 
 #include "override.h"       /* Must be the last header included */
 
@@ -188,7 +188,7 @@ static const struct {
 	{ "kz", N_("Kazakhstan") },
 	{ "la", N_("Laos") },
 	{ "lb", N_("Lebanon") },
-	{ "lc", N_("Saint lucia") },
+	{ "lc", N_("Saint Lucia") },
 	{ "li", N_("Liechtenstein") },
 	{ "lk", N_("Sri Lanka") },
 	{ "lr", N_("Liberia") },
@@ -201,6 +201,7 @@ static const struct {
 	{ "mc", N_("Monaco") },
 	{ "md", N_("Moldova") },
 	{ "me", N_("Montenegro") },
+	{ "mf", N_("Saint Martin") },
 	{ "mg", N_("Madagascar") },
 	{ "mh", N_("Marshall Islands") },
 	{ "mk", N_("Macedonia") },
@@ -311,13 +312,12 @@ static const struct {
 
 static iso3166_entry_t iso3166_entries[G_N_ELEMENTS(iso3166_tab)];
 
-#define NUM_CODES (36 * 35 + 35)
-static iso3166_entry_t *iso3166_countries[NUM_CODES];
+static iso3166_entry_t *iso3166_countries[ISO3166_NUM_CODES];
 
 /**
- * Decodes a valid 2-letter country code into an integer.
+ * Decodes a valid country code into a 2-letter country string.
  *
- * @return NULL integer isn't a validly encoded country code. If the country
+ * @return NULL if integer isn't a validly encoded country code. If the country
  *		   is valid, a string pointing two the 2-letter code is returned. The
  *		   string is in a static buffer.
  */
@@ -359,8 +359,11 @@ iso3166_encode_cc(const char *cc)
 		int error;
 
 		code = parse_uint16(cc, NULL, 36, &error);
-        g_assert(code < G_N_ELEMENTS(iso3166_countries));
-        g_assert(0 == error);
+
+		if (code >= G_N_ELEMENTS(iso3166_countries))
+			return ISO3166_INVALID;
+
+		g_assert(0 == error);
 
 		if (NULL != iso3166_countries[code]) {
         	d = iso3166_decode_cc(code);
