@@ -1,5 +1,5 @@
 /*
- * $Id: iso3166.h 16103 2009-03-10 14:53:39Z rmanfredi $
+ * $Id: iso3166.h 17182 2009-11-04 17:37:32Z rmanfredi $
  *
  * Copyright (c) 2004, Christian Biere
  *
@@ -29,6 +29,7 @@
 #include "common.h"
 
 #define ISO3166_INVALID ((guint16)-1)
+#define ISO3166_NUM_CODES (36 * 35 + 35)
 
 guint16 iso3166_encode_cc(const char *cc);
 void iso3166_init(void);
@@ -36,5 +37,11 @@ void iso3166_close(void);
 const char *iso3166_country_name(guint16 code);
 const char *iso3166_country_cc(guint16 code);
 
+static inline gboolean iso3166_code_is_valid(guint16 code)
+{
+	return ISO3166_INVALID == code || code < (unsigned) ISO3166_NUM_CODES;
+}
+
 #endif /* _lib_iso3166_h_ */
+
 /* vi: set ts=4 sw=4 cindent: */
