@@ -1,5 +1,5 @@
 /*
- * $Id: list.c 13882 2007-06-19 00:05:04Z cbiere $
+ * $Id: list.c 16284 2009-03-28 21:11:40Z cbiere $
  *
  * Copyright (c) 2003, Christian Biere
  *
@@ -40,7 +40,7 @@
 
 #include "common.h"
 
-RCSID("$Id: list.c 13882 2007-06-19 00:05:04Z cbiere $")
+RCSID("$Id: list.c 16284 2009-03-28 21:11:40Z cbiere $")
 
 #include "list.h"
 #include "misc.h"
@@ -58,10 +58,10 @@ typedef enum {
 
 struct list {
 	list_magic_t magic;
-	gint refcount;
+	int refcount;
 	GList *head;
 	GList *tail;
-	gint length;
+	int length;
 	guint stamp;
 };
 
@@ -276,7 +276,11 @@ list_remove(list_t *list, gpointer key)
 		if (item == list->tail) {
 			list->tail = g_list_previous(list->tail);
 		}
-		IGNORE_RESULT(g_list_delete_link(item, item));
+		/* @note: Return value is only assigned to "item" because
+	 	 *        g_slist_delete_link is incorrectly tagged to
+	 	 *        cause a GCC compiler warning otherwise.
+	 	 */
+		item = g_list_delete_link(item, item);
 
 		list->length--;
 		list->stamp++;
