@@ -1,5 +1,5 @@
 /*
- * $Id: listener.h 16103 2009-03-10 14:53:39Z rmanfredi $
+ * $Id: listener.h 16468 2009-04-10 13:29:24Z cbiere $
  *
  * Copyright (c) 2001-2003, Richard Eckart
  *
@@ -59,14 +59,14 @@ typedef GSList *listeners_t;
 
 #define LISTENER_ADD(signal, callback) 										\
 G_STMT_START {																\
-	void *p = (callback);			\
+	void *p = cast_func_to_pointer((func_ptr_t) callback);					\
 	g_assert(NULL != p);				 									\
 	CAT2(signal,_listeners) = g_slist_append(CAT2(signal,_listeners), p);	\
 } G_STMT_END
 
 #define LISTENER_REMOVE(signal, callback)									\
 G_STMT_START {																\
-	void *p = (callback);			\
+	void *p = cast_func_to_pointer((func_ptr_t) callback);					\
 	g_assert(NULL != p);													\
 	CAT2(signal,_listeners) = g_slist_remove(CAT2(signal,_listeners), p);	\
 } G_STMT_END
