@@ -1,5 +1,5 @@
 /*
- * $Id: magnet.h 16238 2009-03-23 13:31:54Z rmanfredi $
+ * $Id: magnet.h 17265 2010-02-20 19:50:51Z rmanfredi $
  *
  * Copyright (c) 2006, Christian Biere
  *
@@ -40,7 +40,7 @@
 
 #include "lib/host_addr.h"
 #include "lib/misc.h"
-#include "lib/hashlist.h"
+#include "lib/sequence.h"
 
 struct magnet_source {
 	const char *url;			/* string atom */
@@ -59,6 +59,8 @@ struct magnet_resource {
 	const struct sha1 *sha1;	/* SHA1 atom */
 	const struct tth *tth;		/* TTH atom */
 	const char *parq_id;		/* string atom */
+	const char *vendor;			/* string atom */
+	const char *guid;			/* string atom */
 	GSList *sources;	/* List of walloc()ed (struct magnet_source *) */
 	GSList *searches;	/* List of string atoms */
 	filesize_t size;
@@ -88,7 +90,9 @@ void magnet_add_sha1_source(struct magnet_resource *res,
 
 /* Extensions */
 void magnet_set_parq_id(struct magnet_resource *res, const char *parq_id);
-char *magnet_proxies_to_string(hash_list_t *proxies);
+void magnet_set_guid(struct magnet_resource *res, const char *guid);
+void magnet_set_vendor(struct magnet_resource *res, const char *vendor);
+char *magnet_proxies_to_string(const sequence_t *proxies);
 
 #endif /* _magnet_h_ */
 /* vi: set ts=4 sw=4 cindent: */
