@@ -1,5 +1,5 @@
 /*
- * $Id: malloc.h 16247 2009-03-26 23:03:44Z rmanfredi $
+ * $Id: malloc.h 16938 2009-06-13 10:18:10Z rmanfredi $
  *
  * Copyright (c) 2004, Raphael Manfredi
  *
@@ -64,6 +64,7 @@
 #include "hashlist.h"
 
 #undef strdup			/**< Defined in <bits/string2.h> */
+#undef strndup
 
 #define malloc(s)		malloc_track((s), _WHERE_, __LINE__)
 #define calloc(n,s)		malloc0_track((n)*(s), _WHERE_, __LINE__)
@@ -72,17 +73,28 @@
 #define strdup(s)		strdup_track((s), _WHERE_, __LINE__)
 #define strndup(s,n)	strndup_track((s), (n), _WHERE_, __LINE__)
 
+#define halloc(s)		malloc_track((s), _WHERE_, __LINE__)
+#define halloc0(s)		malloc0_track((s), _WHERE_, __LINE__)
+#define hfree(s)		free_track((s), _WHERE_, __LINE__)
+#define hrealloc(o,s)	realloc_track(o, (s), _WHERE_, __LINE__)
+#define hcopy(p,s)		memdup_track((p), (s), _WHERE_, __LINE__)
+
 #define g_malloc(s)		malloc_track((s), _WHERE_, __LINE__)
 #define g_malloc0(s)	malloc0_track((s), _WHERE_, __LINE__)
 #define g_free(o)		free_track(o, _WHERE_, __LINE__)
 #define g_realloc(o,s)	realloc_track(o, (s), _WHERE_, __LINE__)
-#define g_strdup(s)		((s) ? strdup_track(s, _WHERE_, __LINE__) : 0)
-#define g_strndup(s,n)	((s) ? strndup_track(s, (n), _WHERE_, __LINE__) : 0)
+#define g_strdup(s)		strdup_track(s, _WHERE_, __LINE__)
+#define g_strndup(s,n)	strndup_track(s,(n),_WHERE_,__LINE__)
 #define g_strjoinv(s,v)	strjoinv_track(s, (v), _WHERE_, __LINE__)
-#define g_memdup(p,s)	((p) ? memdup_track((p), (s), _WHERE_, __LINE__) : 0)
+#define g_memdup(p,s)	memdup_track((p), (s), _WHERE_, __LINE__)
 #define g_strfreev(v)	strfreev_track((v), _WHERE_, __LINE__)
 
-/* FIXME: This only correct if xmlFree() is equivalent to free(). */
+#define h_strdup(s)		strdup_track(s,_WHERE_,__LINE__)
+#define h_strndup(s,n)	strndup_track(s,(n),_WHERE_,__LINE__)
+#define h_strjoinv(s,v)	strjoinv_track(s, (v), _WHERE_, __LINE__)
+#define h_strfreev(v)	strfreev_track((v), _WHERE_, __LINE__)
+
+/* FIXME: This is only correct if xmlFree() is equivalent to free(). */
 #define xmlFree(o)		free_track(o, _WHERE_, __LINE__)
 
 #undef g_new
@@ -169,6 +181,11 @@
 #define g_string_sprintfa(s,fmt,...) \
 	string_sprintf_tracka((s),_WHERE_,__LINE__,(fmt), __VA_ARGS__)
 
+#define h_strdup_printf(fmt, ...) \
+	strdup_printf_track(_WHERE_, __LINE__, fmt, __VA_ARGS__)
+
+#define h_strconcat(s, ...) \
+	strconcat_track(_WHERE_, __LINE__, s, __VA_ARGS__)
 
 /*
  * Use STRTRACK() to track an allocated string by some obscure routine that
@@ -178,119 +195,129 @@
  *
  * Likewise, use MEMTRACK() to track some random memory buffer known to have
  * been allocated by a routine and not via any of the trapped calls.
+ *
+ * Use NOT_LEAKING() to remember the object as an exception: it will not
+ * be flagged as a leak if it was not freed at the end.  This is intended
+ * to be used in lazy-style functions that keep allocated data in a static
+ * variable, which is freed at the next invocation, or for once functions that
+ * compute data on the first call and then always return the same value.
  */
 #define STRTRACK(o)		string_record((o), _WHERE_, __LINE__)
-#define MEMTRACK(o,s)	malloc_record((o), (s), _WHERE_, __LINE__)
+#define MEMTRACK(o,s)	malloc_record((o), (s), FALSE, _WHERE_, __LINE__)
 #define GSLISTTRACK(o)	gslist_record((o), _WHERE_, __LINE__)
 #define GLISTTRACK(o)	glist_record((o), _WHERE_, __LINE__)
+#define NOT_LEAKING(o)	malloc_not_leaking(o, _WHERE_, __LINE__)
 
 #else	/* !TRACK_MALLOC || MALLOC_SOURCE */
 
-#define STRTRACK(o)		o
-#define MEMTRACK(o,s)	o
-#define GSLISTTRACK(o)	o
-#define GLISTTRACK(o)	o
+#define STRTRACK(o)		(o)
+#define MEMTRACK(o,s)	(o)
+#define GSLISTTRACK(o)	(o)
+#define GLISTTRACK(o)	(o)
+#define NOT_LEAKING(o)	(o)
 
 #endif	/* TRACK_MALLOC && !MALLOC_SOURCE */
 
 #if defined(TRACK_MALLOC) || defined(MALLOC_SOURCE)
 
-char *string_record(const char *s, char *file, int line);
-gpointer malloc_record(gconstpointer o, size_t size, char *file, int line);
-GSList *gslist_record(const GSList *, char *file, int line);
-GList *glist_record(const GList *, char *file, int line);
+char *string_record(const char *s, const char *file, int line);
+gpointer malloc_record(gconstpointer o, size_t size, gboolean owned,
+	const char *file, int line);
+GSList *gslist_record(const GSList *, const char *file, int line);
+GList *glist_record(const GList *, const char *file, int line);
+gpointer malloc_not_leaking(gconstpointer o, const char *file, int line);
 
-gpointer malloc_track(size_t size, char *file, int line);
-gpointer malloc0_track(size_t size, char *file, int line);
-void free_track(gpointer o, char *file, int line);
-void strfreev_track(char **v, char *file, int line);
-gpointer realloc_track(gpointer o, size_t size, char *file, int line);
-char *strdup_track(const char *s, char *file, int line);
-char *strndup_track(const char *s, size_t n, char *file, int line);
-gpointer memdup_track(gconstpointer p, size_t size, char *file, int line);
-char *strjoinv_track(const char *s, char **vec, char *file, int line);
-char *strconcat_track(char *file, int line, const char *s, ...);
-char *strdup_printf_track(char *file, int line, const char *fmt, ...)
+gpointer malloc_track(size_t size, const char *file, int line);
+gpointer malloc0_track(size_t size, const char *file, int line);
+void free_track(gpointer o, const char *file, int line);
+void strfreev_track(char **v, const char *file, int line);
+gpointer realloc_track(gpointer o, size_t size, const char *file, int line);
+char *strdup_track(const char *s, const char *file, int line);
+char *strndup_track(const char *s, size_t n, const char *file, int line);
+gpointer memdup_track(gconstpointer p, size_t size, const char *file, int line);
+char *strjoinv_track(const char *s, char **vec, const char *file, int line);
+char *strconcat_track(const char *file, int line, const char *s, ...);
+char *strdup_printf_track(const char *file, int line, const char *fmt, ...)
 	G_GNUC_PRINTF(3, 4);
 char **strsplit_track(
-	const char *s, const char *d, size_t m, char *file, int line);
+	const char *s, const char *d, size_t m, const char *file, int line);
 
 GHashTable *hashtable_new_track(
-	GHashFunc h, GCompareFunc y, char *file, int line);
-void hashtable_destroy_track(GHashTable *h, char *file, int line);
+	GHashFunc h, GCompareFunc y, const char *file, int line);
+void hashtable_destroy_track(GHashTable *h, const char *file, int line);
 
 hash_list_t *hash_list_new_track(
-	GHashFunc hash_func, GEqualFunc eq_func, char *file, int line);
-void hash_list_free_track(hash_list_t **hl_ptr, char *file, int line);
+	GHashFunc hash_func, GEqualFunc eq_func, const char *file, int line);
+void hash_list_free_track(hash_list_t **hl_ptr, const char *file, int line);
 
-GSList *track_slist_alloc(char *file, int line);
-GSList *track_slist_append(GSList *l, gpointer data, char *file, int line);
-GSList *track_slist_prepend(GSList *l, gpointer data, char *file, int line);
-GSList *track_slist_copy(GSList *l, char *file, int line);
-void track_slist_free(GSList *l, char *file, int line);
-void track_slist_free1(GSList *l, char *file, int line);
-GSList *track_slist_remove(GSList *l, gpointer data, char *file, int line);
+GSList *track_slist_alloc(const char *file, int line);
+GSList *track_slist_append(GSList *l, gpointer data,
+	const char *file, int line);
+GSList *track_slist_prepend(GSList *l, gpointer data,
+	const char *file, int line);
+GSList *track_slist_copy(GSList *l, const char *file, int line);
+void track_slist_free(GSList *l, const char *file, int line);
+void track_slist_free1(GSList *l, const char *file, int line);
+GSList *track_slist_remove(GSList *l, gpointer data,
+	const char *file, int line);
 GSList *track_slist_insert(
-	GSList *l, gpointer d, int pos, char *file, int line);
+	GSList *l, gpointer d, int pos, const char *file, int line);
 GSList *track_slist_insert_sorted(
-	GSList *l, gpointer d, GCompareFunc c, char *file, int line);
+	GSList *l, gpointer d, GCompareFunc c, const char *file, int line);
 GSList *track_slist_insert_after(
-	GSList *l, GSList *lk, gpointer data, char *file, int line);
-GSList *track_slist_delete_link(GSList *l, GSList *lk, char *file, int line);
+	GSList *l, GSList *lk, gpointer data, const char *file, int line);
+GSList *track_slist_delete_link(GSList *l, GSList *lk,
+	const char *file, int line);
 
-GList *track_list_alloc(char *file, int line);
-GList *track_list_append(GList *l, gpointer data, char *file, int line);
-GList *track_list_prepend(GList *l, gpointer data, char *file, int line);
-GList *track_list_copy(GList *l, char *file, int line);
-void track_list_free(GList *l, char *file, int line);
-void track_list_free1(GList *l, char *file, int line);
-GList *track_list_remove(GList *l, gpointer data, char *file, int line);
+GList *track_list_alloc(const char *file, int line);
+GList *track_list_append(GList *l, gpointer data, const char *file, int line);
+GList *track_list_prepend(GList *l, gpointer data, const char *file, int line);
+GList *track_list_copy(GList *l, const char *file, int line);
+void track_list_free(GList *l, const char *file, int line);
+void track_list_free1(GList *l, const char *file, int line);
+GList *track_list_remove(GList *l, gpointer data, const char *file, int line);
 GList *track_list_insert(
-	GList *l, gpointer d, int pos, char *file, int line);
+	GList *l, gpointer d, int pos, const char *file, int line);
 GList *track_list_insert_sorted(
-	GList *l, gpointer d, GCompareFunc c, char *file, int line);
+	GList *l, gpointer d, GCompareFunc c, const char *file, int line);
 GList *track_list_insert_after(
-	GList *l, GList *lk, gpointer data, char *file, int line);
+	GList *l, GList *lk, gpointer data, const char *file, int line);
 GList *track_list_insert_before(
-	GList *l, GList *lk, gpointer data, char *file, int line);
-GList *track_list_delete_link(GList *l, GList *lk, char *file, int line);
+	GList *l, GList *lk, gpointer data, const char *file, int line);
+GList *track_list_delete_link(GList *l, GList *lk, const char *file, int line);
 
-GString *string_new_track(const char *p, char *file, int line);
-GString *string_sized_new_track(size_t size, char *file, int line);
+GString *string_new_track(const char *p, const char *file, int line);
+GString *string_sized_new_track(size_t size, const char *file, int line);
 GString *string_append_track(
-	GString *s, const char *p, char *file, int line);
+	GString *s, const char *p, const char *file, int line);
 GString *string_append_c_track(
-	GString *s, char c, char *file, int line);
+	GString *s, char c, const char *file, int line);
 GString *string_append_len_track(
-	GString *s, const char *val,  gssize len, char *file, int line);
+	GString *s, const char *val,  gssize len, const char *file, int line);
 GString *string_assign_track(
-	GString *s, const char *p, char *file, int line);
-void string_free_track(GString *s, int freestr, char *file, int line);
+	GString *s, const char *p, const char *file, int line);
+void string_free_track(GString *s, int freestr, const char *file, int line);
 GString *string_prepend_track(
-	GString *s, const char *p, char *file, int line);
+	GString *s, const char *p, const char *file, int line);
 GString *string_prepend_c_track(
-	GString *s, char c, char *file, int line);
+	GString *s, char c, const char *file, int line);
 GString *string_insert_track(
-	GString *s, int pos, const char *p, char *file, int line);
+	GString *s, int pos, const char *p, const char *file, int line);
 GString *string_insert_c_track(
-	GString *s, int pos, char c, char *file, int line);
+	GString *s, int pos, char c, const char *file, int line);
 GString *string_sprintf_track(
-	GString *s, char *file, int line, const char *fmt, ...)
+	GString *s, const char *file, int line, const char *fmt, ...)
 	G_GNUC_PRINTF(4, 5);
 GString *string_sprintfa_track(
-	GString *s, char *file, int line, const char *fmt, ...)
+	GString *s, const char *file, int line, const char *fmt, ...)
 	G_GNUC_PRINTF(4, 5);
 
 #endif	/* TRACK_MALLOC || MALLOC_SOURCE */
 
-#ifdef TRACK_MALLOC
-void malloc_close(void);
-#endif
-
 #if defined(TRACK_MALLOC) || defined(TRACK_ZALLOC)
 
 gpointer leak_init(void);
-void leak_add(gpointer o, size_t size, char *file, int line);
+void leak_add(gpointer o, size_t size, const char *file, int line);
 void leak_dump(gpointer o);
 void leak_close(gpointer o);
 
@@ -301,6 +328,16 @@ void alloc_dump(FILE *f, gboolean total);
 void alloc_reset(FILE *f, gboolean total);
 #endif
 
+/*
+ * Public interface, available no matter which compile options are used.
+ */
+
+void malloc_init(const char *argv0);
+void malloc_init_vtable(void);
+void malloc_close(void);
+
+void *real_malloc(size_t size);
+
 #endif /* _malloc_h_ */
 
 /* vi: set ts=4 sw=4 cindent:  */
