@@ -1,5 +1,5 @@
 /*
- * $Id: misc.h 16281 2009-03-28 20:03:21Z cbiere $
+ * $Id: misc.h 17213 2009-11-09 17:13:18Z rmanfredi $
  *
  * Copyright (c) 2001-2003, Raphael Manfredi
  *
@@ -52,12 +52,11 @@
 
 #include "common.h"
 
-#include "compat_sleep_ms.h"
 #include "fs_free_space.h"
 #include "tm.h"
 #include "vmm.h"
 
-#define SIZE_FIELD_MAX 64		/**< Max size of sprintf-ed size quantity */
+#define SIZE_FIELD_MAX		64	/**< Max size of sprintf-ed size quantity */
 #define GUID_RAW_SIZE		16	/**< Binary representation of 128 bits */
 #define GUID_HEX_SIZE		32	/**< Hexadecimal GUID representation */
 #define GUID_BASE32_SIZE	26	/**< base32 GUID representation */
@@ -66,29 +65,6 @@ typedef struct short_string {
 	char str[SIZE_FIELD_MAX];
 } short_string_t;
 
-/**
- * Needs to be defined if we are not using Glib 2
- */
-#ifndef USE_GLIB2
-
-#ifndef HAS_STRLCPY
-size_t strlcpy(char *dst, const char *src, size_t dst_size);
-#endif /* HAS_STRLCPY */
-
-#ifndef HAS_STRLCAT
-size_t strlcat(char *dst, const char *src, size_t dst_size);
-#endif /* HAS_STRLCAT */
-
-#define g_string_printf g_string_sprintf
-#define g_strlcpy strlcpy
-#define g_strlcat strlcat
-#endif
-
-size_t concat_strings(char *dst, size_t size,
-	const char *s, ...) G_GNUC_NULL_TERMINATED;
-size_t w_concat_strings(char **dst,
-	const char *first, ...) G_GNUC_NULL_TERMINATED;
-
 /**
  * Converts an integer to a single hexadecimal ASCII digit. The are no checks,
  * this is just a convenience function.
@@ -128,89 +104,33 @@ skip_dir_separators(const char *s)
  */
 #define CONST_STRLEN(x) (sizeof(x) - 1)
 
-/*
- * Set/clear binary flags
- */
-typedef guint16 flag_t;
-#define set_flags(r,f) (r |= (f))
-#define clear_flags(r,f) (r &= ~(f))
-
-/*
- * Macros to determine the maximum buffer size required to hold a
- * NUL-terminated string.
- */
-#define UINT8_HEX_BUFLEN	(sizeof "FF")
-#define UINT8_DEC_BUFLEN	(sizeof "255")
-#define UINT16_HEX_BUFLEN	(sizeof "01234")
-#define UINT16_DEC_BUFLEN	(sizeof "65535")
-#define UINT32_HEX_BUFLEN	(sizeof "012345678")
-#define UINT32_DEC_BUFLEN	(sizeof "4294967295")
-#define UINT64_HEX_BUFLEN	(sizeof "0123456789ABCDEF")
-#define UINT64_DEC_BUFLEN	(sizeof "18446744073709551615")
-#define IPV4_ADDR_BUFLEN	(sizeof "255.255.255.255")
-#define IPV6_ADDR_BUFLEN \
-	  (sizeof "0001:0203:0405:0607:0809:1011:255.255.255.255")
-#define TIMESTAMP_BUF_LEN	(sizeof "9999-12-31 23:59:61")
-#define OFF_T_DEC_BUFLEN	(sizeof(off_t) * CHAR_BIT) /* very roughly */
-#define TIME_T_DEC_BUFLEN	(sizeof(time_t) * CHAR_BIT) /* very roughly */
-
-#define HOST_ADDR_BUFLEN	(MAX(IPV4_ADDR_BUFLEN, IPV6_ADDR_BUFLEN))
-#define HOST_ADDR_PORT_BUFLEN	(HOST_ADDR_BUFLEN + sizeof ":[65535]")
-
-gboolean parse_ipv6_addr(const char *s, uint8_t *dst, const char **endptr);
-const char *ipv6_to_string(const guint8 *ipv6);
-size_t ipv6_to_string_buf(const guint8 *ipv6, char *dst, size_t size);
-
 /*
  * Network related string routines
  */
-guint32  string_to_ip(const char *);
-gboolean string_to_ip_strict(const char *s, guint32 *addr, const char **ep);
-gboolean string_to_ip_and_mask(const char *str, guint32 *ip, guint32 *netmask);
-gboolean string_to_ip_port(const char *str, guint32 *ip, guint16 *port);
-const char *ip_to_string(guint32);
-size_t ipv4_to_string_buf(guint32 ip, char *buf, size_t size);
-const char *hostname_port_to_string(const char *hostname, guint16 port);
 const char *local_hostname(void);
 #define port_is_valid(port) (port != 0)
 
-/*
- * Date string conversions
- */
-const char *timestamp_to_string(time_t date);
-const char *timestamp_utc_to_string(time_t date);
-const char *timestamp_rfc822_to_string(time_t date);
-const char *timestamp_rfc822_to_string2(time_t date);
-const char *timestamp_rfc1123_to_string(time_t date);
-
-size_t timestamp_to_string_buf(time_t date, char *dst, size_t size);
-size_t timestamp_utc_to_string_buf(time_t date, char *dst, size_t size);
-size_t time_locale_to_string_buf(time_t date, char *dst, size_t size);
-
-short_string_t timestamp_get_string(time_t date);
-
-/*
- * Time string conversions
- */
-const char *short_time(time_delta_t s);
-const char *short_time_ascii(time_delta_t t);
-const char *short_uptime(time_delta_t s);
-const char *compact_time(time_delta_t t);
-
 /*
  * Size string conversions
  */
 const char *short_size(guint64 size, gboolean metric);
 const char *short_html_size(guint64 size, gboolean metric);
 const char *short_kb_size(guint64 size, gboolean metric);
+const char *short_kb_size2(guint64 size, gboolean metric);
 const char *short_rate(guint64 rate, gboolean metric);
+const char *short_byte_size(guint64 size, gboolean metric);
+const char *short_byte_size2(guint64 size, gboolean metric);
 const char *compact_size(guint64 size, gboolean metric);
+const char *compact_size2(guint64 size, gboolean metric);
 const char *compact_rate(guint64 rate, gboolean metric);
 const char *compact_kb_size(guint32 size, gboolean metric);
 const char *nice_size(guint64 size, gboolean metric);
 char *short_value(char *buf, size_t size, guint64 v, gboolean metric);
 char *compact_value(char *buf, size_t size, guint64 v, gboolean metric);
 
+size_t short_byte_size_to_buf(guint64 size, gboolean metric, char *, size_t);
+size_t short_kb_size_to_buf(guint64 size, gboolean metric, char *, size_t);
+
 short_string_t short_rate_get_string(guint64 rate, gboolean metric);
 
 /*
@@ -222,7 +142,7 @@ typedef struct sha1 {
 
 #define SHA1_URN_LENGTH	(CONST_STRLEN("urn:sha1:") + SHA1_BASE32_SIZE)
 
-const char *sha1_to_string(const struct sha1 sha1);
+const char *sha1_to_string(const struct sha1 *sha1);
 const char *sha1_to_urn_string(const struct sha1 *);
 size_t sha1_to_urn_string_buf(const struct sha1 *, char *dst, size_t size);
 char *sha1_to_base32_buf(const struct sha1 *, char *dst, size_t size);
@@ -277,37 +197,44 @@ size_t bin_to_hex_buf(const void *data, size_t len, char *dst, size_t size);
 /*
  * Tests
  */
-gboolean is_absolute_path(const char *pathname);
 gboolean is_directory(const char *pathname);
 gboolean is_regular(const char *pathname);
 gboolean is_symlink(const char *pathname);
 int is_same_file(const char *, const char *);
-gboolean file_exists(const char *pathname);
-gboolean file_does_not_exist(const char *pathname);
-guint32 next_pow2(guint32 n);
 
-#define IS_POWER_OF_2(x) ((x) && 0 == ((x) & ((x) - 1)))
 /**
- * Checks whether the given value is a power of 2.
- *
- * @param value a 32-bit integer
- * @return TRUE if ``value'' is a power of 2. Otherwise FALSE.
+ * Tries to extrace the file mode from a struct dirent. Not all systems
+ * support this, in which case zero is returned. Types other than regular
+ * files, directories and symlinks are ignored and gain a value of zero
+ * as well.
  */
-static inline G_GNUC_CONST gboolean
-is_pow2(guint32 value)
-#ifdef HAS_BUILTIN_POPCOUNT
+static inline mode_t
+dir_entry_mode(const struct dirent *dir_entry)
 {
-	return 1 == __builtin_popcount(value);
+	g_assert(dir_entry);
+#ifdef HAS_DIRENT_D_TYPE
+	switch (dir_entry->d_type) {
+	case DT_DIR:	return S_IFDIR;
+	case DT_LNK:	return S_IFLNK;
+	case DT_REG:	return S_IFREG;
+	case DT_CHR:	return S_IFCHR;
+	case DT_BLK:	return S_IFBLK;
+	case DT_FIFO:	return S_IFIFO;
+#if defined(DT_WHT) && defined(S_IFWHT)
+	case DT_WHT:	return S_IFWHT;
+#endif	/* DT_WHT */
+#if defined(DT_SOCK) && defined(S_IFSOCK)
+	case DT_SOCK:	return S_IFSOCK;
+#endif	/* DT_SOCK */
+	}
+#endif	/* HAS_DIRENT_WITH_D_TYPE */
+	return 0;
 }
-#else /* !HAS_BUILTIN_POPCOUNT */
-{
-	return IS_POWER_OF_2(value);
-}
-#endif /* HAS_BUILTIN_POPCOUNT */
 
 /*
  * Random numbers
  */
+
 void random_init(void);
 guint32 random_value(guint32 max) WARN_UNUSED_RESULT;
 guint32 random_u32(void) WARN_UNUSED_RESULT;
@@ -317,7 +244,10 @@ void guid_random_fill(struct guid *);
 /*
  * Stuff
  */
+
 void misc_init(void);
+void misc_close(void);
+
 size_t str_chomp(char *str, size_t len);
 int hex2int(guchar c);
 gboolean is_printable(const char *buf, int len);
@@ -325,18 +255,10 @@ void dump_hex(FILE *, const char *, gconstpointer, int);
 void dump_string(FILE *out, const char *str, size_t len, const char *trailer);
 gboolean is_printable_iso8859_string(const char *s);
 void locale_strlower(char *, const char *);
-size_t filename_shrink(const char *filename, char *buf, size_t size);
-char *unique_filename(const char *path, const char *file, const char *ext,
-		gboolean (*name_is_uniq)(const char *pathname));
-char *hex_escape(const char *name, gboolean strict);
-char *control_escape(const char *s);
-const char *lazy_string_to_printf_escape(const char *src);
 int highest_bit_set(guint32 n) G_GNUC_CONST;
 size_t common_leading_bits(
 	gconstpointer k1, size_t k1bits, gconstpointer k2, size_t k2bits);
 float force_range(float value, float min, float max);
-char *absolute_pathname(const char *file);
-char *make_pathname(const char *dir, const char *file);
 char *short_filename(char *fullname);
 char *data_hex_str(const char *data, size_t len);
 
@@ -349,61 +271,53 @@ char *data_hex_str(const char *data, size_t len);
 #endif /* S_IROTH && S_IXOTH */
 
 int create_directory(const char *dir, mode_t mode);
-int compat_mkdir(const char *path, mode_t mode);
-gboolean filepath_exists(const char *dir, const char *file);
-const char * filepath_basename(const char *pathname);
-char * filepath_directory(const char *pathname);
-guint16 parse_uint16(const char *, char const **, guint, int *)
-	NON_NULL_PARAM((1, 4));
-guint32 parse_uint32(const char *, char const **, guint, int *)
-	NON_NULL_PARAM((1, 4));
-guint64 parse_uint64(const char *, char const **, guint, int *)
-	NON_NULL_PARAM((1, 4));
-size_t int32_to_string_buf(gint32 v, char *dst, size_t size);
-size_t uint32_to_string_buf(guint32 v, char *dst, size_t size);
-size_t uint64_to_string_buf(guint64 v, char *dst, size_t size);
-size_t off_t_to_string_buf(off_t v, char *dst, size_t size);
-size_t time_t_to_string_buf(time_t v, char *dst, size_t size);
-const char *uint32_to_string(guint32 v);
-const char *uint64_to_string(guint64 v);
-const char *uint64_to_string2(guint64 v);
-const char *off_t_to_string(off_t v);
-const char *time_t_to_string(time_t v);
-const char *filesize_to_string(filesize_t v);
-const char *filesize_to_string2(filesize_t v);
+
 int parse_major_minor(const char *src, char const **endptr,
 	guint *major, guint *minor);
 char *is_strprefix(const char *s, const char *prefix) WARN_UNUSED_RESULT;
 char *is_strcaseprefix(const char *s, const char *prefix) WARN_UNUSED_RESULT;
+gboolean is_strsuffix(const char *str, size_t len, const char *suffix);
 size_t html_escape(const char *src, char *dst, size_t dst_size);
 guint32 html_decode_entity(const char *src, const char **endptr);
 int canonize_path(char *dst, const char *path);
-guint compat_max_fd(void);
-void close_file_descriptors(const int first_fd);
-int reserve_standard_file_descriptors(void);
-gboolean compat_is_superuser(void);
-int compat_daemonize(const char *directory);
-void set_close_on_exec(int fd);
-void compat_fadvise_sequential(int fd, off_t offset, off_t size);
-void *compat_memmem(const void *data, size_t data_size,
-		const void *pattern, size_t pattern_size);
-
-int get_non_stdio_fd(int fd);
 
 typedef void (*signal_handler_t)(int signo);
 signal_handler_t set_signal(int signo, signal_handler_t handler);
 
-char *normalize_dir_separators(const char *s);
+void normalize_dir_separators(char *);
 size_t memcmp_diff(const void *a, const void *b, size_t n);
 guint32 cpu_noise(void);
 
 static inline guint
 pointer_hash_func(const void *p)
 {
-	size_t v = (size_t) p;
+	unsigned long v = pointer_to_ulong(p);
 	return (((guint64) 0x4F1BBCDCUL * v) >> 32) ^ v;
 }
 
+/**
+ * An strcpy() that returns the length of the copied string.
+ */
+static inline size_t
+strcpy_len(char *dest, const char *src)
+{
+	const char *p = src;
+	char *q = dest;
+	int c;
+
+	g_assert(dest != NULL);
+
+	if (NULL == src)
+		return 0;
+	
+	while ((c = *p++))
+		*q++ = c;
+
+	*q = '\0';
+
+	return q - dest;
+}
+
 /**
  * Determines the length of a NUL-terminated string looking only at the first
  * "src_size" bytes. If src[0..size] contains no NUL byte, "src_size" is
@@ -419,12 +333,12 @@ pointer_hash_func(const void *p)
 static inline size_t
 clamp_strlen(const char *src, size_t src_size)
 {
-	const char *p, *endptr = &src[src_size];
+	const char *p;
 
 	/* @NOTE: memchr() is intentionally NOT used because 'src_size' is allowed
 	 *        to exceed the size of the memory object 'src'.
 	 */
-	for (p = src; p != endptr && '\0' != *p; p++)
+	for (p = src; src_size-- > 0 && '\0' != *p; p++)
 		continue;
 
 	return p - src;
@@ -517,6 +431,15 @@ EMPTY_STRING(const char *s)
 	return NULL != s ? s : "";
 }
 
+/**
+ * Is string NULL or empty?
+ */
+static inline gboolean
+is_null_or_empty(const char *s)
+{
+	return NULL == s || '\0' == *s;
+}
+
 /**
  * Swap endianness of a guint32.
  *
@@ -584,7 +507,19 @@ static inline size_t
 round_size(size_t align, size_t n)
 {
 	size_t m = n % align;
-	return m ? n + (align - m) : MAX(n, align);
+	return m ? n + (align - m) : n;
+}
+
+/**
+ * Rounds ``n'' up so that it matches the given alignment ``align''.
+ * Fast version, when ``align'' is known to be a power of 2.
+ */
+static inline size_t
+round_size_fast(size_t align, size_t n)
+{
+	size_t mask = align - 1;
+
+	return (n + mask) & ~mask;
 }
 
 /*
@@ -615,73 +550,6 @@ guint filesize_per_100(filesize_t size, filesize_t part);
 guint filesize_per_1000(filesize_t size, filesize_t part);
 guint filesize_per_10000(filesize_t size, filesize_t part);
 
-/*
- * NOTE: ssize_t is NOT the signed variant of size_t and casting values blindly
- * to ssize_t may cause integer overflows.  Larger values, especially SIZE_MAX
- * (size_t)-1 may be the result of errors or wrap arounds during calculations.
- * Therefore in places where memory objects larger than half of the address
- * space are unreasonable, the following two functions are useful to check for
- * such conditions.
- */
-
-/*
- * Check whether a signed representation of size would be non-negative.
- * @return TRUE if size is equal to zero or larger and smaller than
- *         SIZE_MAX / 2.
- */
-static inline gboolean
-size_is_non_negative(size_t size)
-{
-	return size <= SIZE_MAX / 2;
-}
-
-/**
- * Check whether a signed representation of size would be positive.
- * @return TRUE if size is larger than zero and smaller than SIZE_MAX / 2.
- */
-static inline gboolean
-size_is_positive(size_t size)
-{
-	return size_is_non_negative(size - 1);
-}
-
-/*
- * Calculate the sum of a and b but saturate towards SIZE_MAX.
- * @return SIZE_MAX if a + b > SIZE_MAX, otherwise a + b.
- */
-static inline size_t
-size_saturate_add(size_t a, size_t b)
-{
-	size_t ret = a + b;
-	if (G_UNLIKELY(ret < a))
-		return SIZE_MAX;
-	return ret;
-}
-
-/*
- * Calculate the product of a and b but saturate towards SIZE_MAX.
- * @return SIZE_MAX if a * b > SIZE_MAX, otherwise a * b.
- */
-static inline size_t
-size_saturate_mult(size_t a, size_t b)
-{
-	if (G_UNLIKELY(0 != a && SIZE_MAX / a < b))
-		return SIZE_MAX;
-	return a * b;
-}
-
-/*
- * Calculate the difference between a and b but saturate towards zero.
- * @return zero if a < b, otherwise a - b.
- */
-static inline size_t
-size_saturate_sub(size_t a, size_t b)
-{
-	if (G_UNLIKELY(a < b))
-		return 0;
-	return a - b;
-}
-
 /*
  * CIDR split of IP range.
  */
@@ -751,21 +619,53 @@ G_STMT_START { \
 	} \
 } G_STMT_END
 
-static inline const char *
-print_number(char *dst, size_t size, unsigned long value)
+/**
+ * Converts an integer to a single decimal ASCII digit. The are no checks,
+ * this is just a convenience function.
+ *
+ * @param x An integer between 0 and 9.
+ * @return The ASCII character corresponding to the decimal digit [0-9].
+ */
+static inline guchar
+dec_digit(guchar x)
 {
-	char *p = &dst[size];
+	static const char dec_alphabet[] = "0123456789";
+	return dec_alphabet[x % 10];
+}
 
-	if (size > 0) {
-		*--p = '\0';
+/**
+ * Copies "src_len" chars from "src" to "dst" reversing their order.
+ * The resulting string is always NUL-terminated unless "size" is zero.
+ * If "size" is not larger than "src_len", the resulting string will
+ * be truncated. NUL chars copied from "src" are not treated as string
+ * terminations.
+ *
+ * @param dst The destination buffer.
+ * @param size The size of the destination buffer.
+ * @param src The source buffer.
+ * @param src_len The size of the source buffer.
+ *
+ * @return The resulting length of string not counting the termating NUL.
+ *         Note that NULs that might have been copied from "src" are
+ *         included in this count. Thus strlen(dst) would return a lower
+ *         value in this case. 
+ */
+static inline size_t
+reverse_strlcpy(char * const dst, size_t size,
+	const char *src, size_t src_len)
+{
+	char *p = dst;
+	
+	if (size-- > 0) {
+		const char *q = &src[src_len], *end = &dst[MIN(src_len, size)];
+
+		while (p != end) {
+			*p++ = *--q;
+		}
+		*p = '\0';
 	}
-	while (p != dst) {
-		*--p = (value % 10) + '0';
-		value /= 10;
-		if (0 == value)
-			break;
-	}
-	return p;
+
+	return p - dst;
 }
 
 #endif /* _misc_h_ */
