@@ -1,5 +1,5 @@
 /*
- * $Id: patricia.c 16242 2009-03-24 19:04:20Z cbiere $
+ * $Id: patricia.c 17037 2009-10-08 17:08:51Z rmanfredi $
  *
  * Copyright (c) 2008, Raphael Manfredi
  *
@@ -145,7 +145,7 @@
 
 #include "common.h"
 
-RCSID("$Id: patricia.c 16242 2009-03-24 19:04:20Z cbiere $")
+RCSID("$Id: patricia.c 17037 2009-10-08 17:08:51Z rmanfredi $")
 
 #include "patricia.h"
 #include "endian.h"
@@ -153,12 +153,32 @@ RCSID("$Id: patricia.c 16242 2009-03-24 19:04:20Z cbiere $")
 #include "walloc.h"
 #include "override.h"			/* Must be the last header included */
 
+/**
+ * When defined, extra magic numbers are insert in PATRICIA nodes, to make
+ * sure the internal data structures never become corrupted.  This adds some
+ * memory overhead to each item stored in the tree, so this is not enabled
+ * by default: only the main externally visible data structure is protected.
+ */
+#if 0
+#define PATRICIA_MEMCHECK
+#endif
+
+#ifdef PATRICIA_MEMCHECK
+enum patricia_node_magic   { PATRICIA_NODE_MAGIC   = 0x1cabac44U };
+enum patricia_parent_magic { PATRICIA_PARENT_MAGIC = 0x4d928669U };
+#endif
+
 struct patricia_parent;
 
 /**
  * A PATRICIA node in the tree.
  */
 struct patricia_node {
+
+#ifdef PATRICIA_MEMCHECK
+	enum patricia_node_magic magic;
+#endif
+
 	union {
 		struct patricia_node *parent;	/**< Parent node (NULL if root node) */
 		struct patricia_parent *ext;	/**< Extended: parent + data */
@@ -184,11 +204,76 @@ struct patricia_node {
  * reach the parent node.
  */
 struct patricia_parent {
+
+#ifdef PATRICIA_MEMCHECK
+	enum patricia_parent_magic magic;
+#endif
+
 	struct patricia_node *parent;	/**< Parent node (NULL if root node) */
 	const void *key;				/**< The item key */
 	const void *value;				/**< Data stored for the key in this node */
 };
 
+#ifdef PATRICIA_MEMCHECK
+static inline void
+patricia_node_set_magic(struct patricia_node *pn)
+{
+	pn->magic = PATRICIA_NODE_MAGIC;
+}
+
+static inline void
+patricia_node_check_magic(const struct patricia_node *pn)
+{
+	g_assert(PATRICIA_NODE_MAGIC == pn->magic);
+}
+
+static inline void
+patricia_node_clear_magic(struct patricia_node *pn)
+{
+	pn->magic = 0;
+}
+
+static inline void
+patricia_parent_set_magic(struct patricia_parent *pp)
+{
+	pp->magic = PATRICIA_PARENT_MAGIC;
+}
+
+static inline void
+patricia_parent_clear_magic(struct patricia_parent *pp)
+{
+	pp->magic = 0;
+}
+
+static inline void
+patricia_parent_check_magic(const struct patricia_parent *pp)
+{
+	g_assert(PATRICIA_PARENT_MAGIC == pp->magic);
+}
+
+#else  /* !PATRICIA_MEMCHECK */
+#define patricia_node_set_magic(n_)
+#define patricia_node_clear_magic(n_)
+#define patricia_node_check_magic(n_)
+#define patricia_parent_set_magic(p_)
+#define patricia_parent_clear_magic(p_)
+#define patricia_parent_check_magic(p_)
+#endif /* PATRICIA_MEMCHECK */
+
+static inline void
+patricia_node_check(const struct patricia_node *pn)
+{
+	g_assert(pn != NULL);
+	patricia_node_check_magic(pn);
+}
+
+static inline void
+patricia_parent_check(const struct patricia_parent *pp)
+{
+	g_assert(pp != NULL);
+	patricia_parent_check_magic(pp);
+}
+
 /*
  * Accessing shortcuts.
  */
@@ -197,28 +282,38 @@ struct patricia_parent {
 #define child_one(n_)			((n_)->u.children.o)
 #define leaf_item_key(n_)		((n_)->u.item.key)
 #define leaf_item_value(n_)		((n_)->u.item.value)
-#define leaf_item_keybits(n_)	(size_t) ((n_)->last_kbit + 1)
+#define leaf_item_keybits(n_)	((size_t) ((n_)->last_kbit + 1))
 
 #define parent_node(n_) \
 	((n_)->has_embedded_data ? (n_)->p.ext->parent : (n_)->p.parent)
 
 #define embedded_item_key(n_)		((n_)->p.ext->key)
 #define embedded_item_value(n_)		((n_)->p.ext->value)
-#define embedded_item_keybits(n_)	(size_t) ((n_)->bit)
+#define embedded_item_keybits(n_)	((size_t) ((n_)->bit))
+
+enum patricia_magic { PATRICIA_MAGIC = 0x42123004U };
 
 /**
  * The PATRICIA tree.
  */
 struct patricia {
+	enum patricia_magic magic;		/**< Magic number */
 	struct patricia_node *root;		/**< Root of tree (lazily allocated) */
 	size_t maxbits;					/**< Maximum bitsize of key */
 	size_t count;					/**< Amount of keys stored */
 	size_t nodes;					/**< Total amount of nodes used */
 	size_t embedded;				/**< Nodes holding embedded data */
 	guint stamp;					/**< Stamp to protect iterators */
-	int refcnt;					/**< Reference count */
+	int refcnt;						/**< Reference count */
 };
 
+static inline void
+patricia_check(const patricia_t *pt)
+{
+	g_assert(pt != NULL);
+	g_assert(PATRICIA_MAGIC == pt->magic);
+}
+
 /**
  * Create a new PATRICIA tree.
  *
@@ -235,6 +330,7 @@ patricia_create(size_t maxbits)
 	g_assert(maxbits != 0);
 
 	pt = walloc(sizeof *pt);
+	pt->magic = PATRICIA_MAGIC;
 	pt->root = NULL;
 	pt->count = pt->nodes = pt->embedded = 0;
 	pt->maxbits = maxbits;
@@ -250,7 +346,7 @@ patricia_create(size_t maxbits)
 size_t
 patricia_count(const patricia_t *pt)
 {
-	g_assert(pt);
+	patricia_check(pt);
 
 	return pt->count;
 }
@@ -261,7 +357,7 @@ patricia_count(const patricia_t *pt)
 size_t
 patricia_max_keybits(const patricia_t *pt)
 {
-	g_assert(pt);
+	patricia_check(pt);
 
 	return pt->maxbits;
 }
@@ -272,10 +368,15 @@ patricia_max_keybits(const patricia_t *pt)
 static struct patricia_node *
 allocate_node(patricia_t *pt)
 {
-	g_assert(pt);
+	struct patricia_node *pn;
+
+	patricia_check(pt);
 
 	pt->nodes++;
-	return walloc(sizeof(struct patricia_node));
+	pn = walloc(sizeof(struct patricia_node));
+	patricia_node_set_magic(pn);
+
+	return pn;
 }
 
 /**
@@ -284,11 +385,12 @@ allocate_node(patricia_t *pt)
 static void
 free_node(patricia_t *pt, struct patricia_node *pn)
 {
-	g_assert(pt);
-	g_assert(pn);
+	patricia_check(pt);
+	patricia_node_check(pn);
 	g_assert(pt->nodes > 0);
 
 	pt->nodes--;
+	patricia_node_clear_magic(pn);
 	wfree(pn, sizeof *pn);
 }
 
@@ -298,7 +400,7 @@ free_node(patricia_t *pt, struct patricia_node *pn)
 static inline void
 set_parent(struct patricia_node *pn, struct patricia_node *parent)
 {
-	g_assert(pn);
+	patricia_node_check(pn);
 
 	if (pn->has_embedded_data) {
 		g_assert(!pn->leaf);
@@ -334,12 +436,12 @@ node_prefix(const struct patricia_node *pn)
 	int i;
 	gconstpointer result = NULL;
 
-	g_assert(pn);
+	patricia_node_check(pn);
 
 	for (n = pn, i = 0; i < PATRICIA_MAXBITS; i++) {
 		const struct patricia_node *child;
 
-		g_assert(n);
+		patricia_node_check(n);
 
 		/*
 		 * If the node holds a value, then by construction its leading key
@@ -402,7 +504,7 @@ found:
 static inline size_t
 node_prefix_bits(const struct patricia_node *pn)
 {
-	g_assert(pn);
+	patricia_node_check(pn);
 
 	return pn->leaf ? leaf_item_keybits(pn) : pn->bit;
 }
@@ -413,7 +515,7 @@ node_prefix_bits(const struct patricia_node *pn)
 static inline gboolean
 node_has_data(const struct patricia_node *pn)
 {
-	g_assert(pn);
+	patricia_node_check(pn);
 
 	return pn->leaf || pn->has_embedded_data;
 }
@@ -424,7 +526,7 @@ node_has_data(const struct patricia_node *pn)
 static inline gconstpointer
 node_key(const struct patricia_node *pn)
 {
-	g_assert(pn);
+	patricia_node_check(pn);
 
 	if (pn->leaf)
 		return leaf_item_key(pn);
@@ -440,7 +542,7 @@ node_key(const struct patricia_node *pn)
 static inline size_t
 node_keybits(const struct patricia_node *pn)
 {
-	g_assert(pn);
+	patricia_node_check(pn);
 
 	if (pn->leaf)
 		return leaf_item_keybits(pn);
@@ -456,7 +558,7 @@ node_keybits(const struct patricia_node *pn)
 static inline gconstpointer
 node_value(const struct patricia_node *pn)
 {
-	g_assert(pn);
+	patricia_node_check(pn);
 
 	if (pn->leaf)
 		return leaf_item_value(pn);
@@ -474,7 +576,7 @@ node_matches(const struct patricia_node *pn, gconstpointer key, size_t keybits)
 {
 	const guint8 *k = key;
 
-	g_assert(pn);
+	patricia_node_check(pn);
 	g_assert(key);
 	g_assert(keybits >= (size_t) pn->bit + 1);
 
@@ -535,7 +637,7 @@ matched_bits(
 {
 	gconstpointer prefix;
 
-	g_assert(pn);
+	patricia_node_check(pn);
 
 	prefix = node_prefix(pn);
 	if (pprefix)
@@ -562,7 +664,7 @@ match_best(patricia_t *pt, gconstpointer key, size_t keybits)
 	const struct patricia_node *pn;
 	int i;
 
-	g_assert(pt);
+	patricia_check(pt);
 	g_assert(pt->root);
 	g_assert(key);
 	g_assert(keybits <= pt->maxbits);
@@ -573,7 +675,7 @@ match_best(patricia_t *pt, gconstpointer key, size_t keybits)
 	for (pn = pt->root, i = 0; i <= PATRICIA_MAXBITS; i++) {
 		const struct patricia_node *child;
 
-		g_assert(pn);
+		patricia_node_check(pn);
 
 		/*
 		 * If we have less bits in the key than the first bit to test for at
@@ -613,7 +715,7 @@ match_exact(const patricia_t *pt, gconstpointer key, size_t keybits)
 	const struct patricia_node *pn;
 	int i;
 
-	g_assert(pt);
+	patricia_check(pt);
 	g_assert(key);
 	g_assert(keybits <= pt->maxbits);
 
@@ -631,7 +733,7 @@ match_exact(const patricia_t *pt, gconstpointer key, size_t keybits)
 		for (pn = pt->root, i = 0; i <= PATRICIA_MAXBITS; i++) {
 			const struct patricia_node *child;
 
-			g_assert(pn);
+			patricia_node_check(pn);
 
 			/*
 			 * If we reach a leaf node, compare the key with the value.
@@ -658,7 +760,7 @@ match_exact(const patricia_t *pt, gconstpointer key, size_t keybits)
 		for (pn = pt->root, i = 0; i <= PATRICIA_MAXBITS; i++) {
 			const struct patricia_node *child;
 
-			g_assert(pn);
+			patricia_node_check(pn);
 
 			/*
 			 * If we have exactly the same amount of bits and the node contains
@@ -722,7 +824,7 @@ find_deepest(const struct patricia_node *root, gboolean leftmost)
 	for (pn = root, i = 0; i <= PATRICIA_MAXBITS; i++) {
 		const struct patricia_node *child;
 
-		g_assert(pn);
+		patricia_node_check(pn);
 
 		if (pn->leaf)
 			return pn;
@@ -767,7 +869,7 @@ lookup_best(const patricia_t *pt, gconstpointer key, size_t keybits)
 	const struct patricia_node *pn;
 	int i;
 
-	g_assert(pt);
+	patricia_check(pt);
 	g_assert(key);
 	g_assert(keybits <= pt->maxbits);
 
@@ -775,6 +877,7 @@ lookup_best(const patricia_t *pt, gconstpointer key, size_t keybits)
 		return NULL;
 
 	pn = pt->root;
+	patricia_node_check(pn);
 
 	if (keybits == 0)
 		return (node_has_data(pn) && 0 == node_keybits(pn)) ?  pn : NULL;
@@ -782,6 +885,8 @@ lookup_best(const patricia_t *pt, gconstpointer key, size_t keybits)
 	for (i = 0; i <= PATRICIA_MAXBITS; i++) {
 		const struct patricia_node *child;
 
+		patricia_node_check(pn);
+
 		/*
 		 * If we're already too deep in the tree, the keys we'll find will
 		 * be larger than the one we're trying to match against.  Stop.
@@ -831,6 +936,7 @@ lookup_best(const patricia_t *pt, gconstpointer key, size_t keybits)
 		const struct patricia_node *n = *sp--;
 		size_t nbits = node_keybits(n);
 
+		patricia_node_check(n);
 		g_assert(nbits <= keybits);
 
 		if (nbits == common_leading_bits(node_key(n), nbits, key, keybits))
@@ -866,7 +972,7 @@ find_closest(const patricia_t *pt, const struct patricia_node *root,
 	const struct patricia_node *pn;
 	int i;
 
-	g_assert(pt);
+	patricia_check(pt);
 	g_assert(key);
 	g_assert(keybits == pt->maxbits);
 	g_assert(pt->embedded == 0);		/* All keys must have the same size */
@@ -884,7 +990,7 @@ find_closest(const patricia_t *pt, const struct patricia_node *root,
 		const struct patricia_node *child;
 		gboolean matches;
 
-		g_assert(pn);
+		patricia_node_check(pn);
 
 		if (pn->leaf)
 			return pn;						/* We found the closest node */
@@ -938,6 +1044,8 @@ static void
 fill_leaf(struct patricia_node *pn, struct patricia_node *parent,
 	gconstpointer key, size_t keybits, gconstpointer value)
 {
+	patricia_node_check(pn);
+
 	pn->leaf = TRUE;
 	pn->has_embedded_data = FALSE;
 	pn->p.parent = parent;
@@ -955,11 +1063,16 @@ clear_embedded_data(struct patricia_node *pn)
 {
 	struct patricia_node *parent;
 
-	g_assert(pn);
+	patricia_node_check(pn);
 	g_assert(pn->has_embedded_data);
 
 	parent = parent_node(pn);
+
 	pn->has_embedded_data = FALSE;
+
+	patricia_parent_check(pn->p.ext);
+	patricia_parent_clear_magic(pn->p.ext);
+
 	wfree(pn->p.ext, sizeof *pn->p.ext);
 	pn->p.parent = parent;
 
@@ -975,8 +1088,8 @@ remove_useless_node(patricia_t *pt, struct patricia_node *pn)
 	struct patricia_node *parent;
 	struct patricia_node *child = NULL;
 
-	g_assert(pt);
-	g_assert(pn);
+	patricia_check(pt);
+	patricia_node_check(pn);
 
 	parent = parent_node(pn);
 
@@ -993,6 +1106,7 @@ remove_useless_node(patricia_t *pt, struct patricia_node *pn)
 	}
 
 	if (parent) {
+		patricia_node_check(parent);
 		if (child_one(parent) == pn) {
 			parent->u.children.o = child;
 		} else {
@@ -1017,8 +1131,8 @@ unembed_data(patricia_t *pt, struct patricia_node *pn)
 	size_t old_keybits;
 	struct patricia_node *parent;
 
-	g_assert(pt);
-	g_assert(pn);
+	patricia_check(pt);
+	patricia_node_check(pn);
 	g_assert(!pn->leaf);
 	g_assert(pn->has_embedded_data);
 	g_assert(child_one(pn) == NULL && child_zero(pn) == NULL);
@@ -1028,6 +1142,9 @@ unembed_data(patricia_t *pt, struct patricia_node *pn)
 	old_keybits = embedded_item_keybits(pn);
 	parent = parent_node(pn);
 
+	patricia_parent_check(pn->p.ext);
+	patricia_parent_clear_magic(pn->p.ext);
+
 	wfree(pn->p.ext, sizeof *pn->p.ext);
 
 	g_assert(old_keybits < pt->maxbits);
@@ -1062,6 +1179,8 @@ insert_at(
 	patricia_t *pt, struct patricia_node *pn,
 	gconstpointer key, size_t keybits, gconstpointer value)
 {
+	patricia_check(pt);
+	patricia_node_check(pn);
 	g_assert(keybits >= node_prefix_bits(pn));
 
 	if (!pn->leaf) {
@@ -1082,6 +1201,7 @@ insert_at(
 
 			/* New item */
 			ext = walloc(sizeof *ext);
+			patricia_parent_set_magic(ext);
 			ext->parent = pn->p.parent;
 			ext->key = key;
 			ext->value = value;
@@ -1121,6 +1241,7 @@ insert_at(
 		new->last_kbit = 0;			/* Does not matter for non-leaves */
 		new->bit = keybits;
 		new->p.ext = ext;
+		patricia_parent_set_magic(ext);
 		ext->key = key;
 		ext->value = value;
 		ext->parent = parent;
@@ -1162,6 +1283,9 @@ insert_below(
 {
 	struct patricia_node *new = allocate_node(pt);
 
+	patricia_check(pt);
+	patricia_node_check(pn);
+	patricia_node_check(new);
 	g_assert(common < keybits);
 
 	fill_leaf(new, pn, key, keybits, value);
@@ -1194,6 +1318,7 @@ insert_below(
 
 			g_assert(old_keybits < pt->maxbits);
 
+			patricia_parent_set_magic(ext);
 			ext->parent = pn->p.parent;
 			ext->key = old_key;
 			ext->value = old_value;
@@ -1257,6 +1382,9 @@ insert_above(
 	struct patricia_node *up;
 	struct patricia_node *child;
 
+	patricia_check(pt);
+	patricia_node_check(pn);
+	patricia_node_check(new);
 	g_assert(common <= keybits);
 
 	/*
@@ -1369,7 +1497,7 @@ patricia_insert_k(patricia_t *pt,
 	size_t common_bits;
 	gconstpointer prefix;
 
-	g_assert(pt);
+	patricia_check(pt);
 	g_assert(key);
 	g_assert(keybits <= pt->maxbits);
 
@@ -1760,8 +1888,8 @@ patricia_furthest_extended(
 static void
 remove_node(patricia_t *pt, struct patricia_node *pn)
 {
-	g_assert(pt);
-	g_assert(pn);
+	patricia_check(pt);
+	patricia_node_check(pn);
 
 	pt->stamp++;
 	pt->count--;
@@ -1776,6 +1904,7 @@ remove_node(patricia_t *pt, struct patricia_node *pn)
 		struct patricia_node *parent = parent_node(pn);
 		remove_useless_node(pt, pn);
 		if (parent) {
+			patricia_node_check(parent);
 			if (!parent->has_embedded_data)
 				remove_useless_node(pt, parent);	/* Has only one leaf now */
 			else if (child_one(parent) == NULL && child_zero(parent) == NULL)
@@ -1905,7 +2034,7 @@ traverse(patricia_t *pt,
 	node_cb_t ncb, gpointer un,
 	patricia_cb_t cb, gpointer u)
 {
-	g_assert(pt);
+	patricia_check(pt);
 
 	/*
 	 * Traverse without recursing for maximum efficiency.
@@ -1921,6 +2050,8 @@ traverse(patricia_t *pt,
 			const struct patricia_node *z = NULL;
 			const struct patricia_node *o = NULL;
 
+			patricia_node_check(n);
+
 			if (!n->leaf) {
 				z = child_zero(n);
 				o = child_one(n);
@@ -1968,7 +2099,7 @@ traverse_remove_node(struct patricia_node *pn, gpointer u)
 void
 patricia_destroy(patricia_t *pt)
 {
-	g_assert(pt);
+	patricia_check(pt);
 
 	if (--pt->refcnt > 0)
 		return;			/* Still referenced by something internally */
@@ -1976,6 +2107,7 @@ patricia_destroy(patricia_t *pt)
 	traverse(pt, traverse_remove_node, pt, NULL, NULL);
 	g_assert(pt->nodes == 0);
 
+	pt->magic = 0;
 	wfree(pt, sizeof *pt);
 }
 
@@ -1989,7 +2121,7 @@ patricia_destroy(patricia_t *pt)
 void
 patricia_foreach(const patricia_t *pt, patricia_cb_t cb, gpointer u)
 {
-	g_assert(pt);
+	patricia_check(pt);
 	g_assert(cb);
 
 	traverse(deconstify_gpointer(pt), NULL, NULL, cb, u);
@@ -2015,6 +2147,8 @@ traverse_foreach_node(struct patricia_node *pn, gpointer u)
 {
 	struct remove_ctx *ctx = u;
 
+	patricia_node_check(pn);
+
 	/*
 	 * ctx->last_was_removed was set possibly by previous call to the
 	 * user-supplied callback, through the traverse_foreach_item() trampoline.
@@ -2065,7 +2199,7 @@ patricia_foreach_remove(patricia_t *pt, patricia_cbr_t cb, gpointer u)
 	struct remove_ctx ctx;
 	GSList *sl;
 
-	g_assert(pt);
+	patricia_check(pt);
 	g_assert(cb);
 
 	ctx.sl = NULL;
@@ -2097,13 +2231,16 @@ patricia_foreach_remove(patricia_t *pt, patricia_cbr_t cb, gpointer u)
 enum patricia_iter_type {
 	PATRICIA_ITER_TREE	= 1,		/**< "lexicographic" traversal */
 	PATRICIA_ITER_XOR,				/**< "xor metric" traversal */
-	PATRICIA_ITER_XOR_LAZY,			/**< idem, key not copied */
+	PATRICIA_ITER_XOR_LAZY			/**< idem, key not copied */
 };
 
+enum patricia_iter_magic { PATRICIA_ITER_MAGIC = 0x64083edeU };
+
 /**
  * A PATRICIA iterator.
  */
 struct patricia_iter {
+	enum patricia_iter_magic magic;		/**< Magic number */
 	enum patricia_iter_type type;		/**< Type of iterator */
 	patricia_t *pt;						/**< The PATRICIA tree */
 	const struct patricia_node *last;	/**< Last visited node */
@@ -2115,12 +2252,20 @@ struct patricia_iter {
 	gboolean forward;				/**< Whether iterator is moving forward */
 };
 
+static inline void
+patricia_iter_check(const patricia_iter_t *iter)
+{
+	g_assert(iter != NULL);
+	g_assert(PATRICIA_ITER_MAGIC == iter->magic);
+}
+
 /**
  * Common iterator field initialization.
  */
 static patricia_iter_t *
 common_iter_init(patricia_iter_t *iter, patricia_t *pt, gboolean forward)
 {
+	iter->magic = PATRICIA_ITER_MAGIC;
 	iter->pt = pt;
 	iter->last = NULL;
 	iter->stamp = pt->stamp;
@@ -2142,7 +2287,7 @@ patricia_tree_iterator(patricia_t *pt, gboolean forward)
 {
 	struct patricia_iter *iter;
 
-	g_assert(pt);
+	patricia_check(pt);
 
 	iter = walloc0(sizeof *iter);
 
@@ -2176,7 +2321,7 @@ patricia_metric_iterator(patricia_t *pt, gconstpointer key, gboolean forward)
 	struct patricia_iter *iter;
 	size_t keybytes = bits2bytes(pt->maxbits);
 
-	g_assert(pt);
+	patricia_check(pt);
 	g_assert(pt->embedded == 0);
 
 	iter = walloc(sizeof *iter);
@@ -2191,7 +2336,7 @@ patricia_metric_iterator(patricia_t *pt, gconstpointer key, gboolean forward)
 }
 
 /**
- * Same as atricia_metric_iterator() but the initial key is not copied.
+ * Same as patricia_metric_iterator() but the initial key is not copied.
  * It must therefore remain allocated until the iterator is released, which
  * is usually the case when the iteration is done within a single routine.
  */
@@ -2201,7 +2346,7 @@ patricia_metric_iterator_lazy(
 {
 	struct patricia_iter *iter;
 
-	g_assert(pt);
+	patricia_check(pt);
 	g_assert(pt->embedded == 0);
 
 	iter = walloc(sizeof *iter);
@@ -2230,6 +2375,8 @@ next_tree_node(const struct patricia_node *prev, gboolean forward)
 		const struct patricia_node *parent = parent_node(pn);
 		const struct patricia_node *next = NULL;
 
+		patricia_node_check(pn);
+
 		if (!parent)
 			return NULL;
 		else if (forward && parent->u.children.z == pn)
@@ -2270,6 +2417,8 @@ next_metric_node(const patricia_t *pt, const struct patricia_node *prev,
 	for (pn = prev, i = 0; i <= PATRICIA_MAXBITS; i++) {
 		const struct patricia_node *parent = parent_node(pn);
 
+		patricia_node_check(pn);
+
 		if (!parent) {
 			return NULL;
 		} else {
@@ -2310,6 +2459,8 @@ next_item(patricia_iter_t *iter, const struct patricia_node *prev)
 {
 	const struct patricia_node *next = NULL;
 
+	patricia_iter_check(iter);
+
 	switch (iter->type) {
 	case PATRICIA_ITER_TREE:
 		next = next_tree_node(prev, iter->forward);
@@ -2336,6 +2487,8 @@ next_item(patricia_iter_t *iter, const struct patricia_node *prev)
 gboolean
 patricia_iter_has_next(patricia_iter_t *iter)
 {
+	patricia_iter_check(iter);
+
 	if (!iter->knows_next) {
 		g_assert(iter->stamp == iter->pt->stamp);
 		g_assert(iter->last != NULL);
@@ -2361,6 +2514,8 @@ patricia_iter_next_value(patricia_iter_t *iter)
 {
 	gboolean has_next;
 
+	patricia_iter_check(iter);
+
 	has_next = iter->knows_next ?
 		iter->next != NULL : patricia_iter_has_next(iter);
 
@@ -2388,6 +2543,8 @@ patricia_iter_next(patricia_iter_t *iter,
 {
 	gboolean has_next;
 
+	patricia_iter_check(iter);
+
 	has_next = iter->knows_next ?
 		iter->next != NULL : patricia_iter_has_next(iter);
 
@@ -2417,11 +2574,14 @@ patricia_iterator_release(patricia_iter_t **iter_ptr)
 	iter = *iter_ptr;
 
 	if (iter) {
+		patricia_iter_check(iter);
+
 		patricia_destroy(iter->pt);
 		if (iter->type == PATRICIA_ITER_XOR) {
 			wfree(iter->key, bits2bytes(iter->keybits));
 			iter->key = NULL;
 		}
+		iter->magic = 0;
 		wfree(iter, sizeof *iter);
 		*iter_ptr = NULL;
 	}
@@ -2690,6 +2850,7 @@ test_keys(guint32 keys[], size_t nkeys)
 	g_assert(pt->embedded == 0);
 
 	G_FREE_NULL(data);
+	patricia_destroy(pt);
 }
 
 /**
