@@ -1,5 +1,5 @@
 /*
- * $Id: pmsg.c 16108 2009-03-10 15:19:52Z rmanfredi $
+ * $Id: pmsg.c 17159 2009-11-02 18:14:33Z rmanfredi $
  *
  * Copyright (c) 2002-2003, Raphael Manfredi
  *
@@ -35,9 +35,10 @@
 
 #include "common.h"
 
-RCSID("$Id: pmsg.c 16108 2009-03-10 15:19:52Z rmanfredi $")
+RCSID("$Id: pmsg.c 17159 2009-11-02 18:14:33Z rmanfredi $")
 
 #include "pmsg.h"
+#include "halloc.h"
 #include "zalloc.h"
 #include "walloc.h"
 #include "override.h"			/* Must be the last header included */
@@ -388,6 +389,20 @@ pmsg_free(pmsg_t *mb)
 	pdata_unref(db);
 }
 
+/**
+ * Free message block referenced in the variable and nullify it.
+ */
+void
+pmsg_free_null(pmsg_t **mb_ptr)
+{
+	pmsg_t *mb = *mb_ptr;
+
+	if (mb) {
+		pmsg_free(mb);
+		*mb_ptr = NULL;
+	}
+}
+
 /**
  * @return amount of data that can be written at the end of the message.
  */
@@ -530,7 +545,7 @@ pdata_allocb(void *buf, int len, pdata_free_t freecb, gpointer freearg)
 	pdata_t *db;
 
 	g_assert(valid_ptr(buf));
-	g_assert(len >= (int) EMBEDDED_OFFSET);
+	g_assert(UNSIGNED(len) >= EMBEDDED_OFFSET);
 	g_assert(implies(freecb, valid_ptr(freecb)));
 
 	db = buf;
@@ -631,6 +646,8 @@ pdata_unref(pdata_t *db)
 
 /**
  * Creates an iovec from a singly-linked list of pmsg_t buffers.
+ * It should be freed via hfree().
+ *
  * NOTE: The iovec will hold no more than MAX_IOV_COUNT items. That means
  *       the iovec might not cover the whole buffered data. This limited
  *		 is applied because writev() could fail with EINVAL otherwise
@@ -652,7 +669,7 @@ pmsg_slist_to_iovec(slist_t *slist, int *iovcnt_ptr, size_t *size_ptr)
 		int i;
 
 		n = MIN(n, MAX_IOV_COUNT);
-		iov = g_malloc(n * sizeof *iov);
+		iov = halloc(n * sizeof *iov);
 
 		iter = slist_iter_before_head(slist);
 		for (i = 0; i < n; i++) {
@@ -715,11 +732,13 @@ pmsg_slist_discard(slist_t *slist, size_t n_bytes)
 	slist_iter_free(&iter);
 }
 
+#define PMSG_SLIST_GROW_MIN	1024	/**< Minimum to allocate on new blocks */
+
 /**
  * Appends `n_bytes' to the pmsg_t buffer. If the last pmsg_t is writable
  * it is filled with as much data as space is still available. Otherwise
  * or if this space is not sufficient another pmsg_t is created and
- * append to the list.
+ * appendded to the list.
  */
 void
 pmsg_slist_append(slist_t *slist, const void *data, size_t n_bytes)
@@ -741,7 +760,7 @@ pmsg_slist_append(slist_t *slist, const void *data, size_t n_bytes)
 		n_bytes -= n;
 	}
 	if (n_bytes > 0) {
-		mb = pmsg_new(PMSG_P_DATA, NULL, n_bytes);
+		mb = pmsg_new(PMSG_P_DATA, NULL, MAX(n_bytes, PMSG_SLIST_GROW_MIN));
 		pmsg_write(mb, data, n_bytes);
 		slist_append(slist, mb);
 	}
