@@ -1,5 +1,5 @@
 /*
- * $Id: prop.c 16108 2009-03-10 15:19:52Z rmanfredi $
+ * $Id: prop.c 17078 2009-10-16 21:12:34Z rmanfredi $
  *
  * Copyright (c) 2001-2003, Richard Eckart
  *
@@ -25,17 +25,25 @@
 
 #include "common.h"
 
-RCSID("$Id: prop.c 16108 2009-03-10 15:19:52Z rmanfredi $")
+RCSID("$Id: prop.c 17078 2009-10-16 21:12:34Z rmanfredi $")
+
+#include "gtk-gnutella.h"
 
 #include "ascii.h"
+#include "concat.h"
 #include "getdate.h"
 #include "prop.h"
 #include "debug.h"
 #include "file.h"
-#include "misc.h"
-#include "sha1.h"
 #include "glib-missing.h"
+#include "halloc.h"
+#include "parse.h"
+#include "path.h"
+#include "sha1.h"
+#include "stringify.h"
+#include "timestamp.h"
 #include "tm.h"
+#include "walloc.h"
 #include "override.h"		/* Must be the last header included */
 
 #define PROP_FILE_ID	"_id"
@@ -340,7 +348,7 @@ prop_parse_storage(const char *name, const char *str, size_t size, char *t)
 
 /**
  * Copy the property definition from the property set and return it.
- * Use the prop_free_def call to free the memory again. A simple g_free
+ * Use the prop_free_def call to free the memory again. A simple hfree
  * won't do, since there are lot's of pointers to allocated memory
  * in the definition structure.
  *
@@ -356,17 +364,17 @@ prop_get_def(prop_set_t *ps, property_t p)
 	if (!prop_in_range(ps, p))
 		g_error("prop_get_def: unknown property %d", p);
 
-	buf = g_memdup(&PROP(ps, p), sizeof(prop_def_t));
-	buf->name = g_strdup(PROP(ps, p).name);
-	buf->desc = g_strdup(PROP(ps, p).desc);
+	buf = wcopy(&PROP(ps, p), sizeof(prop_def_t));
+	buf->name = h_strdup(PROP(ps, p).name);
+	buf->desc = h_strdup(PROP(ps, p).desc);
 	buf->ev_changed = NULL;
 
 	switch (buf->type) {
 	case PROP_TYPE_BOOLEAN:
-		buf->data.boolean.def = g_memdup(
+		buf->data.boolean.def = hcopy(
 			PROP(ps,p).data.boolean.def,
 			sizeof(gboolean) * PROP(ps,p).vector_size);
-		buf->data.boolean.value = g_memdup(
+		buf->data.boolean.value = hcopy(
 			PROP(ps,p).data.boolean.value,
 			sizeof(gboolean) * PROP(ps,p).vector_size);
 		break;
@@ -378,7 +386,7 @@ prop_get_def(prop_set_t *ps, property_t p)
 
 		n ++; /* Keep space for terminating {NULL, 0} field */
 
-		buf->data.guint32.choices = g_memdup(
+		buf->data.guint32.choices = hcopy(
 			PROP(ps,p).data.guint32.choices,
 			sizeof(prop_def_choice_t) * n);
 
@@ -388,53 +396,53 @@ prop_get_def(prop_set_t *ps, property_t p)
 		n = 0;
 		while (PROP(ps,p).data.guint32.choices[n].title != NULL) {
 			buf->data.guint32.choices[n].title =
-				g_strdup(PROP(ps,p).data.guint32.choices[n].title);
+				h_strdup(PROP(ps,p).data.guint32.choices[n].title);
 			n++;
 		}
 		/* no break -> continue to PROP_TYPE_GUINT32 */
 	}
 	case PROP_TYPE_GUINT32:
-		buf->data.guint32.def = g_memdup(
+		buf->data.guint32.def = hcopy(
 			PROP(ps,p).data.guint32.def,
 			sizeof(guint32) * PROP(ps,p).vector_size);
-		buf->data.guint32.value = g_memdup(
+		buf->data.guint32.value = hcopy(
 			PROP(ps,p).data.guint32.value,
 			sizeof(guint32) * PROP(ps,p).vector_size);
 		break;
 
 	case PROP_TYPE_GUINT64:
-		buf->data.guint64.def = g_memdup(
+		buf->data.guint64.def = hcopy(
 			PROP(ps,p).data.guint64.def,
 			sizeof(guint64) * PROP(ps,p).vector_size);
-		buf->data.guint64.value = g_memdup(
+		buf->data.guint64.value = hcopy(
 			PROP(ps,p).data.guint64.value,
 			sizeof(guint64) * PROP(ps,p).vector_size);
 		break;
 
 	case PROP_TYPE_TIMESTAMP:
-		buf->data.timestamp.def = g_memdup(
+		buf->data.timestamp.def = hcopy(
 			PROP(ps,p).data.timestamp.def,
 			sizeof(time_t) * PROP(ps,p).vector_size);
-		buf->data.timestamp.value = g_memdup(
+		buf->data.timestamp.value = hcopy(
 			PROP(ps,p).data.timestamp.value,
 			sizeof(time_t) * PROP(ps,p).vector_size);
 		break;
 
 	case PROP_TYPE_IP:
-		buf->data.ip.value = g_memdup(PROP(ps,p).data.ip.value,
+		buf->data.ip.value = hcopy(PROP(ps,p).data.ip.value,
 			sizeof buf->data.ip.value * PROP(ps,p).vector_size);
 		break;
 
 	case PROP_TYPE_STRING:
-		buf->data.string.def	= g_new(char*, 1);
-		*buf->data.string.def   = g_strdup(*PROP(ps,p).data.string.def);
-		buf->data.string.value  = g_new(char*, 1);
-		*buf->data.string.value = g_strdup(*PROP(ps,p).data.string.value);
+		buf->data.string.def	= walloc(sizeof(char *));
+		*buf->data.string.def   = h_strdup(*PROP(ps,p).data.string.def);
+		buf->data.string.value  = walloc(sizeof(char *));
+		*buf->data.string.value = h_strdup(*PROP(ps,p).data.string.value);
 		break;
 
 	case PROP_TYPE_STORAGE:
-		buf->data.storage.value = g_memdup
-			(PROP(ps,p).data.storage.value, PROP(ps,p).vector_size);
+		buf->data.storage.value = hcopy(
+			PROP(ps,p).data.storage.value, PROP(ps,p).vector_size);
 		break;
 		
 	case NUM_PROP_TYPES:
@@ -451,50 +459,50 @@ prop_free_def(prop_def_t *d)
 
 	switch (d->type) {
 	case PROP_TYPE_BOOLEAN:
-		G_FREE_NULL(d->data.boolean.value);
-		G_FREE_NULL(d->data.boolean.def);
+		HFREE_NULL(d->data.boolean.value);
+		HFREE_NULL(d->data.boolean.def);
 		break;
 	case PROP_TYPE_MULTICHOICE: {
 		guint n = 0;
 
 		while (d->data.guint32.choices[n].title != NULL) {
-			G_FREE_NULL(d->data.guint32.choices[n].title);
+			HFREE_NULL(d->data.guint32.choices[n].title);
 			n++;
 		}
 
-		G_FREE_NULL(d->data.guint32.choices);
+		HFREE_NULL(d->data.guint32.choices);
 		/* no break -> continue to PROP_TYPE_GUINT32 */
 	}
 	case PROP_TYPE_GUINT32:
-		G_FREE_NULL(d->data.guint32.value);
-		G_FREE_NULL(d->data.guint32.def);
+		HFREE_NULL(d->data.guint32.value);
+		HFREE_NULL(d->data.guint32.def);
 		break;
 	case PROP_TYPE_GUINT64:
-		G_FREE_NULL(d->data.guint64.value);
-		G_FREE_NULL(d->data.guint64.def);
+		HFREE_NULL(d->data.guint64.value);
+		HFREE_NULL(d->data.guint64.def);
 		break;
 	case PROP_TYPE_TIMESTAMP:
-		G_FREE_NULL(d->data.timestamp.value);
-		G_FREE_NULL(d->data.timestamp.def);
+		HFREE_NULL(d->data.timestamp.value);
+		HFREE_NULL(d->data.timestamp.def);
 		break;
 	case PROP_TYPE_IP:
-		G_FREE_NULL(d->data.ip.value);
+		HFREE_NULL(d->data.ip.value);
 		break;
 	case PROP_TYPE_STRING:
-		G_FREE_NULL(*d->data.string.value);
-		G_FREE_NULL(*d->data.string.def);
-		G_FREE_NULL(d->data.string.value);
-		G_FREE_NULL(d->data.string.def);
+		HFREE_NULL(*d->data.string.value);
+		HFREE_NULL(*d->data.string.def);
+		WFREE_NULL(d->data.string.value, sizeof(char *));
+		WFREE_NULL(d->data.string.def, sizeof(char *));
 		break;
 	case PROP_TYPE_STORAGE:
-		G_FREE_NULL(d->data.storage.value);
+		HFREE_NULL(d->data.storage.value);
 		break;
 	case NUM_PROP_TYPES:
 		g_assert_not_reached();
 	}
-	G_FREE_NULL(d->name);
-	G_FREE_NULL(d->desc);
-	G_FREE_NULL(d);
+	HFREE_NULL(d->name);
+	HFREE_NULL(d->desc);
+	WFREE_NULL(d, sizeof *d);
 }
 
 /**
@@ -1184,32 +1192,18 @@ prop_set_string(prop_set_t *ps, property_t prop, const char *val)
 	prop_assert(ps, prop, PROP(ps,prop).vector_size == 1);
 
 	old = *PROP(ps,prop).data.string.value;
-
-	if (val == NULL) {
-		/*
-		 * Clear property.
-		 */
-		if (old != NULL) {
-			G_FREE_NULL(old);
-			differ = TRUE;
-		}
+	if (old && val) {
+		differ = 0 != strcmp(old, val);
 	} else {
-		/*
-		 * Update property.
-		 */
-		if (old != NULL) {
-			differ = strcmp(old, val) != 0;
-			G_FREE_NULL(old);
-		} else {
-			differ = TRUE;
-		}
-		*PROP(ps,prop).data.string.value = g_strdup(val);
+		differ = old != val;
 	}
+	*PROP(ps,prop).data.string.value = g_strdup(val);
+	G_FREE_NULL(old);
 
 	if (differ && debug >= 5)
 		printf("updated property [%s] = \"%s\"\n",
 			PROP(ps,prop).name,
-			*PROP(ps,prop).data.string.value);
+			NULL_STRING(*PROP(ps,prop).data.string.value));
 
 	if (differ)
 		prop_emit_prop_changed(ps, prop);
@@ -1259,7 +1253,7 @@ prop_get_string(prop_set_t *ps, property_t prop, char *t, size_t size)
 			target[0] = '\0';
 			target = NULL;
 		} else {
-			g_strlcpy(target, s, size);
+			clamp_strcpy(target, size, s);
 		}
 	}
 
@@ -1367,7 +1361,7 @@ prop_to_string(prop_set_t *ps, property_t prop)
 			gboolean val;
 
 			prop_get_boolean(ps, prop, &val, 0, 1);
-			g_strlcpy(s, val ? "TRUE" : "FALSE", sizeof s);
+			clamp_strcpy(s, sizeof s, val ? "TRUE" : "FALSE");
 		}
 		break;
 	case PROP_TYPE_MULTICHOICE:
@@ -1426,13 +1420,13 @@ prop_default_to_string(prop_set_t *ps, property_t prop)
 		uint64_to_string_buf(p->data.timestamp.def[0], s, sizeof s);
 		break;
 	case PROP_TYPE_STRING:
-		g_strlcpy(s, *p->data.string.def ? *p->data.string.def : "", sizeof s);
+		clamp_strcpy(s, sizeof s, *p->data.string.def ? *p->data.string.def : "");
 		break;
 	case PROP_TYPE_IP:
-		g_strlcpy(s, "", sizeof s);
+		clamp_strcpy(s, sizeof s, "");
 		break;
 	case PROP_TYPE_BOOLEAN:
-		g_strlcpy(s, p->data.boolean.def[0] ? "TRUE" : "FALSE", sizeof s);
+		clamp_strcpy(s, sizeof s, p->data.boolean.def[0] ? "TRUE" : "FALSE");
 		break;
 	case PROP_TYPE_MULTICHOICE:
 		{
@@ -1598,7 +1592,7 @@ prop_save_to_file(prop_set_t *ps, const char *dir, const char *filename)
 		 */
 
 		if (ps->mtime && delta_time(sb.st_mtime, ps->mtime) > 0) {
-			char *old = g_strconcat(pathname, ".old", (void *) 0);
+			char *old = h_strconcat(pathname, ".old", (void *) 0);
 			g_warning("config file \"%s\" changed whilst I was running",
 				pathname);
 			if (-1 == rename(pathname, old))
@@ -1606,7 +1600,7 @@ prop_save_to_file(prop_set_t *ps, const char *dir, const char *filename)
 					old, g_strerror(errno));
 			else
 				g_warning("renamed old copy as \"%s\"", old);
-			G_FREE_NULL(old);
+			HFREE_NULL(old);
 		}
 	}
 
@@ -1615,7 +1609,7 @@ prop_save_to_file(prop_set_t *ps, const char *dir, const char *filename)
 	 * clobber a good configuration file should we fail abruptly.
 	 */
 
-	newfile = g_strconcat(pathname, ".new", (void *) 0);
+	newfile = h_strconcat(pathname, ".new", (void *) 0);
 	config = file_fopen(newfile, "w");
 
 	if (config == NULL)
@@ -1653,7 +1647,7 @@ prop_save_to_file(prop_set_t *ps, const char *dir, const char *filename)
 		if (p->save == FALSE)
 			continue;
 
-		vbuf = g_new(char*, p->vector_size+1);
+		vbuf = halloc((p->vector_size + 1) * sizeof(char *));
 		vbuf[0] = NULL;
 
 		{
@@ -1671,11 +1665,11 @@ prop_save_to_file(prop_set_t *ps, const char *dir, const char *filename)
 				v = p->data.boolean.value[i];
 				if (v != p->data.boolean.def[i])
 					defaultvalue = FALSE;
-				vbuf[i] = g_strdup(config_boolean(v));
+				vbuf[i] = h_strdup(config_boolean(v));
 			}
 			vbuf[p->vector_size] = NULL;
 
-			val = g_strjoinv(",", vbuf);
+			val = h_strjoinv(",", vbuf);
 			break;
 		case PROP_TYPE_MULTICHOICE:
 		case PROP_TYPE_GUINT32:
@@ -1686,11 +1680,11 @@ prop_save_to_file(prop_set_t *ps, const char *dir, const char *filename)
 				if (v != p->data.guint32.def[i])
 					defaultvalue = FALSE;
 				gm_snprintf(sbuf, sizeof(sbuf), "%u", v);
-				vbuf[i] = g_strdup(sbuf);
+				vbuf[i] = h_strdup(sbuf);
 			}
 			vbuf[p->vector_size] = NULL;
 
-			val = g_strjoinv(",", vbuf);
+			val = h_strjoinv(",", vbuf);
 			break;
 		case PROP_TYPE_GUINT64:
 			for (i = 0; i < p->vector_size; i++) {
@@ -1701,11 +1695,11 @@ prop_save_to_file(prop_set_t *ps, const char *dir, const char *filename)
 					defaultvalue = FALSE;
 
 				uint64_to_string_buf(v, sbuf, sizeof sbuf);
-				vbuf[i] = g_strdup(sbuf);
+				vbuf[i] = h_strdup(sbuf);
 			}
 			vbuf[p->vector_size] = NULL;
 
-			val = g_strjoinv(",", vbuf);
+			val = h_strjoinv(",", vbuf);
 			break;
 		case PROP_TYPE_TIMESTAMP:
 			for (i = 0; i < p->vector_size; i++) {
@@ -1716,14 +1710,14 @@ prop_save_to_file(prop_set_t *ps, const char *dir, const char *filename)
 					defaultvalue = FALSE;
 
 				timestamp_utc_to_string_buf(t, sbuf, sizeof sbuf);
-				vbuf[i] = g_strdup(sbuf);
+				vbuf[i] = h_strdup(sbuf);
 			}
 			vbuf[p->vector_size] = NULL;
-			val = g_strjoinv(",", vbuf);
+			val = h_strjoinv(",", vbuf);
 			quotes = TRUE;
 			break;
 		case PROP_TYPE_STRING:
-			val = g_strdup(*p->data.string.value);
+			val = h_strdup(*p->data.string.value);
 			if (
 				val != *p->data.string.def &&
 				NULL != val &&
@@ -1732,6 +1726,10 @@ prop_save_to_file(prop_set_t *ps, const char *dir, const char *filename)
 			) {
 				defaultvalue = FALSE;
 			}
+			if (NULL == val) {
+				val = h_strdup("");
+				defaultvalue = FALSE;
+			}
 			quotes = TRUE;
 			break;
 		case PROP_TYPE_IP:
@@ -1739,11 +1737,11 @@ prop_save_to_file(prop_set_t *ps, const char *dir, const char *filename)
 				host_addr_t addr;
 
 				addr = p->data.ip.value[i];
-				vbuf[i] = g_strdup(host_addr_to_string(addr));
+				vbuf[i] = h_strdup(host_addr_to_string(addr));
 			}
 			vbuf[p->vector_size] = NULL;
 
-			val = g_strjoinv(",", vbuf);
+			val = h_strjoinv(",", vbuf);
 			quotes = TRUE;
 			defaultvalue = FALSE;
 			break;
@@ -1751,7 +1749,7 @@ prop_save_to_file(prop_set_t *ps, const char *dir, const char *filename)
 			{
 				size_t hex_size = (p->vector_size * 2) + 1;
 
-				val = g_malloc(hex_size);
+				val = halloc(hex_size);
 				bin_to_hex_buf(p->data.storage.value, p->vector_size,
 					val, hex_size);
 				quotes = TRUE;
@@ -1769,8 +1767,8 @@ prop_save_to_file(prop_set_t *ps, const char *dir, const char *filename)
 		fprintf(config, "%s%s = %s%s%s\n\n", defaultvalue ? "#" : "",
 			p->name, quotes ? "\"" : "", val, quotes ? "\"" : "");
 
-		G_FREE_NULL(val);
-		g_strfreev(vbuf);
+		HFREE_NULL(val);
+		h_strfreev(vbuf);
 	}
 
 	/*
@@ -1800,8 +1798,8 @@ prop_save_to_file(prop_set_t *ps, const char *dir, const char *filename)
 		g_warning("could not flush %s: %s", newfile, g_strerror(errno));
 
 end:
-	G_FREE_NULL(newfile);
-	G_FREE_NULL(pathname);
+	HFREE_NULL(newfile);
+	HFREE_NULL(pathname);
 }
 
 /**
@@ -1954,7 +1952,7 @@ prop_load_from_file(prop_set_t *ps, const char *dir, const char *filename)
 	path = make_pathname(dir, filename);
 	config = file_fopen(path, "r");
 	if (!config) {
-		G_FREE_NULL(path);
+		HFREE_NULL(path);
 		return TRUE;
 	}
 
@@ -1967,7 +1965,7 @@ prop_load_from_file(prop_set_t *ps, const char *dir, const char *filename)
 		file_id = unique_file_token(&buf);
 	}
 
-	G_FREE_NULL(path);
+	HFREE_NULL(path);
 
 	/*
 	 * Lines should match the following expression:
