@@ -1,5 +1,5 @@
 /*
- * $Id: socket.h,v 1.2 2005/12/22 10:24:06 cbiere Exp $
+ * $Id: socket.h 12500 2006-11-26 10:53:21Z cbiere $
  * 
  * Copyright (c) 2005, Jeroen Asselman
  *
@@ -23,6 +23,12 @@
  * ----------------------------------------------------------------------
  */
 
+#ifndef _lib_socket_h_
+#define _lib_socket_h_
+
+#include "common.h"
+
 void socket_set_nonblocking(int fd);
 
+#endif	/* _lib_socket_h_ */
 /* vi: set ts=4 sw=4 cindent: */
