@@ -1,5 +1,5 @@
 /*
- * $Id: stats.c 16188 2009-03-17 17:42:15Z rmanfredi $
+ * $Id: stats.c 17033 2009-10-05 20:51:00Z rmanfredi $
  *
  * Copyright (c) 2004, Raphael Manfredi
  *
@@ -35,7 +35,7 @@
 
 #include "common.h"
 
-RCSID("$Id: stats.c 16188 2009-03-17 17:42:15Z rmanfredi $")
+RCSID("$Id: stats.c 17033 2009-10-05 20:51:00Z rmanfredi $")
 
 #ifdef I_MATH
 #include <math.h>
@@ -43,6 +43,7 @@ RCSID("$Id: stats.c 16188 2009-03-17 17:42:15Z rmanfredi $")
 
 #include "stats.h"
 #include "glib-missing.h"	/* For g_slist_delete_link() */
+#include "halloc.h"
 #include "walloc.h"
 #include "override.h"		/* Must be the last header included */
 
@@ -140,8 +141,9 @@ statx_opx(statx_t *sx, double val, stats_op_t op)
 
 			for (l = sx->data; l; l = g_slist_next(l)) {
 				double *vp = (double *) l->data;
+				double delta = *vp - val;
 
-				if (*vp == val) {
+				if (ABS(delta) < 1e-56) {
 					sx->data = g_slist_remove(sx->data, vp);
 					wfree(vp, sizeof(*vp));
 					break;
@@ -270,7 +272,7 @@ statx_var(const statx_t *sx)
 }
 
 /**
- * @return an array of datapoints which can be freed when done.
+ * @return an array of datapoints which can be freed via hfree() when done.
  */
 double *
 statx_data(const statx_t *sx)
@@ -282,7 +284,7 @@ statx_data(const statx_t *sx)
 	g_assert(!sx->no_data);
 	g_assert(sx->n > 0);
 
-	array = g_malloc(sizeof(double) * sx->n);
+	array = halloc(sizeof(double) * sx->n);
 
 	for (i = 0, l = sx->data; i < sx->n && l; l = g_slist_next(l), i++) {
 		double *vp = (double *) l->data;
