@@ -1,5 +1,5 @@
 /*
- * $Id: tiger.c 13583 2007-05-10 02:06:12Z cbiere $
+ * $Id: tiger.c 16103 2009-03-10 14:53:39Z rmanfredi $
  *
  * Copyright (c) 2003, Jeroen Asselman
  *
@@ -48,7 +48,7 @@
 
 #include "common.h"
 
-RCSID("$Id: tiger.c 13583 2007-05-10 02:06:12Z cbiere $")
+RCSID("$Id: tiger.c 16103 2009-03-10 14:53:39Z rmanfredi $")
 
 #include "endian.h"
 #include "misc.h"
@@ -165,7 +165,7 @@ tiger_compress(const guint64 *data, guint64 state[3])
 }
 
 void
-tiger(gconstpointer data, guint64 length, gchar hash[24])
+tiger(gconstpointer data, guint64 length, char hash[24])
 {
   guint64 i, j, res[3];
   const guint8 *data_u8 = data;
@@ -249,7 +249,7 @@ tiger(gconstpointer data, guint64 length, gchar hash[24])
 void
 tiger_check(void)
 {
-	static const gchar zeros[1025];
+	static const char zeros[1025];
     static const struct {
 		const char *r;
 		const char *s;
@@ -266,8 +266,8 @@ tiger_check(void)
 	guint i;
 
 	for (i = 0; i < G_N_ELEMENTS(tests); i++) {
-		gchar hash[24];
-		gchar buf[40];
+		char hash[24];
+		char buf[40];
 		gboolean ok;
 
 		memset(buf, 0, sizeof buf);	
