@@ -1,5 +1,5 @@
 /*
- * $Id: tiger.h 13285 2007-04-13 01:03:20Z cbiere $
+ * $Id: tiger.h 16103 2009-03-10 14:53:39Z rmanfredi $
  *
  * Copyright (c) 2003, Jeroen Asselman
  *
@@ -39,7 +39,7 @@
 #include "common.h"
 
 void tiger_check(void);
-void tiger(gconstpointer data, guint64 length, gchar hash[24]);
+void tiger(gconstpointer data, guint64 length, char hash[24]);
 
 #endif /* _tiger_h_ */
 /* vi: set ts=4 sw=4 cindent: */
