@@ -1,5 +1,5 @@
 /*
- * $Id: url.c 16242 2009-03-24 19:04:20Z cbiere $
+ * $Id: url.c 17240 2010-02-13 12:26:50Z rmanfredi $
  *
  * Copyright (c) 2002-2003, Raphael Manfredi
  *
@@ -35,13 +35,15 @@
 
 #include "common.h"
 
-RCSID("$Id: url.c 16242 2009-03-24 19:04:20Z cbiere $")
+RCSID("$Id: url.c 17240 2010-02-13 12:26:50Z rmanfredi $")
 
 #include "ascii.h"
 #include "glib-missing.h"
+#include "halloc.h"
 #include "host_addr.h"
+#include "parse.h"
+#include "path.h"
 #include "url.h"
-#include "misc.h"
 #include "walloc.h"
 #include "override.h"		/* Must be the last header included */
 
@@ -94,7 +96,8 @@ static const char hex_alphabet[] = "0123456789ABCDEF";
  * @param `url' no brief description.
  * @param `mask' tells us whether we're escaping an URL path or a query string.
  *
- * @return argument if no escaping is necessary, or a new string otherwise.
+ * @return argument if no escaping is necessary, or a new string otherwise,
+ * which must be freed via hfree().
  */
 static char *
 url_escape_mask(const char *url, guint8 mask)
@@ -112,7 +115,7 @@ url_escape_mask(const char *url, guint8 mask)
 	if (need_escape == 0)
 		return deconstify_gchar(url);
 
-	new = g_malloc(p - url + (need_escape << 1));
+	new = halloc(p - url + (need_escape << 1));
 
 	for (p = url, q = new, c = *p++; c; c = *p++) {
 		if (is_transparent_char(c, mask))
@@ -214,7 +217,7 @@ url_from_absolute_path(const char *path)
 	escaped = url_escape_mask(path, SHELL_MASK);
 	url = g_strconcat("file://", escaped, (void *) 0);
 	if (escaped != path) {
-		G_FREE_NULL(escaped);
+		HFREE_NULL(escaped);
 	}
 	return url;
 }
@@ -285,7 +288,7 @@ url_escape_cntrl(const char *url)
 		guchar c;
 
 		size = p - url + 1 + need_escape * 2;
-		escaped = g_malloc(size);
+		escaped = halloc(size);
 		q = escaped;
 
 		for (p = url; '\0' != (c = *p); p++) {
@@ -349,7 +352,7 @@ url_unescape(char *url, gboolean inplace)
 		new = url;
 	else {
 		unescaped_memory = p - url - (need_unescape << 1) + 1;
-		new = g_malloc(unescaped_memory);
+		new = halloc(unescaped_memory);
 	}
 
 	for (p = url, q = new, c = *p++; c; c = *p++) {
@@ -408,12 +411,12 @@ url_params_parse(char *query)
 				*q = '\0';
 				value = url_unescape(start, FALSE);
 				if (!value) {
-					G_FREE_NULL(name);
+					HFREE_NULL(name);
 					url_params_free(up);
 					return NULL;
 				}
 				if (value == start)				/* No unescaping took place */
-					value = g_strdup(start);
+					value = h_strdup(start);
 				*q = c;
 				g_hash_table_insert(up->params, name, value);
 				up->count++;
@@ -425,7 +428,7 @@ url_params_parse(char *query)
 		} else {
 			if (c == '=') {						/* End of parameter name */
 				*q = '\0';
-				name = g_strdup(start);
+				name = h_strdup(start);
 				*q = c;
 				in_value = TRUE;
 				start = q + 1;					/* Value will start there */
@@ -459,8 +462,8 @@ static void
 free_params_kv(gpointer key, gpointer value, gpointer unused_udata)
 {
 	(void) unused_udata;
-	G_FREE_NULL(key);
-	G_FREE_NULL(value);
+	HFREE_NULL(key);
+	HFREE_NULL(value);
 }
 
 /**
