@@ -1,5 +1,5 @@
 /*
- * $Id: url.h 13958 2007-06-24 04:31:11Z cbiere $
+ * $Id: url.h 16108 2009-03-10 15:19:52Z rmanfredi $
  *
  * Copyright (c) 2002-2003, Raphael Manfredi
  *
@@ -43,7 +43,7 @@
  */
 typedef struct {
 	GHashTable *params;		/**< parameter => value (malloc'ed) */
-	gint count;				/**< Amount of parameters */
+	int count;				/**< Amount of parameters */
 } url_params_t;
 
 #define url_params_count(x)	((x)->count)
@@ -62,19 +62,19 @@ typedef enum {
  * Public interface.
  */
 
-gchar *url_escape(const gchar *url);
-gchar *url_escape_query(const gchar *url);
-gchar *url_escape_shell(const gchar *url);
-gchar *url_fix_escape(const gchar *url);
-gint url_escape_into(const gchar *url, gchar *target, gint len);
-gchar *url_escape_cntrl(const gchar *url);
-gchar *url_unescape(gchar *url, gboolean inplace);
-gchar *url_from_absolute_path(const gchar *path);
+char *url_escape(const char *url);
+char *url_escape_query(const char *url);
+char *url_escape_shell(const char *url);
+char *url_fix_escape(const char *url);
+int url_escape_into(const char *url, char *target, int len);
+char *url_escape_cntrl(const char *url);
+char *url_unescape(char *url, gboolean inplace);
+char *url_from_absolute_path(const char *path);
 
-url_params_t *url_params_parse(gchar *query);
-const gchar *url_params_get(url_params_t *up, const gchar *name);
+url_params_t *url_params_parse(char *query);
+const char *url_params_get(url_params_t *up, const char *name);
 void url_params_free(url_params_t *up);
-gchar *url_normalize(gchar *url, url_policy_t pol);
+char *url_normalize(char *url, url_policy_t pol);
 
 #endif	/* _url_h_ */
 
