@@ -1,7 +1,7 @@
 /*
- * $Id: urn.c 16103 2009-03-10 14:53:39Z rmanfredi $
+ * $Id: urn.c 17162 2009-11-03 19:55:08Z rmanfredi $
  *
- * Copyright (c) 2002-2004, Raphael Manfredi
+ * Copyright (c) 2002-2004, 2009, Raphael Manfredi
  *
  *----------------------------------------------------------------------
  * This file is part of gtk-gnutella.
@@ -30,13 +30,14 @@
  * URN handling of specific formats.
  *
  * @author Raphael Manfredi
- * @date 2002-2004
+ * @date 2002-2004, 2009
  */
 
 #include "common.h"
 
-RCSID("$Id: urn.c 16103 2009-03-10 14:53:39Z rmanfredi $")
+RCSID("$Id: urn.c 17162 2009-11-03 19:55:08Z rmanfredi $")
 
+#include "base16.h"
 #include "base32.h"
 #include "misc.h"
 #include "urn.h"
@@ -70,6 +71,35 @@ parse_base32_sha1(const char *buf, size_t size, struct sha1 *sha1)
 	return TRUE;
 }
 
+/**
+ * Validate SHA1 starting in NUL-terminated `buf' as a proper base16 encoding
+ * of a SHA1 hash, and write decoded value in `retval'.
+ *
+ * The SHA1 typically comes from HTTP or magnet URIs, made by people who do
+ * not follow the specs which says a magnet must hold a base32-encoded SHA1.
+ *
+ * @return TRUE if the SHA1 was valid and properly decoded, FALSE on error.
+ */
+gboolean
+parse_base16_sha1(const char *buf, size_t size, struct sha1 *sha1)
+{
+	struct sha1 raw;
+	size_t len;
+
+	if (!sha1) {
+		sha1 = &raw;
+	}
+
+	if (size < SHA1_BASE16_SIZE)
+		return FALSE;
+		
+	len = base16_decode(sha1->data, SHA1_RAW_SIZE, buf, SHA1_BASE16_SIZE);
+	if (SHA1_RAW_SIZE != len)
+		return FALSE;
+
+	return TRUE;
+}
+
 /**
  * Locate the start of "urn:sha1:" or "urn:bitprint:" indications and extract
  * the SHA1 out of it, placing it in the supplied `digest' buffer.
@@ -93,10 +123,30 @@ urn_get_sha1(const char *buf, struct sha1 *sha1)
 	)
 		return FALSE;
 
+	/*
+	 * Following the specifications, these should be base32-encoded.
+	 */
+
 	len = clamp_strlen(p, SHA1_BASE32_SIZE);
-	return parse_base32_sha1(p, len, sha1);
+	if (parse_base32_sha1(p, len, sha1))
+		return TRUE;
+
+	/*
+	 * Maybe it was generated by clueless people who use an hexadecimal
+	 * representation?
+	 */
+
+	len = clamp_strlen(p, SHA1_BASE16_SIZE);
+	return parse_base16_sha1(p, len, sha1);
 }
 
+/**
+ * Locate the start of "urn:bitprint:" indications and extract
+ * the SHA1 and TTH out of it, placing them in the supplied buffers.
+ *
+ * @return whether we successfully extracted the bitprint, i.e. the two
+ * hashes.
+ */
 gboolean
 urn_get_bitprint(const char *buf, size_t size,
 	struct sha1 *sha1, struct tth *tth)
@@ -104,32 +154,57 @@ urn_get_bitprint(const char *buf, size_t size,
 	static const char prefix[] = "urn:bitprint:";
 	size_t len;
 	const char *p;
+	gboolean base16_tth = FALSE;
 
 	g_assert(0 == size || NULL != buf);
 	g_assert(sha1);
 	g_assert(tth);
 
-	if (size < CONST_STRLEN(prefix) + BITPRINT_BASE32_SIZE) {
+	/*
+	 * Because some clueless sites list magnets with hexadecimal-encoded
+	 * values, we attempt to parse both base32 and base16 encoded hashes.
+	 *
+	 * Note that we expect both hashes to be similarily encoded.
+	 */
+
+	if (size < CONST_STRLEN(prefix) + BITPRINT_BASE32_SIZE)
 		return FALSE;
-	}
 	p = is_strcaseprefix(buf, prefix);
-	if (NULL == p) {
+	if (NULL == p)
 		return FALSE;
-	}
 	if (!parse_base32_sha1(p, SHA1_BASE32_SIZE, sha1)) {
-		return FALSE;
+		if (
+			size >= CONST_STRLEN(prefix) + BITPRINT_BASE16_SIZE &&
+			parse_base16_sha1(p, SHA1_BASE16_SIZE, sha1)
+		) {
+			p += SHA1_BASE16_SIZE;
+			base16_tth = TRUE;		/* SHA1 was hexa, expects TTH as hexa */
+		} else {
+			return FALSE;
+		}
+	} else {
+		p += SHA1_BASE32_SIZE;
 	}
-	p += SHA1_BASE32_SIZE;
 	if ('.' != *p++) {
 		return FALSE;
 	}
-	len = base32_decode(tth->data, TTH_RAW_SIZE, p, TTH_BASE32_SIZE);
+	if (base16_tth) {
+		len = base16_decode(tth->data, TTH_RAW_SIZE, p, TTH_BASE16_SIZE);
+	} else {
+		len = base32_decode(tth->data, TTH_RAW_SIZE, p, TTH_BASE32_SIZE);
+	}
 	if (len != TTH_RAW_SIZE) {
 		return FALSE;
 	}
 	return TRUE;
 }
 
+/**
+ * Extract the TTH out of a "urn:tree:tiger" URN.
+ * Note that the TTH is expected to be solely base32-encoded here.
+ *
+ * @return whether TTH was successfully extracted.
+ */
 gboolean
 urn_get_tth(const char *buf, size_t size, struct tth *tth)
 {
@@ -193,7 +268,16 @@ urn_get_sha1_no_prefix(const char *buf, struct sha1 *sha1)
 		return FALSE;
 
 	len = clamp_strlen(p, SHA1_BASE32_SIZE);
-	return parse_base32_sha1(p, len, sha1);
+	if (parse_base32_sha1(p, len, sha1))
+		return TRUE;
+
+	/*
+	 * Maybe it was generated by clueless people who use an hexadecimal
+	 * representation?
+	 */
+
+	len = clamp_strlen(p, SHA1_BASE16_SIZE);
+	return parse_base16_sha1(p, len, sha1);
 }
 
 /*
