@@ -1,5 +1,5 @@
 /*
- * $Id: vendors.c 15950 2009-02-07 20:57:26Z cbiere $
+ * $Id: vendors.c 16990 2009-09-25 15:23:43Z rmanfredi $
  *
  * Copyright (c) 2001-2003, Richard Eckart
  *
@@ -35,11 +35,12 @@
 
 #include "common.h"
 
-RCSID("$Id: vendors.c 15950 2009-02-07 20:57:26Z cbiere $")
+RCSID("$Id: vendors.c 16990 2009-09-25 15:23:43Z rmanfredi $")
 
 #include "vendors.h"
 #include "ascii.h"
 #include "endian.h"
+#include "glib-missing.h"	/* For g_strlcpy() */
 #include "misc.h"
 #include "override.h"	/* Must be the last header included */
 
@@ -65,6 +66,7 @@ static const struct vendor {
     { T_FEVR, "FileFever" },
     { T_FIRE, "FireFly" },
     { T_FISH, "PEERanha" },
+    { T_FOXY, "Foxy" },
     { T_FZZN, "Fuzzon" },
     { T_GDNA, "Gnucleus DNA" },
     { T_GIFT, "giFT" },
@@ -76,6 +78,7 @@ static const struct vendor {
     { T_GNUM, "Gnuminous" },
     { T_GNUT, "Gnut" },
     { T_GNZL, "Gnoozle" },
+    { T_GSHR, "GnuShare" },
     { T_GTKG, "gtk-gnutella" },
     { T_HSLG, "Hagelslag" },
     { T_HUIT, "Huitella" },
