@@ -1,8 +1,8 @@
 /*
- * $Id: vmm.h 13590 2007-05-10 05:55:39Z cbiere $
+ * $Id: vmm.h 16977 2009-09-21 21:46:02Z rmanfredi $
  *
  * Copyright (c) 2006, Christian Biere
- * Copyright (c) 2006, Raphael Manfredi
+ * Copyright (c) 2006, 2009 Raphael Manfredi
  *
  *----------------------------------------------------------------------
  * This file is part of gtk-gnutella.
@@ -30,9 +30,10 @@
  *
  * Virtual memory management.
  *
- * @author Raphael Manfredi
  * @author Christian Biere
  * @date 2006
+ * @author Raphael Manfredi
+ * @date 2006, 2009
  */
 
 #ifndef _vmm_h_
@@ -42,22 +43,34 @@
 
 size_t round_pagesize(size_t n);
 size_t compat_pagesize(void);
-void *alloc_pages(size_t size) WARN_UNUSED_RESULT G_GNUC_MALLOC;
-void *alloc_pages0(size_t size) WARN_UNUSED_RESULT G_GNUC_MALLOC;
-void free_pages(void *p, size_t size);
-size_t prune_page_cache(void);
+void *vmm_alloc(size_t size) WARN_UNUSED_RESULT G_GNUC_MALLOC;
+void *vmm_alloc0(size_t size) WARN_UNUSED_RESULT G_GNUC_MALLOC;
+void vmm_free(void *p, size_t size);
 const char *prot_strdup(const char *s);
 const void *vmm_trap_page(void);
+gboolean vmm_is_fragment(const void *base, size_t size);
+gboolean vmm_grows_upwards(void);
+
+void set_vmm_debug(guint32 level);
+void vmm_init(const void *sp);
+void vmm_malloc_inited(void);
+void vmm_post_init(void);
+void vmm_pre_close(void);
+void vmm_stop_freeing(void);
 
 void vmm_madvise_free(void *p, size_t size);
 void vmm_madvise_normal(void *p, size_t size);
 void vmm_madvise_sequential(void *p, size_t size);
 void vmm_madvise_willneed(void *p, size_t size);
 
-#define FREE_PAGES_NULL(p, size) \
+void *vmm_mmap(void *addr, size_t length,
+	int prot, int flags, int fd, off_t offset);
+int vmm_munmap(void *addr, size_t length);
+
+#define VMM_FREE_NULL(p, size) \
 G_STMT_START { \
 	if (p) { \
-		free_pages((p), (size)); \
+		vmm_free((p), (size)); \
 		p = NULL; \
 	} \
 } G_STMT_END
