@@ -1,5 +1,5 @@
 /*
- * $Id: walloc.c 16108 2009-03-10 15:19:52Z rmanfredi $
+ * $Id: walloc.c 16990 2009-09-25 15:23:43Z rmanfredi $
  *
  * Copyright (c) 2002-2003, Raphael Manfredi
  *
@@ -35,50 +35,96 @@
 
 #include "common.h"
 
-RCSID("$Id: walloc.c 16108 2009-03-10 15:19:52Z rmanfredi $")
+RCSID("$Id: walloc.c 16990 2009-09-25 15:23:43Z rmanfredi $")
 
 #include "glib-missing.h"
-#include "misc.h"
+#include "pow2.h"
+#include "unsigned.h"
+#include "halloc.h"
 #include "walloc.h"
 #include "zalloc.h"
+#include "vmm.h"
 
 #include "override.h"		/* Must be the last header included */
 
-#if ZALLOC_ALIGNBYTES == 2
-#define ZALLOC_ALIGNBITS 1
-#elif ZALLOC_ALIGNBYTES == 4
-#define ZALLOC_ALIGNBITS 2
-#elif ZALLOC_ALIGNBYTES == 8
-#define ZALLOC_ALIGNBITS 3
-#else
-#error "Unexpected ZALLOC_ALIGNBYTES value"
-#endif
-
 #ifdef TRACK_ZALLOC
 #undef walloc				/* We want to define the real routines */
 #undef walloc0
 #undef wrealloc
 #endif
 
-#define WALLOC_MAX		4096	/**< Passed this size, use malloc() */
-#define WALLOC_CHUNK	4096	/**< Target chunk size for small structs */
-#define WALLOC_MINCOUNT	8		/**< Minimum amount of structs in a chunk */
+/**
+ * Maximum size for a walloc().  Anything larger is allocated by using
+ * either halloc() or malloc().
+ */
+#define WALLOC_MAX		4096
 
-#define WZONE_SIZE	(WALLOC_MAX / ZALLOC_ALIGNBYTES)
+#define WALLOC_CHUNK	WALLOC_MAX	/**< Target chunk size for small structs */
+#define WALLOC_MINCOUNT	8			/**< Minimum amount of structs in a chunk */
+
+#define WZONE_SIZE	(WALLOC_MAX / ZALLOC_ALIGNBYTES + 1)
 
 static struct zone *wzone[WZONE_SIZE];
+static size_t halloc_threshold = -1;
 
 /*
  * Under REMAP_ZALLOC, do not define walloc(), wfree() and wrealloc().
  */
 
 #ifndef REMAP_ZALLOC
+/**
+ * Compute index in wzone[] for a (rounded) size.
+ */
+static inline size_t
+wzone_index(size_t rounded)
+{
+	size_t idx;
+
+	g_assert(rounded == zalloc_round(rounded));
+	g_assert(size_is_positive(rounded) && rounded <= WALLOC_MAX);
+
+	STATIC_ASSERT(IS_POWER_OF_2(ZALLOC_ALIGNBYTES));
+	idx = rounded / ZALLOC_ALIGNBYTES;
+
+	STATIC_ASSERT(WALLOC_MAX / ZALLOC_ALIGNBYTES + 1 == WZONE_SIZE);
+	g_assert(idx < WZONE_SIZE);
+
+	return idx;
+}
+
+/**
+ * Allocate a new zone of given rounded size.
+ */
+static zone_t *
+wzone_get(size_t rounded)
+{
+	size_t count;
+	zone_t *zone;
+
+	g_assert(rounded == zalloc_round(rounded));
+
+	/*
+	 * We're paying this computation/allocation cost once per size!
+	 *
+	 * Try to create approximately WALLOC_CHUNK byte chunks, but
+	 * capable of holding at least WALLOC_MINCOUNT structures.
+	 */
+
+	count = WALLOC_CHUNK / rounded;
+	count = MAX(count, WALLOC_MINCOUNT);
+
+	if (!(zone = zget(rounded, count)))
+		g_error("zget() failed?");
+
+	return zone;
+}
+
 /**
  * Allocate memory from a zone suitable for the given size.
  *
  * The basics for this algorithm is to allocate from fixed-sized zones, which
  * are multiples of ZALLOC_ALIGNBYTES until WALLOC_MAX (e.g. 8, 16, 24, 40, ...)
- * and to malloc() if size is greater or equal to WALLOC_MAX.
+ * and to halloc()/malloc() if size is greater than WALLOC_MAX.
  * Naturally, zones are allocated on demand only.
  *
  * @return a pointer to the start of the allocated block.
@@ -90,33 +136,22 @@ walloc(size_t size)
 	size_t rounded = zalloc_round(size);
 	size_t idx;
 
-	g_assert(size > 0);
+	g_assert(size_is_positive(size));
 
-	if (rounded >= WALLOC_MAX)
-		return malloc(size);		/* Too big for efficient zalloc() */
+	if (rounded > WALLOC_MAX) {
+		/* Too big for efficient zalloc() */
+		void *p = size >= halloc_threshold ? halloc(size) : malloc(size);
+		if (NULL == p)
+			g_error("out of memory");
 
-	idx = rounded >> ZALLOC_ALIGNBITS;
-
-	STATIC_ASSERT(WALLOC_MAX >> ZALLOC_ALIGNBITS == WZONE_SIZE);
-	g_assert(idx < WZONE_SIZE);
-
-	if (!(zone = wzone[idx])) {
-		size_t count;
-
-		/*
-		 * We're paying this computation/allocation cost once per size!
-		 *
-		 * Try to create approximately WALLOC_CHUNK byte chunks, but
-		 * capable of holding at least WALLOC_MINCOUNT structures.
-		 */
-
-		count = WALLOC_CHUNK / rounded;
-		count = MAX(count, WALLOC_MINCOUNT);
-
-		if (!(zone = wzone[idx] = zget(rounded, count)))
-			g_error("zget() failed?");
+		return p;
 	}
 
+	idx = wzone_index(rounded);
+
+	if (!(zone = wzone[idx]))
+		zone = wzone[idx] = wzone_get(rounded);
+
 	return zalloc(zone);
 }
 
@@ -154,12 +189,16 @@ wfree(gpointer ptr, size_t size)
 	memset(ptr, 1, size);
 #endif
 
-	if (rounded >= WALLOC_MAX) {
-		free(ptr);
+	if (rounded > WALLOC_MAX) {
+		if (rounded >= halloc_threshold) {
+			hfree(ptr);
+		} else {
+			free(ptr);
+		}
 		return;
 	}
 
-	idx = rounded >> ZALLOC_ALIGNBITS;
+	idx = rounded / ZALLOC_ALIGNBYTES;
 
 	g_assert(idx < WZONE_SIZE);
 
@@ -169,20 +208,66 @@ wfree(gpointer ptr, size_t size)
 	zfree(zone, ptr);
 }
 
+/**
+ * Move block around if that can serve memory compaction.
+ * @return new location for block.
+ */
+void *
+wmove(void *ptr, size_t size)
+{
+	size_t idx = wzone_index(zalloc_round(size));
+	zone_t *zone = wzone[idx];
+
+	g_assert(zone != NULL);
+
+	return zmove(zone, ptr);
+}
+
 /**
  * Reallocate a block allocated via walloc().
  *
+ * @param old		old block address (may be NULL)
+ * @param old_size	size of the old block (ignored if old is NULL)
+ * @param new_size	the new size of the block
+ *
  * @return new block address.
  */
 gpointer
 wrealloc(gpointer old, size_t old_size, size_t new_size)
 {
 	gpointer new;
-	size_t rounded = zalloc_round(new_size);
+	size_t new_rounded = zalloc_round(new_size);
+	size_t old_rounded = zalloc_round(old_size);
+	size_t idx_old, idx_new;
 
-	if (zalloc_round(old_size) == rounded)
+	if (NULL == old)
+		return walloc(new_size);
+
+	if (old_rounded == new_rounded)
 		return old;
 
+	if (new_rounded > WALLOC_MAX || old_rounded > WALLOC_MAX)
+		goto resize_block;
+
+	/*
+	 * Due to upward rounding in zalloc() to avoid wasting bytes, it is
+	 * possible that the two sizes be actually served by the same underlying
+	 * zone, in which case there is nothing to do.
+	 */
+
+	idx_old = wzone_index(old_rounded);
+	idx_new = wzone_index(new_rounded);
+
+	g_assert(wzone[idx_old] != NULL);
+
+	if (NULL == wzone[idx_new])
+		wzone[idx_new] = wzone_get(new_rounded);
+
+	if (wzone[idx_old] == wzone[idx_new])
+		return zmove(wzone[idx_old], old);	/* Move around if interesting */
+
+resize_block:
+
 	new = walloc(new_size);
 	memcpy(new, old, MIN(old_size, new_size));
 	wfree(old, old_size);
@@ -210,16 +295,23 @@ walloc_track(size_t size, char *file, int line)
 
 	g_assert(size > 0);
 
-	if (rounded >= WALLOC_MAX)
+	if (rounded > WALLOC_MAX) {
+		/* Too big for efficient zalloc() */
+		void *p = 
 #ifdef TRACK_MALLOC
-		return malloc_track(size, file, line);
+			malloc_track(size, file, line);
 #else
-		return malloc(size);		/* Too big for efficient zalloc() */
+			rounded >= halloc_threshold ? halloc(size) : malloc(size);
 #endif
+			if (NULL == p)
+				g_error("out of memory");
 
-	idx = rounded >> ZALLOC_ALIGNBITS;
+			return p;
+	}
 
-	g_assert(WALLOC_MAX >> ZALLOC_ALIGNBITS == WZONE_SIZE);
+	idx = rounded / ZALLOC_ALIGNBYTES;
+
+	g_assert(WALLOC_MAX / ZALLOC_ALIGNBYTES + 1 == WZONE_SIZE);
 	g_assert(idx < WZONE_SIZE);
 
 	if (!(zone = wzone[idx])) {
@@ -283,6 +375,9 @@ wrealloc_track(gpointer old, size_t old_size, size_t new_size,
 	gpointer new;
 	size_t rounded = zalloc_round(new_size);
 
+	if (NULL == old)
+		return walloc_track(new_size, file, line);
+
 	if (zalloc_round(old_size) == (size_t) rounded)
 		return old;
 
@@ -302,21 +397,6 @@ wdestroy(void)
 {
 	size_t i;
 
-#if defined(USE_HALLOC) && !defined(TRACK_MALLOC)
-	/*
-	 * We cannot do this currently for GLib 2.0 because g_malloc() is
-	 * mapped to halloc() and a g_warning() or other GLib functions may
-	 * use g_malloc().
-	 *
-	 * Same thing for GTK1+ where we remap g_malloc() to halloc() the hard
-	 * way -- we'll have to find a way to clean this up if we want to spot
-	 * leaks.
-	 */
-
-	if (!g_mem_is_system_malloc())
-		return;
-#endif	/* USE_HALLOC */
-
 	for (i = 0; i < WZONE_SIZE; i++) {
 		if (wzone[i] != NULL) {
 			zdestroy(wzone[i]);
@@ -325,4 +405,20 @@ wdestroy(void)
 	}
 }
 
+/**
+ * Initialize the width-based allocator.
+ */
+void
+walloc_init(void)
+{
+	/*
+	 * We know that halloc() will redirect to walloc() if the size of the
+	 * block is slightly smaller than the system's page size.  If the maximum
+	 * size of blocks we can handle via walloc() is equal to the page size
+	 * or larger, then it is safe to halloc() the larger blocks.
+	 */
+
+	halloc_threshold = MAX(WALLOC_MAX, compat_pagesize());
+}
+
 /* vi: set ts=4 sw=4 cindent: */
