@@ -1,5 +1,5 @@
 /*
- * $Id: walloc.h 16108 2009-03-10 15:19:52Z rmanfredi $
+ * $Id: walloc.h 17110 2009-10-23 17:11:14Z rmanfredi $
  *
  * Copyright (c) 2002-2003, Raphael Manfredi
  *
@@ -61,6 +61,7 @@ static inline void wfree(gpointer p, size_t size) { g_slice_free1(size, p); }
 
 static inline gpointer
 wcopy(gconstpointer p, size_t size)
+	WARN_UNUSED_RESULT G_GNUC_MALLOC
 {
 	gpointer x = g_slice_alloc(size);
 	memcpy(x, p, size);
@@ -69,6 +70,7 @@ wcopy(gconstpointer p, size_t size)
 
 static inline gpointer
 wrealloc(gpointer p, size_t old_size, size_t new_size)
+	WARN_UNUSED_RESULT G_GNUC_MALLOC
 {
 	gpointer x = g_slice_alloc(new_size);
 	memcpy(x, p, MIN(new_size, old_size));
@@ -82,6 +84,7 @@ wrealloc(gpointer p, size_t old_size, size_t new_size)
 
 static inline gpointer
 wcopy(gconstpointer p, size_t size)
+	WARN_UNUSED_RESULT G_GNUC_MALLOC
 {
 	return g_memdup(p, size);
 }
@@ -95,10 +98,19 @@ wfree(gpointer p, size_t size)
 
 static inline gpointer
 wrealloc(gpointer p, size_t o, size_t n)
+	WARN_UNUSED_RESULT G_GNUC_MALLOC
 {
 	(void) o;
 	return g_realloc(p, n);
 }
+
+static inline void *
+wmove(void *p, size_t n)
+	WARN_UNUSED_RESULT G_GNUC_MALLOC
+{
+	(void) n;
+	return p;
+}
 #endif	/* GLib >= 2.10 */
 
 #else	/* !REMAP_ZALLOC */
@@ -108,6 +120,7 @@ gpointer walloc0(size_t size) WARN_UNUSED_RESULT G_GNUC_MALLOC;
 void wfree(gpointer ptr, size_t size);
 gpointer wrealloc(gpointer old, size_t old_size, size_t new_size)
 			WARN_UNUSED_RESULT G_GNUC_MALLOC;
+void *wmove(void *ptr, size_t size) WARN_UNUSED_RESULT;
 
 /* Don't define both an inline routine and a macro... */
 #ifndef TRACK_ZALLOC
@@ -140,6 +153,7 @@ gpointer wrealloc_track(gpointer old, size_t old_size, size_t new_size,
 
 #endif	/* TRACK_ZALLOC */
 
+void walloc_init(void);
 void wdestroy(void);
 
 #define WFREE_NULL(p,size)	\
