@@ -1,5 +1,5 @@
 /*
- * $Id: watcher.c 16103 2009-03-10 14:53:39Z rmanfredi $
+ * $Id: watcher.c 16990 2009-09-25 15:23:43Z rmanfredi $
  *
  * Copyright (c) 2004, Raphael Manfredi
  *
@@ -38,13 +38,14 @@
 
 #include "common.h"
 
-RCSID("$Id: watcher.c 16103 2009-03-10 14:53:39Z rmanfredi $")
+RCSID("$Id: watcher.c 16990 2009-09-25 15:23:43Z rmanfredi $")
 
 #include "watcher.h"
 #include "atoms.h"
 #include "cq.h"
 #include "glib-missing.h"
-#include "misc.h"
+#include "halloc.h"
+#include "path.h"
 #include "walloc.h"
 #include "override.h"		/* Must be the last header included */
 
@@ -60,7 +61,6 @@ struct monitored {
 	gpointer udata;			/**< User supplied data to hand-out to callback */
 };
 
-static cevent_t *monitor_ev;		/**< Monitoring event */
 static GHashTable *monitored;	/**< filename -> struct monitored */
 
 /**
@@ -98,27 +98,17 @@ watcher_check_mtime(gpointer unused_key, gpointer value, gpointer unused_udata)
 }
 
 /**
- * Callout queue callback to perform periodic monitoring of the
+ * Callout queue periodic event to perform periodic monitoring of the
  * registered files.
  */
-static void
-watcher_timer(cqueue_t *unused_cq, gpointer unused_udata)
+static gboolean
+watcher_timer(gpointer unused_udata)
 {
-	(void) unused_cq;
 	(void) unused_udata;
 
-	/*
-	 * Re-install timer for next time.
-	 */
-
-	monitor_ev = cq_insert(callout_queue,
-		MONITOR_PERIOD_MS, watcher_timer, NULL);
-
-	/*
-	 * Monitor each registered file.
-	 */
-
 	g_hash_table_foreach(monitored, watcher_check_mtime, NULL);
+
+	return TRUE;		/* Keep calling */
 }
 
 /**
@@ -159,7 +149,7 @@ watcher_register_path(const file_path_t *fp, watcher_cb_t cb, gpointer udata)
 
 	path = make_pathname(fp->dir, fp->name);
 	watcher_register(path, cb, udata);
-	G_FREE_NULL(path);
+	HFREE_NULL(path);
 }
 
 /**
@@ -199,7 +189,7 @@ watcher_unregister_path(const file_path_t *fp)
 
 	path = make_pathname(fp->dir, fp->name);
 	watcher_unregister(path);
-	G_FREE_NULL(path);
+	HFREE_NULL(path);
 }
 
 /**
@@ -209,8 +199,7 @@ void
 watcher_init(void)
 {
 	monitored = g_hash_table_new(g_str_hash, g_str_equal);
-	monitor_ev = cq_insert(callout_queue,
-		MONITOR_PERIOD_MS, watcher_timer, NULL);
+	cq_periodic_add(callout_queue, MONITOR_PERIOD_MS, watcher_timer, NULL);
 }
 
 /**
@@ -234,8 +223,6 @@ watcher_close(void)
 {
 	g_hash_table_foreach(monitored, free_monitored_kv, NULL);
 	g_hash_table_destroy(monitored);
-
-	cq_cancel(callout_queue, &monitor_ev);
 }
 
 /* vi: set ts=4 sw=4 cindent: */
