@@ -1,5 +1,5 @@
 /*
- * $Id: zalloc.h 16108 2009-03-10 15:19:52Z rmanfredi $
+ * $Id: zalloc.h 17176 2009-11-04 00:44:06Z bpringlemeir $
  *
  * Copyright (c) 2002-2003, Raphael Manfredi
  *
@@ -54,8 +54,8 @@ typedef struct zone zone_t;
  * Memory allocation routines.
  */
 
-zone_t *zcreate(int, int);
-zone_t *zget(int, int);
+zone_t *zcreate(size_t, unsigned);
+zone_t *zget(size_t, unsigned);
 void zdestroy(zone_t *zone);
 
 /*
@@ -71,14 +71,21 @@ void zdestroy(zone_t *zone);
 #error "TRACK_ZALLOC and REMAP_ZALLOC are mutually exclusive"
 #endif	/* REMAP_ZALLOC && TRACK_ZALLOC */
 
-gpointer zalloc(zone_t *);
-void zfree(zone_t *, gpointer);
+void *zalloc(zone_t *) WARN_UNUSED_RESULT G_GNUC_MALLOC;
+void zfree(zone_t *, void *);
+void *zmove(zone_t *zone, void *p) WARN_UNUSED_RESULT;
+void zgc(gboolean overloaded);
+
+void zinit(void);
+void zclose(void);
+void set_zalloc_debug(guint32 level);
+void set_zalloc_always_gc(gboolean val);
 
 #ifdef TRACK_ZALLOC
 
 #define zalloc(z)	zalloc_track(z, _WHERE_, __LINE__)
 
-gpointer zalloc_track(zone_t *z, char *file, int line);
+void *zalloc_track(zone_t *z, char *file, int line);
 
 #endif	/* TRACK_ZALLOC */
 
