@@ -1,5 +1,5 @@
 /*
- * $Id: zlib_util.c 16108 2009-03-10 15:19:52Z rmanfredi $
+ * $Id: zlib_util.c 17181 2009-11-04 17:35:20Z rmanfredi $
  *
  * Copyright (c) 2002-2003, Raphael Manfredi
  *
@@ -35,12 +35,13 @@
 
 #include "common.h"
 
-RCSID("$Id: zlib_util.c 16108 2009-03-10 15:19:52Z rmanfredi $")
+RCSID("$Id: zlib_util.c 17181 2009-11-04 17:35:20Z rmanfredi $")
 
 #include <zlib.h>
 
 #include "zlib_util.h"
 #include "misc.h"
+#include "halloc.h"
 #include "walloc.h"
 #include "override.h"		/* Must be the last header included */
 
@@ -79,14 +80,14 @@ zlib_alloc_func(gpointer unused_opaque, guint n, guint m)
 	g_return_val_if_fail(m > 0, NULL);
 	g_return_val_if_fail(m < ((size_t) -1) / n, NULL);
 
-	return g_malloc((size_t) n * m);
+	return halloc((size_t) n * m);
 }
 
 void
 zlib_free_func(gpointer unused_opaque, gpointer p)
 {
 	(void) unused_opaque;
-	g_free(p);
+	hfree(p);
 }
 
 /**
@@ -109,6 +110,8 @@ zlib_deflater_alloc(
 	z_streamp outz;
 	int ret;
 
+	g_assert(len >= 0);
+	g_assert(destlen >= 0);
 	g_assert(level == Z_DEFAULT_COMPRESSION || (level >= 0 && level <= 9));
 
 	outz = walloc(sizeof(*outz));
@@ -138,7 +141,7 @@ zlib_deflater_alloc(
 	 *
 	 * NB: strictly speaking, we shouldn't need to store this information
 	 * here, we could rely on the information in the Z stream.  However, to
-	 * be able to inspect what's going on and add assertion,s, let's be
+	 * be able to inspect what's going on and add assertions, let's be
 	 * redundant.
 	 */
 
@@ -147,8 +150,11 @@ zlib_deflater_alloc(
 		if (data == NULL && len == 0)
 			len = 512;
 
-		zd->outlen = (int) (len * 1.005 + 12.0);
-		zd->out = g_malloc(zd->outlen);
+		zd->outlen = (unsigned) (UNSIGNED(len) * 1.005 + 12.0);
+		g_assert(zd->outlen > len);
+		g_assert(zd->outlen - len >= 12);
+
+		zd->out = halloc(zd->outlen);
 		zd->allocated = TRUE;
 	} else {
 		/* Compressed data go to a supplied buffer, not-resizable */
@@ -252,7 +258,7 @@ zlib_deflate_step(zlib_deflater_t *zd, int amount, gboolean may_close)
 
 			if (zd->allocated) {
 				zd->outlen += OUT_GROW;
-				zd->out = g_realloc(zd->out, zd->outlen);
+				zd->out = hrealloc(zd->out, zd->outlen);
 				outz->next_out = (guchar *) zd->out + (zd->outlen - OUT_GROW);
 				outz->avail_out = OUT_GROW;
 			} else
@@ -376,7 +382,7 @@ zlib_deflater_free(zlib_deflater_t *zd, gboolean output)
 	}
 
 	if (output && zd->allocated) {
-		G_FREE_NULL(zd->out);
+		HFREE_NULL(zd->out);
 	}
 	wfree(zd, sizeof(*zd));
 }
@@ -384,13 +390,14 @@ zlib_deflater_free(zlib_deflater_t *zd, gboolean output)
 /**
  * Inflate data, whose final uncompressed size is known.
  *
- * @return allocated uncompressed data if OK, NULL on error.
+ * @return allocated uncompressed data via halloc() if OK, NULL on error.
+ * Use hfree() to free the data.
  */
 gpointer
 zlib_uncompress(gconstpointer data, int len, gulong uncompressed_len)
 {
 	int ret;
-	guchar *out = g_malloc(uncompressed_len);
+	guchar *out = halloc(uncompressed_len);
 	gulong retlen = uncompressed_len;
 
 	ret = uncompress(out, &retlen, data, len);
@@ -403,7 +410,7 @@ zlib_uncompress(gconstpointer data, int len, gulong uncompressed_len)
 	}
 
 	g_warning("while decompressing data: %s", zlib_strerror(ret));
-	G_FREE_NULL(out);
+	HFREE_NULL(out);
 
 	return NULL;
 }
@@ -413,7 +420,7 @@ zlib_uncompress(gconstpointer data, int len, gulong uncompressed_len)
  *
  * @param data		the data to inflate
  * @param len		length of data
- * @param out		buffer where inflate data should be put
+ * @param out		buffer where inflated data should be put
  * @param outlen	on input the length of out buffer, on output the length of
  *					the deflated data.
  *
@@ -479,6 +486,7 @@ zlib_inflate_into(gconstpointer data, int len, gpointer out, int *outlen)
 	/* FALL THROUGH */
 
 done:
+	(void) inflateEnd(inz);
 	wfree(inz, sizeof(*inz));
 	return ret;
 }
