@@ -1,5 +1,5 @@
 /*
- * $Id: main.c 16232 2009-03-22 08:12:19Z rmanfredi $
+ * $Id: main.c 17393 2010-03-18 17:04:45Z rmanfredi $
  *
  * Copyright (c) 2001-2003, Raphael Manfredi
  *
@@ -35,6 +35,10 @@
 
 #include "common.h"
 #include "revision.h"
+#include "gtk-gnutella.h"
+
+#include <libxml/xmlversion.h>		/* For LIBXML_TEST_VERSION */
+#include <libxml/xmlmemory.h>		/* For xmlMemSetup() */
 
 #define CORE_SOURCES
 
@@ -42,6 +46,7 @@
 #include "core/bitzi.h"
 #include "core/bogons.h"
 #include "core/bsched.h"
+#include "core/ctl.h"
 #include "core/clock.h"
 #include "core/dh.h"
 #include "core/dmesh.h"
@@ -53,6 +58,7 @@
 #include "core/fileinfo.h"
 #include "core/file_object.h"
 #include "core/gdht.h"
+#include "core/pdht.h"
 #include "core/geo_ip.h"
 #include "core/gmsg.h"
 #include "core/gnet_stats.h"
@@ -74,6 +80,7 @@
 #include "core/parq.h"
 #include "core/pcache.h"
 #include "core/pproxy.h"
+#include "core/publisher.h"
 #include "core/routing.h"
 #include "core/rx.h"
 #include "core/search.h"
@@ -99,20 +106,28 @@
 #include "lib/adns.h"
 #include "lib/atoms.h"
 #include "lib/bg.h"
+#include "lib/compat_misc.h"
+#include "lib/compat_sleep_ms.h"
 #include "lib/cq.h"
 #include "lib/crash.h"
 #include "lib/crc.h"
 #include "lib/debug.h"
 #include "lib/dbus_util.h"
 #include "lib/eval.h"
+#include "lib/fd.h"
 #include "lib/glib-missing.h"
 #include "lib/halloc.h"
 #include "lib/iso3166.h"
 #include "lib/map.h"
 #include "lib/mime_type.h"
+#include "lib/offtime.h"
+#include "lib/palloc.h"
+#include "lib/parse.h"
 #include "lib/patricia.h"
 #include "lib/pattern.h"
+#include "lib/pow2.h"
 #include "lib/socket.h"
+#include "lib/stringify.h"
 #include "lib/strtok.h"
 #include "lib/tea.h"
 #include "lib/tiger.h"
@@ -120,9 +135,11 @@
 #include "lib/tm.h"
 #include "lib/utf8.h"
 #include "lib/vendors.h"
+#include "lib/vmm.h"
 #include "lib/walloc.h"
 #include "lib/watcher.h"
 #include "lib/wordvec.h"
+#include "lib/zalloc.h"
 #include "shell/shell.h"
 
 #include "ui/gtk/gui.h"
@@ -140,7 +157,7 @@
 
 #include "lib/override.h"		/* Must be the last header included */
 
-RCSID("$Id: main.c 16232 2009-03-22 08:12:19Z rmanfredi $")
+RCSID("$Id: main.c 17393 2010-03-18 17:04:45Z rmanfredi $")
 
 #define SLOW_UPDATE_PERIOD		20	/**< Update period for `main_slow_update' */
 #define EXIT_GRACE				30	/**< Seconds to wait before exiting */
@@ -149,18 +166,17 @@ RCSID("$Id: main.c 16232 2009-03-22 08:12:19Z rmanfredi $")
 #define LOAD_HIGH_WATERMARK		90	/**< % amount over which we're overloaded */
 #define LOAD_LOW_WATERMARK		55	/**< lower threshold to clear condition */
 
-static guint main_slow_update = 0;
-static gboolean exiting = FALSE;
-static volatile sig_atomic_t from_atexit = FALSE;
-static volatile sig_atomic_t signal_received = 0;
-static volatile sig_atomic_t shutdown_requested = 0;
-static volatile sig_atomic_t sig_hup_received = 0;
+static unsigned main_slow_update;
+static volatile sig_atomic_t exiting;
+static volatile sig_atomic_t from_atexit;
+static volatile sig_atomic_t signal_received;
+static volatile sig_atomic_t shutdown_requested;
+static volatile sig_atomic_t sig_hup_received;
 static jmp_buf atexit_env;
-static volatile const gchar *exit_step = "gtk_gnutella_exit";
+static volatile const char *exit_step = "gtk_gnutella_exit";
 static tm_t start_time;
 
-void gtk_gnutella_exit(gint n);
-static gint reopen_log_files(void);
+static int reopen_log_files(void);
 static gboolean stderr_disabled;
 
 /**
@@ -216,16 +232,22 @@ sig_malloc(int n)
 guint32
 main_get_build(void)
 {
-	static guint32 build = 0;
-	const gchar *p;
+	static guint32 build;
+	static int initialized;
 
 	if (build)
 		return build;
 
-	p = is_strprefix(GTA_BUILD, "$Revision: ");
-	if (p)
-		build = atoi(p);
+	if (!initialized) {
+		const char *p;
 
+		initialized = TRUE;
+		p = is_strprefix(GTA_BUILD, "$Revision: ");
+		if (p) {
+			int error;
+			build = parse_uint32(p, NULL, 10, &error);
+		}
+	}
 	return build;
 }
 
@@ -266,7 +288,9 @@ debugging(guint t)
 		GNET_PROPERTY(udp_debug) > t ||
 		GNET_PROPERTY(upload_debug) > t ||
 		GNET_PROPERTY(url_debug) > t ||
+		GNET_PROPERTY(vmm_debug) > t ||
 		GNET_PROPERTY(vmsg_debug) > t ||
+		GNET_PROPERTY(zalloc_debug) > t ||
 
 		/* Above line left blank for easy "!}sort" under vi */
 		0;
@@ -294,7 +318,10 @@ gtk_gnutella_atexit(void)
 	if (!exiting) {
 		g_warning("trapped foreign exit(), cleaning up...");
 		from_atexit = TRUE;
-
+#ifndef USE_TOPLESS
+		running_topless = TRUE;		/* X connection may be broken, avoid GUI */
+#endif
+        
 #ifdef SIGALRM
 		set_signal(SIGALRM, sig_alarm);
 #endif
@@ -317,24 +344,24 @@ gtk_gnutella_atexit(void)
  * @param prev_sys		previous total system time, updated if not NULL
  */
 static void
-log_cpu_usage(tm_t *since_time, gdouble *prev_user, gdouble *prev_sys)
+log_cpu_usage(tm_t *since_time, double *prev_user, double *prev_sys)
 {
-	gdouble user;
-	gdouble sys;
-	gdouble total;
+	double user;
+	double sys;
+	double total;
 	tm_t cur_time;
-	gdouble elapsed;
+	double elapsed;
 
 	tm_now_exact(&cur_time);
 	total = tm_cputime(&user, &sys);
 	if (prev_user) {
-		gdouble v = *prev_user;
+		double v = *prev_user;
 		*prev_user = user;
 		user -= v;
 		total -= v;
 	}
 	if (prev_sys) {
-		gdouble v = *prev_sys;
+		double v = *prev_sys;
 		*prev_sys = sys;
 		sys -= v;
 		total -= v;
@@ -356,47 +383,63 @@ gtk_gnutella_request_shutdown(void)
 }
 
 /**
- * Exit program, return status `n' to parent process.
+ * Exit program, return status `exit_code' to parent process.
  *
  * Shutdown systems, so we can track memory leaks, and wait for EXIT_GRACE
  * seconds so that BYE messages can be sent to other nodes.
  */
 void
-gtk_gnutella_exit(gint n)
+gtk_gnutella_exit(int exit_code)
 {
+	static volatile sig_atomic_t safe_to_exit;
 	time_t exit_time = time(NULL);
 	time_delta_t exit_grace = EXIT_GRACE;
-	static gboolean safe_to_exit = FALSE;
 
 	if (exiting) {
 		if (safe_to_exit) {
-			g_warning("forced exit(%d), good bye.", n);
-			exit(n);
+			g_warning("forced exit(%d), good bye.", exit_code);
+			exit(exit_code);
 		}
 		g_warning("ignoring re-entrant exit(%d), unsafe now (in %s)",
-			n, exit_step);
+			exit_code, exit_step);
 		return;
 	}
 
 	exiting = TRUE;
 
-#define DO(fn) 	do { exit_step = #fn; fn(); } while (0)
+#define DO(fn) 	do {					\
+	exit_step = STRINGIFY(fn);			\
+	if (GNET_PROPERTY(shutdown_debug))	\
+		g_message("SHUTDOWN calling %s", exit_step);	\
+	fn();								\
+} while (0)
+
+#define DO_ARG(fn, arg)	do {			\
+	exit_step = STRINGIFY(fn);					\
+	if (GNET_PROPERTY(shutdown_debug))	\
+		g_message("SHUTDOWN calling %s(%s)", exit_step, STRINGIFY(arg));	\
+	fn(arg);							\
+} while (0)
 
 	DO(shell_close);
-	DO(file_info_store_if_dirty);
+	DO(file_info_store_if_dirty);	/* For safety, will run again below */
 	DO(file_info_close_pre);
 	DO(node_bye_all);
 	DO(upload_close);	/* Done before upload_stats_close() for stats update */
 	DO(upload_stats_close);
-	DO(parq_close);
+	DO(parq_close_pre);
 	DO(verify_sha1_close);
 	DO(verify_tth_close);
 	DO(download_close);
+	DO(file_info_store_if_dirty);	/* In case downloads had buffered data */
+	DO(parq_close);
 	DO(pproxy_close);
 	DO(http_close);
 	DO(uhc_close);
 	DO(move_close);
-	DO(dht_close);
+	DO(publisher_close);
+	DO(pdht_close);
+	DO_ARG(dht_close, TRUE);
 	DO(ipp_cache_save_all);
 
 	/*
@@ -415,25 +458,49 @@ gtk_gnutella_exit(gint n)
 	if (from_atexit)
 		return;
 
-#undef DO
+	/*
+	 * When halloc() is replacing malloc(), we need to make sure no memory
+	 * allocated through halloc() is going to get invalidated because some
+	 * GTK callbacks seem to access freed memory.
+	 *
+	 * Also, later on when we finally cleanup all the allocated memory, we may
+	 * run into similar problems with glib if we don't take this precaution.
+	 *
+	 * Therefore, before starting the final shutdown routines, prevent any
+	 * freeing.  We don't care much as we're now going to exit() anyway.
+	 *
+	 * Note that only the actual freeing is suppressed, but all internal
+	 * data structures are still updated, meaning memory leak detection will
+	 * still work correctly.
+	 *
+	 * Used to do that only when halloc_replaces_malloc() was true, but
+	 * now doing it unconditionally because of problems with GTK1 callbacks.
+	 * This may not be due to GTK and be a bug in our usage of callbacks,
+	 * but it happens only with memory allocated through the VMM layer,
+	 * i.e. impacting walloc() / zalloc() as well since their arena are
+	 * now allocated through VMM.
+	 *		--RAM, 2010-02-19
+	 */
+
+	DO(vmm_stop_freeing);
 
 	if (!running_topless) {
-		settings_gui_save_if_dirty();
-		main_gui_shutdown();
+		DO(settings_gui_save_if_dirty);
+		DO(main_gui_shutdown);
 
 		if (debugging(0) || signal_received || shutdown_requested)
 			g_message("GUI shutdown completed");
 	}
 
-	cq_halt();				/* No more callbacks, with everything shutdown */
-	hcache_shutdown();		/* Save host caches to disk */
-	settings_shutdown();
-	oob_shutdown();			/* No longer deliver outstanding OOB hits */
-	socket_shutdown();
-	bsched_shutdown();
+	DO(cq_halt);			/* No more callbacks, with everything shutdown */
+	DO(hcache_shutdown);	/* Save host caches to disk */
+	DO(settings_shutdown);
+	DO(oob_shutdown);		/* No longer deliver outstanding OOB hits */
+	DO(socket_shutdown);
+	DO(bsched_shutdown);
 
 	if (!running_topless) {
-		settings_gui_shutdown();
+		DO(settings_gui_shutdown);
 	}
 
 	/*
@@ -479,69 +546,84 @@ gtk_gnutella_exit(gint n)
 	if (debugging(0) || signal_received || shutdown_requested)
 		g_message("running final shutdown sequence...");
 
-	search_shutdown();		/* Disable now, since we can get queries above */
+	DO(search_shutdown);	/* Disable now, since we can get queries above */
 
-	bitzi_close();
-	ntp_close();
-	gdht_close();
-	sq_close();
-	dh_close();
-	dq_close();
-	hsep_close();
-	file_info_close();
-	ext_close();
-	share_close();
-	node_close();
-	udp_close();
-	routing_close();	/* After node_close() */
-	bsched_close();
-	dmesh_close();
-	host_close();
-	hcache_close();		/* After host_close() */
-	bogons_close();		/* Idem, since host_close() can touch the cache */
-	tx_collect();		/* Prevent spurious leak notifications */
-	rx_collect();		/* Idem */
-	hostiles_close();
-	spam_close();
-	gip_close();
-	ban_close();
-	inet_close();
-	whitelist_close();
-	features_close();
-	clock_close();
-	vmsg_close();
-	watcher_close();
-	tsync_close();
-	cq_close();
-	word_vec_close();
-	pattern_close();
-	pmsg_close();
-	version_close();
-	ignore_close();
-	bg_close();
-	eval_close();
-	iso3166_close();
+	DO(bitzi_close);
+	DO(ntp_close);
+	DO(gdht_close);
+	DO(sq_close);
+	DO(dh_close);
+	DO(dq_close);
+	DO(hsep_close);
+	DO(file_info_close);
+	DO(ext_close);
+	DO(share_close);
+	DO(node_close);
+	DO(udp_close);
+	DO(routing_close);	/* After node_close() */
+	DO(bsched_close);
+	DO(dmesh_close);
+	DO(host_close);
+	DO(hcache_close);	/* After host_close() */
+	DO(bogons_close);	/* Idem, since host_close() can touch the cache */
+	DO(tx_collect);		/* Prevent spurious leak notifications */
+	DO(rx_collect);		/* Idem */
+	DO(hostiles_close);
+	DO(spam_close);
+	DO(gip_close);
+	DO(ban_close);
+	DO(inet_close);
+	DO(ctl_close);
+	DO(whitelist_close);
+	DO(features_close);
+	DO(clock_close);
+	DO(vmsg_close);
+	DO(watcher_close);
+	DO(tsync_close);
+	DO(word_vec_close);
+	DO(pattern_close);
+	DO(pmsg_close);
+	DO(version_close);
+	DO(ignore_close);
+	DO(bg_close);
+	DO(eval_close);
+	DO(iso3166_close);
 	atom_str_free_null(&start_rfc822_date);
-	adns_close();
-	dbus_util_close();  /* After adns_close() to avoid strange crashes */
-	ipp_cache_close();
-	dump_close();
-	file_object_close();
-	settings_close();	/* Must come after hcache_close() */
-	atoms_close();
-	wdestroy();
-	locale_close();
-#ifdef TRACK_MALLOC
-	malloc_close();
-#endif
+	DO(adns_close);
+	DO(dbus_util_close);  /* After adns_close() to avoid strange crashes */
+	DO(ipp_cache_close);
+	DO(dump_close);
+	DO(tls_global_close);
+	DO(file_object_close);
+	DO(settings_close);	/* Must come after hcache_close() */
+	DO(misc_close);
+	DO(inputevt_close);
+	DO(locale_close);
+	DO(cq_close);
+
+	/*
+	 * Memory shutdown must come last.
+	 */
+
+	gm_mem_set_safe_vtable();
+	DO(vmm_pre_close);
+	DO(atoms_close);
+	DO(wdestroy);
+	DO(zclose);
+	DO(malloc_close);
 
 	if (debugging(0) || signal_received || shutdown_requested) {
 		g_message("gtk-gnutella shut down cleanly.");
 	}
 	if (!running_topless) {
-		main_gui_exit(n);
+		main_gui_exit(exit_code);
 	}
-	exit(n);
+	hdestroy();
+	exit(exit_code);
+
+#undef DO
+#undef DO_ARG
+
 }
 
 static void
@@ -550,18 +632,18 @@ sig_terminate(int n)
 	signal_received = n;		/* Terminate asynchronously in main_timer() */
 
 	if (from_atexit)			/* Might be stuck in some cleanup callback */
-		exit(1);				/* Terminate ASAP */
+		exit(EXIT_FAILURE);		/* Terminate ASAP */
 }
 
 static void
 slow_main_timer(time_t now)
 {
-	static guint i = 0;
+	static unsigned i = 0;
 
 	if (GNET_PROPERTY(cpu_debug)) {
 		static tm_t since = { 0, 0 };
-		static gdouble user = 0.0;
-		static gdouble sys = 0.0;
+		static double user = 0.0;
+		static double sys = 0.0;
 
 		if (since.tv_sec == 0)
 			since = start_time;
@@ -573,6 +655,7 @@ slow_main_timer(time_t now)
 	case 0:
 		dmesh_store();
 		dmesh_ban_store();
+		version_ancient_warn();
 		break;
 	case 1:
 		hcache_store_if_dirty(HOST_ANY);
@@ -603,7 +686,6 @@ slow_main_timer(time_t now)
 	}
 	tx_collect();					/* Collect freed TX stacks */
 	rx_collect();					/* Idem for freed RX stacks */
-	prune_page_cache();
 
 	download_slow_timer(now);
 	node_slow_timer(now);
@@ -619,16 +701,16 @@ static time_t
 check_cpu_usage(void)
 {
 	static tm_t last_tm;
-	static gdouble last_cpu = 0.0;
-	static gint ticks = 0;
-	static gint load_avg = 0;		/* 100 * cpu% for integer arithmetic */
-	static gint avg = 0;			/* cpu% */
+	static double last_cpu = 0.0;
+	static int ticks = 0;
+	static int load_avg = 0;	/* 100 * cpu% for integer arithmetic */
+	static int avg = 0;			/* cpu% */
 	tm_t cur_tm;
-	gint load = 0;
-	gdouble cpu;
-	gdouble elapsed;
-	gdouble cpu_percent;
-	gdouble coverage;
+	int load = 0;
+	double cpu;
+	double elapsed;
+	double cpu_percent;
+	double coverage;
 
 	/*
 	 * Compute CPU time used this period.
@@ -645,9 +727,10 @@ check_cpu_usage(void)
 	coverage = callout_queue_coverage(ticks);
 	coverage = MAX(coverage, 0.001);	/* Prevent division by zero */
 
-	if (GNET_PROPERTY(cq_debug)) {
-		g_message("CQ: callout queue holds %d items and processed %d ticks",
-			cq_count(callout_queue), cq_ticks(callout_queue));
+	if (GNET_PROPERTY(cq_debug) > 2) {
+		g_message("CQ: callout queue \"%s\" items=%d ticks=%d coverage=%d%%",
+			cq_name(callout_queue), cq_count(callout_queue),
+			cq_ticks(callout_queue), (int) (coverage * 100.0 + 0.5));
 	}
 
 	/*
@@ -669,11 +752,11 @@ check_cpu_usage(void)
 	else if (coverage <= 0.5)
 		cpu_percent *= 1.5;
 
-	load = (gint) cpu_percent * 100;
+	load = (unsigned) cpu_percent * 100;
 	load_avg += (load >> 3) - (load_avg >> 3);
 	avg = load_avg / 100;
 
-	if (GNET_PROPERTY(cpu_debug) > 1 && last_cpu)
+	if (GNET_PROPERTY(cpu_debug) > 1 && last_cpu > 0.0)
 		g_message("CPU: %.3f secs in %.3f secs (~%.3f%% @ cover=%.2f) avg=%d%%",
 			cpu - last_cpu, elapsed, cpu_percent, coverage, avg);
 
@@ -709,16 +792,16 @@ check_cpu_usage(void)
  * Main timer routine, called once per second.
  */
 static gboolean
-main_timer(gpointer p)
+main_timer(void *unused_data)
 {
 	time_t now;
 
-	(void) p;
+	(void) unused_data;
 	if (signal_received || shutdown_requested) {
 		if (signal_received) {
 			g_warning("caught signal #%d, exiting...", signal_received);
 		}
-		gtk_gnutella_exit(1);
+		gtk_gnutella_exit(EXIT_FAILURE);
 	}
 
 	now = check_cpu_usage();
@@ -770,51 +853,71 @@ main_timer(gpointer p)
 		slow_main_timer(now);
 	}
 
-	/*
-	 * The following are low-priority tasks, not called if we've
-	 * detected a high CPU load.
-	 */
 	bg_sched_timer(GNET_PROPERTY(overloaded_cpu));	/* Background tasks */
 
 	return TRUE;
 }
 
+/**
+ * Called when the main callout queue is idle.
+ */
+static gboolean
+callout_queue_idle(void *unused_data)
+{
+	gboolean overloaded = GNET_PROPERTY(overloaded_cpu);
+	(void) unused_data;
+
+	if (GNET_PROPERTY(cq_debug) > 1)
+		g_message("CQ: callout queue is idle (CPU %s)",
+			overloaded ? "OVERLOADED" : "available");
+
+	/* Idle tasks always scheduled */
+	zgc(overloaded);
+
+	if (!overloaded) {
+		/* Idle tasks scheduled only when CPU is not overloaded */
+		pgc();
+	}
+
+	return TRUE;		/* Keep scheduling this */
+}
+
 /**
  * Scan files when the GUI is up.
  */
 static gboolean
-scan_files_once(gpointer p)
+scan_files_once(void *unused_data)
 {
-	(void) p;
+	(void) unused_data;
 	share_scan();
 
 	return FALSE;
 }
 
-static const gchar * const log_domains[] = {
+static const char * const log_domains[] = {
 	G_LOG_DOMAIN, "Gtk", "GLib", "Pango"
 };
 
 static void
-log_handler(const gchar *domain, GLogLevelFlags level,
-	const gchar *message, gpointer user_data)
+log_handler(const char *unused_domain, GLogLevelFlags level,
+	const char *message, void *unused_data)
 {
-	gint saved_errno = errno;
+	int saved_errno = errno;
 	time_t now;
 	struct tm *ct;
 	const char *prefix;
-	gchar *safer;
+	char *safer;
 
-	(void) domain;
-	(void) user_data;
+	(void) unused_domain;
+	(void) unused_data;
 
 	if (stderr_disabled)
 		return;
 
-	now = tm_time();
+	now = tm_time_exact();
 	ct = localtime(&now);
 
-	switch (level) {
+	switch (level & ~(G_LOG_FLAG_RECURSION | G_LOG_FLAG_FATAL)) {
 #define CASE(x) case CAT2(G_LOG_LEVEL_,x): prefix = #x; break;
 
 	CASE(CRITICAL)
@@ -828,23 +931,31 @@ log_handler(const gchar *domain, GLogLevelFlags level,
 		prefix = "UNKNOWN";
 	}
 
-	safer = control_escape(message);
+	if (level & G_LOG_FLAG_RECURSION) {
+		/* Probably logging from memory allocator, string should be safe */
+		safer = deconstify_gpointer(message);
+	} else {
+		safer = control_escape(message);
+	}
 
-	fprintf(stderr, "%02d-%02d-%02d %.2d:%.2d:%.2d (%s): %s\n",
-		(1900 + ct->tm_year) % 100, ct->tm_mon + 1, ct->tm_mday,
-		ct->tm_hour, ct->tm_min, ct->tm_sec, prefix, safer);
+	fprintf(stderr, "%02d-%02d-%02d %.2d:%.2d:%.2d (%s)%s%s: %s\n",
+		(TM_YEAR_ORIGIN + ct->tm_year) % 100, ct->tm_mon + 1, ct->tm_mday,
+		ct->tm_hour, ct->tm_min, ct->tm_sec, prefix,
+		(level & G_LOG_FLAG_RECURSION) ? " (recursive)" : "",
+		(level & G_LOG_FLAG_FATAL) ? " (FATAL)" : "",
+		safer);
 
 	if (safer != message) {
-		G_FREE_NULL(safer);
+		HFREE_NULL(safer);
 	}
 
 #if 0
 	/* Define to debug Glib or Gtk problems */
 	if (domain) {
-		guint i;
+		unsigned i;
 
 		for (i = 0; i < G_N_ELEMENTS(log_domains); i++) {
-			const gchar *dom = log_domains[i];
+			const char *dom = log_domains[i];
 			if (dom && 0 == strcmp(domain, dom)) {
 				raise(SIGTRAP);
 				break;
@@ -859,10 +970,11 @@ log_handler(const gchar *domain, GLogLevelFlags level,
 static void
 log_init(void)
 {
-	guint i;
+	unsigned i;
 
 	for (i = 0; i < G_N_ELEMENTS(log_domains); i++) {
 		g_log_set_handler(log_domains[i],
+			G_LOG_FLAG_RECURSION | G_LOG_FLAG_FATAL |
 			G_LOG_LEVEL_ERROR | G_LOG_LEVEL_CRITICAL | G_LOG_LEVEL_WARNING |
 			G_LOG_LEVEL_MESSAGE | G_LOG_LEVEL_INFO | G_LOG_LEVEL_DEBUG,
 			log_handler, NULL);
@@ -872,6 +984,7 @@ log_init(void)
 extern char **environ;
 
 enum main_arg {
+	main_arg_compile_info,
 	main_arg_daemonize,
 	main_arg_exec_on_crash,
 	main_arg_geometry,
@@ -907,15 +1020,16 @@ enum arg_type {
 
 static struct {
 	const enum main_arg id;
-	const gchar * const name;
-	const gchar * const summary;
+	const char * const name;
+	const char * const summary;
 	const enum arg_type type;
-	const gchar *arg;
+	const char *arg;	/* memory will be allocated via halloc() */
 	gboolean used;
 } options[] = {
 #define OPTION(name, type, summary) \
 	{ main_arg_ ## name , #name, summary, ARG_TYPE_ ## type, NULL, FALSE }
 
+	OPTION(compile_info,	NONE, "Display compile-time information."),
 	OPTION(daemonize, 		NONE, "Daemonize the process."),
 	OPTION(exec_on_crash, 	PATH, "Execute a command on crash."),
 	OPTION(geometry,		TEXT, "Placement of the main GUI window."),
@@ -950,8 +1064,8 @@ static struct {
 #undef OPTION
 };
 
-static inline gchar
-underscore_to_hyphen(gchar c)
+static inline char
+underscore_to_hyphen(char c)
 {
 	return '_' == c ? '-' : c;
 }
@@ -963,17 +1077,19 @@ underscore_to_hyphen(gchar c)
  * @return whether the two strings qualify as equivalent or not.
  */
 static gboolean
-option_match(const gchar *a, const gchar *b)
+option_match(const char *a, const char *b)
 {
 	g_assert(a);
 	g_assert(b);
 
-	do {
-		if (underscore_to_hyphen(*a) != underscore_to_hyphen(*b)) {
+	for (;;) {	
+		if (underscore_to_hyphen(*a) != underscore_to_hyphen(*b))
 			return FALSE;
-		}
+		if ('\0' == *a)
+			break;
+		a++;
 		b++;
-	} while ('\0' != *a++);
+	}
 
 	return TRUE;
 }
@@ -985,10 +1101,10 @@ option_match(const gchar *a, const gchar *b)
  * @return a pointer to a static buffer holding the pretty version of the
  *         option name. 
  */
-static const gchar *
-option_pretty_name(const gchar *name)
+static const char *
+option_pretty_name(const char *name)
 {
-	static gchar buf[128];
+	static char buf[128];
 	size_t i;
 
 	for (i = 0; i < G_N_ELEMENTS(buf) - 1; i++) {
@@ -1000,7 +1116,7 @@ option_pretty_name(const gchar *name)
 	return buf;
 }
 
-static gint
+static int
 reopen_log_files(void)
 {
 	gboolean failure = FALSE;
@@ -1037,7 +1153,7 @@ static void
 usage(int exit_code)
 {
 	FILE *f;
-	guint i;
+	unsigned i;
 
 	f = EXIT_SUCCESS == exit_code ? stdout : stderr;
 	fprintf(f, "Usage: gtk-gnutella [ options ... ]\n");
@@ -1047,7 +1163,7 @@ usage(int exit_code)
 		g_assert(options[i].id == i);
 
 		if (options[i].summary) {
-			const gchar *arg, *name;
+			const char *arg, *name;
 			size_t pad;
 
 			arg = "";
@@ -1071,7 +1187,7 @@ usage(int exit_code)
 			}
 
 			fprintf(f, "  --%s%s%-*s%s\n",
-				name, arg, (gint) pad, "", options[i].summary);
+				name, arg, (int) MIN(pad, INT_MAX), "", options[i].summary);
 		}
 	}
 	
@@ -1085,8 +1201,8 @@ prehandle_arguments(char **argv)
 	argv++;
 
 	while (argv[0]) {
-		const gchar *s;
-		guint i;
+		const char *s;
+		unsigned i;
 
 		s = is_strprefix(argv[0], "--");
 		if (NULL == s || '\0' == s[0])
@@ -1119,11 +1235,14 @@ prehandle_arguments(char **argv)
 	}
 }
 
+/**
+ * Parse arguments, but do not take any action (excepted re-opening log files).
+ */
 static void
-handle_arguments(int argc, char **argv)
+parse_arguments(int argc, char **argv)
 {
 	const char *argv0;
-	guint i;
+	unsigned i;
 
 	STATIC_ASSERT(G_N_ELEMENTS(options) == num_main_args);
 	for (i = 0; i < G_N_ELEMENTS(options); i++) {
@@ -1139,7 +1258,7 @@ handle_arguments(int argc, char **argv)
 	argc--;
 
 	while (argc > 0) {
-		const gchar *s;
+		const char *s;
 
 		s = is_strprefix(argv[0], "--");
 		if (!s)
@@ -1171,10 +1290,10 @@ handle_arguments(int argc, char **argv)
 			}
 			switch (options[i].type) {
 			case ARG_TYPE_TEXT:
-				options[i].arg = g_strdup(argv[0]);
+				options[i].arg = NOT_LEAKING(h_strdup(argv[0]));
 				break;
 			case ARG_TYPE_PATH:
-				options[i].arg = absolute_pathname(argv[0]);
+				options[i].arg = NOT_LEAKING(absolute_pathname(argv[0]));
 				if (NULL == options[i].arg) {
 					fprintf(stderr,
 						"Could not determine absolute path for \"--%s\"\n", s);
@@ -1193,67 +1312,152 @@ handle_arguments(int argc, char **argv)
 	if (0 != reopen_log_files()) {
 		exit(EXIT_FAILURE);
 	}
+}
 
-	crash_init(options[main_arg_exec_on_crash].arg, argv0,
-		options[main_arg_pause_on_crash].used);
+static void
+handle_version_argument(void)
+{
+	printf("%s\n", version_build_string());
 
+#ifndef OFFICIAL_BUILD
+	printf("(unofficial build, accessing \"%s\")\n", PACKAGE_SOURCE_DIR);
+#endif
+
+	printf("GLib %u.%u.%u",
+			glib_major_version, glib_minor_version, glib_micro_version);
+	if (
+			GLIB_MAJOR_VERSION != glib_major_version ||
+			GLIB_MINOR_VERSION != glib_minor_version ||
+			GLIB_MICRO_VERSION != glib_micro_version
+	   ) {
+		printf(" (compiled against %u.%u.%u)",
+				GLIB_MAJOR_VERSION, GLIB_MINOR_VERSION, GLIB_MICRO_VERSION);
+	}
+	printf("\n");
+
+#if defined(GTK_MAJOR_VERSION) && defined(GTK_MINOR_VERSION)
+	printf("Gtk+ %u.%u.%u",
+			gtk_major_version, gtk_minor_version, gtk_micro_version);
+	if (
+			GTK_MAJOR_VERSION != gtk_major_version ||
+			GTK_MINOR_VERSION != gtk_minor_version ||
+			GTK_MICRO_VERSION != gtk_micro_version
+	   ) {
+		printf(" (compiled against %u.%u.%u)",
+				GTK_MAJOR_VERSION, GTK_MINOR_VERSION, GTK_MICRO_VERSION);
+	}
+	printf("\n");
+#endif	/* Gtk+ */
+
+	if (tls_version_string()) {
+		printf("%s\n", tls_version_string());
+	}
+	exit(EXIT_SUCCESS);
+}
+
+static void
+handle_compile_info_argument(void)
+{
+	/*
+	 * The output should be easily parseable, not look beautiful.
+	 */
+
+	/*
+	 * If you want quoted paths, you have to escape embedded quotes!
+	 */
+	printf("user-interface=%s\n", GTA_INTERFACE);
+	printf("bindir=%s\n", BIN);
+	printf("datadir=%s\n", PRIVLIB_EXP);
+	printf("localedir=%s\n", LOCALE_EXP);
+
+#if !defined(OFFICIAL_BUILD) && defined(PACKAGE_SOURCE_DIR)
+	printf("sourcedir=%s\n", PACKAGE_SOURCE_DIR);
+#endif	/* !OFFICIAL_BUILD  && PACKAGE_SOURCE_DIR */
+
+	/*
+	 * Maybe the following should rather be printed like this:
+	 *
+	 * features=ipv6,dbus,gnutls,...
+	 */
+#ifdef ENABLE_NLS
+	printf("nls=enabled\n");
+#else
+	printf("nls=disabled\n");
+#endif	/* ENABLE_NLS */
+
+#ifdef HAS_DBUS
+	printf("dbus=enabled\n");
+#else
+	printf("dbus=disabled\n");
+#endif	/* HAS_DBUS */
+
+#ifdef HAS_GNUTLS
+	printf("gnutls=enabled\n");
+#else
+	printf("gnutls=disabled\n");
+#endif	/* HAS_GNUTLS */
+
+#ifdef HAS_SOCKER_GET
+	printf("socker=enabled\n");
+#else
+	printf("socker=disabled\n");
+#endif	/* HAS_SOCKER_GET */
+
+#ifdef HAS_IPV6
+	printf("ipv6=enabled\n");
+#else
+	printf("ipv6=disabled\n");
+#endif	/* HAS_IPV6 */
+
+	printf("largefile-support=%s\n",
+		MAX_INT_VAL(off_t) > MAX_INT_VAL(guint32) ? "enabled" : "disabled");
+
+	exit(EXIT_SUCCESS);
+}
+
+/* Handle certain arguments as soon as possible */
+static void
+handle_arguments_asap(void)
+{
 	if (options[main_arg_help].used) {
 		usage(EXIT_SUCCESS);
 	}
+
 #ifndef USE_TOPLESS
 	if (options[main_arg_topless].used) {
 		running_topless = TRUE;
 	}
 #endif	/* USE_TOPLESS */
+
 	if (options[main_arg_version].used) {
-		printf("%s\n", version_build_string());
-#ifndef OFFICIAL_BUILD
-		printf("(unofficial build, accessing \"%s\")\n", PACKAGE_SOURCE_DIR);
-#endif
-		printf("GLib %u.%u.%u",
-			glib_major_version, glib_minor_version, glib_micro_version);
-		if (
-			GLIB_MAJOR_VERSION != glib_major_version ||
-			GLIB_MINOR_VERSION != glib_minor_version ||
-			GLIB_MICRO_VERSION != glib_micro_version
-		) {
-			printf(" (compiled against %u.%u.%u)",
-				GLIB_MAJOR_VERSION, GLIB_MINOR_VERSION, GLIB_MICRO_VERSION);
-		}
-		printf("\n");
-
-#if defined(GTK_MAJOR_VERSION) && defined(GTK_MINOR_VERSION)
-		printf("Gtk+ %u.%u.%u",
-			gtk_major_version, gtk_minor_version, gtk_micro_version);
-		if (
-			GTK_MAJOR_VERSION != gtk_major_version ||
-			GTK_MINOR_VERSION != gtk_minor_version ||
-			GTK_MICRO_VERSION != gtk_micro_version
-		) {
-			printf(" (compiled against %u.%u.%u)",
-				GTK_MAJOR_VERSION, GTK_MINOR_VERSION, GTK_MICRO_VERSION);
-		}
-		printf("\n");
-#endif	/* Gtk+ */
-
-		if (tls_version_string()) {
-			printf("%s\n", tls_version_string());
-		}
-		exit(EXIT_SUCCESS);
+		handle_version_argument();
 	}
-	if (options[main_arg_ping].used) {
-		if (0 != settings_ensure_unicity(TRUE) && EEXIST == errno) {
-			/* gtk-gnutella was running. */
-			exit(EXIT_SUCCESS);
-		}
-		/* gtk-gnutella was not running or the PID file could
-		 * not be created. */
-		exit(EXIT_FAILURE);
+
+	if (options[main_arg_compile_info].used) {
+		handle_compile_info_argument();
 	}
+}
+
+/**
+ * Act on the options we parsed.
+ */
+static void
+handle_arguments(void)
+{
 	if (options[main_arg_shell].used) {
 		local_shell(settings_local_socket_path());
 		exit(EXIT_SUCCESS);
 	}
+	if (options[main_arg_ping].used) {
+		if (settings_is_unique_instance()) {
+			/* gtk-gnutella was running. */
+			exit(EXIT_SUCCESS);
+		} else {
+			/* gtk-gnutella was not running or the PID file could
+			 * not be created. */
+			exit(EXIT_FAILURE);
+		}
+	}
 	if (options[main_arg_daemonize].used) {
 		if (0 != compat_daemonize(NULL)) {
 			exit(EXIT_FAILURE);
@@ -1268,6 +1472,8 @@ handle_arguments(int argc, char **argv)
 int
 main(int argc, char **argv)
 {
+	int sp;
+
 	if (compat_is_superuser()) {
 		fprintf(stderr, "Never ever run this as root! You may use:\n\n");
 		fprintf(stderr, "    su - username -c 'gtk-gnutella --daemonize'\n\n");
@@ -1275,6 +1481,9 @@ main(int argc, char **argv)
 		exit(EXIT_FAILURE);
 	}
 
+	tm_now_exact(&start_time);
+	gm_savemain(argc, argv, environ);	/* For gm_setproctitle() */
+
 	/*
 	 * This must be run before we allocate memory because we might
 	 * use mmap() with /dev/zero and then accidently close this
@@ -1285,14 +1494,14 @@ main(int argc, char **argv)
 		exit(EXIT_FAILURE);
 	}
 
-	prehandle_arguments(argv);
+	/* First inits -- initialize custom memory allocator, if needed */
 
-	if (!options[main_arg_no_halloc].used) {
-		halloc_init();
-	}
-	
 	misc_init();
-	tm_now_exact(&start_time);
+
+	prehandle_arguments(argv);
+	vmm_init(&sp);
+	halloc_init(!options[main_arg_no_halloc].used);
+	malloc_init_vtable();
 
 	set_signal(SIGINT, SIG_IGN);	/* ignore SIGINT in adns (e.g. for gdb) */
 	set_signal(SIGHUP, sig_hup);
@@ -1308,24 +1517,38 @@ main(int argc, char **argv)
 	set_signal(SIGUSR2, sig_malloc);
 #endif
 
-	gm_savemain(argc, argv, environ);	/* For gm_setproctitle() */
+	/* Early inits */
 
+	parse_arguments(argc, argv);
+	crash_init(options[main_arg_exec_on_crash].arg, argv[0],
+		options[main_arg_pause_on_crash].used);
+	handle_arguments_asap();
+
+	log_init();
+	malloc_init(argv[0]);
+	vmm_malloc_inited();
+	zinit();
+	walloc_init();
 	atoms_init();
 	eval_init();
 	settings_early_init();
 
-	handle_arguments(argc, argv);
+	handle_arguments();
+
+	/* Our regular inits */
 
-	/* Our inits */
-	(void) tm_time_exact();
-	log_init();
 #ifndef OFFICIAL_BUILD
 	g_warning("%s \"%s\"",
-		_("This is an unofficial build which accesses "
-			"files in this directory:"),
+		_("unofficial build, accessing files from"),
 		PACKAGE_SOURCE_DIR);
 #endif
 
+#if !defined(REMAP_ZALLOC) && !defined(TRACK_MALLOC) && !defined(TRACK_ZALLOC)
+	xmlMemSetup(hfree, halloc, hrealloc, h_strdup);
+#endif
+
+	LIBXML_TEST_VERSION
+
 	/*
 	 * If one of the two below fails, the GLib installation is broken.
 	 * Gtk+ 1.2 and GLib 1.2 are not 64-bit clean, thus must not be
@@ -1337,6 +1560,7 @@ main(int argc, char **argv)
 	STATIC_ASSERT(UNSIGNED(INT_MIN) > 0);
 	STATIC_ASSERT(UNSIGNED(LONG_MIN) > 0);
 	STATIC_ASSERT(UNSIGNED(-1) > 0);
+	STATIC_ASSERT(IS_POWER_OF_2(MEM_ALIGNBYTES));
 
 	inputevt_init();
 	tiger_check();
@@ -1360,15 +1584,16 @@ main(int argc, char **argv)
 		main_gui_early_init(argc, argv, options[main_arg_no_xshm].used);
 	}
 
-	cq_init();
+	cq_init(callout_queue_idle, GNET_PROPERTY_PTR(cq_debug));
 	vmsg_init();
 	tsync_init();
 	watcher_init();
+	ctl_init();
 	hcache_init();			/* before settings_init() */
 	bsched_early_init();	/* before settings_init() */
 	ipp_cache_init();		/* before settings_init() */
 	settings_init();
-	set_library_debug(GNET_PROPERTY(lib_debug));
+	vmm_post_init();		/* after settings_init() */
 	map_test();				/* after settings_init() */
 	ipp_cache_load_all();	/* after settings_init() */
 	tls_global_init();
@@ -1413,7 +1638,8 @@ main(int argc, char **argv)
 	bitzi_init();
 	sq_init();
 	gdht_init();
-	file_info_init_post();
+	pdht_init();
+	publisher_init();
 
 	kmsg_init();			/* DHT */
 	dht_init();
@@ -1422,7 +1648,7 @@ main(int argc, char **argv)
 		main_gui_init();
 	}
 	node_post_init();
-
+	file_info_init_post();
 	download_restore_state();
 	ntp_init();
 
@@ -1449,8 +1675,7 @@ main(int argc, char **argv)
 	(void) tm_time_exact();
 	bsched_enable_all();
 	version_ancient_warn();
-	if (dht_enabled())
-		dht_attempt_bootstrap();
+	dht_attempt_bootstrap();
 
 	if (running_topless) {
 		topless_main_run();
