@@ -1,5 +1,5 @@
 /*
- * $Id: cmd.h 15811 2008-09-07 02:16:28Z cbiere $
+ * $Id: cmd.h 16467 2009-04-10 12:38:46Z cbiere $
  *
  *   Copyright (c) 2002-2003, Richard Eckart
  *
@@ -55,7 +55,7 @@ guint64 shell_line_count(struct gnutella_shell *);
 gboolean shell_request_library_rescan(void);
 
 int shell_options_parse(struct gnutella_shell *,
-	const char *argv[], option_t *ovec, int ovcnt);
+	const char *argv[], const option_t *ovec, int ovcnt);
 
 /* Implemented commands */
 
