@@ -1,5 +1,5 @@
 /*
- * $Id: download.c 16238 2009-03-23 13:31:54Z rmanfredi $
+ * $Id: download.c 17171 2009-11-03 20:08:11Z rmanfredi $
  *
  * Copyright (c) 2002-2003, Richard Eckart
  *
@@ -23,9 +23,19 @@
  *----------------------------------------------------------------------
  */
 
+/**
+ * @ingroup shell
+ * @file
+ *
+ * The "download" command.
+ *
+ * @author Richard Eckart
+ * @date 2002-2003
+ */
+
 #include "common.h"
 
-RCSID("$Id: download.c 16238 2009-03-23 13:31:54Z rmanfredi $")
+RCSID("$Id: download.c 17171 2009-11-03 20:08:11Z rmanfredi $")
 
 #include "cmd.h"
 
@@ -38,6 +48,8 @@ RCSID("$Id: download.c 16238 2009-03-23 13:31:54Z rmanfredi $")
 
 #include "lib/ascii.h"
 #include "lib/glib-missing.h"
+#include "lib/stringify.h"
+#include "lib/timestamp.h"
 
 #include "lib/override.h"		/* Must be the last header included */
 
