@@ -1,5 +1,5 @@
 /*
- * $Id: downloads.c 15815 2008-09-07 03:46:53Z cbiere $
+ * $Id: downloads.c 17171 2009-11-03 20:08:11Z rmanfredi $
  *
  * Copyright (c) 2002-2003, Richard Eckart
  *
@@ -23,14 +23,27 @@
  *----------------------------------------------------------------------
  */
 
+/**
+ * @ingroup shell
+ * @file
+ *
+ * The "downloads" command.
+ *
+ * @author Richard Eckart
+ * @date 2002-2003
+ */
+
 #include "common.h"
 
-RCSID("$Id: downloads.c 15815 2008-09-07 03:46:53Z cbiere $")
+RCSID("$Id: downloads.c 17171 2009-11-03 20:08:11Z rmanfredi $")
 
 #include "cmd.h"
 
 #include "core/downloads.h"
 
+#include "lib/parse.h"
+#include "lib/stringify.h"
+
 #include "if/bridge/ui2c.h"
 
 #include "if/gnet_property.h"
