@@ -1,5 +1,5 @@
 /*
- * $Id: node.c 15781 2008-09-02 16:30:18Z rmanfredi $
+ * $Id: node.c 17171 2009-11-03 20:08:11Z rmanfredi $
  *
  * Copyright (c) 2002-2003, Richard Eckart
  *
@@ -23,10 +23,21 @@
  *----------------------------------------------------------------------
  */
 
+/**
+ * @ingroup shell
+ * @file
+ *
+ * The "node" command.
+ *
+ * @author Richard Eckart
+ * @date 2002-2003
+ */
+
 #include "common.h"
 
-RCSID("$Id: node.c 15781 2008-09-02 16:30:18Z rmanfredi $")
+RCSID("$Id: node.c 17171 2009-11-03 20:08:11Z rmanfredi $")
 
+#include "gtk-gnutella.h"
 #include "cmd.h"
 
 #include "core/nodes.h"
@@ -35,6 +46,7 @@ RCSID("$Id: node.c 15781 2008-09-02 16:30:18Z rmanfredi $")
 
 #include "lib/ascii.h"
 #include "lib/glib-missing.h"
+#include "lib/parse.h"
 
 #include "lib/override.h"		/* Must be the last header included */
 
