@@ -1,5 +1,5 @@
 /*
- * $Id: nodes.c 15781 2008-09-02 16:30:18Z rmanfredi $
+ * $Id: nodes.c 17206 2009-11-08 21:05:08Z rmanfredi $
  *
  * Copyright (c) 2002-2003, Richard Eckart
  *
@@ -23,9 +23,19 @@
  *----------------------------------------------------------------------
  */
 
+/**
+ * @ingroup shell
+ * @file
+ *
+ * The "nodes" command.
+ *
+ * @author Richard Eckart
+ * @date 2002-2003
+ */
+
 #include "common.h"
 
-RCSID("$Id: nodes.c 15781 2008-09-02 16:30:18Z rmanfredi $")
+RCSID("$Id: nodes.c 17206 2009-11-08 21:05:08Z rmanfredi $")
 
 #include "cmd.h"
 
@@ -34,6 +44,8 @@ RCSID("$Id: nodes.c 15781 2008-09-02 16:30:18Z rmanfredi $")
 #include "lib/ascii.h"
 #include "lib/glib-missing.h"
 #include "lib/iso3166.h"
+#include "lib/halloc.h"
+#include "lib/stringify.h"
 #include "lib/options.h"
 #include "lib/tm.h"
 
@@ -44,25 +56,19 @@ print_node_info(struct gnutella_shell *sh, const struct gnutella_node *n)
 {
 	gnet_node_flags_t flags;
 	time_delta_t up, con;
-	time_t now;
 	char buf[1024];
-	char vendor_escaped[20];
+	char vendor_escaped[50];
 	char uptime_buf[8];
 	char contime_buf[8];
 
 	g_return_if_fail(sh);
 	g_return_if_fail(n);
 	
-	if (
-		!NODE_IS_ESTABLISHED(n) ||
-		!node_fill_flags(NODE_ID(n), &flags)
-	) {
+	if (!node_fill_flags(NODE_ID(n), &flags))
 		return;
-	}
 
-	now = tm_time();
-	con = n->connect_date ? delta_time(now, n->connect_date) : 0;
-	up = n->up_date ? delta_time(now, n->up_date) : 0;
+	con = n->connect_date ? delta_time(tm_time(), n->connect_date) : 0;
+	up = n->up_date ? delta_time(tm_time(), n->up_date) : 0;
 
 	{
 		const char *vendor;
@@ -70,19 +76,19 @@ print_node_info(struct gnutella_shell *sh, const struct gnutella_node *n)
 		
 		vendor = node_vendor(n);
 		escaped = hex_escape(vendor, TRUE);
-		g_strlcpy(vendor_escaped, escaped, sizeof vendor_escaped);
+		clamp_strcpy(vendor_escaped, sizeof vendor_escaped, escaped);
 		if (escaped != vendor) {
-			G_FREE_NULL(escaped);
+			HFREE_NULL(escaped);
 		}
 	}
 
-	g_strlcpy(uptime_buf, up > 0 ? compact_time(up) : "?",
-		sizeof uptime_buf);
-	g_strlcpy(contime_buf, con > 0 ? compact_time(con) : "?",
-		sizeof contime_buf);
+	clamp_strcpy(uptime_buf, sizeof uptime_buf,
+		up > 0 ? compact_time(up) : "?");
+	clamp_strcpy(contime_buf, sizeof contime_buf,
+		con > 0 ? compact_time(con) : "?");
 
 	gm_snprintf(buf, sizeof buf,
-		"%-21.45s %5.1u %s %2.2s %6.6s %6.6s %.30s",
+		"%-21.45s %5.1u %s %2.2s %6.6s %6.6s %.50s",
 		node_addr(n),
 		(unsigned) n->gnet_port,
 		node_flags_to_string(&flags),
