@@ -1,11 +1,8 @@
-/* -*- mode: cc-mode; tab-width:4; -*-
- *
- * $Id: bitzi.h,v 1.2 2004/10/06 05:39:54 cbiere Exp $
+/*
+ * $Id: bitzi.h 8376 2005-06-25 01:37:45Z daichik $
  *
  * Copyright (c) 2004, Alex Bennee <alex@bennee.com>
  *
- * Bitzi GTK Headers
- *
  *----------------------------------------------------------------------
  * This file is part of gtk-gnutella.
  *
@@ -26,14 +23,25 @@
  *----------------------------------------------------------------------
  */
 
+/**
+ * @ingroup gtk
+ * @file
+ *
+ * Bitzi GTK+ Headers.
+ *
+ * @author Alex Bennee <alex@bennee.com>
+ * @date 2004
+ */
+
 #ifndef _gtk_bitzi_h_
 #define _gtk_bitzi_h_
 
 #include "if/ui/gtk/bitzi.h"
 
-const gchar *bitzi_fjtostring(bitzi_fj_t fj);
+const gchar *bitzi_fj_to_string(bitzi_fj_t fj);
 gchar *bitzi_gui_get_metadata(const bitzi_data_t *bitzi_data);
 
 #endif /* _gtk_bitzi_h_ */
 
+/* -*- mode: cc-mode; tab-width:4; -*- */
 /* vi: set ts=4 sw=4 cindent: */
