@@ -1,5 +1,5 @@
 /*
- * $Id: downloads_common.c 16243 2009-03-24 19:21:16Z cbiere $
+ * $Id: downloads_common.c 17262 2010-02-20 10:33:43Z rmanfredi $
  *
  * Copyright (c) 2001-2003, Raphael Manfredi, Richard Eckart
  *
@@ -25,7 +25,7 @@
 
 #include "gui.h"
 
-RCSID("$Id: downloads_common.c 16243 2009-03-24 19:21:16Z cbiere $")
+RCSID("$Id: downloads_common.c 17262 2010-02-20 10:33:43Z rmanfredi $")
 
 #include "gtk/columns.h"
 #include "gtk/downloads.h"
@@ -47,8 +47,12 @@ RCSID("$Id: downloads_common.c 16243 2009-03-24 19:21:16Z cbiere $")
 #include "if/gui_property_priv.h"
 
 #include "lib/atoms.h"
+#include "lib/concat.h"
 #include "lib/glib-missing.h"
 #include "lib/hashlist.h"
+#include "lib/halloc.h"
+#include "lib/stringify.h"
+#include "lib/timestamp.h"
 #include "lib/url_factory.h"
 #include "lib/utf8.h"
 #include "lib/walloc.h"
@@ -59,7 +63,7 @@ RCSID("$Id: downloads_common.c 16243 2009-03-24 19:21:16Z cbiere $")
 
 struct fileinfo_data {
 	const char *filename;	/* atom */
-	char *status;			/* g_strdup */
+	char *status;			/* h_strdup */
 	hash_list_t *sources;	/* struct download * */
 
 	void *user_data;
@@ -417,14 +421,20 @@ downloads_gui_status_string(const struct download *d)
 
 	case GTA_DL_VERIFY_WAIT:
 		g_assert(FILE_INFO_COMPLETE(fi));
-		g_strlcpy(tmpstr, _("Waiting for SHA1 checking..."), sizeof(tmpstr));
+		g_strlcpy(tmpstr,
+			fi->tth_check ?
+				_("Waiting for TTH checking...") :
+				_("Waiting for SHA1 checking..."),
+			sizeof(tmpstr));
 		status = tmpstr;
 		break;
 
 	case GTA_DL_VERIFYING:
 		g_assert(FILE_INFO_COMPLETE(fi));
 		gm_snprintf(tmpstr, sizeof(tmpstr),
-			_("Computing SHA1 (%.02f%%)"), fi->cha1_hashed * 100.0 / fi->size);
+			_("Computing %s (%.02f%%)"),
+			fi->tth_check ? "TTH" : "SHA1",
+			fi->vrfy_hashed * 100.0 / fi->size);
 		status = tmpstr;
 		break;
 
@@ -433,14 +443,14 @@ downloads_gui_status_string(const struct download *d)
 	case GTA_DL_MOVING:
 	case GTA_DL_DONE:
 		g_assert(FILE_INFO_COMPLETE(fi));
-		g_assert(fi->cha1_hashed <= fi->size);
+		g_assert(fi->vrfy_hashed <= fi->size);
 		{
 			const char *sha1_status;
 			
 			if (fi->cha1) {
 				if (fi->sha1) {
 					sha1_status = sha1_eq(fi->sha1, fi->cha1)
-						? _("SHA-1 OK")
+						? (fi->tth_check ? _("TTH OK") : _("SHA-1 OK"))
 						: _("SHA-1 MISMATCH");
 				} else {
 					sha1_status = _("SHA-1 calculated");
@@ -450,14 +460,14 @@ downloads_gui_status_string(const struct download *d)
 			}
 			rw = gm_snprintf(tmpstr, sizeof tmpstr, "%s", sha1_status);
 
-			if (fi->cha1 && fi->cha1_hashed) {
-				unsigned elapsed = fi->cha1_elapsed;
+			if (fi->cha1 && fi->vrfy_hashed) {
+				unsigned elapsed = fi->vrfy_elapsed;
 			
 				rw += gm_snprintf(&tmpstr[rw], sizeof(tmpstr)-rw,
 					" (%s) %s",
-					short_rate(fi->cha1_hashed / (elapsed ? elapsed : 1),
+					short_rate(fi->vrfy_hashed / (elapsed ? elapsed : 1),
 						show_metric_units()),
-					short_time(fi->cha1_elapsed));
+					short_time(fi->vrfy_elapsed));
 			}
 
 			switch (d->status) {
@@ -1222,8 +1232,8 @@ fi_gui_file_fill_status(struct fileinfo_data *file)
 	file->finished = 0 != status.finished;
 	file->seeding = 0 != status.seeding;
 
-	G_FREE_NULL(file->status);	
-	file->status = g_strdup(guc_file_info_status_to_string(&status));
+	HFREE_NULL(file->status);	
+	file->status = h_strdup(guc_file_info_status_to_string(&status));
 }
 
 static void
@@ -1321,7 +1331,7 @@ static void
 fi_gui_file_free(struct fileinfo_data *file)
 {
 	atom_str_free_null(&file->filename);
-	G_FREE_NULL(file->status);
+	HFREE_NULL(file->status);
 	wfree(file, sizeof *file);
 }
 
@@ -1400,7 +1410,7 @@ fi_gui_source_add(struct download *d)
 	if (NULL == file->sources) {
 		file->sources = hash_list_new(NULL, NULL);
 	}
-	g_return_if_fail(!hash_list_contains(file->sources, d, NULL));
+	g_return_if_fail(!hash_list_contains(file->sources, d));
 
 	/*
 	 * NOTE: Always append items so that we can jump through the hashlist with
@@ -1431,7 +1441,7 @@ fi_gui_source_remove(struct download *d)
 	g_return_if_fail(file);
 
 	g_return_if_fail(file->sources);
-	g_return_if_fail(hash_list_contains(file->sources, d, NULL));
+	g_return_if_fail(hash_list_contains(file->sources, d));
 
 	hash_list_remove(file->sources, d);
 	if (0 == hash_list_length(file->sources)) {
@@ -2310,6 +2320,7 @@ fi_gui_common_shutdown(void)
     guc_fi_remove_listener(fi_gui_fi_status_changed, EV_FI_STATUS_CHANGED);
     guc_fi_remove_listener(fi_gui_fi_status_changed_transient,
 		EV_FI_STATUS_CHANGED_TRANSIENT);
+    guc_fi_remove_listener(fi_gui_fi_info_changed, EV_FI_INFO_CHANGED);
 
     guc_src_remove_listener(fi_gui_src_added, EV_SRC_ADDED);
     guc_src_remove_listener(fi_gui_src_removed, EV_SRC_REMOVED);
