@@ -1,5 +1,5 @@
 /*
- * $Id: drop.h 8376 2005-06-25 01:37:45Z daichik $
+ * $Id: drop.h 14001 2007-07-01 14:16:15Z cbiere $
  *
  * Copyright (c) 2004, Christian Biere
  *
@@ -33,12 +33,18 @@
  * @date 2004
  */
 
-#ifndef _drop_init_h_
-#define _drop_init_h_
+#ifndef _drop_h_
+#define _drop_h_
 
-void drop_init(void);
-void drop_close(void);
+#include "gui.h"
 
-#endif /* _drop_init_h_ */
+typedef void (*drag_data_received_cb)(GtkWidget *widget, GdkDragContext *dc,
+				gint x, gint y, GtkSelectionData *data,
+				guint info, guint stamp, gpointer udata);
+
+void drop_widget_init(GtkWidget *widget, drag_data_received_cb callback,
+		void *user_data);
+
+#endif /* _drop_h_ */
 
 /* vi: set ts=4 sw=4 cindent: */
