@@ -1,5 +1,5 @@
 /*
- * $Id: filter.c 15781 2008-09-02 16:30:18Z rmanfredi $
+ * $Id: filter.c 16990 2009-09-25 15:23:43Z rmanfredi $
  *
  * Copyright (c) 2001-2003, Raphael Manfredi, Richard Eckart
  *
@@ -36,7 +36,7 @@
 
 #include "gui.h"
 
-RCSID("$Id: filter.c 15781 2008-09-02 16:30:18Z rmanfredi $")
+RCSID("$Id: filter.c 16990 2009-09-25 15:23:43Z rmanfredi $")
 
 #include "gtk/misc.h"
 
@@ -48,6 +48,8 @@ RCSID("$Id: filter.c 15781 2008-09-02 16:30:18Z rmanfredi $")
 
 #include "lib/ascii.h"
 #include "lib/glib-missing.h"
+#include "lib/parse.h"
+#include "lib/stringify.h"
 #include "lib/utf8.h"
 
 #include "lib/override.h"		/* Must be the last header included */
