@@ -1,5 +1,5 @@
 /*
- * $Id: filter_core.h 14555 2007-08-19 03:51:47Z cbiere $
+ * $Id: filter_core.h 16909 2009-06-05 23:49:38Z cbiere $
  *
  * Copyright (c) 2001-2003, Raphael Manfredi, Richard Eckart
  *
@@ -44,7 +44,7 @@ typedef struct filter {
     struct search *search;
     gboolean visited;
     gint32 refcount;
-    flag_t flags;
+    guint32 flags;
     guint32 match_count;
     guint32 fail_count;
 } filter_t;
@@ -54,7 +54,7 @@ enum {
 	 RULE_FLAG_VALID  = 1 << 3,	/**< rule has valid target */
 	 RULE_FLAG_ACTIVE = 1 << 2,
 	 RULE_FLAG_SOFT   = 1 << 1,
-	 RULE_FLAG_SHADOW = 1 << 0,	/**< rule is not yet committed */
+	 RULE_FLAG_SHADOW = 1 << 0	/**< rule is not yet committed */
 };
 
 enum {
@@ -62,7 +62,7 @@ enum {
 	FILTER_FLAG_GLOBAL	= 1 << 3,
 	FILTER_FLAG_BUILTIN	= 1 << 2,
 	FILTER_FLAG_ACTIVE	= 1 << 1,
-	FILTER_FLAG_SHADOW	= 1 << 0,	/**< filter is not yet committed */
+	FILTER_FLAG_SHADOW	= 1 << 0	/**< filter is not yet committed */
 };
 
 #define RULE_IS_VALID(r) ((r != NULL) && (r->flags & RULE_FLAG_VALID))
@@ -108,7 +108,7 @@ typedef struct filter_result {
  */
 typedef struct rule {
     enum rule_type type;	            /**< type of rule, see above */
-    flag_t flags;
+    guint32 flags;
     guint32 match_count;
     guint32 fail_count;
     filter_t *target;
