@@ -1,5 +1,5 @@
 /*
- * $Id: gnet_stats_common.c 16177 2009-03-16 17:44:43Z rmanfredi $
+ * $Id: gnet_stats_common.c 17323 2010-03-01 17:32:29Z rmanfredi $
  *
  * Copyright (c) 2001-2003, Richard Eckart
  *
@@ -25,7 +25,7 @@
 
 #include "gui.h"
 
-RCSID("$Id: gnet_stats_common.c 16177 2009-03-16 17:44:43Z rmanfredi $")
+RCSID("$Id: gnet_stats_common.c 17323 2010-03-01 17:32:29Z rmanfredi $")
 
 #include "gtk/notebooks.h"
 
@@ -36,6 +36,7 @@ RCSID("$Id: gnet_stats_common.c 16177 2009-03-16 17:44:43Z rmanfredi $")
 #include "if/bridge/ui2c.h"
 
 #include "lib/glib-missing.h"
+#include "lib/stringify.h"
 #include "lib/override.h"		/* Must be the last header included */
 
 /**
@@ -147,8 +148,11 @@ general_type_str(gint value)
 		N_("Duplicate downloads found during server consolidation"),
 		N_("Discovered server GUIDs"),
 		N_("Changed server GUIDs"),
+		N_("Detected GUID collisions"),
 		N_("Detected collisions with our own GUID"),
 		N_("Firewalled node info for known hosts received in upload requests"),
+		N_("Revitalized PUSH routes"),
+		N_("Collected new PUSH proxies from other query hits"),
 		N_("Attempted download resource switching on completion"),
 		N_("Attempted download resource switching after error"),
 		N_("Successful download resource switching (all kind)"),
@@ -172,7 +176,10 @@ general_type_str(gint value)
 		N_("PARQ QUEUE sending attempts"),
 		N_("PARQ QUEUE messages sent"),
 		N_("PARQ QUEUE follow-up requests received"),
+		N_("Launched SHA-1 file verifications"),
+		N_("Launched TTH file verifications"),
 		N_("Re-seeding of orphan downloads through query hits"),
+		N_("Re-seeding of orphan downloads through upload requests"),
 		N_("DHT estimated amount of nodes"),
 		N_("DHT k-ball furthest frontier (bits)"),
 		N_("DHT k-ball closeest frontier (bits)"),
@@ -185,7 +192,9 @@ general_type_str(gint value)
 		N_("DHT routing table evicted nodes"),
 		N_("DHT routing table evicted firewalled nodes"),
 		N_("DHT routing table promoted pending nodes"),
+		N_("DHT routing table pinged promoted nodes"),
 		N_("DHT completed bucket refreshes"),
+		N_("DHT forced bucket refreshes"),
 		N_("DHT denied non-splitable bucket refresh"),
 		N_("DHT initiated bucket alive checks"),
 		N_("DHT alive pings sent to good nodes"),
@@ -195,6 +204,16 @@ general_type_str(gint value)
 		N_("DHT keys held"),
 		N_("DHT cached keys held"),
 		N_("DHT values held"),
+		N_("DHT cached KUID targets held"),
+		N_("DHT cached closest root nodes"),
+		N_("DHT cached roots exact hits"),
+		N_("DHT cached roots approximate hits"),
+		N_("DHT cached roots misses"),
+		N_("DHT cached roots lookups within k-ball"),
+		N_("DHT cached roots contact address refreshed"),
+		N_("DHT cached security tokens held"),
+		N_("DHT cached security tokens hits"),
+		N_("DHT stable node information held"),
 		N_("DHT local hits on value lookups"),
 		N_("DHT local hits returning values from cached keys"),
 		N_("DHT returned expanded values"),
@@ -212,10 +231,31 @@ general_type_str(gint value)
 		N_("DHT duplicate values returned in lookups"),
 		N_("DHT detected KUID collisions"),
 		N_("DHT detected collisions with our own KUID"),
+		N_("DHT detected KUID mismatches on RPC reply"),
+		N_("DHT caching attempts"),
+		N_("DHT caching ended successfully"),
+		N_("DHT caching partially completed"),
+		N_("DHT key-offloading checks after discovering new closest node"),
+		N_("DHT keys selected for offloading"),
+		N_("DHT key-offloading attempts"),
+		N_("DHT key-offloading ended successfully"),
+		N_("DHT key-offloading partially completed"),
+		N_("DHT values successfully offloaded"),
+		N_("DHT publishing attempts"),
+		N_("DHT publishing ended successfully (all roots)"),
+		N_("DHT publishing partially completed (root subset only)"),
+		N_("DHT publishing ending with proper value presence"),
+		N_("DHT value republishing occurring too late (after expiry)"),
+		N_("DHT publishing to self"),
+		N_("DHT background publishing completion attempts"),
+		N_("DHT background publishing completion showing improvements"),
+		N_("DHT background publishing completion successful (all roots)"),
+		N_("DHT SHA1 data type collisions"),
 		N_("DHT alt-loc lookups issued"),
 		N_("DHT push-proxy lookups issued"),
 		N_("DHT successful alt-loc lookups"),
 		N_("DHT successful push-proxy lookups"),
+		N_("DHT successful node push-entry lookups"),
 		N_("DHT re-seeding of orphan downloads"),
 	};
 
