@@ -1,5 +1,5 @@
 /*
- * $Id: gtk-missing.c 14457 2007-08-14 04:23:12Z cbiere $
+ * $Id: gtk-missing.c 16397 2009-04-04 17:42:58Z cbiere $
  *
  * Copyright (c) 2002-2003, Richard Eckart
  *
@@ -37,7 +37,7 @@
 
 #include "gui.h"
 
-RCSID("$Id: gtk-missing.c 14457 2007-08-14 04:23:12Z cbiere $")
+RCSID("$Id: gtk-missing.c 16397 2009-04-04 17:42:58Z cbiere $")
 
 #include "columns.h"
 
@@ -77,8 +77,14 @@ void
 gtk_clist_set_column_name(GtkCList *clist, int col, const char *title)
 {
     if (col >= 0 && col < clist->columns) {
-		G_FREE_NULL(clist->column[col].title);
-		clist->column[col].title = g_strdup(title);
+		if (
+			NULL == title ||
+			NULL == clist->column[col].title ||
+			0 != strcmp(clist->column[col].title, title)
+		) {
+			G_FREE_NULL(clist->column[col].title);
+			clist->column[col].title = g_strdup(title);
+		}
 	}
 }
 
