@@ -1,5 +1,5 @@
 /*
- * $Id: search.c 15941 2009-01-31 23:29:27Z cbiere $
+ * $Id: search.c 16990 2009-09-25 15:23:43Z rmanfredi $
  *
  * Copyright (c) 2001-2003, Raphael Manfredi, Richard Eckart
  *
@@ -37,7 +37,7 @@
  */
 #include "gtk/gui.h"
 
-RCSID("$Id: search.c 15941 2009-01-31 23:29:27Z cbiere $")
+RCSID("$Id: search.c 16990 2009-09-25 15:23:43Z rmanfredi $")
 
 #include "search_cb.h"
 
@@ -60,6 +60,8 @@ RCSID("$Id: search.c 15941 2009-01-31 23:29:27Z cbiere $")
 #include "lib/glib-missing.h"
 #include "lib/iso3166.h"
 #include "lib/mime_type.h"
+#include "lib/stringify.h"
+#include "lib/timestamp.h"
 #include "lib/tm.h"
 #include "lib/url.h"
 #include "lib/urn.h"
