@@ -1,5 +1,5 @@
 /*
- * $Id: nodes.c 15814 2008-09-07 03:09:56Z cbiere $
+ * $Id: nodes.c 16990 2009-09-25 15:23:43Z rmanfredi $
  *
  * Copyright (c) 2001-2003, Raphael Manfredi, Richard Eckart
  *
@@ -36,7 +36,7 @@
 
 #include "gtk/gui.h"
 
-RCSID("$Id: nodes.c 15814 2008-09-07 03:09:56Z cbiere $")
+RCSID("$Id: nodes.c 16990 2009-09-25 15:23:43Z rmanfredi $")
 
 #include "gtk/misc.h"
 #include "gtk/nodes_common.h"
@@ -52,8 +52,10 @@ RCSID("$Id: nodes.c 15814 2008-09-07 03:09:56Z cbiere $")
 #include "if/bridge/ui2c.h"
 
 #include "lib/atoms.h"
+#include "lib/concat.h"
 #include "lib/glib-missing.h"
 #include "lib/iso3166.h"
+#include "lib/stringify.h"
 #include "lib/utf8.h"
 #include "lib/walloc.h"
 
