@@ -1,5 +1,5 @@
 /*
- * $Id: search.c 15941 2009-01-31 23:29:27Z cbiere $
+ * $Id: search.c 16990 2009-09-25 15:23:43Z rmanfredi $
  *
  * Copyright (c) 2001-2003, Raphael Manfredi, Richard Eckart
  *
@@ -59,12 +59,14 @@
 #include "lib/misc.h"
 #include "lib/tm.h"
 #include "lib/url.h"
+#include "lib/stringify.h"
+#include "lib/timestamp.h"
 #include "lib/utf8.h"
 #include "lib/walloc.h"
 
 #include "lib/override.h"		/* Must be the last header included */
 
-RCSID("$Id: search.c 15941 2009-01-31 23:29:27Z cbiere $")
+RCSID("$Id: search.c 16990 2009-09-25 15:23:43Z rmanfredi $")
 
 static GtkTreeView *tree_view_search;
 
@@ -77,7 +79,6 @@ struct result_data {
 	record_t *record;
 	const gchar *meta;	/**< Atom */
 	guint children;		/**< count of children */
-	guint32 rank;		/**< for stable sorting */
 	enum gui_color color;
 };
 
@@ -171,25 +172,6 @@ on_search_list_column_clicked(GtkTreeViewColumn *column, gpointer unused_udata)
 		GTK_TREE_VIEW(column->tree_view)));
 }
 
-/**
- * Callback handler used with gtk_tree_model_foreach() to record the current
- * rank/position in tree enabling stable sorting. 
- */
-gboolean
-search_gui_update_rank(
-	GtkTreeModel *model, GtkTreePath *path, GtkTreeIter *iter, gpointer udata)
-{
-	guint32 *rank_ptr = udata;
-	struct result_data *data;
-
-	(void) path;
-	
-	data = get_result_data(model, iter);
-	data->rank = *rank_ptr;
-	(*rank_ptr)++;
-	return FALSE;
-}
-
 static void
 cell_renderer(GtkTreeViewColumn *column, GtkCellRenderer *cell, 
 	GtkTreeModel *model, GtkTreeIter *iter, gpointer udata)
@@ -854,7 +836,8 @@ search_gui_cmp(GtkTreeModel *model, GtkTreeIter *iter1, GtkTreeIter *iter2,
 	case c_sr_ctime:	ret = search_gui_cmp_ctime(a, b); break;
 	case c_sr_num: 		g_assert_not_reached(); break;
 	}
-	return ret ? ret : CMP(a->rank, b->rank);
+	/* Use address to stabilize sorting */
+	return ret ? ret : CMP(pointer_to_ulong(a), pointer_to_ulong(b));
 }
 
 void
