@@ -1,5 +1,5 @@
 /* 
- * $Id: search_cb.c 15086 2008-01-13 01:28:03Z cbiere $
+ * $Id: search_cb.c 15804 2008-09-04 22:41:07Z cbiere $
  *
  * Copyright (c) 2001-2003, Raphael Manfredi, Richard Eckart
  *
@@ -36,7 +36,7 @@
 
 #include "gtk/gui.h"
 
-RCSID("$Id: search_cb.c 15086 2008-01-13 01:28:03Z cbiere $")
+RCSID("$Id: search_cb.c 15804 2008-09-04 22:41:07Z cbiere $")
 
 #include "search_cb.h"
 
@@ -144,6 +144,7 @@ search_set_xml(GtkWidget *widget, const char *xml)
 	g_return_if_fail(widget);
 
 	txt = gtk_text_view_get_buffer(GTK_TEXT_VIEW(widget));
+	g_return_if_fail(txt);
 	
 	/*
 	 * Character set detection usually fails here because XML
@@ -162,21 +163,21 @@ search_set_xml(GtkWidget *widget, const char *xml)
 	G_FREE_NULL(xml_txt);
 }
 
-/* Display Bitzi data for the result if any */
-static void
-search_set_bitzi_metadata(const record_t *rc)
+void
+search_gui_set_bitzi_metadata_text(const char *text)
 {
-	const char *xml;
+	GtkTextBuffer *buffer;
+	GtkWidget *widget;
 
-	if (rc && NULL != rc->sha1 && guc_bitzi_has_cached_ticket(rc->sha1)) {
-		xml = guc_bitzi_ticket_by_sha1(rc->sha1, rc->size);
-		if (!xml) {
-			xml = _("Not in database");
-		}
-	} else {
-		xml = NULL;
-	}
-	search_set_xml(gui_main_window_lookup("textview_result_info_bitzi"), xml);
+	g_return_if_fail(text);
+
+	widget = gui_main_window_lookup("textview_result_info_bitzi");
+	g_return_if_fail(widget);
+
+	buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(widget));
+	g_return_if_fail(buffer);
+
+	gtk_text_buffer_set_text(buffer, text, -1);
 }
 
 /* Display XML data from the result if any */
@@ -189,28 +190,39 @@ search_set_xml_metadata(const record_t *rc)
 	search_set_xml(gui_main_window_lookup("textview_result_info_xml"), rc->xml);
 }
 
-static GtkTreeView *treeview_search_details;
+static GtkTreeView *
+search_gui_treeview_search_details(void)
+{
+	static GtkTreeView *tv;
+
+	if (NULL == tv) {
+		tv = GTK_TREE_VIEW(gui_main_window_lookup("treeview_search_details"));
+	}
+	return tv;
+}
 
 void
 search_gui_clear_details(void)
 {
-	GtkTreeModel *model;
+	GtkTreeView *tv;
 
-	g_return_if_fail(treeview_search_details);
+	tv = search_gui_treeview_search_details();
+	g_return_if_fail(tv);
 
-	model = gtk_tree_view_get_model(treeview_search_details);
-	gtk_list_store_clear(GTK_LIST_STORE(model));
+	gtk_list_store_clear(GTK_LIST_STORE(gtk_tree_view_get_model(tv)));
 }
 
 void
 search_gui_append_detail(const gchar *title, const gchar *value)
 {
 	GtkTreeModel *model;
+	GtkTreeView *tv;
 	GtkTreeIter iter;
 
-	g_return_if_fail(treeview_search_details);
+	tv = search_gui_treeview_search_details();
+	g_return_if_fail(tv);
 
-	model = gtk_tree_view_get_model(treeview_search_details);
+	model = gtk_tree_view_get_model(tv);
 	gtk_list_store_append(GTK_LIST_STORE(model), &iter);
 	gtk_list_store_set(GTK_LIST_STORE(model), &iter, 0, title, 1, value, (-1));
 }
@@ -218,17 +230,16 @@ search_gui_append_detail(const gchar *title, const gchar *value)
 static void
 search_gui_refresh_details(const record_t *rc)
 {
-	if (NULL == treeview_search_details) {
-		static const gchar name[] = "treeview_search_details";
-		treeview_search_details = GTK_TREE_VIEW(gui_main_window_lookup(name));
-	}
-	g_return_if_fail(treeview_search_details);
+	GtkTreeView *tv;
 
-	g_object_freeze_notify(G_OBJECT(treeview_search_details));
+	tv = search_gui_treeview_search_details();
+	g_return_if_fail(tv);
+
+	g_object_freeze_notify(G_OBJECT(tv));
 	search_gui_set_details(rc);
-	g_object_thaw_notify(G_OBJECT(treeview_search_details));
+	g_object_thaw_notify(G_OBJECT(tv));
 	search_set_xml_metadata(rc);
-	search_set_bitzi_metadata(rc);
+	search_gui_set_bitzi_metadata(rc);
 }
 
 static void
@@ -310,9 +321,9 @@ on_popup_search_metadata_activate(GtkMenuItem *unused_menuitem,
 	(void) unused_udata;
 
     gnet_prop_get_guint32_val(PROP_BITZI_DEBUG, &bitzi_debug);
-	if (bitzi_debug)
+	if (bitzi_debug) {
 		g_message("on_search_meta_data_active: called");
-
+	}
 	search_gui_request_bitzi_data(search_gui_get_current_search());
 }
 
