@@ -1,5 +1,5 @@
 /*
- * $Id: upload_stats.c 15837 2008-09-10 01:36:04Z cbiere $
+ * $Id: upload_stats.c 17204 2009-11-07 19:38:16Z rmanfredi $
  *
  * Copyright (c) 2001-2003, Raphael Manfredi
  * Copyright (c) 2002, Michael Tesch
@@ -56,7 +56,7 @@
 
 #include "gtk/gui.h"
 
-RCSID("$Id: upload_stats.c 15837 2008-09-10 01:36:04Z cbiere $")
+RCSID("$Id: upload_stats.c 17204 2009-11-07 19:38:16Z rmanfredi $")
 
 #include "interface-glade.h" /* for create_popup_upload_stats */
 
@@ -70,6 +70,8 @@ RCSID("$Id: upload_stats.c 15837 2008-09-10 01:36:04Z cbiere $")
 
 #include "lib/atoms.h"
 #include "lib/glib-missing.h"
+#include "lib/stringify.h"
+#include "lib/timestamp.h"
 #include "lib/utf8.h"
 #include "lib/walloc.h"
 
@@ -467,7 +469,7 @@ upload_stats_gui_update_name(struct ul_stats *us)
  * Update the visible statistics for a given file.
  */
 void
-upload_stats_gui_update(struct ul_stats *us)
+upload_stats_gui_update_model(struct ul_stats *us)
 {
 	GtkListStore *store;
 	struct upload_data *data;
