@@ -1,5 +1,5 @@
 /*
- * $Id: main.c 15392 2008-07-07 20:42:46Z cbiere $
+ * $Id: main.c 17203 2009-11-07 13:24:28Z rmanfredi $
  *
  * Copyright (c) 2001-2003, Raphael Manfredi & Richard Eckart
  *
@@ -36,12 +36,14 @@
 
 #include "gui.h"
 
-RCSID("$Id: main.c 15392 2008-07-07 20:42:46Z cbiere $")
+RCSID("$Id: main.c 17203 2009-11-07 13:24:28Z rmanfredi $")
 
 #ifdef I_PWD
 #include <pwd.h>
 #endif /* I_PWD */
 
+#include "gtk-gnutella.h"
+
 #include "notebooks.h"
 #include "main.h"
 #include "misc.h"
@@ -68,6 +70,7 @@ RCSID("$Id: main.c 15392 2008-07-07 20:42:46Z cbiere $")
 #include "if/bridge/ui2c.h"
 
 #include "lib/glib-missing.h"
+#include "lib/halloc.h"
 #include "lib/tm.h"
 #include "lib/utf8.h"
 #include "lib/override.h"			/* Must be the last header included */
@@ -150,7 +153,7 @@ main_gui_page_visibility_change(int page_num, gboolean visible)
 
 	iter = visibility_listeners[page_num];
 	for (/* NOTHING */; NULL != iter; iter = g_slist_next(iter)) {
-		main_gui_visibility_cb func = iter->data;
+		main_gui_visibility_cb func = cast_pointer_to_func(iter->data);
 
 		g_assert(func);
 		(*func)(visible);
@@ -221,7 +224,7 @@ main_gui_add_page_visibility_listener(main_gui_visibility_cb func,
 	g_return_if_fail(UNSIGNED(page_num) < nb_main_page_num);
 
 	visibility_listeners[page_num] = g_slist_append(
-										visibility_listeners[page_num], func);
+			visibility_listeners[page_num], cast_func_to_pointer(func));
 }
 
 void
@@ -232,7 +235,7 @@ main_gui_remove_page_visibility_listener(main_gui_visibility_cb func,
 	g_return_if_fail(UNSIGNED(page_num) < nb_main_page_num);
 
 	visibility_listeners[page_num] = g_slist_remove(
-										visibility_listeners[page_num], func);
+			visibility_listeners[page_num], cast_func_to_pointer(func));
 }
 
 int
@@ -461,7 +464,8 @@ gui_init_dlg_about(void)
 		"U\304\237ur \303\207etin <ugur.jnmbk\100gmail.com>",
 		"Lloyd Bryant <lloydbaz\100msn.com>",
 		"Martijn van Oosterhout <kleptog\100svana.org>",
-		"Jochen Kemnade <jochenkemnade@web.de>",
+		"Jochen Kemnade <jochenkemnade\100web.de>",
+		"Larry Nieves <lanieves\100gmail.com>",
     };
 	GtkWidget *text;
     guint i;
@@ -510,26 +514,26 @@ main_gui_gtkrc_init(void)
 	/* parse gtkrc files (thx to the sylpheed-claws developers for the tip) */
 	userrc = make_pathname(guc_settings_home_dir(), rchfn);
 	gtk_rc_parse(userrc);
-	G_FREE_NULL(userrc);
+	HFREE_NULL(userrc);
 
-	userrc = g_strconcat(guc_settings_home_dir(),
+	userrc = h_strconcat(guc_settings_home_dir(),
 		G_DIR_SEPARATOR_S, ".gtk", G_DIR_SEPARATOR_S, "gtkrc", (void *) 0);
 	gtk_rc_parse(userrc);
-	G_FREE_NULL(userrc);
+	HFREE_NULL(userrc);
 
 #ifdef USE_GTK2
-	userrc = g_strconcat(guc_settings_home_dir(),
+	userrc = h_strconcat(guc_settings_home_dir(),
 		G_DIR_SEPARATOR_S, ".gtk2", G_DIR_SEPARATOR_S, "gtkrc", (void *) 0);
 #else
-	userrc = g_strconcat(guc_settings_home_dir(),
+	userrc = h_strconcat(guc_settings_home_dir(),
 		G_DIR_SEPARATOR_S, ".gtk1", G_DIR_SEPARATOR_S, "gtkrc", (void *) 0);
 #endif
 	gtk_rc_parse(userrc);
-	G_FREE_NULL(userrc);
+	HFREE_NULL(userrc);
 
 	userrc = make_pathname(guc_settings_config_dir(), rcfn);
 	gtk_rc_parse(userrc);
-	G_FREE_NULL(userrc);
+	HFREE_NULL(userrc);
 }
 
 /***
@@ -643,6 +647,7 @@ main_gui_init(void)
     gnet_stats_gui_init();
     search_stats_gui_init();
     uploads_gui_init();
+	upload_stats_common_gui_init();
     upload_stats_gui_init();
     /* Must come before search_init() so searches/filters can be loaded.*/
 	filter_init();
@@ -705,7 +710,7 @@ main_gui_add_timer(main_gui_timer_cb func)
 	}
 	g_return_if_fail(!slist_contains_identical(timers, func));
 
-	slist_append(timers, func);
+	slist_append(timers, cast_func_to_pointer(func));
 }
 
 void
@@ -714,7 +719,7 @@ main_gui_remove_timer(main_gui_timer_cb func)
 	g_return_if_fail(func);
 	g_return_if_fail(timers);
 
-	slist_remove(timers, func);
+	slist_remove(timers, cast_func_to_pointer(func));
 }
 
 /**
@@ -738,11 +743,14 @@ main_gui_timer(time_t now)
 
 	length = timers ? slist_length(timers) : 0;
 	while (length-- > 0) {
-		main_gui_timer_cb func;
+		void *p;
 
-		func = slist_shift(timers);
-		if (func) {
-			slist_append(timers, func);
+		p = slist_shift(timers);
+		if (p) {
+			main_gui_timer_cb func;
+
+			slist_append(timers, p);
+			func = cast_pointer_to_func(p);
 			(*func)(now);
 		}
 		if (overloaded)
@@ -799,9 +807,11 @@ main_gui_shutdown(void)
     fi_gui_shutdown();
     nodes_gui_shutdown();
     uploads_gui_shutdown();
+    upload_stats_common_gui_shutdown();
     upload_stats_gui_shutdown();
 	gnet_stats_gui_shutdown();
     hcache_gui_shutdown();
+    statusbar_gui_shutdown();
 }
 
 /* vi: set ts=4 sw=4 cindent: */
