@@ -1,5 +1,5 @@
 /*
- * $Id: main.h 14389 2007-08-09 22:10:53Z cbiere $
+ * $Id: main.h 17033 2009-10-05 20:51:00Z rmanfredi $
  *
  * Copyright (c) 2001-2003, Raphael Manfredi, Richard Eckart
  *
@@ -39,10 +39,8 @@ WIDGET(dlg_ancient)
 WIDGET(dlg_faq)
 WIDGET(dlg_prefs)
 WIDGET(dlg_quit)
-WIDGET(filter_dialog)
 WIDGET(main_window)
 WIDGET(popup_downloads)
-WIDGET(popup_filter_rule)
 WIDGET(popup_monitor)
 WIDGET(popup_nodes)
 WIDGET(popup_search)
