@@ -1,5 +1,5 @@
 /*
- * $Id: main_cb.c 14231 2007-07-29 20:03:34Z cbiere $
+ * $Id: main_cb.c 16822 2009-05-30 11:37:44Z rmanfredi $
  *
  * Copyright (c) 2001-2003, Raphael Manfredi, Richard Eckart
  *
@@ -25,7 +25,7 @@
 
 #include "gui.h"
 
-RCSID("$Id: main_cb.c 14231 2007-07-29 20:03:34Z cbiere $")
+RCSID("$Id: main_cb.c 16822 2009-05-30 11:37:44Z rmanfredi $")
 
 #include "html_view.h"
 #include "main.h"
@@ -53,6 +53,7 @@ load_faq(void)
 {
 	static const gchar faq_file[] = "FAQ";
 	static file_path_t fp[4];
+	static int initialized;
 	GtkWidget *textview;
 	const gchar *lang;
 	guint i = 0;
@@ -63,16 +64,23 @@ load_faq(void)
 	textview = gui_dlg_faq_lookup("textview_faq");
 	lang = locale_get_language();
 
-	file_path_set(&fp[i++], make_pathname(PRIVLIB_EXP, lang), faq_file);
-	file_path_set(&fp[i++], PRIVLIB_EXP G_DIR_SEPARATOR_S "en", faq_file);
+	if (initialized != 0) {
+		i = initialized;
+	} else {
+		file_path_set(&fp[i++],
+			NOT_LEAKING(make_pathname(PRIVLIB_EXP, lang)), faq_file);
+		file_path_set(&fp[i++], PRIVLIB_EXP G_DIR_SEPARATOR_S "en", faq_file);
 	
 #ifndef OFFICIAL_BUILD
-	file_path_set(&fp[i++],
-		make_pathname(PACKAGE_EXTRA_SOURCE_DIR, lang), faq_file);
+		file_path_set(&fp[i++],
+			NOT_LEAKING(make_pathname(PACKAGE_EXTRA_SOURCE_DIR, lang)),
+			faq_file);
 
-	file_path_set(&fp[i++],
-		PACKAGE_EXTRA_SOURCE_DIR G_DIR_SEPARATOR_S "en", faq_file);
+		file_path_set(&fp[i++],
+			PACKAGE_EXTRA_SOURCE_DIR G_DIR_SEPARATOR_S "en", faq_file);
 #endif /* !OFFICIAL_BUILD */
+		initialized = i;
+	}
 
 	g_assert(i <= G_N_ELEMENTS(fp));
 
@@ -101,7 +109,6 @@ load_faq(void)
 	}
 }
 
-
 static void
 quit(gboolean force)
 {
