@@ -1,5 +1,5 @@
 /*
- * $Id: misc.c 15781 2008-09-02 16:30:18Z rmanfredi $
+ * $Id: misc.c 17077 2009-10-16 20:51:38Z rmanfredi $
  *
  * Copyright (c) 2001-2003, Raphael Manfredi & Richard Eckart
  *
@@ -36,7 +36,7 @@
 
 #include "gui.h"
 
-RCSID("$Id: misc.c 15781 2008-09-02 16:30:18Z rmanfredi $")
+RCSID("$Id: misc.c 17077 2009-10-16 20:51:38Z rmanfredi $")
 
 #include "gtk-shared/callbacks.h"
 
@@ -53,6 +53,8 @@ RCSID("$Id: misc.c 15781 2008-09-02 16:30:18Z rmanfredi $")
 
 #include "lib/ascii.h"
 #include "lib/glib-missing.h"
+#include "lib/parse.h"
+#include "lib/stringify.h"
 
 #include "lib/override.h"		/* Must be the last header included */
 
@@ -165,6 +167,8 @@ gui_update_traffic_stats(void)
     static guint32 gnet_out_max = 0;
     static guint32 leaf_in_max = 0;
     static guint32 leaf_out_max = 0;
+    static guint32 dht_in_max = 0;
+    static guint32 dht_out_max = 0;
     gnet_bw_stats_t s;
     gnet_bw_stats_t s2;
     static GtkProgressBar *pg_http_in = NULL;
@@ -173,6 +177,8 @@ gui_update_traffic_stats(void)
     static GtkProgressBar *pg_gnet_out = NULL;
     static GtkProgressBar *pg_leaf_in = NULL;
     static GtkProgressBar *pg_leaf_out = NULL;
+    static GtkProgressBar *pg_dht_in = NULL;
+    static GtkProgressBar *pg_dht_out = NULL;
 
 	if (pg_http_in == NULL) {
 		pg_http_in = GTK_PROGRESS_BAR(
@@ -187,6 +193,10 @@ gui_update_traffic_stats(void)
 			(gui_main_window_lookup("progressbar_bws_lin"));
 		pg_leaf_out = GTK_PROGRESS_BAR
 			(gui_main_window_lookup("progressbar_bws_lout"));
+		pg_dht_in = GTK_PROGRESS_BAR
+			(gui_main_window_lookup("progressbar_bws_dht_in"));
+		pg_dht_out = GTK_PROGRESS_BAR
+			(gui_main_window_lookup("progressbar_bws_dht_out"));
 	}
 
   	/*
@@ -206,25 +216,59 @@ gui_update_traffic_stats(void)
     guc_gnet_get_bw_stats(BW_HTTP_IN, &s);
     update_stat(&http_in_max, pg_http_in, &s,
 		GUI_PROPERTY(progressbar_bws_in_avg), 1);
+
     guc_gnet_get_bw_stats(BW_HTTP_OUT, &s);
     update_stat(&http_out_max, pg_http_out, &s,
 		GUI_PROPERTY(progressbar_bws_out_avg), 0);
+
     guc_gnet_get_bw_stats(BW_GNET_IN, &s);
     guc_gnet_get_bw_stats(BW_GNET_UDP_IN, &s2);
 	gnet_bw_stats_sum(&s, &s2);
     update_stat(&gnet_in_max, pg_gnet_in, &s,
 		GUI_PROPERTY(progressbar_bws_gin_avg), 1);
+
     guc_gnet_get_bw_stats(BW_GNET_OUT, &s);
     guc_gnet_get_bw_stats(BW_GNET_UDP_OUT, &s2);
 	gnet_bw_stats_sum(&s, &s2);
     update_stat(&gnet_out_max, pg_gnet_out, &s,
 		GUI_PROPERTY(progressbar_bws_gout_avg), 0);
+
     guc_gnet_get_bw_stats(BW_LEAF_IN, &s);
     update_stat(&leaf_in_max, pg_leaf_in, &s,
 		GUI_PROPERTY(progressbar_bws_glin_avg), 1);
+
     guc_gnet_get_bw_stats(BW_LEAF_OUT, &s);
     update_stat(&leaf_out_max, pg_leaf_out, &s,
 		GUI_PROPERTY(progressbar_bws_glout_avg), 0);
+
+    guc_gnet_get_bw_stats(BW_DHT_IN, &s);
+    update_stat(&dht_in_max, pg_dht_in, &s,
+		GUI_PROPERTY(progressbar_bws_dht_in_avg), 1);
+
+    guc_gnet_get_bw_stats(BW_DHT_OUT, &s);
+    update_stat(&dht_out_max, pg_dht_out, &s,
+		GUI_PROPERTY(progressbar_bws_dht_out_avg), 0);
+}
+
+/**
+ * Utility routine to dynamically resize a widget after some items it holds
+ * were hidden, so that the GTK layer can remove any uncessary hole that
+ * would remain.
+ */
+void
+gui_shrink_widget_named(const char *name)
+{
+#ifdef USE_GTK1
+	GtkWidget *w = gui_main_window_lookup(name);
+
+	if (w == NULL)
+		return;
+
+	gtk_widget_hide(w);
+	gtk_widget_show(w);
+#else	/* !USE_GTK1 */
+	(void) name;
+#endif	/* USE_GTK1 */
 }
 
 void
@@ -233,12 +277,15 @@ gui_update_stats_frames(void)
     static GtkWidget *frame_bws_inout = NULL;
     static GtkWidget *frame_bws_ginout = NULL;
     static GtkWidget *frame_bws_glinout = NULL;
+    static GtkWidget *frame_bws_dht_inout = NULL;
     guint32 peermode;
+	gboolean hidden = FALSE;
 
 	if (frame_bws_inout == NULL) {
 		frame_bws_inout = gui_main_window_lookup("frame_bws_inout");
 		frame_bws_ginout = gui_main_window_lookup("frame_bws_ginout");
 		frame_bws_glinout = gui_main_window_lookup("frame_bws_glinout");
+		frame_bws_dht_inout = gui_main_window_lookup("frame_bws_dht_inout");
 	}
 
    	gnet_prop_get_guint32_val(PROP_CURRENT_PEERMODE, &peermode);
@@ -246,18 +293,22 @@ gui_update_stats_frames(void)
     if (
 		GUI_PROPERTY(progressbar_bws_in_visible) ||
 		GUI_PROPERTY(progressbar_bws_out_visible)
-	)
+	) {
         gtk_widget_show(frame_bws_inout);
-    else
+    } else {
         gtk_widget_hide(frame_bws_inout);
+		hidden = TRUE;
+	}
 
     if (
 		GUI_PROPERTY(progressbar_bws_gin_visible) ||
 		GUI_PROPERTY(progressbar_bws_gout_visible)
-	)
+	) {
         gtk_widget_show(frame_bws_ginout);
-    else
+    } else {
         gtk_widget_hide(frame_bws_ginout);
+		hidden = TRUE;
+	}
 
     if (
 		(
@@ -265,14 +316,32 @@ gui_update_stats_frames(void)
 			GUI_PROPERTY(progressbar_bws_glout_visible)
 		) &&
         (peermode == NODE_P_ULTRA || !GUI_PROPERTY(autohide_bws_gleaf))
-	)
+	) {
         gtk_widget_show(frame_bws_glinout);
-    else
+    } else {
         gtk_widget_hide(frame_bws_glinout);
+		hidden = TRUE;
+	}
+
+    if (
+		(
+			GUI_PROPERTY(progressbar_bws_dht_in_visible) ||
+			GUI_PROPERTY(progressbar_bws_dht_out_visible)
+		) &&
+        (guc_dht_enabled() || !GUI_PROPERTY(autohide_bws_dht))
+	) {
+        gtk_widget_show(frame_bws_dht_inout);
+    } else {
+        gtk_widget_hide(frame_bws_dht_inout);
+		hidden = TRUE;
+	}
+
+	if (hidden)
+		gui_shrink_widget_named("vbox_sidebar_stats");
 }
 
 /**
- * If the given coordinates are not reasonable, they're appriopriately
+ * If the given coordinates are not reasonable, they're appropriately
  * adjusted.
  *
  * @param	coord must point to an array of 4 guint32 values
@@ -870,12 +939,18 @@ show_popup_menu(widget_popup_menu_cb handler)
 	}
 }
 
+static inline widget_popup_menu_cb
+cast_to_widget_popup_menu_cb(const void *p)
+{
+	return (widget_popup_menu_cb) cast_pointer_to_func(p);
+}
+
 #if GTK_CHECK_VERSION(2,0,0)
 static gboolean
 on_popup_menu(GtkWidget *unused_widget, void *user_data)
 {
 	(void) unused_widget;
-	return show_popup_menu(user_data);
+	return show_popup_menu(cast_to_widget_popup_menu_cb(user_data));
 }
 #endif	/* Gtk+ >= 2.0*/
 
@@ -891,7 +966,7 @@ on_key_press_event(GtkWidget *unused_widget,
 	case 0x1008FE0A:	/* Shift+F10 under Xnest */
 		modifier = gtk_accelerator_get_default_mod_mask() & event->state;
 		if (GDK_SHIFT_MASK == modifier) {
-			return show_popup_menu(user_data);
+			return show_popup_menu(cast_to_widget_popup_menu_cb(user_data));
 		}
 		break;
 	}
@@ -909,7 +984,7 @@ on_button_press_event(GtkWidget *unused_widget,
 		3 == event->button &&
 		0 == (gtk_accelerator_get_default_mod_mask() & event->state)
 	) {
-		return show_popup_menu(user_data);
+		return show_popup_menu(cast_to_widget_popup_menu_cb(user_data));
 	}
 	return FALSE;
 }
@@ -920,18 +995,20 @@ on_button_press_event(GtkWidget *unused_widget,
 void
 widget_add_popup_menu(GtkWidget *widget, widget_popup_menu_cb handler)
 {
+	void *data = cast_func_to_pointer((func_ptr_t) handler);
+
 	g_return_if_fail(widget);
 	g_return_if_fail(handler);
 
 	GTK_WIDGET_SET_FLAGS(widget, GTK_CAN_FOCUS);
 	GTK_WIDGET_SET_FLAGS(widget, GTK_CAN_DEFAULT);
 #if GTK_CHECK_VERSION(2,0,0)
-	gui_signal_connect(widget, "popup-menu", on_popup_menu, handler);
+	gui_signal_connect(widget, "popup-menu", on_popup_menu, data);
 #endif	/* Gtk+ >= 2.0*/
 
-	gui_signal_connect(widget, "key-press-event", on_key_press_event, handler);
+	gui_signal_connect(widget, "key-press-event", on_key_press_event, data);
 	gui_signal_connect(widget, "button-press-event", on_button_press_event,
-		handler);
+		data);
 }
 
 /* vi: set ts=4 sw=4 cindent: */
