@@ -1,5 +1,5 @@
 /*
- * $Id: nodes_common.c 15781 2008-09-02 16:30:18Z rmanfredi $
+ * $Id: nodes_common.c 17078 2009-10-16 21:12:34Z rmanfredi $
  *
  * Copyright (c) 2001-2003, Raphael Manfredi, Richard Eckart
  *
@@ -36,7 +36,9 @@
 
 #include "gui.h"
 
-RCSID("$Id: nodes_common.c 15781 2008-09-02 16:30:18Z rmanfredi $")
+RCSID("$Id: nodes_common.c 17078 2009-10-16 21:12:34Z rmanfredi $")
+
+#include "gtk-gnutella.h"
 
 #include "nodes_common.h"
 #include "settings.h"
@@ -50,7 +52,9 @@ RCSID("$Id: nodes_common.c 15781 2008-09-02 16:30:18Z rmanfredi $")
 
 #include "lib/ascii.h"
 #include "lib/glib-missing.h"
+#include "lib/parse.h"
 
+#include "lib/halloc.h"
 #include "lib/walloc.h"
 #include "lib/override.h"	/* Must be the last header included */
 
@@ -102,8 +106,8 @@ nodes_gui_common_status_str(const gnet_node_status_t *n)
 			}
 
 			if (n->tx_compressed && GUI_PROPERTY(show_gnet_info_txc))
-				slen += gm_snprintf(gui_tmp, sizeof(gui_tmp), "TXc=%u,%u%%",
-						n->sent, (unsigned) (n->tx_compression_ratio * 100));
+				slen += gm_snprintf(gui_tmp, sizeof(gui_tmp), "TXc=%u,%d%%",
+						n->sent, (int) (n->tx_compression_ratio * 100));
 			else
 				slen += gm_snprintf(gui_tmp, sizeof(gui_tmp), "TX=%u",
 						n->sent);
@@ -134,8 +138,8 @@ nodes_gui_common_status_str(const gnet_node_status_t *n)
 
 			if (n->rx_compressed && GUI_PROPERTY(show_gnet_info_rxc))
 				slen += gm_snprintf(&gui_tmp[slen], sizeof(gui_tmp)-slen,
-					" RXc=%u,%u%%",
-					n->received, (unsigned) (n->rx_compression_ratio * 100));
+					" RXc=%u,%d%%",
+					n->received, (int) (n->rx_compression_ratio * 100));
 			else
 				slen += gm_snprintf(&gui_tmp[slen], sizeof(gui_tmp)-slen,
 					" RX=%u", n->received);
@@ -456,7 +460,7 @@ nodes_gui_common_connect_by_name(const gchar *line)
 				size_t n = 1 + hostname_len;
 
 				g_assert(n > hostname_len);
-				p = g_malloc(n);
+				p = halloc(n);
 				g_strlcpy(p, hostname, n);
 				hostname = p;
 			}
@@ -466,7 +470,7 @@ nodes_gui_common_connect_by_name(const gchar *line)
 			ctx->flags = flags;
 			guc_adns_resolve(hostname, add_node_helper, ctx);
 
-			G_FREE_NULL(p);
+			HFREE_NULL(p);
 		}
 	}
 }
