@@ -1,5 +1,5 @@
 /*
- * $Id: search_common.h 15804 2008-09-04 22:41:07Z cbiere $
+ * $Id: search_common.h 16909 2009-06-05 23:49:38Z cbiere $
  *
  * Copyright (c) 2003, Raphael Manfredi
  *
@@ -127,7 +127,7 @@ const char *search_gui_get_route(const struct results_set *);
 const char *search_gui_get_filename_extension(const char *filename_utf8);
 void search_gui_set_sort_defaults(void);
 
-struct query *search_gui_handle_query(const char *, flag_t flags,
+struct query *search_gui_handle_query(const char *, guint32 flags,
 						const char **error_str);
 void search_gui_query_free(struct query **query_ptr);
 void search_gui_filter_new(search_t *, GList *rules);
