@@ -1,5 +1,5 @@
 /*
- * $Id: search_result.h 15814 2008-09-07 03:09:56Z cbiere $
+ * $Id: search_result.h 16909 2009-06-05 23:49:38Z cbiere $
  *
  * Copyright (c) 2003, Raphael Manfredi
  *
@@ -29,7 +29,7 @@
 struct guid;
 
 typedef enum {
-	RESULTS_SET_MAGIC = 0xa44eb853U
+	RESULTS_SET_MAGIC = 0x244eb853U
 } results_set_magic_t;
 
 /**
@@ -104,7 +104,7 @@ typedef struct record {
 	filesize_t size;			/**< Size of file, in bytes */
 	time_t  create_time;		/**< Create Time of file; zero if unknown */
 	guint32 file_index;			/**< Index for GET command */
-    flag_t  flags;              /**< same flags as in gnet_record_t */
+    guint32 flags;              /**< same flags as in gnet_record_t */
 } record_t;
 
 static inline void
