@@ -1,5 +1,5 @@
 /*
- * $Id: search_xml.c 15781 2008-09-02 16:30:18Z rmanfredi $
+ * $Id: search_xml.c 17078 2009-10-16 21:12:34Z rmanfredi $
  *
  * Copyright (c) 2002-2003, Richard Eckart
  *
@@ -35,11 +35,13 @@
 
 #include "gui.h"
 
-RCSID("$Id: search_xml.c 15781 2008-09-02 16:30:18Z rmanfredi $")
+RCSID("$Id: search_xml.c 17078 2009-10-16 21:12:34Z rmanfredi $")
 
 #include <libxml/tree.h>
 #include <libxml/parser.h>
 
+#include "gtk-gnutella.h"
+
 #include "filter_core.h"
 #include "search_xml.h"
 #include "settings.h"
@@ -50,8 +52,13 @@ RCSID("$Id: search_xml.c 15781 2008-09-02 16:30:18Z rmanfredi $")
 #include "if/bridge/ui2c.h"
 
 #include "lib/ascii.h"
+#include "lib/file.h"
 #include "lib/getdate.h"
 #include "lib/glib-missing.h"
+#include "lib/halloc.h"
+#include "lib/parse.h"
+#include "lib/stringify.h"
+#include "lib/timestamp.h"
 #include "lib/tm.h"
 #include "lib/utf8.h"
 #include "lib/override.h"		/* Must be the last header included */
@@ -274,8 +281,10 @@ parse_target(const gchar *buf, gint *error)
 	 * machine doesn't use 32-bit wide pointers. Older versions used
 	 * the pointer casted to an integer type as target ID.
 	 */
-	if (0 == *error && 4 == sizeof(gpointer) && v > (~(guint32) 0)) {
-		*error = ERANGE;
+	if (4 == sizeof(gpointer)) {
+		if (0 == *error && v > (~(guint32) 0)) {
+			*error = ERANGE;
+		}
 	}
 	if (0 != *error) {
 		g_message("buf=\"%s\"", buf);
@@ -470,10 +479,10 @@ search_store_xml(void)
 			g_warning("could not rename %s as %s: %s",
 				filename_new, filename, g_strerror(errno));
 
-		G_FREE_NULL(filename);
+		HFREE_NULL(filename);
     }
 
-	G_FREE_NULL(filename_new);
+	HFREE_NULL(filename_new);
 
 	xmlFreeDoc(doc);
 
@@ -496,8 +505,6 @@ search_retrieve_xml(void)
 	gchar *path = NULL;
 	gchar *path_orig = NULL;
 
-	LIBXML_TEST_VERSION
-
 	/*
 	 * We can't use routines from file.c here because libxml2 only defines
 	 * interfaces for parsing a path or memory, but not for parsing a FILE
@@ -521,15 +528,15 @@ search_retrieve_xml(void)
 		if (-1 == rename(path, path_orig)) {
 			g_warning("could not rename \"%s\" as \"%s\": %s",
 				path, path_orig, g_strerror(errno));
-			G_FREE_NULL(path_orig);
+			HFREE_NULL(path_orig);
 			path_orig = path;
 			path = NULL;
 		} else {
-			G_FREE_NULL(path);
+			HFREE_NULL(path);
 		}
 	} else {
         g_warning("searches file does not exist: %s", path);
-		G_FREE_NULL(path);
+		HFREE_NULL(path);
 
 		if (!file_exists(path_orig))
 			goto out;
@@ -562,7 +569,7 @@ search_retrieve_xml(void)
 		xmlFreeDoc(doc);
 		goto out;
 	}
-	G_FREE_NULL(path_orig);
+	HFREE_NULL(path_orig);
 
     id_map = g_hash_table_new(pointer_hash_func, NULL);
 
@@ -616,7 +623,7 @@ search_retrieve_xml(void)
 					break;
 				}
                 rule->target = new_target;
-                set_flags(rule->flags, RULE_FLAG_VALID);
+                rule->flags |= RULE_FLAG_VALID;
 
                 /*
                  * We circumvent the shadows, so we must do refcounting
@@ -683,8 +690,8 @@ search_retrieve_xml(void)
 	return TRUE;
 
 out:
-	G_FREE_NULL(path);
-	G_FREE_NULL(path_orig);
+	HFREE_NULL(path);
+	HFREE_NULL(path_orig);
 
 	return FALSE;
 }
@@ -1200,11 +1207,11 @@ xml_to_filter(xmlNodePtr xmlnode, gpointer unused_udata)
 		}
         active = 0 != v;
     }
-    if (active)
-        set_flags(filter->flags, FILTER_FLAG_ACTIVE);
-    else
-        clear_flags(filter->flags, FILTER_FLAG_ACTIVE);
-
+    if (active) {
+        filter->flags |= FILTER_FLAG_ACTIVE;
+	} else {
+        filter->flags &= ~FILTER_FLAG_ACTIVE;
+	}
     buf = STRTRACK(xml_get_string(xmlnode, TAG_FILTER_UID));
     g_assert(buf);
     dest = parse_target(buf, &error);
@@ -1275,7 +1282,7 @@ xml_to_text_rule(xmlNodePtr xmlnode, gpointer data)
 
     flags = get_rule_flags_from_xml(xmlnode);
     rule = filter_new_text_rule(match, type, case_sensitive, target, flags);
-    clear_flags(rule->flags, RULE_FLAG_VALID);
+    rule->flags &= ~RULE_FLAG_VALID;
 
     if (GUI_PROPERTY(gui_debug) >= 4) {
         g_message("added to filter \"%s\" rule with target %p",
@@ -1344,7 +1351,7 @@ xml_to_ip_rule(xmlNodePtr xmlnode, gpointer data)
 
     flags = get_rule_flags_from_xml(xmlnode);
     rule = filter_new_ip_rule(addr, mask, target, flags);
-    clear_flags(rule->flags, RULE_FLAG_VALID);
+    rule->flags &= ~RULE_FLAG_VALID;
 
     if (GUI_PROPERTY(gui_debug) >= 4) {
         g_message("added to filter \"%s\" rule with target %p",
@@ -1408,7 +1415,7 @@ xml_to_size_rule(xmlNodePtr xmlnode, gpointer data)
 
     flags = get_rule_flags_from_xml(xmlnode);
     rule = filter_new_size_rule(lower, upper, target, flags);
-    clear_flags(rule->flags, RULE_FLAG_VALID);
+    rule->flags &= ~RULE_FLAG_VALID;
 
     if (GUI_PROPERTY(gui_debug) >= 4) {
         g_message("added to filter \"%s\" rule with target %p",
@@ -1447,7 +1454,7 @@ xml_to_jump_rule(xmlNodePtr xmlnode, gpointer data)
 
     flags = get_rule_flags_from_xml(xmlnode);
     rule = filter_new_jump_rule(target,flags);
-    clear_flags(rule->flags, RULE_FLAG_VALID);
+    rule->flags &= ~RULE_FLAG_VALID;
 
     if (GUI_PROPERTY(gui_debug) >= 4) {
         g_message("added to filter \"%s\" rule with target %p",
@@ -1478,7 +1485,7 @@ xml_to_sha1_rule(xmlNodePtr xmlnode, gpointer data)
 					NODE_RULE_SHA1));
 
     buf = STRTRACK(xml_get_string(xmlnode, TAG_RULE_SHA1_FILENAME));
-    filename = g_strdup(buf != NULL ? buf : "[Unknown]");
+    filename = h_strdup(buf != NULL ? buf : "[Unknown]");
 	xml_free_null(&buf);
 
     buf = STRTRACK(xml_get_string(xmlnode, TAG_RULE_SHA1_HASH));
@@ -1504,7 +1511,7 @@ xml_to_sha1_rule(xmlNodePtr xmlnode, gpointer data)
 
     flags = get_rule_flags_from_xml(xmlnode);
     rule = filter_new_sha1_rule(sha1, filename, target, flags);
-    clear_flags(rule->flags, RULE_FLAG_VALID);
+    rule->flags &= ~RULE_FLAG_VALID;
 
     if (GUI_PROPERTY(gui_debug) >= 4) {
         g_message("added to filter \"%s\" rule with target %p",
@@ -1514,7 +1521,7 @@ xml_to_sha1_rule(xmlNodePtr xmlnode, gpointer data)
     filter->ruleset = g_list_append(filter->ruleset, rule);
 	
 failure:
-    G_FREE_NULL(filename);
+    HFREE_NULL(filename);
 	xml_free_null(&buf);
 }
 
@@ -1584,7 +1591,7 @@ xml_to_flag_rule(xmlNodePtr xmlnode, gpointer data)
 
     flags = get_rule_flags_from_xml(xmlnode);
     rule = filter_new_flag_rule(stable, busy, push, target, flags);
-    clear_flags(rule->flags, RULE_FLAG_VALID);
+    rule->flags &= ~RULE_FLAG_VALID;
 
     if (GUI_PROPERTY(gui_debug) >= 4) {
         g_message("added to filter \"%s\" rule with target %p",
@@ -1651,7 +1658,7 @@ xml_to_state_rule(xmlNodePtr xmlnode, gpointer data)
 
     flags = get_rule_flags_from_xml(xmlnode);
     rule = filter_new_state_rule(display, download, target, flags);
-    clear_flags(rule->flags, RULE_FLAG_VALID);
+    rule->flags &= ~RULE_FLAG_VALID;
 
     if (GUI_PROPERTY(gui_debug) >= 4) {
         g_message("added to filter \"%s\" rule with target %p",
