@@ -1,5 +1,5 @@
 /*
- * $Id: settings_cb.c 15835 2008-09-10 00:58:54Z cbiere $
+ * $Id: settings_cb.c 17077 2009-10-16 20:51:38Z rmanfredi $
  *
  * Copyright (c) 2001-2003, Raphael Manfredi, Richard Eckart
  *
@@ -36,7 +36,7 @@
 
 #include "gui.h"
 
-RCSID("$Id: settings_cb.c 15835 2008-09-10 00:58:54Z cbiere $")
+RCSID("$Id: settings_cb.c 17077 2009-10-16 20:51:38Z rmanfredi $")
 
 #include "misc.h"
 #include "settings_cb.h"
@@ -47,6 +47,7 @@ RCSID("$Id: settings_cb.c 15835 2008-09-10 00:58:54Z cbiere $")
 #include "if/gnet_property.h"
 #include "if/bridge/ui2c.h"
 
+#include "lib/glib-missing.h"	/* For g_strlcpy() */
 #include "lib/override.h"		/* Must be the last header included */
 
 /*
@@ -1049,4 +1050,28 @@ on_menu_autohide_bws_gleaf_activate(GtkMenuItem *menuitem,
 	checkmenu_changed(gui, PROP_AUTOHIDE_BWS_GLEAF, menuitem);
 }
 
+void
+on_menu_autohide_bws_dht_activate(GtkMenuItem *menuitem,
+	gpointer unused_udata)
+{
+	(void) unused_udata;
+	checkmenu_changed(gui, PROP_AUTOHIDE_BWS_DHT, menuitem);
+}
+
+void
+on_menu_bws_dht_in_visible_activate(GtkMenuItem *menuitem,
+	gpointer unused_udata)
+{
+	(void) unused_udata;
+	checkmenu_changed(gui, PROP_PROGRESSBAR_BWS_DHT_IN_VISIBLE, menuitem);
+}
+
+void
+on_menu_bws_dht_out_visible_activate(GtkMenuItem *menuitem,
+	gpointer unused_udata)
+{
+	(void) unused_udata;
+	checkmenu_changed(gui, PROP_PROGRESSBAR_BWS_DHT_OUT_VISIBLE, menuitem);
+}
+
 /* vi: set ts=4 sw=4 cindent: */
